begin;
set analyze_threshold_percent to 0;
select 'Schema ro may already exist; that''s okay.' as info;
select 'If other schemas, or tables, already exist, you''re installing twice; it won''t do any harm.' as info;
create schema if not exists ro;
create schema if not exists base_241129;
create schema if not exists lf_241129;
create schema if not exists sf_241129;
create or replace view
  base_241129.fixed_pg_user
as
  select
    usesysid::int4           as user_id,
    trim( usename )::varchar as user_name
  from
    pg_user;


create or replace view
  lf_241129.analyze_totals_for_users_by_minute
as
  with
    cte_s1 as
    (
      select
        userid                         as user_id,
        date_trunc( 'minute', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        userid,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
             cte_s1
   left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  lf_241129.analyze_totals_for_users_by_day
as
  with
    cte_s1 as
    (
      select
        userid                         as user_id,
        date_trunc( 'day', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        userid,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
             cte_s1
   left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  sf_241129.analyze_totals_for_users_by_day
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_users_by_day
    )

  select
    user_name::varchar as uuser,
    start_ts::date,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;



create or replace view
  lf_241129.ttables_creation_for_all_by_hour
as
  select
    date_trunc( 'hour', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    start_ts;


create table if not exists base_241129.numbers
(
  number  int4  not null encode zstd primary key
)
diststyle all
compound sortkey( number );

create or replace view
  base_241129.generate_days
as
  select
    ( date_trunc('day',current_timestamp) - (interval '1 day' * number) )::timestamp as start_ts
  from
    base_241129.numbers
  where
    number >= 0 and number < 7;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_all_by_day
as
  with
    cte_aggr as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime)  as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_days.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_days
     left join cte_aggr_stats             using ( node_type, start_ts )
     left join cte_bcast_stats            using ( node_type, start_ts )
     left join cte_delete_stats           using ( node_type, start_ts )
     left join cte_dist_stats             using ( node_type, start_ts )
     left join cte_hash_stats             using ( node_type, start_ts )
     left join cte_hashjoin_stats         using ( node_type, start_ts )
     left join cte_insert_stats           using ( node_type, start_ts )
     left join cte_limit_stats            using ( node_type, start_ts )
     left join cte_merge_stats            using ( node_type, start_ts )
     left join cte_mergejoin_stats        using ( node_type, start_ts )
     left join cte_nestloop_stats         using ( node_type, start_ts )
     left join cte_parse_stats            using ( node_type, start_ts )
     left join cte_project_stats          using ( node_type, start_ts )
     left join cte_return_stats           using ( node_type, start_ts )
     left join cte_save_stats             using ( node_type, start_ts )
     left join cte_sort_stats             using ( node_type, start_ts )
     left join cte_spatial_join_stats     using ( node_type, start_ts )
     left join cte_unique_stats           using ( node_type, start_ts )
     left join cte_window_stats           using ( node_type, start_ts );


create or replace view
  lf_241129.analyze_totals_for_database_by_day
as
  select
    database::varchar                                                                                             as database_name,
    date_trunc( 'day', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    database,
    start_ts;


create or replace view
  lf_241129.vacuum_totals_for_users_by_second
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        userid          as user_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        user_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        user_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        user_id,
        date_trunc( 'second', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        user_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        user_id,
        date_trunc( 'second', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        user_id,
        start_ts
    )

  select
    cte_number_vacuums.user_id::int4,
    trim( pg_user.usename )::varchar as user_name,
    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( user_id, start_ts )
         join pg_user on pg_user.usesysid = cte_number_vacuums.user_id;


create or replace view
  sf_241129.vacuum_totals_for_users_by_second
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_users_by_second
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace function
  base_241129.hr_number( number int8 ) returns varchar immutable
as
  $$
    select
      case
        when abs($1) >=             0 and abs($1) <             1000 then ($1                        )::numeric(18,0)::varchar
        when abs($1) >=          1000 and abs($1) <          1000000 then ($1::float4 / 1000         )::numeric(18,1)::varchar || 'k'
        when abs($1) >=       1000000 and abs($1) <       1000000000 then ($1::float4 / 1000000      )::numeric(18,1)::varchar || 'm'
        when abs($1) >=    1000000000 and abs($1) <    1000000000000 then ($1::float4 / 1000000000   )::numeric(18,1)::varchar || 'b'
        when abs($1) >= 1000000000000 and abs($1) < 1000000000000000 then ($1::float4 / 1000000000000)::numeric(18,1)::varchar || 't'
        when $1 is null                                              then null
      end
  $$
language sql;

create or replace function
  base_241129.hr_number( number int4 ) returns varchar immutable
as
  $$
    select
      case
        when abs($1) >=             0 and abs($1) <             1000 then ($1                        )::numeric(18,0)::varchar
        when abs($1) >=          1000 and abs($1) <          1000000 then ($1::float4 / 1000         )::numeric(18,1)::varchar || 'k'
        when abs($1) >=       1000000 and abs($1) <       1000000000 then ($1::float4 / 1000000      )::numeric(18,1)::varchar || 'm'
        when abs($1) >=    1000000000 and abs($1) <    1000000000000 then ($1::float4 / 1000000000   )::numeric(18,1)::varchar || 'b'
        when $1 is null                                              then null
      end
  $$
language sql;

create or replace function
  base_241129.hr_number( number float4 ) returns varchar immutable
as
  $$
    select
      case
        when abs($1) >=             0 and abs($1) <             1000 then ($1                )::numeric(18,0)::varchar       
        when abs($1) >=          1000 and abs($1) <          1000000 then ($1 / 1000         )::numeric(18,1)::varchar || 'k'
        when abs($1) >=       1000000 and abs($1) <       1000000000 then ($1 / 1000000      )::numeric(18,1)::varchar || 'm'
        when abs($1) >=    1000000000 and abs($1) <    1000000000000 then ($1 / 1000000000   )::numeric(18,1)::varchar || 'b'
        when abs($1) >= 1000000000000 and abs($1) < 1000000000000000 then ($1 / 1000000000000)::numeric(18,1)::varchar || 't'
        when $1 is null                                              then null
      end
  $$
language sql;


create or replace view
  base_241129.generate_hours
as
  select
    ( date_trunc('hour',current_timestamp) - (interval '1 hour' * number) )::timestamp as start_ts
  from
    base_241129.numbers
  where
    number >= 0 and number < 7 * 24;


create or replace view
  base_241129.fixed_stl_vacuum_detail
as
  select
    compact::int4,
    empty_space_size::int8,
    end_row::int8,
    insufficient_local_blk_cnt::int4,
    local_block_count::int8,
    local_to_total_ratio::float8,
    num_blocks_replaced::int8,
    num_deleted_rows::int8,
    num_new_blocks::int8,
    num_new_rows::int8,
    part::int4,
    pid::int4      as session_id,
    query::int4    as query_id,
    slice::int2    as slice_ordinal,
    start_row::int8,
    table_id::int4  as table_id,
    table_partition_number::int4,
    total_block_io_estimate::int8,
    userid::int4    as user_id,
    window_num::int4,
    xid::int8       as transaction_id
  from
    stl_vacuum_detail;


create or replace view
  base_241129.fixed_stl_query
as
  select
    case
      when aborted = 0 then false
      when aborted = 1 then true
                       else null
    end::bool                  as query_abort_flag,
    case
      when concurrency_scaling_status = 1 then true
                                          else false
    end::bool                  as query_csc_flag,
    trim( database )::varchar  as database_partial_name,
    case
      when endtime = '2000-01-01 00:00:00' then null
                                           else endtime
    end::timestamp             as query_end_ts,
    -- insert_pristine::int4,
    -- trim( label )::varchar  as query_label,
    pid::int4                  as session_id,
    query::int4                as query_id,
    -- MG2 : is not actually char, can contain UTF-8
    trim( querytxt )::varchar  as query_text,
    case
      when starttime = '2000-01-01 00:00:00' then null
                                             else starttime
    end::timestamp             as query_start_ts,
    userid::int4               as user_id,
    xid::int8                  as transaction_id,

    ( query_end_ts - query_start_ts )::interval as query_total_duration
  from
    stl_query
  where
        query_start_ts >= current_date - interval '6 day'
    and query_start_ts <= current_timestamp;


create or replace view
  lf_241129.vacuum_stats_for_all_by_hour
as
  with
    cte_xids as 
    (
      select
        sum( num_new_blocks      ) as sum_number_new_blocks,
        sum( num_blocks_replaced ) as sum_number_replaced_blocks,

        sum( num_new_rows        ) as sum_number_new_rows,
        sum( num_deleted_rows    ) as sum_number_deleted_rows,

        count(*)                   as number_qids,

        min( query_start_ts ) as min_query_start_ts,
        max( query_end_ts   ) as max_query_end_ts,

        max_query_end_ts - min_query_start_ts as duration
      from
             base_241129.fixed_stl_vacuum_detail
        join base_241129.fixed_stl_query using( query_id, transaction_id )
      where
            -- MG2 : procably we should include aborted - need a general approcch to this
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_end_ts     <  current_timestamp
      group by
        transaction_id
      having
        -- MG2 : ignore xids in partial first start_ts
        min_query_start_ts >= current_date - interval '6 day'
    ),

    cte_xids_stats as
    (
      select
        date_trunc( 'hour', min_query_start_ts ) as start_ts,

        min( sum_number_new_blocks )        over () as min_sum_number_new_blocks,
        max( sum_number_new_blocks )        over () as max_sum_number_new_blocks,
        median( sum_number_new_blocks )     over () as med_sum_number_new_blocks,
        avg( sum_number_new_blocks )        over () as avg_sum_number_new_blocks,
        stddev_pop( sum_number_new_blocks ) over () as std_sum_number_new_blocks,
        sum( sum_number_new_blocks )        over () as sum_sum_number_new_blocks,

        min( sum_number_replaced_blocks )        over () as min_sum_number_replaced_blocks,
        max( sum_number_replaced_blocks )        over () as max_sum_number_replaced_blocks,
        median( sum_number_replaced_blocks )     over () as med_sum_number_replaced_blocks,
        avg( sum_number_replaced_blocks )        over () as avg_sum_number_replaced_blocks,
        stddev_pop( sum_number_replaced_blocks ) over () as std_sum_number_replaced_blocks,
        sum( sum_number_replaced_blocks )        over () as sum_sum_number_replaced_blocks,

        min( sum_number_new_rows )        over () as min_sum_number_new_rows,
        max( sum_number_new_rows )        over () as max_sum_number_new_rows,
        median( sum_number_new_rows )     over () as med_sum_number_new_rows,
        avg( sum_number_new_rows )        over () as avg_sum_number_new_rows,
        stddev_pop( sum_number_new_rows ) over () as std_sum_number_new_rows,
        sum( sum_number_new_rows )        over () as sum_sum_number_new_rows,

        min( sum_number_deleted_rows )        over () as min_sum_number_deleted_rows,
        max( sum_number_deleted_rows )        over () as max_sum_number_deleted_rows,
        median( sum_number_deleted_rows )     over () as med_sum_number_deleted_rows,
        avg( sum_number_deleted_rows )        over () as avg_sum_number_deleted_rows,
        stddev_pop( sum_number_deleted_rows ) over () as std_sum_number_deleted_rows,
        sum( sum_number_deleted_rows )        over () as sum_sum_number_deleted_rows,

        min( duration ) over ()                              as min_duration,
        max( duration ) over ()                              as max_duration,
        median( extract(epoch from (duration)) ) over ()     as med_duration,
        avg( duration ) over ()                              as avg_duration,
        stddev_pop( extract(epoch from (duration)) ) over () as std_duration,
        sum( duration ) over ()                              as sum_duration,

        sum( number_qids ) over () as number_qids,
        count(*) over ()           as number_xids
      from
        cte_xids
    )

  select
    start_ts::timestamp,

    min_sum_number_new_blocks::int8,
    max_sum_number_new_blocks::int8,
    med_sum_number_new_blocks::int8,
    avg_sum_number_new_blocks::float4,
    std_sum_number_new_blocks::float4,
    sum_sum_number_new_blocks::float4,

    min_sum_number_replaced_blocks::int8,
    max_sum_number_replaced_blocks::int8,
    med_sum_number_replaced_blocks::int8,
    avg_sum_number_replaced_blocks::float4,
    std_sum_number_replaced_blocks::float4,
    sum_sum_number_replaced_blocks::float4,

    min_sum_number_new_rows::int8,
    max_sum_number_new_rows::int8,
    med_sum_number_new_rows::int8,
    avg_sum_number_new_rows::float4,
    std_sum_number_new_rows::float4,
    sum_sum_number_new_rows::float4,

    min_sum_number_deleted_rows::int8,
    max_sum_number_deleted_rows::int8,
    med_sum_number_deleted_rows::int8,
    avg_sum_number_deleted_rows::float4,
    std_sum_number_deleted_rows::float4,
    sum_sum_number_deleted_rows::float4,

    min_duration::interval,
    max_duration::interval,
    med_duration::float4,
    avg_duration::interval,
    std_duration::float4,
    sum_duration::interval,

    number_qids::int8,
    number_xids::int8
  from
              base_241129.generate_hours
    left join cte_xids_stats using ( start_ts )
  group by
    start_ts,
    min_sum_number_new_blocks,
    max_sum_number_new_blocks,
    med_sum_number_new_blocks,
    avg_sum_number_new_blocks,
    std_sum_number_new_blocks,
    sum_sum_number_new_blocks,
    min_sum_number_replaced_blocks,
    max_sum_number_replaced_blocks,
    med_sum_number_replaced_blocks,
    avg_sum_number_replaced_blocks,
    std_sum_number_replaced_blocks,
    sum_sum_number_replaced_blocks,
    min_sum_number_new_rows,
    max_sum_number_new_rows,
    med_sum_number_new_rows,
    avg_sum_number_new_rows,
    std_sum_number_new_rows,
    sum_sum_number_new_rows,
    min_sum_number_deleted_rows,
    max_sum_number_deleted_rows,
    med_sum_number_deleted_rows,
    avg_sum_number_deleted_rows,
    std_sum_number_deleted_rows,
    sum_sum_number_deleted_rows,
    min_duration,
    max_duration,
    med_duration,
    avg_duration,
    std_duration,
    sum_duration,
    number_qids,
    number_xids;


create or replace view
  sf_241129.vacuum_stats_for_all_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        base_241129.hr_number( min_sum_number_new_blocks ) as hr_min_sum_number_new_blocks,
        base_241129.hr_number( max_sum_number_new_blocks ) as hr_max_sum_number_new_blocks,
        base_241129.hr_number( med_sum_number_new_blocks ) as hr_med_sum_number_new_blocks,
        base_241129.hr_number( avg_sum_number_new_blocks ) as hr_avg_sum_number_new_blocks,
        base_241129.hr_number( std_sum_number_new_blocks ) as hr_std_sum_number_new_blocks,
        base_241129.hr_number( sum_sum_number_new_blocks ) as hr_sum_sum_number_new_blocks,

        base_241129.hr_number( min_sum_number_replaced_blocks ) as hr_min_sum_number_replaced_blocks,
        base_241129.hr_number( max_sum_number_replaced_blocks ) as hr_max_sum_number_replaced_blocks,
        base_241129.hr_number( med_sum_number_replaced_blocks ) as hr_med_sum_number_replaced_blocks,
        base_241129.hr_number( avg_sum_number_replaced_blocks ) as hr_avg_sum_number_replaced_blocks,
        base_241129.hr_number( std_sum_number_replaced_blocks ) as hr_std_sum_number_replaced_blocks,
        base_241129.hr_number( sum_sum_number_replaced_blocks ) as hr_sum_sum_number_replaced_blocks,

        base_241129.hr_number( min_sum_number_new_rows ) as hr_min_sum_number_new_rows,
        base_241129.hr_number( max_sum_number_new_rows ) as hr_max_sum_number_new_rows,
        base_241129.hr_number( med_sum_number_new_rows ) as hr_med_sum_number_new_rows,
        base_241129.hr_number( avg_sum_number_new_rows ) as hr_avg_sum_number_new_rows,
        base_241129.hr_number( std_sum_number_new_rows ) as hr_std_sum_number_new_rows,
        base_241129.hr_number( sum_sum_number_new_rows ) as hr_sum_sum_number_new_rows,

        base_241129.hr_number( min_sum_number_deleted_rows ) as hr_min_sum_number_deleted_rows,
        base_241129.hr_number( max_sum_number_deleted_rows ) as hr_max_sum_number_deleted_rows,
        base_241129.hr_number( med_sum_number_deleted_rows ) as hr_med_sum_number_deleted_rows,
        base_241129.hr_number( avg_sum_number_deleted_rows ) as hr_avg_sum_number_deleted_rows,
        base_241129.hr_number( std_sum_number_deleted_rows ) as hr_std_sum_number_deleted_rows,
        base_241129.hr_number( sum_sum_number_deleted_rows ) as hr_sum_sum_number_deleted_rows,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_sum_number_new_blocks) ) over () as max_len_hr_min_sum_number_new_blocks,
        max( len(hr_max_sum_number_new_blocks) ) over () as max_len_hr_max_sum_number_new_blocks,
        max( len(hr_med_sum_number_new_blocks) ) over () as max_len_hr_med_sum_number_new_blocks,
        max( len(hr_avg_sum_number_new_blocks) ) over () as max_len_hr_avg_sum_number_new_blocks,
        max( len(hr_std_sum_number_new_blocks) ) over () as max_len_hr_std_sum_number_new_blocks,
        max( len(hr_sum_sum_number_new_blocks) ) over () as max_len_hr_sum_sum_number_new_blocks,

        max( len(hr_min_sum_number_replaced_blocks) ) over () as max_len_hr_min_sum_number_replaced_blocks,
        max( len(hr_max_sum_number_replaced_blocks) ) over () as max_len_hr_max_sum_number_replaced_blocks,
        max( len(hr_med_sum_number_replaced_blocks) ) over () as max_len_hr_med_sum_number_replaced_blocks,
        max( len(hr_avg_sum_number_replaced_blocks) ) over () as max_len_hr_avg_sum_number_replaced_blocks,
        max( len(hr_std_sum_number_replaced_blocks) ) over () as max_len_hr_std_sum_number_replaced_blocks,
        max( len(hr_sum_sum_number_replaced_blocks) ) over () as max_len_hr_sum_sum_number_replaced_blocks,

        max( len(hr_min_sum_number_new_rows) ) over () as max_len_hr_min_sum_number_new_rows,
        max( len(hr_max_sum_number_new_rows) ) over () as max_len_hr_max_sum_number_new_rows,
        max( len(hr_med_sum_number_new_rows) ) over () as max_len_hr_med_sum_number_new_rows,
        max( len(hr_avg_sum_number_new_rows) ) over () as max_len_hr_avg_sum_number_new_rows,
        max( len(hr_std_sum_number_new_rows) ) over () as max_len_hr_std_sum_number_new_rows,
        max( len(hr_sum_sum_number_new_rows) ) over () as max_len_hr_sum_sum_number_new_rows,

        max( len(hr_min_sum_number_deleted_rows) ) over () as max_len_hr_min_sum_number_deleted_rows,
        max( len(hr_max_sum_number_deleted_rows) ) over () as max_len_hr_max_sum_number_deleted_rows,
        max( len(hr_med_sum_number_deleted_rows) ) over () as max_len_hr_med_sum_number_deleted_rows,
        max( len(hr_avg_sum_number_deleted_rows) ) over () as max_len_hr_avg_sum_number_deleted_rows,
        max( len(hr_std_sum_number_deleted_rows) ) over () as max_len_hr_std_sum_number_deleted_rows,
        max( len(hr_sum_sum_number_deleted_rows) ) over () as max_len_hr_sum_sum_number_deleted_rows,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_qids,
        number_xids
      from
        lf_241129.vacuum_stats_for_all_by_hour
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_sum_number_new_blocks, max_len_hr_min_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_new_blocks, max_len_hr_max_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_new_blocks, max_len_hr_med_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_new_blocks, max_len_hr_avg_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_new_blocks, max_len_hr_std_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_new_blocks, max_len_hr_sum_sum_number_new_blocks ) )::varchar as "new blocks",

    ( lpad( hr_min_sum_number_replaced_blocks, max_len_hr_min_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_replaced_blocks, max_len_hr_max_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_replaced_blocks, max_len_hr_med_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_replaced_blocks, max_len_hr_avg_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_replaced_blocks, max_len_hr_std_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_replaced_blocks, max_len_hr_sum_sum_number_replaced_blocks ) )::varchar as "replaced blocks",

    ( lpad( hr_min_sum_number_new_rows, max_len_hr_min_sum_number_new_rows ) || ' / ' ||
      lpad( hr_max_sum_number_new_rows, max_len_hr_max_sum_number_new_rows ) || ' / ' ||
      lpad( hr_med_sum_number_new_rows, max_len_hr_med_sum_number_new_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_new_rows, max_len_hr_avg_sum_number_new_rows ) || ' / ' ||
      lpad( hr_std_sum_number_new_rows, max_len_hr_std_sum_number_new_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_new_rows, max_len_hr_sum_sum_number_new_rows ) )::varchar as "new rows",

    ( lpad( hr_min_sum_number_deleted_rows, max_len_hr_min_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_max_sum_number_deleted_rows, max_len_hr_max_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_med_sum_number_deleted_rows, max_len_hr_med_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_deleted_rows, max_len_hr_avg_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_std_sum_number_deleted_rows, max_len_hr_std_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_deleted_rows, max_len_hr_sum_sum_number_deleted_rows ) )::varchar as "deleted rows",

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as "xid duration",

    number_qids::int8 as qids,
    number_xids::int8 as xids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.sessions_fast
as
  -- MG2 : I wanted to put the count of locks here, but this leads to queries about half the time
  --       taking a long time to execute (say, 8s vs 0.2s)

  select
    stv_sessions.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    stv_sessions.process::int4                                                  as session_id,
    trim( stv_sessions.db_name )::varchar                                       as truncated_database_name,
    ( current_timestamp at time zone 'utc' - stv_sessions.starttime )::interval as session_duration
  from
         stv_sessions
    join base_241129.fixed_pg_user using ( user_id );


create or replace view
  base_241129.generate_minutes
as
  select
    ( date_trunc('minute',current_timestamp) - (interval '1 minute' * number) )::timestamp as start_ts
  from
    base_241129.numbers
  where
    number >= 0 and number < 7 * 24 * 60;


create or replace view
  base_241129.fixed_pg_class
as
  select
    oid::int4                as object_id,
    trim( relname )::varchar as object_name,
    relnamespace::int4       as schema_id,
    relowner::int4           as user_id,
    relnatts::int2,
    -- MG2 : Redshift cannot cast relkind to char(1)
    case
      when relkind = 'r' then 'r'
      when relkind = 'v' then 'v'
    end::char(1) as relkind
  from
    pg_class
  where
    relkind in ( 'r', 'v' );


create or replace view
  base_241129.fixed_pg_class_for_tables
as
  select
    object_id::int4      as table_id,
    object_name::varchar as table_name,
    schema_id::int4,
    user_id::int4,
    relnatts::int2
  from
    base_241129.fixed_pg_class
  where
    relkind = 'r';


create or replace view
  base_241129.fixed_pg_namespace
as
  select
    oid::int4                as schema_id,
    trim( nspname )::varchar as schema_name,
    nspowner::int4           as user_id
  from
    pg_namespace;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_tables_by_minute
as
  with
    cte_aggr as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    node_type::varchar,
    base_241129.generate_minutes.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_minutes
    cross join base_241129.fixed_pg_class_for_tables
     left join base_241129.fixed_pg_namespace using ( schema_id )
     left join cte_aggr_stats          using ( start_ts, node_type, table_id )
     left join cte_delete_stats        using ( start_ts, node_type, table_id )
     left join cte_hashjoin_stats      using ( start_ts, node_type, table_id )
     left join cte_insert_stats        using ( start_ts, node_type, table_id )
     left join cte_mergejoin_stats     using ( start_ts, node_type, table_id )
     left join cte_nestloop_stats      using ( start_ts, node_type, table_id )
     left join cte_save_stats          using ( start_ts, node_type, table_id )
     left join cte_sort_stats          using ( start_ts, node_type, table_id )
     left join cte_spatial_join_stats  using ( start_ts, node_type, table_id )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_minute_concise
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_minute
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  base_241129.generate_seconds
as
  select
    ( date_trunc('second',current_timestamp) - (interval '1 second' * number) )::timestamp as start_ts
  from
    base_241129.numbers; -- MG2 : we have 7 days of seconds in the table


create or replace view
  base_241129.fixed_stl_wlm_query
as
  select
    -- est_peak_mem::int8,
    case when exec_end_time   = '2000-01-01 00:00:00' then null else exec_end_time   end::timestamp as query_compile_exec_end_ts,
    case when exec_start_time = '2000-01-01 00:00:00' then null else exec_start_time end::timestamp as query_compile_exec_start_ts,
    final_state::char(16),
    query::int4          as query_id,
    -- query_priority::char(20),
    case when queue_end_time   = '2000-01-01 00:00:00' then null else queue_end_time   end::timestamp as query_queue_end_ts,
    case when queue_start_time = '2000-01-01 00:00:00' then null else queue_start_time end::timestamp as query_queue_start_ts,
    service_class::int2  as queue_id,
    case when service_class_end_time = '2000-01-01 00:00:00' then null else service_class_end_time end::timestamp as query_wlm_end_ts,
    -- service_class_name::char(64),
    case when service_class_start_time = '2000-01-01 00:00:00' then null else service_class_start_time end::timestamp as query_wlm_start_ts,
    slot_count::int2,
    task::int4           as task_id,
    -- total_exec_time::int8,
    -- total_queue_time::int8,
    userid::int4         as user_id,
    xid::int8            as transaction_id,

    ( query_queue_end_ts        - query_queue_start_ts        )::interval as query_queue_duration,
    ( query_compile_exec_end_ts - query_compile_exec_start_ts )::interval as query_compile_exec_duration,
    ( query_wlm_end_ts          - query_wlm_start_ts          )::interval as query_wlm_duration
  from
    stl_wlm_query
  where
        query_wlm_start_ts >= current_date - interval '6 day'
    and query_wlm_start_ts <= current_timestamp;


create or replace view
  base_241129.fixed_svl_compile
as
  select
    case when compile = 1 then true else false end::bool as compile_flag,
    case when endtime = '2000-01-01 00:00:00' then null else endtime end::timestamp as segment_compile_end_ts,
    -- locus::int4,
    pid::int4             as session_id,
    query::int4           as query_id,
    segment::int2         as segment_ordinal,
    case when starttime = '2000-01-01 00:00:00' then null else starttime end::timestamp as segment_compile_start_ts,
    userid::int4          as user_id,
    xid::int8             as transaction_id,

    ( segment_compile_end_ts - segment_compile_start_ts )::interval as segment_compile_duration
  from
    svl_compile
  where
        segment_compile_start_ts >= current_date - interval '6 day'
    and segment_compile_start_ts <  current_timestamp;


create or replace view
  base_241129.queries_compilation
as
  select
    user_id::int4,
    query_id::int4,
    sum( segment_compile_duration )::interval as query_compile_duration
  from
    base_241129.fixed_svl_compile
  where
        segment_compile_start_ts >= current_date - interval '6 day'
    and segment_compile_start_ts <= current_timestamp
  group by
    user_id,
    query_id;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_all_by_second
as
  with
    cte_queries as
    (
      select
        date_trunc( 'second', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        ( query_compile_exec_duration - query_compile_duration ) as query_exec_duration
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id )
        left join base_241129.queries_compilation  using ( query_id )
      where
            query_abort_flag      = false
        and query_csc_flag          = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts           <= current_timestamp
        and query_start_ts         >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        start_ts,

        min( query_queue_duration ) over ( partition by start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by start_ts ) as number_queries
      from
        cte_queries
    )

  select
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_seconds
    left join cte_queries_stats using ( start_ts )
  group by
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_all_by_second
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_all_by_second
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  base_241129.fixed_pg_attribute
as
  select
    attrelid::int4            as object_id,
    trim( attname ) ::varchar as column_name,
    attnum::int2              as column_ordinal,
    atttypmod::int4,
    attnotnull::bool          as column_notnull_flag
  from
    pg_attribute;


create or replace view
  base_241129.fixed_stv_slices
as
  with
    cte_nodes_and_slices as
    (
      select
        node::int2                                                   as node_ordinal,
        slice::int2                                                  as slice_ordinal,
        case when type = 'D' then 'full' else 'partial' end::varchar as slice_type
      from
        stv_slices

      union all

      select
        -1::int2          as node_ordinal,
        -1::int2          as slice_ordinal,
        'leader'::varchar as slice_type
    )

  select
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar
  from
    cte_nodes_and_slices;


create or replace view
  lf_241129.steps_totals_data_on_both_for_node_by_hour
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_hours
     left join cte_aggr         using( slice_ordinal, start_ts )
     left join cte_bcast        using( slice_ordinal, start_ts )
     left join cte_delete       using( slice_ordinal, start_ts )
     left join cte_dist         using( slice_ordinal, start_ts )
     left join cte_hash         using( slice_ordinal, start_ts )
     left join cte_hashjoin     using( slice_ordinal, start_ts )
     left join cte_insert       using( slice_ordinal, start_ts )
     left join cte_limit        using( slice_ordinal, start_ts )
     left join cte_merge        using( slice_ordinal, start_ts )
     left join cte_mergejoin    using( slice_ordinal, start_ts )
     left join cte_nestloop     using( slice_ordinal, start_ts )
     left join cte_parse        using( slice_ordinal, start_ts )
     left join cte_project      using( slice_ordinal, start_ts )
     left join cte_return       using( slice_ordinal, start_ts )
     left join cte_save         using( slice_ordinal, start_ts )
     left join cte_sort         using( slice_ordinal, start_ts )
     left join cte_spatial_join using( slice_ordinal, start_ts )
     left join cte_unique       using( slice_ordinal, start_ts )
     left join cte_window       using( slice_ordinal, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null
  group by
    node_ordinal,
    start_ts;


create or replace procedure
  ro.sp_drop_versionless_sf_schema()
as
  $proc$
    begin
      drop schema sf cascade;
    end
  $proc$
language plpgsql;


create or replace function
  base_241129.hr_bytes( bytes int8 ) returns varchar immutable
as
  $$
    select
      case
        when abs($1) >=             0 and abs($1) <             1024 then ($1                        )::numeric(18,0)::varchar
        when abs($1) >=          1024 and abs($1) <          1048576 then ($1::float4 / 1024         )::numeric(18,1)::varchar || 'kb'
        when abs($1) >=       1048576 and abs($1) <       1073741824 then ($1::float4 / 1048576      )::numeric(18,1)::varchar || 'mb'
        when abs($1) >=    1073741824 and abs($1) <    1099511627776 then ($1::float4 / 1073741824   )::numeric(18,1)::varchar || 'gb'
        when abs($1) >= 1099511627776 and abs($1) < 1125899906842624 then ($1::float4 / 1099511627776)::numeric(18,1)::varchar || 'tb'
        when $1 is null                                              then null
      end
  $$
language sql;

create or replace function
  base_241129.hr_bytes( bytes float4 ) returns varchar immutable
as
  $$
    select
      case
        when abs($1) >=             0 and abs($1) <             1024 then ($1                )::numeric(18,0)::varchar
        when abs($1) >=          1024 and abs($1) <          1048576 then ($1 / 1024         )::numeric(18,1)::varchar || 'kb'
        when abs($1) >=       1048576 and abs($1) <       1073741824 then ($1 / 1048576      )::numeric(18,1)::varchar || 'mb'
        when abs($1) >=    1073741824 and abs($1) <    1099511627776 then ($1 / 1073741824   )::numeric(18,1)::varchar || 'gb'
        when abs($1) >= 1099511627776 and abs($1) < 1125899906842624 then ($1 / 1099511627776)::numeric(18,1)::varchar || 'tb'
        when $1 is null                                              then null
      end
  $$
language sql;


create or replace view
  base_241129.fixed_stl_scan
as
  select
    userid::int4         as user_id,
    query::int4          as query_id,
    case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
    segment::int2        as segment_ordinal,
    step::int2           as step_ordinal,
    starttime::timestamp as scan_start_ts,
    endtime::timestamp   as scan_end_ts,
    tasknum::int4,
    rows::int8,
    bytes::int8,
    fetches::int8,
    type::int4,
    tbl::int4            as table_id,
    is_rrscan::char(1),
    is_delayed_scan::char(1),
    rows_pre_filter::int8,
    rows_pre_user_filter::int8,
    perm_table_name::char(136),
    is_rlf_scan::char(1),
    is_rlf_scan_reason::int4,
    num_em_blocks::int4,
    checksum::int8,
    runtime_filtering::char(1),
    scan_region::int4,
    num_sortkey_as_predicate::int4,
    row_fetcher_state::int8,
    consumed_scan_ranges::int8,
    work_stealing_reason::int8,
    is_vectorized_scan::char(1),
    is_vectorized_scan_reason::int4,
    row_fetcher_reason::int8,
    topology_signature::int8,
    use_tpm_partition::char(1),
    is_rrscan_expr::char(1),
    use_fwd_mtx_opt::char(1),
    scanned_mega_value::char(1)
  from
    stl_scan
  where
        scan_start_ts >= current_date - interval '6 day'
    and scan_start_ts <= current_timestamp;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_schemas_by_all
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    node_type::varchar,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r'
  group by
    schema_id,
    schema_name,
    node_type;



create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_schemas_by_all
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        schema_name,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_schemas_by_all
    )

 select
    node_type::varchar   as node,
    schema_name::varchar as sschema,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    schema_name;


create or replace view
  base_241129.fixed_stv_wlm_service_class_config
as
  with
    cte_queues as
    (
      select
        case
          when concurrency_scaling = 'off'  then false
          when concurrency_scaling = 'auto' then true
                                            else null -- MG2 : this should not happen
        end::bool as queue_csc_flag,
        -- evictable::char(8),
        -- eviction_threshold::int8,
        max_execution_time::int4,
        -- min_step_mem::int4,
        case
          when name = 'Service class for system user (Health Check)'       then 'AWS health-check'
          when name = 'Service class for system user (Metrics Collection)' then 'AWS metrics'
          when name = 'Service class for system user (CM Stats)'           then 'AWS cm-stats'
          when name = 'Service class for system user (Operator)'           then 'AWS operator'
          when name = 'Service class for super user'                       then 'AWS superuser'
          when name = 'Service class for vacuum/analyze'                   then 'AWS auto-vacuum'
                                                                           else trim( name )
        end::varchar as queue_name,
        num_query_tasks::int4,
        case
          when query_group_wild_card = 'true'  then true
          when query_group_wild_card = 'false' then false
                                               else null
        end::bool              as query_group_wildcard_flag,
        -- query_priority::char(20),
        query_working_mem::int4,
        -- queueing_strategy::char(32),
        service_class::int2    as queue_id,
        -- target_num_query_tasks::int4,
        -- target_query_working_mem::int4,
        case
          when user_group_wild_card = 'true'  then true
          when user_group_wild_card = 'false' then false
                                              else null
        end::bool              as user_group_wildcard_flag,
        case
          when user_role_wild_card = 'true'  then true
          when user_role_wild_card = 'false' then false
                                             else null
        end::bool              as user_role_wildcard_flag
      from
        stv_wlm_service_class_config

      union all

      select
        false              as queue_csc_flag,
        0::int4 as max_execution_time,
        'leader'::varchar  as queue_name,
        null::int2         as num_query_tasks,
        false              as query_group_wildcard_flag,
        0::int4            as query_working_mem,
        -1::int2           as queue_id,
        false              as user_group_wildcard_flag,
        false              as user_role_wildcard_flag
    )

  select
    queue_csc_flag::bool,
    max_execution_time::int4,
    queue_name::varchar,
    num_query_tasks::int2,
    query_group_wildcard_flag::bool,
    query_working_mem::int4,
    queue_id::int2,
    user_group_wildcard_flag::bool,
    user_role_wildcard_flag::bool
  from
    cte_queues;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_queues_by_hour
as
  with
    cte_queries as
    (
      select
        queue_id,
        date_trunc( 'hour', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
                  base_241129.fixed_stl_wlm_query
        left join base_241129.fixed_stl_query    using ( query_id )
        left join base_241129.queries_compilation using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        queue_id,
        start_ts,

        min( query_queue_duration ) over ( partition by queue_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by queue_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by queue_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by queue_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by queue_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by queue_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by queue_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by queue_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by queue_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by queue_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by queue_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by queue_id, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by queue_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_hours
    left join cte_queries_stats using ( start_ts )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_queues_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_queues_by_hour
    )

  select
    queue_name::varchar as qname,
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    queue_name,
    start_ts;


create or replace view
  base_241129.fixed_stv_wlm_classification_config
as
  select
    trim( action )::varchar     as action,
    action_seq::int2,
    action_service_class::int2  as queue_id,
    trim( condition )::varchar  as condition,
    id::int2
  from
    stv_wlm_classification_config;



create or replace view
  lf_241129.wqueues_routing_rules
as
  select
    row_number() over ( order by base_241129.fixed_stv_wlm_classification_config.id )::int2 as rule_order,
    query_group_wildcard_flag::bool,
    user_group_wildcard_flag::bool,
    condition::varchar,
    queue_id::int2,
    queue_name::varchar
  from
              base_241129.fixed_stv_wlm_classification_config
         join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  lf_241129.sessions_full
as
  with
    cte_most_recent_pid_connection as
    (
      select distinct
        first_value( pid )        over ( partition by pid order by recordtime desc rows between unbounded preceding and unbounded following ) as process,
        first_value( remotehost ) over ( partition by pid order by recordtime desc rows between unbounded preceding and unbounded following ) as remotehost,
        -- MG2 : sigh, why is port a string anyway, and we get an empty string rather than a NULL
        first_value( case when remoteport = '' then null else remoteport end ) over ( partition by pid order by recordtime desc rows between unbounded preceding and unbounded following ) as remoteport
      from
        stl_connection_log
      where
        stl_connection_log.event = 'authenticated'
    )

  select
    stv_sessions.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    stv_sessions.process::int4                                                  as session_id,
    trim( stv_sessions.db_name )::varchar                                       as truncated_database_name,
    ( current_timestamp at time zone 'utc' - stv_sessions.starttime )::interval as session_duration,
    cte_most_recent_pid_connection.remotehost::varchar,
    cte_most_recent_pid_connection.remoteport::int4,
    sum( case when svv_transactions.pid is not null then 1 else 0 end )::int4   as number_locks
  from
              stv_sessions
         join base_241129.fixed_pg_user             using ( user_id )
         join cte_most_recent_pid_connection using ( process )
    left join svv_transactions               on svv_transactions.pid = stv_sessions.process
  group by
    user_id,
    base_241129.fixed_pg_user.user_name,
    session_id,
    truncated_database_name,
    session_duration,
    remotehost,
    remoteport;


create or replace view
  sf_241129.sessions_fast
as
  select
    user_name::varchar                                    as uuser,
    session_id::int4                                      as pid,
    truncated_database_name::varchar                      as tdb,
    extract( epoch from session_duration )::numeric(18,2) as dur
  from
    lf_241129.sessions_full
  order by
    user_id,
    user_name,
    truncated_database_name,
    session_duration;


create or replace view
  base_241129.fixed_stl_ddltext
as
  -- MG2 : this table can log queries twice
  --       which means you end up with two identical rows, for every row, for such a query

  select distinct
    userid::int4           as user_id,
    xid::int8              as transaction_id,
    pid::int4              as session_id,
    (-1)::int2             as queue_id,
    -- trim( label )::varchar as query_label,
    starttime::timestamp   as query_start_ts,
    endtime::timestamp     as query_end_ts,
    sequence::int4,
    -- MG2 : is not actually char, can contain UTF-8
    text::varchar(200)     as query_text,

    ( interval '0 second' )::interval           as query_queue_duration,
    ( interval '0 second' )::interval           as query_wlm_duration,
    ( query_end_ts - query_start_ts )::interval as query_compile_exec_duration,
    ( query_end_ts - query_start_ts )::interval as query_total_duration
  from
    stl_ddltext
  where
        query_start_ts >= current_date - interval '6 day'
    and query_start_ts <= current_timestamp;


create or replace view
  base_241129.fixed_stl_commit_stats
as
  with
    -- MG2 : remove xid 0 (no clue what it is, suspect maybe something to do with transaction aborts/cluster startup rowlbacks)
    --       remove negative xids (occasionally turn up, look like bugs in logging)
    --       keeps only xids which show up once
    --         - stl_vacuum xids are re-used and make a complete mess
    --         - very small numbers (0.1% maybe) of xids which are in no query logging table, also show up
    --         - and I think leader-node rows can on occasion be duplicated
    --       note startqueue is '2000-01-01 00:00:00' on node -1 only with xid 0 (and is '2000-01-01 00:00:00' on all worker nodes)

    cte_fixed_stl_commit_stats as
    (
      select
        batched_by                                                                   as batch_id,
        xid                                                                          as transaction_id,
        node                                                                         as node_ordinal,
        case when startqueue  = '2000-01-01 00:00:00' then null else startqueue  end as startqueue,
        case when startwork   = '2000-01-01 00:00:00' then null else startwork   end as startwork,
        case when endflush    = '2000-01-01 00:00:00' then null else endflush    end as endflush,
        case when endstage    = '2000-01-01 00:00:00' then null else endstage    end as endstage,
        case when endlocal    = '2000-01-01 00:00:00' then null else endlocal    end as endlocal,
        case when startglobal = '2000-01-01 00:00:00' then null else startglobal end as startglobal,
        case when endtime     = '2000-01-01 00:00:00' then null else endtime     end as endtime,
        newblocks,
        dirtyblocks,
        headers
      from
        stl_commit_stats
      where
            xid >= 1
        and xid        not in ( select distinct xid from stl_commit_stats group by xid, node having count(*) > 1 )
        and xid        not in ( select distinct xid from stl_vacuum  )
        and batched_by not in ( select distinct xid from stl_vacuum  )
        -- MG2 : really, should be startqueue, but startqueue is only set for the leader node rows, so it's a lot of work to use
        and startwork      >= current_date - interval '6 day'
        and startwork      <= current_timestamp
    )

  select
    batch_id::int8,
    transaction_id::int8,
    node_ordinal::int2,
    startqueue::timestamp,
    startwork::timestamp,
    endflush::timestamp,
    endstage::timestamp,
    endlocal::timestamp,
    startglobal::timestamp,
    endtime::timestamp,
    newblocks::int8,
    dirtyblocks::int8,
    headers::int8
  from
    cte_fixed_stl_commit_stats;


create or replace view
  base_241129.fixed_stl_utilitytext
as
  -- MG2 : 1. `select distinct` because this table can log queries twice
  --          which means you end up with two identical rows, for every row, for such a query
  --       2. The stuff wth `left join base_241129.fixed_stl_commit_stats` is because RS for `COMMIT` records
  --          the start time not as when the query was issued, but as when the commit batch began sessioning
  --          that commit
  --       3. We get xid 0 in this table (as well as `stl_commit_stats`, and currently its not understood and is removed
  --       4. for `commit`, we starttime is actually startqueue from stl_commit_stats, and endtime is not known
  --          because `commit` returns quite a long time after the final endtime for itself or the batch

  select distinct
    stl_utilitytext.userid::int4           as user_id,
    stl_utilitytext.xid::int8              as transaction_id,
    stl_utilitytext.pid::int4              as session_id,
    (-1)::int2                             as queue_id,
    -- trim( label )::varchar as query_label,
    case when base_241129.fixed_stl_commit_stats.transaction_id is null then stl_utilitytext.starttime else base_241129.fixed_stl_commit_stats.startqueue end::timestamp as query_start_ts,
    case when base_241129.fixed_stl_commit_stats.transaction_id is null then stl_utilitytext.endtime   else null                                   end::timestamp as query_end_ts,
    stl_utilitytext.sequence::int4,
    -- MG2 : is not actually char, can contain UTF-8
    stl_utilitytext.text::varchar(200)        as query_text,

    ( interval '0 second' )::interval           as query_queue_duration,
    ( interval '0 second' )::interval           as query_wlm_duration,
    ( query_end_ts - query_start_ts )::interval as query_compile_exec_duration,
    ( query_end_ts - query_start_ts )::interval as query_total_duration
  from
              stl_utilitytext
    left join base_241129.fixed_stl_commit_stats on     stl_utilitytext.xid                      = base_241129.fixed_stl_commit_stats.transaction_id
                                             and lower(trim(stl_utilitytext.text))        = 'commit'
                                             and base_241129.fixed_stl_commit_stats.node_ordinal = -1
  where
        query_start_ts      >= current_date - interval '6 day'
    and query_start_ts      <= current_timestamp
    and stl_utilitytext.xid >= 1;


create or replace view
  base_241129.queries_for_leader
as
  with
    cte_queries as
    (
      select
        user_id,
        session_id,
        transaction_id,
        queue_id,
        query_start_ts,
        query_end_ts,
        query_total_duration,
        query_text
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        user_id,
        session_id,
        transaction_id,
        queue_id,
        query_start_ts,
        query_end_ts,
        query_total_duration,
        query_text
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    user_id::int4,
    session_id::int4,
    transaction_id::int8,
    queue_id::int2,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_total_duration::interval,
    true::bool as query_leader_node_flag,
    query_text::varchar
  from
    cte_queries
  where
        query_start_ts >= current_date - interval '6 day'
    and query_start_ts <= current_timestamp;


create or replace view
  base_241129.queries_on_leader_as_for_worker
as
  select
    user_id::int4,
    session_id::int4,
    transaction_id::int8,
    null::int4 as query_id,
    queue_id::int2,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    ( interval '0 second' )::interval as query_queue_duration,
    query_total_duration::interval as query_compile_exec_duration,
    ( interval '0 second' )::interval as query_wlm_duration,
    query_total_duration::interval,
    false::bool as query_abort_flag,
    false::bool as query_csc_flag,
    query_leader_node_flag::bool,
    false::bool as query_rc_flag,
    query_text::varchar
  from
    base_241129.queries_for_leader;


create or replace view
  base_241129.queries_for_worker
as
  with
    cte_queries as
    (
      select
        user_id,
        session_id,
        transaction_id,
        query_id,
        queue_id,
        query_start_ts,
        query_end_ts,
        query_queue_duration,
        query_compile_exec_duration,
        query_wlm_duration,
        query_total_duration,
        query_abort_flag,
        query_csc_flag,
        false as query_leader_node_flag,
        case when base_241129.fixed_stl_wlm_query.query_id is null then true else false end as query_rc_flag,
        query_text
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, transaction_id, user_id )
    )

  select
    user_id::int4,
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    queue_id::int2,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_queue_duration::interval,
    query_compile_exec_duration::interval,
    query_wlm_duration::interval,
    query_total_duration::interval,
    query_abort_flag::bool,
    query_csc_flag::bool,
    query_leader_node_flag::bool,
    query_rc_flag::bool,
    query_text::varchar
  from
    cte_queries;


create or replace view
  base_241129.queries_for_all
as
  with
    cte_queries as
    (
      select * from base_241129.queries_on_leader_as_for_worker
      union all
      select * from base_241129.queries_for_worker
    )

  select
    user_id::int4,
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    queue_id::int2,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_queue_duration::interval,
    query_compile_exec_duration::interval,
    query_wlm_duration::interval,
    query_total_duration::interval,
    query_abort_flag::bool,
    query_csc_flag::bool,
    query_leader_node_flag::bool,
    query_rc_flag::bool,
    query_text::varchar
  from
    cte_queries;


create or replace view
  base_241129.queries_transactions
as
  select
    user_id::int4,
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    queue_id::int2,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_queue_duration::interval,
    query_compile_exec_duration::interval,
    query_wlm_duration::interval,
    query_total_duration::interval,
    query_abort_flag::bool,
    query_csc_flag::bool,
    query_leader_node_flag::bool,
    query_rc_flag::bool,
    query_text::varchar,

    min( query_start_ts )              over ( partition by transaction_id )::timestamp as transaction_start_ts,
    max( query_end_ts )                over ( partition by transaction_id )::timestamp as transaction_end_ts,
    sum( query_total_duration )        over ( partition by transaction_id )::interval as transaction_query_total_duration,
    sum( query_wlm_duration )          over ( partition by transaction_id )::interval as transaction_query_wlm_duration,
    sum( query_queue_duration )        over ( partition by transaction_id )::interval as transaction_query_queue_duration,
    sum( query_compile_exec_duration ) over ( partition by transaction_id )::interval as transaction_query_compile_exec_duration,
    sum( case when query_rc_flag = true then 1 else 0 end ) over ( partition by transaction_id )::int4 as transaction_number_rc_queries,
    sum( case when query_leader_node_flag = true  then 1 else 0 end )::int4 as transaction_number_leader_queries,
    sum( case when query_leader_node_flag = false then 1 else 0 end )::int4 as transaction_number_worker_queries,
    count(*)                           over ( partition by transaction_id )::int4 as transaction_number_total_queries,
    case when max( case when query_abort_flag = true then 1 else 0 end ) over ( partition by transaction_id ) = 1 then true else false end::bool as transaction_abort_flag,

    ( transaction_end_ts   - transaction_start_ts             )::interval as transaction_duration,
    ( transaction_duration - transaction_query_total_duration )::interval as transaction_idle_duration
  from
    base_241129.queries_for_all
  group by
    user_id,
    session_id,
    transaction_id,
    query_id,
    queue_id,
    query_start_ts,
    query_end_ts,
    query_queue_duration,
    query_compile_exec_duration,
    query_wlm_duration,
    query_total_duration,
    query_abort_flag,
    query_csc_flag,
    query_leader_node_flag,
    query_rc_flag,
    query_text;


create or replace view
  lf_241129.queries_compilation
as
  with
    cte_query_stream_compilation as
    (
      select
        svl_compile.pid                   as session_id,
        svl_compile.xid                   as transaction_id,
        svl_compile.query                 as query_id,
        stl_stream_segs.stream            as stream_ordinal,
        sum( 1-compile )                  as stream_number_leader_node_cache_segments,
        sum(   compile )                  as stream_number_offload_segments,
        count(*)                          as stream_number_segments,
        ( max(endtime) - min(starttime) ) as stream_compile_duration
      from
                     svl_compile
        natural join stl_stream_segs
      group by
        svl_compile.pid,
        svl_compile.xid,
        svl_compile.query,
        stl_stream_segs.stream
    ),

    cte_queries_compilation as
    (
      select
        session_id,
        transaction_id,
        query_id,
        sum( stream_number_leader_node_cache_segments ) as query_number_leader_node_cache_segments,
        sum( stream_number_offload_segments           ) as query_number_offload_segments,
        sum( stream_number_segments                   ) as query_number_segments,
        sum( stream_compile_duration                  ) as query_compile_duration
      from
        cte_query_stream_compilation
      group by
        session_id,
        transaction_id,
        query_id
    )

  select
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    query_number_leader_node_cache_segments::int2,
    query_number_offload_segments::int2,
    query_number_segments::int2,
    query_compile_duration::interval
  from
    cte_queries_compilation;


create or replace view
  base_241129.fixed_stl_query_metrics
as
  select
    case when blocks_read    != -1 then blocks_read    else 0 end::int8 as query_number_blocks_read,
    case when blocks_to_disk != -1 then blocks_to_disk else 0 end::int8 as query_number_blocks_to_disk,
    cpu_time::int8 as query_cpu_time,
    -- max_blocks_read::int4
    -- max_blocks_to_disk::int8
    -- max_cpu_time::int8
    -- max_query_scan_size::int8
    -- max_rows::int8
    -- max_run_time::int8
    query::int4 as query_id,
    -- query_priority::int4,
    -- query_queue_time::int8,
    -- query_scan_size::int8,
    rows::int8 as query_number_rows,
    -- run_time::int8,
    -- segment::int2    as segment_ordinal,
    service_class::int2 as queue_id,
    -- service_class_name::char(64)
    -- slices::int4
    starttime::timestamp   as query_start_ts,
    -- step::int2          as step_ordinal,
    -- step_type::int2,
    userid::int4        as user_id
  from
    stl_query_metrics
  where
        segment   = -1
    and step_type = -1
    and query_start_ts >= current_date - interval '6 day'
    and query_start_ts <= current_timestamp;


create or replace view
  base_241129.queries_blocks
as
  select
    query_id::int4,
    sum( query_number_blocks_read    )::int8 as query_number_blocks_read,
    sum( query_number_blocks_to_disk )::int8 as query_number_blocks_to_disk
  from
    base_241129.fixed_stl_query_metrics
  group by
    query_id;


create or replace view
  base_241129.transactions_in_batches
as
  select
    batch_id::int8,
    transaction_id::int8,
    startqueue::timestamp as transaction_batch_queue_start
  from
    base_241129.fixed_stl_commit_stats
  where
    node_ordinal = -1;


create or replace view
  lf_241129.batches
as
  -- MG2 : keep in mind we have leader-node query only.batches, where workers are not involved
  --       also, workers finish after the final leader node work finishes, but the next batch can begin as soon as the final leader node is finished

  select
    batch_id::int8,
    min( startqueue )::timestamp                                                        as batch_startqueue,         -- earliest/first startqueue of batch
    min( case when node_ordinal  = -1 then startwork else null end )::timestamp         as batch_leader_startwork,   -- batch begins executing
    min( case when node_ordinal != -1 then startwork else null end )::timestamp         as batch_worker_startwork,   -- workers begin their local part
    max( endlocal )::timestamp                                                          as batch_endlocal,           -- 
    min( case when node_ordinal != -1 then startglobal else null end )::timestamp       as batch_worker_startglobal, -- workers begin their global part
    max( case when node_ordinal  = -1 then endtime     else null end )::timestamp       as batch_leader_endtime,     -- final leader work completely ends (which is the real end time of the batch)
    max( case when node_ordinal != -1 then endtime     else null end )::timestamp       as batch_worker_endtime,     -- workers finally finish
    max( endtime  )::timestamp                                                          as batch_endtime,            -- the very latest timestamp (when the workers end)

    ( batch_leader_startwork - batch_startqueue )::interval                             as batch_queue_duration,
    ( batch_endlocal - batch_worker_startwork )::interval                               as batch_worker_local_duration,
    ( batch_worker_endtime - batch_worker_startglobal )::interval                       as batch_worker_global_duration,
    ( batch_worker_local_duration + batch_worker_global_duration )::interval            as batch_worker_total_duration,
    ( batch_worker_local_duration + batch_worker_global_duration )::interval            as batch_worker_duration,
    sum( case when node_ordinal = -1 then endtime - startwork else null end )::interval as batch_leader_and_worker_local_duration,
    ( batch_leader_and_worker_local_duration - batch_worker_local_duration )::interval  as batch_leader_duration,
    ( batch_leader_endtime - batch_leader_startwork )::interval                         as batch_exec_duration, -- next batch begins after this duration
    ( batch_endtime - batch_leader_startwork )::interval                                as batch_total_duration,

    sum( dirtyblocks )::int8                                                            as number_batch_blocks,
    sum( headers )::int8                                                                as number_batch_headers,
    count( distinct transaction_id )::int8                                              as number_batch_transactions
  from
    base_241129.fixed_stl_commit_stats
  group by
    batch_id
  having
        batch_leader_startwork >= current_date - interval '6 day'
    and batch_endtime          <= current_timestamp;


create or replace view
  lf_241129.queries_transactions_batches
as
  select
    user_id::int4,
    user_name::varchar,
    session_id::int4,
    transaction_id::int8,
    transaction_start_ts::timestamp,
    query_id::int4,
    queue_id::int2,
    queue_name::varchar,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_queue_duration::interval,
    lf_241129.queries_compilation.query_compile_duration::interval,
    case
      when query_leader_node_flag is true  then null
      when query_leader_node_flag = false then query_compile_exec_duration - lf_241129.queries_compilation.query_compile_duration
    end::interval as query_exec_duration,
    query_wlm_duration::interval,
    query_total_duration::interval,
    query_abort_flag::bool,
    query_csc_flag::bool,
    query_leader_node_flag::bool,
    query_rc_flag::bool,
    query_text::varchar,
    query_number_blocks_read::int8,
    query_number_blocks_to_disk::int8,
    transaction_end_ts::timestamp,
    transaction_duration::interval,
    transaction_idle_duration::interval,
    transaction_number_leader_queries::int4,
    transaction_number_worker_queries::int4,
    transaction_number_total_queries::int4,
    transaction_abort_flag::bool,
    batch_id::int8,
    base_241129.transactions_in_batches.transaction_batch_queue_start::timestamp,
    ( batch_startqueue - base_241129.transactions_in_batches.transaction_batch_queue_start )::interval as transaction_batch_queue_duration,
    batch_leader_startwork::timestamp as batch_start_ts,
    batch_endtime::timestamp as batch_end_ts,
    batch_leader_duration::interval,
    batch_worker_duration::interval,
    batch_queue_duration::interval,
    batch_exec_duration::interval,
    batch_total_duration::interval,
    number_batch_blocks::int8,
    number_batch_headers::int8,
    number_batch_transactions::int8
  from
              base_241129.queries_transactions
         join base_241129.fixed_pg_user                      using ( user_id )
         join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
    left join lf_241129.queries_compilation                  using ( session_id, query_id, transaction_id )
    left join base_241129.queries_blocks                     using ( query_Id )
    left join base_241129.transactions_in_batches            using ( transaction_id )
    left join lf_241129.batches                              using ( batch_id );


create or replace view
  sf_241129.queries_transactions_batches
as
  select
    user_name::varchar   as uuser,
    session_id::int4     as pid,
    transaction_id::int8 as xid,
    query_id::int4       as qid,

    queue_name::varchar  as qname,

    query_start_ts::timestamp as qid_start_ts,
    extract( epoch from query_queue_duration   )::numeric(18,2) as queue,
    extract( epoch from query_compile_duration )::numeric(18,2) as compile,
    extract( epoch from query_exec_duration    )::numeric(18,2) as exec,
    extract( epoch from query_total_duration   )::numeric(18,2) as total,
    extract( epoch from (transaction_end_ts - query_end_ts) )::numeric(18,2) as xid_end,
    extract( epoch from (batch_end_ts - transaction_end_ts) )::numeric(18,2) as bid_end,
    (
      case when query_abort_flag       = true then 'A' else ' ' end || 
      case when query_csc_flag         = true then 'C' else ' ' end || 
      case when query_leader_node_flag = true then 'L' else ' ' end ||
      case when query_rc_flag          = true then 'R' else ' ' end 
    )::varchar as flags,

    query_number_blocks_read::int8 as br,
    query_number_blocks_to_disk::int8 as btd,

    batch_id::int8 as bid,

    query_text::varchar
  from
    lf_241129.queries_transactions_batches
  order by
    query_start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_slices_by_day
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'day', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal );


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_slices_by_day
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        slice_ordinal,
        slice_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_slices_by_day
    )

  select
    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,
    start_ts::date,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    slice_ordinal,
    start_ts;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_all_by_minute
as
  select
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_minutes
    left join base_241129.fixed_stl_query on date_trunc( 'minute', query_start_ts ) = base_241129.generate_minutes.start_ts
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_all_by_minute
as
  select
    start_ts::timestamp,
    number_abort::int4 as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_all_by_minute
  order by
    start_ts;


create or replace view
  lf_241129.queries_totals_result_cache_for_users_by_minute
as
  with
    cte_s1 as
    (
      select
        user_id,
        date_trunc( 'minute', query_start_ts )                                                                      as start_ts,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
        count(*)                                                                                                 as number_total,
        ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_start_ts   >= current_date - interval '6 day'
        and query_end_ts     <= current_timestamp
      group by
        user_id,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    number_hit::int8,
    number_miss::int8,
    number_total::int8,
    percentage::float4
  from
              cte_s1
    left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  sf_241129.queries_totals_result_cache_for_users_by_minute
as
  select
    user_name::varchar as uuser,
    start_ts::timestamp,
    number_hit::int8    as hit,
    number_miss::int8   as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_users_by_minute
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.vacuum_totals_for_tables_by_hour
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        table_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        table_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        table_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        table_id,
        date_trunc( 'hour', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        table_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        table_id,
        date_trunc( 'hour', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    cte_number_vacuums.table_id::int4,
    trim( pg_class.relname )::varchar     as table_name,

    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration                using ( table_id, start_ts )
    left join pg_class     on pg_class.oid          = cte_number_vacuums.table_id
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.locks
as
  with
    cte_table_transactions_locks as
    (
      select
        pg_namespace.oid                                as schema_id,
        trim( pg_namespace.nspname )                    as schema_name,
        pg_class.oid                                    as table_id,
        trim( pg_class.relname )                        as table_name,
        current_timestamp - svv_transactions.txn_start  as duration,
        trim( svv_transactions.lock_mode )              as lock_type,
        case
          when svv_transactions.granted = True  then 'granted'
          when svv_transactions.granted = False then 'requested'
        end                                             as lock_state,
        svv_transactions.pid                            as session_id,
        svv_transactions.xid                            as transaction_id,
        pg_user.usesysid                                as user_id,
        trim( pg_user.usename )                         as user_name,
        left( ltrim(stv_inflight.text), 48 )            as query_text
      from
                  svv_transactions
             join pg_class      on svv_transactions.relation  = pg_class.oid
             join pg_namespace  on pg_class.relnamespace      = pg_namespace.oid
             join pg_user       on svv_transactions.txn_owner = pg_user.usename
        left join stv_inflight  on svv_transactions.pid = stv_inflight.pid and
                                   svv_transactions.xid = stv_inflight.xid
      where
        lockable_object_type = 'relation'
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    duration::interval,
    lock_type::varchar,
    lock_state::varchar,
    session_id::int4,
    transaction_id::int8,
    user_id::int4,
    user_name::varchar,
    query_text::varchar
  from
    cte_table_transactions_locks;


create or replace view
  sf_241129.locks_by_transaction
as
  select
    user_name::varchar          as uuser,
    session_id::int4::int4      as pid,
    transaction_id::int8::int8  as xid,
    schema_name::varchar        as sschema,
    table_name::varchar         as ttable,
    extract( epoch from duration )::numeric(18,2) as dur,
    lock_type::varchar          as ttype,
    case
      when lock_state = 'granted'   then '✔'
      when lock_state = 'requested' then ''
    end::varchar                as s,
    query_text::varchar         as ttext
  from
    lf_241129.locks
  order by
    transaction_id,
    duration desc,
    ttype;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_schemas_by_second
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    start_ts::timestamp,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_scan.sum_scan_rows )::int8     as sum_scan_rows
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_seconds
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null
  group by
    base_241129.fixed_pg_namespace.schema_id,
    base_241129.fixed_pg_namespace.schema_name,
    start_ts;



create or replace view
  sf_241129.steps_totals_rows_on_workers_for_schemas_by_second
as
  select
    schema_name::varchar                                     as sschema,
    start_ts::timestamp,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_schemas_by_second
  order by
    schema_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_node_by_all
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    )

  select
    node_ordinal::int2,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               base_241129.fixed_stv_slices
     left join cte_aggr         using( slice_ordinal )
     left join cte_bcast        using( slice_ordinal )
     left join cte_delete       using( slice_ordinal )
     left join cte_dist         using( slice_ordinal )
     left join cte_hash         using( slice_ordinal )
     left join cte_hashjoin     using( slice_ordinal )
     left join cte_insert       using( slice_ordinal )
     left join cte_limit        using( slice_ordinal )
     left join cte_merge        using( slice_ordinal )
     left join cte_mergejoin    using( slice_ordinal )
     left join cte_nestloop     using( slice_ordinal )
     left join cte_parse        using( slice_ordinal )
     left join cte_project      using( slice_ordinal )
     left join cte_return       using( slice_ordinal )
     left join cte_save         using( slice_ordinal )
     left join cte_sort         using( slice_ordinal )
     left join cte_spatial_join using( slice_ordinal )
     left join cte_unique       using( slice_ordinal )
     left join cte_window       using( slice_ordinal )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null
  group by
    node_ordinal;


create or replace view
  sf_241129.steps_totals_data_on_both_for_node_by_all
as
  select
    node_ordinal::int2 as node,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_node_by_all
  order by
    node_ordinal;


create or replace view
  lf_241129.ttables_creation_for_schemas_by_day
as
  select
    trim( pg_namespace.nspname )::varchar           as schema_name,
    date_trunc( 'day', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    schema_name,
    start_ts;


create or replace view
  sf_241129.ttables_creation_for_schemas_by_day
as
  select
    schema_name::varchar as sschema,
    start_ts::date,
    number_tables::int4  as ttables
  from
    lf_241129.ttables_creation_for_schemas_by_day
  order by
    schema_name,
    start_ts;


create or replace view
  lf_241129.uusers_default_privileges
as
  with
    cte_s1 as
    (
      select
        defacluser                                       as user_id,
        pg_user.usename                                  as user_name,
        case
          when defaclnamespace = 0 then NULL else defaclnamespace
        end                                              as schema_id,
        pg_namespace.nspname                             as schema_name,
        case
          when defaclobjtype = 'r' then 'table'
          when defaclobjtype = 'f' then 'function'
          when defaclobjtype = 'p' then 'procedure'
        end                                              as object_type,
        defaclacl                                        as default_acl,
        generate_series( 1, array_upper(defaclacl, 1) )  as acl_index
      from
                  pg_default_acl
             join pg_user        on pg_default_acl.defacluser      = pg_user.usesysid
        left join pg_namespace   on pg_default_acl.defaclnamespace = pg_namespace.oid
    ),

    cte_s2 as
    (
      select
        user_id,
        user_name,
        schema_id,
        schema_name,
        object_type,
        array_to_string( default_acl[acl_index:acl_index], ',' ) as acl_item
      from
        cte_s1
    ),

    cte_s3 as
    (
      select
        user_id,
        user_name,
        schema_id,
        schema_name,
        object_type,
        acl_item,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then 'user'
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then 'group'
        end as acl_recipient_type,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then substring( acl_item, 1, strpos(acl_item,'=')-1 ) -- MG2 : acl_recipient_type 'user'
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'                                         --       acl_recipient_type 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then substring( acl_item, 7, strpos(acl_item,'=')-7 ) --       acl_recipient_type 'group'
        end as acl_raw_recipient_name
      from
        cte_s2
    ),

    cte_s4 as
    (
      select
        user_id,
        user_name,
        schema_id,
        schema_name,
        object_type,
        acl_recipient_type,
        case
          when left(acl_raw_recipient_name,1) = '"' and right(acl_raw_recipient_name,1) = '"' then substring( acl_raw_recipient_name from 2 for length(acl_raw_recipient_name)-2 )
                                                                                              else acl_raw_recipient_name
        end                                                               as acl_recipient_name,
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        cte_s3
    ),

    cte_s5 as
    (
      select
        user_id,
        user_name,
        schema_id,
        schema_name,
        object_type,
        acl_recipient_type,
        case
          when acl_recipient_type = 'user'   then pg_user.usesysid
          when acl_recipient_type = 'public' then null
          when acl_recipient_type = 'group'  then pg_group.grosysid
        end as acl_recipient_id,
        acl_recipient_name,
        acl_privileges
      from
                  cte_s4
        left join pg_user  on acl_recipient_type = 'user'  and acl_recipient_name = pg_user.usename
        left join pg_group on acl_recipient_type = 'group' and acl_recipient_name = pg_group.groname
    )

  select
    user_id::int4,
    user_name::varchar,
    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    acl_recipient_type::varchar as priv_recipient_type,
    acl_recipient_id::int4      as priv_recipient_id,
    acl_recipient_name::varchar as priv_recipient_name,

    -- MG2 : if the priv to grant to others is given, the underlying priv is always given as well, so for "with grant option"s, we only need to check for the priv letter plus the asterisk

    case
      when object_type = 'function'  and strpos(acl_privileges,'X')  != 0 then true else false
    end::bool          as priv_function_execute,

    case
      when object_type = 'function'  and strpos(acl_privileges,'X*') != 0 then true else false
    end::bool          as priv_function_grant_execute,

    case
      when object_type = 'procedure' and strpos(acl_privileges,'X')  != 0 then true else false
    end::bool          as priv_procedure_execute,

    case
      when object_type = 'procedure' and strpos(acl_privileges,'X*') != 0 then true else false
    end::bool          as priv_procedure_grant_execute,

    case
      when object_type = 'table'     and strpos(acl_privileges,'d')  != 0 then true else false
    end::bool          as priv_table_delete,

    case
      when object_type = 'table'     and strpos(acl_privileges,'d*') != 0 then true else false
    end::bool          as priv_table_grant_delete,

    case
      when object_type = 'table'     and strpos(acl_privileges,'D')  != 0 then true else false
    end::bool          as priv_table_drop,

    case
      when object_type = 'table'     and strpos(acl_privileges,'D*') != 0 then true else false
    end::bool          as priv_table_grant_drop,

    case
      when object_type = 'table'     and strpos(acl_privileges,'a')  != 0 then true else false
    end::bool          as priv_table_insert,

    case
      when object_type = 'table'     and strpos(acl_privileges,'a*') != 0 then true else false
    end::bool          as priv_table_grant_insert,

    case
      when object_type = 'table'     and strpos(acl_privileges,'x')  != 0 then true else false
    end::bool          as priv_table_reference,

    case
      when object_type = 'table'     and strpos(acl_privileges,'x*') != 0 then true else false
    end::bool          as priv_table_grant_reference,

    case
      when object_type = 'table'     and strpos(acl_privileges,'r')  != 0 then true else false
    end::bool          as priv_table_select,

    case
      when object_type = 'table'     and strpos(acl_privileges,'r*') != 0 then true else false
    end::bool          as priv_table_grant_select,

    case
      when object_type = 'table'     and strpos(acl_privileges,'w')  != 0 then true else false
    end::bool          as priv_table_update,

    case
      when object_type = 'table'     and strpos(acl_privileges,'w*') != 0 then true else false
    end::bool          as priv_table_grant_update
  from
    cte_s5;


create or replace view
  sf_241129.uusers_default_privileges
as
  with
    cte_user_default_privileges as
    (
      select
        user_name,
        schema_name,
        object_type,
        priv_recipient_type,
        priv_recipient_name,

        case
          when object_type = 'table' and priv_table_grant_delete = True                                then 'D'
          when object_type = 'table' and priv_table_grant_delete = False and priv_table_delete = True  then 'd'
          when object_type = 'table' and priv_table_grant_delete = False and priv_table_delete = False then ' '
                                                                                                        else ' '
        end

        ||

        case
          when object_type = 'table' and priv_table_grant_reference = True                                   then 'F'
          when object_type = 'table' and priv_table_grant_reference = False and priv_table_reference = True  then 'f'
          when object_type = 'table' and priv_table_grant_reference = False and priv_table_reference = False then ' '
                                                                                                              else ' '
        end

        ||

        case
          when object_type = 'table' and priv_table_grant_drop = True                              then 'P'
          when object_type = 'table' and priv_table_grant_drop = False and priv_table_drop = True  then 'p'
          when object_type = 'table' and priv_table_grant_drop = False and priv_table_drop = False then ' '
                                                                                                    else ' '
        end

        ||

        case
          when object_type = 'table' and priv_table_grant_insert = True                                then 'I'
          when object_type = 'table' and priv_table_grant_insert = False and priv_table_insert = True  then 'i'
          when object_type = 'table' and priv_table_grant_insert = False and priv_table_insert = False then ' '
                                                                                                        else ' '
        end

        ||

        case
          when object_type = 'table' and priv_table_grant_select = True                                then 'S'
          when object_type = 'table' and priv_table_grant_select = False and priv_table_select = True  then 's'
          when object_type = 'table' and priv_table_grant_select = False and priv_table_select = False then ' '
                                                                                                        else ' '
        end

        ||

        case
          when object_type = 'table' and priv_table_grant_update = True                                then 'U'
          when object_type = 'table' and priv_table_grant_update = False and priv_table_update = True  then 'u'
          when object_type = 'table' and priv_table_grant_update = False and priv_table_update = False then ' '
                                                                                                        else ' '
        end

        ||

        case
          when object_type in ('function', 'procedure') and priv_function_grant_execute = True                                    then 'E'
          when object_type in ('function', 'procedure') and priv_function_grant_execute = False and priv_function_execute = True  then 'e'
          when object_type in ('function', 'procedure') and priv_function_grant_execute = False and priv_function_execute = False then ' '
                                                                                                                                    else ' '
        end as privs
      from
        lf_241129.uusers_default_privileges
    )

  select
    user_name::varchar           as uuser,
    schema_name::varchar         as sschema,
    object_type::varchar         as ttype,
    priv_recipient_type::varchar as recipient_type,
    priv_recipient_name::varchar as recipient_name,
    privs::varchar
  from
    cte_user_default_privileges
  order by
    user_name,
    schema_name,
    object_type,
    priv_recipient_type,
    priv_recipient_name;


create or replace view
  base_241129.fixed_stl_load_errors
as
  select
    -- col_length::char(10),
    trim( colname )::varchar    as column_name,
    -- copy_job_id::int8,
    -- err_code::int4,
    trim( err_reason )::varchar as reason,
    trim( filename )::varchar   as file_name,
    -- is_partial::int4,
    line_number::int8,
    -- position::int4
    query::int4                 as query_id,
    trim( raw_field_value )::varchar as field_value,
    -- raw_line::char(1024),
    -- session::int4,
    slice::int2                 as slice_ordinal,
    -- start_offset::int8,
    starttime::timestamp        as event_ts,
    tbl::int4                   as table_id,
    trim( type )::varchar       as column_full_data_type_name,
    userid::int4                as user_id
  from
    stl_load_errors
  where
        event_ts >= current_date - interval '6 day'
    and event_ts <= current_timestamp;


create or replace view
  lf_241129.utils_most_recent_copy_error_for_current_user
as
  select
    event_ts::timestamp,
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    column_name::varchar,
    column_full_data_type_name::varchar,
    line_number::varchar,
    field_value::varchar,
    reason::varchar,
    file_name::varchar
  from
         base_241129.fixed_stl_load_errors
    join base_241129.fixed_pg_class_for_tables using ( table_id )
    join base_241129.fixed_pg_namespace        using ( schema_id )
  where
    base_241129.fixed_stl_load_errors.user_id = current_user_id
  order by
    event_ts desc
  limit 1;


create or replace view
  sf_241129.utils_most_recent_copy_error_for_current_user
as
  with
    cte_rows as
    (
      select
        row_number() over () as rn,
        event_ts,
        ( schema_name || '.' || table_name || '.' || column_name || '( ' || column_full_data_type_name || ' )' ) as ccolumn,
        column_full_data_type_name as data_type,
        reason,
        file_name,
        line_number,
        field_value
      from
                   lf_241129.utils_most_recent_copy_error_for_current_user
        cross join base_241129.numbers
      limit 7
    )

  select
    case
      when rn = 1 then 'event_ts'
      when rn = 2 then 'column'
      when rn = 3 then 'data_type'
      when rn = 4 then 'reason'
      when rn = 5 then 'file_name'
      when rn = 6 then 'line'
      when rn = 7 then 'value'
                  else null
    end::varchar as kkey,

    case
      when rn = 1 then event_ts::varchar
      when rn = 2 then ccolumn
      when rn = 3 then data_type
      when rn = 4 then reason
      when rn = 5 then file_name
      when rn = 6 then line_number::varchar
      when rn = 7 then field_value
                  else null
    end::varchar as value
  from
    cte_rows
  order by
    rn;

/*

  -- MG2 : I'd like to use this code, but it's not postgres compliant
  --       AWS do not publish the RS BNF, so I can't parse this code
  --       and parsing my SQL central to documentation generation and validation

  select
    *
  from
    (
      select
        event_ts::varchar,
        ( schema_name || '.' || table_name || '.' || column_name || '( ' || column_full_data_type_name || ' )' )::varchar as ccolumn,
        reason::varchar,
        file_name::varchar,
        line_number::varchar  as line,
        field_value::varchar  as value
      from
        lf_241129.util_most_recent_copy_error_for_current_user
    )
    unpivot
    (
      key for value in ( reason, ccolumn, reason, file_name, line, value )
    );

*/



create or replace view
  sf_241129.analyze_totals_for_users_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_users_by_minute
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;



create or replace view
  lf_241129.steps_stats_durations_on_both_for_tables_by_hour
as
  with
    cte_aggr as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    node_type::varchar,
    base_241129.generate_hours.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_hours
    cross join base_241129.fixed_pg_class_for_tables
     left join base_241129.fixed_pg_namespace using ( schema_id )
     left join cte_aggr_stats          using ( start_ts, node_type, table_id )
     left join cte_delete_stats        using ( start_ts, node_type, table_id )
     left join cte_hashjoin_stats      using ( start_ts, node_type, table_id )
     left join cte_insert_stats        using ( start_ts, node_type, table_id )
     left join cte_mergejoin_stats     using ( start_ts, node_type, table_id )
     left join cte_nestloop_stats      using ( start_ts, node_type, table_id )
     left join cte_save_stats          using ( start_ts, node_type, table_id )
     left join cte_sort_stats          using ( start_ts, node_type, table_id )
     left join cte_spatial_join_stats  using ( start_ts, node_type, table_id )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_hour_complete
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_hour
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;



create or replace procedure
  lf_241129.sp_create_role_if_not_exists( role_name in varchar(127) )
as
  $proc$
    declare
      msg  varchar(4096);
      row  record;
      sql  varchar(4096);

    begin
      -- MG2 : check the role exists; if not, make it
      sql := 'select count(*) as number_roles from svv_roles where role_name = ' || $$'$$ || role_name || $$'$$ || ';';
      execute sql into row;

      if row.number_roles = 0 then
        sql := 'create role ' || role_name || 'nologin;';
        execute sql;
        msg := 'Role "' || role_name || '" created.';
      else
        msg := 'Role "' || role_name || '" already exists.';
      end if;

      raise info '%', msg;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.steps_totals_data_on_both_for_users_by_minute
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4           as usesysid,
    trim( pg_user.usename )::varchar as user_name,
    cte_node_types.node_type::varchar,
    base_241129.generate_minutes.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               pg_user
    cross join cte_node_types
    cross join base_241129.generate_minutes
     left join cte_aggr         using( node_type, usesysid, start_ts )
     left join cte_bcast        using( node_type, usesysid, start_ts )
     left join cte_delete       using( node_type, usesysid, start_ts )
     left join cte_dist         using( node_type, usesysid, start_ts )
     left join cte_hash         using( node_type, usesysid, start_ts )
     left join cte_hashjoin     using( node_type, usesysid, start_ts )
     left join cte_insert       using( node_type, usesysid, start_ts )
     left join cte_limit        using( node_type, usesysid, start_ts )
     left join cte_merge        using( node_type, usesysid, start_ts )
     left join cte_mergejoin    using( node_type, usesysid, start_ts )
     left join cte_nestloop     using( node_type, usesysid, start_ts )
     left join cte_parse        using( node_type, usesysid, start_ts )
     left join cte_project      using( node_type, usesysid, start_ts )
     left join cte_return       using( node_type, usesysid, start_ts )
     left join cte_save         using( node_type, usesysid, start_ts )
     left join cte_sort         using( node_type, usesysid, start_ts )
     left join cte_spatial_join using( node_type, usesysid, start_ts )
     left join cte_unique       using( node_type, usesysid, start_ts )
     left join cte_window       using( node_type, usesysid, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.steps_totals_data_on_both_for_users_by_minute
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_users_by_minute
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  base_241129.ttables_activity_full
as
  with
    cte_delete as
    (
      select
        stl_delete.tbl         as table_id,
        sum( stl_delete.rows ) as number_rows_deleted
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and starttime <= current_timestamp
      group by
        stl_delete.tbl
    ),

    cte_insert as
    (
      select
        stl_insert.tbl         as table_id,
        sum( stl_insert.rows ) as number_rows_inserted
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and starttime <= current_timestamp
      group by
        stl_insert.tbl
    ),

    cte_scan as
    (
      select
        stl_scan.tbl         as table_id,
        sum( stl_scan.rows ) as number_rows_scanned
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and starttime <= current_timestamp
      group by
        stl_scan.tbl
    )

  select
    table_id::int4,
    cte_delete.number_rows_deleted::int8,
    cte_insert.number_rows_inserted::int8,
    cte_scan.number_rows_scanned::int8
  from
              base_241129.fixed_pg_class_for_tables
    left join cte_delete using ( table_id )
    left join cte_insert using ( table_id )
    left join cte_scan   using ( table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null;


create or replace view
  lf_241129.queries_text_on_workers_fast
as
  with
    cte_query_text_worker_fast as
    (
      select
        session_id,
        transaction_id,
        query_id,
        -- MG2 : query_start_ts is presented for use by query_text_all_fast/full
        query_start_ts,
        query_text
      from
        base_241129.fixed_stl_query
    )

  select
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    query_start_ts::timestamp,
    query_text::varchar
  from
    cte_query_text_worker_fast;


create or replace function
  base_241129.hr_blks( blocks int8 ) returns varchar immutable
as
  $$
    select
      case
        when abs($1) >=          0 and abs($1) <          1024 then ($1                     )::numeric(18,0)::varchar || 'mb'
        when abs($1) >=       1024 and abs($1) <       1048576 then ($1::float4 / 1024      )::numeric(18,1)::varchar || 'gb'
        when abs($1) >=    1048576 and abs($1) <    1073741824 then ($1::float4 / 1048576   )::numeric(18,1)::varchar || 'tb'
        when abs($1) >= 1073741824 and abs($1) < 1099511627776 then ($1::float4 / 1073741824)::numeric(18,1)::varchar || 'pb'
        when $1 is null                                        then null
      end
  $$
language sql;

create or replace function
  base_241129.hr_blks( blocks float4 ) returns varchar immutable
as
  $$
    select
      case
        when abs($1) >=          0 and abs($1) <          1024 then ($1             )::numeric(18,0)::varchar || 'mb'
        when abs($1) >=       1024 and abs($1) <       1048576 then ($1 / 1024      )::numeric(18,1)::varchar || 'gb'
        when abs($1) >=    1048576 and abs($1) <    1073741824 then ($1 / 1048576   )::numeric(18,1)::varchar || 'tb'
        when abs($1) >= 1073741824 and abs($1) < 1099511627776 then ($1 / 1073741824)::numeric(18,1)::varchar || 'pb'
        when $1 is null                                        then null
      end
  $$
language sql;


create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_all_by_second
as
  with
    cte_blocks as
    (
      select
        date_trunc( 'second', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;

create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_all_by_second
as
  select
    start_ts::timestamp,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_all_by_second
  order by
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_users_by_second
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4           as usesysid,
    trim( pg_user.usename )::varchar as user_name,
    cte_node_types.node_type::varchar,
    base_241129.generate_seconds.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               pg_user
    cross join cte_node_types
    cross join base_241129.generate_seconds
     left join cte_aggr         using( node_type, usesysid, start_ts )
     left join cte_bcast        using( node_type, usesysid, start_ts )
     left join cte_delete       using( node_type, usesysid, start_ts )
     left join cte_dist         using( node_type, usesysid, start_ts )
     left join cte_hash         using( node_type, usesysid, start_ts )
     left join cte_hashjoin     using( node_type, usesysid, start_ts )
     left join cte_insert       using( node_type, usesysid, start_ts )
     left join cte_limit        using( node_type, usesysid, start_ts )
     left join cte_merge        using( node_type, usesysid, start_ts )
     left join cte_mergejoin    using( node_type, usesysid, start_ts )
     left join cte_nestloop     using( node_type, usesysid, start_ts )
     left join cte_parse        using( node_type, usesysid, start_ts )
     left join cte_project      using( node_type, usesysid, start_ts )
     left join cte_return       using( node_type, usesysid, start_ts )
     left join cte_save         using( node_type, usesysid, start_ts )
     left join cte_sort         using( node_type, usesysid, start_ts )
     left join cte_spatial_join using( node_type, usesysid, start_ts )
     left join cte_unique       using( node_type, usesysid, start_ts )
     left join cte_window       using( node_type, usesysid, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  lf_241129.analyze_totals_for_tables_by_hour
as
  with
    cte_s1 as
    (
      select
        table_id,
        date_trunc( 'hour', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            database   = current_database()
        and starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4::int4              as table_id,
    trim( pg_class.relname )::varchar     as table_name,

    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_statements_by_all
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query as query_id
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query as query_id
      from
        stl_unload_log
    ),

    cte_queries as
    (
      select
        cte_base.statement_name,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
             cte_base
        join base_241129.fixed_stl_wlm_query using ( query_id )
        join base_241129.fixed_stl_query     using ( query_id )
        join base_241129.queries_compilation   using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        statement_name,

        min( query_queue_duration ) over ( partition by statement_name )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by statement_name )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by statement_name )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by statement_name )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by statement_name )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by statement_name )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by statement_name )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by statement_name )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by statement_name )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by statement_name )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by statement_name )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by statement_name )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by statement_name ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by statement_name )                            as sum_query_exec_duration,

        count(*) over ( partition by statement_name ) as number_queries
      from
        cte_queries
    )

  select
    statement_name::varchar,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
    cte_queries_stats
  group by
    statement_name,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace procedure
  ro.sp_revoke_version_from_group( version in varchar(6), group_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'revoke usage on schema base_' || version || ' from group ' || group_name;
      execute sql;

      sql := 'revoke usage on schema lf_' || version || ' from group ' || group_name;
      execute sql;

      sql := 'revoke usage on schema sf_' || version || ' from group ' || group_name;
      execute sql;

      sql := 'revoke select on all tables in schema base_' || version || ' from group ' || group_name;
      execute sql;

      sql := 'revoke select on all tables in schema lf_' || version || ' from group ' || group_name;
      execute sql;

      sql := 'revoke select on all tables in schema sf_' || version || ' from group ' || group_name;
      execute sql;

      sql := 'revoke execute on all functions  in schema base_' || version || ' from group ' || group_name;
      execute sql;

      sql := 'revoke execute on all procedures in schema lf_' || version || ' from group ' || group_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_statements_by_second
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query as query_id
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query as query_id
      from
        stl_unload_log
    ),

    cte_queries as
    (
      select
        cte_base.statement_name,
        date_trunc( 'second', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
             cte_base
        join base_241129.fixed_stl_wlm_query using ( query_id )
        join base_241129.fixed_stl_query     using ( query_id )
        join base_241129.queries_compilation   using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        statement_name,
        start_ts,

        min( query_queue_duration ) over ( partition by statement_name, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by statement_name, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by statement_name, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by statement_name, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by statement_name, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by statement_name, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by statement_name, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by statement_name, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by statement_name, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by statement_name, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by statement_name, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by statement_name, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by statement_name, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    statement_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_seconds
    left join cte_queries_stats using ( start_ts )
  group by
    statement_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_statements_by_second
as

  with
    cte_hr_formatting as
    (
      select
        statement_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_statements_by_second
    )

  select
    statement_name::varchar as stmt,
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    statement_name,
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_node_by_second
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
             cte_aggr
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
             cte_bcast
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
             cte_delete
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
             cte_dist
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
             cte_hash
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
             cte_hashjoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
             cte_insert
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
             cte_limit
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
             cte_merge
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
             cte_mergejoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
             cte_nestloop
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
             cte_parse
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
             cte_project
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
             cte_return
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
             cte_save
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
             cte_sort
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
             cte_spatial_join
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
             cte_unique
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
             cte_window
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    )

  select
    base_241129.generate_seconds.start_ts::timestamp as start_ts,
    base_241129.fixed_stv_slices.node_ordinal::int2,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_seconds
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, node_ordinal )
     left join cte_bcast_stats            using ( start_ts, node_ordinal )
     left join cte_delete_stats           using ( start_ts, node_ordinal )
     left join cte_dist_stats             using ( start_ts, node_ordinal )
     left join cte_hash_stats             using ( start_ts, node_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, node_ordinal )
     left join cte_insert_stats           using ( start_ts, node_ordinal )
     left join cte_limit_stats            using ( start_ts, node_ordinal )
     left join cte_merge_stats            using ( start_ts, node_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, node_ordinal )
     left join cte_nestloop_stats         using ( start_ts, node_ordinal )
     left join cte_parse_stats            using ( start_ts, node_ordinal )
     left join cte_project_stats          using ( start_ts, node_ordinal )
     left join cte_return_stats           using ( start_ts, node_ordinal )
     left join cte_save_stats             using ( start_ts, node_ordinal )
     left join cte_sort_stats             using ( start_ts, node_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, node_ordinal )
     left join cte_unique_stats           using ( start_ts, node_ordinal )
     left join cte_window_stats           using ( start_ts, node_ordinal );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_second_concise
as
  select
    start_ts::timestamp,
    node_ordinal::int2 as n,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_second
  order by
    start_ts,
    node_ordinal;


create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_users_by_hour
as
  with
    cte_blocks as
    (
      select
        user_id,
        user_name,
        date_trunc( 'hour', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        user_name,
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        user_id,
        user_name,
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_hour_concise
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_hour
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_schemas_by_hour
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'hour', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r'
  group by
    schema_id,
    schema_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.queries_text_for_leader
as
  with
    cte_query_text_leader as
    (
      select
        session_id,
        transaction_id,
        query_start_ts,
        sequence / (3000 / 200)                                                                  as query_text_order,
        replace( listagg(query_text) within group (order by sequence), chr(92) || 'n', chr(10) ) as query_text
      from
        base_241129.fixed_stl_ddltext
      group by
        session_id,
        transaction_id,
        query_start_ts,
        sequence / (3000 / 200)

      union all

      select
        session_id,
        transaction_id,
        query_start_ts,
        sequence / (3000 / 200)                                                                  as query_text_order,
        replace( listagg(query_text) within group (order by sequence), chr(92) || 'n', chr(10) ) as query_text
      from
        base_241129.fixed_stl_utilitytext
      group by
        session_id,
        transaction_id,
        query_start_ts,
        sequence / (3000 / 200)
    )

  select
    session_id::int4,
    transaction_id::int8,
    query_start_ts::timestamp,
    query_text_order::int4,
    query_text::varchar
  from
    cte_query_text_leader;


create or replace view
  sf_241129.queries_text_for_leader
as
  select
    session_id::int4       as pid,
    transaction_id::int8   as xid,
    query_start_ts::timestamp as start_ts,
    query_text_order::int4 as n,
    trim( chr(10) || ' ' from query_text )::varchar as ttext
  from
    lf_241129.queries_text_for_leader
  order by
    session_id,
    transaction_id,
    query_start_ts,
    query_text_order;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_all_by_hour
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_scan as
    (
      select
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    )

  select
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
         base_241129.generate_hours
    left join cte_delete using ( start_ts )
    left join cte_insert using ( start_ts )
    left join cte_scan   using ( start_ts );


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_all_by_hour
as
  select
    start_ts::timestamp,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_all_by_hour
  order by
    start_ts;



create or replace view
  lf_241129.steps_stats_durations_on_both_for_tables_by_second
as
  with
    cte_aggr as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    node_type::varchar,
    base_241129.generate_seconds.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_seconds
    cross join base_241129.fixed_pg_class_for_tables
     left join base_241129.fixed_pg_namespace using ( schema_id )
     left join cte_aggr_stats          using ( start_ts, node_type, table_id )
     left join cte_delete_stats        using ( start_ts, node_type, table_id )
     left join cte_hashjoin_stats      using ( start_ts, node_type, table_id )
     left join cte_insert_stats        using ( start_ts, node_type, table_id )
     left join cte_mergejoin_stats     using ( start_ts, node_type, table_id )
     left join cte_nestloop_stats      using ( start_ts, node_type, table_id )
     left join cte_save_stats          using ( start_ts, node_type, table_id )
     left join cte_sort_stats          using ( start_ts, node_type, table_id )
     left join cte_spatial_join_stats  using ( start_ts, node_type, table_id )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_queues_by_all
as
  with
    cte_queries as
    (
      select
        queue_id,
        date_trunc( 'day', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
                  base_241129.fixed_stl_wlm_query
        left join base_241129.fixed_stl_query    using ( query_id )
        left join base_241129.queries_compilation using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        queue_id,

        min( query_queue_duration ) over ( partition by queue_id )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by queue_id )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by queue_id )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by queue_id )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by queue_id )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by queue_id )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by queue_id )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by queue_id )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by queue_id )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by queue_id )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by queue_id )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by queue_id )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by queue_id ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by queue_id )                            as sum_query_exec_duration,

        count(*) over ( partition by queue_id ) as number_queries
      from
        cte_queries
    )

  select
    queue_id::int2,
    queue_name::varchar,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              cte_queries_stats
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_queues_by_all
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_queues_by_all
    )

  select
    queue_name::varchar as qname,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    queue_name;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_node_by_hour
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
             cte_aggr
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
             cte_bcast
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
             cte_delete
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
             cte_dist
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
             cte_hash
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
             cte_hashjoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
             cte_insert
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
             cte_limit
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
             cte_merge
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
             cte_mergejoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
             cte_nestloop
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
             cte_parse
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
             cte_project
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
             cte_return
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
             cte_save
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
             cte_sort
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
             cte_spatial_join
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
             cte_unique
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
             cte_window
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    )

  select
    base_241129.generate_hours.start_ts::timestamp as start_ts,
    base_241129.fixed_stv_slices.node_ordinal::int2,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_hours
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, node_ordinal )
     left join cte_bcast_stats            using ( start_ts, node_ordinal )
     left join cte_delete_stats           using ( start_ts, node_ordinal )
     left join cte_dist_stats             using ( start_ts, node_ordinal )
     left join cte_hash_stats             using ( start_ts, node_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, node_ordinal )
     left join cte_insert_stats           using ( start_ts, node_ordinal )
     left join cte_limit_stats            using ( start_ts, node_ordinal )
     left join cte_merge_stats            using ( start_ts, node_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, node_ordinal )
     left join cte_nestloop_stats         using ( start_ts, node_ordinal )
     left join cte_parse_stats            using ( start_ts, node_ordinal )
     left join cte_project_stats          using ( start_ts, node_ordinal )
     left join cte_return_stats           using ( start_ts, node_ordinal )
     left join cte_save_stats             using ( start_ts, node_ordinal )
     left join cte_sort_stats             using ( start_ts, node_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, node_ordinal )
     left join cte_unique_stats           using ( start_ts, node_ordinal )
     left join cte_window_stats           using ( start_ts, node_ordinal );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_hour_concise
as
  select
    start_ts::timestamp,
    node_ordinal::int2 as n,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_hour
  order by
    start_ts,
    node_ordinal;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_all_by_minute
as
  with
    cte_transactions as
    (
      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        transaction_id,
        date_trunc( 'minute', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        transaction_id
    )

  select
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
  group by
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_slices_by_day
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        start_ts,
        slice_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        start_ts,
        slice_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        start_ts,
        slice_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        start_ts,
        slice_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        start_ts,
        slice_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        start_ts,
        slice_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        start_ts,
        slice_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        start_ts,
        slice_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        start_ts,
        slice_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        start_ts,
        slice_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        start_ts,
        slice_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        start_ts,
        slice_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        start_ts,
        slice_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        start_ts,
        slice_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        start_ts,
        slice_ordinal
    )

  select
    base_241129.generate_days.start_ts::timestamp as start_ts,

    base_241129.fixed_stv_slices.node_ordinal::int2,
    base_241129.fixed_stv_slices.slice_ordinal::int2,
    base_241129.fixed_stv_slices.slice_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_days
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, slice_ordinal )
     left join cte_bcast_stats            using ( start_ts, slice_ordinal )
     left join cte_delete_stats           using ( start_ts, slice_ordinal )
     left join cte_dist_stats             using ( start_ts, slice_ordinal )
     left join cte_hash_stats             using ( start_ts, slice_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, slice_ordinal )
     left join cte_insert_stats           using ( start_ts, slice_ordinal )
     left join cte_limit_stats            using ( start_ts, slice_ordinal )
     left join cte_merge_stats            using ( start_ts, slice_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, slice_ordinal )
     left join cte_nestloop_stats         using ( start_ts, slice_ordinal )
     left join cte_parse_stats            using ( start_ts, slice_ordinal )
     left join cte_project_stats          using ( start_ts, slice_ordinal )
     left join cte_return_stats           using ( start_ts, slice_ordinal )
     left join cte_save_stats             using ( start_ts, slice_ordinal )
     left join cte_sort_stats             using ( start_ts, slice_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, slice_ordinal )
     left join cte_unique_stats           using ( start_ts, slice_ordinal )
     left join cte_window_stats           using ( start_ts, slice_ordinal );


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_users_by_second
as
  with
    cte_s1 as
    (
      select
        user_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'second', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        user_id,
        node_type,
        start_ts
    )

  select
    user_id::int4,
    node_type::varchar,
    trim( pg_user.usename )::varchar as user_name,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_user on cte_s1.user_id = pg_user.usesysid;


create or replace view
  base_241129.privileges_users
as
  with
    cte_pg_namespace as
    (
      select
        schema_id,
        schema_name,
        'schema'          as object_type,
        null::oid         as object_return_base_data_type_id,
        null::varchar     as object_return_base_data_type_name,
        schema_id         as object_id,
        schema_name       as object_name,
        null::varchar     as object_argument_base_data_type_ids,
        null::varchar     as object_argument_base_data_type_names,
        schema_owner_id   as object_owner_user_id,
        schema_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 1, strpos(acl_item,'=')-1 ) as acl_recipient_user_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            schema_owner_id,
            schema_owner_name,
            schema_acl,
            schema_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when schema_acl is     null then array_to_string( schema_default_acl[acl_index:acl_index], ',' )
              when schema_acl is not null then array_to_string( schema_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid                                       as schema_id,
                pg_namespace.nspname                                   as schema_name,
                pg_namespace.nspowner                                  as schema_owner_id,
                pg_user.usename                                        as schema_owner_name,
                pg_namespace.nspacl                                    as schema_acl,
                -- MG2 : I would be alphabetic, but Postgres/Redshift uses 'UC'
                array[ pg_user.usename || '=UC/' || pg_user.usename ]  as schema_default_acl,
                generate_series( 1,
                                 case
                                   when schema_acl is     null then array_upper( schema_default_acl, 1 )
                                   when schema_acl is not null then array_upper( schema_acl, 1 )
                                 end ) as acl_index
              from
                     pg_namespace
                join pg_user on pg_namespace.nspowner = pg_user.usesysid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '='
        ) as subquery_3
    ),

    cte_pg_class as
    (
      select
        schema_id,
        schema_name,
        case
          when class_type = 'r' then 'table'
          when class_type = 'v' then 'view'
        end              as object_type,
        null::oid        as object_return_base_data_type_id,
        null::varchar    as object_return_base_data_type_name,
        class_id         as object_id,
        class_name       as object_name,
        null::varchar    as object_argument_base_data_type_ids,
        null::varchar    as object_argument_base_data_type_names,
        class_owner_id   as object_owner_user_id,
        class_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 1, strpos(acl_item,'=')-1 ) as acl_recipient_user_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            class_id,
            class_type,
            class_name,
            class_owner_id,
            class_owner_name,
            class_acl,
            class_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when class_acl is     null then array_to_string( class_default_acl[acl_index:acl_index], ',' )
              when class_acl is not null then array_to_string( class_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_class.oid         as class_id,
                pg_class.relkind     as class_type,
                pg_class.relname     as class_name,
                pg_class.relowner    as class_owner_id,
                pg_user.usename      as class_owner_name,
                pg_class.relacl      as class_acl,
                  -- MG2 : 'arwdRxtD' is the ACL string Redshift procuces; I could curtail it to only those which exist on Redshift, but matching RS makes testing easier
                array[ pg_user.usename || '=arwdRxtD/' || pg_user.usename ] as class_default_acl,
                generate_series( 1,
                                 case
                                   when class_acl is     null then array_upper( class_default_acl, 1 )
                                   when class_acl is not null then array_upper( class_acl, 1 )
                                 end ) as acl_index
              from
                     pg_class
                join pg_namespace on pg_class.relnamespace = pg_namespace.oid
                join pg_user      on pg_class.relowner     = pg_user.usesysid
              where
                pg_class.relkind in ( 'r', 'v' )
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '='
        ) as subquery_3
    ),

    cte_pg_proc as
    (
      select
        schema_id,
        schema_name,
        case
          when proctype = 'c'         then 'C func'
          when proctype = 'exfunc'    then 'lambda func'
          when proctype = 'internal'  then 'internal func'
          when proctype = 'mlfunc'    then 'ML func'
          when proctype = 'plpgsql'   then 'procedure'
          when proctype = 'plpythonu' then 'python func'
          when proctype = 'sql'       then 'SQL func'
                                       else trim( proctype )
        end                            as object_type,
        procreturn_data_type_id       as object_return_base_data_type_id,
        procreturn_data_type_name     as object_return_base_data_type_name,
        procid                        as object_id,
        procname                      as object_name,
        procargument_data_type_ids    as object_argument_base_data_type_ids,
        procargument_data_type_names  as object_argument_base_data_type_names,
        procowner_id                  as object_owner_user_id,
        procowner_name                as object_owner_user_name,
        acl_item,
        substring( acl_item, 1, strpos(acl_item,'=')-1 ) as acl_recipient_user_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            procid,
            proctype,
            procreturn_data_type_id,
            procreturn_data_type_name,
            procname,
            procargument_data_type_ids,
            procargument_data_type_names,
            procowner_id,
            procowner_name,
            procacl,
            procdefault_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when procacl is     null then array_to_string( procdefault_acl[acl_index:acl_index], ',' )
              when procacl is not null then array_to_string( procacl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_proc.oid          as procid,
                pg_language.lanname  as proctype,
                pg_proc.prorettype   as procreturn_data_type_id,
                case
                  when pg_proc.proretset = True  then '_' || pg_type_return.typname
                  when pg_proc.proretset = False then        pg_type_return.typname
                end                  as procreturn_data_type_name,
                pg_proc.proname      as procname,

                                                                         pg_type_arg_00.oid::varchar     ||
                case when pg_type_arg_01.oid is null then '' else ',' || pg_type_arg_01.oid::varchar end ||
                case when pg_type_arg_02.oid is null then '' else ',' || pg_type_arg_02.oid::varchar end ||
                case when pg_type_arg_03.oid is null then '' else ',' || pg_type_arg_03.oid::varchar end ||
                case when pg_type_arg_04.oid is null then '' else ',' || pg_type_arg_04.oid::varchar end ||
                case when pg_type_arg_05.oid is null then '' else ',' || pg_type_arg_05.oid::varchar end ||
                case when pg_type_arg_06.oid is null then '' else ',' || pg_type_arg_06.oid::varchar end ||
                case when pg_type_arg_07.oid is null then '' else ',' || pg_type_arg_07.oid::varchar end ||
                case when pg_type_arg_08.oid is null then '' else ',' || pg_type_arg_08.oid::varchar end ||
                case when pg_type_arg_09.oid is null then '' else ',' || pg_type_arg_09.oid::varchar end ||
                case when pg_type_arg_10.oid is null then '' else ',' || pg_type_arg_10.oid::varchar end ||
                case when pg_type_arg_11.oid is null then '' else ',' || pg_type_arg_11.oid::varchar end ||
                case when pg_type_arg_12.oid is null then '' else ',' || pg_type_arg_12.oid::varchar end ||
                case when pg_type_arg_13.oid is null then '' else ',' || pg_type_arg_13.oid::varchar end ||
                case when pg_type_arg_14.oid is null then '' else ',' || pg_type_arg_14.oid::varchar end ||
                case when pg_type_arg_15.oid is null then '' else ',' || pg_type_arg_15.oid::varchar end ||
                case when pg_type_arg_16.oid is null then '' else ',' || pg_type_arg_16.oid::varchar end ||
                case when pg_type_arg_17.oid is null then '' else ',' || pg_type_arg_17.oid::varchar end ||
                case when pg_type_arg_18.oid is null then '' else ',' || pg_type_arg_18.oid::varchar end ||
                case when pg_type_arg_19.oid is null then '' else ',' || pg_type_arg_19.oid::varchar end ||
                case when pg_type_arg_20.oid is null then '' else ',' || pg_type_arg_20.oid::varchar end ||
                case when pg_type_arg_21.oid is null then '' else ',' || pg_type_arg_21.oid::varchar end ||
                case when pg_type_arg_22.oid is null then '' else ',' || pg_type_arg_22.oid::varchar end ||
                case when pg_type_arg_23.oid is null then '' else ',' || pg_type_arg_23.oid::varchar end ||
                case when pg_type_arg_24.oid is null then '' else ',' || pg_type_arg_24.oid::varchar end ||
                case when pg_type_arg_25.oid is null then '' else ',' || pg_type_arg_25.oid::varchar end ||
                case when pg_type_arg_26.oid is null then '' else ',' || pg_type_arg_26.oid::varchar end ||
                case when pg_type_arg_27.oid is null then '' else ',' || pg_type_arg_27.oid::varchar end ||
                case when pg_type_arg_28.oid is null then '' else ',' || pg_type_arg_28.oid::varchar end ||
                case when pg_type_arg_29.oid is null then '' else ',' || pg_type_arg_29.oid::varchar end ||
                case when pg_type_arg_30.oid is null then '' else ',' || pg_type_arg_30.oid::varchar end ||
                case when pg_type_arg_31.oid is null then '' else ',' || pg_type_arg_31.oid::varchar end     as procargument_data_type_ids,

                                                                             pg_type_arg_00.typname     ||
                case when pg_type_arg_01.typname is null then '' else ',' || pg_type_arg_01.typname end ||
                case when pg_type_arg_02.typname is null then '' else ',' || pg_type_arg_02.typname end ||
                case when pg_type_arg_03.typname is null then '' else ',' || pg_type_arg_03.typname end ||
                case when pg_type_arg_04.typname is null then '' else ',' || pg_type_arg_04.typname end ||
                case when pg_type_arg_05.typname is null then '' else ',' || pg_type_arg_05.typname end ||
                case when pg_type_arg_06.typname is null then '' else ',' || pg_type_arg_06.typname end ||
                case when pg_type_arg_07.typname is null then '' else ',' || pg_type_arg_07.typname end ||
                case when pg_type_arg_08.typname is null then '' else ',' || pg_type_arg_08.typname end ||
                case when pg_type_arg_09.typname is null then '' else ',' || pg_type_arg_09.typname end ||
                case when pg_type_arg_10.typname is null then '' else ',' || pg_type_arg_10.typname end ||
                case when pg_type_arg_11.typname is null then '' else ',' || pg_type_arg_11.typname end ||
                case when pg_type_arg_12.typname is null then '' else ',' || pg_type_arg_12.typname end ||
                case when pg_type_arg_13.typname is null then '' else ',' || pg_type_arg_13.typname end ||
                case when pg_type_arg_14.typname is null then '' else ',' || pg_type_arg_14.typname end ||
                case when pg_type_arg_15.typname is null then '' else ',' || pg_type_arg_15.typname end ||
                case when pg_type_arg_16.typname is null then '' else ',' || pg_type_arg_16.typname end ||
                case when pg_type_arg_17.typname is null then '' else ',' || pg_type_arg_17.typname end ||
                case when pg_type_arg_18.typname is null then '' else ',' || pg_type_arg_18.typname end ||
                case when pg_type_arg_19.typname is null then '' else ',' || pg_type_arg_19.typname end ||
                case when pg_type_arg_20.typname is null then '' else ',' || pg_type_arg_20.typname end ||
                case when pg_type_arg_21.typname is null then '' else ',' || pg_type_arg_21.typname end ||
                case when pg_type_arg_22.typname is null then '' else ',' || pg_type_arg_22.typname end ||
                case when pg_type_arg_23.typname is null then '' else ',' || pg_type_arg_23.typname end ||
                case when pg_type_arg_24.typname is null then '' else ',' || pg_type_arg_24.typname end ||
                case when pg_type_arg_25.typname is null then '' else ',' || pg_type_arg_25.typname end ||
                case when pg_type_arg_26.typname is null then '' else ',' || pg_type_arg_26.typname end ||
                case when pg_type_arg_27.typname is null then '' else ',' || pg_type_arg_27.typname end ||
                case when pg_type_arg_28.typname is null then '' else ',' || pg_type_arg_28.typname end ||
                case when pg_type_arg_29.typname is null then '' else ',' || pg_type_arg_29.typname end ||
                case when pg_type_arg_30.typname is null then '' else ',' || pg_type_arg_30.typname end ||
                case when pg_type_arg_31.typname is null then '' else ',' || pg_type_arg_31.typname end     as procargument_data_type_names,

                pg_proc.proowner as procowner_id,
                pg_user.usename  as procowner_name,
                pg_proc.proacl   as procacl,
                case
                  when pg_language.lanname = 'c'         then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  -- MG2 : internal are all {=X/rdsdb}, and we can't make more languages (no "create language" in Redshift)
                  when pg_language.lanname = 'internal'  then array[ '=X/rdsdb' ]
                  when pg_language.lanname = 'plpgsql'   then array[ pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'plpythonu' then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'sql'       then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                end              as procdefault_acl,
                generate_series( 1,
                                 case
                                   when procacl is     null then array_upper( procdefault_acl, 1 )
                                   when procacl is not null then array_upper( procacl, 1 )
                                 end ) as acl_index
              from
                          pg_proc
                     join pg_language               on pg_proc.prolang          = pg_language.oid
                     join pg_namespace              on pg_proc.pronamespace     = pg_namespace.oid
                     join pg_user                   on pg_proc.proowner         = pg_user.usesysid
                -- MG2 : procedures do not have a return type
                left join pg_type as pg_type_return on pg_proc.prorettype       = pg_type_return.oid
                -- MG2 : pg_proc.proargtypes is an oidvector, which is an array of 32 oids, when an element is empty, the value is 0
                left join pg_type as pg_type_arg_00 on pg_proc.proargtypes[00] != 0 and pg_proc.proargtypes[00] = pg_type_arg_00.oid
                left join pg_type as pg_type_arg_01 on pg_proc.proargtypes[01] != 0 and pg_proc.proargtypes[01] = pg_type_arg_01.oid
                left join pg_type as pg_type_arg_02 on pg_proc.proargtypes[02] != 0 and pg_proc.proargtypes[02] = pg_type_arg_02.oid
                left join pg_type as pg_type_arg_03 on pg_proc.proargtypes[03] != 0 and pg_proc.proargtypes[03] = pg_type_arg_03.oid
                left join pg_type as pg_type_arg_04 on pg_proc.proargtypes[04] != 0 and pg_proc.proargtypes[04] = pg_type_arg_04.oid
                left join pg_type as pg_type_arg_05 on pg_proc.proargtypes[05] != 0 and pg_proc.proargtypes[05] = pg_type_arg_05.oid
                left join pg_type as pg_type_arg_06 on pg_proc.proargtypes[06] != 0 and pg_proc.proargtypes[06] = pg_type_arg_06.oid
                left join pg_type as pg_type_arg_07 on pg_proc.proargtypes[07] != 0 and pg_proc.proargtypes[07] = pg_type_arg_07.oid
                left join pg_type as pg_type_arg_08 on pg_proc.proargtypes[08] != 0 and pg_proc.proargtypes[08] = pg_type_arg_08.oid
                left join pg_type as pg_type_arg_09 on pg_proc.proargtypes[09] != 0 and pg_proc.proargtypes[09] = pg_type_arg_09.oid
                left join pg_type as pg_type_arg_10 on pg_proc.proargtypes[10] != 0 and pg_proc.proargtypes[10] = pg_type_arg_10.oid
                left join pg_type as pg_type_arg_11 on pg_proc.proargtypes[11] != 0 and pg_proc.proargtypes[11] = pg_type_arg_11.oid
                left join pg_type as pg_type_arg_12 on pg_proc.proargtypes[12] != 0 and pg_proc.proargtypes[12] = pg_type_arg_12.oid
                left join pg_type as pg_type_arg_13 on pg_proc.proargtypes[13] != 0 and pg_proc.proargtypes[13] = pg_type_arg_13.oid
                left join pg_type as pg_type_arg_14 on pg_proc.proargtypes[14] != 0 and pg_proc.proargtypes[14] = pg_type_arg_14.oid
                left join pg_type as pg_type_arg_15 on pg_proc.proargtypes[15] != 0 and pg_proc.proargtypes[15] = pg_type_arg_15.oid
                left join pg_type as pg_type_arg_16 on pg_proc.proargtypes[16] != 0 and pg_proc.proargtypes[16] = pg_type_arg_16.oid
                left join pg_type as pg_type_arg_17 on pg_proc.proargtypes[17] != 0 and pg_proc.proargtypes[17] = pg_type_arg_17.oid
                left join pg_type as pg_type_arg_18 on pg_proc.proargtypes[18] != 0 and pg_proc.proargtypes[18] = pg_type_arg_18.oid
                left join pg_type as pg_type_arg_19 on pg_proc.proargtypes[19] != 0 and pg_proc.proargtypes[19] = pg_type_arg_19.oid
                left join pg_type as pg_type_arg_20 on pg_proc.proargtypes[20] != 0 and pg_proc.proargtypes[20] = pg_type_arg_20.oid
                left join pg_type as pg_type_arg_21 on pg_proc.proargtypes[21] != 0 and pg_proc.proargtypes[21] = pg_type_arg_21.oid
                left join pg_type as pg_type_arg_22 on pg_proc.proargtypes[22] != 0 and pg_proc.proargtypes[22] = pg_type_arg_22.oid
                left join pg_type as pg_type_arg_23 on pg_proc.proargtypes[23] != 0 and pg_proc.proargtypes[23] = pg_type_arg_23.oid
                left join pg_type as pg_type_arg_24 on pg_proc.proargtypes[24] != 0 and pg_proc.proargtypes[24] = pg_type_arg_24.oid
                left join pg_type as pg_type_arg_25 on pg_proc.proargtypes[25] != 0 and pg_proc.proargtypes[25] = pg_type_arg_25.oid
                left join pg_type as pg_type_arg_26 on pg_proc.proargtypes[26] != 0 and pg_proc.proargtypes[26] = pg_type_arg_26.oid
                left join pg_type as pg_type_arg_27 on pg_proc.proargtypes[27] != 0 and pg_proc.proargtypes[27] = pg_type_arg_27.oid
                left join pg_type as pg_type_arg_28 on pg_proc.proargtypes[28] != 0 and pg_proc.proargtypes[28] = pg_type_arg_28.oid
                left join pg_type as pg_type_arg_29 on pg_proc.proargtypes[29] != 0 and pg_proc.proargtypes[29] = pg_type_arg_29.oid
                left join pg_type as pg_type_arg_30 on pg_proc.proargtypes[30] != 0 and pg_proc.proargtypes[30] = pg_type_arg_30.oid
                left join pg_type as pg_type_arg_31 on pg_proc.proargtypes[31] != 0 and pg_proc.proargtypes[31] = pg_type_arg_31.oid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '='
        ) as subquery_3
    ),

    cte_pg_language as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'language'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        language_id         as object_id,
        language_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        language_owner_id   as object_owner_user_id,
        language_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 1, strpos(acl_item,'=')-1 ) as acl_recipient_user_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            language_id,
            language_name,
            language_owner_id,
            language_owner_name,
            language_acl,
            language_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when language_acl is     null then array_to_string( language_default_acl[acl_index:acl_index], ',' )
              when language_acl is not null then array_to_string( language_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                language_id,
                language_name,
                language_owner_id,
                language_owner_name,
                language_acl,
                language_default_acl,
                generate_series( 1,
                                 case
                                   when language_acl is     null then array_upper( language_default_acl, 1 )
                                   when language_acl is not null then array_upper( language_acl, 1 )
                                 end ) as acl_index
              from
                (
                  select
                    pg_language.oid     as language_id,
                    pg_language.lanname as language_name,
                    1::oid              as language_owner_id,
                    'rdsdb'             as language_owner_name,
                    pg_language.lanacl  as language_acl,
                    case
                      -- MG2 : no CREATE LANGUAGE in Redshift, so owner is always rdsdb (OID 1)
                      --        usage on language seems brolen and is not checked
                      --        the usual usage grant to group public is not made for plpgsql and plpythonu
                      --        (but this has no affect, since the check for usage is not made)
                      when pg_language.lanname not in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb,=U/rdsdb' ]
                      when pg_language.lanname     in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb' ]
                    end                 as language_default_acl
                  from
                    pg_language
                ) as subquery_1
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '='
        ) as subquery_3
    ),

    cte_pg_database as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'database'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        database_id         as object_id,
        database_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        database_owner_id   as object_owner_user_id,
        database_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 1, strpos(acl_item,'=')-1 ) as acl_recipient_user_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            database_id,
            database_name,
            database_owner_id,
            database_owner_name,
            database_acl,
            database_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when database_acl is     null then array_to_string( database_default_acl[acl_index:acl_index], ',' )
              when database_acl is not null then array_to_string( database_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_database.oid     as database_id,
                pg_database.datname as database_name,
                pg_database.datdba  as database_owner_id,
                pg_user.usename     as database_owner_name,
                pg_database.datacl  as database_acl,
                -- MG2 : note we do not reprocuce the array-indicating open and close curly braces
                --       we could, to make the database_acl strings look identical but then they're a pain later on
                array[ '=T/' || pg_user.usename || ',' || pg_user.usename || '=CT/' || pg_user.usename ] as database_default_acl,
                generate_series( 1,
                                 case
                                   when database_acl is     null then array_upper( database_default_acl, 1 )
                                   when database_acl is not null then array_upper( database_acl, 1 )
                                 end ) as acl_index
              from
                     pg_database
                join pg_user on pg_database.datdba = pg_user.usesysid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '='
        ) as subquery_3
    ),

    cte_all as
    (
      select * from cte_pg_namespace
      union all
      select * from cte_pg_class
      union all
      select * from cte_pg_proc
      union all
      select * from cte_pg_language
      union all
      select * from cte_pg_database
    )

  select
    pg_user.usesysid::int4 as acl_recipient_user_id,
    acl_recipient_user_name::varchar,

    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    acl_privileges::varchar
  from
         cte_all
    join pg_user on pg_user.usename = cte_all.acl_recipient_user_name;



create or replace view
  lf_241129.privileges_granted_to_users
as
  select
    acl_recipient_user_id::int4      as user_id,
    acl_recipient_user_name::varchar as user_name,

    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    -- MG2 : if the priv to grant to others is given, the underlying priv is always given as well, so for "with grant option"s, we only need to check for the priv letter plus the asterisk

    case
      when object_type = 'database' and strpos(acl_privileges, 'C') != 0 then true else false
    end::bool as priv_database_create,

    case
      when object_type = 'database' and strpos(acl_privileges, 'C*') != 0 then true else false
    end::bool as priv_database_grant_create,

    case
      when object_type = 'database' and strpos(acl_privileges, 'T') != 0 then true else false
    end::bool as priv_database_temp,

    case
      when object_type = 'database' and strpos(acl_privileges, 'T*') != 0 then true else false
    end::bool as priv_database_grant_temp,

    case
      when object_type = 'function' and strpos(acl_privileges, 'X') != 0 then true else false
    end::bool as priv_function_execute,

    case
      when object_type = 'function' and strpos(acl_privileges, 'X*') != 0 then true else false
    end::bool as priv_function_grant_execute,

    case
      when object_type = 'language' and strpos(acl_privileges, 'U') != 0 then true else false
    end::bool as priv_language_usage,

    case
      when object_type = 'language' and strpos(acl_privileges, 'U*') != 0 then true else false
    end::bool as priv_language_grant_usage,

    case
      when object_type = 'procedure' and strpos(acl_privileges, 'X') != 0 then true else false
    end::bool as priv_procedure_execute,

    case
      when object_type = 'procedure' and strpos(acl_privileges, 'X*') != 0 then true else false
    end::bool as priv_procedure_grant_execute,

    case
      when object_type = 'schema'   and strpos(acl_privileges, 'C') != 0 then true else false
    end::bool as priv_schema_create,

    case
      when object_type = 'schema'   and strpos(acl_privileges, 'C*') != 0 then true else false
    end::bool as priv_schema_grant_create,

    case
      when object_type = 'schema'   and strpos(acl_privileges, 'U') != 0 then true else false
    end::bool as priv_schema_usage,

    case
      when object_type = 'schema'   and strpos(acl_privileges, 'U*') != 0 then true else false
    end::bool as priv_schema_grant_usage,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'd') != 0 then true else false
    end::bool as priv_table_delete,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'd*') != 0 then true else false
    end::bool as priv_table_grant_delete,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'D') != 0 then true else false
    end::bool as priv_table_drop,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'D*') != 0 then true else false
    end::bool as priv_table_grant_drop,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'a') != 0 then true else false
    end::bool as priv_table_insert,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'a*') != 0 then true else false
    end::bool as priv_table_grant_insert,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'x') != 0 then true else false
    end::bool as priv_table_reference,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'x*') != 0 then true else false
    end::bool as priv_table_grant_reference,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'r') != 0 then true else false
    end::bool as priv_table_select,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'r*') != 0 then true else false
    end::bool as priv_table_grant_select,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'w') != 0 then true else false
    end::bool as priv_table_update,

    case
      when object_type = 'table'    and strpos(acl_privileges, 'w*') != 0 then true else false
    end::bool as priv_table_grant_update,

    case
      when object_type = 'view'     and strpos(acl_privileges, 'D') != 0 then true else false
    end::bool as priv_view_drop,

    case
      when object_type = 'view'     and strpos(acl_privileges, 'D*') != 0 then true else false
    end::bool as priv_view_grant_drop,

    case
      when object_type = 'view'     and strpos(acl_privileges, 'r') != 0 then true else false
    end::bool as priv_view_select,

    case
      when object_type = 'view'     and strpos(acl_privileges, 'r*') != 0 then true else false
    end::bool as priv_view_grant_select
  from
    base_241129.privileges_users;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_users_by_second
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'second', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        user_id,
        date_trunc( 'second', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                   as user_name,
    base_241129.generate_seconds.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_seconds
    left join cte_queries using ( start_ts )
    left join pg_user on cte_queries.user_id = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.analyze_totals_for_tables_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_tables_by_hour
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  base_241129.privileges_all
as
  with
    cte_pg_namespace as
    (
      select
        schema_id,
        schema_name,
        'schema'          as object_type,
        null::oid         as object_return_base_data_type_id,
        null::varchar     as object_return_base_data_type_name,
        schema_id         as object_id,
        schema_name       as object_name,
        null::varchar     as object_argument_base_data_type_ids,
        null::varchar     as object_argument_base_data_type_names,
        schema_owner_id   as object_owner_user_id,
        schema_owner_name as object_owner_user_name,
        acl_item,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then 'user'
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then 'group'
        end               as acl_recipient_type,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then substring( acl_item, 1, strpos(acl_item,'=')-1 )
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then substring( acl_item, 7, strpos(acl_item,'=')-7 )
        end               as acl_recipient_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            schema_owner_id,
            schema_owner_name,
            schema_acl,
            schema_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when schema_acl is     null then array_to_string( schema_default_acl[acl_index:acl_index], ',' )
              when schema_acl is not null then array_to_string( schema_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid                                       as schema_id,
                pg_namespace.nspname                                   as schema_name,
                pg_namespace.nspowner                                  as schema_owner_id,
                pg_user.usename                                        as schema_owner_name,
                pg_namespace.nspacl                                    as schema_acl,
                -- MG2 : I would be alphabetic, but Postgres/Redshift uses 'UC'
                array[ pg_user.usename || '=UC/' || pg_user.usename ]  as schema_default_acl,
                generate_series( 1,
                                 case
                                   when schema_acl is     null then array_upper( schema_default_acl, 1 )
                                   when schema_acl is not null then array_upper( schema_acl, 1 )
                                 end ) as acl_index
              from
                     pg_namespace
                join pg_user on pg_namespace.nspowner = pg_user.usesysid
            ) as subquery_2
        ) as subquery_3
    ),

    cte_pg_class as
    (
      select
        schema_id,
        schema_name,
        case
          when class_type = 'r' then 'table'
          when class_type = 'v' then 'view'
        end              as object_type,
        null::oid        as object_return_base_data_type_id,
        null::varchar    as object_return_base_data_type_name,
        class_id         as object_id,
        class_name       as object_name,
        null::varchar    as object_argument_base_data_type_ids,
        null::varchar    as object_argument_base_data_type_names,
        class_owner_id   as object_owner_user_id,
        class_owner_name as object_owner_user_name,
        acl_item,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then 'user'
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then 'group'
        end as acl_recipient_type,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then substring( acl_item, 1, strpos(acl_item,'=')-1 )
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then substring( acl_item, 7, strpos(acl_item,'=')-7 )
        end as acl_recipient_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            class_id,
            class_type,
            class_name,
            class_owner_id,
            class_owner_name,
            class_acl,
            class_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when class_acl is     null then array_to_string( class_default_acl[acl_index:acl_index], ',' )
              when class_acl is not null then array_to_string( class_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_class.oid         as class_id,
                pg_class.relkind     as class_type,
                pg_class.relname     as class_name,
                pg_class.relowner    as class_owner_id,
                pg_user.usename      as class_owner_name,
                pg_class.relacl      as class_acl,
                  -- MG2 : 'arwdRxtD' is the ACL string Redshift procuces; I could curtail it to only those which exist on Redshift, but matching RS makes testing easier
                array[ pg_user.usename || '=arwdRxtD/' || pg_user.usename ] as class_default_acl,
                generate_series( 1,
                                 case
                                   when class_acl is     null then array_upper( class_default_acl, 1 )
                                   when class_acl is not null then array_upper( class_acl, 1 )
                                 end ) as acl_index
              from
                     pg_class
                join pg_namespace on pg_class.relnamespace = pg_namespace.oid
                join pg_user      on pg_class.relowner     = pg_user.usesysid
              where
                pg_class.relkind in ( 'r', 'v' )
            ) as subquery_2
        ) as subquery_3
    ),

    cte_pg_proc as
    (
      select
        schema_id,
        schema_name,
        case
          when proctype = 'c'         then 'C func'
          when proctype = 'exfunc'    then 'lambda func'
          when proctype = 'internal'  then 'internal func'
          when proctype = 'mlfunc'    then 'ML func'
          when proctype = 'plpgsql'   then 'procedure'
          when proctype = 'plpythonu' then 'python func'
          when proctype = 'sql'       then 'SQL func'
                                       else trim( proctype )
        end                            as object_type,
        procreturn_data_type_id       as object_return_base_data_type_id,
        procreturn_data_type_name     as object_return_base_data_type_name,
        procid                        as object_id,
        procname                      as object_name,
        procargument_data_type_ids    as object_argument_base_data_type_ids,
        procargument_data_type_names  as object_argument_base_data_type_names,
        procowner_id                  as object_owner_user_id,
        procowner_name                as object_owner_user_name,
        acl_item,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then 'user'
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then 'group'
        end                            as acl_recipient_type,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then substring( acl_item, 1, strpos(acl_item,'=')-1 )
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then substring( acl_item, 7, strpos(acl_item,'=')-7 )
        end                            as acl_recipient_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            procid,
            proctype,
            procreturn_data_type_id,
            procreturn_data_type_name,
            procname,
            procargument_data_type_ids,
            procargument_data_type_names,
            procowner_id,
            procowner_name,
            procacl,
            procdefault_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when procacl is     null then array_to_string( procdefault_acl[acl_index:acl_index], ',' )
              when procacl is not null then array_to_string( procacl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_proc.oid          as procid,
                pg_language.lanname  as proctype,
                pg_proc.prorettype   as procreturn_data_type_id,
                case
                  when pg_proc.proretset = True  then '_' || pg_type_return.typname
                  when pg_proc.proretset = False then        pg_type_return.typname
                end                  as procreturn_data_type_name,
                pg_proc.proname      as procname,

                                                                         pg_type_arg_00.oid::varchar     ||
                case when pg_type_arg_01.oid is null then '' else ',' || pg_type_arg_01.oid::varchar end ||
                case when pg_type_arg_02.oid is null then '' else ',' || pg_type_arg_02.oid::varchar end ||
                case when pg_type_arg_03.oid is null then '' else ',' || pg_type_arg_03.oid::varchar end ||
                case when pg_type_arg_04.oid is null then '' else ',' || pg_type_arg_04.oid::varchar end ||
                case when pg_type_arg_05.oid is null then '' else ',' || pg_type_arg_05.oid::varchar end ||
                case when pg_type_arg_06.oid is null then '' else ',' || pg_type_arg_06.oid::varchar end ||
                case when pg_type_arg_07.oid is null then '' else ',' || pg_type_arg_07.oid::varchar end ||
                case when pg_type_arg_08.oid is null then '' else ',' || pg_type_arg_08.oid::varchar end ||
                case when pg_type_arg_09.oid is null then '' else ',' || pg_type_arg_09.oid::varchar end ||
                case when pg_type_arg_10.oid is null then '' else ',' || pg_type_arg_10.oid::varchar end ||
                case when pg_type_arg_11.oid is null then '' else ',' || pg_type_arg_11.oid::varchar end ||
                case when pg_type_arg_12.oid is null then '' else ',' || pg_type_arg_12.oid::varchar end ||
                case when pg_type_arg_13.oid is null then '' else ',' || pg_type_arg_13.oid::varchar end ||
                case when pg_type_arg_14.oid is null then '' else ',' || pg_type_arg_14.oid::varchar end ||
                case when pg_type_arg_15.oid is null then '' else ',' || pg_type_arg_15.oid::varchar end ||
                case when pg_type_arg_16.oid is null then '' else ',' || pg_type_arg_16.oid::varchar end ||
                case when pg_type_arg_17.oid is null then '' else ',' || pg_type_arg_17.oid::varchar end ||
                case when pg_type_arg_18.oid is null then '' else ',' || pg_type_arg_18.oid::varchar end ||
                case when pg_type_arg_19.oid is null then '' else ',' || pg_type_arg_19.oid::varchar end ||
                case when pg_type_arg_20.oid is null then '' else ',' || pg_type_arg_20.oid::varchar end ||
                case when pg_type_arg_21.oid is null then '' else ',' || pg_type_arg_21.oid::varchar end ||
                case when pg_type_arg_22.oid is null then '' else ',' || pg_type_arg_22.oid::varchar end ||
                case when pg_type_arg_23.oid is null then '' else ',' || pg_type_arg_23.oid::varchar end ||
                case when pg_type_arg_24.oid is null then '' else ',' || pg_type_arg_24.oid::varchar end ||
                case when pg_type_arg_25.oid is null then '' else ',' || pg_type_arg_25.oid::varchar end ||
                case when pg_type_arg_26.oid is null then '' else ',' || pg_type_arg_26.oid::varchar end ||
                case when pg_type_arg_27.oid is null then '' else ',' || pg_type_arg_27.oid::varchar end ||
                case when pg_type_arg_28.oid is null then '' else ',' || pg_type_arg_28.oid::varchar end ||
                case when pg_type_arg_29.oid is null then '' else ',' || pg_type_arg_29.oid::varchar end ||
                case when pg_type_arg_30.oid is null then '' else ',' || pg_type_arg_30.oid::varchar end ||
                case when pg_type_arg_31.oid is null then '' else ',' || pg_type_arg_31.oid::varchar end     as procargument_data_type_ids,

                                                                             pg_type_arg_00.typname     ||
                case when pg_type_arg_01.typname is null then '' else ',' || pg_type_arg_01.typname end ||
                case when pg_type_arg_02.typname is null then '' else ',' || pg_type_arg_02.typname end ||
                case when pg_type_arg_03.typname is null then '' else ',' || pg_type_arg_03.typname end ||
                case when pg_type_arg_04.typname is null then '' else ',' || pg_type_arg_04.typname end ||
                case when pg_type_arg_05.typname is null then '' else ',' || pg_type_arg_05.typname end ||
                case when pg_type_arg_06.typname is null then '' else ',' || pg_type_arg_06.typname end ||
                case when pg_type_arg_07.typname is null then '' else ',' || pg_type_arg_07.typname end ||
                case when pg_type_arg_08.typname is null then '' else ',' || pg_type_arg_08.typname end ||
                case when pg_type_arg_09.typname is null then '' else ',' || pg_type_arg_09.typname end ||
                case when pg_type_arg_10.typname is null then '' else ',' || pg_type_arg_10.typname end ||
                case when pg_type_arg_11.typname is null then '' else ',' || pg_type_arg_11.typname end ||
                case when pg_type_arg_12.typname is null then '' else ',' || pg_type_arg_12.typname end ||
                case when pg_type_arg_13.typname is null then '' else ',' || pg_type_arg_13.typname end ||
                case when pg_type_arg_14.typname is null then '' else ',' || pg_type_arg_14.typname end ||
                case when pg_type_arg_15.typname is null then '' else ',' || pg_type_arg_15.typname end ||
                case when pg_type_arg_16.typname is null then '' else ',' || pg_type_arg_16.typname end ||
                case when pg_type_arg_17.typname is null then '' else ',' || pg_type_arg_17.typname end ||
                case when pg_type_arg_18.typname is null then '' else ',' || pg_type_arg_18.typname end ||
                case when pg_type_arg_19.typname is null then '' else ',' || pg_type_arg_19.typname end ||
                case when pg_type_arg_20.typname is null then '' else ',' || pg_type_arg_20.typname end ||
                case when pg_type_arg_21.typname is null then '' else ',' || pg_type_arg_21.typname end ||
                case when pg_type_arg_22.typname is null then '' else ',' || pg_type_arg_22.typname end ||
                case when pg_type_arg_23.typname is null then '' else ',' || pg_type_arg_23.typname end ||
                case when pg_type_arg_24.typname is null then '' else ',' || pg_type_arg_24.typname end ||
                case when pg_type_arg_25.typname is null then '' else ',' || pg_type_arg_25.typname end ||
                case when pg_type_arg_26.typname is null then '' else ',' || pg_type_arg_26.typname end ||
                case when pg_type_arg_27.typname is null then '' else ',' || pg_type_arg_27.typname end ||
                case when pg_type_arg_28.typname is null then '' else ',' || pg_type_arg_28.typname end ||
                case when pg_type_arg_29.typname is null then '' else ',' || pg_type_arg_29.typname end ||
                case when pg_type_arg_30.typname is null then '' else ',' || pg_type_arg_30.typname end ||
                case when pg_type_arg_31.typname is null then '' else ',' || pg_type_arg_31.typname end     as procargument_data_type_names,

                pg_proc.proowner as procowner_id,
                pg_user.usename  as procowner_name,
                pg_proc.proacl   as procacl,
                case
                  when pg_language.lanname = 'c'         then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  -- MG2 : internal are all {=X/rdsdb}, and we can't make more languages (no "create language" in Redshift)
                  when pg_language.lanname = 'internal'  then array[ '=X/rdsdb' ]
                  when pg_language.lanname = 'plpgsql'   then array[ pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'plpythonu' then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'sql'       then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                end              as procdefault_acl,
                generate_series( 1,
                                 case
                                   when procacl is     null then array_upper( procdefault_acl, 1 )
                                   when procacl is not null then array_upper( procacl, 1 )
                                 end ) as acl_index
              from
                          pg_proc
                     join pg_language               on pg_proc.prolang          = pg_language.oid
                     join pg_namespace              on pg_proc.pronamespace     = pg_namespace.oid
                     join pg_user                   on pg_proc.proowner         = pg_user.usesysid
                -- MG2 : procedures do not have a return type
                left join pg_type as pg_type_return on pg_proc.prorettype       = pg_type_return.oid
                -- MG2 : pg_proc.proargtypes is an oidvector, which is an array of 32 oids, when an element is empty, the value is 0
                left join pg_type as pg_type_arg_00 on pg_proc.proargtypes[00] != 0 and pg_proc.proargtypes[00] = pg_type_arg_00.oid
                left join pg_type as pg_type_arg_01 on pg_proc.proargtypes[01] != 0 and pg_proc.proargtypes[01] = pg_type_arg_01.oid
                left join pg_type as pg_type_arg_02 on pg_proc.proargtypes[02] != 0 and pg_proc.proargtypes[02] = pg_type_arg_02.oid
                left join pg_type as pg_type_arg_03 on pg_proc.proargtypes[03] != 0 and pg_proc.proargtypes[03] = pg_type_arg_03.oid
                left join pg_type as pg_type_arg_04 on pg_proc.proargtypes[04] != 0 and pg_proc.proargtypes[04] = pg_type_arg_04.oid
                left join pg_type as pg_type_arg_05 on pg_proc.proargtypes[05] != 0 and pg_proc.proargtypes[05] = pg_type_arg_05.oid
                left join pg_type as pg_type_arg_06 on pg_proc.proargtypes[06] != 0 and pg_proc.proargtypes[06] = pg_type_arg_06.oid
                left join pg_type as pg_type_arg_07 on pg_proc.proargtypes[07] != 0 and pg_proc.proargtypes[07] = pg_type_arg_07.oid
                left join pg_type as pg_type_arg_08 on pg_proc.proargtypes[08] != 0 and pg_proc.proargtypes[08] = pg_type_arg_08.oid
                left join pg_type as pg_type_arg_09 on pg_proc.proargtypes[09] != 0 and pg_proc.proargtypes[09] = pg_type_arg_09.oid
                left join pg_type as pg_type_arg_10 on pg_proc.proargtypes[10] != 0 and pg_proc.proargtypes[10] = pg_type_arg_10.oid
                left join pg_type as pg_type_arg_11 on pg_proc.proargtypes[11] != 0 and pg_proc.proargtypes[11] = pg_type_arg_11.oid
                left join pg_type as pg_type_arg_12 on pg_proc.proargtypes[12] != 0 and pg_proc.proargtypes[12] = pg_type_arg_12.oid
                left join pg_type as pg_type_arg_13 on pg_proc.proargtypes[13] != 0 and pg_proc.proargtypes[13] = pg_type_arg_13.oid
                left join pg_type as pg_type_arg_14 on pg_proc.proargtypes[14] != 0 and pg_proc.proargtypes[14] = pg_type_arg_14.oid
                left join pg_type as pg_type_arg_15 on pg_proc.proargtypes[15] != 0 and pg_proc.proargtypes[15] = pg_type_arg_15.oid
                left join pg_type as pg_type_arg_16 on pg_proc.proargtypes[16] != 0 and pg_proc.proargtypes[16] = pg_type_arg_16.oid
                left join pg_type as pg_type_arg_17 on pg_proc.proargtypes[17] != 0 and pg_proc.proargtypes[17] = pg_type_arg_17.oid
                left join pg_type as pg_type_arg_18 on pg_proc.proargtypes[18] != 0 and pg_proc.proargtypes[18] = pg_type_arg_18.oid
                left join pg_type as pg_type_arg_19 on pg_proc.proargtypes[19] != 0 and pg_proc.proargtypes[19] = pg_type_arg_19.oid
                left join pg_type as pg_type_arg_20 on pg_proc.proargtypes[20] != 0 and pg_proc.proargtypes[20] = pg_type_arg_20.oid
                left join pg_type as pg_type_arg_21 on pg_proc.proargtypes[21] != 0 and pg_proc.proargtypes[21] = pg_type_arg_21.oid
                left join pg_type as pg_type_arg_22 on pg_proc.proargtypes[22] != 0 and pg_proc.proargtypes[22] = pg_type_arg_22.oid
                left join pg_type as pg_type_arg_23 on pg_proc.proargtypes[23] != 0 and pg_proc.proargtypes[23] = pg_type_arg_23.oid
                left join pg_type as pg_type_arg_24 on pg_proc.proargtypes[24] != 0 and pg_proc.proargtypes[24] = pg_type_arg_24.oid
                left join pg_type as pg_type_arg_25 on pg_proc.proargtypes[25] != 0 and pg_proc.proargtypes[25] = pg_type_arg_25.oid
                left join pg_type as pg_type_arg_26 on pg_proc.proargtypes[26] != 0 and pg_proc.proargtypes[26] = pg_type_arg_26.oid
                left join pg_type as pg_type_arg_27 on pg_proc.proargtypes[27] != 0 and pg_proc.proargtypes[27] = pg_type_arg_27.oid
                left join pg_type as pg_type_arg_28 on pg_proc.proargtypes[28] != 0 and pg_proc.proargtypes[28] = pg_type_arg_28.oid
                left join pg_type as pg_type_arg_29 on pg_proc.proargtypes[29] != 0 and pg_proc.proargtypes[29] = pg_type_arg_29.oid
                left join pg_type as pg_type_arg_30 on pg_proc.proargtypes[30] != 0 and pg_proc.proargtypes[30] = pg_type_arg_30.oid
                left join pg_type as pg_type_arg_31 on pg_proc.proargtypes[31] != 0 and pg_proc.proargtypes[31] = pg_type_arg_31.oid
            ) as subquery_2
        ) as subquery_3
    ),

    cte_pg_language as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'language'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        language_id         as object_id,
        language_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        language_owner_id   as object_owner_user_id,
        language_owner_name as object_owner_user_name,
        acl_item,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then 'user'
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then 'group'
        end                 as acl_recipient_type,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then substring( acl_item, 1, strpos(acl_item,'=')-1 )
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then substring( acl_item, 7, strpos(acl_item,'=')-7 )
        end                 as acl_recipient_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            language_id,
            language_name,
            language_owner_id,
            language_owner_name,
            language_acl,
            language_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when language_acl is     null then array_to_string( language_default_acl[acl_index:acl_index], ',' )
              when language_acl is not null then array_to_string( language_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                language_id,
                language_name,
                language_owner_id,
                language_owner_name,
                language_acl,
                language_default_acl,
                generate_series( 1,
                                 case
                                   when language_acl is     null then array_upper( language_default_acl, 1 )
                                   when language_acl is not null then array_upper( language_acl, 1 )
                                 end ) as acl_index
              from
                (
                  select
                    pg_language.oid     as language_id,
                    pg_language.lanname as language_name,
                    1::oid              as language_owner_id,
                    'rdsdb'             as language_owner_name,
                    pg_language.lanacl  as language_acl,
                    case
                      -- MG2 : no CREATE LANGUAGE in Redshift, so owner is always rdsdb (OID 1)
                      --        usage on language seems brolen and is not checked
                      --        the usual usage grant to group public is not made for plpgsql and plpythonu
                      --        (but this has no affect, since the check for usage is not made)
                      when pg_language.lanname not in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb,=U/rdsdb' ]
                      when pg_language.lanname     in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb' ]
                    end                 as language_default_acl
                  from
                    pg_language
                ) as subquery_1
            ) as subquery_2
        ) as subquery_3
    ),

    cte_pg_database as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'database'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        database_id         as object_id,
        database_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        database_owner_id   as object_owner_user_id,
        database_owner_name as object_owner_user_name,
        acl_item,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then 'user'
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then 'group'
        end                 as acl_recipient_type,
        case
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) != '=' then substring( acl_item, 1, strpos(acl_item,'=')-1 )
          when substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1)  = '=' then 'public'
          when substring(acl_item, 1, 5)  = 'group'                                      then substring( acl_item, 7, strpos(acl_item,'=')-7 )
        end                 as acl_recipient_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            database_id,
            database_name,
            database_owner_id,
            database_owner_name,
            database_acl,
            database_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when database_acl is     null then array_to_string( database_default_acl[acl_index:acl_index], ',' )
              when database_acl is not null then array_to_string( database_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_database.oid     as database_id,
                pg_database.datname as database_name,
                pg_database.datdba  as database_owner_id,
                pg_user.usename     as database_owner_name,
                pg_database.datacl  as database_acl,
                -- MG2 : note we do not reprocuce the array-indicating open and close curly braces
                --       we could, to make the database_acl strings look identical but then they're a pain later on
                array[ '=T/' || pg_user.usename || ',' || pg_user.usename || '=CT/' || pg_user.usename ] as database_default_acl,
                generate_series( 1,
                                 case
                                   when database_acl is     null then array_upper( database_default_acl, 1 )
                                   when database_acl is not null then array_upper( database_acl, 1 )
                                 end ) as acl_index
              from
                     pg_database
                join pg_user on pg_database.datdba = pg_user.usesysid
            ) as subquery_2
        ) as subquery_3
    ),

    cte_all as
    (
      select * from cte_pg_namespace
      union all
      select * from cte_pg_class
      union all
      select * from cte_pg_proc
      union all
      select * from cte_pg_language
      union all
      select * from cte_pg_database
    )

  select
    acl_recipient_type::varchar,
    case
      when acl_recipient_type = 'group'  then pg_group.grosysid
      when acl_recipient_type = 'public' then null
      when acl_recipient_type = 'user'   then pg_user.usesysid
    end::int4 as acl_recipient_id,
    acl_recipient_name::varchar,

    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    acl_privileges::varchar
  from
              cte_all
    left join pg_group on acl_recipient_type = 'group' and pg_group.groname = acl_recipient_name
    left join pg_user  on acl_recipient_type = 'user'  and pg_user.usename  = acl_recipient_name;


create or replace view
  lf_241129.uusers_privileges
as
  with
    cte_groups_users_s1 as
    (
      select
        grosysid,
        generate_series( 1, array_upper(grolist,1) ) as user_array_index
      from
        pg_group
    ),

    cte_groups_users_s2 as
    (
      select
        pg_group.grosysid::int4           as group_id,
        trim( pg_group.groname )::varchar as group_name,
        -- MG2 : have to rename this from user_id, or RS gets confused about which alias
        --       to use in the case which computes user_owner
        pg_user.usesysid::int4            as group_user_id,
        trim( pg_user.usename  )::varchar as user_name
      from
             pg_group
        join cte_groups_users_s1 using ( grosysid )
        -- MG2 : the view for users in groups has a left join here, which is wrowg for us
        join pg_user                on pg_user.usesysid = pg_group.grolist[ cte_groups_users_s1.user_array_index ]
    )

  select
    case
      when base_241129.privileges_all.acl_recipient_type = 'group'  then cte_groups_users_s2.group_user_id
      when base_241129.privileges_all.acl_recipient_type = 'public' then pg_user.usesysid
      when base_241129.privileges_all.acl_recipient_type = 'user'   then acl_recipient_id
    end::int4 as user_id,
    case
      when base_241129.privileges_all.acl_recipient_type = 'group'  then cte_groups_users_s2.user_name
      when base_241129.privileges_all.acl_recipient_type = 'public' then pg_user.usename
      when base_241129.privileges_all.acl_recipient_type = 'user'   then acl_recipient_name
    end::varchar as user_name,

    case when user_id = object_owner_user_id then true else false end::bool as user_owner,

    base_241129.privileges_all.object_owner_user_id::int4,
    base_241129.privileges_all.object_owner_user_name::varchar,

    base_241129.privileges_all.acl_recipient_type::varchar as priv_origin_type,
    base_241129.privileges_all.acl_recipient_id::int4      as priv_origin_id,
    base_241129.privileges_all.acl_recipient_name::varchar as priv_origin_name,

    base_241129.privileges_all.schema_id::int4,
    base_241129.privileges_all.schema_name::varchar,
    base_241129.privileges_all.object_type::varchar,
    base_241129.privileges_all.object_return_base_data_type_id::int4,
    base_241129.privileges_all.object_return_base_data_type_name::varchar,
    base_241129.privileges_all.object_id::int4,
    base_241129.privileges_all.object_name::varchar,
    base_241129.privileges_all.object_argument_base_data_type_ids::varchar,
    base_241129.privileges_all.object_argument_base_data_type_names::varchar,

    -- MG2 : if the priv to grant to others is given, the underlying priv is always given as well, so for "with grant option"s, we only need to check for the priv letter plus the asterisk

    case
      when base_241129.privileges_all.object_type = 'database' and strpos(base_241129.privileges_all.acl_privileges, 'C') != 0 then true else false
    end::bool as priv_database_create,

    case
      when base_241129.privileges_all.object_type = 'database' and strpos(base_241129.privileges_all.acl_privileges, 'C*') != 0 then true else false
    end::bool as priv_database_grant_create,

    case
      when base_241129.privileges_all.object_type = 'database' and strpos(base_241129.privileges_all.acl_privileges, 'T') != 0 then true else false
    end::bool as priv_database_temp,

    case
      when base_241129.privileges_all.object_type = 'database' and strpos(base_241129.privileges_all.acl_privileges, 'T*') != 0 then true else false
    end::bool as priv_database_grant_temp,

    case
      when base_241129.privileges_all.object_type = 'function' and strpos(base_241129.privileges_all.acl_privileges, 'X') != 0 then true else false
    end::bool as priv_function_execute,

    case
      when base_241129.privileges_all.object_type = 'function' and strpos(base_241129.privileges_all.acl_privileges, 'X*') != 0 then true else false
    end::bool as priv_function_grant_execute,

    case
      when base_241129.privileges_all.object_type = 'language' and strpos(base_241129.privileges_all.acl_privileges, 'U') != 0 then true else false
    end::bool as priv_language_usage,

    case
      when base_241129.privileges_all.object_type = 'language' and strpos(base_241129.privileges_all.acl_privileges, 'U*') != 0 then true else false
    end::bool as priv_language_grant_usage,

    case
      when base_241129.privileges_all.object_type = 'procedure' and strpos(base_241129.privileges_all.acl_privileges, 'X') != 0 then true else false
    end::bool as priv_procedure_execute,

    case
      when base_241129.privileges_all.object_type = 'procedure' and strpos(base_241129.privileges_all.acl_privileges, 'X*') != 0 then true else false
    end::bool as priv_procedure_grant_execute,

    case
      when base_241129.privileges_all.object_type = 'schema'   and strpos(base_241129.privileges_all.acl_privileges, 'C') != 0 then true else false
    end::bool as priv_schema_create,

    case
      when base_241129.privileges_all.object_type = 'schema'   and strpos(base_241129.privileges_all.acl_privileges, 'C*') != 0 then true else false
    end::bool as priv_schema_grant_create,

    case
      when base_241129.privileges_all.object_type = 'schema'   and strpos(base_241129.privileges_all.acl_privileges, 'U') != 0 then true else false
    end::bool as priv_schema_usage,

    case
      when base_241129.privileges_all.object_type = 'schema'   and strpos(base_241129.privileges_all.acl_privileges, 'U*') != 0 then true else false
    end::bool as priv_schema_grant_usage,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'd') != 0 then true else false
    end::bool as priv_table_delete,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'd*') != 0 then true else false
    end::bool as priv_table_grant_delete,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'D') != 0 then true else false
    end::bool as priv_table_drop,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'D*') != 0 then true else false
    end::bool as priv_table_grant_drop,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'a') != 0 then true else false
    end::bool as priv_table_insert,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'a*') != 0 then true else false
    end::bool as priv_table_grant_insert,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'x') != 0 then true else false
    end::bool as priv_table_reference,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'x*') != 0 then true else false
    end::bool as priv_table_grant_reference,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'r') != 0 then true else false
    end::bool as priv_table_select,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'r*') != 0 then true else false
    end::bool as priv_table_grant_select,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'w') != 0 then true else false
    end::bool as priv_table_update,

    case
      when base_241129.privileges_all.object_type = 'table'    and strpos(base_241129.privileges_all.acl_privileges, 'w*') != 0 then true else false
    end::bool as priv_table_grant_update,

    case
      when base_241129.privileges_all.object_type = 'view'     and strpos(base_241129.privileges_all.acl_privileges, 'D') != 0 then true else false
    end::bool as priv_view_drop,

    case
      when base_241129.privileges_all.object_type = 'view'     and strpos(base_241129.privileges_all.acl_privileges, 'D*') != 0 then true else false
    end::bool as priv_view_grant_drop,

    case
      when base_241129.privileges_all.object_type = 'view'     and strpos(base_241129.privileges_all.acl_privileges, 'r') != 0 then true else false
    end::bool as priv_view_select,

    case
      when base_241129.privileges_all.object_type = 'view'     and strpos(base_241129.privileges_all.acl_privileges, 'r*') != 0 then true else false
    end::bool as priv_view_grant_select
  from
              base_241129.privileges_all
    left join cte_groups_users_s2 on base_241129.privileges_all.acl_recipient_type = 'group'  and cte_groups_users_s2.group_name = base_241129.privileges_all.acl_recipient_name
    left join pg_user             on base_241129.privileges_all.acl_recipient_type = 'public'
  where
    -- MG2 : removes rows procuced by groups which have privileges but no users
    user_id is not null;


create or replace view
  sf_241129.uusers_privileges
as
  select
    user_name::varchar as uuser,

    object_type::varchar as ttype,
    schema_name::varchar as sschema,
    object_name::varchar as oobject,

    object_return_base_data_type_name::varchar    as rt,
    object_argument_base_data_type_names::varchar as args,

    object_owner_user_name::varchar as oowner,

    priv_origin_type::varchar,
    priv_origin_name::varchar,

    -- case when user_id = object_owner_user_id then true else false end::bool as user_owner,

    (
      case
        when priv_database_grant_create = true then 'C'
        when priv_database_create       = true then 'c'
                                               else ' '
      end
      ||
      case
        when priv_database_grant_temp = true then 'T'
        when priv_database_temp       = true then 't'
                                             else ' '
      end
      ||
      case
        when priv_language_grant_usage = true then 'U'
        when priv_language_usage       = true then 'u'
                                              else ' '
      end
      ||
      '.'
      ||
      case
        when priv_schema_grant_create = true then 'C'
        when priv_schema_create       = true then 'c'
                                             else ' '
      end
      ||
      case
        when priv_schema_grant_usage = true then 'U'
        when priv_schema_usage       = true then 'u'
                                            else ' '
      end
      ||
      '.'
      ||
      case
        when priv_table_grant_delete = true then 'D'
        when priv_table_delete       = true then 'd'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_reference = true then 'F'
        when priv_table_reference       = true then 'f'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_insert = true then 'I'
        when priv_table_insert       = true then 'i'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_delete = true then 'D'
        when priv_table_delete       = true then 'd'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_drop = true then 'P'
        when priv_table_drop       = true then 'p'
                                          else ' '
      end
      ||
      case
        when priv_table_grant_select = true or priv_view_grant_select = true then 'S'
        when priv_table_select       = true or priv_view_select       = true then 's'
                                                                        else ' '
      end
      ||
      case
        when priv_table_grant_update = true then 'U'
        when priv_table_update       = true then 'u'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_delete = true then 'D'
        when priv_table_delete       = true then 'd'
                                            else ' '
      end
      ||
      case
        when priv_function_grant_execute = true or priv_procedure_grant_execute = true then 'X'
        when priv_function_execute       = true or priv_procedure_execute       = true then 'x'
                                                                                  else ' '
      end
    )::varchar as privs
  from
    lf_241129.uusers_privileges
  order by
    user_name,
    schema_name,
    case when object_type = 'schema' then 0 else 1 end,
    object_type,
    object_name;


create or replace view
  lf_241129.queries_totals_result_cache_for_users_by_all
as
  with
    cte_s1 as
    (
      select
        user_id,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
        count(*)                                                                                                 as number_total,
        ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_start_ts   >= current_date - interval '6 day'
        and query_end_ts     <= current_timestamp
      group by
        user_id
    )

  select
    user_id::int4,
    user_name::varchar,
    number_hit::int8,
    number_miss::int8,
    number_total::int8,
    percentage::float4
  from
              cte_s1
    left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  sf_241129.queries_totals_result_cache_for_users_by_all
as
  select
    user_name::varchar as uuser,
    number_hit::int8    as hit,
    number_miss::int8   as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_users_by_all
  order by
    user_name;



create or replace view
  lf_241129.queries_stats_durations_on_workers_for_users_by_hour
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'hour', query_queue_start_ts )        as start_ts,
        query_wlm_start_ts   - query_start_ts        as leadin_duration,
        query_queue_start_ts - query_wlm_start_ts    as wlm_leadin_duration,
        query_queue_end_ts   - query_queue_start_ts  as query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration   as query_exec_duration,
        query_wlm_end_ts     - query_compile_exec_end_ts as wlm_leadout_duration,
        query_end_ts         - query_wlm_end_ts          as leadout_duration,
        query_total_duration,
        query_total_duration - leadin_duration - wlm_leadin_duration - query_queue_duration + query_compile_duration - query_exec_duration - wlm_leadout_duration - leadout_duration as unaccounted_duration
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query         using ( query_id, user_id )
        left join base_241129.queries_compilation using ( query_id, user_id )
      where
            query_abort_flag    = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        user_id,
        start_ts,

        min( leadin_duration ) over ( partition by user_id, start_ts )                              as min_leadin_duration,
        max( leadin_duration ) over ( partition by user_id, start_ts )                              as max_leadin_duration,
        median( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts )     as med_leadin_duration,
        avg( leadin_duration ) over ( partition by user_id, start_ts )                              as avg_leadin_duration,
        stddev_pop( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts ) as std_leadin_duration,
        sum( leadin_duration ) over ( partition by user_id, start_ts )                              as sum_leadin_duration,

        min( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadin_duration,
        max( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadin_duration,
        median( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadin_duration,
        avg( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadin_duration,
        stddev_pop( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadin_duration,
        sum( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadin_duration,

        min( query_queue_duration ) over ( partition by user_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by user_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by user_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by user_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by user_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by user_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by user_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by user_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by user_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by user_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by user_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by user_id, start_ts )                            as sum_query_exec_duration,

        min( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadout_duration,
        max( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadout_duration,
        median( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadout_duration,
        avg( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadout_duration,
        stddev_pop( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadout_duration,
        sum( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadout_duration,

        min( leadout_duration ) over ( partition by user_id, start_ts )                              as min_leadout_duration,
        max( leadout_duration ) over ( partition by user_id, start_ts )                              as max_leadout_duration,
        median( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts )     as med_leadout_duration,
        avg( leadout_duration ) over ( partition by user_id, start_ts )                              as avg_leadout_duration,
        stddev_pop( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts ) as std_leadout_duration,
        sum( leadout_duration ) over ( partition by user_id, start_ts )                              as sum_leadout_duration,

        min( query_total_duration ) over ( partition by user_id, start_ts )                              as min_query_total_duration,
        max( query_total_duration ) over ( partition by user_id, start_ts )                              as max_query_total_duration,
        median( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts )     as med_query_total_duration,
        avg( query_total_duration ) over ( partition by user_id, start_ts )                              as avg_query_total_duration,
        stddev_pop( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts ) as std_query_total_duration,
        sum( query_total_duration ) over ( partition by user_id, start_ts )                              as sum_query_total_duration,

        min( unaccounted_duration ) over ( partition by user_id, start_ts )                              as min_unaccounted_duration,
        max( unaccounted_duration ) over ( partition by user_id, start_ts )                              as max_unaccounted_duration,
        median( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts )     as med_unaccounted_duration,
        avg( unaccounted_duration ) over ( partition by user_id, start_ts )                              as avg_unaccounted_duration,
        stddev_pop( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts ) as std_unaccounted_duration,
        sum( unaccounted_duration ) over ( partition by user_id, start_ts )                              as sum_unaccounted_duration,

        count(*) over ( partition by user_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    cte_queries_stats.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,

    min_leadin_duration::interval,
    max_leadin_duration::interval,
    med_leadin_duration::float4,
    avg_leadin_duration::interval,
    std_leadin_duration::float4,
    sum_leadin_duration::interval,

    min_wlm_leadin_duration::interval,
    max_wlm_leadin_duration::interval,
    med_wlm_leadin_duration::float4,
    avg_wlm_leadin_duration::interval,
    std_wlm_leadin_duration::float4,
    sum_wlm_leadin_duration::interval,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    min_wlm_leadout_duration::interval,
    max_wlm_leadout_duration::interval,
    med_wlm_leadout_duration::float4,
    avg_wlm_leadout_duration::interval,
    std_wlm_leadout_duration::float4,
    sum_wlm_leadout_duration::interval,

    min_leadout_duration::interval,
    max_leadout_duration::interval,
    med_leadout_duration::float4,
    avg_leadout_duration::interval,
    std_leadout_duration::float4,
    sum_leadout_duration::interval,

    min_query_total_duration::interval,
    max_query_total_duration::interval,
    med_query_total_duration::float4,
    avg_query_total_duration::interval,
    std_query_total_duration::float4,
    sum_query_total_duration::interval,

    min_unaccounted_duration::interval,
    max_unaccounted_duration::interval,
    med_unaccounted_duration::float4,
    avg_unaccounted_duration::interval,
    std_unaccounted_duration::float4,
    sum_unaccounted_duration::interval,

    number_queries::int4
  from
              base_241129.generate_hours
    left join cte_queries_stats     using ( start_ts )
    left join base_241129.fixed_pg_user using ( user_id )
  group by
    cte_queries_stats.user_id,
    user_name,
    start_ts,
    min_leadin_duration,
    max_leadin_duration,
    med_leadin_duration,
    avg_leadin_duration,
    std_leadin_duration,
    sum_leadin_duration,
    min_wlm_leadin_duration,
    max_wlm_leadin_duration,
    med_wlm_leadin_duration,
    avg_wlm_leadin_duration,
    std_wlm_leadin_duration,
    sum_wlm_leadin_duration,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    min_wlm_leadout_duration,
    max_wlm_leadout_duration,
    med_wlm_leadout_duration,
    avg_wlm_leadout_duration,
    std_wlm_leadout_duration,
    sum_wlm_leadout_duration,
    min_leadout_duration,
    max_leadout_duration,
    med_leadout_duration,
    avg_leadout_duration,
    std_leadout_duration,
    sum_leadout_duration,
    min_query_total_duration,
    max_query_total_duration,
    med_query_total_duration,
    avg_query_total_duration,
    std_query_total_duration,
    sum_query_total_duration,
    min_unaccounted_duration,
    max_unaccounted_duration,
    med_unaccounted_duration,
    avg_unaccounted_duration,
    std_unaccounted_duration,
    sum_unaccounted_duration,
    number_queries;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_slices_by_second
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        start_ts,
        slice_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        start_ts,
        slice_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        start_ts,
        slice_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        start_ts,
        slice_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        start_ts,
        slice_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        start_ts,
        slice_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        start_ts,
        slice_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        start_ts,
        slice_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        start_ts,
        slice_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        start_ts,
        slice_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        start_ts,
        slice_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        start_ts,
        slice_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        start_ts,
        slice_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        start_ts,
        slice_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'second', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        start_ts,
        slice_ordinal
    )

  select
    base_241129.generate_seconds.start_ts::timestamp as start_ts,

    base_241129.fixed_stv_slices.node_ordinal::int2,
    base_241129.fixed_stv_slices.slice_ordinal::int2,
    base_241129.fixed_stv_slices.slice_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_seconds
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, slice_ordinal )
     left join cte_bcast_stats            using ( start_ts, slice_ordinal )
     left join cte_delete_stats           using ( start_ts, slice_ordinal )
     left join cte_dist_stats             using ( start_ts, slice_ordinal )
     left join cte_hash_stats             using ( start_ts, slice_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, slice_ordinal )
     left join cte_insert_stats           using ( start_ts, slice_ordinal )
     left join cte_limit_stats            using ( start_ts, slice_ordinal )
     left join cte_merge_stats            using ( start_ts, slice_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, slice_ordinal )
     left join cte_nestloop_stats         using ( start_ts, slice_ordinal )
     left join cte_parse_stats            using ( start_ts, slice_ordinal )
     left join cte_project_stats          using ( start_ts, slice_ordinal )
     left join cte_return_stats           using ( start_ts, slice_ordinal )
     left join cte_save_stats             using ( start_ts, slice_ordinal )
     left join cte_sort_stats             using ( start_ts, slice_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, slice_ordinal )
     left join cte_unique_stats           using ( start_ts, slice_ordinal )
     left join cte_window_stats           using ( start_ts, slice_ordinal );


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_all_by_hour
as
  select
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_hours
    left join base_241129.fixed_stl_query on date_trunc( 'hour', query_start_ts ) = base_241129.generate_hours.start_ts
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_users_by_day
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4               as user_id,
    trim( pg_user.usename )::varchar     as user_name,
    base_241129.generate_days.start_ts::timestamp as start_ts,
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_days
    cross join pg_user
     left join cte_aggr_stats         using ( start_ts, node_type, usesysid )
     left join cte_bcast_stats        using ( start_ts, node_type, usesysid )
     left join cte_delete_stats       using ( start_ts, node_type, usesysid )
     left join cte_dist_stats         using ( start_ts, node_type, usesysid )
     left join cte_hash_stats         using ( start_ts, node_type, usesysid )
     left join cte_hashjoin_stats     using ( start_ts, node_type, usesysid )
     left join cte_insert_stats       using ( start_ts, node_type, usesysid )
     left join cte_limit_stats        using ( start_ts, node_type, usesysid )
     left join cte_merge_stats        using ( start_ts, node_type, usesysid )
     left join cte_mergejoin_stats    using ( start_ts, node_type, usesysid )
     left join cte_nestloop_stats     using ( start_ts, node_type, usesysid )
     left join cte_parse_stats        using ( start_ts, node_type, usesysid )
     left join cte_project_stats      using ( start_ts, node_type, usesysid )
     left join cte_return_stats       using ( start_ts, node_type, usesysid )
     left join cte_save_stats         using ( start_ts, node_type, usesysid )
     left join cte_sort_stats         using ( start_ts, node_type, usesysid )
     left join cte_spatial_join_stats using ( start_ts, node_type, usesysid )
     left join cte_unique_stats       using ( start_ts, node_type, usesysid )
     left join cte_window_stats       using ( start_ts, node_type, usesysid )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_bcast_duration is not null
    or max_bcast_duration is not null
    or med_bcast_duration is not null
    or avg_bcast_duration is not null
    or std_bcast_duration is not null
    or number_bcast_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_dist_duration is not null
    or max_dist_duration is not null
    or med_dist_duration is not null
    or avg_dist_duration is not null
    or std_dist_duration is not null
    or number_dist_steps is not null
    or min_hash_duration is not null
    or max_hash_duration is not null
    or med_hash_duration is not null
    or avg_hash_duration is not null
    or std_hash_duration is not null
    or number_hash_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or min_limit_duration is not null
    or max_limit_duration is not null
    or med_limit_duration is not null
    or avg_limit_duration is not null
    or std_limit_duration is not null
    or number_limit_steps is not null
    or min_merge_duration is not null
    or max_merge_duration is not null
    or med_merge_duration is not null
    or avg_merge_duration is not null
    or std_merge_duration is not null
    or number_merge_steps is not null
    or min_mergejoin_duration is not null
    or max_mergejoin_duration is not null
    or med_mergejoin_duration is not null
    or avg_mergejoin_duration is not null
    or std_mergejoin_duration is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or min_parse_duration is not null
    or max_parse_duration is not null
    or med_parse_duration is not null
    or avg_parse_duration is not null
    or std_parse_duration is not null
    or number_parse_steps is not null
    or min_project_duration is not null
    or max_project_duration is not null
    or med_project_duration is not null
    or avg_project_duration is not null
    or std_project_duration is not null
    or number_project_steps is not null
    or min_return_duration is not null
    or max_return_duration is not null
    or med_return_duration is not null
    or avg_return_duration is not null
    or std_return_duration is not null
    or number_return_steps is not null
    or min_save_duration is not null
    or max_save_duration is not null
    or med_save_duration is not null
    or avg_save_duration is not null
    or std_save_duration is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null
    or min_unique_duration is not null
    or max_unique_duration is not null
    or med_unique_duration is not null
    or avg_unique_duration is not null
    or std_unique_duration is not null
    or number_unique_steps is not null
    or min_window_duration is not null
    or max_window_duration is not null
    or med_window_duration is not null
    or avg_window_duration is not null
    or std_window_duration is not null
    or number_window_steps is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_day_concise
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::date,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_day
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_tables_by_minute
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'minute', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    cte_s1.table_id::int4,
    trim( pg_class.relname )::varchar      as table_name,
    node_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r';


create or replace view
  lf_241129.analyze_totals_for_tables_by_all
as
  with
    cte_s1 as
    (
      select
        table_id,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            database   = current_database()
        and starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        table_id
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4::int4              as table_id,
    trim( pg_class.relname )::varchar     as table_name,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_slices_by_minute
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        start_ts,
        slice_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        start_ts,
        slice_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        start_ts,
        slice_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        start_ts,
        slice_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        start_ts,
        slice_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        start_ts,
        slice_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        start_ts,
        slice_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        start_ts,
        slice_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        start_ts,
        slice_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        start_ts,
        slice_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        start_ts,
        slice_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        start_ts,
        slice_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        start_ts,
        slice_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        start_ts,
        slice_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        start_ts,
        slice_ordinal
    )

  select
    base_241129.generate_minutes.start_ts::timestamp as start_ts,

    base_241129.fixed_stv_slices.node_ordinal::int2,
    base_241129.fixed_stv_slices.slice_ordinal::int2,
    base_241129.fixed_stv_slices.slice_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_minutes
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, slice_ordinal )
     left join cte_bcast_stats            using ( start_ts, slice_ordinal )
     left join cte_delete_stats           using ( start_ts, slice_ordinal )
     left join cte_dist_stats             using ( start_ts, slice_ordinal )
     left join cte_hash_stats             using ( start_ts, slice_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, slice_ordinal )
     left join cte_insert_stats           using ( start_ts, slice_ordinal )
     left join cte_limit_stats            using ( start_ts, slice_ordinal )
     left join cte_merge_stats            using ( start_ts, slice_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, slice_ordinal )
     left join cte_nestloop_stats         using ( start_ts, slice_ordinal )
     left join cte_parse_stats            using ( start_ts, slice_ordinal )
     left join cte_project_stats          using ( start_ts, slice_ordinal )
     left join cte_return_stats           using ( start_ts, slice_ordinal )
     left join cte_save_stats             using ( start_ts, slice_ordinal )
     left join cte_sort_stats             using ( start_ts, slice_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, slice_ordinal )
     left join cte_unique_stats           using ( start_ts, slice_ordinal )
     left join cte_window_stats           using ( start_ts, slice_ordinal );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_minute_concise
as
  select
    start_ts::timestamp,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_minute
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_tables_by_day
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'day', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    cte_s1.table_id::int4,
    trim( pg_class.relname )::varchar      as table_name,
    node_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r';


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_users_by_second
as
  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    count(*)::int4                       as number_abort
  from
              base_241129.generate_seconds
    left join base_241129.fixed_stl_query    on date_trunc( 'second', query_start_ts ) = base_241129.generate_seconds.start_ts
    left join base_241129.fixed_pg_user   using ( user_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    user_id,
    user_name,
    start_ts;



create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_users_by_second
as
  select
    user_name::varchar as uuser,
    start_ts::timestamp,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_users_by_second
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_all_by_minute
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_scan as
    (
      select
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    )

  select
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
         base_241129.generate_minutes
    left join cte_delete using ( start_ts )
    left join cte_insert using ( start_ts )
    left join cte_scan   using ( start_ts );


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_all_by_minute
as
  select
    start_ts::timestamp,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_all_by_minute
  order by
    start_ts;



create or replace view
  lf_241129.blocks_rows_totals_for_schema
as
  with
    cte_table_sort_type as
    (
      select
        pg_class.oid as table_id,
        case
          when ( min(pg_attribute.attsortkeyord) >= 0 and max(pg_attribute.attsortkeyord) >  0 ) then 'compound'
          when ( min(pg_attribute.attsortkeyord) <  0                                          ) then 'interleaved'
          when ( min(pg_attribute.attsortkeyord)  = 0 and max(pg_attribute.attsortkeyord)  = 0 ) then 'unsorted'
        end          as table_sort_type
      from
             pg_class
        join pg_attribute on pg_class.oid = pg_attribute.attrelid
      where
        pg_class.relkind = 'r'
      group by
        pg_class.oid
    ),

    cte_blocks_and_rows_by_slice as
    (
      select
        stv_blocklist.tbl                                                                                              as table_id,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      where
        tbl != 0
      group by
        stv_blocklist.tbl
    ),

    cte_nodes as
    (
      select
        count( distinct node ) as number_nodes
      from
        stv_slices
    ),

    cte_s2 as
    (
      select
        pg_class_info.relnamespace    as schema_id,
        trim( pg_namespace.nspname )  as schema_name,

        sum( number_sorted_blocks                                                                               ) as number_sorted_blocks,
        sum( case when table_sort_type = 'unsorted' then 0                      else number_unsorted_blocks end ) as number_desorted_blocks,
        sum( case when table_sort_type = 'unsorted' then number_unsorted_blocks else 0                      end ) as number_unsorted_blocks,
        sum( number_total_blocks                                                                                ) as number_total_blocks,

        sum( case when (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_by_slice.number_sorted_rows   / cte_nodes.number_nodes else cte_blocks_and_rows_by_slice.number_sorted_rows   end ) as number_sorted_rows,
        sum
        (
          case
            when cte_table_sort_type.table_sort_type  = 'unsorted'                                                                                                                       then 0
            when cte_table_sort_type.table_sort_type != 'unsorted' and (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_by_slice.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                                                                                         else cte_blocks_and_rows_by_slice.number_unsorted_rows
          end
        ) as number_desorted_rows,
        sum
        (
          case
            when cte_table_sort_type.table_sort_type != 'unsorted'                                                                                                                       then 0
            when cte_table_sort_type.table_sort_type  = 'unsorted' and (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_by_slice.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                                                                                         else cte_blocks_and_rows_by_slice.number_unsorted_rows
          end
        ) as number_unsorted_rows,
        sum( case when (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_by_slice.number_total_rows    / cte_nodes.number_nodes else cte_blocks_and_rows_by_slice.number_total_rows    end ) as number_total_rows
      from
                   cte_blocks_and_rows_by_slice
              join cte_table_sort_type using ( table_id )
              join pg_class_info          on pg_class_info.reloid       = cte_blocks_and_rows_by_slice.table_id
              join pg_namespace           on pg_class_info.relnamespace = pg_namespace.oid
        cross join cte_nodes
      group by
        schema_id,
        schema_name
    )

  select
    schema_id::int4,
    schema_name::varchar,
    number_sorted_blocks::int8,
    number_desorted_blocks::int8,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    number_desorted_rows::int8,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
    cte_s2;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_statements_by_all
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'aggr' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'bcast' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'delete' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'dist' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hash' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hashjoin' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'insert' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'limit' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'merge' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'mergejoin' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'nestloop_op' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'parse' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'project' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'return' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'save' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'sort' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'spatial_join' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'unique' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'window' as step_type,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_steps as
    (
      select * from cte_aggr         union all
      select * from cte_bcast        union all
      select * from cte_delete       union all
      select * from cte_dist         union all
      select * from cte_hash         union all
      select * from cte_hashjoin     union all
      select * from cte_insert       union all
      select * from cte_limit        union all
      select * from cte_merge        union all
      select * from cte_mergejoin    union all
      select * from cte_nestloop     union all
      select * from cte_parse        union all
      select * from cte_project      union all
      select * from cte_return       union all
      select * from cte_save         union all
      select * from cte_sort         union all
      select * from cte_spatial_join union all
      select * from cte_unique       union all
      select * from cte_window
    )

  select
    statement_name::varchar,
    node_type::varchar,
    step_type::varchar,

    min( duration )::interval                          as min_duration,
    max( duration )::interval                          as max_duration,
    median( extract(epoch from duration) )::float4     as med_duration,
    avg( duration )::interval                          as avg_duration,
    stddev_pop( extract(epoch from duration) )::float4 as std_duration,
    count(*)::int8                                     as number_steps
  from
                 cte_base
    natural join cte_steps
  group by
    statement_name,
    node_type,
    step_type;


create or replace view
  lf_241129.wqueues_totals_slot_utilization_by_day
as
  with
    cte_events as
    (
      select
        queue_id,
        query_compile_exec_start_ts as event_ts,
        1                   as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'

      union all

      select
        queue_id,
        query_compile_exec_end_ts as event_ts,
        -1                as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_slot_durations as
    (
      select
        queue_id,
        sum( event_delta ) over ( partition by queue_id, start_ts order by event_ts rows between unbounded preceding and current row ) as slot_number,
        date_trunc( 'day', event_ts )                                                                                             as start_ts,
        lead( event_ts   ) over ( partition by queue_id, start_ts order by event_ts ) - event_ts                                       as duration
      from
        cte_events
    ),

    cte_durations as
    (
      select
        queue_id,
        slot_number,
        start_ts,
        sum( duration ) as sum_duration
      from
        cte_slot_durations
      group by
        queue_id,
        slot_number,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    slot_number::int2,
    start_ts::timestamp,
    sum_duration::interval as duration
  from
              cte_durations
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  sf_241129.wqueues_totals_slot_utilization_by_day
as
  select
    queue_name::varchar                           as qname,
    slot_number::int2                             as slot,
    start_ts::date,
    extract( epoch from duration )::numeric(18,2) as dur
  from
    lf_241129.wqueues_totals_slot_utilization_by_day
  order by
    queue_name,
    slot_number,
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_database_by_all
as
  select
    database::varchar                                                                                             as database_name,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    database;


create or replace view
  sf_241129.analyze_totals_for_database_by_all
as
  with
    cte_hr_formatting as
    (
      select
        database_name,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_database_by_all
    )

  select
    database_name::varchar as dbname,

    ( hr_number_manual_full_analyze             || ' / ' ||
      hr_number_manual_predictatecolumn_analyze || ' / ' ||
      hr_number_manual_skipped_analyze          )::varchar as manual,

    ( hr_number_automatic_full_analyze             || ' / ' ||
      hr_number_automatic_predictatecolumn_analyze || ' / ' ||
      hr_number_automatic_skipped_analyze          )::varchar as automatic,

    ( hr_number_autoanalyze_full_analyze             || ' / ' ||
      hr_number_autoanalyze_predictatecolumn_analyze || ' / ' ||
      hr_number_autoanalyze_skipped_analyze          )::varchar as autoanalyze,

    ( hr_number_total_full_analyze             || ' / ' ||
      hr_number_total_predictatecolumn_analyze || ' / ' ||
      hr_number_total_skipped_analyze          )::varchar as total
  from
    cte_hr_formatting
  order by
    database_name;



create or replace view
  lf_241129.steps_stats_durations_on_both_for_all_by_hour
as
  with
    cte_aggr as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime)  as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_hours.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_hours
     left join cte_aggr_stats             using ( node_type, start_ts )
     left join cte_bcast_stats            using ( node_type, start_ts )
     left join cte_delete_stats           using ( node_type, start_ts )
     left join cte_dist_stats             using ( node_type, start_ts )
     left join cte_hash_stats             using ( node_type, start_ts )
     left join cte_hashjoin_stats         using ( node_type, start_ts )
     left join cte_insert_stats           using ( node_type, start_ts )
     left join cte_limit_stats            using ( node_type, start_ts )
     left join cte_merge_stats            using ( node_type, start_ts )
     left join cte_mergejoin_stats        using ( node_type, start_ts )
     left join cte_nestloop_stats         using ( node_type, start_ts )
     left join cte_parse_stats            using ( node_type, start_ts )
     left join cte_project_stats          using ( node_type, start_ts )
     left join cte_return_stats           using ( node_type, start_ts )
     left join cte_save_stats             using ( node_type, start_ts )
     left join cte_sort_stats             using ( node_type, start_ts )
     left join cte_spatial_join_stats     using ( node_type, start_ts )
     left join cte_unique_stats           using ( node_type, start_ts )
     left join cte_window_stats           using ( node_type, start_ts );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_hour_complete
as
  select
    node_type::varchar,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_hour
  order by
    node_type,
    start_ts;



create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_all_by_day
as
  with
    cte_blocks as
    (
      select
        date_trunc( 'day', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;

create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_all_by_day
as
  select
    start_ts::date,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_all_by_day
  order by
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_node_by_all
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
             cte_aggr
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
             cte_bcast
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
             cte_delete
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
             cte_dist
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
             cte_hash
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
             cte_hashjoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
             cte_insert
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
             cte_limit
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
             cte_merge
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
             cte_mergejoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
             cte_nestloop
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
             cte_parse
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
             cte_project
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
             cte_return
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
             cte_save
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
             cte_sort
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
             cte_spatial_join
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
             cte_unique
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        node_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
             cte_window
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        node_ordinal
    )

  select
    base_241129.fixed_stv_slices.node_ordinal::int2,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( node_ordinal )
     left join cte_bcast_stats            using ( node_ordinal )
     left join cte_delete_stats           using ( node_ordinal )
     left join cte_dist_stats             using ( node_ordinal )
     left join cte_hash_stats             using ( node_ordinal )
     left join cte_hashjoin_stats         using ( node_ordinal )
     left join cte_insert_stats           using ( node_ordinal )
     left join cte_limit_stats            using ( node_ordinal )
     left join cte_merge_stats            using ( node_ordinal )
     left join cte_mergejoin_stats        using ( node_ordinal )
     left join cte_nestloop_stats         using ( node_ordinal )
     left join cte_parse_stats            using ( node_ordinal )
     left join cte_project_stats          using ( node_ordinal )
     left join cte_return_stats           using ( node_ordinal )
     left join cte_save_stats             using ( node_ordinal )
     left join cte_sort_stats             using ( node_ordinal )
     left join cte_spatial_join_stats     using ( node_ordinal )
     left join cte_unique_stats           using ( node_ordinal )
     left join cte_window_stats           using ( node_ordinal );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_all_complete
as
  select
    node_ordinal::int2 as n,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_all
  order by
    node_ordinal;


create or replace view
  lf_241129.steps_totals_data_on_both_for_node_by_second
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_seconds
     left join cte_aggr         using( slice_ordinal, start_ts )
     left join cte_bcast        using( slice_ordinal, start_ts )
     left join cte_delete       using( slice_ordinal, start_ts )
     left join cte_dist         using( slice_ordinal, start_ts )
     left join cte_hash         using( slice_ordinal, start_ts )
     left join cte_hashjoin     using( slice_ordinal, start_ts )
     left join cte_insert       using( slice_ordinal, start_ts )
     left join cte_limit        using( slice_ordinal, start_ts )
     left join cte_merge        using( slice_ordinal, start_ts )
     left join cte_mergejoin    using( slice_ordinal, start_ts )
     left join cte_nestloop     using( slice_ordinal, start_ts )
     left join cte_parse        using( slice_ordinal, start_ts )
     left join cte_project      using( slice_ordinal, start_ts )
     left join cte_return       using( slice_ordinal, start_ts )
     left join cte_save         using( slice_ordinal, start_ts )
     left join cte_sort         using( slice_ordinal, start_ts )
     left join cte_spatial_join using( slice_ordinal, start_ts )
     left join cte_unique       using( slice_ordinal, start_ts )
     left join cte_window       using( slice_ordinal, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null
  group by
    node_ordinal,
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_schemas_by_hour
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    start_ts::timestamp,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_scan.sum_scan_rows )::int8     as sum_scan_rows
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_hours
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null
  group by
    base_241129.fixed_pg_namespace.schema_id,
    base_241129.fixed_pg_namespace.schema_name,
    start_ts;



create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_all_by_all
as
  with
    cte_blocks as
    (
      select
        sum( query_number_blocks_read    ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
    ),

    cte_nulls_to_values as
    (
      select
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_all_by_all
as
  select
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_all_by_all;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_queues_by_all
as
  select
    queue_id::int2,
    queue_name::varchar,
    count(*)::int4 as number_abort
  from
              base_241129.fixed_stl_query
         join base_241129.fixed_stl_wlm_query                using ( query_id )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    queue_id,
    queue_name;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_queues_by_all
as
  select
    queue_name::varchar as qname,
    number_abort::int4   as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_queues_by_all
  order by
    queue_name;



create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_all_by_second
as
  select
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_seconds
    left join base_241129.fixed_stl_query on date_trunc( 'second', query_start_ts ) = base_241129.generate_seconds.start_ts
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_node_by_day
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
             cte_aggr
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
             cte_bcast
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
             cte_delete
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
             cte_dist
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
             cte_hash
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
             cte_hashjoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
             cte_insert
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
             cte_limit
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
             cte_merge
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
             cte_mergejoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
             cte_nestloop
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
             cte_parse
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
             cte_project
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
             cte_return
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
             cte_save
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
             cte_sort
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
             cte_spatial_join
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
             cte_unique
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'day', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
             cte_window
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    )

  select
    base_241129.generate_days.start_ts::timestamp as start_ts,
    base_241129.fixed_stv_slices.node_ordinal::int2,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_days
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, node_ordinal )
     left join cte_bcast_stats            using ( start_ts, node_ordinal )
     left join cte_delete_stats           using ( start_ts, node_ordinal )
     left join cte_dist_stats             using ( start_ts, node_ordinal )
     left join cte_hash_stats             using ( start_ts, node_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, node_ordinal )
     left join cte_insert_stats           using ( start_ts, node_ordinal )
     left join cte_limit_stats            using ( start_ts, node_ordinal )
     left join cte_merge_stats            using ( start_ts, node_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, node_ordinal )
     left join cte_nestloop_stats         using ( start_ts, node_ordinal )
     left join cte_parse_stats            using ( start_ts, node_ordinal )
     left join cte_project_stats          using ( start_ts, node_ordinal )
     left join cte_return_stats           using ( start_ts, node_ordinal )
     left join cte_save_stats             using ( start_ts, node_ordinal )
     left join cte_sort_stats             using ( start_ts, node_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, node_ordinal )
     left join cte_unique_stats           using ( start_ts, node_ordinal )
     left join cte_window_stats           using ( start_ts, node_ordinal );


create or replace view
  lf_241129.objects_totals_for_all
as
  with
    cte_pg_class as
    (
      select
        -- MG2 : number tables
        sum( case when pg_class.relkind = 'r'                                                             then 1 else 0 end ) as number_tables,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then 1 else 0 end ) as number_normal_views,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts  = 0 and stv_mv_info.name is     null then 1 else 0 end ) as number_late_binding_views,
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then 1 else 0 end ) as number_materialized_views,

        -- MG2 : number columns
        sum( case when pg_class.relkind = 'r'                                                             then pg_class.relnatts + 3 else 0 end ) as number_table_columns,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then pg_class.relnatts     else 0 end ) as number_normal_view_columns,
        -- MG2 : relnatts is 0 for late binding views
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then pg_class.relnatts     else 0 end ) as number_materialized_view_columns
      from
                  pg_class
             join pg_namespace on pg_class.relnamespace = pg_namespace.oid
        left join stv_mv_info  on stv_mv_info.db_name = current_database()   and
                                  stv_mv_info.schema  = pg_namespace.nspname and
                                  stv_mv_info.name    = pg_class.relname
      where
        pg_class.relkind = 'r' or pg_class.relkind = 'v'
    ),

    cte_pg_namespace as
    (
      select
        count(*) as number_schemas
      from
        pg_namespace
    ),

    cte_pg_proc as
    (
      select
        sum( case when pg_language.lanname != 'plpgsql' then 1 else 0 end ) as number_functions,
        sum( case when pg_language.lanname  = 'plpgsql' then 1 else 0 end ) as number_procedures
      from
             pg_proc
        join pg_language on pg_proc.prolang = pg_language.oid
    )

  select
    number_schemas::int2,
    number_tables::int4,
    number_table_columns::int4,
    number_normal_views::int4,
    number_normal_view_columns::int4,
    number_late_binding_views::int4,
    number_materialized_views::int4,
    number_materialized_view_columns::int4,
    number_functions::int4,
    number_procedures::int4
  from
               cte_pg_namespace
    cross join cte_pg_class
    cross join cte_pg_proc;


create or replace view
  base_241129.fixed_pg_proc
as
  select
    pg_proc.oid::int4        as object_id,
    trim( proname )::varchar as object_name,
    pronamespace::int4       as schema_id,
    proowner::int4           as user_id,
    prolang::int4            as language_id,
    trim( pg_language.lanname )::varchar as language_name,
    provolatile::char,
    prorettype::oid,
    proretset::bool,
    prosrc::text,
    proargtypes::oidvector
  from
         pg_proc
    join pg_language on pg_proc.prolang = pg_language.oid;


create or replace view
  base_241129.fixed_pg_proc_for_functions
as
  select
    object_id::int4      as function_id,
    object_name::varchar as function_name,
    schema_id::int4,
    user_id::int4,
    language_id::int4,
    language_name::varchar,
    provolatile::char,
    prorettype::oid,
    proretset::bool,
    prosrc::text as function_text,
    proargtypes::oidvector
  from
    base_241129.fixed_pg_proc
  where
    language_name != 'plpgsql';


create or replace view
  lf_241129.transactions_stats_commits_by_day
as
  with
    cte_transaction_queuing_stats as
    (
      select
        date_trunc( 'day', transaction_batch_queue_start )                                         as start_ts,
        min( batch_leader_startwork - transaction_batch_queue_start )                              as min_transaction_batch_queue_start,
        max( batch_leader_startwork - transaction_batch_queue_start )                              as max_transaction_batch_queue_start,
        median( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) )     as med_transaction_batch_queue_start,
        avg( batch_leader_startwork - transaction_batch_queue_start )                              as avg_transaction_batch_queue_start,
        stddev_pop( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) ) as std_transaction_batch_queue_start,
        sum( batch_leader_startwork - transaction_batch_queue_start )                              as sum_transaction_batch_queue_start
      from
             base_241129.transactions_in_batches
        join lf_241129.batches using ( batch_id )
      group by
        start_ts
    ),

    cte_batch_stats as
    (
      select
        date_trunc( 'day', batch_leader_startwork ) as start_ts,

        min( batch_queue_duration )                            over ( partition by start_ts ) as min_batch_queue_duration,
        max( batch_queue_duration )                            over ( partition by start_ts ) as max_batch_queue_duration,
        median( extract(epoch from batch_queue_duration) )     over ( partition by start_ts ) as med_batch_queue_duration,
        avg( batch_queue_duration )                            over ( partition by start_ts ) as avg_batch_queue_duration,
        stddev_pop( extract(epoch from batch_queue_duration) ) over ( partition by start_ts ) as std_batch_queue_duration,
        sum( batch_queue_duration )                            over ( partition by start_ts ) as sum_batch_queue_duration,

        min( batch_leader_duration )                            over ( partition by start_ts ) as min_batch_leader_duration,
        max( batch_leader_duration )                            over ( partition by start_ts ) as max_batch_leader_duration,
        median( extract(epoch from batch_leader_duration) )     over ( partition by start_ts ) as med_batch_leader_duration,
        avg( batch_leader_duration )                            over ( partition by start_ts ) as avg_batch_leader_duration,
        stddev_pop( extract(epoch from batch_leader_duration) ) over ( partition by start_ts ) as std_batch_leader_duration,
        sum( batch_leader_duration )                            over ( partition by start_ts ) as sum_batch_leader_duration,

        min( batch_worker_duration )                            over ( partition by start_ts ) as min_batch_worker_duration,
        max( batch_worker_duration )                            over ( partition by start_ts ) as max_batch_worker_duration,
        median( extract(epoch from batch_worker_duration) )     over ( partition by start_ts ) as med_batch_worker_duration,
        avg( batch_worker_duration )                            over ( partition by start_ts ) as avg_batch_worker_duration,
        stddev_pop( extract(epoch from batch_worker_duration) ) over ( partition by start_ts ) as std_batch_worker_duration,
        sum( batch_worker_duration )                            over ( partition by start_ts ) as sum_batch_worker_duration,

        min( number_batch_transactions )         over ( partition by start_ts ) as min_number_batch_transactions,
        max( number_batch_transactions )         over ( partition by start_ts ) as max_number_batch_transactions,
        median( number_batch_transactions )      over ( partition by start_ts ) as med_number_batch_transactions,
        avg( number_batch_transactions::float4 ) over ( partition by start_ts ) as avg_number_batch_transactions,
        stddev_pop( number_batch_transactions )  over ( partition by start_ts ) as std_number_batch_transactions,
        sum( number_batch_transactions )         over ( partition by start_ts ) as sum_number_batch_transactions,

        min( number_batch_blocks )         over ( partition by start_ts ) as min_number_batch_blocks,
        max( number_batch_blocks )         over ( partition by start_ts ) as max_number_batch_blocks,
        median( number_batch_blocks )      over ( partition by start_ts ) as med_number_batch_blocks,
        avg( number_batch_blocks::float4 ) over ( partition by start_ts ) as avg_number_batch_blocks,
        stddev_pop( number_batch_blocks )  over ( partition by start_ts ) as std_number_batch_blocks,
        sum( number_batch_blocks )         over ( partition by start_ts ) as sum_number_batch_blocks,

        count(*) over ( partition by start_ts ) as number_batches
      from
        lf_241129.batches
    )

  select
    cte_batch_stats.start_ts::timestamp,

    cte_transaction_queuing_stats.min_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start::interval,

    cte_batch_stats.min_batch_queue_duration::interval,
    cte_batch_stats.max_batch_queue_duration::interval,
    cte_batch_stats.med_batch_queue_duration::float4,
    cte_batch_stats.avg_batch_queue_duration::interval,
    cte_batch_stats.std_batch_queue_duration::float4,
    cte_batch_stats.sum_batch_queue_duration::interval,

    cte_batch_stats.min_batch_leader_duration::interval,
    cte_batch_stats.max_batch_leader_duration::interval,
    cte_batch_stats.med_batch_leader_duration::float4,
    cte_batch_stats.avg_batch_leader_duration::interval,
    cte_batch_stats.std_batch_leader_duration::float4,
    cte_batch_stats.sum_batch_leader_duration::interval,

    cte_batch_stats.min_batch_worker_duration::interval,
    cte_batch_stats.max_batch_worker_duration::interval,
    cte_batch_stats.med_batch_worker_duration::float4,
    cte_batch_stats.avg_batch_worker_duration::interval,
    cte_batch_stats.std_batch_worker_duration::float4,
    cte_batch_stats.sum_batch_worker_duration::interval,

    cte_batch_stats.min_number_batch_transactions::int8,
    cte_batch_stats.max_number_batch_transactions::int8,
    cte_batch_stats.med_number_batch_transactions::int8,
    cte_batch_stats.avg_number_batch_transactions::float4,
    cte_batch_stats.std_number_batch_transactions::float4,
    cte_batch_stats.sum_number_batch_transactions::int8,

    cte_batch_stats.min_number_batch_blocks::int8,
    cte_batch_stats.max_number_batch_blocks::int8,
    cte_batch_stats.med_number_batch_blocks::int8,
    cte_batch_stats.avg_number_batch_blocks::float4,
    cte_batch_stats.std_number_batch_blocks::float4,
    cte_batch_stats.sum_number_batch_blocks::int8,

    cte_batch_stats.number_batches::int8
  from
              cte_batch_stats
    left join cte_transaction_queuing_stats using ( start_ts )
  group by
    start_ts,
    cte_transaction_queuing_stats.min_transaction_batch_queue_start,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start,
    cte_batch_stats.min_batch_queue_duration,
    cte_batch_stats.max_batch_queue_duration,
    cte_batch_stats.med_batch_queue_duration,
    cte_batch_stats.avg_batch_queue_duration,
    cte_batch_stats.std_batch_queue_duration,
    cte_batch_stats.sum_batch_queue_duration,
    cte_batch_stats.min_batch_leader_duration,
    cte_batch_stats.max_batch_leader_duration,
    cte_batch_stats.med_batch_leader_duration,
    cte_batch_stats.avg_batch_leader_duration,
    cte_batch_stats.std_batch_leader_duration,
    cte_batch_stats.sum_batch_leader_duration,
    cte_batch_stats.min_batch_worker_duration,
    cte_batch_stats.max_batch_worker_duration,
    cte_batch_stats.med_batch_worker_duration,
    cte_batch_stats.avg_batch_worker_duration,
    cte_batch_stats.std_batch_worker_duration,
    cte_batch_stats.sum_batch_worker_duration,
    cte_batch_stats.min_number_batch_transactions,
    cte_batch_stats.max_number_batch_transactions,
    cte_batch_stats.med_number_batch_transactions,
    cte_batch_stats.avg_number_batch_transactions,
    cte_batch_stats.std_number_batch_transactions,
    cte_batch_stats.sum_number_batch_transactions,
    cte_batch_stats.min_number_batch_blocks,
    cte_batch_stats.max_number_batch_blocks,
    cte_batch_stats.med_number_batch_blocks,
    cte_batch_stats.avg_number_batch_blocks,
    cte_batch_stats.std_number_batch_blocks,
    cte_batch_stats.sum_number_batch_blocks,
    cte_batch_stats.number_batches;


create or replace view
  sf_241129.transactions_stats_commits_by_day
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_min_transaction_batch_queue_start,
        extract(epoch from max_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_max_transaction_batch_queue_start,
        med_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_med_transaction_batch_queue_start,
        extract(epoch from avg_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_avg_transaction_batch_queue_start,
        std_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_std_transaction_batch_queue_start,
        extract(epoch from sum_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_sum_transaction_batch_queue_start,

        extract(epoch from min_batch_queue_duration)::numeric(18,2)::varchar as hr_min_batch_queue_duration,
        extract(epoch from max_batch_queue_duration)::numeric(18,2)::varchar as hr_max_batch_queue_duration,
        med_batch_queue_duration::numeric(18,2)::varchar                     as hr_med_batch_queue_duration,
        extract(epoch from avg_batch_queue_duration)::numeric(18,2)::varchar as hr_avg_batch_queue_duration,
        std_batch_queue_duration::numeric(18,2)::varchar                     as hr_std_batch_queue_duration,
        extract(epoch from sum_batch_queue_duration)::numeric(18,2)::varchar as hr_sum_batch_queue_duration,

        extract(epoch from min_batch_leader_duration)::numeric(18,2)::varchar as hr_min_batch_leader_duration,
        extract(epoch from max_batch_leader_duration)::numeric(18,2)::varchar as hr_max_batch_leader_duration,
        med_batch_leader_duration::numeric(18,2)::varchar                     as hr_med_batch_leader_duration,
        extract(epoch from avg_batch_leader_duration)::numeric(18,2)::varchar as hr_avg_batch_leader_duration,
        std_batch_leader_duration::numeric(18,2)::varchar                     as hr_std_batch_leader_duration,
        extract(epoch from sum_batch_leader_duration)::numeric(18,2)::varchar as hr_sum_batch_leader_duration,

        extract(epoch from min_batch_worker_duration)::numeric(18,2)::varchar as hr_min_batch_worker_duration,
        extract(epoch from max_batch_worker_duration)::numeric(18,2)::varchar as hr_max_batch_worker_duration,
        med_batch_worker_duration::numeric(18,2)::varchar                     as hr_med_batch_worker_duration,
        extract(epoch from avg_batch_worker_duration)::numeric(18,2)::varchar as hr_avg_batch_worker_duration,
        std_batch_worker_duration::numeric(18,2)::varchar                     as hr_std_batch_worker_duration,
        extract(epoch from sum_batch_worker_duration)::numeric(18,2)::varchar as hr_sum_batch_worker_duration,

        min_number_batch_transactions::varchar                as hr_min_number_batch_transactions,
        max_number_batch_transactions::varchar                as hr_max_number_batch_transactions,
        med_number_batch_transactions::varchar                as hr_med_number_batch_transactions,
        avg_number_batch_transactions::numeric(18,2)::varchar as hr_avg_number_batch_transactions,
        std_number_batch_transactions::numeric(18,2)::varchar as hr_std_number_batch_transactions,
        sum_number_batch_transactions::varchar                as hr_sum_number_batch_transactions,

        base_241129.hr_number( min_number_batch_blocks ) as hr_min_number_batch_blocks,
        base_241129.hr_number( max_number_batch_blocks ) as hr_max_number_batch_blocks,
        base_241129.hr_number( med_number_batch_blocks ) as hr_med_number_batch_blocks,
        base_241129.hr_number( avg_number_batch_blocks ) as hr_avg_number_batch_blocks,
        base_241129.hr_number( std_number_batch_blocks ) as hr_std_number_batch_blocks,
        base_241129.hr_number( sum_number_batch_blocks ) as hr_sum_number_batch_blocks,

        number_batches,

        max( len(hr_min_transaction_batch_queue_start) ) over () as max_len_hr_min_transaction_batch_queue_start,
        max( len(hr_max_transaction_batch_queue_start) ) over () as max_len_hr_max_transaction_batch_queue_start,
        max( len(hr_med_transaction_batch_queue_start) ) over () as max_len_hr_med_transaction_batch_queue_start,
        max( len(hr_avg_transaction_batch_queue_start) ) over () as max_len_hr_avg_transaction_batch_queue_start,
        max( len(hr_std_transaction_batch_queue_start) ) over () as max_len_hr_std_transaction_batch_queue_start,
        max( len(hr_sum_transaction_batch_queue_start) ) over () as max_len_hr_sum_transaction_batch_queue_start,

        max( len(hr_min_batch_queue_duration) ) over () as max_len_hr_min_batch_queue_duration,
        max( len(hr_max_batch_queue_duration) ) over () as max_len_hr_max_batch_queue_duration,
        max( len(hr_med_batch_queue_duration) ) over () as max_len_hr_med_batch_queue_duration,
        max( len(hr_avg_batch_queue_duration) ) over () as max_len_hr_avg_batch_queue_duration,
        max( len(hr_std_batch_queue_duration) ) over () as max_len_hr_std_batch_queue_duration,
        max( len(hr_sum_batch_queue_duration) ) over () as max_len_hr_sum_batch_queue_duration,

        max( len(hr_min_batch_leader_duration) ) over () as max_len_hr_min_batch_leader_duration,
        max( len(hr_max_batch_leader_duration) ) over () as max_len_hr_max_batch_leader_duration,
        max( len(hr_med_batch_leader_duration) ) over () as max_len_hr_med_batch_leader_duration,
        max( len(hr_avg_batch_leader_duration) ) over () as max_len_hr_avg_batch_leader_duration,
        max( len(hr_std_batch_leader_duration) ) over () as max_len_hr_std_batch_leader_duration,
        max( len(hr_sum_batch_leader_duration) ) over () as max_len_hr_sum_batch_leader_duration,

        max( len(hr_min_batch_worker_duration) ) over () as max_len_hr_min_batch_worker_duration,
        max( len(hr_max_batch_worker_duration) ) over () as max_len_hr_max_batch_worker_duration,
        max( len(hr_med_batch_worker_duration) ) over () as max_len_hr_med_batch_worker_duration,
        max( len(hr_avg_batch_worker_duration) ) over () as max_len_hr_avg_batch_worker_duration,
        max( len(hr_std_batch_worker_duration) ) over () as max_len_hr_std_batch_worker_duration,
        max( len(hr_sum_batch_worker_duration) ) over () as max_len_hr_sum_batch_worker_duration,

        max( len(hr_min_number_batch_transactions) ) over () as max_len_hr_min_number_batch_transactions,
        max( len(hr_max_number_batch_transactions) ) over () as max_len_hr_max_number_batch_transactions,
        max( len(hr_med_number_batch_transactions) ) over () as max_len_hr_med_number_batch_transactions,
        max( len(hr_avg_number_batch_transactions) ) over () as max_len_hr_avg_number_batch_transactions,
        max( len(hr_std_number_batch_transactions) ) over () as max_len_hr_std_number_batch_transactions,
        max( len(hr_sum_number_batch_transactions) ) over () as max_len_hr_sum_number_batch_transactions,

        max( len(hr_min_number_batch_blocks) ) over () as max_len_hr_min_number_batch_blocks,
        max( len(hr_max_number_batch_blocks) ) over () as max_len_hr_max_number_batch_blocks,
        max( len(hr_med_number_batch_blocks) ) over () as max_len_hr_med_number_batch_blocks,
        max( len(hr_avg_number_batch_blocks) ) over () as max_len_hr_avg_number_batch_blocks,
        max( len(hr_std_number_batch_blocks) ) over () as max_len_hr_std_number_batch_blocks,
        max( len(hr_sum_number_batch_blocks) ) over () as max_len_hr_sum_number_batch_blocks
      from
        lf_241129.transactions_stats_commits_by_day
    )

  select
    start_ts::date,

    ( lpad( hr_min_transaction_batch_queue_start, max_len_hr_min_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_max_transaction_batch_queue_start, max_len_hr_max_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_med_transaction_batch_queue_start, max_len_hr_med_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_avg_transaction_batch_queue_start, max_len_hr_avg_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_std_transaction_batch_queue_start, max_len_hr_std_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_sum_transaction_batch_queue_start, max_len_hr_sum_transaction_batch_queue_start ) )::varchar              as "xid queuing",

    ( lpad( hr_min_batch_queue_duration, max_len_hr_min_batch_queue_duration ) || ' / ' ||
      lpad( hr_max_batch_queue_duration, max_len_hr_max_batch_queue_duration ) || ' / ' ||
      lpad( hr_med_batch_queue_duration, max_len_hr_med_batch_queue_duration ) || ' / ' ||
      lpad( hr_avg_batch_queue_duration, max_len_hr_avg_batch_queue_duration ) || ' / ' ||
      lpad( hr_std_batch_queue_duration, max_len_hr_std_batch_queue_duration ) || ' / ' ||
      lpad( hr_sum_batch_queue_duration, max_len_hr_sum_batch_queue_duration ) )::varchar          as "bid queuing",

    ( lpad( hr_min_batch_leader_duration, max_len_hr_min_batch_leader_duration ) || ' / ' ||
      lpad( hr_max_batch_leader_duration, max_len_hr_max_batch_leader_duration ) || ' / ' ||
      lpad( hr_med_batch_leader_duration, max_len_hr_med_batch_leader_duration ) || ' / ' ||
      lpad( hr_avg_batch_leader_duration, max_len_hr_avg_batch_leader_duration ) || ' / ' ||
      lpad( hr_std_batch_leader_duration, max_len_hr_std_batch_leader_duration ) || ' / ' ||
      lpad( hr_sum_batch_leader_duration, max_len_hr_sum_batch_leader_duration ) )::varchar          as "ldr duration",

    ( lpad( hr_min_batch_worker_duration, max_len_hr_min_batch_worker_duration ) || ' / ' ||
      lpad( hr_max_batch_worker_duration, max_len_hr_max_batch_worker_duration ) || ' / ' ||
      lpad( hr_med_batch_worker_duration, max_len_hr_med_batch_worker_duration ) || ' / ' ||
      lpad( hr_avg_batch_worker_duration, max_len_hr_avg_batch_worker_duration ) || ' / ' ||
      lpad( hr_std_batch_worker_duration, max_len_hr_std_batch_worker_duration ) || ' / ' ||
      lpad( hr_sum_batch_worker_duration, max_len_hr_sum_batch_worker_duration ) )::varchar          as "wrk duration",

    ( lpad( hr_min_number_batch_transactions, max_len_hr_min_number_batch_transactions ) || ' / ' ||
      lpad( hr_max_number_batch_transactions, max_len_hr_max_number_batch_transactions ) || ' / ' ||
      lpad( hr_med_number_batch_transactions, max_len_hr_med_number_batch_transactions ) || ' / ' ||
      lpad( hr_avg_number_batch_transactions, max_len_hr_avg_number_batch_transactions ) || ' / ' ||
      lpad( hr_std_number_batch_transactions, max_len_hr_std_number_batch_transactions ) || ' / ' ||
      lpad( hr_sum_number_batch_transactions, max_len_hr_sum_number_batch_transactions ) )::varchar  as "xids/bid",

    ( lpad( hr_min_number_batch_blocks, max_len_hr_min_number_batch_blocks ) || ' / ' ||
      lpad( hr_max_number_batch_blocks, max_len_hr_max_number_batch_blocks ) || ' / ' ||
      lpad( hr_med_number_batch_blocks, max_len_hr_med_number_batch_blocks ) || ' / ' ||
      lpad( hr_avg_number_batch_blocks, max_len_hr_avg_number_batch_blocks ) || ' / ' ||
      lpad( hr_std_number_batch_blocks, max_len_hr_std_number_batch_blocks ) || ' / ' ||
      lpad( hr_sum_number_batch_blocks, max_len_hr_sum_number_batch_blocks ) )::varchar              as "blks/bid",

    number_batches::int8                                                                                    as bids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_all_by_day
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_scan as
    (
      select
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    )

  select
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
         base_241129.generate_days
    left join cte_delete using ( start_ts )
    left join cte_insert using ( start_ts )
    left join cte_scan   using ( start_ts );


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_all_by_day
as
  select
    start_ts::date,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_all_by_day
  order by
    start_ts;



create or replace view
  lf_241129.steps_stats_durations_on_both_for_users_by_minute
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4               as user_id,
    trim( pg_user.usename )::varchar     as user_name,
    base_241129.generate_minutes.start_ts::timestamp as start_ts,
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_minutes
    cross join pg_user
     left join cte_aggr_stats         using ( start_ts, node_type, usesysid )
     left join cte_bcast_stats        using ( start_ts, node_type, usesysid )
     left join cte_delete_stats       using ( start_ts, node_type, usesysid )
     left join cte_dist_stats         using ( start_ts, node_type, usesysid )
     left join cte_hash_stats         using ( start_ts, node_type, usesysid )
     left join cte_hashjoin_stats     using ( start_ts, node_type, usesysid )
     left join cte_insert_stats       using ( start_ts, node_type, usesysid )
     left join cte_limit_stats        using ( start_ts, node_type, usesysid )
     left join cte_merge_stats        using ( start_ts, node_type, usesysid )
     left join cte_mergejoin_stats    using ( start_ts, node_type, usesysid )
     left join cte_nestloop_stats     using ( start_ts, node_type, usesysid )
     left join cte_parse_stats        using ( start_ts, node_type, usesysid )
     left join cte_project_stats      using ( start_ts, node_type, usesysid )
     left join cte_return_stats       using ( start_ts, node_type, usesysid )
     left join cte_save_stats         using ( start_ts, node_type, usesysid )
     left join cte_sort_stats         using ( start_ts, node_type, usesysid )
     left join cte_spatial_join_stats using ( start_ts, node_type, usesysid )
     left join cte_unique_stats       using ( start_ts, node_type, usesysid )
     left join cte_window_stats       using ( start_ts, node_type, usesysid )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_bcast_duration is not null
    or max_bcast_duration is not null
    or med_bcast_duration is not null
    or avg_bcast_duration is not null
    or std_bcast_duration is not null
    or number_bcast_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_dist_duration is not null
    or max_dist_duration is not null
    or med_dist_duration is not null
    or avg_dist_duration is not null
    or std_dist_duration is not null
    or number_dist_steps is not null
    or min_hash_duration is not null
    or max_hash_duration is not null
    or med_hash_duration is not null
    or avg_hash_duration is not null
    or std_hash_duration is not null
    or number_hash_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or min_limit_duration is not null
    or max_limit_duration is not null
    or med_limit_duration is not null
    or avg_limit_duration is not null
    or std_limit_duration is not null
    or number_limit_steps is not null
    or min_merge_duration is not null
    or max_merge_duration is not null
    or med_merge_duration is not null
    or avg_merge_duration is not null
    or std_merge_duration is not null
    or number_merge_steps is not null
    or min_mergejoin_duration is not null
    or max_mergejoin_duration is not null
    or med_mergejoin_duration is not null
    or avg_mergejoin_duration is not null
    or std_mergejoin_duration is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or min_parse_duration is not null
    or max_parse_duration is not null
    or med_parse_duration is not null
    or avg_parse_duration is not null
    or std_parse_duration is not null
    or number_parse_steps is not null
    or min_project_duration is not null
    or max_project_duration is not null
    or med_project_duration is not null
    or avg_project_duration is not null
    or std_project_duration is not null
    or number_project_steps is not null
    or min_return_duration is not null
    or max_return_duration is not null
    or med_return_duration is not null
    or avg_return_duration is not null
    or std_return_duration is not null
    or number_return_steps is not null
    or min_save_duration is not null
    or max_save_duration is not null
    or med_save_duration is not null
    or avg_save_duration is not null
    or std_save_duration is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null
    or min_unique_duration is not null
    or max_unique_duration is not null
    or med_unique_duration is not null
    or avg_unique_duration is not null
    or std_unique_duration is not null
    or number_unique_steps is not null
    or min_window_duration is not null
    or max_window_duration is not null
    or med_window_duration is not null
    or avg_window_duration is not null
    or std_window_duration is not null
    or number_window_steps is not null;


create or replace view
  lf_241129.vacuum_totals_for_tables_by_all
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        table_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        table_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        table_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        table_id,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        table_id
    ),

    cte_number_vacuums as
    (
      select
        table_id,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        table_id
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    cte_number_vacuums.table_id::int4,
    trim( pg_class.relname )::varchar     as table_name,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration                using ( table_id )
    left join pg_class     on pg_class.oid          = cte_number_vacuums.table_id
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  sf_241129.vacuum_totals_for_tables_by_all
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_tables_by_all
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name;


create or replace view
  base_241129.ttables_basic_info_fast
as
  with
    cte_tables as
    (
      select
        pg_class_info.relnamespace                                            as schema_id,
        trim( pg_namespace.nspname )                                          as schema_name,
        pg_class_info.reloid                                                  as table_id,
        trim( pg_class_info.relname )                                         as table_name,
        pg_class_info.relowner                                                as user_id,
        pg_class_info.relcreationtime                                         as table_creation,
        current_timestamp at time zone 'utc' - pg_class_info.relcreationtime  as table_age,
        case
          when pg_class_info.reldiststyle = 0                                              then 'even'
          when pg_class_info.reldiststyle = 1                                              then 'key'
          when pg_class_info.reldiststyle = 8                                              then 'all'
          when pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10 then 'all'
          when pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 11 then 'even'
          when pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 12 then 'key'
                                                                                           else 'none'
        end                                                                   as table_distribution_type,
        case
          when pg_class_info.reldiststyle  = 9 and pg_class_info.releffectivediststyle in (10, 11, 12) then True
          when pg_class_info.reldiststyle != 9                                                         then False
        end                                                                   as table_distribution_auto_flag
      from
             pg_class_info
        join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
      where
        pg_class_info.relkind = 'r'
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    user_id::int4,
    table_creation::timestamp,
    table_age::interval,
    table_distribution_type::varchar,
    table_distribution_auto_flag::bool
  from
    cte_tables;


create or replace view
  base_241129.ttables_basic_info_full
as
  with
    cte_columns as
    (
      -- MG2 : using pg_attribute only this means we get views and tables, but it's substantially faster than joining to pg_class
      --       and we do join to pg_class in cte_tables, which restricts our output to tables

      select
        pg_attribute.attrelid                                              as table_id,
        case
          when ( min(pg_attribute.attsortkeyord) >= 0 and max(pg_attribute.attsortkeyord) >  0 ) then 'compound'
          when ( min(pg_attribute.attsortkeyord) <  0                                          ) then 'interleaved'
          when ( min(pg_attribute.attsortkeyord)  = 0 and max(pg_attribute.attsortkeyord)  = 0 ) then 'unsorted'
        end                                                                as table_sort_type,
        sum( case when pg_attribute.attsortkeyord = 0 then 0 else 1 end )  as table_number_sortkeys,
        sum( case when pg_attribute.attnum >= 1 then 1 else 0 end )        as table_number_columns
      from
        pg_attribute
      group by
        pg_attribute.attrelid
    ),

    cte_tables as
    (
      select
        schema_id,
        schema_name,
        table_id,
        table_name,
        user_id,
        table_creation,
        table_age,
        table_distribution_type,
        table_distribution_auto_flag,
        cte_columns.table_number_columns,
        cte_columns.table_sort_type,
        cte_columns.table_number_sortkeys
      from
             base_241129.ttables_basic_info_fast
        join cte_columns using ( table_id )
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    user_id::int4,
    table_creation::timestamp,
    table_age::interval,
    table_number_columns::int2,
    table_sort_type::varchar,
    table_number_sortkeys::int2,
    table_distribution_type::varchar,
    table_distribution_auto_flag::bool
  from
    cte_tables;


create or replace view
  lf_241129.ttables
as
  with
    cte_table_stats as
    (
      select
        pg_class.oid as table_id,
        case
          when pg_statistic_indicator.stairelid is null                                                                                             then 'missing'
          when pg_class.reltuples  = 0.001::float4                                                                                                  then 'missing'
          when pg_class.reltuples != pg_statistic_indicator.stairows or  pg_statistic_indicator.staiins > 0 or  pg_statistic_indicator.staidels > 0 then 'stale'
          when pg_class.reltuples  = pg_statistic_indicator.stairows and pg_statistic_indicator.staiins = 0 and pg_statistic_indicator.staidels = 0 then 'good'
                                                                                                                                                    else 'unknown'
        end          as table_stats
      from
                  pg_class
        left join pg_statistic_indicator on pg_class.oid = pg_statistic_indicator.stairelid
    )

  select
    base_241129.ttables_basic_info_full.schema_id::int4,
    base_241129.ttables_basic_info_full.schema_name::varchar,
    base_241129.ttables_basic_info_full.table_id::int4,
    base_241129.ttables_basic_info_full.table_name::varchar,
    base_241129.ttables_basic_info_full.user_id::int4,
    user_name::varchar,
    base_241129.ttables_basic_info_full.table_creation::timestamp,
    base_241129.ttables_basic_info_full.table_age::interval,
    base_241129.ttables_basic_info_full.table_number_columns::int2,
    base_241129.ttables_basic_info_full.table_sort_type::varchar,
    base_241129.ttables_basic_info_full.table_number_sortkeys::int2,
    base_241129.ttables_basic_info_full.table_distribution_type::varchar,
    base_241129.ttables_basic_info_full.table_distribution_auto_flag::bool,
    cte_table_stats.table_stats::varchar
  from
                 base_241129.ttables_basic_info_full
            join base_241129.fixed_pg_user using ( user_id )
    natural join cte_table_stats;


create or replace view
  base_241129.ttables_blocks_rows
as
  with
    cte_table_sort_type as
    (
      select
        pg_class.oid as table_id,
        case
          when ( min(pg_attribute.attsortkeyord) >= 0 and max(pg_attribute.attsortkeyord) >  0 ) then 'compound'
          when ( min(pg_attribute.attsortkeyord) <  0                                          ) then 'interleaved'
          when ( min(pg_attribute.attsortkeyord)  = 0 and max(pg_attribute.attsortkeyord)  = 0 ) then 'unsorted'
        end          as table_sort_type
      from
             pg_class
        join pg_attribute on pg_class.oid = pg_attribute.attrelid
      where
        pg_class.relkind = 'r'
      group by
        pg_class.oid
    ),

    cte_blocks_and_rows_s1 as
    (
      select
        stv_blocklist.tbl                                                                                              as table_id,
        count( distinct slice )                                                                                        as number_slices,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      group by
        stv_blocklist.tbl
    ),

    cte_nodes as
    (
      -- MG2 : needed to get row counts correct for `all` distribution

      select
        count( distinct node ) as number_nodes
      from
        stv_slices
    ),

    cte_blocks_and_rows_s2 as
    (
      select
        pg_class_info.relnamespace           as schema_id,
        cte_blocks_and_rows_s1.table_id      as table_id,
        cte_blocks_and_rows_s1.number_slices as number_slices,

        sum( number_sorted_blocks                                                                               ) as number_sorted_blocks,
        sum( case when table_sort_type = 'unsorted' then 0                      else number_unsorted_blocks end ) as number_desorted_blocks,
        sum( case when table_sort_type = 'unsorted' then number_unsorted_blocks else 0                      end ) as number_unsorted_blocks,
        sum( number_total_blocks                                                                                ) as number_total_blocks,

        sum( case when (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_sorted_rows   / cte_nodes.number_nodes else cte_blocks_and_rows_s1.number_sorted_rows   end ) as number_sorted_rows,
        sum
        (
          case
            when cte_table_sort_type.table_sort_type  = 'unsorted'                                                                                                                       then 0
            when cte_table_sort_type.table_sort_type != 'unsorted' and (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                                                                                         else cte_blocks_and_rows_s1.number_unsorted_rows
          end
        ) as number_desorted_rows,
        sum
        (
          case
            when cte_table_sort_type.table_sort_type != 'unsorted'                                                                                                                       then 0
            when cte_table_sort_type.table_sort_type  = 'unsorted' and (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                                                                                         else cte_blocks_and_rows_s1.number_unsorted_rows
          end
        ) as number_unsorted_rows,
        sum( case when (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_total_rows    / cte_nodes.number_nodes else cte_blocks_and_rows_s1.number_total_rows    end ) as number_total_rows
      from
                   cte_blocks_and_rows_s1
              join pg_class_info on pg_class_info.reloid = cte_blocks_and_rows_s1.table_id
        cross join cte_nodes
              join cte_table_sort_type using ( table_id )
      where
        pg_class_info.relkind = 'r'
      group by
        pg_class_info.relnamespace,
        cte_blocks_and_rows_s1.table_id,
        cte_blocks_and_rows_s1.number_slices
    )

  select
    schema_id::int4,
    table_id::int4,
    number_slices::int2,
    number_sorted_blocks::int8,
    number_desorted_blocks::int8,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    number_desorted_rows::int8,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
    cte_blocks_and_rows_s2;


create or replace view
  lf_241129.ttables_blocks_rows
as
  select
    lf_241129.ttables.schema_id::int4,
    lf_241129.ttables.schema_name::varchar,
    lf_241129.ttables.table_id::int4,
    lf_241129.ttables.table_name::varchar,
    lf_241129.ttables.user_id::int4,
    lf_241129.ttables.user_name::varchar,
    lf_241129.ttables.table_creation::timestamp,
    lf_241129.ttables.table_age::interval,
    lf_241129.ttables.table_number_columns::int2,
    lf_241129.ttables.table_sort_type::varchar,
    lf_241129.ttables.table_number_sortkeys::int2,
    lf_241129.ttables.table_distribution_type::varchar,
    lf_241129.ttables.table_distribution_auto_flag::bool,
    base_241129.ttables_blocks_rows.number_slices::int2,
    base_241129.ttables_blocks_rows.number_sorted_blocks::int8,
    base_241129.ttables_blocks_rows.number_desorted_blocks::int8,
    base_241129.ttables_blocks_rows.number_unsorted_blocks::int8,
    base_241129.ttables_blocks_rows.number_total_blocks::int8,
    base_241129.ttables_blocks_rows.number_sorted_rows::int8,
    base_241129.ttables_blocks_rows.number_desorted_rows::int8,
    base_241129.ttables_blocks_rows.number_unsorted_rows::int8,
    base_241129.ttables_blocks_rows.number_total_rows::int8
  from
              lf_241129.ttables
    left join base_241129.ttables_blocks_rows using ( table_id );


create or replace view
  base_241129.fixed_stl_wlm_rule_action
as
  select
    trim( action )::varchar        as action,
    case
      when action = 'change_query_priority' then 'priority'
                                            else trim( action )
    end::varchar                   as rule_action,
    query::int4                    as query_id,
    recordtime::timestamp          as event_ts,
    trim( rule )::varchar          as rule_name,
    service_class::int2            as queue_id,
    -- service_class_name::char(64)
    userid::int4                   as user_id
  from
    stl_wlm_rule_action
  where
        event_ts >= current_date - interval '6 day'
    and event_ts <= current_timestamp;


create or replace view
  lf_241129.wqueues_qmr_log
as
  select
    event_ts::timestamp,
    rule_name::varchar,
    rule_action::varchar,
    queue_id::int2,
    queue_name::varchar,
    user_id::int4,
    user_name::varchar,
    query_id::int4
  from
         base_241129.fixed_stl_wlm_rule_action
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
    join base_241129.fixed_pg_user                      using ( user_id );


create or replace view
  lf_241129.vacuum_totals_for_all_by_minute
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        date_trunc( 'minute', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        date_trunc( 'minute', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        start_ts
    )

  select
    start_ts::timestamp,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( start_ts );


create or replace view
  sf_241129.vacuum_totals_for_all_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_all_by_minute
    )

  select
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.ttables_stats_dists_sorts_for_all_by_blocks
as
  select
    -- MG2 : all

    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then number_sorted_blocks else 0 end )::int8  as all_compound_sorted,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then number_sorted_blocks else 0 end )::int8  as all_interleaved_sorted,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then number_sorted_blocks else 0 end )::int8  as all_unsorted_sorted,

    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then number_unsorted_blocks else 0 end )::int8  as all_compound_unsorted,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then number_unsorted_blocks else 0 end )::int8  as all_interleaved_unsorted,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then number_unsorted_blocks else 0 end )::int8  as all_unsorted_unsorted,

    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then number_total_blocks else 0 end )::int8  as all_compound_total,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then number_total_blocks else 0 end )::int8  as all_interleaved_total,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then number_total_blocks else 0 end )::int8  as all_unsorted_total,

    -- MG2 : even

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then number_sorted_blocks else 0 end )::int8  as even_compound_sorted,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then number_sorted_blocks else 0 end )::int8  as even_interleaved_sorted,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then number_sorted_blocks else 0 end )::int8  as even_unsorted_sorted,

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then number_unsorted_blocks else 0 end )::int8  as even_compound_unsorted,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then number_unsorted_blocks else 0 end )::int8  as even_interleaved_unsorted,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then number_unsorted_blocks else 0 end )::int8  as even_unsorted_unsorted,

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then number_total_blocks else 0 end )::int8  as even_compound_total,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then number_total_blocks else 0 end )::int8  as even_interleaved_total,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then number_total_blocks else 0 end )::int8  as even_unsorted_total,

    -- MG2 : key

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then number_sorted_blocks else 0 end )::int8  as key_compound_sorted,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then number_sorted_blocks else 0 end )::int8  as key_interleaved_sorted,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then number_sorted_blocks else 0 end )::int8  as key_unsorted_sorted,

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then number_unsorted_blocks else 0 end )::int8  as key_compound_unsorted,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then number_unsorted_blocks else 0 end )::int8  as key_interleaved_unsorted,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then number_unsorted_blocks else 0 end )::int8  as key_unsorted_unsorted,

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then number_total_blocks else 0 end )::int8  as key_compound_total,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then number_total_blocks else 0 end )::int8  as key_interleaved_total,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then number_total_blocks else 0 end )::int8  as key_unsorted_total
  from
         base_241129.ttables_basic_info_full
    join base_241129.ttables_blocks_rows using ( table_id )
  where
        user_id                 != 1
    and schema_name         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(schema_name,8)     != 'pg_temp_';


create or replace view
  sf_241129.ttables_stats_dists_sorts_for_all_by_blocks
as
  select
    ( base_241129.hr_blks(all_compound_sorted)    || '/' || base_241129.hr_blks(all_compound_unsorted)    || '/' || base_241129.hr_blks(all_compound_total)    || ' | ' ||
      base_241129.hr_blks(all_interleaved_sorted) || '/' || base_241129.hr_blks(all_interleaved_unsorted) || '/' || base_241129.hr_blks(all_interleaved_total) || ' | ' ||
      base_241129.hr_blks(all_unsorted_sorted)    || '/' || base_241129.hr_blks(all_unsorted_unsorted)    || '/' || base_241129.hr_blks(all_unsorted_total) )::varchar as aall,

    ( base_241129.hr_blks(even_compound_sorted)    || '/' || base_241129.hr_blks(even_compound_unsorted)    || '/' || base_241129.hr_blks(even_compound_total)    || ' | ' ||
      base_241129.hr_blks(even_interleaved_sorted) || '/' || base_241129.hr_blks(even_interleaved_unsorted) || '/' || base_241129.hr_blks(even_interleaved_total) || ' | ' ||
      base_241129.hr_blks(even_unsorted_sorted)    || '/' || base_241129.hr_blks(even_unsorted_unsorted)    || '/' || base_241129.hr_blks(even_unsorted_total) )::varchar as even,

    ( base_241129.hr_blks(key_compound_sorted)    || '/' || base_241129.hr_blks(key_compound_unsorted)    || '/' || base_241129.hr_blks(key_compound_total)    || ' | ' ||
      base_241129.hr_blks(key_interleaved_sorted) || '/' || base_241129.hr_blks(key_interleaved_unsorted) || '/' || base_241129.hr_blks(key_interleaved_total) || ' | ' ||
      base_241129.hr_blks(key_unsorted_sorted)    || '/' || base_241129.hr_blks(key_unsorted_unsorted)    || '/' || base_241129.hr_blks(key_unsorted_total) )::varchar as kkey
  from
    lf_241129.ttables_stats_dists_sorts_for_all_by_blocks;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_query
as
  with
    cte_delete as
    (
      select
        query       as query_id,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        query_id
    ),

    cte_insert as
    (
      select
        query       as query_id,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        query_id
    ),

    cte_scan as
    (
      select
        query       as query_id,
        sum( rows ) as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        query_id
    ),

    cte_nulls_to_values as
    (
      select
        query_id,
        case when sum_delete_rows is null then 0 else sum_delete_rows end as notnulled_sum_delete_rows,
        case when sum_insert_rows is null then 0 else sum_insert_rows end as notnulled_sum_insert_rows,
        case when sum_scan_rows   is null then 0 else sum_scan_rows   end as notnulled_sum_scan_rows
      from
                  base_241129.fixed_stl_wlm_query
        left join cte_delete using ( query_id )
        left join cte_insert using ( query_id )
        left join cte_scan   using ( query_id )
    )

  select
    query_id::int4,
    notnulled_sum_delete_rows::int8,
    notnulled_sum_insert_rows::int8,
    notnulled_sum_scan_rows::int8
  from
    cte_nulls_to_values;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_slices_by_hour
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        start_ts,
        slice_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        start_ts,
        slice_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        start_ts,
        slice_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        start_ts,
        slice_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        start_ts,
        slice_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        start_ts,
        slice_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        start_ts,
        slice_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        start_ts,
        slice_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        start_ts,
        slice_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        start_ts,
        slice_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        start_ts,
        slice_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        start_ts,
        slice_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        start_ts,
        slice_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        start_ts,
        slice_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        start_ts,
        slice_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        start_ts,
        slice_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'hour', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        slice_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        start_ts,
        slice_ordinal
    )

  select
    base_241129.generate_hours.start_ts::timestamp as start_ts,

    base_241129.fixed_stv_slices.node_ordinal::int2,
    base_241129.fixed_stv_slices.slice_ordinal::int2,
    base_241129.fixed_stv_slices.slice_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_hours
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, slice_ordinal )
     left join cte_bcast_stats            using ( start_ts, slice_ordinal )
     left join cte_delete_stats           using ( start_ts, slice_ordinal )
     left join cte_dist_stats             using ( start_ts, slice_ordinal )
     left join cte_hash_stats             using ( start_ts, slice_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, slice_ordinal )
     left join cte_insert_stats           using ( start_ts, slice_ordinal )
     left join cte_limit_stats            using ( start_ts, slice_ordinal )
     left join cte_merge_stats            using ( start_ts, slice_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, slice_ordinal )
     left join cte_nestloop_stats         using ( start_ts, slice_ordinal )
     left join cte_parse_stats            using ( start_ts, slice_ordinal )
     left join cte_project_stats          using ( start_ts, slice_ordinal )
     left join cte_return_stats           using ( start_ts, slice_ordinal )
     left join cte_save_stats             using ( start_ts, slice_ordinal )
     left join cte_sort_stats             using ( start_ts, slice_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, slice_ordinal )
     left join cte_unique_stats           using ( start_ts, slice_ordinal )
     left join cte_window_stats           using ( start_ts, slice_ordinal );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_hour_complete
as
  select
    start_ts::timestamp,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_hour
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_second_concise
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_second
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_all_by_minute
as
  with
    cte_queries as
    (
      select
        date_trunc( 'minute', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        date_trunc( 'minute', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    base_241129.generate_minutes.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_minutes
    left join cte_queries using ( start_ts )
  group by
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_all_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_all_by_minute
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_second_complete
as
  select
    start_ts::timestamp,
    node_ordinal::int2 as n,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_second
  order by
    start_ts,
    node_ordinal;


create or replace view
  base_241129.privileges_groups
as
  with
    cte_pg_namespace as
    (
      select
        schema_id,
        schema_name,
        'schema'          as object_type,
        null::oid         as object_return_base_data_type_id,
        null::varchar     as object_return_base_data_type_name,
        schema_id         as object_id,
        schema_name       as object_name,
        null::varchar     as object_argument_base_data_type_ids,
        null::varchar     as object_argument_base_data_type_names,
        schema_owner_id   as object_owner_user_id,
        schema_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 7, strpos(acl_item,'=')-7 ) as acl_recipient_group_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            schema_owner_id,
            schema_owner_name,
            schema_acl,
            schema_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when schema_acl is     null then array_to_string( schema_default_acl[acl_index:acl_index], ',' )
              when schema_acl is not null then array_to_string( schema_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid                                       as schema_id,
                pg_namespace.nspname                                   as schema_name,
                pg_namespace.nspowner                                  as schema_owner_id,
                pg_user.usename                                        as schema_owner_name,
                pg_namespace.nspacl                                    as schema_acl,
                -- MG2 : I would be alphabetic, but Postgres/Redshift uses 'UC'
                array[ pg_user.usename || '=UC/' || pg_user.usename ]  as schema_default_acl,
                generate_series( 1,
                                 case
                                   when schema_acl is     null then array_upper( schema_default_acl, 1 )
                                   when schema_acl is not null then array_upper( schema_acl, 1 )
                                 end ) as acl_index
              from
                     pg_namespace
                join pg_user on pg_namespace.nspowner = pg_user.usesysid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) = 'group'
        ) as subquery_3
    ),

    cte_pg_class as
    (
      select
        schema_id,
        schema_name,
        case
          when class_type = 'r' then 'table'
          when class_type = 'v' then 'view'
        end              as object_type,
        null::oid        as object_return_base_data_type_id,
        null::varchar    as object_return_base_data_type_name,
        class_id         as object_id,
        class_name       as object_name,
        null::varchar    as object_argument_base_data_type_ids,
        null::varchar    as object_argument_base_data_type_names,
        class_owner_id   as object_owner_user_id,
        class_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 7, strpos(acl_item,'=')-7 ) as acl_recipient_group_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            class_id,
            class_type,
            class_name,
            class_owner_id,
            class_owner_name,
            class_acl,
            class_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when class_acl is     null then array_to_string( class_default_acl[acl_index:acl_index], ',' )
              when class_acl is not null then array_to_string( class_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_class.oid         as class_id,
                pg_class.relkind     as class_type,
                pg_class.relname     as class_name,
                pg_class.relowner    as class_owner_id,
                pg_user.usename      as class_owner_name,
                pg_class.relacl      as class_acl,
                  -- MG2 : 'arwdRxtD' is the ACL string Redshift procuces; I could curtail it to only those which exist on Redshift, but matching RS makes testing easier
                array[ pg_user.usename || '=arwdRxtD/' || pg_user.usename ] as class_default_acl,
                generate_series( 1,
                                 case
                                   when class_acl is     null then array_upper( class_default_acl, 1 )
                                   when class_acl is not null then array_upper( class_acl, 1 )
                                 end ) as acl_index
              from
                     pg_class
                join pg_namespace on pg_class.relnamespace = pg_namespace.oid
                join pg_user      on pg_class.relowner     = pg_user.usesysid
              where
                pg_class.relkind in ( 'r', 'v' )
            ) as subquery_2
          where
            substring(acl_item, 1, 5) = 'group'
        ) as subquery_3
    ),

    cte_pg_proc as
    (
      select
        schema_id,
        schema_name,
        case
          when proctype = 'c'         then 'C func'
          when proctype = 'exfunc'    then 'lambda func'
          when proctype = 'internal'  then 'internal func'
          when proctype = 'mlfunc'    then 'ML func'
          when proctype = 'plpgsql'   then 'procedure'
          when proctype = 'plpythonu' then 'python func'
          when proctype = 'sql'       then 'SQL func'
                                       else trim( proctype )
        end                            as object_type,
        procreturn_data_type_id       as object_return_base_data_type_id,
        procreturn_data_type_name     as object_return_base_data_type_name,
        procid                        as object_id,
        procname                      as object_name,
        procargument_data_type_ids    as object_argument_base_data_type_ids,
        procargument_data_type_names  as object_argument_base_data_type_names,
        procowner_id                  as object_owner_user_id,
        procowner_name                as object_owner_user_name,
        acl_item,
        substring( acl_item, 7, strpos(acl_item,'=')-7 ) as acl_recipient_group_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            procid,
            proctype,
            procreturn_data_type_id,
            procreturn_data_type_name,
            procname,
            procargument_data_type_ids,
            procargument_data_type_names,
            procowner_id,
            procowner_name,
            procacl,
            procdefault_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when procacl is     null then array_to_string( procdefault_acl[acl_index:acl_index], ',' )
              when procacl is not null then array_to_string( procacl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_proc.oid          as procid,
                pg_language.lanname  as proctype,
                pg_proc.prorettype   as procreturn_data_type_id,
                case
                  when pg_proc.proretset = True  then '_' || pg_type_return.typname
                  when pg_proc.proretset = False then        pg_type_return.typname
                end                  as procreturn_data_type_name,
                pg_proc.proname      as procname,

                                                                         pg_type_arg_00.oid::varchar     ||
                case when pg_type_arg_01.oid is null then '' else ',' || pg_type_arg_01.oid::varchar end ||
                case when pg_type_arg_02.oid is null then '' else ',' || pg_type_arg_02.oid::varchar end ||
                case when pg_type_arg_03.oid is null then '' else ',' || pg_type_arg_03.oid::varchar end ||
                case when pg_type_arg_04.oid is null then '' else ',' || pg_type_arg_04.oid::varchar end ||
                case when pg_type_arg_05.oid is null then '' else ',' || pg_type_arg_05.oid::varchar end ||
                case when pg_type_arg_06.oid is null then '' else ',' || pg_type_arg_06.oid::varchar end ||
                case when pg_type_arg_07.oid is null then '' else ',' || pg_type_arg_07.oid::varchar end ||
                case when pg_type_arg_08.oid is null then '' else ',' || pg_type_arg_08.oid::varchar end ||
                case when pg_type_arg_09.oid is null then '' else ',' || pg_type_arg_09.oid::varchar end ||
                case when pg_type_arg_10.oid is null then '' else ',' || pg_type_arg_10.oid::varchar end ||
                case when pg_type_arg_11.oid is null then '' else ',' || pg_type_arg_11.oid::varchar end ||
                case when pg_type_arg_12.oid is null then '' else ',' || pg_type_arg_12.oid::varchar end ||
                case when pg_type_arg_13.oid is null then '' else ',' || pg_type_arg_13.oid::varchar end ||
                case when pg_type_arg_14.oid is null then '' else ',' || pg_type_arg_14.oid::varchar end ||
                case when pg_type_arg_15.oid is null then '' else ',' || pg_type_arg_15.oid::varchar end ||
                case when pg_type_arg_16.oid is null then '' else ',' || pg_type_arg_16.oid::varchar end ||
                case when pg_type_arg_17.oid is null then '' else ',' || pg_type_arg_17.oid::varchar end ||
                case when pg_type_arg_18.oid is null then '' else ',' || pg_type_arg_18.oid::varchar end ||
                case when pg_type_arg_19.oid is null then '' else ',' || pg_type_arg_19.oid::varchar end ||
                case when pg_type_arg_20.oid is null then '' else ',' || pg_type_arg_20.oid::varchar end ||
                case when pg_type_arg_21.oid is null then '' else ',' || pg_type_arg_21.oid::varchar end ||
                case when pg_type_arg_22.oid is null then '' else ',' || pg_type_arg_22.oid::varchar end ||
                case when pg_type_arg_23.oid is null then '' else ',' || pg_type_arg_23.oid::varchar end ||
                case when pg_type_arg_24.oid is null then '' else ',' || pg_type_arg_24.oid::varchar end ||
                case when pg_type_arg_25.oid is null then '' else ',' || pg_type_arg_25.oid::varchar end ||
                case when pg_type_arg_26.oid is null then '' else ',' || pg_type_arg_26.oid::varchar end ||
                case when pg_type_arg_27.oid is null then '' else ',' || pg_type_arg_27.oid::varchar end ||
                case when pg_type_arg_28.oid is null then '' else ',' || pg_type_arg_28.oid::varchar end ||
                case when pg_type_arg_29.oid is null then '' else ',' || pg_type_arg_29.oid::varchar end ||
                case when pg_type_arg_30.oid is null then '' else ',' || pg_type_arg_30.oid::varchar end ||
                case when pg_type_arg_31.oid is null then '' else ',' || pg_type_arg_31.oid::varchar end     as procargument_data_type_ids,

                                                                             pg_type_arg_00.typname     ||
                case when pg_type_arg_01.typname is null then '' else ',' || pg_type_arg_01.typname end ||
                case when pg_type_arg_02.typname is null then '' else ',' || pg_type_arg_02.typname end ||
                case when pg_type_arg_03.typname is null then '' else ',' || pg_type_arg_03.typname end ||
                case when pg_type_arg_04.typname is null then '' else ',' || pg_type_arg_04.typname end ||
                case when pg_type_arg_05.typname is null then '' else ',' || pg_type_arg_05.typname end ||
                case when pg_type_arg_06.typname is null then '' else ',' || pg_type_arg_06.typname end ||
                case when pg_type_arg_07.typname is null then '' else ',' || pg_type_arg_07.typname end ||
                case when pg_type_arg_08.typname is null then '' else ',' || pg_type_arg_08.typname end ||
                case when pg_type_arg_09.typname is null then '' else ',' || pg_type_arg_09.typname end ||
                case when pg_type_arg_10.typname is null then '' else ',' || pg_type_arg_10.typname end ||
                case when pg_type_arg_11.typname is null then '' else ',' || pg_type_arg_11.typname end ||
                case when pg_type_arg_12.typname is null then '' else ',' || pg_type_arg_12.typname end ||
                case when pg_type_arg_13.typname is null then '' else ',' || pg_type_arg_13.typname end ||
                case when pg_type_arg_14.typname is null then '' else ',' || pg_type_arg_14.typname end ||
                case when pg_type_arg_15.typname is null then '' else ',' || pg_type_arg_15.typname end ||
                case when pg_type_arg_16.typname is null then '' else ',' || pg_type_arg_16.typname end ||
                case when pg_type_arg_17.typname is null then '' else ',' || pg_type_arg_17.typname end ||
                case when pg_type_arg_18.typname is null then '' else ',' || pg_type_arg_18.typname end ||
                case when pg_type_arg_19.typname is null then '' else ',' || pg_type_arg_19.typname end ||
                case when pg_type_arg_20.typname is null then '' else ',' || pg_type_arg_20.typname end ||
                case when pg_type_arg_21.typname is null then '' else ',' || pg_type_arg_21.typname end ||
                case when pg_type_arg_22.typname is null then '' else ',' || pg_type_arg_22.typname end ||
                case when pg_type_arg_23.typname is null then '' else ',' || pg_type_arg_23.typname end ||
                case when pg_type_arg_24.typname is null then '' else ',' || pg_type_arg_24.typname end ||
                case when pg_type_arg_25.typname is null then '' else ',' || pg_type_arg_25.typname end ||
                case when pg_type_arg_26.typname is null then '' else ',' || pg_type_arg_26.typname end ||
                case when pg_type_arg_27.typname is null then '' else ',' || pg_type_arg_27.typname end ||
                case when pg_type_arg_28.typname is null then '' else ',' || pg_type_arg_28.typname end ||
                case when pg_type_arg_29.typname is null then '' else ',' || pg_type_arg_29.typname end ||
                case when pg_type_arg_30.typname is null then '' else ',' || pg_type_arg_30.typname end ||
                case when pg_type_arg_31.typname is null then '' else ',' || pg_type_arg_31.typname end     as procargument_data_type_names,

                pg_proc.proowner as procowner_id,
                pg_user.usename  as procowner_name,
                pg_proc.proacl   as procacl,
                case
                  when pg_language.lanname = 'c'         then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  -- MG2 : internal are all {=X/rdsdb}, and we can't make more languages (no "create language" in Redshift)
                  when pg_language.lanname = 'internal'  then array[ '=X/rdsdb' ]
                  when pg_language.lanname = 'plpgsql'   then array[ pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'plpythonu' then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'sql'       then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                end              as procdefault_acl,
                generate_series( 1,
                                 case
                                   when procacl is     null then array_upper( procdefault_acl, 1 )
                                   when procacl is not null then array_upper( procacl, 1 )
                                 end ) as acl_index
              from
                          pg_proc
                     join pg_language               on pg_proc.prolang          = pg_language.oid
                     join pg_namespace              on pg_proc.pronamespace     = pg_namespace.oid
                     join pg_user                   on pg_proc.proowner         = pg_user.usesysid
                -- MG2 : procedures do not have a return type
                left join pg_type as pg_type_return on pg_proc.prorettype       = pg_type_return.oid
                -- MG2 : pg_proc.proargtypes is an oidvector, which is an array of 32 oids, when an element is empty, the value is 0
                left join pg_type as pg_type_arg_00 on pg_proc.proargtypes[00] != 0 and pg_proc.proargtypes[00] = pg_type_arg_00.oid
                left join pg_type as pg_type_arg_01 on pg_proc.proargtypes[01] != 0 and pg_proc.proargtypes[01] = pg_type_arg_01.oid
                left join pg_type as pg_type_arg_02 on pg_proc.proargtypes[02] != 0 and pg_proc.proargtypes[02] = pg_type_arg_02.oid
                left join pg_type as pg_type_arg_03 on pg_proc.proargtypes[03] != 0 and pg_proc.proargtypes[03] = pg_type_arg_03.oid
                left join pg_type as pg_type_arg_04 on pg_proc.proargtypes[04] != 0 and pg_proc.proargtypes[04] = pg_type_arg_04.oid
                left join pg_type as pg_type_arg_05 on pg_proc.proargtypes[05] != 0 and pg_proc.proargtypes[05] = pg_type_arg_05.oid
                left join pg_type as pg_type_arg_06 on pg_proc.proargtypes[06] != 0 and pg_proc.proargtypes[06] = pg_type_arg_06.oid
                left join pg_type as pg_type_arg_07 on pg_proc.proargtypes[07] != 0 and pg_proc.proargtypes[07] = pg_type_arg_07.oid
                left join pg_type as pg_type_arg_08 on pg_proc.proargtypes[08] != 0 and pg_proc.proargtypes[08] = pg_type_arg_08.oid
                left join pg_type as pg_type_arg_09 on pg_proc.proargtypes[09] != 0 and pg_proc.proargtypes[09] = pg_type_arg_09.oid
                left join pg_type as pg_type_arg_10 on pg_proc.proargtypes[10] != 0 and pg_proc.proargtypes[10] = pg_type_arg_10.oid
                left join pg_type as pg_type_arg_11 on pg_proc.proargtypes[11] != 0 and pg_proc.proargtypes[11] = pg_type_arg_11.oid
                left join pg_type as pg_type_arg_12 on pg_proc.proargtypes[12] != 0 and pg_proc.proargtypes[12] = pg_type_arg_12.oid
                left join pg_type as pg_type_arg_13 on pg_proc.proargtypes[13] != 0 and pg_proc.proargtypes[13] = pg_type_arg_13.oid
                left join pg_type as pg_type_arg_14 on pg_proc.proargtypes[14] != 0 and pg_proc.proargtypes[14] = pg_type_arg_14.oid
                left join pg_type as pg_type_arg_15 on pg_proc.proargtypes[15] != 0 and pg_proc.proargtypes[15] = pg_type_arg_15.oid
                left join pg_type as pg_type_arg_16 on pg_proc.proargtypes[16] != 0 and pg_proc.proargtypes[16] = pg_type_arg_16.oid
                left join pg_type as pg_type_arg_17 on pg_proc.proargtypes[17] != 0 and pg_proc.proargtypes[17] = pg_type_arg_17.oid
                left join pg_type as pg_type_arg_18 on pg_proc.proargtypes[18] != 0 and pg_proc.proargtypes[18] = pg_type_arg_18.oid
                left join pg_type as pg_type_arg_19 on pg_proc.proargtypes[19] != 0 and pg_proc.proargtypes[19] = pg_type_arg_19.oid
                left join pg_type as pg_type_arg_20 on pg_proc.proargtypes[20] != 0 and pg_proc.proargtypes[20] = pg_type_arg_20.oid
                left join pg_type as pg_type_arg_21 on pg_proc.proargtypes[21] != 0 and pg_proc.proargtypes[21] = pg_type_arg_21.oid
                left join pg_type as pg_type_arg_22 on pg_proc.proargtypes[22] != 0 and pg_proc.proargtypes[22] = pg_type_arg_22.oid
                left join pg_type as pg_type_arg_23 on pg_proc.proargtypes[23] != 0 and pg_proc.proargtypes[23] = pg_type_arg_23.oid
                left join pg_type as pg_type_arg_24 on pg_proc.proargtypes[24] != 0 and pg_proc.proargtypes[24] = pg_type_arg_24.oid
                left join pg_type as pg_type_arg_25 on pg_proc.proargtypes[25] != 0 and pg_proc.proargtypes[25] = pg_type_arg_25.oid
                left join pg_type as pg_type_arg_26 on pg_proc.proargtypes[26] != 0 and pg_proc.proargtypes[26] = pg_type_arg_26.oid
                left join pg_type as pg_type_arg_27 on pg_proc.proargtypes[27] != 0 and pg_proc.proargtypes[27] = pg_type_arg_27.oid
                left join pg_type as pg_type_arg_28 on pg_proc.proargtypes[28] != 0 and pg_proc.proargtypes[28] = pg_type_arg_28.oid
                left join pg_type as pg_type_arg_29 on pg_proc.proargtypes[29] != 0 and pg_proc.proargtypes[29] = pg_type_arg_29.oid
                left join pg_type as pg_type_arg_30 on pg_proc.proargtypes[30] != 0 and pg_proc.proargtypes[30] = pg_type_arg_30.oid
                left join pg_type as pg_type_arg_31 on pg_proc.proargtypes[31] != 0 and pg_proc.proargtypes[31] = pg_type_arg_31.oid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) = 'group'
        ) as subquery_3
    ),

    cte_pg_language as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'language'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        language_id         as object_id,
        language_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        language_owner_id   as object_owner_user_id,
        language_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 7, strpos(acl_item,'=')-7 ) as acl_recipient_group_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            language_id,
            language_name,
            language_owner_id,
            language_owner_name,
            language_acl,
            language_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when language_acl is     null then array_to_string( language_default_acl[acl_index:acl_index], ',' )
              when language_acl is not null then array_to_string( language_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                language_id,
                language_name,
                language_owner_id,
                language_owner_name,
                language_acl,
                language_default_acl,
                generate_series( 1,
                                 case
                                   when language_acl is     null then array_upper( language_default_acl, 1 )
                                   when language_acl is not null then array_upper( language_acl, 1 )
                                 end ) as acl_index
              from
                (
                  select
                    pg_language.oid     as language_id,
                    pg_language.lanname as language_name,
                    1::oid              as language_owner_id,
                    'rdsdb'             as language_owner_name,
                    pg_language.lanacl  as language_acl,
                    case
                      -- MG2 : no CREATE LANGUAGE in Redshift, so owner is always rdsdb (OID 1)
                      --        usage on language seems brolen and is not checked
                      --        the usual usage grant to group public is not made for plpgsql and plpythonu
                      --        (but this has no affect, since the check for usage is not made)
                      when pg_language.lanname not in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb,=U/rdsdb' ]
                      when pg_language.lanname     in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb' ]
                    end                 as language_default_acl
                  from
                    pg_language
                ) as subquery_1
            ) as subquery_2
          where
            substring(acl_item, 1, 5) = 'group'
        ) as subquery_3
    ),

    cte_pg_database as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'database'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        database_id         as object_id,
        database_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        database_owner_id   as object_owner_user_id,
        database_owner_name as object_owner_user_name,
        acl_item,
        substring( acl_item, 7, strpos(acl_item,'=')-7 ) as acl_recipient_group_name,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            database_id,
            database_name,
            database_owner_id,
            database_owner_name,
            database_acl,
            database_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when database_acl is     null then array_to_string( database_default_acl[acl_index:acl_index], ',' )
              when database_acl is not null then array_to_string( database_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_database.oid     as database_id,
                pg_database.datname as database_name,
                pg_database.datdba  as database_owner_id,
                pg_user.usename     as database_owner_name,
                pg_database.datacl  as database_acl,
                -- MG2 : note we do not reprocuce the array-indicating open and close curly braces
                --       we could, to make the database_acl strings look identical but then they're a pain later on
                array[ '=T/' || pg_user.usename || ',' || pg_user.usename || '=CT/' || pg_user.usename ] as database_default_acl,
                generate_series( 1,
                                 case
                                   when database_acl is     null then array_upper( database_default_acl, 1 )
                                   when database_acl is not null then array_upper( database_acl, 1 )
                                 end ) as acl_index
              from
                     pg_database
                join pg_user on pg_database.datdba = pg_user.usesysid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) = 'group'
        ) as subquery_3
    ),

    cte_all as
    (
      select * from cte_pg_namespace
      union all
      select * from cte_pg_class
      union all
      select * from cte_pg_proc
      union all
      select * from cte_pg_language
      union all
      select * from cte_pg_database
    )

  select
    pg_group.grosysid::int4 as acl_recipient_group_id,
    acl_recipient_group_name::varchar,

    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    acl_privileges::varchar
  from
         cte_all
    join pg_group on pg_group.groname = cte_all.acl_recipient_group_name;


create or replace view
  lf_241129.groups_privileges
as
  select
    acl_recipient_group_id::int4      as group_id,
    acl_recipient_group_name::varchar as group_name,

    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    case
      when object_type = 'database'  and strpos(acl_privileges,'C') != 0 then true else false
    end::bool as priv_database_create,

    case
      when object_type = 'database'  and strpos(acl_privileges,'T') != 0 then true else false
    end::bool as priv_database_temp,

    case
      when object_type = 'function'  and strpos(acl_privileges,'X') != 0 then true else false
    end::bool as priv_function_execute,

    case
      when object_type = 'language'  and strpos(acl_privileges,'U') != 0 then true else false
    end::bool as priv_language_usage,

    case
      when object_type = 'procedure' and strpos(acl_privileges,'X') != 0 then true else false
    end::bool as priv_procedure_execute,

    case
      when object_type = 'schema'    and strpos(acl_privileges,'C') != 0 then true else false
    end::bool as priv_schema_create,

    case
      when object_type = 'schema'    and strpos(acl_privileges,'U') != 0 then true else false
    end::bool as priv_schema_usage,

    case
      when object_type = 'table'     and strpos(acl_privileges,'d') != 0 then true else false
    end::bool as priv_table_delete,

    case
      when object_type = 'table'     and strpos(acl_privileges,'D') != 0 then true else false
    end::bool as priv_table_drop,

    case
      when object_type = 'table'     and strpos(acl_privileges,'a') != 0 then true else false
    end::bool as priv_table_insert,

    case
      when object_type = 'table'     and strpos(acl_privileges,'x') != 0 then true else false
    end::bool as priv_table_reference,

    case
      when object_type = 'table'     and strpos(acl_privileges,'r') != 0 then true else false
    end::bool as priv_table_select,

    case
      when object_type = 'table'     and strpos(acl_privileges,'w') != 0 then true else false
    end::bool as priv_table_update,

    case
      when object_type = 'view'      and strpos(acl_privileges,'D') != 0 then true else false
    end::bool as priv_view_drop,

    case
      when object_type = 'view'      and strpos(acl_privileges,'r') != 0 then true else false
    end::bool as priv_view_select
  from
    base_241129.privileges_groups;


create or replace view
  sf_241129.groups_privileges
as
  select
    group_name::varchar as ggroup,

    object_type::varchar as ttype,
    schema_name::varchar as sschema,
    object_name::varchar as oobject,

    object_return_base_data_type_name::varchar    as rt,
    object_argument_base_data_type_names::varchar as args,

    object_owner_user_name::varchar as oowner,

    (
      case when priv_database_create = true then 'c' else ' ' end
      ||
      case when priv_database_temp   = true then 't' else ' ' end
      ||
      case when priv_language_usage  = true then 'u' else ' ' end
      ||
      '.'
      ||
      case when priv_schema_create   = true then 'c' else ' ' end
      ||
      case when priv_schema_usage    = true then 'u' else ' ' end
      ||
      '.'
      ||
      case when priv_table_delete    = true then 'd' else ' ' end
      ||
      case when priv_table_reference = true then 'f' else ' ' end
      ||
      case when priv_table_insert    = true then 'i' else ' ' end
      ||
      case when priv_table_drop      = true then 'p' else ' ' end
      ||
      case when priv_table_select = true or priv_view_select = true then 's' else ' ' end
      ||
      case when priv_table_update    = true then 'u' else ' ' end
      ||
      case when priv_function_execute = true or priv_procedure_execute = true then 'x' else ' ' end
    )::varchar as privs
  from
    lf_241129.groups_privileges
  order by
    group_name,
    schema_name,
    case when object_type = 'schema' then 0 else 1 end,
    object_type,
    object_name;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_tables_by_all
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl
    ),

    cte_scan as
    (
      select
        tbl         as table_id,
        sum( rows ) as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    cte_delete.sum_delete_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_scan.sum_scan_rows::int8
  from
              base_241129.fixed_pg_class_for_tables
         join base_241129.fixed_pg_namespace using ( schema_id )
    left join cte_delete              using ( table_id )
    left join cte_insert              using ( table_id )
    left join cte_scan                using ( table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_tables_by_all
as
  select
    schema_name::varchar                       as sschema,
    table_name::varchar                        as ttable,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_tables_by_all
  order by
    schema_name,
    table_name;


create or replace view
  lf_241129.analyze_totals_for_users_by_all
as
  with
    cte_s1 as
    (
      select
        userid                         as user_id,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        userid
    )

  select
    user_id::int4,
    user_name::varchar,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
             cte_s1
   left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  base_241129.fixed_stl_querytext
as
  -- MG2 : this table can log queries twice
  --       which means you end up with two identical rows, for every row, for such a query
  --       on current cluster, I see about 24k such queries, out of about 10m queries in total

  select distinct
    userid::int4    as user_id,
    xid::int8       as transaction_id,
    pid::int4       as session_id,
    query::int4     as query_id,
    sequence::int4,
    -- MG2 : is not actually char, can contain UTF-8
    text::varchar(200) as query_text
  from
    stl_querytext;


create or replace view
  lf_241129.queries_text_on_workers_full
as
  with
    cte_query_text_worker_full as
    (
      select
        session_id,
        transaction_id,
        query_id,
        sequence / (3000 / 200)                                                                  as query_text_order,
        replace( listagg(query_text) within group (order by sequence), chr(92) || 'n', chr(10) ) as query_text
      from
        base_241129.fixed_stl_querytext
      group by
        session_id,
        transaction_id,
        query_id,
        sequence / (3000 / 200)
    )

  select
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    query_text_order::int4,
    query_text::varchar
  from
    cte_query_text_worker_full;


create or replace view
  lf_241129.queries_text_on_both_full
as
  with
    -- MG2 : make leader queries look like worker queries so we can union

    cte_query_text_leader as
    (
      select
        session_id,
        transaction_id,
        null::int4 as query_id,
        query_start_ts,
        query_text_order,
        query_text
      from
        lf_241129.queries_text_for_leader
    ),

    -- MG2 : make worker queries look like leader queries so we can union

    cte_query_text_worker_leader as
    (
      select
        session_id,
        transaction_id,
        query_id,
        query_start_ts,
        query_text_order,
        lf_241129.queries_text_on_workers_full.query_text
      from
             lf_241129.queries_text_on_workers_full
        join base_241129.fixed_stl_query using ( session_id, query_id, transaction_id )
    ),

    cte_query_text_all_full as
    (
      select * from cte_query_text_leader
      union all
      select * from cte_query_text_worker_leader
    )

  select
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    query_start_ts::timestamp,
    query_text_order::int4,
    query_text::varchar
  from
    cte_query_text_all_full;


create or replace view
  sf_241129.queries_text_on_both_full
as
  select
    session_id::int4       as pid,
    transaction_id::int8   as xid,
    query_id::int4         as qid,
    query_start_ts::timestamp as start_ts,
    query_text_order::int4 as n,
    trim( chr(10) || ' ' from query_text )::varchar as ttext
  from
    lf_241129.queries_text_on_both_full
  order by
    session_id,
    transaction_id,
    -- MG2 : not query_id, as leader node queries have this set to NULL
    --       we order only by query start time, which we get for both leader and worker
    query_start_ts,
    query_text_order;


create or replace view
  base_241129.privileges_public
as
  with
    cte_pg_namespace as
    (
      select
        schema_id,
        schema_name,
        'schema'          as object_type,
        null::oid         as object_return_base_data_type_id,
        null::varchar     as object_return_base_data_type_name,
        schema_id         as object_id,
        schema_name       as object_name,
        null::varchar     as object_argument_base_data_type_ids,
        null::varchar     as object_argument_base_data_type_names,
        schema_owner_id   as object_owner_user_id,
        schema_owner_name as object_owner_user_name,
        acl_item,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            schema_owner_id,
            schema_owner_name,
            schema_acl,
            schema_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when schema_acl is     null then array_to_string( schema_default_acl[acl_index:acl_index], ',' )
              when schema_acl is not null then array_to_string( schema_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid                                       as schema_id,
                pg_namespace.nspname                                   as schema_name,
                pg_namespace.nspowner                                  as schema_owner_id,
                pg_user.usename                                        as schema_owner_name,
                pg_namespace.nspacl                                    as schema_acl,
                -- MG2 : I would be alphabetic, but Postgres/Redshift uses 'UC'
                array[ pg_user.usename || '=UC/' || pg_user.usename ]  as schema_default_acl,
                generate_series( 1,
                                 case
                                   when schema_acl is     null then array_upper( schema_default_acl, 1 )
                                   when schema_acl is not null then array_upper( schema_acl, 1 )
                                 end ) as acl_index
              from
                     pg_namespace
                join pg_user on pg_namespace.nspowner = pg_user.usesysid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) = '='
        ) as subquery_3
    ),

    cte_pg_class as
    (
      select
        schema_id,
        schema_name,
        case
          when class_type = 'r' then 'table'
          when class_type = 'v' then 'view'
        end              as object_type,
        null::oid        as object_return_base_data_type_id,
        null::varchar    as object_return_base_data_type_name,
        class_id         as object_id,
        class_name       as object_name,
        null::varchar    as object_argument_base_data_type_ids,
        null::varchar    as object_argument_base_data_type_names,
        class_owner_id   as object_owner_user_id,
        class_owner_name as object_owner_user_name,
        acl_item,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            class_id,
            class_type,
            class_name,
            class_owner_id,
            class_owner_name,
            class_acl,
            class_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when class_acl is     null then array_to_string( class_default_acl[acl_index:acl_index], ',' )
              when class_acl is not null then array_to_string( class_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_class.oid         as class_id,
                pg_class.relkind     as class_type,
                pg_class.relname     as class_name,
                pg_class.relowner    as class_owner_id,
                pg_user.usename      as class_owner_name,
                pg_class.relacl      as class_acl,
                  -- MG2 : 'arwdRxtD' is the ACL string Redshift procuces; I could curtail it to only those which exist on Redshift, but matching RS makes testing easier
                array[ pg_user.usename || '=arwdRxtD/' || pg_user.usename ] as class_default_acl,
                generate_series( 1,
                                 case
                                   when class_acl is     null then array_upper( class_default_acl, 1 )
                                   when class_acl is not null then array_upper( class_acl, 1 )
                                 end ) as acl_index
              from
                     pg_class
                join pg_namespace on pg_class.relnamespace = pg_namespace.oid
                join pg_user      on pg_class.relowner     = pg_user.usesysid
              where
                pg_class.relkind in ( 'r', 'v' )
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) = '='
        ) as subquery_3
    ),

    cte_pg_proc as
    (
      select
        schema_id,
        schema_name,
        case
          when proctype = 'c'         then 'C func'
          when proctype = 'exfunc'    then 'lambda func'
          when proctype = 'internal'  then 'internal func'
          when proctype = 'mlfunc'    then 'ML func'
          when proctype = 'plpgsql'   then 'procedure'
          when proctype = 'plpythonu' then 'python func'
          when proctype = 'sql'       then 'SQL func'
                                       else trim( proctype )
        end                            as object_type,
        procreturn_data_type_id       as object_return_base_data_type_id,
        procreturn_data_type_name     as object_return_base_data_type_name,
        procid                        as object_id,
        procname                      as object_name,
        procargument_data_type_ids    as object_argument_base_data_type_ids,
        procargument_data_type_names  as object_argument_base_data_type_names,
        procowner_id                  as object_owner_user_id,
        procowner_name                as object_owner_user_name,
        acl_item,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            schema_id,
            schema_name,
            procid,
            proctype,
            procreturn_data_type_id,
            procreturn_data_type_name,
            procname,
            procargument_data_type_ids,
            procargument_data_type_names,
            procowner_id,
            procowner_name,
            procacl,
            procdefault_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when procacl is     null then array_to_string( procdefault_acl[acl_index:acl_index], ',' )
              when procacl is not null then array_to_string( procacl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_namespace.oid     as schema_id,
                pg_namespace.nspname as schema_name,
                pg_proc.oid          as procid,
                pg_language.lanname  as proctype,
                pg_proc.prorettype   as procreturn_data_type_id,
                case
                  when pg_proc.proretset = True  then '_' || pg_type_return.typname
                  when pg_proc.proretset = False then        pg_type_return.typname
                end                  as procreturn_data_type_name,
                pg_proc.proname      as procname,

                                                                         pg_type_arg_00.oid::varchar     ||
                case when pg_type_arg_01.oid is null then '' else ',' || pg_type_arg_01.oid::varchar end ||
                case when pg_type_arg_02.oid is null then '' else ',' || pg_type_arg_02.oid::varchar end ||
                case when pg_type_arg_03.oid is null then '' else ',' || pg_type_arg_03.oid::varchar end ||
                case when pg_type_arg_04.oid is null then '' else ',' || pg_type_arg_04.oid::varchar end ||
                case when pg_type_arg_05.oid is null then '' else ',' || pg_type_arg_05.oid::varchar end ||
                case when pg_type_arg_06.oid is null then '' else ',' || pg_type_arg_06.oid::varchar end ||
                case when pg_type_arg_07.oid is null then '' else ',' || pg_type_arg_07.oid::varchar end ||
                case when pg_type_arg_08.oid is null then '' else ',' || pg_type_arg_08.oid::varchar end ||
                case when pg_type_arg_09.oid is null then '' else ',' || pg_type_arg_09.oid::varchar end ||
                case when pg_type_arg_10.oid is null then '' else ',' || pg_type_arg_10.oid::varchar end ||
                case when pg_type_arg_11.oid is null then '' else ',' || pg_type_arg_11.oid::varchar end ||
                case when pg_type_arg_12.oid is null then '' else ',' || pg_type_arg_12.oid::varchar end ||
                case when pg_type_arg_13.oid is null then '' else ',' || pg_type_arg_13.oid::varchar end ||
                case when pg_type_arg_14.oid is null then '' else ',' || pg_type_arg_14.oid::varchar end ||
                case when pg_type_arg_15.oid is null then '' else ',' || pg_type_arg_15.oid::varchar end ||
                case when pg_type_arg_16.oid is null then '' else ',' || pg_type_arg_16.oid::varchar end ||
                case when pg_type_arg_17.oid is null then '' else ',' || pg_type_arg_17.oid::varchar end ||
                case when pg_type_arg_18.oid is null then '' else ',' || pg_type_arg_18.oid::varchar end ||
                case when pg_type_arg_19.oid is null then '' else ',' || pg_type_arg_19.oid::varchar end ||
                case when pg_type_arg_20.oid is null then '' else ',' || pg_type_arg_20.oid::varchar end ||
                case when pg_type_arg_21.oid is null then '' else ',' || pg_type_arg_21.oid::varchar end ||
                case when pg_type_arg_22.oid is null then '' else ',' || pg_type_arg_22.oid::varchar end ||
                case when pg_type_arg_23.oid is null then '' else ',' || pg_type_arg_23.oid::varchar end ||
                case when pg_type_arg_24.oid is null then '' else ',' || pg_type_arg_24.oid::varchar end ||
                case when pg_type_arg_25.oid is null then '' else ',' || pg_type_arg_25.oid::varchar end ||
                case when pg_type_arg_26.oid is null then '' else ',' || pg_type_arg_26.oid::varchar end ||
                case when pg_type_arg_27.oid is null then '' else ',' || pg_type_arg_27.oid::varchar end ||
                case when pg_type_arg_28.oid is null then '' else ',' || pg_type_arg_28.oid::varchar end ||
                case when pg_type_arg_29.oid is null then '' else ',' || pg_type_arg_29.oid::varchar end ||
                case when pg_type_arg_30.oid is null then '' else ',' || pg_type_arg_30.oid::varchar end ||
                case when pg_type_arg_31.oid is null then '' else ',' || pg_type_arg_31.oid::varchar end     as procargument_data_type_ids,

                                                                             pg_type_arg_00.typname     ||
                case when pg_type_arg_01.typname is null then '' else ',' || pg_type_arg_01.typname end ||
                case when pg_type_arg_02.typname is null then '' else ',' || pg_type_arg_02.typname end ||
                case when pg_type_arg_03.typname is null then '' else ',' || pg_type_arg_03.typname end ||
                case when pg_type_arg_04.typname is null then '' else ',' || pg_type_arg_04.typname end ||
                case when pg_type_arg_05.typname is null then '' else ',' || pg_type_arg_05.typname end ||
                case when pg_type_arg_06.typname is null then '' else ',' || pg_type_arg_06.typname end ||
                case when pg_type_arg_07.typname is null then '' else ',' || pg_type_arg_07.typname end ||
                case when pg_type_arg_08.typname is null then '' else ',' || pg_type_arg_08.typname end ||
                case when pg_type_arg_09.typname is null then '' else ',' || pg_type_arg_09.typname end ||
                case when pg_type_arg_10.typname is null then '' else ',' || pg_type_arg_10.typname end ||
                case when pg_type_arg_11.typname is null then '' else ',' || pg_type_arg_11.typname end ||
                case when pg_type_arg_12.typname is null then '' else ',' || pg_type_arg_12.typname end ||
                case when pg_type_arg_13.typname is null then '' else ',' || pg_type_arg_13.typname end ||
                case when pg_type_arg_14.typname is null then '' else ',' || pg_type_arg_14.typname end ||
                case when pg_type_arg_15.typname is null then '' else ',' || pg_type_arg_15.typname end ||
                case when pg_type_arg_16.typname is null then '' else ',' || pg_type_arg_16.typname end ||
                case when pg_type_arg_17.typname is null then '' else ',' || pg_type_arg_17.typname end ||
                case when pg_type_arg_18.typname is null then '' else ',' || pg_type_arg_18.typname end ||
                case when pg_type_arg_19.typname is null then '' else ',' || pg_type_arg_19.typname end ||
                case when pg_type_arg_20.typname is null then '' else ',' || pg_type_arg_20.typname end ||
                case when pg_type_arg_21.typname is null then '' else ',' || pg_type_arg_21.typname end ||
                case when pg_type_arg_22.typname is null then '' else ',' || pg_type_arg_22.typname end ||
                case when pg_type_arg_23.typname is null then '' else ',' || pg_type_arg_23.typname end ||
                case when pg_type_arg_24.typname is null then '' else ',' || pg_type_arg_24.typname end ||
                case when pg_type_arg_25.typname is null then '' else ',' || pg_type_arg_25.typname end ||
                case when pg_type_arg_26.typname is null then '' else ',' || pg_type_arg_26.typname end ||
                case when pg_type_arg_27.typname is null then '' else ',' || pg_type_arg_27.typname end ||
                case when pg_type_arg_28.typname is null then '' else ',' || pg_type_arg_28.typname end ||
                case when pg_type_arg_29.typname is null then '' else ',' || pg_type_arg_29.typname end ||
                case when pg_type_arg_30.typname is null then '' else ',' || pg_type_arg_30.typname end ||
                case when pg_type_arg_31.typname is null then '' else ',' || pg_type_arg_31.typname end     as procargument_data_type_names,

                pg_proc.proowner as procowner_id,
                pg_user.usename  as procowner_name,
                pg_proc.proacl   as procacl,
                case
                  when pg_language.lanname = 'c'         then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  -- MG2 : internal are all {=X/rdsdb}, and we can't make more languages (no "create language" in Redshift)
                  when pg_language.lanname = 'internal'  then array[ '=X/rdsdb' ]
                  when pg_language.lanname = 'plpgsql'   then array[ pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'plpythonu' then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                  when pg_language.lanname = 'sql'       then array[ '=X/' || pg_user.usename || ',' || pg_user.usename || '=X/' || pg_user.usename ]
                end              as procdefault_acl,
                generate_series( 1,
                                 case
                                   when procacl is     null then array_upper( procdefault_acl, 1 )
                                   when procacl is not null then array_upper( procacl, 1 )
                                 end ) as acl_index
              from
                          pg_proc
                     join pg_language               on pg_proc.prolang          = pg_language.oid
                     join pg_namespace              on pg_proc.pronamespace     = pg_namespace.oid
                     join pg_user                   on pg_proc.proowner         = pg_user.usesysid
                -- MG2 : procedures do not have a return type
                left join pg_type as pg_type_return on pg_proc.prorettype       = pg_type_return.oid
                -- MG2 : pg_proc.proargtypes is an oidvector, which is an array of 32 oids, when an element is empty, the value is 0
                left join pg_type as pg_type_arg_00 on pg_proc.proargtypes[00] != 0 and pg_proc.proargtypes[00] = pg_type_arg_00.oid
                left join pg_type as pg_type_arg_01 on pg_proc.proargtypes[01] != 0 and pg_proc.proargtypes[01] = pg_type_arg_01.oid
                left join pg_type as pg_type_arg_02 on pg_proc.proargtypes[02] != 0 and pg_proc.proargtypes[02] = pg_type_arg_02.oid
                left join pg_type as pg_type_arg_03 on pg_proc.proargtypes[03] != 0 and pg_proc.proargtypes[03] = pg_type_arg_03.oid
                left join pg_type as pg_type_arg_04 on pg_proc.proargtypes[04] != 0 and pg_proc.proargtypes[04] = pg_type_arg_04.oid
                left join pg_type as pg_type_arg_05 on pg_proc.proargtypes[05] != 0 and pg_proc.proargtypes[05] = pg_type_arg_05.oid
                left join pg_type as pg_type_arg_06 on pg_proc.proargtypes[06] != 0 and pg_proc.proargtypes[06] = pg_type_arg_06.oid
                left join pg_type as pg_type_arg_07 on pg_proc.proargtypes[07] != 0 and pg_proc.proargtypes[07] = pg_type_arg_07.oid
                left join pg_type as pg_type_arg_08 on pg_proc.proargtypes[08] != 0 and pg_proc.proargtypes[08] = pg_type_arg_08.oid
                left join pg_type as pg_type_arg_09 on pg_proc.proargtypes[09] != 0 and pg_proc.proargtypes[09] = pg_type_arg_09.oid
                left join pg_type as pg_type_arg_10 on pg_proc.proargtypes[10] != 0 and pg_proc.proargtypes[10] = pg_type_arg_10.oid
                left join pg_type as pg_type_arg_11 on pg_proc.proargtypes[11] != 0 and pg_proc.proargtypes[11] = pg_type_arg_11.oid
                left join pg_type as pg_type_arg_12 on pg_proc.proargtypes[12] != 0 and pg_proc.proargtypes[12] = pg_type_arg_12.oid
                left join pg_type as pg_type_arg_13 on pg_proc.proargtypes[13] != 0 and pg_proc.proargtypes[13] = pg_type_arg_13.oid
                left join pg_type as pg_type_arg_14 on pg_proc.proargtypes[14] != 0 and pg_proc.proargtypes[14] = pg_type_arg_14.oid
                left join pg_type as pg_type_arg_15 on pg_proc.proargtypes[15] != 0 and pg_proc.proargtypes[15] = pg_type_arg_15.oid
                left join pg_type as pg_type_arg_16 on pg_proc.proargtypes[16] != 0 and pg_proc.proargtypes[16] = pg_type_arg_16.oid
                left join pg_type as pg_type_arg_17 on pg_proc.proargtypes[17] != 0 and pg_proc.proargtypes[17] = pg_type_arg_17.oid
                left join pg_type as pg_type_arg_18 on pg_proc.proargtypes[18] != 0 and pg_proc.proargtypes[18] = pg_type_arg_18.oid
                left join pg_type as pg_type_arg_19 on pg_proc.proargtypes[19] != 0 and pg_proc.proargtypes[19] = pg_type_arg_19.oid
                left join pg_type as pg_type_arg_20 on pg_proc.proargtypes[20] != 0 and pg_proc.proargtypes[20] = pg_type_arg_20.oid
                left join pg_type as pg_type_arg_21 on pg_proc.proargtypes[21] != 0 and pg_proc.proargtypes[21] = pg_type_arg_21.oid
                left join pg_type as pg_type_arg_22 on pg_proc.proargtypes[22] != 0 and pg_proc.proargtypes[22] = pg_type_arg_22.oid
                left join pg_type as pg_type_arg_23 on pg_proc.proargtypes[23] != 0 and pg_proc.proargtypes[23] = pg_type_arg_23.oid
                left join pg_type as pg_type_arg_24 on pg_proc.proargtypes[24] != 0 and pg_proc.proargtypes[24] = pg_type_arg_24.oid
                left join pg_type as pg_type_arg_25 on pg_proc.proargtypes[25] != 0 and pg_proc.proargtypes[25] = pg_type_arg_25.oid
                left join pg_type as pg_type_arg_26 on pg_proc.proargtypes[26] != 0 and pg_proc.proargtypes[26] = pg_type_arg_26.oid
                left join pg_type as pg_type_arg_27 on pg_proc.proargtypes[27] != 0 and pg_proc.proargtypes[27] = pg_type_arg_27.oid
                left join pg_type as pg_type_arg_28 on pg_proc.proargtypes[28] != 0 and pg_proc.proargtypes[28] = pg_type_arg_28.oid
                left join pg_type as pg_type_arg_29 on pg_proc.proargtypes[29] != 0 and pg_proc.proargtypes[29] = pg_type_arg_29.oid
                left join pg_type as pg_type_arg_30 on pg_proc.proargtypes[30] != 0 and pg_proc.proargtypes[30] = pg_type_arg_30.oid
                left join pg_type as pg_type_arg_31 on pg_proc.proargtypes[31] != 0 and pg_proc.proargtypes[31] = pg_type_arg_31.oid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) = '='
        ) as subquery_3
    ),

    cte_pg_language as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'language'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        language_id         as object_id,
        language_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        language_owner_id   as object_owner_user_id,
        language_owner_name as object_owner_user_name,
        acl_item,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            language_id,
            language_name,
            language_owner_id,
            language_owner_name,
            language_acl,
            language_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when language_acl is     null then array_to_string( language_default_acl[acl_index:acl_index], ',' )
              when language_acl is not null then array_to_string( language_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                language_id,
                language_name,
                language_owner_id,
                language_owner_name,
                language_acl,
                language_default_acl,
                generate_series( 1,
                                 case
                                   when language_acl is     null then array_upper( language_default_acl, 1 )
                                   when language_acl is not null then array_upper( language_acl, 1 )
                                 end ) as acl_index
              from
                (
                  select
                    pg_language.oid     as language_id,
                    pg_language.lanname as language_name,
                    1::oid              as language_owner_id,
                    'rdsdb'             as language_owner_name,
                    pg_language.lanacl  as language_acl,
                    case
                      -- MG2 : no CREATE LANGUAGE in Redshift, so owner is always rdsdb (OID 1)
                      --        usage on language seems brolen and is not checked
                      --        the usual usage grant to group public is not made for plpgsql and plpythonu
                      --        (but this has no affect, since the check for usage is not made)
                      when pg_language.lanname not in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb,=U/rdsdb' ]
                      when pg_language.lanname     in ('plpgsql','plpythonu') then array[ 'rdsdb=U/rdsdb' ]
                    end                 as language_default_acl
                  from
                    pg_language
                ) as subquery_1
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) = '='
        ) as subquery_3
    ),

    cte_pg_database as
    (
      select
        null::oid           as schema_id,
        null::name          as schema_name,
        'database'          as object_type,
        null::oid           as object_return_base_data_type_id,
        null::name          as object_return_base_data_type_name,
        database_id         as object_id,
        database_name       as object_name,
        null::varchar       as object_argument_base_data_type_ids,
        null::varchar       as object_argument_base_data_type_names,
        database_owner_id   as object_owner_user_id,
        database_owner_name as object_owner_user_name,
        acl_item,
        -- MG2 : the substring gives us the ACL permissions and the owner, "CT/rdsdb", and then from this we take the permissions, "CT"
        split_part( substring(acl_item, strpos(acl_item,'=')+1), '/', 1 ) as acl_privileges
      from
        (
          select
            database_id,
            database_name,
            database_owner_id,
            database_owner_name,
            database_acl,
            database_default_acl,
            case
              -- MG2 : the second argument to array_to_string() is the separator to be used when extracting multiple elements, but we only ever extract a single element
              when database_acl is     null then array_to_string( database_default_acl[acl_index:acl_index], ',' )
              when database_acl is not null then array_to_string( database_acl[acl_index:acl_index], ',' )
            end as acl_item
          from
            (
              select
                pg_database.oid     as database_id,
                pg_database.datname as database_name,
                pg_database.datdba  as database_owner_id,
                pg_user.usename     as database_owner_name,
                pg_database.datacl  as database_acl,
                -- MG2 : note we do not reprocuce the array-indicating open and close curly braces
                --       we could, to make the database_acl strings look identical but then they're a pain later on
                array[ '=T/' || pg_user.usename || ',' || pg_user.usename || '=CT/' || pg_user.usename ] as database_default_acl,
                generate_series( 1,
                                 case
                                   when database_acl is     null then array_upper( database_default_acl, 1 )
                                   when database_acl is not null then array_upper( database_acl, 1 )
                                 end ) as acl_index
              from
                     pg_database
                join pg_user on pg_database.datdba = pg_user.usesysid
            ) as subquery_2
          where
            substring(acl_item, 1, 5) != 'group' and substring(acl_item, 1, 1) = '='
        ) as subquery_3
    ),

    cte_all as
    (
      select * from cte_pg_namespace
      union all
      select * from cte_pg_class
      union all
      select * from cte_pg_proc
      union all
      select * from cte_pg_language
      union all
      select * from cte_pg_database
    )

  select
    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    acl_privileges::varchar
  from
    cte_all;


create or replace view
  lf_241129.privileges_granted_to_public
as
  select
    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    case
      when object_type = 'database'  and strpos(acl_privileges,'C') != 0 then true else false
    end::bool as priv_database_create,

    case
      when object_type = 'database'  and strpos(acl_privileges,'T') != 0 then true else false
    end::bool as priv_database_temp,

    case
      when object_type = 'function'  and strpos(acl_privileges,'X') != 0 then true else false
    end::bool as priv_function_execute,

    case
      when object_type = 'language'  and strpos(acl_privileges,'U') != 0 then true else false
    end::bool as priv_language_usage,

    case
      when object_type = 'procedure' and strpos(acl_privileges,'X') != 0 then true else false
    end::bool as priv_procedure_execute,

    case
      when object_type = 'schema'    and strpos(acl_privileges,'C') != 0 then true else false
    end::bool as priv_schema_create,

    case
      when object_type = 'schema'    and strpos(acl_privileges,'U') != 0 then true else false
    end::bool as priv_schema_usage,

    case
      when object_type = 'table'     and strpos(acl_privileges,'d') != 0 then true else false
    end::bool as priv_table_delete,

    case
      when object_type = 'table'     and strpos(acl_privileges,'D') != 0 then true else false
    end::bool as priv_table_drop,

    case
      when object_type = 'table'     and strpos(acl_privileges,'a') != 0 then true else false
    end::bool as priv_table_insert,

    case
      when object_type = 'table'     and strpos(acl_privileges,'x') != 0 then true else false
    end::bool as priv_table_reference,

    case
      when object_type = 'table'     and strpos(acl_privileges,'r') != 0 then true else false
    end::bool as priv_table_select,

    case
      when object_type = 'table'     and strpos(acl_privileges,'w') != 0 then true else false
    end::bool as priv_table_update,

    case
      when object_type = 'view'      and strpos(acl_privileges,'D') != 0 then true else false
    end::bool as priv_view_drop,

    case
      when object_type = 'view'      and strpos(acl_privileges,'r') != 0 then true else false
    end::bool as priv_view_select
  from
    base_241129.privileges_public;


create or replace view
  sf_241129.privileges_granted_to_public
as
  select
    object_type::varchar as ttype,
    schema_name::varchar as sschema,
    object_name::varchar as oobject,

    object_return_base_data_type_name::varchar    as rt,
    object_argument_base_data_type_names::varchar as args,

    object_owner_user_name::varchar as oowner,

    (
      case when priv_database_create = true then 'c' else ' ' end
      ||
      case when priv_database_temp   = true then 't' else ' ' end
      ||
      case when priv_language_usage  = true then 'u' else ' ' end
      ||
      '.'
      ||
      case when priv_schema_create   = true then 'c' else ' ' end
      ||
      case when priv_schema_usage    = true then 'u' else ' ' end
      ||
      '.'
      ||
      case when priv_table_delete    = true then 'd' else ' ' end
      ||
      case when priv_table_reference = true then 'f' else ' ' end
      ||
      case when priv_table_insert    = true then 'i' else ' ' end
      ||
      case when priv_table_drop      = true then 'p' else ' ' end
      ||
      case when priv_table_select = true or priv_view_select = true then 's' else ' ' end
      ||
      case when priv_table_update    = true then 'u' else ' ' end
      ||
      case when priv_function_execute = true or priv_procedure_execute = true then 'x' else ' ' end
    )::varchar as privs
  from
    lf_241129.privileges_granted_to_public
  where
    object_owner_user_name != 'rdsdb'
  order by
    schema_name,
    case when object_type = 'schema' then 0 else 1 end,
    object_type,
    object_name;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_query
as
  select
    query_id::int4,

    case
      when slice_ordinal  = -1 then 'leader'
      when slice_ordinal != -1 then 'worker'
    end::varchar as node_type,

    sum( case when type =  1 then rows  else 0 end )::int8 as sum_type_1_rows,
    sum( case when type =  1 then bytes else 0 end )::int8 as sum_type_1_bytes,

    sum( case when type =  2 then rows_pre_filter      else 0 end )::int8 as sum_type_2m_rows,
    sum( case when type =  2 then rows_pre_user_filter else 0 end )::int8 as sum_type_2e_rows,
    sum( case when type =  2 then rows                 else 0 end )::int8 as sum_type_2_rows,
    sum( case when type =  2 then bytes                else 0 end )::int8 as sum_type_2_bytes,

    sum( case when type =  3 then rows  else 0 end )::int8 as sum_type_3_rows,
    sum( case when type =  3 then bytes else 0 end )::int8 as sum_type_3_bytes,

    sum( case when type =  9 then rows  else 0 end )::int8 as sum_type_9_rows,
    sum( case when type =  9 then bytes else 0 end )::int8 as sum_type_9_bytes,

    sum( case when type = 16 then rows  else 0 end )::int8 as sum_type_16_rows,
    sum( case when type = 16 then bytes else 0 end )::int8 as sum_type_16_bytes,

    sum( case when type = 17 then rows  else 0 end )::int8 as sum_type_17_rows,
    sum( case when type = 17 then bytes else 0 end )::int8 as sum_type_17_bytes,

    sum( case when type = 18 then rows  else 0 end )::int8 as sum_type_18_rows,
    sum( case when type = 18 then bytes else 0 end )::int8 as sum_type_18_bytes,

    sum( case when type = 21 then rows  else 0 end )::int8 as sum_type_21_rows,
    sum( case when type = 21 then bytes else 0 end )::int8 as sum_type_21_bytes,

    sum( case when type = 22 then rows  else 0 end )::int8 as sum_type_22_rows,
    sum( case when type = 22 then bytes else 0 end )::int8 as sum_type_22_bytes,

    sum( case when type = 23 then rows  else 0 end )::int8 as sum_type_23_rows,
    sum( case when type = 23 then bytes else 0 end )::int8 as sum_type_23_bytes,

    sum( case when type = 24 then rows  else 0 end )::int8 as sum_type_24_rows,
    sum( case when type = 24 then bytes else 0 end )::int8 as sum_type_24_bytes,

    sum( case when type = 25 then rows  else 0 end )::int8 as sum_type_25_rows,
    sum( case when type = 25 then bytes else 0 end )::int8 as sum_type_25_bytes,

    sum( case when type = 28 then rows  else 0 end )::int8 as sum_type_28_rows,
    sum( case when type = 28 then bytes else 0 end )::int8 as sum_type_28_bytes,

    sum( case when type = 29 then rows_pre_filter      else 0 end )::int8 as sum_type_29m_rows,
    sum( case when type = 29 then rows_pre_user_filter else 0 end )::int8 as sum_type_29e_rows,
    sum( case when type = 29 then rows                 else 0 end )::int8 as sum_type_29_rows,
    sum( case when type = 29 then bytes                else 0 end )::int8 as sum_type_29_bytes,

    sum( case when type = 30 then rows  else 0 end )::int8 as sum_type_30_rows,
    sum( case when type = 30 then bytes else 0 end )::int8 as sum_type_30_bytes,

    sum( case when type = 32 then rows  else 0 end )::int8 as sum_type_32_rows,
    sum( case when type = 32 then bytes else 0 end )::int8 as sum_type_32_bytes,

    sum( case when type = 33 then rows  else 0 end )::int8 as sum_type_33_rows,
    sum( case when type = 33 then bytes else 0 end )::int8 as sum_type_33_bytes,

    sum( case when type = 35 then rows  else 0 end )::int8 as sum_type_35_rows,
    sum( case when type = 35 then bytes else 0 end )::int8 as sum_type_35_bytes,

    sum( case when type = 36 then rows  else 0 end )::int8 as sum_type_36_rows,
    sum( case when type = 36 then bytes else 0 end )::int8 as sum_type_36_bytes,

    sum( case when type = 37 then rows  else 0 end )::int8 as sum_type_37_rows,
    sum( case when type = 37 then bytes else 0 end )::int8 as sum_type_37_bytes
  from
    base_241129.fixed_stl_scan
  where
        scan_start_ts >= current_date - interval '6 day'
    and scan_end_ts   <= current_timestamp
  group by
    query_id,
    node_type;



create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_query
as
  with
    cte_hr_formatting as
    (
      select
        query_id,
        node_type,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_query
    )

 select
    query_id::int4     as qid,
    node_type::varchar as node,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    query_id,
    node_type;




create or replace view
  lf_241129.steps_totals_rows_on_workers_for_tables_by_hour
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    start_ts::timestamp,
    cte_delete.sum_delete_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_scan.sum_scan_rows::int8
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_hours
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_tables_by_hour
as
  select
    schema_name::varchar                       as sschema,
    table_name::varchar                        as ttable,
    start_ts::timestamp,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_tables_by_hour
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_database_by_second
as
  select
    database::varchar                                                                                             as database_name,
    date_trunc( 'second', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    database,
    start_ts;


create or replace view
  lf_241129.uusers
as
  with
    cte_pg_class as
    (
      select
        relowner                                                                                                              as user_id,
        sum( case when pg_class.relkind = 'r'                                                             then 1 else 0 end ) as number_tables,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then 1 else 0 end ) as number_normal_views,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts  = 0 and stv_mv_info.name is     null then 1 else 0 end ) as number_late_binding_views,
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then 1 else 0 end ) as number_materialized_views
      from
                  pg_class
             join pg_namespace on pg_class.relnamespace = pg_namespace.oid
        left join stv_mv_info  on stv_mv_info.db_name = current_database()   and
                                  stv_mv_info.schema  = pg_namespace.nspname and
                                  stv_mv_info.name    = pg_class.relname
      where
        pg_class.relkind = 'r' or pg_class.relkind = 'v'
      group by
        pg_class.relowner
    ),

    cte_pg_databases as
    (
      select
        datdba   as user_id,
        count(*) as number_databases
      from
        pg_database
      group by
        datdba
    ),

    cte_pg_proc as
    (
      select
        pg_proc.proowner                                                    as user_id,
        sum( case when pg_language.lanname != 'plpgsql' then 1 else 0 end ) as number_functions,
        sum( case when pg_language.lanname  = 'plpgsql' then 1 else 0 end ) as number_procedures
      from
             pg_proc
        join pg_language on pg_proc.prolang = pg_language.oid
      group by
        pg_proc.proowner
    ),

    cte_pg_namespace as
    (
      select
        nspowner as user_id,
        count(*) as number_schemas
      from
        pg_namespace
      group by
        nspowner
    ),

    cte_users as
    (
      select
        pg_user.usesysid                                                                                      as user_id,
        trim( pg_user.usename )                                                                               as user_name,
        case
          when pg_user.usename = 'rdsdb' then 'rowt'
          when pg_user.usesuper = true   then 'super'
                                         else 'normal'
        end                                                                                                   as user_type,
        case
          when pg_user.usename = 'rdsdb' then 'unrestricted'
                                         else lower( svl_user_info.syslogaccess )
        end                                                                                                             as user_syslog,
        case when cte_pg_databases.number_databases      is null then 0 else cte_pg_databases.number_databases      end as number_databases,
        case when cte_pg_proc.number_functions           is null then 0 else cte_pg_proc.number_functions           end as number_functions,
        case when cte_pg_proc.number_procedures          is null then 0 else cte_pg_proc.number_procedures          end as number_procedures,
        case when cte_pg_namespace.number_schemas        is null then 0 else cte_pg_namespace.number_schemas        end as number_schemas,
        case when cte_pg_class.number_tables             is null then 0 else cte_pg_class.number_tables             end as number_tables,
        case when cte_pg_class.number_normal_views       is null then 0 else cte_pg_class.number_normal_views       end as number_normal_views,
        case when cte_pg_class.number_late_binding_views is null then 0 else cte_pg_class.number_late_binding_views end as number_late_binding_views,
        case when cte_pg_class.number_materialized_views is null then 0 else cte_pg_class.number_materialized_views end as number_materialized_views
      from
                  pg_user
        left join svl_user_info    on pg_user.usesysid = svl_user_info.usesysid
        left join cte_pg_class     on pg_user.usesysid = cte_pg_class.user_id 
        left join cte_pg_databases on pg_user.usesysid = cte_pg_databases.user_id
        left join cte_pg_proc      on pg_user.usesysid = cte_pg_proc.user_id
        left join cte_pg_namespace on pg_user.usesysid = cte_pg_namespace.user_id
    )

  select
    user_id::int4,
    user_name::varchar,
    user_type::varchar,
    user_syslog::varchar,
    number_databases::int2,
    number_functions::int4,
    number_procedures::int4,
    number_schemas::int2,
    number_tables::int4,
    number_normal_views::int4,
    number_late_binding_views::int4,
    number_materialized_views::int4
  from
    cte_users;


create or replace procedure
  ro.sp_grant_version_to_public( version in varchar(6) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'grant usage on schema base_' || version || ' to public';
      execute sql;

      sql := 'grant usage on schema lf_' || version || ' to public';
      execute sql;

      sql := 'grant usage on schema sf_' || version || ' to public';
      execute sql;

      sql := 'grant select on all tables in schema base_' || version || ' to public';
      execute sql;

      sql := 'grant select on all tables in schema lf_' || version || ' to public';
      execute sql;

      sql := 'grant select on all tables in schema sf_' || version || ' to public';
      execute sql;

      sql := 'grant execute on all functions  in schema base_' || version || ' to public';
      execute sql;

      sql := 'grant execute on all procedures in schema lf_' || version || ' to public';
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_node_by_second
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'second', scan_start_ts )::timestamp as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal )
  group by
    node_ordinal,
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_users_by_second
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4               as user_id,
    trim( pg_user.usename )::varchar     as user_name,
    base_241129.generate_seconds.start_ts::timestamp as start_ts,
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_seconds
    cross join pg_user
     left join cte_aggr_stats         using ( start_ts, node_type, usesysid )
     left join cte_bcast_stats        using ( start_ts, node_type, usesysid )
     left join cte_delete_stats       using ( start_ts, node_type, usesysid )
     left join cte_dist_stats         using ( start_ts, node_type, usesysid )
     left join cte_hash_stats         using ( start_ts, node_type, usesysid )
     left join cte_hashjoin_stats     using ( start_ts, node_type, usesysid )
     left join cte_insert_stats       using ( start_ts, node_type, usesysid )
     left join cte_limit_stats        using ( start_ts, node_type, usesysid )
     left join cte_merge_stats        using ( start_ts, node_type, usesysid )
     left join cte_mergejoin_stats    using ( start_ts, node_type, usesysid )
     left join cte_nestloop_stats     using ( start_ts, node_type, usesysid )
     left join cte_parse_stats        using ( start_ts, node_type, usesysid )
     left join cte_project_stats      using ( start_ts, node_type, usesysid )
     left join cte_return_stats       using ( start_ts, node_type, usesysid )
     left join cte_save_stats         using ( start_ts, node_type, usesysid )
     left join cte_sort_stats         using ( start_ts, node_type, usesysid )
     left join cte_spatial_join_stats using ( start_ts, node_type, usesysid )
     left join cte_unique_stats       using ( start_ts, node_type, usesysid )
     left join cte_window_stats       using ( start_ts, node_type, usesysid )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_bcast_duration is not null
    or max_bcast_duration is not null
    or med_bcast_duration is not null
    or avg_bcast_duration is not null
    or std_bcast_duration is not null
    or number_bcast_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_dist_duration is not null
    or max_dist_duration is not null
    or med_dist_duration is not null
    or avg_dist_duration is not null
    or std_dist_duration is not null
    or number_dist_steps is not null
    or min_hash_duration is not null
    or max_hash_duration is not null
    or med_hash_duration is not null
    or avg_hash_duration is not null
    or std_hash_duration is not null
    or number_hash_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or min_limit_duration is not null
    or max_limit_duration is not null
    or med_limit_duration is not null
    or avg_limit_duration is not null
    or std_limit_duration is not null
    or number_limit_steps is not null
    or min_merge_duration is not null
    or max_merge_duration is not null
    or med_merge_duration is not null
    or avg_merge_duration is not null
    or std_merge_duration is not null
    or number_merge_steps is not null
    or min_mergejoin_duration is not null
    or max_mergejoin_duration is not null
    or med_mergejoin_duration is not null
    or avg_mergejoin_duration is not null
    or std_mergejoin_duration is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or min_parse_duration is not null
    or max_parse_duration is not null
    or med_parse_duration is not null
    or avg_parse_duration is not null
    or std_parse_duration is not null
    or number_parse_steps is not null
    or min_project_duration is not null
    or max_project_duration is not null
    or med_project_duration is not null
    or avg_project_duration is not null
    or std_project_duration is not null
    or number_project_steps is not null
    or min_return_duration is not null
    or max_return_duration is not null
    or med_return_duration is not null
    or avg_return_duration is not null
    or std_return_duration is not null
    or number_return_steps is not null
    or min_save_duration is not null
    or max_save_duration is not null
    or med_save_duration is not null
    or avg_save_duration is not null
    or std_save_duration is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null
    or min_unique_duration is not null
    or max_unique_duration is not null
    or med_unique_duration is not null
    or avg_unique_duration is not null
    or std_unique_duration is not null
    or number_unique_steps is not null
    or min_window_duration is not null
    or max_window_duration is not null
    or med_window_duration is not null
    or avg_window_duration is not null
    or std_window_duration is not null
    or number_window_steps is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_second_concise
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_second
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.wqueues_totals_slot_utilization_by_second
as
  with
    cte_events as
    (
      select
        queue_id,
        query_compile_exec_start_ts as event_ts,
        1                   as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'

      union all

      select
        queue_id,
        query_compile_exec_end_ts as event_ts,
        -1                as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_slot_durations as
    (
      select
        queue_id,
        sum( event_delta ) over ( partition by queue_id, start_ts order by event_ts rows between unbounded preceding and current row ) as slot_number,
        date_trunc( 'second', event_ts )                                                                                             as start_ts,
        lead( event_ts   ) over ( partition by queue_id, start_ts order by event_ts ) - event_ts                                       as duration
      from
        cte_events
    ),

    cte_durations as
    (
      select
        queue_id,
        slot_number,
        start_ts,
        sum( duration ) as sum_duration
      from
        cte_slot_durations
      group by
        queue_id,
        slot_number,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    slot_number::int2,
    start_ts::timestamp,
    sum_duration::interval as duration
  from
              cte_durations
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  sf_241129.wqueues_totals_slot_utilization_by_second
as
  select
    queue_name::varchar                           as qname,
    slot_number::int2                             as slot,
    start_ts::timestamp,
    extract( epoch from duration )::numeric(18,2) as dur
  from
    lf_241129.wqueues_totals_slot_utilization_by_second
  order by
    queue_name,
    slot_number,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_queues_by_day
as
  with
    cte_queries as
    (
      select
        queue_id,
        date_trunc( 'day', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
                  base_241129.fixed_stl_wlm_query
        left join base_241129.fixed_stl_query    using ( query_id )
        left join base_241129.queries_compilation using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        queue_id,
        start_ts,

        min( query_queue_duration ) over ( partition by queue_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by queue_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by queue_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by queue_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by queue_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by queue_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by queue_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by queue_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by queue_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by queue_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by queue_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by queue_id, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by queue_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_days
    left join cte_queries_stats using ( start_ts )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_queues_by_day
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_queues_by_day
    )

  select
    queue_name::varchar as qname,
    start_ts::date,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    queue_name,
    start_ts;


create or replace view
  base_241129.fixed_stv_query_metrics
as
  select
    case when blocks_read    != -1 then blocks_read    else 0 end::int8 as query_number_blocks_read,
    case when blocks_to_disk != -1 then blocks_to_disk else 0 end::int8 as query_number_blocks_to_disk,
    cpu_time::int8 as query_cpu_time,
    -- max_blocks_read::int4
    -- max_blocks_to_disk::int8
    -- max_cpu_time::int8
    -- max_query_scan_size::int8
    -- max_rows::int8
    -- max_run_time::int8
    query::int4 as query_id,
    -- query_priority::int4,
    -- query_queue_time::int8,
    -- query_scan_size::int8,
    rows::int8 as query_number_rows,
    -- run_time::int8,
    -- segment::int2    as segment_ordinal,
    service_class::int2 as queue_id,
    -- service_class_name::char(64)
    -- slices::int4
    starttime::timestamp   as query_start_ts,
    -- step::int2          as step_ordinal,
    -- step_type::int2,
    userid::int4        as user_id
  from
    stv_query_metrics
  where
        segment   = -1
    and step_type = -1;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_queues_by_hour
as
  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_hours
    left join base_241129.fixed_stl_query                       on date_trunc( 'hour', query_start_ts ) = base_241129.generate_hours.start_ts
         join base_241129.fixed_stl_wlm_query                using ( query_id )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    queue_id,
    queue_name,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_queues_by_minute
as
  with
    cte_queries as
    (
      select
        queue_id,
        date_trunc( 'minute', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
                  base_241129.fixed_stl_wlm_query
        left join base_241129.fixed_stl_query    using ( query_id )
        left join base_241129.queries_compilation using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        queue_id,
        start_ts,

        min( query_queue_duration ) over ( partition by queue_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by queue_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by queue_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by queue_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by queue_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by queue_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by queue_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by queue_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by queue_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by queue_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by queue_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by queue_id, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by queue_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_minutes
    left join cte_queries_stats using ( start_ts )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  lf_241129.sschemas
as
  with
    cte_number_tables_and_views as
    (
      select
        relnamespace                                                                                                          as schema_id,
        sum( case when pg_class.relkind = 'r'                                                             then 1 else 0 end ) as number_tables,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then 1 else 0 end ) as number_normal_views,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts  = 0 and stv_mv_info.name is     null then 1 else 0 end ) as number_late_binding_views,
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then 1 else 0 end ) as number_materialized_views
      from
                  pg_class
             join pg_namespace on pg_class.relnamespace = pg_namespace.oid
        left join stv_mv_info  on stv_mv_info.db_name = current_database()   and
                                  stv_mv_info.schema  = pg_namespace.nspname and
                                  stv_mv_info.name    = pg_class.relname
      where
        pg_class.relkind = 'r' or pg_class.relkind = 'v'
      group by
        schema_id
    ),

    cte_number_funcs_and_procs as
    (
      select
        pg_proc.pronamespace as schema_id,
        sum( case when pg_language.lanname != 'plpgsql' then 1 else 0 end ) as number_functions,
        sum( case when pg_language.lanname  = 'plpgsql' then 1 else 0 end ) as number_procedures
      from
             pg_proc
        join pg_language on pg_proc.prolang = pg_language.oid
      group by
        pg_proc.pronamespace
    ),

    cte_schemas as
    (
      select
        pg_namespace.oid                                                                                                                            as schema_id,
        trim( pg_namespace.nspname )                                                                                                                as schema_name,
        pg_namespace.nspowner                                                                                                                       as user_id,
        trim( pg_user.usename )                                                                                                                     as user_name,
        case when cte_number_tables_and_views.number_tables             is null then 0 else cte_number_tables_and_views.number_tables             end as number_tables,
        case when cte_number_tables_and_views.number_normal_views       is null then 0 else cte_number_tables_and_views.number_normal_views       end as number_normal_views,
        case when cte_number_tables_and_views.number_late_binding_views is null then 0 else cte_number_tables_and_views.number_late_binding_views end as number_late_binding_views,
        case when cte_number_tables_and_views.number_materialized_views is null then 0 else cte_number_tables_and_views.number_materialized_views end as number_materialized_views,
        case when cte_number_funcs_and_procs.number_functions           is null then 0 else cte_number_funcs_and_procs.number_functions           end as number_functions,
        case when cte_number_funcs_and_procs.number_procedures          is null then 0 else cte_number_funcs_and_procs.number_procedures          end as number_procedures
      from
                  pg_namespace
             join pg_user                     on pg_namespace.nspowner = pg_user.usesysid
        left join cte_number_tables_and_views on pg_namespace.oid      = cte_number_tables_and_views.schema_id
        left join cte_number_funcs_and_procs  on pg_namespace.oid      = cte_number_funcs_and_procs.schema_id
    )

  select
    schema_id::int4,
    schema_name::varchar,
    user_id::int4,
    user_name::varchar,
    number_tables::int4,
    number_normal_views::int4,
    number_late_binding_views::int4,
    number_materialized_views::int4,
    number_functions::int4,
    number_procedures::int4
  from
    cte_schemas;


create or replace view
  lf_241129.sschemas_blocks_rows
as
  with
    cte_schemas_blocks_rows as
    (
      select
        schema_id,
        sum( base_241129.ttables_blocks_rows.number_sorted_blocks   ) as number_sorted_blocks,
        sum( base_241129.ttables_blocks_rows.number_unsorted_blocks ) as number_desorted_blocks,
        sum( base_241129.ttables_blocks_rows.number_unsorted_blocks ) as number_unsorted_blocks,
        sum( base_241129.ttables_blocks_rows.number_total_blocks    ) as number_total_blocks,
        sum( base_241129.ttables_blocks_rows.number_sorted_rows     ) as number_sorted_rows,
        sum( base_241129.ttables_blocks_rows.number_sorted_rows     ) as number_desorted_rows,
        sum( base_241129.ttables_blocks_rows.number_unsorted_rows   ) as number_unsorted_rows,
        sum( base_241129.ttables_blocks_rows.number_total_rows      ) as number_total_rows
      from
        base_241129.ttables_blocks_rows
      group by
        schema_id
    )

    select
      lf_241129.sschemas.schema_id::int4,
      lf_241129.sschemas.schema_name::varchar,
      lf_241129.sschemas.user_id::int4,
      lf_241129.sschemas.user_name::varchar,
      lf_241129.sschemas.number_tables::int4,
      lf_241129.sschemas.number_normal_views::int4,
      lf_241129.sschemas.number_late_binding_views::int4,
      lf_241129.sschemas.number_materialized_views::int4,
      lf_241129.sschemas.number_functions::int4,
      lf_241129.sschemas.number_procedures::int4,
      cte_schemas_blocks_rows.number_sorted_blocks::int8,
      cte_schemas_blocks_rows.number_desorted_blocks::int8,
      cte_schemas_blocks_rows.number_unsorted_blocks::int8,
      cte_schemas_blocks_rows.number_total_blocks::int8,
      cte_schemas_blocks_rows.number_sorted_rows::int8,
      cte_schemas_blocks_rows.number_desorted_rows::int8,
      cte_schemas_blocks_rows.number_unsorted_rows::int8,
      cte_schemas_blocks_rows.number_total_rows::int8
    from
                lf_241129.sschemas
      left join cte_schemas_blocks_rows using ( schema_id );


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_users_by_all
as
  with
    cte_queries as
    (
      select
        user_id,
        query_wlm_start_ts   - query_start_ts        as leadin_duration,
        query_queue_start_ts - query_wlm_start_ts    as wlm_leadin_duration,
        query_queue_end_ts   - query_queue_start_ts  as query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration   as query_exec_duration,
        query_wlm_end_ts     - query_compile_exec_end_ts as wlm_leadout_duration,
        query_end_ts         - query_wlm_end_ts          as leadout_duration,
        query_total_duration,
        query_total_duration - leadin_duration - wlm_leadin_duration - query_queue_duration + query_compile_duration - query_exec_duration - wlm_leadout_duration - leadout_duration as unaccounted_duration
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query         using ( query_id, user_id )
        left join base_241129.queries_compilation using ( query_id, user_id )
      where
            query_abort_flag      = false
        and query_csc_flag          = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts           <= current_timestamp
        and query_start_ts         >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        user_id,

        min( leadin_duration ) over ( partition by user_id )                              as min_leadin_duration,
        max( leadin_duration ) over ( partition by user_id )                              as max_leadin_duration,
        median( extract(epoch from (leadin_duration)) ) over ( partition by user_id )     as med_leadin_duration,
        avg( leadin_duration ) over ( partition by user_id )                              as avg_leadin_duration,
        stddev_pop( extract(epoch from (leadin_duration)) ) over ( partition by user_id ) as std_leadin_duration,
        sum( leadin_duration ) over ( partition by user_id )                              as sum_leadin_duration,

        min( wlm_leadin_duration ) over ( partition by user_id )                              as min_wlm_leadin_duration,
        max( wlm_leadin_duration ) over ( partition by user_id )                              as max_wlm_leadin_duration,
        median( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id )     as med_wlm_leadin_duration,
        avg( wlm_leadin_duration ) over ( partition by user_id )                              as avg_wlm_leadin_duration,
        stddev_pop( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id ) as std_wlm_leadin_duration,
        sum( wlm_leadin_duration ) over ( partition by user_id )                              as sum_wlm_leadin_duration,

        min( query_queue_duration ) over ( partition by user_id )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by user_id )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by user_id )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by user_id )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by user_id ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by user_id )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by user_id )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by user_id )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by user_id )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by user_id )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by user_id ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by user_id )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by user_id )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by user_id )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by user_id )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by user_id )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by user_id ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by user_id )                            as sum_query_exec_duration,

        min( wlm_leadout_duration ) over ( partition by user_id )                              as min_wlm_leadout_duration,
        max( wlm_leadout_duration ) over ( partition by user_id )                              as max_wlm_leadout_duration,
        median( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id )     as med_wlm_leadout_duration,
        avg( wlm_leadout_duration ) over ( partition by user_id )                              as avg_wlm_leadout_duration,
        stddev_pop( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id ) as std_wlm_leadout_duration,
        sum( wlm_leadout_duration ) over ( partition by user_id )                              as sum_wlm_leadout_duration,

        min( leadout_duration ) over ( partition by user_id )                              as min_leadout_duration,
        max( leadout_duration ) over ( partition by user_id )                              as max_leadout_duration,
        median( extract(epoch from (leadout_duration)) ) over ( partition by user_id )     as med_leadout_duration,
        avg( leadout_duration ) over ( partition by user_id )                              as avg_leadout_duration,
        stddev_pop( extract(epoch from (leadout_duration)) ) over ( partition by user_id ) as std_leadout_duration,
        sum( leadout_duration ) over ( partition by user_id )                              as sum_leadout_duration,

        min( query_total_duration ) over ( partition by user_id )                              as min_query_total_duration,
        max( query_total_duration ) over ( partition by user_id )                              as max_query_total_duration,
        median( extract(epoch from (query_total_duration)) ) over ( partition by user_id )     as med_query_total_duration,
        avg( query_total_duration ) over ( partition by user_id )                              as avg_query_total_duration,
        stddev_pop( extract(epoch from (query_total_duration)) ) over ( partition by user_id ) as std_query_total_duration,
        sum( query_total_duration ) over ( partition by user_id )                              as sum_query_total_duration,

        min( unaccounted_duration ) over ( partition by user_id )                              as min_unaccounted_duration,
        max( unaccounted_duration ) over ( partition by user_id )                              as max_unaccounted_duration,
        median( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id )     as med_unaccounted_duration,
        avg( unaccounted_duration ) over ( partition by user_id )                              as avg_unaccounted_duration,
        stddev_pop( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id ) as std_unaccounted_duration,
        sum( unaccounted_duration ) over ( partition by user_id )                              as sum_unaccounted_duration,

        count(*) over ( partition by user_id ) as number_queries
      from
        cte_queries
    )

  select
    cte_queries_stats.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,

    min_leadin_duration::interval,
    max_leadin_duration::interval,
    med_leadin_duration::float4,
    avg_leadin_duration::interval,
    std_leadin_duration::float4,
    sum_leadin_duration::interval,

    min_wlm_leadin_duration::interval,
    max_wlm_leadin_duration::interval,
    med_wlm_leadin_duration::float4,
    avg_wlm_leadin_duration::interval,
    std_wlm_leadin_duration::float4,
    sum_wlm_leadin_duration::interval,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    min_wlm_leadout_duration::interval,
    max_wlm_leadout_duration::interval,
    med_wlm_leadout_duration::float4,
    avg_wlm_leadout_duration::interval,
    std_wlm_leadout_duration::float4,
    sum_wlm_leadout_duration::interval,

    min_leadout_duration::interval,
    max_leadout_duration::interval,
    med_leadout_duration::float4,
    avg_leadout_duration::interval,
    std_leadout_duration::float4,
    sum_leadout_duration::interval,

    min_query_total_duration::interval,
    max_query_total_duration::interval,
    med_query_total_duration::float4,
    avg_query_total_duration::interval,
    std_query_total_duration::float4,
    sum_query_total_duration::interval,

    min_unaccounted_duration::interval,
    max_unaccounted_duration::interval,
    med_unaccounted_duration::float4,
    avg_unaccounted_duration::interval,
    std_unaccounted_duration::float4,
    sum_unaccounted_duration::interval,

    number_queries::int4
  from
              cte_queries_stats
    left join base_241129.fixed_pg_user using ( user_id )
  group by
    cte_queries_stats.user_id,
    user_name,
    min_leadin_duration,
    max_leadin_duration,
    med_leadin_duration,
    avg_leadin_duration,
    std_leadin_duration,
    sum_leadin_duration,
    min_wlm_leadin_duration,
    max_wlm_leadin_duration,
    med_wlm_leadin_duration,
    avg_wlm_leadin_duration,
    std_wlm_leadin_duration,
    sum_wlm_leadin_duration,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    min_wlm_leadout_duration,
    max_wlm_leadout_duration,
    med_wlm_leadout_duration,
    avg_wlm_leadout_duration,
    std_wlm_leadout_duration,
    sum_wlm_leadout_duration,
    min_leadout_duration,
    max_leadout_duration,
    med_leadout_duration,
    avg_leadout_duration,
    std_leadout_duration,
    sum_leadout_duration,
    min_query_total_duration,
    max_query_total_duration,
    med_query_total_duration,
    avg_query_total_duration,
    std_query_total_duration,
    sum_query_total_duration,
    min_unaccounted_duration,
    max_unaccounted_duration,
    med_unaccounted_duration,
    avg_unaccounted_duration,
    std_unaccounted_duration,
    sum_unaccounted_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_users_by_all
as
  with
    cte_hr_formatting as
    (
      select
        user_name,

        extract(epoch from min_leadin_duration)::numeric(18,2)::varchar as hr_min_leadin_duration,
        extract(epoch from max_leadin_duration)::numeric(18,2)::varchar as hr_max_leadin_duration,
        med_leadin_duration::numeric(18,2)::varchar                     as hr_med_leadin_duration,
        extract(epoch from avg_leadin_duration)::numeric(18,2)::varchar as hr_avg_leadin_duration,
        std_leadin_duration::numeric(18,2)::varchar                     as hr_std_leadin_duration,
        extract(epoch from sum_leadin_duration)::numeric(18,2)::varchar as hr_sum_leadin_duration,

        extract(epoch from min_wlm_leadin_duration)::numeric(18,2)::varchar as hr_min_wlm_leadin_duration,
        extract(epoch from max_wlm_leadin_duration)::numeric(18,2)::varchar as hr_max_wlm_leadin_duration,
        med_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadin_duration,
        extract(epoch from avg_wlm_leadin_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadin_duration,
        std_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadin_duration,
        extract(epoch from sum_wlm_leadin_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadin_duration,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        extract(epoch from min_wlm_leadout_duration)::numeric(18,2)::varchar as hr_min_wlm_leadout_duration,
        extract(epoch from max_wlm_leadout_duration)::numeric(18,2)::varchar as hr_max_wlm_leadout_duration,
        med_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadout_duration,
        extract(epoch from avg_wlm_leadout_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadout_duration,
        std_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadout_duration,
        extract(epoch from sum_wlm_leadout_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadout_duration,

        extract(epoch from min_leadout_duration)::numeric(18,2)::varchar as hr_min_leadout_duration,
        extract(epoch from max_leadout_duration)::numeric(18,2)::varchar as hr_max_leadout_duration,
        med_leadout_duration::numeric(18,2)::varchar                     as hr_med_leadout_duration,
        extract(epoch from avg_leadout_duration)::numeric(18,2)::varchar as hr_avg_leadout_duration,
        std_leadout_duration::numeric(18,2)::varchar                     as hr_std_leadout_duration,
        extract(epoch from sum_leadout_duration)::numeric(18,2)::varchar as hr_sum_leadout_duration,

        extract(epoch from min_query_total_duration)::numeric(18,2)::varchar as hr_min_query_total_duration,
        extract(epoch from max_query_total_duration)::numeric(18,2)::varchar as hr_max_query_total_duration,
        med_query_total_duration::numeric(18,2)::varchar                     as hr_med_query_total_duration,
        extract(epoch from avg_query_total_duration)::numeric(18,2)::varchar as hr_avg_query_total_duration,
        std_query_total_duration::numeric(18,2)::varchar                     as hr_std_query_total_duration,
        extract(epoch from sum_query_total_duration)::numeric(18,2)::varchar as hr_sum_query_total_duration,

        extract(epoch from min_unaccounted_duration)::numeric(18,2)::varchar as hr_min_unaccounted_duration,
        extract(epoch from max_unaccounted_duration)::numeric(18,2)::varchar as hr_max_unaccounted_duration,
        med_unaccounted_duration::numeric(18,2)::varchar                     as hr_med_unaccounted_duration,
        extract(epoch from avg_unaccounted_duration)::numeric(18,2)::varchar as hr_avg_unaccounted_duration,
        std_unaccounted_duration::numeric(18,2)::varchar                     as hr_std_unaccounted_duration,
        extract(epoch from sum_unaccounted_duration)::numeric(18,2)::varchar as hr_sum_unaccounted_duration,

        max( len(hr_min_leadin_duration) ) over () as max_len_hr_min_leadin_duration,
        max( len(hr_max_leadin_duration) ) over () as max_len_hr_max_leadin_duration,
        max( len(hr_med_leadin_duration) ) over () as max_len_hr_med_leadin_duration,
        max( len(hr_avg_leadin_duration) ) over () as max_len_hr_avg_leadin_duration,
        max( len(hr_std_leadin_duration) ) over () as max_len_hr_std_leadin_duration,
        max( len(hr_sum_leadin_duration) ) over () as max_len_hr_sum_leadin_duration,

        max( len(hr_min_wlm_leadin_duration) ) over () as max_len_hr_min_wlm_leadin_duration,
        max( len(hr_max_wlm_leadin_duration) ) over () as max_len_hr_max_wlm_leadin_duration,
        max( len(hr_med_wlm_leadin_duration) ) over () as max_len_hr_med_wlm_leadin_duration,
        max( len(hr_avg_wlm_leadin_duration) ) over () as max_len_hr_avg_wlm_leadin_duration,
        max( len(hr_std_wlm_leadin_duration) ) over () as max_len_hr_std_wlm_leadin_duration,
        max( len(hr_sum_wlm_leadin_duration) ) over () as max_len_hr_sum_wlm_leadin_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        max( len(hr_min_wlm_leadout_duration) ) over () as max_len_hr_min_wlm_leadout_duration,
        max( len(hr_max_wlm_leadout_duration) ) over () as max_len_hr_max_wlm_leadout_duration,
        max( len(hr_med_wlm_leadout_duration) ) over () as max_len_hr_med_wlm_leadout_duration,
        max( len(hr_avg_wlm_leadout_duration) ) over () as max_len_hr_avg_wlm_leadout_duration,
        max( len(hr_std_wlm_leadout_duration) ) over () as max_len_hr_std_wlm_leadout_duration,
        max( len(hr_sum_wlm_leadout_duration) ) over () as max_len_hr_sum_wlm_leadout_duration,

        max( len(hr_min_leadout_duration) ) over () as max_len_hr_min_leadout_duration,
        max( len(hr_max_leadout_duration) ) over () as max_len_hr_max_leadout_duration,
        max( len(hr_med_leadout_duration) ) over () as max_len_hr_med_leadout_duration,
        max( len(hr_avg_leadout_duration) ) over () as max_len_hr_avg_leadout_duration,
        max( len(hr_std_leadout_duration) ) over () as max_len_hr_std_leadout_duration,
        max( len(hr_sum_leadout_duration) ) over () as max_len_hr_sum_leadout_duration,

        max( len(hr_min_query_total_duration) ) over () as max_len_hr_min_query_total_duration,
        max( len(hr_max_query_total_duration) ) over () as max_len_hr_max_query_total_duration,
        max( len(hr_med_query_total_duration) ) over () as max_len_hr_med_query_total_duration,
        max( len(hr_avg_query_total_duration) ) over () as max_len_hr_avg_query_total_duration,
        max( len(hr_std_query_total_duration) ) over () as max_len_hr_std_query_total_duration,
        max( len(hr_sum_query_total_duration) ) over () as max_len_hr_sum_query_total_duration,

        max( len(hr_min_unaccounted_duration) ) over () as max_len_hr_min_unaccounted_duration,
        max( len(hr_max_unaccounted_duration) ) over () as max_len_hr_max_unaccounted_duration,
        max( len(hr_med_unaccounted_duration) ) over () as max_len_hr_med_unaccounted_duration,
        max( len(hr_avg_unaccounted_duration) ) over () as max_len_hr_avg_unaccounted_duration,
        max( len(hr_std_unaccounted_duration) ) over () as max_len_hr_std_unaccounted_duration,
        max( len(hr_sum_unaccounted_duration) ) over () as max_len_hr_sum_unaccounted_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_users_by_all
    )

  select
    user_name::varchar as uuser,

    ( lpad( hr_min_leadin_duration, max_len_hr_min_leadin_duration ) || ' / ' ||
      lpad( hr_max_leadin_duration, max_len_hr_max_leadin_duration ) || ' / ' ||
      lpad( hr_med_leadin_duration, max_len_hr_med_leadin_duration ) || ' / ' ||
      lpad( hr_avg_leadin_duration, max_len_hr_avg_leadin_duration ) || ' / ' ||
      lpad( hr_std_leadin_duration, max_len_hr_std_leadin_duration ) || ' / ' ||
      lpad( hr_sum_leadin_duration, max_len_hr_sum_leadin_duration ) )::varchar as leadin,

    ( lpad( hr_min_wlm_leadin_duration, max_len_hr_min_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadin_duration, max_len_hr_max_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadin_duration, max_len_hr_med_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadin_duration, max_len_hr_avg_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadin_duration, max_len_hr_std_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_sum_wlm_leadin_duration, max_len_hr_sum_wlm_leadin_duration ) )::varchar as wlm_leadin,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    ( lpad( hr_min_wlm_leadout_duration, max_len_hr_min_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadout_duration, max_len_hr_max_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadout_duration, max_len_hr_med_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadout_duration, max_len_hr_avg_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadout_duration, max_len_hr_std_wlm_leadout_duration ) || ' / ' || 
      lpad( hr_sum_wlm_leadout_duration, max_len_hr_sum_wlm_leadout_duration ) )::varchar as wlm_leadout,

    ( lpad( hr_min_leadout_duration, max_len_hr_min_leadout_duration ) || ' / ' ||
      lpad( hr_max_leadout_duration, max_len_hr_max_leadout_duration ) || ' / ' ||
      lpad( hr_med_leadout_duration, max_len_hr_med_leadout_duration ) || ' / ' ||
      lpad( hr_avg_leadout_duration, max_len_hr_avg_leadout_duration ) || ' / ' ||
      lpad( hr_std_leadout_duration, max_len_hr_std_leadout_duration ) || ' / ' || 
      lpad( hr_sum_leadout_duration, max_len_hr_sum_leadout_duration ) )::varchar as leadout,

    ( lpad( hr_min_query_total_duration, max_len_hr_min_query_total_duration ) || ' / ' ||
      lpad( hr_max_query_total_duration, max_len_hr_max_query_total_duration ) || ' / ' ||
      lpad( hr_med_query_total_duration, max_len_hr_med_query_total_duration ) || ' / ' ||
      lpad( hr_avg_query_total_duration, max_len_hr_avg_query_total_duration ) || ' / ' ||
      lpad( hr_std_query_total_duration, max_len_hr_std_query_total_duration ) || ' / ' || 
      lpad( hr_sum_query_total_duration, max_len_hr_sum_query_total_duration ) )::varchar as total,

    ( lpad( hr_min_unaccounted_duration, max_len_hr_min_unaccounted_duration ) || ' / ' ||
      lpad( hr_max_unaccounted_duration, max_len_hr_max_unaccounted_duration ) || ' / ' ||
      lpad( hr_med_unaccounted_duration, max_len_hr_med_unaccounted_duration ) || ' / ' ||
      lpad( hr_avg_unaccounted_duration, max_len_hr_avg_unaccounted_duration ) || ' / ' ||
      lpad( hr_std_unaccounted_duration, max_len_hr_std_unaccounted_duration ) || ' / ' || 
      lpad( hr_sum_unaccounted_duration, max_len_hr_sum_unaccounted_duration ) )::varchar as unact,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_users_by_second
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_users_by_second
    )

  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_all_by_day
as
  select
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_days
    left join base_241129.fixed_stl_query on date_trunc( 'day', query_start_ts ) = base_241129.generate_days.start_ts
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  base_241129.fixed_pg_get_late_binding_view_cols
as
  with
    cte_stage_1 as
    (
      select
        pg_class.oid as table_id,
        lvb_columns.column_ordinal,
        lvb_columns.column_name,
        case
          -- MG2 : check for 'character varying' first or 'char' picks it up
          when left(lvb_columns.column_full_data_type_name,17) = 'character varying'   then 'varchar'
          when left(lvb_columns.column_full_data_type_name, 9) = 'character'           then 'char'
          when lvb_columns.column_full_data_type_name = 'boolean'                      then 'bool'
          when lvb_columns.column_full_data_type_name = 'date'                         then 'date'
          when lvb_columns.column_full_data_type_name = 'real'                         then 'float4'
          when lvb_columns.column_full_data_type_name = 'double precision'             then 'float8'
          when lvb_columns.column_full_data_type_name = 'geography'                    then 'geography'
          when lvb_columns.column_full_data_type_name = 'geometry'                     then 'geometry'
          when lvb_columns.column_full_data_type_name = 'hllsketch'                    then 'hllsketch'
          when lvb_columns.column_full_data_type_name = 'smallint'                     then 'int2'
          when lvb_columns.column_full_data_type_name = 'super'                        then 'super'
          when lvb_columns.column_full_data_type_name = 'integer'                      then 'int4'
          when lvb_columns.column_full_data_type_name = 'bigint'                       then 'int8'
          when left(lvb_columns.column_full_data_type_name,7) = 'numeric'              then 'numeric'
          when lvb_columns.column_full_data_type_name = 'time without time zone'       then 'time'
          when lvb_columns.column_full_data_type_name = 'timestamp without time zone'  then 'timestamp'
          when lvb_columns.column_full_data_type_name = 'timestamp with time zone'     then 'timestamptz'
          when lvb_columns.column_full_data_type_name = 'time with time zone'          then 'timetz'
        end         as column_base_data_type_name,
        pg_type.oid as column_base_data_type_id,
        case
          when column_base_data_type_name = 'char'    then substring( lvb_columns.column_full_data_type_name, 11, len(lvb_columns.column_full_data_type_name)-11 )::int4
          when column_base_data_type_name = 'varchar' then substring( lvb_columns.column_full_data_type_name, 19, len(lvb_columns.column_full_data_type_name)-19 )::int4
                                                      else null
        end                                  as column_octet_length,
        case
          when column_base_data_type_name = 'numeric' then substring( lvb_columns.column_full_data_type_name, 9, charindex(',',lvb_columns.column_full_data_type_name) - 9 )::int2
                                                      else null
        end                                  as column_precision,
        case
          when column_base_data_type_name = 'numeric' then substring( lvb_columns.column_full_data_type_name, charindex(',',lvb_columns.column_full_data_type_name)+1, len(lvb_columns.column_full_data_type_name) - charindex(',',lvb_columns.column_full_data_type_name)-1 )::int2
                                                      else null
        end                                  as column_scale
      from
        pg_get_late_binding_view_cols() lvb_columns( schema_name name, view_name name, column_name name, column_full_data_type_name varchar, column_ordinal int ),
        pg_class,
        pg_namespace,
        pg_type
      where
            pg_namespace.nspname = lvb_columns.schema_name
        and pg_namespace.oid     = pg_class.relnamespace
        and pg_class.relname     = lvb_columns.view_name
        and pg_class.relkind     = 'v'
        and pg_class.relnatts    = 0
        and pg_type.typname      = column_base_data_type_name
    ),

    cte_stage_2 as
    (
      select
        column_ordinal           as attnum,
        column_name              as attname,
        table_id                 as attrelid,
        case
          when column_base_data_type_name = 'char'    then column_octet_length + 4
          when column_base_data_type_name = 'varchar' then column_octet_length + 4
          when column_base_data_type_name = 'numeric' then ( (column_precision << 16) | (column_scale & 65535) ) + 4
                                                      else null -- MG2 : not used with other data types, or shown in the view, so NULL is fine
        end                      as atttypmod,
        column_base_data_type_id as atttypid
      from
        cte_stage_1
    )

  select
    attnum::int2,
    attname::varchar,
    attrelid::int4,
    atttypmod::int4,
    atttypid::int4
  from
    cte_stage_2;


create or replace procedure
  ro.sp_grant_version_to_group( version in varchar(6), group_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'grant usage on schema base_' || version || ' to group ' || group_name;
      execute sql;

      sql := 'grant usage on schema lf_' || version || ' to group ' || group_name;
      execute sql;

      sql := 'grant usage on schema sf_' || version || ' to group ' || group_name;
      execute sql;

      sql := 'grant select on all tables in schema base_' || version || ' to group ' || group_name;
      execute sql;

      sql := 'grant select on all tables in schema lf_' || version || ' to group ' || group_name;
      execute sql;

      sql := 'grant select on all tables in schema sf_' || version || ' to group ' || group_name;
      execute sql;

      sql := 'grant execute on all functions  in schema base_' || version || ' to group ' || group_name;
      execute sql;

      sql := 'grant execute on all procedures in schema lf_' || version || ' to group ' || group_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.queries_text_on_both_fast
as
  with
    -- MG2 : make leader queries look like worker queries so we can union

    cte_query_text_leader as
    (
      select
        session_id,
        transaction_id,
        null::int4 as query_id,
        query_start_ts,
        query_text_order,
        query_text
      from
        lf_241129.queries_text_for_leader
    ),

    -- MG2 : make worker queries look like leader queries so we can union

    cte_query_text_worker_leader as
    (
      select
        session_id,
        transaction_id,
        query_id,
        query_start_ts,
        0::int4 as query_text_order,
        query_text
      from
        lf_241129.queries_text_on_workers_fast
    ),

    cte_query_text_all_fast as
    (
      select * from cte_query_text_leader
      union all
      select * from cte_query_text_worker_leader
    )

  select
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    query_start_ts::timestamp,
    query_text_order::int4,
    query_text::varchar
  from
    cte_query_text_all_fast;


create or replace view
  sf_241129.queries_text_on_both_fast
as
  select
    session_id::int4       as pid,
    transaction_id::int8   as xid,
    query_id::int4         as qid,
    query_start_ts::timestamp as start_ts,
    query_text_order::int4 as n,
    trim( chr(10) || ' ' from query_text )::varchar as ttext
  from
    lf_241129.queries_text_on_both_fast
  order by
    session_id,
    transaction_id,
    -- MG2 : not query_id, as leader node queries have this set to NULL
    --       we order only by query start time, which we get for both leader and worker
    query_start_ts,
    query_text_order;


create or replace view
  lf_241129.vacuum_stats_for_all_by_all
as
  with
    cte_xids as 
    (
      select
        sum( num_new_blocks      ) as sum_number_new_blocks,
        sum( num_blocks_replaced ) as sum_number_replaced_blocks,

        sum( num_new_rows        ) as sum_number_new_rows,
        sum( num_deleted_rows    ) as sum_number_deleted_rows,

        count(*)                   as number_qids,

        min( query_start_ts ) as min_query_start_ts,
        max( query_end_ts   ) as max_query_end_ts,

        max_query_end_ts - min_query_start_ts as duration
      from
             base_241129.fixed_stl_vacuum_detail
        join base_241129.fixed_stl_query using( query_id, transaction_id )
      where
            -- MG2 : procably we should include aborted - need a general approcch to this
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_end_ts     <  current_timestamp
      group by
        transaction_id
      having
        -- MG2 : ignore xids in partial first start_ts
        min_query_start_ts >= current_date - interval '6 day'
    ),

    cte_xids_stats as
    (
      select
        min( sum_number_new_blocks )        over () as min_sum_number_new_blocks,
        max( sum_number_new_blocks )        over () as max_sum_number_new_blocks,
        median( sum_number_new_blocks )     over () as med_sum_number_new_blocks,
        avg( sum_number_new_blocks )        over () as avg_sum_number_new_blocks,
        stddev_pop( sum_number_new_blocks ) over () as std_sum_number_new_blocks,
        sum( sum_number_new_blocks )        over () as sum_sum_number_new_blocks,

        min( sum_number_replaced_blocks )        over () as min_sum_number_replaced_blocks,
        max( sum_number_replaced_blocks )        over () as max_sum_number_replaced_blocks,
        median( sum_number_replaced_blocks )     over () as med_sum_number_replaced_blocks,
        avg( sum_number_replaced_blocks )        over () as avg_sum_number_replaced_blocks,
        stddev_pop( sum_number_replaced_blocks ) over () as std_sum_number_replaced_blocks,
        sum( sum_number_replaced_blocks )        over () as sum_sum_number_replaced_blocks,

        min( sum_number_new_rows )        over () as min_sum_number_new_rows,
        max( sum_number_new_rows )        over () as max_sum_number_new_rows,
        median( sum_number_new_rows )     over () as med_sum_number_new_rows,
        avg( sum_number_new_rows )        over () as avg_sum_number_new_rows,
        stddev_pop( sum_number_new_rows ) over () as std_sum_number_new_rows,
        sum( sum_number_new_rows )        over () as sum_sum_number_new_rows,

        min( sum_number_deleted_rows )        over () as min_sum_number_deleted_rows,
        max( sum_number_deleted_rows )        over () as max_sum_number_deleted_rows,
        median( sum_number_deleted_rows )     over () as med_sum_number_deleted_rows,
        avg( sum_number_deleted_rows )        over () as avg_sum_number_deleted_rows,
        stddev_pop( sum_number_deleted_rows ) over () as std_sum_number_deleted_rows,
        sum( sum_number_deleted_rows )        over () as sum_sum_number_deleted_rows,

        min( duration ) over ()                              as min_duration,
        max( duration ) over ()                              as max_duration,
        median( extract(epoch from (duration)) ) over ()     as med_duration,
        avg( duration ) over ()                              as avg_duration,
        stddev_pop( extract(epoch from (duration)) ) over () as std_duration,
        sum( duration ) over ()                              as sum_duration,

        sum( number_qids ) over () as number_qids,
        count(*) over ()           as number_xids
      from
        cte_xids
    )

  select
    min_sum_number_new_blocks::int8,
    max_sum_number_new_blocks::int8,
    med_sum_number_new_blocks::int8,
    avg_sum_number_new_blocks::float4,
    std_sum_number_new_blocks::float4,
    sum_sum_number_new_blocks::float4,

    min_sum_number_replaced_blocks::int8,
    max_sum_number_replaced_blocks::int8,
    med_sum_number_replaced_blocks::int8,
    avg_sum_number_replaced_blocks::float4,
    std_sum_number_replaced_blocks::float4,
    sum_sum_number_replaced_blocks::float4,

    min_sum_number_new_rows::int8,
    max_sum_number_new_rows::int8,
    med_sum_number_new_rows::int8,
    avg_sum_number_new_rows::float4,
    std_sum_number_new_rows::float4,
    sum_sum_number_new_rows::float4,

    min_sum_number_deleted_rows::int8,
    max_sum_number_deleted_rows::int8,
    med_sum_number_deleted_rows::int8,
    avg_sum_number_deleted_rows::float4,
    std_sum_number_deleted_rows::float4,
    sum_sum_number_deleted_rows::float4,

    min_duration::interval,
    max_duration::interval,
    med_duration::float4,
    avg_duration::interval,
    std_duration::float4,
    sum_duration::interval,

    number_qids::int8,
    number_xids::int8
  from
    cte_xids_stats
  group by
    min_sum_number_new_blocks,
    max_sum_number_new_blocks,
    med_sum_number_new_blocks,
    avg_sum_number_new_blocks,
    std_sum_number_new_blocks,
    sum_sum_number_new_blocks,
    min_sum_number_replaced_blocks,
    max_sum_number_replaced_blocks,
    med_sum_number_replaced_blocks,
    avg_sum_number_replaced_blocks,
    std_sum_number_replaced_blocks,
    sum_sum_number_replaced_blocks,
    min_sum_number_new_rows,
    max_sum_number_new_rows,
    med_sum_number_new_rows,
    avg_sum_number_new_rows,
    std_sum_number_new_rows,
    sum_sum_number_new_rows,
    min_sum_number_deleted_rows,
    max_sum_number_deleted_rows,
    med_sum_number_deleted_rows,
    avg_sum_number_deleted_rows,
    std_sum_number_deleted_rows,
    sum_sum_number_deleted_rows,
    min_duration,
    max_duration,
    med_duration,
    avg_duration,
    std_duration,
    sum_duration,
    number_qids,
    number_xids;


create or replace view
  sf_241129.vacuum_stats_for_all_by_all
as
  with
    cte_hr_formatting as
    (
      select
        base_241129.hr_number( min_sum_number_new_blocks ) as hr_min_sum_number_new_blocks,
        base_241129.hr_number( max_sum_number_new_blocks ) as hr_max_sum_number_new_blocks,
        base_241129.hr_number( med_sum_number_new_blocks ) as hr_med_sum_number_new_blocks,
        base_241129.hr_number( avg_sum_number_new_blocks ) as hr_avg_sum_number_new_blocks,
        base_241129.hr_number( std_sum_number_new_blocks ) as hr_std_sum_number_new_blocks,
        base_241129.hr_number( sum_sum_number_new_blocks ) as hr_sum_sum_number_new_blocks,

        base_241129.hr_number( min_sum_number_replaced_blocks ) as hr_min_sum_number_replaced_blocks,
        base_241129.hr_number( max_sum_number_replaced_blocks ) as hr_max_sum_number_replaced_blocks,
        base_241129.hr_number( med_sum_number_replaced_blocks ) as hr_med_sum_number_replaced_blocks,
        base_241129.hr_number( avg_sum_number_replaced_blocks ) as hr_avg_sum_number_replaced_blocks,
        base_241129.hr_number( std_sum_number_replaced_blocks ) as hr_std_sum_number_replaced_blocks,
        base_241129.hr_number( sum_sum_number_replaced_blocks ) as hr_sum_sum_number_replaced_blocks,

        base_241129.hr_number( min_sum_number_new_rows ) as hr_min_sum_number_new_rows,
        base_241129.hr_number( max_sum_number_new_rows ) as hr_max_sum_number_new_rows,
        base_241129.hr_number( med_sum_number_new_rows ) as hr_med_sum_number_new_rows,
        base_241129.hr_number( avg_sum_number_new_rows ) as hr_avg_sum_number_new_rows,
        base_241129.hr_number( std_sum_number_new_rows ) as hr_std_sum_number_new_rows,
        base_241129.hr_number( sum_sum_number_new_rows ) as hr_sum_sum_number_new_rows,

        base_241129.hr_number( min_sum_number_deleted_rows ) as hr_min_sum_number_deleted_rows,
        base_241129.hr_number( max_sum_number_deleted_rows ) as hr_max_sum_number_deleted_rows,
        base_241129.hr_number( med_sum_number_deleted_rows ) as hr_med_sum_number_deleted_rows,
        base_241129.hr_number( avg_sum_number_deleted_rows ) as hr_avg_sum_number_deleted_rows,
        base_241129.hr_number( std_sum_number_deleted_rows ) as hr_std_sum_number_deleted_rows,
        base_241129.hr_number( sum_sum_number_deleted_rows ) as hr_sum_sum_number_deleted_rows,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_sum_number_new_blocks) ) over () as max_len_hr_min_sum_number_new_blocks,
        max( len(hr_max_sum_number_new_blocks) ) over () as max_len_hr_max_sum_number_new_blocks,
        max( len(hr_med_sum_number_new_blocks) ) over () as max_len_hr_med_sum_number_new_blocks,
        max( len(hr_avg_sum_number_new_blocks) ) over () as max_len_hr_avg_sum_number_new_blocks,
        max( len(hr_std_sum_number_new_blocks) ) over () as max_len_hr_std_sum_number_new_blocks,
        max( len(hr_sum_sum_number_new_blocks) ) over () as max_len_hr_sum_sum_number_new_blocks,

        max( len(hr_min_sum_number_replaced_blocks) ) over () as max_len_hr_min_sum_number_replaced_blocks,
        max( len(hr_max_sum_number_replaced_blocks) ) over () as max_len_hr_max_sum_number_replaced_blocks,
        max( len(hr_med_sum_number_replaced_blocks) ) over () as max_len_hr_med_sum_number_replaced_blocks,
        max( len(hr_avg_sum_number_replaced_blocks) ) over () as max_len_hr_avg_sum_number_replaced_blocks,
        max( len(hr_std_sum_number_replaced_blocks) ) over () as max_len_hr_std_sum_number_replaced_blocks,
        max( len(hr_sum_sum_number_replaced_blocks) ) over () as max_len_hr_sum_sum_number_replaced_blocks,

        max( len(hr_min_sum_number_new_rows) ) over () as max_len_hr_min_sum_number_new_rows,
        max( len(hr_max_sum_number_new_rows) ) over () as max_len_hr_max_sum_number_new_rows,
        max( len(hr_med_sum_number_new_rows) ) over () as max_len_hr_med_sum_number_new_rows,
        max( len(hr_avg_sum_number_new_rows) ) over () as max_len_hr_avg_sum_number_new_rows,
        max( len(hr_std_sum_number_new_rows) ) over () as max_len_hr_std_sum_number_new_rows,
        max( len(hr_sum_sum_number_new_rows) ) over () as max_len_hr_sum_sum_number_new_rows,

        max( len(hr_min_sum_number_deleted_rows) ) over () as max_len_hr_min_sum_number_deleted_rows,
        max( len(hr_max_sum_number_deleted_rows) ) over () as max_len_hr_max_sum_number_deleted_rows,
        max( len(hr_med_sum_number_deleted_rows) ) over () as max_len_hr_med_sum_number_deleted_rows,
        max( len(hr_avg_sum_number_deleted_rows) ) over () as max_len_hr_avg_sum_number_deleted_rows,
        max( len(hr_std_sum_number_deleted_rows) ) over () as max_len_hr_std_sum_number_deleted_rows,
        max( len(hr_sum_sum_number_deleted_rows) ) over () as max_len_hr_sum_sum_number_deleted_rows,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_qids,
        number_xids
      from
        lf_241129.vacuum_stats_for_all_by_all
    )

  select
    ( lpad( hr_min_sum_number_new_blocks, max_len_hr_min_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_new_blocks, max_len_hr_max_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_new_blocks, max_len_hr_med_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_new_blocks, max_len_hr_avg_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_new_blocks, max_len_hr_std_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_new_blocks, max_len_hr_sum_sum_number_new_blocks ) )::varchar as "new blocks",

    ( lpad( hr_min_sum_number_replaced_blocks, max_len_hr_min_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_replaced_blocks, max_len_hr_max_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_replaced_blocks, max_len_hr_med_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_replaced_blocks, max_len_hr_avg_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_replaced_blocks, max_len_hr_std_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_replaced_blocks, max_len_hr_sum_sum_number_replaced_blocks ) )::varchar as "replaced blocks",

    ( lpad( hr_min_sum_number_new_rows, max_len_hr_min_sum_number_new_rows ) || ' / ' ||
      lpad( hr_max_sum_number_new_rows, max_len_hr_max_sum_number_new_rows ) || ' / ' ||
      lpad( hr_med_sum_number_new_rows, max_len_hr_med_sum_number_new_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_new_rows, max_len_hr_avg_sum_number_new_rows ) || ' / ' ||
      lpad( hr_std_sum_number_new_rows, max_len_hr_std_sum_number_new_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_new_rows, max_len_hr_sum_sum_number_new_rows ) )::varchar as "new rows",

    ( lpad( hr_min_sum_number_deleted_rows, max_len_hr_min_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_max_sum_number_deleted_rows, max_len_hr_max_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_med_sum_number_deleted_rows, max_len_hr_med_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_deleted_rows, max_len_hr_avg_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_std_sum_number_deleted_rows, max_len_hr_std_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_deleted_rows, max_len_hr_sum_sum_number_deleted_rows ) )::varchar as "deleted rows",

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as "xid duration",

    number_qids::int8 as qids,
    number_xids::int8 as xids
  from
    cte_hr_formatting;


create or replace view
  base_241129.ttables_columns_basic_info
as
  with
    cte_tables_columns as
    (
      select
        pg_class.relnamespace                                                                                         as schema_id,
        trim( pg_namespace.nspname )                                                                                  as schema_name,
        pg_class.oid                                                                                                  as table_id,
        trim( pg_class.relname )                                                                                      as table_name,
        pg_attribute.attnum                                                                                           as column_ordinal,
        trim( pg_attribute.attname )                                                                                  as column_name,
        pg_attribute.atttypid                                                                                         as column_data_type_id,
        case
          when pg_type.typname in ( 'bpchar', 'char' ) then 'char'
          when left(pg_type.typname, 1) = '_'          then substring(pg_type.typname from 2) || '[]'
                                                       else trim( pg_type.typname )::varchar
        end                                                                                                           as column_base_data_type_name,
        case
          when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod != -1 then pg_attribute.atttypmod-4
          when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod  = -1 then 1
        end                                                                                                           as column_octet_length,
        case
          when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then (pg_attribute.atttypmod-4) & 65535
          when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null -- MG2 : variable length, so unknown
        end                                                                                                           as column_precision,
        case
          when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then (pg_attribute.atttypmod-4) >> 16
          when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null -- MG2 : variable length, so unknown
        end                                                                                                           as column_scale,
        case
          when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
          when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')' || '[]'
          when pg_type.typname = '_numeric'              and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')' || '[]'
          when pg_type.typname = '_numeric'              and pg_attribute.atttypmod  = -1 then 'numeric(variable)[]'
          when pg_type.typname = '_varchar'              and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
          when pg_type.typname = '_varchar'              and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')' || '[]'

          when pg_type.typname in ( 'bpchar', 'char' )   and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
          when pg_type.typname in ( 'bpchar', 'char' )   and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')'
          when pg_type.typname = 'numeric'               and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')'
          when pg_type.typname = 'numeric'               and pg_attribute.atttypmod  = -1 then 'numeric(variable)'
          when pg_type.typname = 'varchar'               and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
          when pg_type.typname = 'varchar'               and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')'

          when pg_type.typname = 'varbyte'               and pg_attribute.atttypmod != -1 then 'varbyte' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
          -- MG2 : not seen pg_attribute.atttypmod = -1 for 'varbyte'

          when left(pg_type.typname, 1) = '_' then substring(pg_type.typname from 2) || '[]'
                                              else pg_type.typname::varchar
        end                                                                                                           as column_full_data_type_name,
        pg_attribute.attencodingtype                                                                                  as column_encoding_id,
        case
          -- MG2 : some encodings have two values; I wonder if this is a bug, and this is really a seven-bit value
          --        I've ordered by attencodingtype, since I suspect it reflects the order of implementation and is interesting to see
          --        I wonder what value 6 was, or would have been!
          when pg_attribute.attencodingtype in (0, 128)  then 'raw'
          when pg_attribute.attencodingtype =  1         then 'bytedict'
          when pg_attribute.attencodingtype =  2         then 'delta'
          when pg_attribute.attencodingtype in (3, 131)  then 'lzo'
          when pg_attribute.attencodingtype =  4         then 'runlength'
          when pg_attribute.attencodingtype =  5         then 'delta32k'
          when pg_attribute.attencodingtype =  7         then 'text255'
          when pg_attribute.attencodingtype =  15        then 'mostly8'
          when pg_attribute.attencodingtype =  16        then 'mostly16'
          when pg_attribute.attencodingtype =  17        then 'mostly32'
          when pg_attribute.attencodingtype =  18        then 'text32k'
          when pg_attribute.attencodingtype in (19, 147) then 'zstd'
          when pg_attribute.attencodingtype in (20, 148) then 'az64'
                                                         else 'unknown ' || '(' || pg_attribute.attencodingtype::varchar || ')'
        end                                                                                                           as column_encoding_name,
        abs( pg_attribute.attsortkeyord )                                                                             as column_sortkey_ordinal,
        pg_attribute.attisdistkey                                                                                     as column_distkey,
        pg_attribute.attnotnull                                                                                       as column_notnull
      from
             pg_class
        join pg_namespace on pg_namespace.oid           = pg_class.relnamespace
        join pg_attribute on pg_attribute.attrelid      = pg_class.oid and
                             pg_attribute.attnum   not in ( -1, -3, -4, -5, -6, -7 )
        join pg_type      on pg_type.oid                = pg_attribute.atttypid
      where
        pg_class.relkind = 'r'
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    column_ordinal::int2,
    column_name::varchar,
    column_data_type_id::int4,
    column_base_data_type_name::varchar,
    column_octet_length::int4,
    column_precision::int2,
    column_scale::int2,
    column_full_data_type_name::varchar,
    column_encoding_id::int4,
    column_encoding_name::varchar,
    column_sortkey_ordinal::int2,
    column_distkey::bool,
    column_notnull::bool
  from
    cte_tables_columns;


create or replace view
  lf_241129.transactions_totals_commits_by_minute
as
  select
    date_trunc( 'minute', batch_leader_startwork )::timestamp as start_ts,
    sum( batch_leader_duration )::interval                     as sum_batch_leader_duration,
    sum( batch_worker_duration )::interval                     as sum_batch_worker_duration,
    sum( batch_exec_duration - batch_leader_duration - batch_worker_duration )::interval as sum_batch_idle_duration,
    sum( batch_exec_duration  )::interval                      as sum_batch_exec_duration
  from
    lf_241129.batches
  group by
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_all_by_day
as
  select
    date_trunc( 'day', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    start_ts;


create or replace view
  sf_241129.analyze_totals_for_all_by_day
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_all_by_day
    )

  select
    start_ts::date,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_node_by_hour
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'hour', scan_start_ts )::timestamp as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal )
  group by
    node_ordinal,
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_tables_by_minute
as
  with
    cte_s1 as
    (
      select
        table_id,
        date_trunc( 'minute', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            database   = current_database()
        and starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4::int4              as table_id,
    trim( pg_class.relname )::varchar     as table_name,

    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_tables_by_day
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    start_ts::timestamp,
    cte_delete.sum_delete_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_scan.sum_scan_rows::int8
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_days
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null;


create or replace view
  lf_241129.views_materialized
as
  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4                    as view_id,
    trim( pg_class.relname )::varchar     as view_name,
    pg_class.relowner::int4               as user_id,
    trim( pg_user.usename )::varchar      as user_name,
    case
      when stv_mv_info.state =   0 then 'none'
      when stv_mv_info.state =   1 then 'none'
      when stv_mv_info.state = 101 then 'dropped column'
      when stv_mv_info.state = 102 then 'changed data type'
      when stv_mv_info.state = 103 then 'renamed table'
      when stv_mv_info.state = 104 then 'renamed column'
      when stv_mv_info.state = 105 then 'renamed schema'
    end::varchar                          as failures,
    case
      when stv_mv_info.autorefresh = 't' then 'automatic'
      when stv_mv_info.autorefresh = 'f' then 'manual'
    end::varchar                          as autorefresh,
    case
      when stv_mv_info.state  = 0                            then 'full'
      when stv_mv_info.state  = 1                            then 'incremental'
      when stv_mv_info.state != 0 and stv_mv_info.state != 1 then 'unavailable'
    end::varchar                          as refresh_type
  from
         stv_mv_info
    join pg_class     on stv_mv_info.name   = pg_class.relname
    join pg_namespace on stv_mv_info.schema    = pg_namespace.nspname and
                         pg_class.relnamespace = pg_namespace.oid
    join pg_user      on pg_class.relowner  = pg_user.usesysid
  where
        stv_mv_info.db_name  = current_database()
    and pg_class.relkind     = 'v'
    and pg_class.relnatts   >  0         -- MG2 : no late-binding views
    and stv_mv_info.name    is not null; -- MG2 : only materialized views


create or replace view
  lf_241129.views_materialized_table
as
  -- MG2 : do not use materialized views, ever
  --       the underlying implementation is no good
  --       and the internal design is an anti-pattern for sorted relational databases
  --       you've been warned

  with
    cte_underlying_tables as
    (
      select
        pg_class_child_class_view.oid         as view_id,
        pg_class_parent_class.oid             as table_id
      from
        ( select classid, objid, refclassid, refobjid from pg_depend where deptype = 'n' group by classid, objid, refclassid, refobjid ) as pg_depend_distinct

             -- MG2 : first, parents
             join pg_class      as pg_class_parent_type       on pg_class_parent_type.oid = pg_depend_distinct.refclassid

        -- MG2 : parents who are in pg_class (and by the where clause will be tables or views)
        left join pg_class      as pg_class_parent_class      on pg_class_parent_type.relname = 'pg_class'  and pg_depend_distinct.refobjid        = pg_class_parent_class.oid
        left join pg_namespace  as pg_namespace_parent_class  on pg_class_parent_type.relname = 'pg_class'  and pg_class_parent_class.relnamespace = pg_namespace_parent_class.oid

              -- MG2 : next, children
             join pg_class      as pg_class_child_type        on pg_class_child_type.oid = pg_depend_distinct.classid

        -- MG2 : children who are in pg_class (and which surprisingly can be both tables or views - turns out not all views go via pg_rewrite)
        left join pg_class      as pg_class_child_class_table_or_view      on pg_class_child_type.relname = 'pg_class' and pg_depend_distinct.objid                   = pg_class_child_class_table_or_view.oid
        left join pg_namespace  as pg_namespace_child_class_table_or_view  on pg_class_child_type.relname = 'pg_class' and pg_namespace_child_class_table_or_view.oid = pg_class_child_class_table_or_view.relnamespace
        left join ( select usesysid, usename from pg_user ) as pg_user_child_class_table_or_view       on pg_class_child_type.relname = 'pg_class' and pg_user_child_class_table_or_view.usesysid = pg_class_child_class_table_or_view.relowner

        -- MG2 : children who are in pg_rewrite (and so are really in pg_class, and are views)
        left join pg_rewrite    as pg_rewrite_child_class          on pg_class_child_type.relname = 'pg_rewrite' and pg_rewrite_child_class.oid         = pg_depend_distinct.objid
        left join pg_class      as pg_class_child_class_view       on pg_class_child_type.relname = 'pg_rewrite' and pg_class_child_class_view.oid      = pg_rewrite_child_class.ev_class
        left join pg_namespace  as pg_namespace_child_class_view   on pg_class_child_type.relname = 'pg_rewrite' and pg_namespace_child_class_view.oid  = pg_class_child_class_view.relnamespace
        left join ( select usesysid, usename from pg_user ) as pg_user_child_class_view        on pg_class_child_type.relname = 'pg_rewrite' and pg_class_child_class_view.relowner = pg_user_child_class_view.usesysid
      where
            pg_class_child_class_view.relkind  = 'v'
        and pg_class_parent_class.relkind      = 'r'
        and pg_class_child_class_view.oid     in ( select view_id from lf_241129.views_materialized )
        and pg_class_parent_class.oid         != pg_class_child_class_view.oid
    )

  select
    lf_241129.views_materialized.schema_id::int4,
    lf_241129.views_materialized.schema_name::varchar,
    lf_241129.views_materialized.view_id::int4,
    lf_241129.views_materialized.view_name::varchar,
    lf_241129.views_materialized.user_id::int4,
    lf_241129.views_materialized.user_name::varchar,
    lf_241129.views_materialized.failures::varchar,
    lf_241129.views_materialized.autorefresh::varchar,
    lf_241129.views_materialized.refresh_type::varchar,
    base_241129.ttables_basic_info_full.table_id::int4,
    base_241129.ttables_basic_info_full.table_name::varchar,
    base_241129.ttables_basic_info_full.table_creation::timestamp,
    base_241129.ttables_basic_info_full.table_age::interval,
    base_241129.ttables_basic_info_full.table_number_columns::int2,
    base_241129.ttables_basic_info_full.table_sort_type::varchar,
    base_241129.ttables_basic_info_full.table_number_sortkeys::int2,
    base_241129.ttables_basic_info_full.table_distribution_type::varchar,
    base_241129.ttables_basic_info_full.table_distribution_auto_flag::bool
  from
         lf_241129.views_materialized
    join cte_underlying_tables          using ( view_id )
    join base_241129.ttables_basic_info_full    on cte_underlying_tables.table_id = base_241129.ttables_basic_info_full.table_id;


create or replace view
  sf_241129.views_materialized_table
as
  select
    schema_name::varchar        as sschema,
    view_name::varchar          as vview,
    user_name::varchar          as uuser,
    (
      case when failures     != 'none'        then 'f' else ' ' end
      ||
      case when autorefresh   = 'automatic'   then 'a' else 'm' end
      ||
      case when refresh_type  = 'unavailable' then 'u' else ' ' end
      ||
      case when refresh_type  = 'full'        then 'f' else 'i' end
    )::varchar                  as status,
    table_name::varchar         as ttable,
    table_age::interval         as age,
    table_number_columns::int2  as cols,
    table_sort_type::varchar    as ssort,
    table_number_sortkeys::int2 as skeys,
    case
      when table_distribution_auto_flag = True  then table_distribution_type || ' (auto)'
      when table_distribution_auto_flag = false then table_distribution_type
    end::varchar                as dist
  from
    lf_241129.views_materialized_table
  order by
    schema_name,
    view_name;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_users_by_day
as
  with
    cte_transactions as
    (
      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        user_id,
        transaction_id,
        date_trunc( 'day', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        user_id,
        transaction_id
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                                      as user_name,
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
    left join pg_user                        on cte_all_transactions.user_id        = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_users_by_day
as
  select
    user_name::varchar                 as uuser,
    start_ts::date,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_users_by_day
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_query
as
  with
    cte_aggr as
    (
      select
        query as query_id,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_bcast as
    (
      select
        query as query_id,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_delete as
    (
      select
        query as query_id,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_dist as
    (
      select
        query as query_id,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_hash as
    (
      select
        query as query_id,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_hashjoin as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_insert as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_limit as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_merge as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_mergejoin as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_nestloop as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_parse as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_project as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_return as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_save as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    -- scan

    cte_sort as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_spatial_join as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_unique as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_window as
    (
      select
        query as query_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    query_id::int4,
    node_type::varchar,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               base_241129.fixed_stl_wlm_query
    cross join cte_node_types
     left join cte_aggr         using ( query_id, node_type )
     left join cte_bcast        using ( query_id, node_type )
     left join cte_delete       using ( query_id, node_type )
     left join cte_dist         using ( query_id, node_type )
     left join cte_hash         using ( query_id, node_type )
     left join cte_hashjoin     using ( query_id, node_type )
     left join cte_insert       using ( query_id, node_type )
     left join cte_limit        using ( query_id, node_type )
     left join cte_merge        using ( query_id, node_type )
     left join cte_mergejoin    using ( query_id, node_type )
     left join cte_nestloop     using ( query_id, node_type )
     left join cte_parse        using ( query_id, node_type )
     left join cte_project      using ( query_id, node_type )
     left join cte_return       using ( query_id, node_type )
     left join cte_save         using ( query_id, node_type )
     left join cte_sort         using ( query_id, node_type )
     left join cte_spatial_join using ( query_id, node_type )
     left join cte_unique       using ( query_id, node_type )
     left join cte_window       using ( query_id, node_type )
  group by
    query_id,
    node_type;


create or replace view
  sf_241129.steps_totals_data_on_both_for_query
as
  select
    query_id::int4 as qid,
    node_type::varchar as node,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_query
  order by
    query_id,
    node_type;


create or replace view
  lf_241129.blocks_rows_totals_for_user
as
  with
    cte_blocks_and_rows_s1 as
    (
      select
        stv_blocklist.tbl                                                                                              as table_id,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      group by
        stv_blocklist.tbl
    ),

    cte_nodes as
    (
      select
        count( distinct node ) as number_nodes
      from
        stv_slices
    ),

    cte_blocks_and_rows_s2 as
    (
      select
        pg_user.usesysid        as user_id,
        trim( pg_user.usename ) as user_name,

        sum( number_sorted_blocks                                                                               ) as number_sorted_blocks,
        sum( case when table_sort_type = 'unsorted' then 0                      else number_unsorted_blocks end ) as number_desorted_blocks,
        sum( case when table_sort_type = 'unsorted' then number_unsorted_blocks else 0                      end ) as number_unsorted_blocks,
        sum( number_total_blocks                                                                                ) as number_total_blocks,

        sum( case when table_distribution_type = 'all' then cte_blocks_and_rows_s1.number_sorted_rows   / cte_nodes.number_nodes else cte_blocks_and_rows_s1.number_sorted_rows end ) as number_sorted_rows,
        sum
        (
          case
            when base_241129.ttables_basic_info_full.table_sort_type  = 'unsorted'                                     then 0
            when base_241129.ttables_basic_info_full.table_sort_type != 'unsorted' and table_distribution_type = 'all' then cte_blocks_and_rows_s1.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                  else cte_blocks_and_rows_s1.number_unsorted_rows
          end
        ) as number_desorted_rows,
        sum
        (
          case
            when base_241129.ttables_basic_info_full.table_sort_type != 'unsorted'                                     then 0
            when base_241129.ttables_basic_info_full.table_sort_type  = 'unsorted' and table_distribution_type = 'all' then cte_blocks_and_rows_s1.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                  else cte_blocks_and_rows_s1.number_unsorted_rows
          end
        ) as number_unsorted_rows,
        sum( case when table_distribution_type = 'all' then cte_blocks_and_rows_s1.number_total_rows    / cte_nodes.number_nodes else cte_blocks_and_rows_s1.number_total_rows end ) as number_total_rows
      from
                   cte_blocks_and_rows_s1
        cross join cte_nodes
         left join base_241129.ttables_basic_info_full using ( table_id )
         left join pg_user                           on base_241129.ttables_basic_info_full.user_id = pg_user.usesysid
      group by
        pg_user.usesysid,
        pg_user.usename
    )

  select
    user_id::int4,
    user_name::varchar,
    number_sorted_blocks::int8,
    number_desorted_blocks::int8,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    number_desorted_rows::int8,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
    cte_blocks_and_rows_s2;


create or replace view
  lf_241129.views_text
as
  select
    pg_class.relnamespace::int4                   as schema_id,
    trim( pg_namespace.nspname )::varchar         as schema_name,
    pg_class.oid::int4                            as view_id,
    trim( pg_class.relname )::varchar             as view_name,
    pg_get_viewdef( pg_class.oid, True )::varchar as view_text
  from
         pg_class
    join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind = 'v';


create or replace view
  sf_241129.views_text
as
  select
    schema_name::varchar as sschema,
    view_name::varchar   as vview,
    view_text::varchar   as ttext
  from
    lf_241129.views_text
  order by
    schema_name,
    view_name;


create or replace view
  lf_241129.steps_totals_data_on_both_for_users_by_hour
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4           as usesysid,
    trim( pg_user.usename )::varchar as user_name,
    cte_node_types.node_type::varchar,
    base_241129.generate_hours.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               pg_user
    cross join cte_node_types
    cross join base_241129.generate_hours
     left join cte_aggr         using( node_type, usesysid, start_ts )
     left join cte_bcast        using( node_type, usesysid, start_ts )
     left join cte_delete       using( node_type, usesysid, start_ts )
     left join cte_dist         using( node_type, usesysid, start_ts )
     left join cte_hash         using( node_type, usesysid, start_ts )
     left join cte_hashjoin     using( node_type, usesysid, start_ts )
     left join cte_insert       using( node_type, usesysid, start_ts )
     left join cte_limit        using( node_type, usesysid, start_ts )
     left join cte_merge        using( node_type, usesysid, start_ts )
     left join cte_mergejoin    using( node_type, usesysid, start_ts )
     left join cte_nestloop     using( node_type, usesysid, start_ts )
     left join cte_parse        using( node_type, usesysid, start_ts )
     left join cte_project      using( node_type, usesysid, start_ts )
     left join cte_return       using( node_type, usesysid, start_ts )
     left join cte_save         using( node_type, usesysid, start_ts )
     left join cte_sort         using( node_type, usesysid, start_ts )
     left join cte_spatial_join using( node_type, usesysid, start_ts )
     left join cte_unique       using( node_type, usesysid, start_ts )
     left join cte_window       using( node_type, usesysid, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.wqueues_routing_rules
as
  select
    rule_order::int2                                                           as oorder,
    case when query_group_wildcard_flag = true then '✔' else '' end::varchar as qgwc,
    case when user_group_wildcard_flag  = true then '✔' else '' end::varchar as ugwc,
    condition::varchar,
    queue_name::varchar                                                        as qname
  from
    lf_241129.wqueues_routing_rules
  order by
    rule_order;

create or replace view
  lf_241129.queries_stats_durations_on_workers_for_all_by_all
as
  with
    cte_queries as
    (
      select
        query_queue_duration,
        query_compile_duration,
        ( query_compile_exec_duration - query_compile_duration ) as query_exec_duration
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id )
        left join base_241129.queries_compilation  using ( query_id )
      where
            query_abort_flag      = false
        and query_csc_flag          = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts           <= current_timestamp
        and query_start_ts         >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        min( query_queue_duration ) over ()                              as min_query_queue_duration,
        max( query_queue_duration ) over ()                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ()     as med_query_queue_duration,
        avg( query_queue_duration ) over ()                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over () as std_query_queue_duration,
        sum( query_queue_duration ) over ()                              as sum_query_queue_duration,

        min( query_compile_duration ) over ()                              as min_query_compile_duration,
        max( query_compile_duration ) over ()                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ()     as med_query_compile_duration,
        avg( query_compile_duration ) over ()                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over () as std_query_compile_duration,
        sum( query_compile_duration ) over ()                              as sum_query_compile_duration,

        min( query_exec_duration ) over ()                            as min_query_exec_duration,
        max( query_exec_duration ) over ()                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ()     as med_query_exec_duration,
        avg( query_exec_duration ) over ()                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over () as std_query_exec_duration,
        sum( query_exec_duration ) over ()                            as sum_query_exec_duration,

        count(*) over () as number_queries
      from
        cte_queries
    )

  select
    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
    cte_queries_stats
  group by
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  ro.installed_versions
as
  select distinct
    split_part( nspname, '_', 2 )::varchar as vversion
  from
    pg_namespace
  where
    regexp_instr( nspname, '^(base|lf|sf)_(debug|[0-9]{6})$' ) != 0
  order by
    vversion;


create or replace view
  lf_241129.vacuum_stats_for_all_by_day
as
  with
    cte_xids as 
    (
      select
        sum( num_new_blocks      ) as sum_number_new_blocks,
        sum( num_blocks_replaced ) as sum_number_replaced_blocks,

        sum( num_new_rows        ) as sum_number_new_rows,
        sum( num_deleted_rows    ) as sum_number_deleted_rows,

        count(*)                   as number_qids,

        min( query_start_ts ) as min_query_start_ts,
        max( query_end_ts   ) as max_query_end_ts,

        max_query_end_ts - min_query_start_ts as duration
      from
             base_241129.fixed_stl_vacuum_detail
        join base_241129.fixed_stl_query using( query_id, transaction_id )
      where
            -- MG2 : procably we should include aborted - need a general approcch to this
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_end_ts     <  current_timestamp
      group by
        transaction_id
      having
        -- MG2 : ignore xids in partial first start_ts
        min_query_start_ts >= current_date - interval '6 day'
    ),

    cte_xids_stats as
    (
      select
        date_trunc( 'day', min_query_start_ts ) as start_ts,

        min( sum_number_new_blocks )        over () as min_sum_number_new_blocks,
        max( sum_number_new_blocks )        over () as max_sum_number_new_blocks,
        median( sum_number_new_blocks )     over () as med_sum_number_new_blocks,
        avg( sum_number_new_blocks )        over () as avg_sum_number_new_blocks,
        stddev_pop( sum_number_new_blocks ) over () as std_sum_number_new_blocks,
        sum( sum_number_new_blocks )        over () as sum_sum_number_new_blocks,

        min( sum_number_replaced_blocks )        over () as min_sum_number_replaced_blocks,
        max( sum_number_replaced_blocks )        over () as max_sum_number_replaced_blocks,
        median( sum_number_replaced_blocks )     over () as med_sum_number_replaced_blocks,
        avg( sum_number_replaced_blocks )        over () as avg_sum_number_replaced_blocks,
        stddev_pop( sum_number_replaced_blocks ) over () as std_sum_number_replaced_blocks,
        sum( sum_number_replaced_blocks )        over () as sum_sum_number_replaced_blocks,

        min( sum_number_new_rows )        over () as min_sum_number_new_rows,
        max( sum_number_new_rows )        over () as max_sum_number_new_rows,
        median( sum_number_new_rows )     over () as med_sum_number_new_rows,
        avg( sum_number_new_rows )        over () as avg_sum_number_new_rows,
        stddev_pop( sum_number_new_rows ) over () as std_sum_number_new_rows,
        sum( sum_number_new_rows )        over () as sum_sum_number_new_rows,

        min( sum_number_deleted_rows )        over () as min_sum_number_deleted_rows,
        max( sum_number_deleted_rows )        over () as max_sum_number_deleted_rows,
        median( sum_number_deleted_rows )     over () as med_sum_number_deleted_rows,
        avg( sum_number_deleted_rows )        over () as avg_sum_number_deleted_rows,
        stddev_pop( sum_number_deleted_rows ) over () as std_sum_number_deleted_rows,
        sum( sum_number_deleted_rows )        over () as sum_sum_number_deleted_rows,

        min( duration ) over ()                              as min_duration,
        max( duration ) over ()                              as max_duration,
        median( extract(epoch from (duration)) ) over ()     as med_duration,
        avg( duration ) over ()                              as avg_duration,
        stddev_pop( extract(epoch from (duration)) ) over () as std_duration,
        sum( duration ) over ()                              as sum_duration,

        sum( number_qids ) over () as number_qids,
        count(*) over ()           as number_xids
      from
        cte_xids
    )

  select
    start_ts::timestamp,

    min_sum_number_new_blocks::int8,
    max_sum_number_new_blocks::int8,
    med_sum_number_new_blocks::int8,
    avg_sum_number_new_blocks::float4,
    std_sum_number_new_blocks::float4,
    sum_sum_number_new_blocks::float4,

    min_sum_number_replaced_blocks::int8,
    max_sum_number_replaced_blocks::int8,
    med_sum_number_replaced_blocks::int8,
    avg_sum_number_replaced_blocks::float4,
    std_sum_number_replaced_blocks::float4,
    sum_sum_number_replaced_blocks::float4,

    min_sum_number_new_rows::int8,
    max_sum_number_new_rows::int8,
    med_sum_number_new_rows::int8,
    avg_sum_number_new_rows::float4,
    std_sum_number_new_rows::float4,
    sum_sum_number_new_rows::float4,

    min_sum_number_deleted_rows::int8,
    max_sum_number_deleted_rows::int8,
    med_sum_number_deleted_rows::int8,
    avg_sum_number_deleted_rows::float4,
    std_sum_number_deleted_rows::float4,
    sum_sum_number_deleted_rows::float4,

    min_duration::interval,
    max_duration::interval,
    med_duration::float4,
    avg_duration::interval,
    std_duration::float4,
    sum_duration::interval,

    number_qids::int8,
    number_xids::int8
  from
              base_241129.generate_days
    left join cte_xids_stats using ( start_ts )
  group by
    start_ts,
    min_sum_number_new_blocks,
    max_sum_number_new_blocks,
    med_sum_number_new_blocks,
    avg_sum_number_new_blocks,
    std_sum_number_new_blocks,
    sum_sum_number_new_blocks,
    min_sum_number_replaced_blocks,
    max_sum_number_replaced_blocks,
    med_sum_number_replaced_blocks,
    avg_sum_number_replaced_blocks,
    std_sum_number_replaced_blocks,
    sum_sum_number_replaced_blocks,
    min_sum_number_new_rows,
    max_sum_number_new_rows,
    med_sum_number_new_rows,
    avg_sum_number_new_rows,
    std_sum_number_new_rows,
    sum_sum_number_new_rows,
    min_sum_number_deleted_rows,
    max_sum_number_deleted_rows,
    med_sum_number_deleted_rows,
    avg_sum_number_deleted_rows,
    std_sum_number_deleted_rows,
    sum_sum_number_deleted_rows,
    min_duration,
    max_duration,
    med_duration,
    avg_duration,
    std_duration,
    sum_duration,
    number_qids,
    number_xids;


create or replace view
  sf_241129.vacuum_stats_for_all_by_day
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        base_241129.hr_number( min_sum_number_new_blocks ) as hr_min_sum_number_new_blocks,
        base_241129.hr_number( max_sum_number_new_blocks ) as hr_max_sum_number_new_blocks,
        base_241129.hr_number( med_sum_number_new_blocks ) as hr_med_sum_number_new_blocks,
        base_241129.hr_number( avg_sum_number_new_blocks ) as hr_avg_sum_number_new_blocks,
        base_241129.hr_number( std_sum_number_new_blocks ) as hr_std_sum_number_new_blocks,
        base_241129.hr_number( sum_sum_number_new_blocks ) as hr_sum_sum_number_new_blocks,

        base_241129.hr_number( min_sum_number_replaced_blocks ) as hr_min_sum_number_replaced_blocks,
        base_241129.hr_number( max_sum_number_replaced_blocks ) as hr_max_sum_number_replaced_blocks,
        base_241129.hr_number( med_sum_number_replaced_blocks ) as hr_med_sum_number_replaced_blocks,
        base_241129.hr_number( avg_sum_number_replaced_blocks ) as hr_avg_sum_number_replaced_blocks,
        base_241129.hr_number( std_sum_number_replaced_blocks ) as hr_std_sum_number_replaced_blocks,
        base_241129.hr_number( sum_sum_number_replaced_blocks ) as hr_sum_sum_number_replaced_blocks,

        base_241129.hr_number( min_sum_number_new_rows ) as hr_min_sum_number_new_rows,
        base_241129.hr_number( max_sum_number_new_rows ) as hr_max_sum_number_new_rows,
        base_241129.hr_number( med_sum_number_new_rows ) as hr_med_sum_number_new_rows,
        base_241129.hr_number( avg_sum_number_new_rows ) as hr_avg_sum_number_new_rows,
        base_241129.hr_number( std_sum_number_new_rows ) as hr_std_sum_number_new_rows,
        base_241129.hr_number( sum_sum_number_new_rows ) as hr_sum_sum_number_new_rows,

        base_241129.hr_number( min_sum_number_deleted_rows ) as hr_min_sum_number_deleted_rows,
        base_241129.hr_number( max_sum_number_deleted_rows ) as hr_max_sum_number_deleted_rows,
        base_241129.hr_number( med_sum_number_deleted_rows ) as hr_med_sum_number_deleted_rows,
        base_241129.hr_number( avg_sum_number_deleted_rows ) as hr_avg_sum_number_deleted_rows,
        base_241129.hr_number( std_sum_number_deleted_rows ) as hr_std_sum_number_deleted_rows,
        base_241129.hr_number( sum_sum_number_deleted_rows ) as hr_sum_sum_number_deleted_rows,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_sum_number_new_blocks) ) over () as max_len_hr_min_sum_number_new_blocks,
        max( len(hr_max_sum_number_new_blocks) ) over () as max_len_hr_max_sum_number_new_blocks,
        max( len(hr_med_sum_number_new_blocks) ) over () as max_len_hr_med_sum_number_new_blocks,
        max( len(hr_avg_sum_number_new_blocks) ) over () as max_len_hr_avg_sum_number_new_blocks,
        max( len(hr_std_sum_number_new_blocks) ) over () as max_len_hr_std_sum_number_new_blocks,
        max( len(hr_sum_sum_number_new_blocks) ) over () as max_len_hr_sum_sum_number_new_blocks,

        max( len(hr_min_sum_number_replaced_blocks) ) over () as max_len_hr_min_sum_number_replaced_blocks,
        max( len(hr_max_sum_number_replaced_blocks) ) over () as max_len_hr_max_sum_number_replaced_blocks,
        max( len(hr_med_sum_number_replaced_blocks) ) over () as max_len_hr_med_sum_number_replaced_blocks,
        max( len(hr_avg_sum_number_replaced_blocks) ) over () as max_len_hr_avg_sum_number_replaced_blocks,
        max( len(hr_std_sum_number_replaced_blocks) ) over () as max_len_hr_std_sum_number_replaced_blocks,
        max( len(hr_sum_sum_number_replaced_blocks) ) over () as max_len_hr_sum_sum_number_replaced_blocks,

        max( len(hr_min_sum_number_new_rows) ) over () as max_len_hr_min_sum_number_new_rows,
        max( len(hr_max_sum_number_new_rows) ) over () as max_len_hr_max_sum_number_new_rows,
        max( len(hr_med_sum_number_new_rows) ) over () as max_len_hr_med_sum_number_new_rows,
        max( len(hr_avg_sum_number_new_rows) ) over () as max_len_hr_avg_sum_number_new_rows,
        max( len(hr_std_sum_number_new_rows) ) over () as max_len_hr_std_sum_number_new_rows,
        max( len(hr_sum_sum_number_new_rows) ) over () as max_len_hr_sum_sum_number_new_rows,

        max( len(hr_min_sum_number_deleted_rows) ) over () as max_len_hr_min_sum_number_deleted_rows,
        max( len(hr_max_sum_number_deleted_rows) ) over () as max_len_hr_max_sum_number_deleted_rows,
        max( len(hr_med_sum_number_deleted_rows) ) over () as max_len_hr_med_sum_number_deleted_rows,
        max( len(hr_avg_sum_number_deleted_rows) ) over () as max_len_hr_avg_sum_number_deleted_rows,
        max( len(hr_std_sum_number_deleted_rows) ) over () as max_len_hr_std_sum_number_deleted_rows,
        max( len(hr_sum_sum_number_deleted_rows) ) over () as max_len_hr_sum_sum_number_deleted_rows,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_qids,
        number_xids
      from
        lf_241129.vacuum_stats_for_all_by_day
    )

  select
    start_ts::date,

    ( lpad( hr_min_sum_number_new_blocks, max_len_hr_min_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_new_blocks, max_len_hr_max_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_new_blocks, max_len_hr_med_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_new_blocks, max_len_hr_avg_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_new_blocks, max_len_hr_std_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_new_blocks, max_len_hr_sum_sum_number_new_blocks ) )::varchar as "new blocks",

    ( lpad( hr_min_sum_number_replaced_blocks, max_len_hr_min_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_replaced_blocks, max_len_hr_max_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_replaced_blocks, max_len_hr_med_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_replaced_blocks, max_len_hr_avg_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_replaced_blocks, max_len_hr_std_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_replaced_blocks, max_len_hr_sum_sum_number_replaced_blocks ) )::varchar as "replaced blocks",

    ( lpad( hr_min_sum_number_new_rows, max_len_hr_min_sum_number_new_rows ) || ' / ' ||
      lpad( hr_max_sum_number_new_rows, max_len_hr_max_sum_number_new_rows ) || ' / ' ||
      lpad( hr_med_sum_number_new_rows, max_len_hr_med_sum_number_new_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_new_rows, max_len_hr_avg_sum_number_new_rows ) || ' / ' ||
      lpad( hr_std_sum_number_new_rows, max_len_hr_std_sum_number_new_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_new_rows, max_len_hr_sum_sum_number_new_rows ) )::varchar as "new rows",

    ( lpad( hr_min_sum_number_deleted_rows, max_len_hr_min_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_max_sum_number_deleted_rows, max_len_hr_max_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_med_sum_number_deleted_rows, max_len_hr_med_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_deleted_rows, max_len_hr_avg_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_std_sum_number_deleted_rows, max_len_hr_std_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_deleted_rows, max_len_hr_sum_sum_number_deleted_rows ) )::varchar as "deleted rows",

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as "xid duration",

    number_qids::int8 as qids,
    number_xids::int8 as xids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  base_241129.transactions_blocks
as
  with
    cte_worker_queries_in_transactions as
    (
      select
        transaction_id,
        query_id
      from
        base_241129.fixed_stl_wlm_query
    )

  select
    transaction_id::int8,
    sum( query_number_blocks_read    )::int8 as transaction_number_blocks_read,
    sum( query_number_blocks_to_disk )::int8 as transaction_number_blocks_to_disk
  from
         base_241129.fixed_stl_query_metrics
    join cte_worker_queries_in_transactions using ( query_id )
  group by
    transaction_id;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_users_by_hour
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        usesysid,
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4               as user_id,
    trim( pg_user.usename )::varchar     as user_name,
    base_241129.generate_hours.start_ts::timestamp as start_ts,
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_hours
    cross join pg_user
     left join cte_aggr_stats         using ( start_ts, node_type, usesysid )
     left join cte_bcast_stats        using ( start_ts, node_type, usesysid )
     left join cte_delete_stats       using ( start_ts, node_type, usesysid )
     left join cte_dist_stats         using ( start_ts, node_type, usesysid )
     left join cte_hash_stats         using ( start_ts, node_type, usesysid )
     left join cte_hashjoin_stats     using ( start_ts, node_type, usesysid )
     left join cte_insert_stats       using ( start_ts, node_type, usesysid )
     left join cte_limit_stats        using ( start_ts, node_type, usesysid )
     left join cte_merge_stats        using ( start_ts, node_type, usesysid )
     left join cte_mergejoin_stats    using ( start_ts, node_type, usesysid )
     left join cte_nestloop_stats     using ( start_ts, node_type, usesysid )
     left join cte_parse_stats        using ( start_ts, node_type, usesysid )
     left join cte_project_stats      using ( start_ts, node_type, usesysid )
     left join cte_return_stats       using ( start_ts, node_type, usesysid )
     left join cte_save_stats         using ( start_ts, node_type, usesysid )
     left join cte_sort_stats         using ( start_ts, node_type, usesysid )
     left join cte_spatial_join_stats using ( start_ts, node_type, usesysid )
     left join cte_unique_stats       using ( start_ts, node_type, usesysid )
     left join cte_window_stats       using ( start_ts, node_type, usesysid )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_bcast_duration is not null
    or max_bcast_duration is not null
    or med_bcast_duration is not null
    or avg_bcast_duration is not null
    or std_bcast_duration is not null
    or number_bcast_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_dist_duration is not null
    or max_dist_duration is not null
    or med_dist_duration is not null
    or avg_dist_duration is not null
    or std_dist_duration is not null
    or number_dist_steps is not null
    or min_hash_duration is not null
    or max_hash_duration is not null
    or med_hash_duration is not null
    or avg_hash_duration is not null
    or std_hash_duration is not null
    or number_hash_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or min_limit_duration is not null
    or max_limit_duration is not null
    or med_limit_duration is not null
    or avg_limit_duration is not null
    or std_limit_duration is not null
    or number_limit_steps is not null
    or min_merge_duration is not null
    or max_merge_duration is not null
    or med_merge_duration is not null
    or avg_merge_duration is not null
    or std_merge_duration is not null
    or number_merge_steps is not null
    or min_mergejoin_duration is not null
    or max_mergejoin_duration is not null
    or med_mergejoin_duration is not null
    or avg_mergejoin_duration is not null
    or std_mergejoin_duration is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or min_parse_duration is not null
    or max_parse_duration is not null
    or med_parse_duration is not null
    or avg_parse_duration is not null
    or std_parse_duration is not null
    or number_parse_steps is not null
    or min_project_duration is not null
    or max_project_duration is not null
    or med_project_duration is not null
    or avg_project_duration is not null
    or std_project_duration is not null
    or number_project_steps is not null
    or min_return_duration is not null
    or max_return_duration is not null
    or med_return_duration is not null
    or avg_return_duration is not null
    or std_return_duration is not null
    or number_return_steps is not null
    or min_save_duration is not null
    or max_save_duration is not null
    or med_save_duration is not null
    or avg_save_duration is not null
    or std_save_duration is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null
    or min_unique_duration is not null
    or max_unique_duration is not null
    or med_unique_duration is not null
    or avg_unique_duration is not null
    or std_unique_duration is not null
    or number_unique_steps is not null
    or min_window_duration is not null
    or max_window_duration is not null
    or med_window_duration is not null
    or avg_window_duration is not null
    or std_window_duration is not null
    or number_window_steps is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_hour_complete
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_hour
  order by
    user_name,
    node_type,
    start_ts;



create or replace view
  base_241129.fixed_stv_blocklist
as
  select
    slice::int2        as slice_ordinal,
    ( col + 1 )::int2  as column_ordinal,
    tbl::int4          as table_id,
    blocknum::int4     as block_ordinal,
    num_values::int4   as block_number_values,
    minvalue::int8     as block_min_value,
    maxvalue::int8     as block_max_value,
    case
      when unsorted = 0 then true
      when unsorted = 1 then false
    end::bool          as block_sorted_flag
  from
    stv_blocklist;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_day_complete
as
  select
    start_ts::date,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_day
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_tables_by_day
as
  with
    cte_aggr as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        table_id,
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        table_id,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    node_type::varchar,
    base_241129.generate_days.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_days
    cross join base_241129.fixed_pg_class_for_tables
     left join base_241129.fixed_pg_namespace using ( schema_id )
     left join cte_aggr_stats          using ( start_ts, node_type, table_id )
     left join cte_delete_stats        using ( start_ts, node_type, table_id )
     left join cte_hashjoin_stats      using ( start_ts, node_type, table_id )
     left join cte_insert_stats        using ( start_ts, node_type, table_id )
     left join cte_mergejoin_stats     using ( start_ts, node_type, table_id )
     left join cte_nestloop_stats      using ( start_ts, node_type, table_id )
     left join cte_save_stats          using ( start_ts, node_type, table_id )
     left join cte_sort_stats          using ( start_ts, node_type, table_id )
     left join cte_spatial_join_stats  using ( start_ts, node_type, table_id )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_day_concise
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::date,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_day
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_all_by_second
as
  select
    date_trunc( 'second', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    start_ts;


create or replace view
  sf_241129.analyze_totals_for_all_by_second
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_all_by_second
    )

  select
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.vacuum_totals_for_tables_by_minute
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        table_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        table_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        table_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        table_id,
        date_trunc( 'minute', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        table_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        table_id,
        date_trunc( 'minute', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    cte_number_vacuums.table_id::int4,
    trim( pg_class.relname )::varchar     as table_name,

    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration                using ( table_id, start_ts )
    left join pg_class     on pg_class.oid          = cte_number_vacuums.table_id
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.queries_totals_result_cache_for_query
as
  select
    query_id::int4,
    case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end::int8 as number_hit,
    case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end::int8 as number_miss,
    1::int8                                                                                                  as number_total,
    ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
  from
              base_241129.fixed_stl_query
    left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
  where
        query_abort_flag  = false
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp;


create or replace view
  sf_241129.queries_totals_result_cache_for_query
as
  select
    query_id::int4      as qid,
    number_hit::int8    as hit,
    number_miss::int8   as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_query
  order by
    query_id;


create or replace view
  lf_241129.steps_totals_data_on_both_for_statements_by_second
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    statement_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               cte_base
    cross join cte_node_types
    cross join base_241129.generate_seconds
     left join cte_aggr         using ( start_ts, query, node_type )
     left join cte_bcast        using ( start_ts, query, node_type )
     left join cte_delete       using ( start_ts, query, node_type )
     left join cte_dist         using ( start_ts, query, node_type )
     left join cte_hash         using ( start_ts, query, node_type )
     left join cte_hashjoin     using ( start_ts, query, node_type )
     left join cte_insert       using ( start_ts, query, node_type )
     left join cte_limit        using ( start_ts, query, node_type )
     left join cte_merge        using ( start_ts, query, node_type )
     left join cte_mergejoin    using ( start_ts, query, node_type )
     left join cte_nestloop     using ( start_ts, query, node_type )
     left join cte_parse        using ( start_ts, query, node_type )
     left join cte_project      using ( start_ts, query, node_type )
     left join cte_return       using ( start_ts, query, node_type )
     left join cte_save         using ( start_ts, query, node_type )
     left join cte_sort         using ( start_ts, query, node_type )
     left join cte_spatial_join using ( start_ts, query, node_type )
     left join cte_unique       using ( start_ts, query, node_type )
     left join cte_window       using ( start_ts, query, node_type )
  group by
    statement_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.transactions_totals_commits_by_second
as
  select
    date_trunc( 'second', batch_leader_startwork )::timestamp as start_ts,
    sum( batch_leader_duration )::interval                     as sum_batch_leader_duration,
    sum( batch_worker_duration )::interval                     as sum_batch_worker_duration,
    sum( batch_exec_duration - batch_leader_duration - batch_worker_duration )::interval as sum_batch_idle_duration,
    sum( batch_exec_duration  )::interval                      as sum_batch_exec_duration
  from
    lf_241129.batches
  group by
    start_ts;


create or replace view
  sf_241129.transactions_totals_commits_by_second
as
  select
    start_ts::timestamp,
    extract( epoch from sum_batch_leader_duration )::numeric(18,2) as leader,
    extract( epoch from sum_batch_worker_duration )::numeric(18,2) as worker,
    extract( epoch from sum_batch_idle_duration   )::numeric(18,2) as idle,
    extract( epoch from sum_batch_exec_duration   )::numeric(18,2) as exec
  from
    lf_241129.transactions_totals_commits_by_second
  order by
    start_ts;


create or replace view
  lf_241129.ttables_creation_for_schemas_by_second
as
  select
    trim( pg_namespace.nspname )::varchar           as schema_name,
    date_trunc( 'second', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    schema_name,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_statements_by_hour
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query as query_id
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query as query_id
      from
        stl_unload_log
    ),

    cte_queries as
    (
      select
        cte_base.statement_name,
        date_trunc( 'hour', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
             cte_base
        join base_241129.fixed_stl_wlm_query using ( query_id )
        join base_241129.fixed_stl_query     using ( query_id )
        join base_241129.queries_compilation   using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        statement_name,
        start_ts,

        min( query_queue_duration ) over ( partition by statement_name, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by statement_name, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by statement_name, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by statement_name, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by statement_name, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by statement_name, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by statement_name, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by statement_name, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by statement_name, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by statement_name, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by statement_name, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by statement_name, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by statement_name, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    statement_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_hours
    left join cte_queries_stats using ( start_ts )
  group by
    statement_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace procedure
  ro.sp_revoke_versionless_sf_schema_from_role( role_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'revoke usage on schema sf from role ' || role_name;
      execute sql;

      sql := 'revoke select on all tables in schema sf from role ' || role_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.wqueues_totals_slot_utilization_by_hour
as
  with
    cte_events as
    (
      select
        queue_id,
        query_compile_exec_start_ts as event_ts,
        1                   as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'

      union all

      select
        queue_id,
        query_compile_exec_end_ts as event_ts,
        -1                as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_slot_durations as
    (
      select
        queue_id,
        sum( event_delta ) over ( partition by queue_id, start_ts order by event_ts rows between unbounded preceding and current row ) as slot_number,
        date_trunc( 'hour', event_ts )                                                                                             as start_ts,
        lead( event_ts   ) over ( partition by queue_id, start_ts order by event_ts ) - event_ts                                       as duration
      from
        cte_events
    ),

    cte_durations as
    (
      select
        queue_id,
        slot_number,
        start_ts,
        sum( duration ) as sum_duration
      from
        cte_slot_durations
      group by
        queue_id,
        slot_number,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    slot_number::int2,
    start_ts::timestamp,
    sum_duration::interval as duration
  from
              cte_durations
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  lf_241129.analyze_totals_for_all_by_minute
as
  select
    date_trunc( 'minute', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_tables_by_second
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'second', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    cte_s1.table_id::int4,
    trim( pg_class.relname )::varchar      as table_name,
    node_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r';


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_tables_by_second
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_tables_by_second
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_tables_by_all
as
  with
    cte_aggr as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        table_id,
        node_type
    ),

    cte_delete as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        table_id,
        node_type
    ),

    cte_hashjoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        table_id,
        node_type
    ),

    cte_insert as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        table_id,
        node_type
    ),

    cte_mergejoin as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        table_id,
        node_type
    ),

    cte_nestloop as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        table_id,
        node_type
    ),

    cte_save as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        table_id,
        node_type
    ),

    cte_sort as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        table_id,
        node_type
    ),

    cte_spatial_join as
    (
      select
        tbl as table_id,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        tbl,
        node_type,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        table_id,
        node_type,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        table_id,
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8
  from
               cte_node_types
    cross join base_241129.fixed_pg_class_for_tables
     left join base_241129.fixed_pg_namespace using ( schema_id )
     left join cte_aggr_stats          using ( node_type, table_id )
     left join cte_delete_stats        using ( node_type, table_id )
     left join cte_hashjoin_stats      using ( node_type, table_id )
     left join cte_insert_stats        using ( node_type, table_id )
     left join cte_mergejoin_stats     using ( node_type, table_id )
     left join cte_nestloop_stats      using ( node_type, table_id )
     left join cte_save_stats          using ( node_type, table_id )
     left join cte_sort_stats          using ( node_type, table_id )
     left join cte_spatial_join_stats  using ( node_type, table_id )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_all_concise
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_all
  order by
    schema_name,
    table_name,
    node_type;



create or replace view
  sf_241129.queries_compilation
as
  select
    session_id::int4                                            as pid,
    transaction_id::int8                                        as xid,
    query_id::int4                                              as qid,
    query_number_leader_node_cache_segments::int2               as lncache,
    query_number_offload_segments::int2                         as offload,
    query_number_segments::int2                                 as total,
    extract( epoch from query_compile_duration )::numeric(18,6) as dur
  from
    lf_241129.queries_compilation
  order by
    session_id,
    transaction_id,
    query_id;


create or replace view
  lf_241129.transactions_stats_commits_by_second
as
  with
    cte_transaction_queuing_stats as
    (
      select
        date_trunc( 'second', transaction_batch_queue_start )                                         as start_ts,
        min( batch_leader_startwork - transaction_batch_queue_start )                              as min_transaction_batch_queue_start,
        max( batch_leader_startwork - transaction_batch_queue_start )                              as max_transaction_batch_queue_start,
        median( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) )     as med_transaction_batch_queue_start,
        avg( batch_leader_startwork - transaction_batch_queue_start )                              as avg_transaction_batch_queue_start,
        stddev_pop( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) ) as std_transaction_batch_queue_start,
        sum( batch_leader_startwork - transaction_batch_queue_start )                              as sum_transaction_batch_queue_start
      from
             base_241129.transactions_in_batches
        join lf_241129.batches using ( batch_id )
      group by
        start_ts
    ),

    cte_batch_stats as
    (
      select
        date_trunc( 'second', batch_leader_startwork ) as start_ts,

        min( batch_queue_duration )                            over ( partition by start_ts ) as min_batch_queue_duration,
        max( batch_queue_duration )                            over ( partition by start_ts ) as max_batch_queue_duration,
        median( extract(epoch from batch_queue_duration) )     over ( partition by start_ts ) as med_batch_queue_duration,
        avg( batch_queue_duration )                            over ( partition by start_ts ) as avg_batch_queue_duration,
        stddev_pop( extract(epoch from batch_queue_duration) ) over ( partition by start_ts ) as std_batch_queue_duration,
        sum( batch_queue_duration )                            over ( partition by start_ts ) as sum_batch_queue_duration,

        min( batch_leader_duration )                            over ( partition by start_ts ) as min_batch_leader_duration,
        max( batch_leader_duration )                            over ( partition by start_ts ) as max_batch_leader_duration,
        median( extract(epoch from batch_leader_duration) )     over ( partition by start_ts ) as med_batch_leader_duration,
        avg( batch_leader_duration )                            over ( partition by start_ts ) as avg_batch_leader_duration,
        stddev_pop( extract(epoch from batch_leader_duration) ) over ( partition by start_ts ) as std_batch_leader_duration,
        sum( batch_leader_duration )                            over ( partition by start_ts ) as sum_batch_leader_duration,

        min( batch_worker_duration )                            over ( partition by start_ts ) as min_batch_worker_duration,
        max( batch_worker_duration )                            over ( partition by start_ts ) as max_batch_worker_duration,
        median( extract(epoch from batch_worker_duration) )     over ( partition by start_ts ) as med_batch_worker_duration,
        avg( batch_worker_duration )                            over ( partition by start_ts ) as avg_batch_worker_duration,
        stddev_pop( extract(epoch from batch_worker_duration) ) over ( partition by start_ts ) as std_batch_worker_duration,
        sum( batch_worker_duration )                            over ( partition by start_ts ) as sum_batch_worker_duration,

        min( number_batch_transactions )         over ( partition by start_ts ) as min_number_batch_transactions,
        max( number_batch_transactions )         over ( partition by start_ts ) as max_number_batch_transactions,
        median( number_batch_transactions )      over ( partition by start_ts ) as med_number_batch_transactions,
        avg( number_batch_transactions::float4 ) over ( partition by start_ts ) as avg_number_batch_transactions,
        stddev_pop( number_batch_transactions )  over ( partition by start_ts ) as std_number_batch_transactions,
        sum( number_batch_transactions )         over ( partition by start_ts ) as sum_number_batch_transactions,

        min( number_batch_blocks )         over ( partition by start_ts ) as min_number_batch_blocks,
        max( number_batch_blocks )         over ( partition by start_ts ) as max_number_batch_blocks,
        median( number_batch_blocks )      over ( partition by start_ts ) as med_number_batch_blocks,
        avg( number_batch_blocks::float4 ) over ( partition by start_ts ) as avg_number_batch_blocks,
        stddev_pop( number_batch_blocks )  over ( partition by start_ts ) as std_number_batch_blocks,
        sum( number_batch_blocks )         over ( partition by start_ts ) as sum_number_batch_blocks,

        count(*) over ( partition by start_ts ) as number_batches
      from
        lf_241129.batches
    )

  select
    cte_batch_stats.start_ts::timestamp,

    cte_transaction_queuing_stats.min_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start::interval,

    cte_batch_stats.min_batch_queue_duration::interval,
    cte_batch_stats.max_batch_queue_duration::interval,
    cte_batch_stats.med_batch_queue_duration::float4,
    cte_batch_stats.avg_batch_queue_duration::interval,
    cte_batch_stats.std_batch_queue_duration::float4,
    cte_batch_stats.sum_batch_queue_duration::interval,

    cte_batch_stats.min_batch_leader_duration::interval,
    cte_batch_stats.max_batch_leader_duration::interval,
    cte_batch_stats.med_batch_leader_duration::float4,
    cte_batch_stats.avg_batch_leader_duration::interval,
    cte_batch_stats.std_batch_leader_duration::float4,
    cte_batch_stats.sum_batch_leader_duration::interval,

    cte_batch_stats.min_batch_worker_duration::interval,
    cte_batch_stats.max_batch_worker_duration::interval,
    cte_batch_stats.med_batch_worker_duration::float4,
    cte_batch_stats.avg_batch_worker_duration::interval,
    cte_batch_stats.std_batch_worker_duration::float4,
    cte_batch_stats.sum_batch_worker_duration::interval,

    cte_batch_stats.min_number_batch_transactions::int8,
    cte_batch_stats.max_number_batch_transactions::int8,
    cte_batch_stats.med_number_batch_transactions::int8,
    cte_batch_stats.avg_number_batch_transactions::float4,
    cte_batch_stats.std_number_batch_transactions::float4,
    cte_batch_stats.sum_number_batch_transactions::int8,

    cte_batch_stats.min_number_batch_blocks::int8,
    cte_batch_stats.max_number_batch_blocks::int8,
    cte_batch_stats.med_number_batch_blocks::int8,
    cte_batch_stats.avg_number_batch_blocks::float4,
    cte_batch_stats.std_number_batch_blocks::float4,
    cte_batch_stats.sum_number_batch_blocks::int8,

    cte_batch_stats.number_batches::int8
  from
              cte_batch_stats
    left join cte_transaction_queuing_stats using ( start_ts )
  group by
    start_ts,
    cte_transaction_queuing_stats.min_transaction_batch_queue_start,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start,
    cte_batch_stats.min_batch_queue_duration,
    cte_batch_stats.max_batch_queue_duration,
    cte_batch_stats.med_batch_queue_duration,
    cte_batch_stats.avg_batch_queue_duration,
    cte_batch_stats.std_batch_queue_duration,
    cte_batch_stats.sum_batch_queue_duration,
    cte_batch_stats.min_batch_leader_duration,
    cte_batch_stats.max_batch_leader_duration,
    cte_batch_stats.med_batch_leader_duration,
    cte_batch_stats.avg_batch_leader_duration,
    cte_batch_stats.std_batch_leader_duration,
    cte_batch_stats.sum_batch_leader_duration,
    cte_batch_stats.min_batch_worker_duration,
    cte_batch_stats.max_batch_worker_duration,
    cte_batch_stats.med_batch_worker_duration,
    cte_batch_stats.avg_batch_worker_duration,
    cte_batch_stats.std_batch_worker_duration,
    cte_batch_stats.sum_batch_worker_duration,
    cte_batch_stats.min_number_batch_transactions,
    cte_batch_stats.max_number_batch_transactions,
    cte_batch_stats.med_number_batch_transactions,
    cte_batch_stats.avg_number_batch_transactions,
    cte_batch_stats.std_number_batch_transactions,
    cte_batch_stats.sum_number_batch_transactions,
    cte_batch_stats.min_number_batch_blocks,
    cte_batch_stats.max_number_batch_blocks,
    cte_batch_stats.med_number_batch_blocks,
    cte_batch_stats.avg_number_batch_blocks,
    cte_batch_stats.std_number_batch_blocks,
    cte_batch_stats.sum_number_batch_blocks,
    cte_batch_stats.number_batches;


create or replace view
  sf_241129.transactions_stats_commits_by_second
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_min_transaction_batch_queue_start,
        extract(epoch from max_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_max_transaction_batch_queue_start,
        med_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_med_transaction_batch_queue_start,
        extract(epoch from avg_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_avg_transaction_batch_queue_start,
        std_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_std_transaction_batch_queue_start,
        extract(epoch from sum_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_sum_transaction_batch_queue_start,

        extract(epoch from min_batch_queue_duration)::numeric(18,2)::varchar as hr_min_batch_queue_duration,
        extract(epoch from max_batch_queue_duration)::numeric(18,2)::varchar as hr_max_batch_queue_duration,
        med_batch_queue_duration::numeric(18,2)::varchar                     as hr_med_batch_queue_duration,
        extract(epoch from avg_batch_queue_duration)::numeric(18,2)::varchar as hr_avg_batch_queue_duration,
        std_batch_queue_duration::numeric(18,2)::varchar                     as hr_std_batch_queue_duration,
        extract(epoch from sum_batch_queue_duration)::numeric(18,2)::varchar as hr_sum_batch_queue_duration,

        extract(epoch from min_batch_leader_duration)::numeric(18,2)::varchar as hr_min_batch_leader_duration,
        extract(epoch from max_batch_leader_duration)::numeric(18,2)::varchar as hr_max_batch_leader_duration,
        med_batch_leader_duration::numeric(18,2)::varchar                     as hr_med_batch_leader_duration,
        extract(epoch from avg_batch_leader_duration)::numeric(18,2)::varchar as hr_avg_batch_leader_duration,
        std_batch_leader_duration::numeric(18,2)::varchar                     as hr_std_batch_leader_duration,
        extract(epoch from sum_batch_leader_duration)::numeric(18,2)::varchar as hr_sum_batch_leader_duration,

        extract(epoch from min_batch_worker_duration)::numeric(18,2)::varchar as hr_min_batch_worker_duration,
        extract(epoch from max_batch_worker_duration)::numeric(18,2)::varchar as hr_max_batch_worker_duration,
        med_batch_worker_duration::numeric(18,2)::varchar                     as hr_med_batch_worker_duration,
        extract(epoch from avg_batch_worker_duration)::numeric(18,2)::varchar as hr_avg_batch_worker_duration,
        std_batch_worker_duration::numeric(18,2)::varchar                     as hr_std_batch_worker_duration,
        extract(epoch from sum_batch_worker_duration)::numeric(18,2)::varchar as hr_sum_batch_worker_duration,

        min_number_batch_transactions::varchar                as hr_min_number_batch_transactions,
        max_number_batch_transactions::varchar                as hr_max_number_batch_transactions,
        med_number_batch_transactions::varchar                as hr_med_number_batch_transactions,
        avg_number_batch_transactions::numeric(18,2)::varchar as hr_avg_number_batch_transactions,
        std_number_batch_transactions::numeric(18,2)::varchar as hr_std_number_batch_transactions,
        sum_number_batch_transactions::varchar                as hr_sum_number_batch_transactions,

        base_241129.hr_number( min_number_batch_blocks ) as hr_min_number_batch_blocks,
        base_241129.hr_number( max_number_batch_blocks ) as hr_max_number_batch_blocks,
        base_241129.hr_number( med_number_batch_blocks ) as hr_med_number_batch_blocks,
        base_241129.hr_number( avg_number_batch_blocks ) as hr_avg_number_batch_blocks,
        base_241129.hr_number( std_number_batch_blocks ) as hr_std_number_batch_blocks,
        base_241129.hr_number( sum_number_batch_blocks ) as hr_sum_number_batch_blocks,

        number_batches,

        max( len(hr_min_transaction_batch_queue_start) ) over () as max_len_hr_min_transaction_batch_queue_start,
        max( len(hr_max_transaction_batch_queue_start) ) over () as max_len_hr_max_transaction_batch_queue_start,
        max( len(hr_med_transaction_batch_queue_start) ) over () as max_len_hr_med_transaction_batch_queue_start,
        max( len(hr_avg_transaction_batch_queue_start) ) over () as max_len_hr_avg_transaction_batch_queue_start,
        max( len(hr_std_transaction_batch_queue_start) ) over () as max_len_hr_std_transaction_batch_queue_start,
        max( len(hr_sum_transaction_batch_queue_start) ) over () as max_len_hr_sum_transaction_batch_queue_start,

        max( len(hr_min_batch_queue_duration) ) over () as max_len_hr_min_batch_queue_duration,
        max( len(hr_max_batch_queue_duration) ) over () as max_len_hr_max_batch_queue_duration,
        max( len(hr_med_batch_queue_duration) ) over () as max_len_hr_med_batch_queue_duration,
        max( len(hr_avg_batch_queue_duration) ) over () as max_len_hr_avg_batch_queue_duration,
        max( len(hr_std_batch_queue_duration) ) over () as max_len_hr_std_batch_queue_duration,
        max( len(hr_sum_batch_queue_duration) ) over () as max_len_hr_sum_batch_queue_duration,

        max( len(hr_min_batch_leader_duration) ) over () as max_len_hr_min_batch_leader_duration,
        max( len(hr_max_batch_leader_duration) ) over () as max_len_hr_max_batch_leader_duration,
        max( len(hr_med_batch_leader_duration) ) over () as max_len_hr_med_batch_leader_duration,
        max( len(hr_avg_batch_leader_duration) ) over () as max_len_hr_avg_batch_leader_duration,
        max( len(hr_std_batch_leader_duration) ) over () as max_len_hr_std_batch_leader_duration,
        max( len(hr_sum_batch_leader_duration) ) over () as max_len_hr_sum_batch_leader_duration,

        max( len(hr_min_batch_worker_duration) ) over () as max_len_hr_min_batch_worker_duration,
        max( len(hr_max_batch_worker_duration) ) over () as max_len_hr_max_batch_worker_duration,
        max( len(hr_med_batch_worker_duration) ) over () as max_len_hr_med_batch_worker_duration,
        max( len(hr_avg_batch_worker_duration) ) over () as max_len_hr_avg_batch_worker_duration,
        max( len(hr_std_batch_worker_duration) ) over () as max_len_hr_std_batch_worker_duration,
        max( len(hr_sum_batch_worker_duration) ) over () as max_len_hr_sum_batch_worker_duration,

        max( len(hr_min_number_batch_transactions) ) over () as max_len_hr_min_number_batch_transactions,
        max( len(hr_max_number_batch_transactions) ) over () as max_len_hr_max_number_batch_transactions,
        max( len(hr_med_number_batch_transactions) ) over () as max_len_hr_med_number_batch_transactions,
        max( len(hr_avg_number_batch_transactions) ) over () as max_len_hr_avg_number_batch_transactions,
        max( len(hr_std_number_batch_transactions) ) over () as max_len_hr_std_number_batch_transactions,
        max( len(hr_sum_number_batch_transactions) ) over () as max_len_hr_sum_number_batch_transactions,

        max( len(hr_min_number_batch_blocks) ) over () as max_len_hr_min_number_batch_blocks,
        max( len(hr_max_number_batch_blocks) ) over () as max_len_hr_max_number_batch_blocks,
        max( len(hr_med_number_batch_blocks) ) over () as max_len_hr_med_number_batch_blocks,
        max( len(hr_avg_number_batch_blocks) ) over () as max_len_hr_avg_number_batch_blocks,
        max( len(hr_std_number_batch_blocks) ) over () as max_len_hr_std_number_batch_blocks,
        max( len(hr_sum_number_batch_blocks) ) over () as max_len_hr_sum_number_batch_blocks
      from
        lf_241129.transactions_stats_commits_by_second
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_transaction_batch_queue_start, max_len_hr_min_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_max_transaction_batch_queue_start, max_len_hr_max_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_med_transaction_batch_queue_start, max_len_hr_med_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_avg_transaction_batch_queue_start, max_len_hr_avg_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_std_transaction_batch_queue_start, max_len_hr_std_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_sum_transaction_batch_queue_start, max_len_hr_sum_transaction_batch_queue_start ) )::varchar              as "xid queuing",

    ( lpad( hr_min_batch_queue_duration, max_len_hr_min_batch_queue_duration ) || ' / ' ||
      lpad( hr_max_batch_queue_duration, max_len_hr_max_batch_queue_duration ) || ' / ' ||
      lpad( hr_med_batch_queue_duration, max_len_hr_med_batch_queue_duration ) || ' / ' ||
      lpad( hr_avg_batch_queue_duration, max_len_hr_avg_batch_queue_duration ) || ' / ' ||
      lpad( hr_std_batch_queue_duration, max_len_hr_std_batch_queue_duration ) || ' / ' ||
      lpad( hr_sum_batch_queue_duration, max_len_hr_sum_batch_queue_duration ) )::varchar          as "bid queuing",

    ( lpad( hr_min_batch_leader_duration, max_len_hr_min_batch_leader_duration ) || ' / ' ||
      lpad( hr_max_batch_leader_duration, max_len_hr_max_batch_leader_duration ) || ' / ' ||
      lpad( hr_med_batch_leader_duration, max_len_hr_med_batch_leader_duration ) || ' / ' ||
      lpad( hr_avg_batch_leader_duration, max_len_hr_avg_batch_leader_duration ) || ' / ' ||
      lpad( hr_std_batch_leader_duration, max_len_hr_std_batch_leader_duration ) || ' / ' ||
      lpad( hr_sum_batch_leader_duration, max_len_hr_sum_batch_leader_duration ) )::varchar          as "ldr duration",

    ( lpad( hr_min_batch_worker_duration, max_len_hr_min_batch_worker_duration ) || ' / ' ||
      lpad( hr_max_batch_worker_duration, max_len_hr_max_batch_worker_duration ) || ' / ' ||
      lpad( hr_med_batch_worker_duration, max_len_hr_med_batch_worker_duration ) || ' / ' ||
      lpad( hr_avg_batch_worker_duration, max_len_hr_avg_batch_worker_duration ) || ' / ' ||
      lpad( hr_std_batch_worker_duration, max_len_hr_std_batch_worker_duration ) || ' / ' ||
      lpad( hr_sum_batch_worker_duration, max_len_hr_sum_batch_worker_duration ) )::varchar          as "wrk duration",

    ( lpad( hr_min_number_batch_transactions, max_len_hr_min_number_batch_transactions ) || ' / ' ||
      lpad( hr_max_number_batch_transactions, max_len_hr_max_number_batch_transactions ) || ' / ' ||
      lpad( hr_med_number_batch_transactions, max_len_hr_med_number_batch_transactions ) || ' / ' ||
      lpad( hr_avg_number_batch_transactions, max_len_hr_avg_number_batch_transactions ) || ' / ' ||
      lpad( hr_std_number_batch_transactions, max_len_hr_std_number_batch_transactions ) || ' / ' ||
      lpad( hr_sum_number_batch_transactions, max_len_hr_sum_number_batch_transactions ) )::varchar  as "xids/bid",

    ( lpad( hr_min_number_batch_blocks, max_len_hr_min_number_batch_blocks ) || ' / ' ||
      lpad( hr_max_number_batch_blocks, max_len_hr_max_number_batch_blocks ) || ' / ' ||
      lpad( hr_med_number_batch_blocks, max_len_hr_med_number_batch_blocks ) || ' / ' ||
      lpad( hr_avg_number_batch_blocks, max_len_hr_avg_number_batch_blocks ) || ' / ' ||
      lpad( hr_std_number_batch_blocks, max_len_hr_std_number_batch_blocks ) || ' / ' ||
      lpad( hr_sum_number_batch_blocks, max_len_hr_sum_number_batch_blocks ) )::varchar              as "blks/bid",

    number_batches::int8                                                                                    as bids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_slices_by_day
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    slice_ordinal::int2,
    start_ts::timestamp,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_days
     left join cte_aggr         using( start_ts, slice_ordinal )
     left join cte_bcast        using( start_ts, slice_ordinal )
     left join cte_delete       using( start_ts, slice_ordinal )
     left join cte_dist         using( start_ts, slice_ordinal )
     left join cte_hash         using( start_ts, slice_ordinal )
     left join cte_hashjoin     using( start_ts, slice_ordinal )
     left join cte_insert       using( start_ts, slice_ordinal )
     left join cte_limit        using( start_ts, slice_ordinal )
     left join cte_merge        using( start_ts, slice_ordinal )
     left join cte_mergejoin    using( start_ts, slice_ordinal )
     left join cte_nestloop     using( start_ts, slice_ordinal )
     left join cte_parse        using( start_ts, slice_ordinal )
     left join cte_project      using( start_ts, slice_ordinal )
     left join cte_return       using( start_ts, slice_ordinal )
     left join cte_save         using( start_ts, slice_ordinal )
     left join cte_sort         using( start_ts, slice_ordinal )
     left join cte_spatial_join using( start_ts, slice_ordinal )
     left join cte_unique       using( start_ts, slice_ordinal )
     left join cte_window       using( start_ts, slice_ordinal )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_all_by_minute
as
  with
    cte_aggr as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime)  as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_minutes.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_minutes
     left join cte_aggr_stats             using ( node_type, start_ts )
     left join cte_bcast_stats            using ( node_type, start_ts )
     left join cte_delete_stats           using ( node_type, start_ts )
     left join cte_dist_stats             using ( node_type, start_ts )
     left join cte_hash_stats             using ( node_type, start_ts )
     left join cte_hashjoin_stats         using ( node_type, start_ts )
     left join cte_insert_stats           using ( node_type, start_ts )
     left join cte_limit_stats            using ( node_type, start_ts )
     left join cte_merge_stats            using ( node_type, start_ts )
     left join cte_mergejoin_stats        using ( node_type, start_ts )
     left join cte_nestloop_stats         using ( node_type, start_ts )
     left join cte_parse_stats            using ( node_type, start_ts )
     left join cte_project_stats          using ( node_type, start_ts )
     left join cte_return_stats           using ( node_type, start_ts )
     left join cte_save_stats             using ( node_type, start_ts )
     left join cte_sort_stats             using ( node_type, start_ts )
     left join cte_spatial_join_stats     using ( node_type, start_ts )
     left join cte_unique_stats           using ( node_type, start_ts )
     left join cte_window_stats           using ( node_type, start_ts );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_minute_complete
as
  select
    node_type::varchar,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_minute
  order by
    node_type,
    start_ts;



create or replace procedure
  ro.sp_revoke_version_from_user( version in varchar(6), user_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'revoke usage on schema base_' || version || ' from ' || user_name;
      execute sql;

      sql := 'revoke usage on schema lf_' || version || ' from ' || user_name;
      execute sql;

      sql := 'revoke usage on schema sf_' || version || ' from ' || user_name;
      execute sql;

      sql := 'revoke select on all tables in schema base_' || version || ' from ' || user_name;
      execute sql;

      sql := 'revoke select on all tables in schema lf_' || version || ' from ' || user_name;
      execute sql;

      sql := 'revoke select on all tables in schema sf_' || version || ' from ' || user_name;
      execute sql;

      sql := 'revoke execute on all functions  in schema base_' || version || ' from ' || user_name;
      execute sql;

      sql := 'revoke execute on all procedures in schema lf_' || version || ' from ' || user_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.queries_for_worker
as
  select
    user_id::int4,
    user_name::varchar,
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    queue_id::int2,
    queue_name::varchar,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_queue_duration::interval,
    query_compile_exec_duration::interval,
    query_wlm_duration::interval,
    query_total_duration::interval,
    query_abort_flag::bool,
    query_csc_flag::bool,
    query_leader_node_flag::bool,
    query_rc_flag::bool,
    query_text::varchar
  from
         base_241129.queries_for_worker
    join base_241129.fixed_pg_user                      using ( user_id )
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  sf_241129.queries_for_worker
as
  select
    user_name::varchar                                               as uuser,
    session_id::int4                                                 as pid,
    transaction_id::int8                                             as xid,
    query_id::int4                                                   as qid,
    queue_name::varchar                                              as qname,
    query_start_ts::timestamp                                        as start_ts,
    query_end_ts::timestamp                                          as end_ts,
    extract( epoch from query_queue_duration )::numeric(18,2)        as queue,
    extract( epoch from query_compile_exec_duration )::numeric(18,2) as cexec,
    extract( epoch from query_total_duration )::numeric(18,2)        as total,
    (
      case when query_abort_flag = true then 'A' else ' ' end || 
      case when query_csc_flag   = true then 'C' else ' ' end ||
      case when query_rc_flag    = true then 'R' else ' ' end 
    )::varchar                                                       as flags,
    left( ltrim(query_text), 48 )::varchar                           as ttext
  from
    lf_241129.queries_for_worker
  order by
    query_start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_second_complete
as
  select
    start_ts::timestamp,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_second
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  lf_241129.ffunctions_text
as
  select
    schema_id::int4,
    schema_name::varchar,
    function_id::int4,
    function_name::varchar,
    function_text::varchar
  from
         base_241129.fixed_pg_proc_for_functions
    join base_241129.fixed_pg_namespace using ( schema_id );


create or replace view
  sf_241129.ffunctions_text
as
  select
    function_id::int4      as fid,
    function_text::varchar as ttext
  from
    lf_241129.ffunctions_text
  order by
    function_id;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_all_by_all
as
  select
    count(*)::int4 as number_abort
  from
    base_241129.fixed_stl_query
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_users_by_minute
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'minute', query_queue_start_ts )        as start_ts,
        query_wlm_start_ts   - query_start_ts        as leadin_duration,
        query_queue_start_ts - query_wlm_start_ts    as wlm_leadin_duration,
        query_queue_end_ts   - query_queue_start_ts  as query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration   as query_exec_duration,
        query_wlm_end_ts     - query_compile_exec_end_ts as wlm_leadout_duration,
        query_end_ts         - query_wlm_end_ts          as leadout_duration,
        query_total_duration,
        query_total_duration - leadin_duration - wlm_leadin_duration - query_queue_duration + query_compile_duration - query_exec_duration - wlm_leadout_duration - leadout_duration as unaccounted_duration
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query         using ( query_id, user_id )
        left join base_241129.queries_compilation using ( query_id, user_id )
      where
            query_abort_flag    = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        user_id,
        start_ts,

        min( leadin_duration ) over ( partition by user_id, start_ts )                              as min_leadin_duration,
        max( leadin_duration ) over ( partition by user_id, start_ts )                              as max_leadin_duration,
        median( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts )     as med_leadin_duration,
        avg( leadin_duration ) over ( partition by user_id, start_ts )                              as avg_leadin_duration,
        stddev_pop( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts ) as std_leadin_duration,
        sum( leadin_duration ) over ( partition by user_id, start_ts )                              as sum_leadin_duration,

        min( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadin_duration,
        max( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadin_duration,
        median( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadin_duration,
        avg( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadin_duration,
        stddev_pop( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadin_duration,
        sum( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadin_duration,

        min( query_queue_duration ) over ( partition by user_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by user_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by user_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by user_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by user_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by user_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by user_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by user_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by user_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by user_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by user_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by user_id, start_ts )                            as sum_query_exec_duration,

        min( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadout_duration,
        max( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadout_duration,
        median( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadout_duration,
        avg( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadout_duration,
        stddev_pop( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadout_duration,
        sum( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadout_duration,

        min( leadout_duration ) over ( partition by user_id, start_ts )                              as min_leadout_duration,
        max( leadout_duration ) over ( partition by user_id, start_ts )                              as max_leadout_duration,
        median( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts )     as med_leadout_duration,
        avg( leadout_duration ) over ( partition by user_id, start_ts )                              as avg_leadout_duration,
        stddev_pop( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts ) as std_leadout_duration,
        sum( leadout_duration ) over ( partition by user_id, start_ts )                              as sum_leadout_duration,

        min( query_total_duration ) over ( partition by user_id, start_ts )                              as min_query_total_duration,
        max( query_total_duration ) over ( partition by user_id, start_ts )                              as max_query_total_duration,
        median( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts )     as med_query_total_duration,
        avg( query_total_duration ) over ( partition by user_id, start_ts )                              as avg_query_total_duration,
        stddev_pop( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts ) as std_query_total_duration,
        sum( query_total_duration ) over ( partition by user_id, start_ts )                              as sum_query_total_duration,

        min( unaccounted_duration ) over ( partition by user_id, start_ts )                              as min_unaccounted_duration,
        max( unaccounted_duration ) over ( partition by user_id, start_ts )                              as max_unaccounted_duration,
        median( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts )     as med_unaccounted_duration,
        avg( unaccounted_duration ) over ( partition by user_id, start_ts )                              as avg_unaccounted_duration,
        stddev_pop( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts ) as std_unaccounted_duration,
        sum( unaccounted_duration ) over ( partition by user_id, start_ts )                              as sum_unaccounted_duration,

        count(*) over ( partition by user_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    cte_queries_stats.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,

    min_leadin_duration::interval,
    max_leadin_duration::interval,
    med_leadin_duration::float4,
    avg_leadin_duration::interval,
    std_leadin_duration::float4,
    sum_leadin_duration::interval,

    min_wlm_leadin_duration::interval,
    max_wlm_leadin_duration::interval,
    med_wlm_leadin_duration::float4,
    avg_wlm_leadin_duration::interval,
    std_wlm_leadin_duration::float4,
    sum_wlm_leadin_duration::interval,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    min_wlm_leadout_duration::interval,
    max_wlm_leadout_duration::interval,
    med_wlm_leadout_duration::float4,
    avg_wlm_leadout_duration::interval,
    std_wlm_leadout_duration::float4,
    sum_wlm_leadout_duration::interval,

    min_leadout_duration::interval,
    max_leadout_duration::interval,
    med_leadout_duration::float4,
    avg_leadout_duration::interval,
    std_leadout_duration::float4,
    sum_leadout_duration::interval,

    min_query_total_duration::interval,
    max_query_total_duration::interval,
    med_query_total_duration::float4,
    avg_query_total_duration::interval,
    std_query_total_duration::float4,
    sum_query_total_duration::interval,

    min_unaccounted_duration::interval,
    max_unaccounted_duration::interval,
    med_unaccounted_duration::float4,
    avg_unaccounted_duration::interval,
    std_unaccounted_duration::float4,
    sum_unaccounted_duration::interval,

    number_queries::int4
  from
              base_241129.generate_minutes
    left join cte_queries_stats     using ( start_ts )
    left join base_241129.fixed_pg_user using ( user_id )
  group by
    cte_queries_stats.user_id,
    user_name,
    start_ts,
    min_leadin_duration,
    max_leadin_duration,
    med_leadin_duration,
    avg_leadin_duration,
    std_leadin_duration,
    sum_leadin_duration,
    min_wlm_leadin_duration,
    max_wlm_leadin_duration,
    med_wlm_leadin_duration,
    avg_wlm_leadin_duration,
    std_wlm_leadin_duration,
    sum_wlm_leadin_duration,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    min_wlm_leadout_duration,
    max_wlm_leadout_duration,
    med_wlm_leadout_duration,
    avg_wlm_leadout_duration,
    std_wlm_leadout_duration,
    sum_wlm_leadout_duration,
    min_leadout_duration,
    max_leadout_duration,
    med_leadout_duration,
    avg_leadout_duration,
    std_leadout_duration,
    sum_leadout_duration,
    min_query_total_duration,
    max_query_total_duration,
    med_query_total_duration,
    avg_query_total_duration,
    std_query_total_duration,
    sum_query_total_duration,
    min_unaccounted_duration,
    max_unaccounted_duration,
    med_unaccounted_duration,
    avg_unaccounted_duration,
    std_unaccounted_duration,
    sum_unaccounted_duration,
    number_queries;


create or replace view
  lf_241129.vacuum_totals_for_all_by_all
as
  -- MG2 : recluster and reindex cannot skip
  --       is_recluster flag looks brolen (not set by auto-vacuum when it silently converts a vacuum to be a recluster?)
  --       but it also then appears that some vacuum I issue are silently converted to recluster
  --       and their status is irrelevant; it's not what actually happens
  --       right now, I simply ignore is_recluster

  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
    ),

    cte_number_vacuums as
    (
      select
        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
    )

  select
    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,
    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
               cte_number_vacuums
    cross join cte_total_duration;


create or replace view
  lf_241129.ttables_zonemap
as
  with
    cte_fixed_stv_blocklist as
    (
      select
        stv_blocklist.tbl      as table_id,
        stv_blocklist.col + 1  as column_ordinal,

        ( (x'000000000000007F'::int8 & stv_blocklist.minvalue) << (7*7 - 0*8) ) | ( (x'0000000000007F00'::int8 & stv_blocklist.minvalue) << (6*7 - 1*8) ) | ( (x'00000000007F0000'::int8 & stv_blocklist.minvalue) << (5*7 - 2*8) ) | ( (x'000000007F000000'::int8 & stv_blocklist.minvalue) << (4*7 - 3*8) ) | ( (x'0000007F00000000'::int8 & stv_blocklist.minvalue) >> (4*8 - 3*7) ) | ( (x'00007F0000000000'::int8 & stv_blocklist.minvalue) >> (5*8 - 2*7) ) | ( (x'007F000000000000'::int8 & stv_blocklist.minvalue) >> (6*8 - 1*7) ) | ( (x'7F00000000000000'::int8 & stv_blocklist.minvalue) >> (7*8 - 0*7) ) as char_block_sortkey_signed_minvalue,
        ( (x'000000000000007F'::int8 & stv_blocklist.maxvalue) << (7*7 - 0*8) ) | ( (x'0000000000007F00'::int8 & stv_blocklist.maxvalue) << (6*7 - 1*8) ) | ( (x'00000000007F0000'::int8 & stv_blocklist.maxvalue) << (5*7 - 2*8) ) | ( (x'000000007F000000'::int8 & stv_blocklist.maxvalue) << (4*7 - 3*8) ) | ( (x'0000007F00000000'::int8 & stv_blocklist.maxvalue) >> (4*8 - 3*7) ) | ( (x'00007F0000000000'::int8 & stv_blocklist.maxvalue) >> (5*8 - 2*7) ) | ( (x'007F000000000000'::int8 & stv_blocklist.maxvalue) >> (6*8 - 1*7) ) | ( (x'7F00000000000000'::int8 & stv_blocklist.maxvalue) >> (7*8 - 0*7) ) as char_block_sortkey_signed_maxvalue,

        ( (x'00000000000000FF'::int8 & stv_blocklist.minvalue) << (7*8) ) | ( (x'000000000000FF00'::int8 & stv_blocklist.minvalue) << (5*8) ) | ( (x'0000000000FF0000'::int8 & stv_blocklist.minvalue) << (3*8) ) | ( (x'00000000FF000000'::int8 & stv_blocklist.minvalue) << (1*8) ) | ( (x'000000FF00000000'::int8 & stv_blocklist.minvalue) >> (1*8) ) | ( (x'0000FF0000000000'::int8 & stv_blocklist.minvalue) >> (3*8) ) | ( (x'00FF000000000000'::int8 & stv_blocklist.minvalue) >> (5*8) ) | ( (x'FF00000000000000'::int8 & stv_blocklist.minvalue) >> (7*8) )                                                 as varchar_block_sortkey_signed_minvalue,
        ( (x'00000000000000FF'::int8 & stv_blocklist.maxvalue) << (7*8) ) | ( (x'000000000000FF00'::int8 & stv_blocklist.maxvalue) << (5*8) ) | ( (x'0000000000FF0000'::int8 & stv_blocklist.maxvalue) << (3*8) ) | ( (x'00000000FF000000'::int8 & stv_blocklist.maxvalue) << (1*8) ) | ( (x'000000FF00000000'::int8 & stv_blocklist.maxvalue) >> (1*8) ) | ( (x'0000FF0000000000'::int8 & stv_blocklist.maxvalue) >> (3*8) ) | ( (x'00FF000000000000'::int8 & stv_blocklist.maxvalue) >> (5*8) ) | ( (x'FF00000000000000'::int8 & stv_blocklist.maxvalue) >> (7*8) )                                                 as varchar_block_sortkey_signed_maxvalue,

        stv_blocklist.minvalue as other_block_sortkey_signed_minvalue,
        stv_blocklist.maxvalue as other_block_sortkey_signed_maxvalue,

        char_block_sortkey_signed_minvalue::numeric(20,0) as char_block_sortkey_sanitized_unsigned_minvalue,
        char_block_sortkey_signed_maxvalue::numeric(20,0) as char_block_sortkey_sanitized_unsigned_maxvalue,

        case
          -- MG2 : make negative varchar values unsigned
          when varchar_block_sortkey_signed_minvalue <  0 then 18446744073709551616::numeric(20,0) + varchar_block_sortkey_signed_minvalue::numeric(20,0)
                                                          else                                       varchar_block_sortkey_signed_minvalue::numeric(20,0)
        end                    as varchar_block_sortkey_sanitized_unsigned_minvalue,
        case
          -- MG2 : make negative varchar values unsigned
          when varchar_block_sortkey_signed_maxvalue <  0 then 18446744073709551616::numeric(20,0) + varchar_block_sortkey_signed_maxvalue::numeric(20,0)
                                                          else                                       varchar_block_sortkey_signed_maxvalue::numeric(20,0)
        end                    as varchar_block_sortkey_sanitized_unsigned_maxvalue,

        char_block_sortkey_signed_minvalue::numeric(20,0) as other_block_sortkey_sanitized_unsigned_minvalue,
        char_block_sortkey_signed_maxvalue::numeric(20,0) as other_block_sortkey_sanitized_unsigned_maxvalue,

        char_block_sortkey_sanitized_unsigned_maxvalue    - char_block_sortkey_sanitized_unsigned_minvalue    + 1 as char_block_sortkey_sanitized_unsigned_range,
        varchar_block_sortkey_sanitized_unsigned_maxvalue - varchar_block_sortkey_sanitized_unsigned_minvalue + 1 as varchar_block_sortkey_sanitized_unsigned_range,
        other_block_sortkey_sanitized_unsigned_maxvalue   - other_block_sortkey_sanitized_unsigned_minvalue   + 1 as other_block_sortkey_sanitized_unsigned_range
      from
        stv_blocklist
    ),

    cte_blocklist_columns as
    (
      select
        table_id,
        column_ordinal,

        min( char_block_sortkey_sanitized_unsigned_minvalue ) as min_char_block_sortkey_sanitized_unsigned_minvalue,
        max( char_block_sortkey_sanitized_unsigned_maxvalue ) as max_char_block_sortkey_sanitized_unsigned_maxvalue,
        max( char_block_sortkey_sanitized_unsigned_maxvalue ) - min( char_block_sortkey_sanitized_unsigned_minvalue ) + 1 as char_column_sortkey_sanitized_unsigned_range,

        min( varchar_block_sortkey_sanitized_unsigned_minvalue ) as min_varchar_block_sortkey_sanitized_unsigned_minvalue,
        max( varchar_block_sortkey_sanitized_unsigned_maxvalue ) as max_varchar_block_sortkey_sanitized_unsigned_maxvalue,
        max( varchar_block_sortkey_sanitized_unsigned_maxvalue ) - min( varchar_block_sortkey_sanitized_unsigned_minvalue ) + 1 as varchar_column_sortkey_sanitized_unsigned_range,

        min( other_block_sortkey_sanitized_unsigned_minvalue ) as min_other_block_sortkey_sanitized_unsigned_minvalue,
        max( other_block_sortkey_sanitized_unsigned_maxvalue ) as max_other_block_sortkey_sanitized_unsigned_maxvalue,
        max( other_block_sortkey_sanitized_unsigned_maxvalue ) - min( other_block_sortkey_sanitized_unsigned_minvalue ) + 1 as other_column_sortkey_sanitized_unsigned_range
      from
        cte_fixed_stv_blocklist
      group by
        table_id,
        column_ordinal
    ),

    cte_table_column_datatypes as
    (
      select
        pg_class.oid                                                                                                  as table_id,
        pg_attribute.attnum                                                                                           as column_ordinal,
        case
          when pg_type.typname in ( 'bpchar', 'char' ) then 'char'
          when left(pg_type.typname, 1) = '_'          then substring(pg_type.typname from 2) || '[]'
                                                       else trim( pg_type.typname )::varchar
        end                                                                                                           as column_base_data_type_name,
        case
          when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then (pg_attribute.atttypmod-4) & 65535
          when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null -- MG2 : variable length, so unknown
        end                                                                                                           as column_precision
      from
                  pg_class
             join pg_attribute on pg_attribute.attrelid      = pg_class.oid and
                                  pg_attribute.attnum   not in ( -1, -3, -4, -5, -6, -7 )
             join pg_type      on pg_type.oid                = pg_attribute.atttypid
        left join pg_attrdef   on pg_attrdef.adrelid         = pg_class.oid and
                                  pg_attribute.attnum        = pg_attrdef.adnum
      where
        pg_class.relkind = 'r'
    ),

    cte_tables_columns_blocks as
    (
      select
        table_id,
        column_ordinal,
        case
          -- MG2 : float4 and float8 can overflow
          when column_base_data_type_name in ( 'float4', 'float8' ) and ( cte_fixed_stv_blocklist.other_block_sortkey_sanitized_unsigned_maxvalue =  9223372036854775808 and cte_fixed_stv_blocklist.other_block_sortkey_sanitized_unsigned_minvalue =  9223372036854775808 ) then -3 -- 'overflow'

          -- MG2 : float4 and float8 can underflow
          when column_base_data_type_name in ( 'float4', 'float8' ) and ( cte_fixed_stv_blocklist.other_block_sortkey_sanitized_unsigned_maxvalue = -9223372036854775808 and cte_fixed_stv_blocklist.other_block_sortkey_sanitized_unsigned_minvalue = -9223372036854775808 ) then -2 -- 'underflow'

          -- MG2 : bpchar (which is really char(256), char, 16 byte numeric and varchar can all saturate
          when column_base_data_type_name in ('bpchar', 'char')  and ( cte_fixed_stv_blocklist.char_block_sortkey_sanitized_unsigned_maxvalue = cte_fixed_stv_blocklist.char_block_sortkey_sanitized_unsigned_minvalue )                                                      then -1 -- 'saturated'

          -- MG2 : bpchar (which is really char(256), char, 16 byte numeric and varchar can all saturate
          when column_base_data_type_name in ('text', 'varchar') and ( cte_fixed_stv_blocklist.varchar_block_sortkey_sanitized_unsigned_maxvalue = cte_fixed_stv_blocklist.varchar_block_sortkey_sanitized_unsigned_minvalue )                                                then -1 -- 'saturated'

          -- MG2 : bpchar (which is really char(256), char, 16 byte numeric and varchar can all saturate
          when ( column_base_data_type_name = 'numeric' and column_precision >= 20 ) and ( cte_fixed_stv_blocklist.other_block_sortkey_sanitized_unsigned_maxvalue = cte_fixed_stv_blocklist.other_block_sortkey_sanitized_unsigned_minvalue )                                then -1 -- 'saturated'

          -- MG2 : now for the normal situations

          when column_base_data_type_name in ('bpchar', 'char')  then round( cte_fixed_stv_blocklist.char_block_sortkey_sanitized_unsigned_range    / cte_blocklist_columns.char_column_sortkey_sanitized_unsigned_range    * 100 )
          when column_base_data_type_name in ('text', 'varchar') then round( cte_fixed_stv_blocklist.varchar_block_sortkey_sanitized_unsigned_range / cte_blocklist_columns.varchar_column_sortkey_sanitized_unsigned_range * 100 )
          else round( cte_fixed_stv_blocklist.other_block_sortkey_sanitized_unsigned_range / cte_blocklist_columns.other_column_sortkey_sanitized_unsigned_range * 100 )
        end as block_sortkey_range_percentage
      from
                     cte_table_column_datatypes
        natural join cte_blocklist_columns
        natural join cte_fixed_stv_blocklist
    ),

    cte_tables_zonemap as
    (
      select
        schema_id,
        schema_name,
        table_id,
        table_name,
        column_ordinal,
        column_name,
        column_base_data_type_name,
        column_full_data_type_name,
        column_encoding_name,
        sum( case when block_sortkey_range_percentage >=  0 and block_sortkey_range_percentage <   20 then 1 else 0 end )::int8 as number_00_20_blocks,
        sum( case when block_sortkey_range_percentage >= 20 and block_sortkey_range_percentage <   40 then 1 else 0 end )::int8 as number_20_40_blocks,
        sum( case when block_sortkey_range_percentage >= 40 and block_sortkey_range_percentage <   60 then 1 else 0 end )::int8 as number_40_60_blocks,
        sum( case when block_sortkey_range_percentage >= 60 and block_sortkey_range_percentage <   80 then 1 else 0 end )::int8 as number_60_80_blocks,
        sum( case when block_sortkey_range_percentage >= 80 and block_sortkey_range_percentage <= 100 then 1 else 0 end )::int8 as number_80_100_blocks,
        sum( case when block_sortkey_range_percentage = -3 then 1 else 0 end )::int8                                            as number_overflow_blocks,
        sum( case when block_sortkey_range_percentage = -2 then 1 else 0 end )::int8                                            as number_underflow_blocks,
        sum( case when block_sortkey_range_percentage = -1 then 1 else 0 end )::int8                                            as number_saturated_blocks
      from
                     base_241129.ttables_columns_basic_info
        natural join cte_tables_columns_blocks
      group by
        schema_id,
        schema_name,
        table_id,
        table_name,
        column_ordinal,
        column_name,
        column_base_data_type_name,
        column_full_data_type_name,
        column_encoding_name
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    column_ordinal::int2,
    column_name::varchar,
    column_full_data_type_name::varchar,
    column_encoding_name::varchar,
    number_00_20_blocks::int8,
    number_20_40_blocks::int8,
    number_40_60_blocks::int8,
    number_60_80_blocks::int8,
    number_80_100_blocks::int8,
    number_overflow_blocks::int8,
    number_underflow_blocks::int8,
    number_saturated_blocks::int8
  from
    cte_tables_zonemap;


create or replace view
  lf_241129.queries_stats_durations_on_both_by_transaction
as
  with
    cte_queries as
    (
      select
        transaction_id,
        query_id,
        query_queue_duration,
        query_compile_duration,
        case
          when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then ( query_compile_exec_duration )
          when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then ( query_compile_exec_duration - query_compile_duration )
        end                                                                                         as query_exec_duration,
        query_start_ts - lag( query_end_ts ) over ( partition by transaction_id order by query_id ) as idle_duration
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, transaction_id )
        left join base_241129.queries_compilation   using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        transaction_id,

        min( query_queue_duration ) over ( partition by transaction_id )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by transaction_id )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by transaction_id )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by transaction_id )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by transaction_id ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by transaction_id )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by transaction_id )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by transaction_id )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by transaction_id )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by transaction_id )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by transaction_id ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by transaction_id )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by transaction_id )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by transaction_id )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by transaction_id )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by transaction_id )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by transaction_id ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by transaction_id )                            as sum_query_exec_duration,

        min( idle_duration ) over ( partition by transaction_id )                            as min_idle_duration,
        max( idle_duration ) over ( partition by transaction_id )                            as max_idle_duration,
        median( extract(epoch from idle_duration) ) over ( partition by transaction_id )     as med_idle_duration,
        avg( idle_duration ) over ( partition by transaction_id )                            as avg_idle_duration,
        stddev_pop( extract(epoch from idle_duration) ) over ( partition by transaction_id ) as std_idle_duration,
        sum( idle_duration ) over ( partition by transaction_id )                            as sum_idle_duration,

        count(*) over ( partition by transaction_id ) as number_queries
      from
        cte_queries
    )

  select
    transaction_id::int8,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    min_idle_duration::interval,
    max_idle_duration::interval,
    med_idle_duration::float4,
    avg_idle_duration::interval,
    std_idle_duration::float4,
    sum_idle_duration::interval,

    number_queries::int4
  from
    cte_queries_stats
  group by
    transaction_id,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    min_idle_duration,
    max_idle_duration,
    med_idle_duration,
    avg_idle_duration,
    std_idle_duration,
    sum_idle_duration,
    number_queries;


create or replace view
  lf_241129.groups
as
  -- MG2 : we could just use `any`, but this procuces a cross join under the hood

  with
    cte_groups_users as
    (
      select
        grosysid,
        generate_series( 1, array_upper(grolist,1) ) as user_array_index
      from
        pg_group
    )

  select
    pg_group.grosysid::int4           as group_id,
    trim( pg_group.groname )::varchar as group_name,
    sum( case when pg_user.usesysid is not null then 1 else 0 end )::int4 as group_number_users
  from
              pg_group
    left join cte_groups_users using ( grosysid )
    left join pg_user             on pg_user.usesysid = pg_group.grolist[ cte_groups_users.user_array_index ]
  group by
    group_id,
    group_name;


create or replace view
  sf_241129.groups
as
  select
    group_name::varchar      as ggroup,
    group_number_users::int4 as users
  from
    lf_241129.groups
  order by
    group_name;


create or replace view
  lf_241129.nodes
as
  -- MG2 : as of 2024-10-11, I've noticed ra3.xlplus nodes using a single partition
  --       for both local and k-safety, so the 1owner = host` code no longer workds
  --       as such now, I add up all storage (and usage) and divide by 2

  with
    cte_nodes as
    (
      select
        node                                          as node_ordinal,
        sum( case when type = 'C' then 1 else 0 end ) as number_partial_slices,
        sum( case when type = 'D' then 1 else 0 end ) as number_full_slices,
        count( * )                                    as number_total_slices
      from
        stv_slices
      group by
        node
    ),

    cte_node_disk_blocks as
    (
      select
        host                    as node_ordinal,
        count( distinct mount ) as number_local_disks,
        sum( used )             as number_used_local_blocks,
        sum( capacity )         as total_disk_blocks
      from
        -- MG2 : this does not include/show table 0
        stv_partitions
      where
        owner = host
      group by
        host
    ),

    cte_node_disk_and_rms_blocks as
    (
      select
        node as node_ordinal,
        used as number_used_local_and_rms_blocks
      from
        stv_node_storage_capacity
    )

  select
    cte_nodes.node_ordinal::int2,
    cte_nodes.number_partial_slices::int2,
    cte_nodes.number_full_slices::int2,
    cte_nodes.number_total_slices::int2,

    cte_node_disk_blocks.number_local_disks::int2,
    cte_node_disk_blocks.total_disk_blocks::int8,
    cte_node_disk_blocks.number_used_local_blocks::int8,

    -- MG2 : stv_node_storage_capacity.used is total disk used plus total RMS used
    ( cte_node_disk_and_rms_blocks.number_used_local_and_rms_blocks - cte_node_disk_blocks.number_used_local_blocks )::int8 as number_used_rms_blocks,

    cte_node_disk_and_rms_blocks.number_used_local_and_rms_blocks::int8 as number_total_blocks_used
  from
                  cte_nodes
     natural join cte_node_disk_blocks
     natural join cte_node_disk_and_rms_blocks;


create or replace view
  sf_241129.nodes
as
  select
    node_ordinal::int2                                                as node,
    ( number_partial_slices || ' / ' || number_full_slices )::varchar as slices,
    number_local_disks::int2                                          as disks,
    base_241129.hr_blks( total_disk_blocks        )::varchar                 as total_blocks,
    base_241129.hr_blks( number_used_local_blocks )::varchar                 as disk_used,
    base_241129.hr_blks( number_used_rms_blocks   )::varchar                 as rms_used,
    base_241129.hr_blks( number_total_blocks_used )::varchar                 as total_used
  from
    lf_241129.nodes
  order by
    node_ordinal;


create or replace view
  sf_241129.sessions_full
as
  select
    user_name::varchar                                    as uuser,
    session_id::int4                                      as pid,
    truncated_database_name::varchar                      as tdb,
    extract( epoch from session_duration )::numeric(18,2) as dur,
    number_locks::int4                                    as locks,
    case
      when remoteport is     null then remotehost
      when remoteport is not null then remotehost || ':' || remoteport
    end::varchar                                          as ip
  from
    lf_241129.sessions_full
  order by
    user_id,
    user_name,
    truncated_database_name,
    session_duration;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_queues_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_queues_by_minute
    )

  select
    queue_name::varchar as qname,
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    queue_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_users_by_second
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        userid                         as user_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_insert as
    (
      select
        userid                         as user_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_scan as
    (
      select
        userid                         as user_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    )

  select
    base_241129.fixed_pg_user.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
               base_241129.fixed_pg_user
    cross join base_241129.generate_seconds
     left join cte_delete using ( start_ts, user_id )
     left join cte_insert using ( start_ts, user_id )
     left join cte_scan   using ( start_ts, user_id )
  where
       cte_delete.user_id is not null
    or cte_insert.user_id is not null
    or cte_scan.user_id   is not null;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_query
as
  select
    query_id::int4,
    query_start_ts::timestamp,
    1::int4 as number_abort
  from
    base_241129.fixed_stl_query
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_query
as
  select
    query_id::int4 as qid,
    query_start_ts::timestamp,
    number_abort::int4 as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_query
  order by
    query_id;


create or replace view
  lf_241129.steps_totals_data_on_both_for_all_by_minute
as
  with
    cte_aggr as
    (
      select
        date_trunc( 'minute', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        date_trunc( 'minute', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        date_trunc( 'minute', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        date_trunc( 'minute', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        date_trunc( 'minute', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        date_trunc( 'minute', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        date_trunc( 'minute', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        date_trunc( 'minute', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        date_trunc( 'minute', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_minutes.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               cte_node_types
    cross join base_241129.generate_minutes
     left join cte_aggr         using ( node_type, start_ts )
     left join cte_bcast        using ( node_type, start_ts )
     left join cte_delete       using ( node_type, start_ts )
     left join cte_dist         using ( node_type, start_ts )
     left join cte_hash         using ( node_type, start_ts )
     left join cte_hashjoin     using ( node_type, start_ts )
     left join cte_insert       using ( node_type, start_ts )
     left join cte_limit        using ( node_type, start_ts )
     left join cte_merge        using ( node_type, start_ts )
     left join cte_mergejoin    using ( node_type, start_ts )
     left join cte_nestloop     using ( node_type, start_ts )
     left join cte_parse        using ( node_type, start_ts )
     left join cte_project      using ( node_type, start_ts )
     left join cte_return       using ( node_type, start_ts )
     left join cte_save         using ( node_type, start_ts )
     left join cte_sort         using ( node_type, start_ts )
     left join cte_spatial_join using ( node_type, start_ts )
     left join cte_unique       using ( node_type, start_ts )
     left join cte_window       using ( node_type, start_ts );


create or replace view
  sf_241129.steps_totals_data_on_both_for_all_by_minute
as
  select
    node_type::varchar as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_all_by_minute
  order by
    node_type,
    start_ts;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_users_by_all
as
  select
    user_id::int4,
    user_name::varchar,
    count(*)::int4 as number_abort
  from
              base_241129.fixed_stl_query
    left join base_241129.fixed_pg_user using ( user_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    user_id,
    user_name;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_users_by_all
as
  select
    user_name::varchar as uuser,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_users_by_all
  order by
    user_name;


create or replace view
  lf_241129.steps_totals_data_on_both_for_slices_by_all
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal
    )

  select
    slice_ordinal::int2,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               base_241129.fixed_stv_slices
     left join cte_aggr         using( slice_ordinal )
     left join cte_bcast        using( slice_ordinal )
     left join cte_delete       using( slice_ordinal )
     left join cte_dist         using( slice_ordinal )
     left join cte_hash         using( slice_ordinal )
     left join cte_hashjoin     using( slice_ordinal )
     left join cte_insert       using( slice_ordinal )
     left join cte_limit        using( slice_ordinal )
     left join cte_merge        using( slice_ordinal )
     left join cte_mergejoin    using( slice_ordinal )
     left join cte_nestloop     using( slice_ordinal )
     left join cte_parse        using( slice_ordinal )
     left join cte_project      using( slice_ordinal )
     left join cte_return       using( slice_ordinal )
     left join cte_save         using( slice_ordinal )
     left join cte_sort         using( slice_ordinal )
     left join cte_spatial_join using( slice_ordinal )
     left join cte_unique       using( slice_ordinal )
     left join cte_window       using( slice_ordinal )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.analyze_totals_for_database_by_second
as
  with
    cte_hr_formatting as
    (
      select
        database_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_database_by_second
    )

  select
    database_name::varchar as dbname,
    start_ts::timestamp,

    ( hr_number_manual_full_analyze             || ' / ' ||
      hr_number_manual_predictatecolumn_analyze || ' / ' ||
      hr_number_manual_skipped_analyze          )::varchar as manual,

    ( hr_number_automatic_full_analyze             || ' / ' ||
      hr_number_automatic_predictatecolumn_analyze || ' / ' ||
      hr_number_automatic_skipped_analyze          )::varchar as automatic,

    ( hr_number_autoanalyze_full_analyze             || ' / ' ||
      hr_number_autoanalyze_predictatecolumn_analyze || ' / ' ||
      hr_number_autoanalyze_skipped_analyze          )::varchar as autoanalyze,

    ( hr_number_total_full_analyze             || ' / ' ||
      hr_number_total_predictatecolumn_analyze || ' / ' ||
      hr_number_total_skipped_analyze          )::varchar as total
  from
    cte_hr_formatting
  order by
    database_name,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_users_by_day
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'day', query_queue_start_ts )        as start_ts,
        query_wlm_start_ts   - query_start_ts        as leadin_duration,
        query_queue_start_ts - query_wlm_start_ts    as wlm_leadin_duration,
        query_queue_end_ts   - query_queue_start_ts  as query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration   as query_exec_duration,
        query_wlm_end_ts     - query_compile_exec_end_ts as wlm_leadout_duration,
        query_end_ts         - query_wlm_end_ts          as leadout_duration,
        query_total_duration,
        query_total_duration - leadin_duration - wlm_leadin_duration - query_queue_duration + query_compile_duration - query_exec_duration - wlm_leadout_duration - leadout_duration as unaccounted_duration
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query         using ( query_id, user_id )
        left join base_241129.queries_compilation using ( query_id, user_id )
      where
            query_abort_flag    = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        user_id,
        start_ts,

        min( leadin_duration ) over ( partition by user_id, start_ts )                              as min_leadin_duration,
        max( leadin_duration ) over ( partition by user_id, start_ts )                              as max_leadin_duration,
        median( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts )     as med_leadin_duration,
        avg( leadin_duration ) over ( partition by user_id, start_ts )                              as avg_leadin_duration,
        stddev_pop( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts ) as std_leadin_duration,
        sum( leadin_duration ) over ( partition by user_id, start_ts )                              as sum_leadin_duration,

        min( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadin_duration,
        max( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadin_duration,
        median( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadin_duration,
        avg( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadin_duration,
        stddev_pop( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadin_duration,
        sum( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadin_duration,

        min( query_queue_duration ) over ( partition by user_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by user_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by user_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by user_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by user_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by user_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by user_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by user_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by user_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by user_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by user_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by user_id, start_ts )                            as sum_query_exec_duration,

        min( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadout_duration,
        max( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadout_duration,
        median( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadout_duration,
        avg( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadout_duration,
        stddev_pop( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadout_duration,
        sum( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadout_duration,

        min( leadout_duration ) over ( partition by user_id, start_ts )                              as min_leadout_duration,
        max( leadout_duration ) over ( partition by user_id, start_ts )                              as max_leadout_duration,
        median( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts )     as med_leadout_duration,
        avg( leadout_duration ) over ( partition by user_id, start_ts )                              as avg_leadout_duration,
        stddev_pop( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts ) as std_leadout_duration,
        sum( leadout_duration ) over ( partition by user_id, start_ts )                              as sum_leadout_duration,

        min( query_total_duration ) over ( partition by user_id, start_ts )                              as min_query_total_duration,
        max( query_total_duration ) over ( partition by user_id, start_ts )                              as max_query_total_duration,
        median( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts )     as med_query_total_duration,
        avg( query_total_duration ) over ( partition by user_id, start_ts )                              as avg_query_total_duration,
        stddev_pop( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts ) as std_query_total_duration,
        sum( query_total_duration ) over ( partition by user_id, start_ts )                              as sum_query_total_duration,

        min( unaccounted_duration ) over ( partition by user_id, start_ts )                              as min_unaccounted_duration,
        max( unaccounted_duration ) over ( partition by user_id, start_ts )                              as max_unaccounted_duration,
        median( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts )     as med_unaccounted_duration,
        avg( unaccounted_duration ) over ( partition by user_id, start_ts )                              as avg_unaccounted_duration,
        stddev_pop( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts ) as std_unaccounted_duration,
        sum( unaccounted_duration ) over ( partition by user_id, start_ts )                              as sum_unaccounted_duration,

        count(*) over ( partition by user_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    cte_queries_stats.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,

    min_leadin_duration::interval,
    max_leadin_duration::interval,
    med_leadin_duration::float4,
    avg_leadin_duration::interval,
    std_leadin_duration::float4,
    sum_leadin_duration::interval,

    min_wlm_leadin_duration::interval,
    max_wlm_leadin_duration::interval,
    med_wlm_leadin_duration::float4,
    avg_wlm_leadin_duration::interval,
    std_wlm_leadin_duration::float4,
    sum_wlm_leadin_duration::interval,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    min_wlm_leadout_duration::interval,
    max_wlm_leadout_duration::interval,
    med_wlm_leadout_duration::float4,
    avg_wlm_leadout_duration::interval,
    std_wlm_leadout_duration::float4,
    sum_wlm_leadout_duration::interval,

    min_leadout_duration::interval,
    max_leadout_duration::interval,
    med_leadout_duration::float4,
    avg_leadout_duration::interval,
    std_leadout_duration::float4,
    sum_leadout_duration::interval,

    min_query_total_duration::interval,
    max_query_total_duration::interval,
    med_query_total_duration::float4,
    avg_query_total_duration::interval,
    std_query_total_duration::float4,
    sum_query_total_duration::interval,

    min_unaccounted_duration::interval,
    max_unaccounted_duration::interval,
    med_unaccounted_duration::float4,
    avg_unaccounted_duration::interval,
    std_unaccounted_duration::float4,
    sum_unaccounted_duration::interval,

    number_queries::int4
  from
              base_241129.generate_days
    left join cte_queries_stats     using ( start_ts )
    left join base_241129.fixed_pg_user using ( user_id )
  group by
    cte_queries_stats.user_id,
    user_name,
    start_ts,
    min_leadin_duration,
    max_leadin_duration,
    med_leadin_duration,
    avg_leadin_duration,
    std_leadin_duration,
    sum_leadin_duration,
    min_wlm_leadin_duration,
    max_wlm_leadin_duration,
    med_wlm_leadin_duration,
    avg_wlm_leadin_duration,
    std_wlm_leadin_duration,
    sum_wlm_leadin_duration,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    min_wlm_leadout_duration,
    max_wlm_leadout_duration,
    med_wlm_leadout_duration,
    avg_wlm_leadout_duration,
    std_wlm_leadout_duration,
    sum_wlm_leadout_duration,
    min_leadout_duration,
    max_leadout_duration,
    med_leadout_duration,
    avg_leadout_duration,
    std_leadout_duration,
    sum_leadout_duration,
    min_query_total_duration,
    max_query_total_duration,
    med_query_total_duration,
    avg_query_total_duration,
    std_query_total_duration,
    sum_query_total_duration,
    min_unaccounted_duration,
    max_unaccounted_duration,
    med_unaccounted_duration,
    avg_unaccounted_duration,
    std_unaccounted_duration,
    sum_unaccounted_duration,
    number_queries;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_query
as
  select
    query_id::int4                                          as qid,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_query
  order by
    query_id;


create or replace procedure
  ro.sp_grant_versionless_sf_schema_to_public()
as
  $proc$
    begin
      grant usage on schema sf to public;
      grant select on all tables in schema sf to public;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.queries_totals_result_cache_for_all_by_minute
as
  select
    date_trunc( 'minute', query_start_ts )::timestamp                                                           as start_ts,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
    count(*)::int8                                                                                           as number_total,
    ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
  from
              base_241129.fixed_stl_query
    left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
  where
        query_abort_flag  = false
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_users_by_minute
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        userid                         as user_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_insert as
    (
      select
        userid                         as user_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_scan as
    (
      select
        userid                         as user_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    )

  select
    base_241129.fixed_pg_user.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
               base_241129.fixed_pg_user
    cross join base_241129.generate_minutes
     left join cte_delete using ( start_ts, user_id )
     left join cte_insert using ( start_ts, user_id )
     left join cte_scan   using ( start_ts, user_id )
  where
       cte_delete.user_id is not null
    or cte_insert.user_id is not null
    or cte_scan.user_id   is not null;


create or replace view
  lf_241129.views_late_binding_columns
as
  select
    pg_class.relnamespace::int4           as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4                    as view_id,
    trim( pg_class.relname )::varchar     as view_name,
    pg_attribute.attnum::int2             as column_ordinal,
    trim( pg_attribute.attname )::varchar as column_name,
    pg_attribute.atttypid::int4           as column_data_type_id,
    case
      when pg_type.typname in ( 'bpchar', 'char' )  then 'char'
      when left(pg_type.typname, 1) = '_'           then substring(pg_type.typname from 2) || '[]'
                                                    else pg_type.typname::varchar
    end::varchar                          as column_base_data_type_name,
    case
      when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod != -1 then pg_attribute.atttypmod-4
      when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod  = -1 then 1::int4
    end::int4                             as column_octet_length,
    case
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then ( (pg_attribute.atttypmod-4) >> 16 )::int2
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null::int2 -- MG2 : variable length, so unknown
    end::int2                             as column_precision,
    case
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then ( (pg_attribute.atttypmod-4) & 65535 )::int2
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null::int2 -- MG2 : variable length, so unknown
    end::int2                             as column_scale,
    case
      when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
      when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')' || '[]'
      when pg_type.typname = '_numeric'              and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')' || '[]'
      when pg_type.typname = '_numeric'              and pg_attribute.atttypmod  = -1 then 'numeric(variable)[]'
      when pg_type.typname = '_varchar'              and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
      when pg_type.typname = '_varchar'              and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')' || '[]'

      when pg_type.typname in ( 'bpchar', 'char' ) and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      when pg_type.typname in ( 'bpchar', 'char' ) and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')'
      when pg_type.typname = 'numeric'             and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')'
      when pg_type.typname = 'numeric'             and pg_attribute.atttypmod  = -1 then 'numeric(variable)'
      when pg_type.typname = 'varchar'             and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      when pg_type.typname = 'varchar'             and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')'

      when pg_type.typname = 'varbyte'             and pg_attribute.atttypmod != -1 then 'varbyte' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      -- MG2 : not seen pg_attribute.atttypmod = -1 for 'varbyte'

      when left(pg_type.typname, 1) = '_' then substring(trim(pg_type.typname) from 2) || '[]'
                                          else trim(pg_type.typname)::varchar
    end::varchar                          as column_full_data_type_name
  from
              pg_class
         join pg_namespace                                                on pg_namespace.oid = pg_class.relnamespace
    left join base_241129.fixed_pg_get_late_binding_view_cols as pg_attribute on pg_attribute.attrelid = pg_class.oid
         join pg_type                                                     on pg_type.oid = pg_attribute.atttypid
    left join stv_mv_info  on     stv_mv_info.db_name = current_database()
                              and stv_mv_info.schema  = pg_namespace.nspname
                              and stv_mv_info.name    = pg_class.relname
  where
        pg_class.relkind   = 'v'
    and pg_class.relnatts  = 0     -- MG2 : only late-binding views
    and stv_mv_info.name  is null; -- MG2 : no materialized views


create or replace view
  sf_241129.steps_totals_data_on_both_for_statements_by_second
as

  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                       as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                      as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                        as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                   as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                   as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                   as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                   as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                   as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                   as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                   as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                   as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                      as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                      as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_statements_by_second
  order by
    statement_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_users_by_hour
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        userid                         as user_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_insert as
    (
      select
        userid                         as user_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_scan as
    (
      select
        userid                         as user_id,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    )

  select
    base_241129.fixed_pg_user.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
               base_241129.fixed_pg_user
    cross join base_241129.generate_hours
     left join cte_delete using ( start_ts, user_id )
     left join cte_insert using ( start_ts, user_id )
     left join cte_scan   using ( start_ts, user_id )
  where
       cte_delete.user_id is not null
    or cte_insert.user_id is not null
    or cte_scan.user_id   is not null;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_minute_complete
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_minute
  order by
    user_name,
    node_type,
    start_ts;



create or replace view
  lf_241129.views_materialized_table_blocks_rows
as
  -- MG2 : do not use materialized views, ever
  --       the underlying implementation is no good
  --       and the internal design is an anti-pattern for sorted relational databases
  --       you've been warned

  select
    lf_241129.views_materialized_table.schema_id::int4,
    lf_241129.views_materialized_table.schema_name::varchar,
    lf_241129.views_materialized_table.view_id::int4,
    lf_241129.views_materialized_table.view_name::varchar,
    lf_241129.views_materialized_table.user_id::int4,
    lf_241129.views_materialized_table.user_name::varchar,
    lf_241129.views_materialized_table.failures::varchar,
    lf_241129.views_materialized_table.autorefresh::varchar,
    lf_241129.views_materialized_table.refresh_type::varchar,
    lf_241129.views_materialized_table.table_id::int4,
    lf_241129.views_materialized_table.table_name::varchar,
    lf_241129.views_materialized_table.table_creation::timestamp,
    lf_241129.views_materialized_table.table_age::interval,
    lf_241129.views_materialized_table.table_number_columns::int2,
    lf_241129.views_materialized_table.table_sort_type::varchar,
    lf_241129.views_materialized_table.table_number_sortkeys::int2,
    lf_241129.views_materialized_table.table_distribution_type::varchar,
    lf_241129.views_materialized_table.table_distribution_auto_flag::bool,
    base_241129.ttables_blocks_rows.number_slices::int2,
    base_241129.ttables_blocks_rows.number_sorted_blocks::int8,
    base_241129.ttables_blocks_rows.number_unsorted_blocks::int8,
    base_241129.ttables_blocks_rows.number_total_blocks::int8,
    base_241129.ttables_blocks_rows.number_sorted_rows::int8,
    base_241129.ttables_blocks_rows.number_unsorted_rows::int8,
    base_241129.ttables_blocks_rows.number_total_rows::int8
  from
              lf_241129.views_materialized_table
    left join base_241129.ttables_blocks_rows using ( table_id );


create or replace view
  sf_241129.views_materialized_table_blocks_rows
as
  select
    schema_name::varchar          as sschema,
    view_name::varchar            as vview,
    user_name::varchar            as uuser,
    (
      case when failures     != 'none'        then 'f' else ' ' end
      ||
      case when autorefresh   = 'automatic'   then 'a' else 'm' end
      ||
      case when refresh_type  = 'unavailable' then 'u' else ' ' end
      ||
      case when refresh_type  = 'full'        then 'f' else 'i' end
    )::varchar                    as status,
    table_name::varchar           as ttable,
    table_age::interval           as age,
    table_number_columns::int2    as cols,
    table_sort_type::varchar      as ssort,
    table_number_sortkeys::int2   as skeys,
    case
      when table_distribution_auto_flag = True  then table_distribution_type || ' (auto)'
      when table_distribution_auto_flag = false then table_distribution_type
    end::varchar                  as dist,
    number_slices::int2           as slices,
    number_sorted_blocks::int8    as sblks,
    number_unsorted_blocks::int8  as ublks,
    number_total_blocks::int8     as tblks,
    number_sorted_rows::int8      as srows,
    number_unsorted_rows::int8    as urows,
    number_total_rows::int8       as trows
  from
    lf_241129.views_materialized_table_blocks_rows
  order by
    schema_name,
    view_name;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_users_by_day
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        userid                         as user_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_insert as
    (
      select
        userid                         as user_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    ),

    cte_scan as
    (
      select
        userid                         as user_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid,
        start_ts
    )

  select
    base_241129.fixed_pg_user.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
               base_241129.fixed_pg_user
    cross join base_241129.generate_days
     left join cte_delete using ( start_ts, user_id )
     left join cte_insert using ( start_ts, user_id )
     left join cte_scan   using ( start_ts, user_id )
  where
       cte_delete.user_id is not null
    or cte_insert.user_id is not null
    or cte_scan.user_id   is not null;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_queues_by_hour
as
  with
    cte_s1 as
    (
      select
        query_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'hour', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        query_id,
        node_type,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
         cte_s1
    join base_241129.fixed_stl_wlm_query                using ( query_id )
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_queues_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_queues_by_hour
    )

  select
    queue_name::varchar as qname,
    node_type::varchar  as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    queue_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_queues_by_all
as
  with
    cte_s1 as
    (
      select
        query_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        query_id,
        node_type
    )

  select
    queue_id::int2,
    queue_name::varchar,
    node_type::varchar,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
         cte_s1
    join base_241129.fixed_stl_wlm_query                using ( query_id )
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    node_type;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_day_complete
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::date,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_day
  order by
    user_name,
    node_type,
    start_ts;



create or replace view
  sf_241129.wqueues_totals_slot_utilization_by_day_as_table
as
  select
    queue_name::varchar as qname,
    slot_number::int2   as slot,
    extract( epoch from max(case when start_ts = current_date   then duration else interval '0 second' end) )::numeric(18,2) as "today",
    extract( epoch from max(case when start_ts = current_date-1 then duration else interval '0 second' end) )::numeric(18,2) as "today-1",
    extract( epoch from max(case when start_ts = current_date-2 then duration else interval '0 second' end) )::numeric(18,2) as "today-2",
    extract( epoch from max(case when start_ts = current_date-3 then duration else interval '0 second' end) )::numeric(18,2) as "today-3",
    extract( epoch from max(case when start_ts = current_date-4 then duration else interval '0 second' end) )::numeric(18,2) as "today-4",
    extract( epoch from max(case when start_ts = current_date-5 then duration else interval '0 second' end) )::numeric(18,2) as "today-5"
  from
    lf_241129.wqueues_totals_slot_utilization_by_day
  group by
    queue_name,
    slot_number
  order by
    queue_name,
    slot_number;


create or replace view
  lf_241129.vacuum_stats_for_all_by_second
as
  with
    cte_xids as 
    (
      select
        sum( num_new_blocks      ) as sum_number_new_blocks,
        sum( num_blocks_replaced ) as sum_number_replaced_blocks,

        sum( num_new_rows        ) as sum_number_new_rows,
        sum( num_deleted_rows    ) as sum_number_deleted_rows,

        count(*)                   as number_qids,

        min( query_start_ts ) as min_query_start_ts,
        max( query_end_ts   ) as max_query_end_ts,

        max_query_end_ts - min_query_start_ts as duration
      from
             base_241129.fixed_stl_vacuum_detail
        join base_241129.fixed_stl_query using( query_id, transaction_id )
      where
            -- MG2 : procably we should include aborted - need a general approcch to this
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_end_ts     <  current_timestamp
      group by
        transaction_id
      having
        -- MG2 : ignore xids in partial first start_ts
        min_query_start_ts >= current_date - interval '6 day'
    ),

    cte_xids_stats as
    (
      select
        date_trunc( 'second', min_query_start_ts ) as start_ts,

        min( sum_number_new_blocks )        over () as min_sum_number_new_blocks,
        max( sum_number_new_blocks )        over () as max_sum_number_new_blocks,
        median( sum_number_new_blocks )     over () as med_sum_number_new_blocks,
        avg( sum_number_new_blocks )        over () as avg_sum_number_new_blocks,
        stddev_pop( sum_number_new_blocks ) over () as std_sum_number_new_blocks,
        sum( sum_number_new_blocks )        over () as sum_sum_number_new_blocks,

        min( sum_number_replaced_blocks )        over () as min_sum_number_replaced_blocks,
        max( sum_number_replaced_blocks )        over () as max_sum_number_replaced_blocks,
        median( sum_number_replaced_blocks )     over () as med_sum_number_replaced_blocks,
        avg( sum_number_replaced_blocks )        over () as avg_sum_number_replaced_blocks,
        stddev_pop( sum_number_replaced_blocks ) over () as std_sum_number_replaced_blocks,
        sum( sum_number_replaced_blocks )        over () as sum_sum_number_replaced_blocks,

        min( sum_number_new_rows )        over () as min_sum_number_new_rows,
        max( sum_number_new_rows )        over () as max_sum_number_new_rows,
        median( sum_number_new_rows )     over () as med_sum_number_new_rows,
        avg( sum_number_new_rows )        over () as avg_sum_number_new_rows,
        stddev_pop( sum_number_new_rows ) over () as std_sum_number_new_rows,
        sum( sum_number_new_rows )        over () as sum_sum_number_new_rows,

        min( sum_number_deleted_rows )        over () as min_sum_number_deleted_rows,
        max( sum_number_deleted_rows )        over () as max_sum_number_deleted_rows,
        median( sum_number_deleted_rows )     over () as med_sum_number_deleted_rows,
        avg( sum_number_deleted_rows )        over () as avg_sum_number_deleted_rows,
        stddev_pop( sum_number_deleted_rows ) over () as std_sum_number_deleted_rows,
        sum( sum_number_deleted_rows )        over () as sum_sum_number_deleted_rows,

        min( duration ) over ()                              as min_duration,
        max( duration ) over ()                              as max_duration,
        median( extract(epoch from (duration)) ) over ()     as med_duration,
        avg( duration ) over ()                              as avg_duration,
        stddev_pop( extract(epoch from (duration)) ) over () as std_duration,
        sum( duration ) over ()                              as sum_duration,

        sum( number_qids ) over () as number_qids,
        count(*) over ()           as number_xids
      from
        cte_xids
    )

  select
    start_ts::timestamp,

    min_sum_number_new_blocks::int8,
    max_sum_number_new_blocks::int8,
    med_sum_number_new_blocks::int8,
    avg_sum_number_new_blocks::float4,
    std_sum_number_new_blocks::float4,
    sum_sum_number_new_blocks::float4,

    min_sum_number_replaced_blocks::int8,
    max_sum_number_replaced_blocks::int8,
    med_sum_number_replaced_blocks::int8,
    avg_sum_number_replaced_blocks::float4,
    std_sum_number_replaced_blocks::float4,
    sum_sum_number_replaced_blocks::float4,

    min_sum_number_new_rows::int8,
    max_sum_number_new_rows::int8,
    med_sum_number_new_rows::int8,
    avg_sum_number_new_rows::float4,
    std_sum_number_new_rows::float4,
    sum_sum_number_new_rows::float4,

    min_sum_number_deleted_rows::int8,
    max_sum_number_deleted_rows::int8,
    med_sum_number_deleted_rows::int8,
    avg_sum_number_deleted_rows::float4,
    std_sum_number_deleted_rows::float4,
    sum_sum_number_deleted_rows::float4,

    min_duration::interval,
    max_duration::interval,
    med_duration::float4,
    avg_duration::interval,
    std_duration::float4,
    sum_duration::interval,

    number_qids::int8,
    number_xids::int8
  from
              base_241129.generate_seconds
    left join cte_xids_stats using ( start_ts )
  group by
    start_ts,
    min_sum_number_new_blocks,
    max_sum_number_new_blocks,
    med_sum_number_new_blocks,
    avg_sum_number_new_blocks,
    std_sum_number_new_blocks,
    sum_sum_number_new_blocks,
    min_sum_number_replaced_blocks,
    max_sum_number_replaced_blocks,
    med_sum_number_replaced_blocks,
    avg_sum_number_replaced_blocks,
    std_sum_number_replaced_blocks,
    sum_sum_number_replaced_blocks,
    min_sum_number_new_rows,
    max_sum_number_new_rows,
    med_sum_number_new_rows,
    avg_sum_number_new_rows,
    std_sum_number_new_rows,
    sum_sum_number_new_rows,
    min_sum_number_deleted_rows,
    max_sum_number_deleted_rows,
    med_sum_number_deleted_rows,
    avg_sum_number_deleted_rows,
    std_sum_number_deleted_rows,
    sum_sum_number_deleted_rows,
    min_duration,
    max_duration,
    med_duration,
    avg_duration,
    std_duration,
    sum_duration,
    number_qids,
    number_xids;


create or replace view
  sf_241129.vacuum_stats_for_all_by_second
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        base_241129.hr_number( min_sum_number_new_blocks ) as hr_min_sum_number_new_blocks,
        base_241129.hr_number( max_sum_number_new_blocks ) as hr_max_sum_number_new_blocks,
        base_241129.hr_number( med_sum_number_new_blocks ) as hr_med_sum_number_new_blocks,
        base_241129.hr_number( avg_sum_number_new_blocks ) as hr_avg_sum_number_new_blocks,
        base_241129.hr_number( std_sum_number_new_blocks ) as hr_std_sum_number_new_blocks,
        base_241129.hr_number( sum_sum_number_new_blocks ) as hr_sum_sum_number_new_blocks,

        base_241129.hr_number( min_sum_number_replaced_blocks ) as hr_min_sum_number_replaced_blocks,
        base_241129.hr_number( max_sum_number_replaced_blocks ) as hr_max_sum_number_replaced_blocks,
        base_241129.hr_number( med_sum_number_replaced_blocks ) as hr_med_sum_number_replaced_blocks,
        base_241129.hr_number( avg_sum_number_replaced_blocks ) as hr_avg_sum_number_replaced_blocks,
        base_241129.hr_number( std_sum_number_replaced_blocks ) as hr_std_sum_number_replaced_blocks,
        base_241129.hr_number( sum_sum_number_replaced_blocks ) as hr_sum_sum_number_replaced_blocks,

        base_241129.hr_number( min_sum_number_new_rows ) as hr_min_sum_number_new_rows,
        base_241129.hr_number( max_sum_number_new_rows ) as hr_max_sum_number_new_rows,
        base_241129.hr_number( med_sum_number_new_rows ) as hr_med_sum_number_new_rows,
        base_241129.hr_number( avg_sum_number_new_rows ) as hr_avg_sum_number_new_rows,
        base_241129.hr_number( std_sum_number_new_rows ) as hr_std_sum_number_new_rows,
        base_241129.hr_number( sum_sum_number_new_rows ) as hr_sum_sum_number_new_rows,

        base_241129.hr_number( min_sum_number_deleted_rows ) as hr_min_sum_number_deleted_rows,
        base_241129.hr_number( max_sum_number_deleted_rows ) as hr_max_sum_number_deleted_rows,
        base_241129.hr_number( med_sum_number_deleted_rows ) as hr_med_sum_number_deleted_rows,
        base_241129.hr_number( avg_sum_number_deleted_rows ) as hr_avg_sum_number_deleted_rows,
        base_241129.hr_number( std_sum_number_deleted_rows ) as hr_std_sum_number_deleted_rows,
        base_241129.hr_number( sum_sum_number_deleted_rows ) as hr_sum_sum_number_deleted_rows,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_sum_number_new_blocks) ) over () as max_len_hr_min_sum_number_new_blocks,
        max( len(hr_max_sum_number_new_blocks) ) over () as max_len_hr_max_sum_number_new_blocks,
        max( len(hr_med_sum_number_new_blocks) ) over () as max_len_hr_med_sum_number_new_blocks,
        max( len(hr_avg_sum_number_new_blocks) ) over () as max_len_hr_avg_sum_number_new_blocks,
        max( len(hr_std_sum_number_new_blocks) ) over () as max_len_hr_std_sum_number_new_blocks,
        max( len(hr_sum_sum_number_new_blocks) ) over () as max_len_hr_sum_sum_number_new_blocks,

        max( len(hr_min_sum_number_replaced_blocks) ) over () as max_len_hr_min_sum_number_replaced_blocks,
        max( len(hr_max_sum_number_replaced_blocks) ) over () as max_len_hr_max_sum_number_replaced_blocks,
        max( len(hr_med_sum_number_replaced_blocks) ) over () as max_len_hr_med_sum_number_replaced_blocks,
        max( len(hr_avg_sum_number_replaced_blocks) ) over () as max_len_hr_avg_sum_number_replaced_blocks,
        max( len(hr_std_sum_number_replaced_blocks) ) over () as max_len_hr_std_sum_number_replaced_blocks,
        max( len(hr_sum_sum_number_replaced_blocks) ) over () as max_len_hr_sum_sum_number_replaced_blocks,

        max( len(hr_min_sum_number_new_rows) ) over () as max_len_hr_min_sum_number_new_rows,
        max( len(hr_max_sum_number_new_rows) ) over () as max_len_hr_max_sum_number_new_rows,
        max( len(hr_med_sum_number_new_rows) ) over () as max_len_hr_med_sum_number_new_rows,
        max( len(hr_avg_sum_number_new_rows) ) over () as max_len_hr_avg_sum_number_new_rows,
        max( len(hr_std_sum_number_new_rows) ) over () as max_len_hr_std_sum_number_new_rows,
        max( len(hr_sum_sum_number_new_rows) ) over () as max_len_hr_sum_sum_number_new_rows,

        max( len(hr_min_sum_number_deleted_rows) ) over () as max_len_hr_min_sum_number_deleted_rows,
        max( len(hr_max_sum_number_deleted_rows) ) over () as max_len_hr_max_sum_number_deleted_rows,
        max( len(hr_med_sum_number_deleted_rows) ) over () as max_len_hr_med_sum_number_deleted_rows,
        max( len(hr_avg_sum_number_deleted_rows) ) over () as max_len_hr_avg_sum_number_deleted_rows,
        max( len(hr_std_sum_number_deleted_rows) ) over () as max_len_hr_std_sum_number_deleted_rows,
        max( len(hr_sum_sum_number_deleted_rows) ) over () as max_len_hr_sum_sum_number_deleted_rows,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_qids,
        number_xids
      from
        lf_241129.vacuum_stats_for_all_by_second
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_sum_number_new_blocks, max_len_hr_min_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_new_blocks, max_len_hr_max_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_new_blocks, max_len_hr_med_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_new_blocks, max_len_hr_avg_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_new_blocks, max_len_hr_std_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_new_blocks, max_len_hr_sum_sum_number_new_blocks ) )::varchar as "new blocks",

    ( lpad( hr_min_sum_number_replaced_blocks, max_len_hr_min_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_replaced_blocks, max_len_hr_max_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_replaced_blocks, max_len_hr_med_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_replaced_blocks, max_len_hr_avg_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_replaced_blocks, max_len_hr_std_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_replaced_blocks, max_len_hr_sum_sum_number_replaced_blocks ) )::varchar as "replaced blocks",

    ( lpad( hr_min_sum_number_new_rows, max_len_hr_min_sum_number_new_rows ) || ' / ' ||
      lpad( hr_max_sum_number_new_rows, max_len_hr_max_sum_number_new_rows ) || ' / ' ||
      lpad( hr_med_sum_number_new_rows, max_len_hr_med_sum_number_new_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_new_rows, max_len_hr_avg_sum_number_new_rows ) || ' / ' ||
      lpad( hr_std_sum_number_new_rows, max_len_hr_std_sum_number_new_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_new_rows, max_len_hr_sum_sum_number_new_rows ) )::varchar as "new rows",

    ( lpad( hr_min_sum_number_deleted_rows, max_len_hr_min_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_max_sum_number_deleted_rows, max_len_hr_max_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_med_sum_number_deleted_rows, max_len_hr_med_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_deleted_rows, max_len_hr_avg_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_std_sum_number_deleted_rows, max_len_hr_std_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_deleted_rows, max_len_hr_sum_sum_number_deleted_rows ) )::varchar as "deleted rows",

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as "xid duration",

    number_qids::int8 as qids,
    number_xids::int8 as xids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_all_by_second
as
  with
    cte_transactions as
    (
      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        transaction_id,
        date_trunc( 'second', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        transaction_id
    )

  select
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
  group by
    start_ts;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_all_by_second
as
  select
    start_ts::timestamp,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_all_by_second
  order by
    start_ts;


create or replace view
  lf_241129.views_materialized_columns
as
  select
    pg_class.relnamespace::int4           as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4                    as view_id,
    trim( pg_class.relname )::varchar     as view_name,
    pg_attribute.attnum::int2             as column_ordinal,
    trim( pg_attribute.attname )::varchar as column_name,
    pg_attribute.atttypid::int4           as column_data_type_id,
    case
      when pg_type.typname in ( 'bpchar', 'char' )  then 'char'
      when left(pg_type.typname, 1) = '_'           then substring(pg_type.typname from 2) || '[]'
                                                    else pg_type.typname::varchar
    end::varchar                          as column_base_data_type_name,
    case
      when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod != -1 then pg_attribute.atttypmod-4
      when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod  = -1 then 1::int4
    end::int4                             as column_octet_length,
    case
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then ( (pg_attribute.atttypmod-4) >> 16 )::int2
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null::int2 -- MG2 : variable length, so unknown
    end::int2                             as column_precision,
    case
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then ( (pg_attribute.atttypmod-4) & 65535 )::int2
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null::int2 -- MG2 : variable length, so unknown
    end::int2                             as column_scale,
    case
      when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
      when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')' || '[]'
      when pg_type.typname = '_numeric'              and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')' || '[]'
      when pg_type.typname = '_numeric'              and pg_attribute.atttypmod  = -1 then 'numeric(variable)[]'
      when pg_type.typname = '_varchar'              and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
      when pg_type.typname = '_varchar'              and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')' || '[]'

      when pg_type.typname in ( 'bpchar', 'char' ) and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      when pg_type.typname in ( 'bpchar', 'char' ) and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')'
      when pg_type.typname = 'numeric'             and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')'
      when pg_type.typname = 'numeric'             and pg_attribute.atttypmod  = -1 then 'numeric(variable)'
      when pg_type.typname = 'varchar'             and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      when pg_type.typname = 'varchar'             and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')'

      when pg_type.typname = 'varbyte'             and pg_attribute.atttypmod != -1 then 'varbyte' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      -- MG2 : not seen pg_attribute.atttypmod = -1 for 'varbyte'

      when left(pg_type.typname, 1) = '_' then substring(trim(pg_type.typname) from 2) || '[]'
                                          else trim(pg_type.typname)::varchar
    end::varchar                          as column_full_data_type_name
  from
         pg_class
    join pg_namespace on pg_namespace.oid      = pg_class.relnamespace
    join pg_attribute on pg_attribute.attrelid = pg_class.oid
    join pg_type      on pg_type.oid           = pg_attribute.atttypid
    join stv_mv_info  on     stv_mv_info.db_name = current_database()
                         and stv_mv_info.schema  = pg_namespace.nspname
                         and stv_mv_info.name    = pg_class.relname
  where
        pg_class.relkind   = 'v'
    and pg_class.relnatts >  0         -- MG2 : no late-binding views
    and stv_mv_info.name  is not null; -- MG2 : only materialized views


create or replace view
  base_241129.ttables_activity_fast
as
  with
    cte_delete as
    (
      select
        stl_delete.tbl         as table_id,
        sum( stl_delete.rows ) as number_rows_deleted
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and starttime <= current_timestamp
      group by
        stl_delete.tbl
    ),

    cte_insert as
    (
      select
        stl_insert.tbl         as table_id,
        sum( stl_insert.rows ) as number_rows_inserted
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and starttime <= current_timestamp
      group by
        stl_insert.tbl
    )

  select
    table_id::int4,
    cte_delete.number_rows_deleted::int8,
    cte_insert.number_rows_inserted::int8
  from
              base_241129.fixed_pg_class_for_tables
    left join cte_delete using ( table_id )
    left join cte_insert using ( table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null;


create or replace view
  lf_241129.sschemas_blocks_rows_activity
as
  with
    cte_schemas_activity as
    (
      select
        pg_class.relnamespace                                    as schema_id,
        sum( base_241129.ttables_activity_fast.number_rows_deleted  ) as number_rows_deleted,
        sum( base_241129.ttables_activity_fast.number_rows_inserted ) as number_rows_inserted
      from
              base_241129.ttables_activity_fast
         join pg_class on base_241129.ttables_activity_fast.table_id = pg_class.oid
      where
        pg_class.relkind = 'r'
      group by
        schema_id
    )

  select
    lf_241129.sschemas_blocks_rows.schema_id::int4,
    lf_241129.sschemas_blocks_rows.schema_name::varchar,
    lf_241129.sschemas_blocks_rows.user_id::int4,
    lf_241129.sschemas_blocks_rows.user_name::varchar,
    lf_241129.sschemas_blocks_rows.number_tables::int4,
    lf_241129.sschemas_blocks_rows.number_normal_views::int4,
    lf_241129.sschemas_blocks_rows.number_late_binding_views::int4,
    lf_241129.sschemas_blocks_rows.number_materialized_views::int4,
    lf_241129.sschemas_blocks_rows.number_functions::int4,
    lf_241129.sschemas_blocks_rows.number_procedures::int4,
    lf_241129.sschemas_blocks_rows.number_sorted_blocks::int8,
    lf_241129.sschemas_blocks_rows.number_desorted_blocks::int8,
    lf_241129.sschemas_blocks_rows.number_unsorted_blocks::int8,
    lf_241129.sschemas_blocks_rows.number_total_blocks::int8,
    lf_241129.sschemas_blocks_rows.number_sorted_rows::int8,
    lf_241129.sschemas_blocks_rows.number_desorted_rows::int8,
    lf_241129.sschemas_blocks_rows.number_unsorted_rows::int8,
    lf_241129.sschemas_blocks_rows.number_total_rows::int8,
    cte_schemas_activity.number_rows_deleted::int8,
    cte_schemas_activity.number_rows_inserted::int8
  from
              lf_241129.sschemas_blocks_rows
    left join cte_schemas_activity using ( schema_id );


create or replace view
  sf_241129.sschemas_blocks_rows_activity
as
  select
    schema_name::varchar            as sschema,
    user_name::varchar              as uuser,
    number_tables::int4             as ttables,
    number_normal_views::int4       as nviews,
    number_late_binding_views::int4 as lviews,
    number_materialized_views::int4 as mviews,
    number_functions::int4          as funcs,
    number_procedures::int4         as procs,
    number_sorted_blocks::int8      as sblks,
    number_desorted_blocks::int8    as dsblks,
    number_unsorted_blocks::int8    as ublks,
    number_total_blocks::int8       as tblks,
    number_sorted_rows::int8        as srows,
    number_desorted_rows::int8      as dsrows,
    number_unsorted_rows::int8      as urows,
    number_total_rows::int8         as trows,
    number_rows_deleted::int8       as drops,
    number_rows_inserted::int8      as irows
  from
    lf_241129.sschemas_blocks_rows_activity
  order by
    schema_name;



create or replace view
  lf_241129.queries_totals_result_cache_for_all_by_day
as
  select
    date_trunc( 'day', query_start_ts )::timestamp                                                           as start_ts,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
    count(*)::int8                                                                                           as number_total,
    ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
  from
              base_241129.fixed_stl_query
    left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
  where
        query_abort_flag  = false
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_statements_by_day
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'aggr' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'bcast' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'delete' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'dist' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hash' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hashjoin' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'insert' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'limit' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'merge' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'mergejoin' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'nestloop_op' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'parse' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'project' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'return' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'save' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'sort' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'spatial_join' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'unique' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'window' as step_type,
        date_trunc( 'day', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_steps as
    (
      select * from cte_aggr         union all
      select * from cte_bcast        union all
      select * from cte_delete       union all
      select * from cte_dist         union all
      select * from cte_hash         union all
      select * from cte_hashjoin     union all
      select * from cte_insert       union all
      select * from cte_limit        union all
      select * from cte_merge        union all
      select * from cte_mergejoin    union all
      select * from cte_nestloop     union all
      select * from cte_parse        union all
      select * from cte_project      union all
      select * from cte_return       union all
      select * from cte_save         union all
      select * from cte_sort         union all
      select * from cte_spatial_join union all
      select * from cte_unique       union all
      select * from cte_window
    )

  select
    statement_name::varchar,
    node_type::varchar,
    step_type::varchar,
    start_ts::timestamp,

    min( duration )::interval                          as min_duration,
    max( duration )::interval                          as max_duration,
    median( extract(epoch from duration) )::float4     as med_duration,
    avg( duration )::interval                          as avg_duration,
    stddev_pop( extract(epoch from duration) )::float4 as std_duration,
    count(*)::int8                                     as number_steps
  from
                 cte_base
    natural join cte_steps
  group by
    statement_name,
    node_type,
    step_type,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_statements_by_day
as
  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    step_type::varchar      as step,
    start_ts::date,

    extract( epoch from min_duration )::numeric(18,2) as min,
    extract( epoch from max_duration )::numeric(18,2) as max,
    med_duration::numeric(18,2)                       as med,
    extract( epoch from avg_duration )::numeric(18,2) as avg,
    std_duration::numeric(18,2)                       as std,
    number_steps::int8                                as num
  from
    lf_241129.steps_stats_durations_on_both_for_statements_by_day
  order by
    statement_name,
    node_type,
    step_type,
    start_ts;



create or replace procedure
  ro.sp_revoke_version_from_role( version in varchar(6), role_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'revoke usage on schema base_' || version || ' from role ' || role_name;
      execute sql;

      sql := 'revoke usage on schema lf_' || version || ' from role ' || role_name;
      execute sql;

      sql := 'revoke usage on schema sf_' || version || ' from role ' || role_name;
      execute sql;

      sql := 'revoke select on all tables in schema base_' || version || ' from role ' || role_name;
      execute sql;

      sql := 'revoke select on all tables in schema lf_' || version || ' from role ' || role_name;
      execute sql;

      sql := 'revoke select on all tables in schema sf_' || version || ' from role ' || role_name;
      execute sql;

      sql := 'revoke execute on all functions  in schema base_' || version || ' from role ' || role_name;
      execute sql;

      sql := 'revoke execute on all procedures in schema lf_' || version || ' from role ' || role_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.steps_totals_data_on_both_for_statements_by_minute
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    statement_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               cte_base
    cross join cte_node_types
    cross join base_241129.generate_minutes
     left join cte_aggr         using ( start_ts, query, node_type )
     left join cte_bcast        using ( start_ts, query, node_type )
     left join cte_delete       using ( start_ts, query, node_type )
     left join cte_dist         using ( start_ts, query, node_type )
     left join cte_hash         using ( start_ts, query, node_type )
     left join cte_hashjoin     using ( start_ts, query, node_type )
     left join cte_insert       using ( start_ts, query, node_type )
     left join cte_limit        using ( start_ts, query, node_type )
     left join cte_merge        using ( start_ts, query, node_type )
     left join cte_mergejoin    using ( start_ts, query, node_type )
     left join cte_nestloop     using ( start_ts, query, node_type )
     left join cte_parse        using ( start_ts, query, node_type )
     left join cte_project      using ( start_ts, query, node_type )
     left join cte_return       using ( start_ts, query, node_type )
     left join cte_save         using ( start_ts, query, node_type )
     left join cte_sort         using ( start_ts, query, node_type )
     left join cte_spatial_join using ( start_ts, query, node_type )
     left join cte_unique       using ( start_ts, query, node_type )
     left join cte_window       using ( start_ts, query, node_type )
  group by
    statement_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.views_materialized_columns
as
  select
    schema_name::varchar                as sschema,
    view_name::varchar                  as vview,
    column_name::varchar                as ccolumn,
    column_full_data_type_name::varchar as data_type
  from
    lf_241129.views_materialized_columns
  order by
    schema_name,
    view_name,
    column_name;


create or replace view
  lf_241129.vacuum_totals_for_tables_by_second
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        table_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        table_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        table_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        table_id,
        date_trunc( 'second', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        table_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        table_id,
        date_trunc( 'second', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    cte_number_vacuums.table_id::int4,
    trim( pg_class.relname )::varchar     as table_name,

    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration                using ( table_id, start_ts )
    left join pg_class     on pg_class.oid          = cte_number_vacuums.table_id
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.vacuum_totals_for_all_by_hour
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        date_trunc( 'hour', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        date_trunc( 'hour', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        start_ts
    )

  select
    start_ts::timestamp,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( start_ts );


create or replace view
  sf_241129.vacuum_totals_for_all_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_all_by_hour
    )

  select
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.queries_totals_result_cache_for_all_by_all
as
  select
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
    count(*)::int8                                                                                           as number_total,
    ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
  from
              base_241129.fixed_stl_query
    left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
  where
        query_abort_flag  = false
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp;


create or replace view
  sf_241129.queries_totals_result_cache_for_all_by_all
as
  select
    number_hit::int8  as hit,
    number_miss::int8 as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_all_by_all;


create or replace view
  lf_241129.vacuum_totals_for_users_by_all
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        userid          as user_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        user_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        user_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        user_id,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        user_id
    ),

    cte_number_vacuums as
    (
      select
        user_id,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        user_id
    )

  select
    cte_number_vacuums.user_id::int4,
    trim( pg_user.usename )::varchar as user_name,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( user_id )
         join pg_user on pg_user.usesysid = cte_number_vacuums.user_id;


create or replace view
  sf_241129.vacuum_totals_for_users_by_all
as
  with
    cte_hr_formatting as
    (
      select
        user_name,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_users_by_all
    )

  select
    user_name::varchar as uuser,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    user_name;


create or replace view
  lf_241129.steps_totals_data_on_both_for_users_by_all
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_bcast as
    (
      select
        userid as usesysid,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_project as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_return as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_save as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    -- scan

    cte_sort as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_window as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4           as user_id,
    trim( pg_user.usename )::varchar as user_name,
    cte_node_types.node_type::varchar,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               pg_user
    cross join cte_node_types
     left join cte_aggr         using( node_type, usesysid )
     left join cte_bcast        using( node_type, usesysid )
     left join cte_delete       using( node_type, usesysid )
     left join cte_dist         using( node_type, usesysid )
     left join cte_hash         using( node_type, usesysid )
     left join cte_hashjoin     using( node_type, usesysid )
     left join cte_insert       using( node_type, usesysid )
     left join cte_limit        using( node_type, usesysid )
     left join cte_merge        using( node_type, usesysid )
     left join cte_mergejoin    using( node_type, usesysid )
     left join cte_nestloop     using( node_type, usesysid )
     left join cte_parse        using( node_type, usesysid )
     left join cte_project      using( node_type, usesysid )
     left join cte_return       using( node_type, usesysid )
     left join cte_save         using( node_type, usesysid )
     left join cte_sort         using( node_type, usesysid )
     left join cte_spatial_join using( node_type, usesysid )
     left join cte_unique       using( node_type, usesysid )
     left join cte_window       using( node_type, usesysid )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.steps_totals_data_on_both_for_users_by_all
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_users_by_all
  order by
    user_name,
    node_type;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_tables_by_hour
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'hour', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    cte_s1.table_id::int4,
    trim( pg_class.relname )::varchar      as table_name,
    node_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r';


create or replace view
  sf_241129.steps_totals_data_on_both_for_node_by_hour
as
  select
    node_ordinal::int2 as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_node_by_hour
  order by
    node_ordinal,
    start_ts;


create or replace view
  lf_241129.vacuum_totals_for_all_by_day
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        date_trunc( 'day', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        date_trunc( 'day', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        start_ts
    )

  select
    start_ts::timestamp,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( start_ts );


create or replace view
  sf_241129.vacuum_totals_for_all_by_day
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_all_by_day
    )

  select
    start_ts::date,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_all_by_day
as
  select
    start_ts::date,
    number_abort::int4 as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_all_by_day
  order by
    start_ts;


create or replace view
  sf_241129.queries_text_on_workers_fast
as
  select
    session_id::int4       as pid,
    transaction_id::int8   as xid,
    query_id::int4         as qid,
    query_start_ts::timestamp as start_ts,
    trim( chr(10) || ' ' from query_text )::varchar as ttext
  from
    lf_241129.queries_text_on_workers_fast
  order by
    query_id;


create or replace view
  base_241129.fixed_pg_class_for_views
as
  select
    object_id::int4      as view_id,
    object_name::varchar as view_name,
    schema_id::int4,
    user_id::int4,
    relnatts::int2
  from
    base_241129.fixed_pg_class
  where
    relkind = 'v';


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_users_by_all
as
  with
    cte_s1 as
    (
      select
        user_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        user_id,
        node_type
    )

  select
    user_id::int4,
    node_type::varchar,
    trim( pg_user.usename )::varchar as user_name,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_user on cte_s1.user_id = pg_user.usesysid;


create or replace view
  lf_241129.ttables_columns
as
  with
    cte_tables_columns as
    (
      select
        base_241129.ttables_columns_basic_info.schema_id,
        base_241129.ttables_columns_basic_info.schema_name,
        base_241129.ttables_columns_basic_info.table_id,
        base_241129.ttables_columns_basic_info.table_name,
        base_241129.ttables_columns_basic_info.column_ordinal,
        base_241129.ttables_columns_basic_info.column_name,
        base_241129.ttables_columns_basic_info.column_data_type_id,
        base_241129.ttables_columns_basic_info.column_base_data_type_name,
        base_241129.ttables_columns_basic_info.column_octet_length,
        base_241129.ttables_columns_basic_info.column_precision,
        base_241129.ttables_columns_basic_info.column_scale,
        base_241129.ttables_columns_basic_info.column_full_data_type_name,
        base_241129.ttables_columns_basic_info.column_encoding_id,
        base_241129.ttables_columns_basic_info.column_encoding_name,
        base_241129.ttables_columns_basic_info.column_sortkey_ordinal,
        base_241129.ttables_columns_basic_info.column_distkey,
        base_241129.ttables_columns_basic_info.column_notnull,
        case when pg_constraint.contype = 'p' then true else false end                                                as column_primary_key,
        case when pg_constraint.contype = 'u' then true else false end                                                as column_unique,
        case when pg_class_reference.oid is not null then true else false end                                         as column_foreign_key,
        case
          when column_full_data_type_name in ('int8', 'int4') and substring(pg_get_expr(adbin, adrelid), 1, 16) = 'default_identity' then True
          when column_full_data_type_name in ('int8', 'int4') and substring(pg_get_expr(adbin, adrelid), 1, 10) = '"identity"'       then True
          when pg_get_expr(adbin, adrelid) is not null                                                                               then True
                                                                                                                                     else False
        end                                                                                                           as column_default,
        case
          when column_full_data_type_name in ('int8', 'int4') and substring(pg_get_expr(adbin, adrelid), 1, 16) = 'default_identity' then 'generated by default as identity'
          when column_full_data_type_name in ('int8', 'int4') and substring(pg_get_expr(adbin, adrelid), 1, 10) = '"identity"'       then 'identity'
          when pg_get_expr(adbin, adrelid) is not null                                                                               then 'default'
                                                                                                                                     else null
        end                                                                                                           as column_default_type,
        case
          when column_full_data_type_name in ('int8', 'int4') and substring(pg_get_expr(adbin, adrelid), 1, 16) = 'default_identity' then split_part( adsrc, '''', 2 )
          when column_full_data_type_name in ('int8', 'int4') and substring(pg_get_expr(adbin, adrelid), 1, 10) = '"identity"'       then split_part( adsrc, '''', 2 )
          when pg_get_expr(adbin, adrelid) is not null                                                                               then adsrc
                                                                                                                                     else null
        end                                                                                                           as column_default_value,
        pg_class_reference.relnamespace                                                                               as column_references_schema_id,
        pg_namespace_reference.nspname                                                                                as column_references_schema_name,
        pg_class_reference.oid                                                                                        as column_references_table_id,
        pg_class_reference.relname                                                                                    as column_references_table_name,
        pg_attribute_reference.attnum                                                                                 as column_references_column_ordinal,
        pg_attribute_reference.attname                                                                                as column_references_column_name
      from
                  base_241129.ttables_columns_basic_info
        left join pg_attrdef                               on pg_attrdef.adrelid                       = base_241129.ttables_columns_basic_info.table_id and
                                                              pg_attrdef.adnum                         = base_241129.ttables_columns_basic_info.column_ordinal
        left join pg_constraint                            on pg_constraint.conrelid                   = base_241129.ttables_columns_basic_info.table_id and
                                                              array_upper(pg_constraint.conkey,1)      = 1            and
                                                              pg_constraint.contype                   != 'f'          and
                                                              pg_constraint.conkey[1]                  = base_241129.ttables_columns_basic_info.column_ordinal
        left join pg_constraint as pg_constraint_reference on pg_constraint_reference.conrelid         = base_241129.ttables_columns_basic_info.table_id and
                                                              pg_constraint_reference.contype          = 'f'          and
                                                              pg_constraint_reference.conkey[1]        = base_241129.ttables_columns_basic_info.column_ordinal
        left join pg_class     as pg_class_reference       on pg_class_reference.oid                   = pg_constraint_reference.confrelid
        left join pg_namespace as pg_namespace_reference   on pg_namespace_reference.oid               = pg_class_reference.relnamespace
        left join pg_attribute as pg_attribute_reference   on     pg_attribute_reference.attrelid    = pg_constraint_reference.confrelid
                                                              and pg_constraint_reference.contype    = 'f'
                                                              and pg_constraint_reference.confkey[1] = pg_attribute_reference.attnum
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    column_ordinal::int2,
    column_name::varchar,
    column_data_type_id::int4,
    column_base_data_type_name::varchar,
    column_octet_length::int4,
    column_precision::int2,
    column_scale::int2,
    column_full_data_type_name::varchar,
    column_encoding_id::int4,
    column_encoding_name::varchar,
    column_sortkey_ordinal::int2,
    column_distkey::bool,
    column_notnull::bool,
    column_primary_key::bool,
    column_unique::bool,
    column_foreign_key::bool,
    column_default::bool,
    column_default_type::varchar,
    column_default_value::varchar,
    column_references_schema_id::int4,
    column_references_schema_name::varchar,
    column_references_table_id::int4,
    column_references_table_name::varchar,
    column_references_column_ordinal::int2,
    column_references_column_name::varchar
  from
    cte_tables_columns;


create or replace view
  sf_241129.ttables_columns
as
  select
    schema_name::varchar                                                                        as sschema,
    table_name::varchar                                                                         as ttable,
    column_name::varchar                                                                        as ccolumn,
    column_full_data_type_name::varchar                                                         as ttype,
    column_encoding_name::varchar                                                               as enc,
    case when column_sortkey_ordinal >  0    then column_sortkey_ordinal else null end::int2    as sk,
    case when column_distkey          = true then 'dk'                   else null end::varchar as dk,
    case when column_notnull          = true then 'nn'                   else null end::varchar as nn,
    case when column_primary_key      = true then 'pk'                   else null end::varchar as pk,
    case when column_unique           = true then 'u'                    else null end::varchar as u
  from
    lf_241129.ttables_columns
  order by
    schema_name,
    table_name,
    column_name;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_users_by_hour
as
  with
    cte_transactions as
    (
      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        user_id,
        transaction_id,
        date_trunc( 'hour', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        user_id,
        transaction_id
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                                      as user_name,
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
    left join pg_user                        on cte_all_transactions.user_id        = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_tables_by_day
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_tables_by_day
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::date,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_all_by_all
as
  select
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day';


create or replace view
  sf_241129.analyze_totals_for_all_by_all
as
  with
    cte_hr_formatting as
    (
      select
        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_all_by_all
    )

  select
    ( hr_number_manual_full_analyze             || ' / ' ||
      hr_number_manual_predictatecolumn_analyze || ' / ' ||
      hr_number_manual_skipped_analyze          )::varchar as manual,

    ( hr_number_automatic_full_analyze             || ' / ' ||
      hr_number_automatic_predictatecolumn_analyze || ' / ' ||
      hr_number_automatic_skipped_analyze          )::varchar as automatic,

    ( hr_number_autoanalyze_full_analyze             || ' / ' ||
      hr_number_autoanalyze_predictatecolumn_analyze || ' / ' ||
      hr_number_autoanalyze_skipped_analyze          )::varchar as autoanalyze,

    ( hr_number_total_full_analyze             || ' / ' ||
      hr_number_total_predictatecolumn_analyze || ' / ' ||
      hr_number_total_skipped_analyze          )::varchar as total
  from
    cte_hr_formatting;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_all_by_minute
as
  select
    case
      when slice_ordinal  = -1 then 'leader'
      when slice_ordinal != -1 then 'worker'
    end::varchar as node_type,

    date_trunc( 'minute', scan_start_ts )::timestamp as start_ts,

    sum( case when type =  1 then rows  else 0 end )::int8 as sum_type_1_rows,
    sum( case when type =  1 then bytes else 0 end )::int8 as sum_type_1_bytes,

    sum( case when type =  2 then rows_pre_filter      else 0 end )::int8 as sum_type_2m_rows,
    sum( case when type =  2 then rows_pre_user_filter else 0 end )::int8 as sum_type_2e_rows,
    sum( case when type =  2 then rows                 else 0 end )::int8 as sum_type_2_rows,
    sum( case when type =  2 then bytes                else 0 end )::int8 as sum_type_2_bytes,

    sum( case when type =  3 then rows  else 0 end )::int8 as sum_type_3_rows,
    sum( case when type =  3 then bytes else 0 end )::int8 as sum_type_3_bytes,

    sum( case when type =  9 then rows  else 0 end )::int8 as sum_type_9_rows,
    sum( case when type =  9 then bytes else 0 end )::int8 as sum_type_9_bytes,

    sum( case when type = 16 then rows  else 0 end )::int8 as sum_type_16_rows,
    sum( case when type = 16 then bytes else 0 end )::int8 as sum_type_16_bytes,

    sum( case when type = 17 then rows  else 0 end )::int8 as sum_type_17_rows,
    sum( case when type = 17 then bytes else 0 end )::int8 as sum_type_17_bytes,

    sum( case when type = 18 then rows  else 0 end )::int8 as sum_type_18_rows,
    sum( case when type = 18 then bytes else 0 end )::int8 as sum_type_18_bytes,

    sum( case when type = 21 then rows  else 0 end )::int8 as sum_type_21_rows,
    sum( case when type = 21 then bytes else 0 end )::int8 as sum_type_21_bytes,

    sum( case when type = 22 then rows  else 0 end )::int8 as sum_type_22_rows,
    sum( case when type = 22 then bytes else 0 end )::int8 as sum_type_22_bytes,

    sum( case when type = 23 then rows  else 0 end )::int8 as sum_type_23_rows,
    sum( case when type = 23 then bytes else 0 end )::int8 as sum_type_23_bytes,

    sum( case when type = 24 then rows  else 0 end )::int8 as sum_type_24_rows,
    sum( case when type = 24 then bytes else 0 end )::int8 as sum_type_24_bytes,

    sum( case when type = 25 then rows  else 0 end )::int8 as sum_type_25_rows,
    sum( case when type = 25 then bytes else 0 end )::int8 as sum_type_25_bytes,

    sum( case when type = 28 then rows  else 0 end )::int8 as sum_type_28_rows,
    sum( case when type = 28 then bytes else 0 end )::int8 as sum_type_28_bytes,

    sum( case when type = 29 then rows_pre_filter      else 0 end )::int8 as sum_type_29m_rows,
    sum( case when type = 29 then rows_pre_user_filter else 0 end )::int8 as sum_type_29e_rows,
    sum( case when type = 29 then rows                 else 0 end )::int8 as sum_type_29_rows,
    sum( case when type = 29 then bytes                else 0 end )::int8 as sum_type_29_bytes,

    sum( case when type = 30 then rows  else 0 end )::int8 as sum_type_30_rows,
    sum( case when type = 30 then bytes else 0 end )::int8 as sum_type_30_bytes,

    sum( case when type = 32 then rows  else 0 end )::int8 as sum_type_32_rows,
    sum( case when type = 32 then bytes else 0 end )::int8 as sum_type_32_bytes,

    sum( case when type = 33 then rows  else 0 end )::int8 as sum_type_33_rows,
    sum( case when type = 33 then bytes else 0 end )::int8 as sum_type_33_bytes,

    sum( case when type = 35 then rows  else 0 end )::int8 as sum_type_35_rows,
    sum( case when type = 35 then bytes else 0 end )::int8 as sum_type_35_bytes,

    sum( case when type = 36 then rows  else 0 end )::int8 as sum_type_36_rows,
    sum( case when type = 36 then bytes else 0 end )::int8 as sum_type_36_bytes,

    sum( case when type = 37 then rows  else 0 end )::int8 as sum_type_37_rows,
    sum( case when type = 37 then bytes else 0 end )::int8 as sum_type_37_bytes
  from
    base_241129.fixed_stl_scan
  where
        scan_start_ts >= current_date - interval '6 day'
    and scan_end_ts   <= current_timestamp
  group by
    node_type,
    start_ts;


create or replace view
  lf_241129.vacuum_totals_for_users_by_minute
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        userid          as user_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        user_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        user_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        user_id,
        date_trunc( 'minute', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        user_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        user_id,
        date_trunc( 'minute', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        user_id,
        start_ts
    )

  select
    cte_number_vacuums.user_id::int4,
    trim( pg_user.usename )::varchar as user_name,
    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( user_id, start_ts )
         join pg_user on pg_user.usesysid = cte_number_vacuums.user_id;


create or replace view
  sf_241129.vacuum_totals_for_users_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_users_by_minute
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.vacuum_totals_for_users_by_day
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        userid          as user_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        user_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        user_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        user_id,
        date_trunc( 'day', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        user_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        user_id,
        date_trunc( 'day', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        user_id,
        start_ts
    )

  select
    cte_number_vacuums.user_id::int4,
    trim( pg_user.usename )::varchar as user_name,
    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( user_id, start_ts )
         join pg_user on pg_user.usesysid = cte_number_vacuums.user_id;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_queues_by_minute
as
  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_minutes
    left join base_241129.fixed_stl_query                       on date_trunc( 'minute', query_start_ts ) = base_241129.generate_minutes.start_ts
         join base_241129.fixed_stl_wlm_query                using ( query_id )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    queue_id,
    queue_name,
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_queues_by_minute
as
  select
    queue_name::varchar as qname,
    start_ts::timestamp,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_queues_by_minute
  order by
    queue_name,
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_users_by_all
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        usesysid,
        node_type
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        usesysid,
        node_type
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        usesysid,
        node_type
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        usesysid,
        node_type
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        usesysid,
        node_type
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        usesysid,
        node_type
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        usesysid,
        node_type
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        usesysid,
        node_type
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        usesysid,
        node_type
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        usesysid,
        node_type
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        usesysid,
        node_type
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        usesysid,
        node_type
    ),

    cte_project as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        usesysid,
        node_type
    ),

    cte_return as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        usesysid,
        node_type
    ),

    cte_save as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        usesysid,
        node_type
    ),

    cte_sort as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        usesysid,
        node_type
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        usesysid,
        node_type
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        usesysid,
        node_type
    ),

    cte_window as
    (
      select
        userid as usesysid,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        userid,
        node_type,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        usesysid,
        node_type,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        usesysid,
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4           as user_id,
    trim( pg_user.usename )::varchar as user_name,
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join pg_user
     left join cte_aggr_stats         using ( node_type, usesysid )
     left join cte_bcast_stats        using ( node_type, usesysid )
     left join cte_delete_stats       using ( node_type, usesysid )
     left join cte_dist_stats         using ( node_type, usesysid )
     left join cte_hash_stats         using ( node_type, usesysid )
     left join cte_hashjoin_stats     using ( node_type, usesysid )
     left join cte_insert_stats       using ( node_type, usesysid )
     left join cte_limit_stats        using ( node_type, usesysid )
     left join cte_merge_stats        using ( node_type, usesysid )
     left join cte_mergejoin_stats    using ( node_type, usesysid )
     left join cte_nestloop_stats     using ( node_type, usesysid )
     left join cte_parse_stats        using ( node_type, usesysid )
     left join cte_project_stats      using ( node_type, usesysid )
     left join cte_return_stats       using ( node_type, usesysid )
     left join cte_save_stats         using ( node_type, usesysid )
     left join cte_sort_stats         using ( node_type, usesysid )
     left join cte_spatial_join_stats using ( node_type, usesysid )
     left join cte_unique_stats       using ( node_type, usesysid )
     left join cte_window_stats       using ( node_type, usesysid )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_bcast_duration is not null
    or max_bcast_duration is not null
    or med_bcast_duration is not null
    or avg_bcast_duration is not null
    or std_bcast_duration is not null
    or number_bcast_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_dist_duration is not null
    or max_dist_duration is not null
    or med_dist_duration is not null
    or avg_dist_duration is not null
    or std_dist_duration is not null
    or number_dist_steps is not null
    or min_hash_duration is not null
    or max_hash_duration is not null
    or med_hash_duration is not null
    or avg_hash_duration is not null
    or std_hash_duration is not null
    or number_hash_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or min_limit_duration is not null
    or max_limit_duration is not null
    or med_limit_duration is not null
    or avg_limit_duration is not null
    or std_limit_duration is not null
    or number_limit_steps is not null
    or min_merge_duration is not null
    or max_merge_duration is not null
    or med_merge_duration is not null
    or avg_merge_duration is not null
    or std_merge_duration is not null
    or number_merge_steps is not null
    or min_mergejoin_duration is not null
    or max_mergejoin_duration is not null
    or med_mergejoin_duration is not null
    or avg_mergejoin_duration is not null
    or std_mergejoin_duration is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or min_parse_duration is not null
    or max_parse_duration is not null
    or med_parse_duration is not null
    or avg_parse_duration is not null
    or std_parse_duration is not null
    or number_parse_steps is not null
    or min_project_duration is not null
    or max_project_duration is not null
    or med_project_duration is not null
    or avg_project_duration is not null
    or std_project_duration is not null
    or number_project_steps is not null
    or min_return_duration is not null
    or max_return_duration is not null
    or med_return_duration is not null
    or avg_return_duration is not null
    or std_return_duration is not null
    or number_return_steps is not null
    or min_save_duration is not null
    or max_save_duration is not null
    or med_save_duration is not null
    or avg_save_duration is not null
    or std_save_duration is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null
    or min_unique_duration is not null
    or max_unique_duration is not null
    or med_unique_duration is not null
    or avg_unique_duration is not null
    or std_unique_duration is not null
    or number_unique_steps is not null
    or min_window_duration is not null
    or max_window_duration is not null
    or med_window_duration is not null
    or avg_window_duration is not null
    or std_window_duration is not null
    or number_window_steps is not null;



create or replace view
  sf_241129.sschemas_blocks_rows
as
  select
    schema_name::varchar            as sschema,
    user_name::varchar              as uuser,
    number_tables::int4             as ttables,
    number_normal_views::int4       as nviews,
    number_late_binding_views::int4 as lviews,
    number_materialized_views::int4 as mviews,
    number_functions::int4          as funcs,
    number_procedures::int4         as procs,
    number_sorted_blocks::int8      as sblks,
    number_desorted_blocks::int8    as dsblks,
    number_unsorted_blocks::int8    as ublks,
    number_total_blocks::int8       as tblks,
    number_sorted_rows::int8        as srows,
    number_desorted_rows::int8      as dsrows,
    number_unsorted_rows::int8      as urows,
    number_total_rows::int8         as trows
  from
    lf_241129.sschemas_blocks_rows
  order by
    schema_name;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_slices_by_all
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal
    )

  select
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal );


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_slices_by_all
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        slice_ordinal,
        slice_type,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_slices_by_all
    )

 select
    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    slice_ordinal;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_statements_by_hour
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'aggr' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'bcast' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'delete' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'dist' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hash' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hashjoin' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'insert' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'limit' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'merge' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'mergejoin' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'nestloop_op' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'parse' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'project' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'return' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'save' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'sort' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'spatial_join' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'unique' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'window' as step_type,
        date_trunc( 'hour', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_steps as
    (
      select * from cte_aggr         union all
      select * from cte_bcast        union all
      select * from cte_delete       union all
      select * from cte_dist         union all
      select * from cte_hash         union all
      select * from cte_hashjoin     union all
      select * from cte_insert       union all
      select * from cte_limit        union all
      select * from cte_merge        union all
      select * from cte_mergejoin    union all
      select * from cte_nestloop     union all
      select * from cte_parse        union all
      select * from cte_project      union all
      select * from cte_return       union all
      select * from cte_save         union all
      select * from cte_sort         union all
      select * from cte_spatial_join union all
      select * from cte_unique       union all
      select * from cte_window
    )

  select
    statement_name::varchar,
    node_type::varchar,
    step_type::varchar,
    start_ts::timestamp,

    min( duration )::interval                          as min_duration,
    max( duration )::interval                          as max_duration,
    median( extract(epoch from duration) )::float4     as med_duration,
    avg( duration )::interval                          as avg_duration,
    stddev_pop( extract(epoch from duration) )::float4 as std_duration,
    count(*)::int8                                     as number_steps
  from
                 cte_base
    natural join cte_steps
  group by
    statement_name,
    node_type,
    step_type,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_statements_by_hour
as
  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    step_type::varchar      as step,
    start_ts::timestamp,

    extract( epoch from min_duration )::numeric(18,2) as min,
    extract( epoch from max_duration )::numeric(18,2) as max,
    med_duration::numeric(18,2)                       as med,
    extract( epoch from avg_duration )::numeric(18,2) as avg,
    std_duration::numeric(18,2)                       as std,
    number_steps::int8                                as num
  from
    lf_241129.steps_stats_durations_on_both_for_statements_by_hour
  order by
    statement_name,
    node_type,
    step_type,
    start_ts;



create table if not exists base_241129.strings
(
  system_source  varchar(32)  not null encode raw,
  system_number  int4             null encode raw,
  system_string  varchar(64)      null encode raw,
  my_string      varchar(64)  not null encode raw
)
diststyle all;


create or replace view
  lf_241129.queries_step_plan
as
  with
    cte_step_aggr as
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'aggregate'         as step_type,
        case when s1.my_string is not null then s1.my_string else 'undocumented type (' || type || ')' end as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        null                as number_packets,
        s2.my_string        as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
                  stl_aggr
        left join base_241129.strings as s1 on s1.system_source = 'stl_aggr'   and s1.system_string = stl_aggr.type
        left join base_241129.strings as s2 on s2.system_source = 'store_type' and s2.system_string = stl_aggr.is_diskbased
    ),

    cte_step_bcast as
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'browdcast'         as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        packets             as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
        stl_bcast
    ),

    cte_step_compile as 
    (
      select
        svl_compile.query       as query_id,
        svl_compile.segment     as segment_ordinal,
        null                    as step_ordinal,
        'compilation'           as step_type,
        case
          when svl_compile.compile = 1 then 'compile off-load'
          when svl_compile.compile = 0 then 'leader-node cache'
        end                     as sub_type,
        -1                      as slice_ordinal,
        null                    as number_materialized_rows,
        null                    as number_eligible_rows,
        null                    as number_rows,
        null                    as number_bytes,
        null                    as number_packets,
        null                    as store_type,
        svl_compile.starttime   as step_start_ts,
        svl_compile.endtime     as step_end_ts,
        null                    as table_id
      from
        svl_compile
    ),

    cte_step_delete as
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'delete'            as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
        stl_delete
    ),

    cte_step_distribute as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as cte_step_ordinal,
        'distribute'        as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        packets             as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
        stl_dist
    ),

    cte_step_hash as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'hash'              as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        null                as number_packets,
        s1.my_string        as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
                  stl_hash
        left join base_241129.strings as s1 on s1.system_source = 'store_type' and s1.system_string = stl_hash.is_diskbased
    ),

    cte_step_hashjoin as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'hash join'         as step_type,
        case when s1.my_string is not null then s1.my_string else 'undocumented type (' || join_type || ')' end as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
                  stl_hashjoin
        left join base_241129.strings as s1 on s1.system_source = 'stl_hashjoin' and s1.system_number = stl_hashjoin.join_type
    ),

    cte_step_insert as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'insert'            as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
        stl_insert
    ),

    cte_step_limit as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'limit'             as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
        stl_limit
    ),

    cte_step_merge as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'merge'             as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
        stl_merge
    ),

    cte_step_mergejoin as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'mergejoin'         as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
        stl_mergejoin
    ),

    cte_step_nestloop_op as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'nested loop join'  as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
        stl_nestloop
    ),

    cte_step_parse as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'parse'             as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
        stl_parse
    ),

    cte_step_project as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'project'           as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
        stl_project
    ),

    cte_step_return as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'return'            as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        packets             as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
        stl_return
    ),

    cte_step_save as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'save'              as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        null                as number_packets,
        s1.my_string        as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
                  stl_save
        left join base_241129.strings as s1 on s1.system_source = 'store_type' and s1.system_string = stl_save.is_diskbased
    ),

    cte_step_scan as 
    (
      select
        query                as query_id,
        segment              as segment_ordinal,
        step                 as step_ordinal,
        'scan'               as step_type,
        case when s1.my_string is not null then s1.my_string else 'undocumented type (' || type || ')' end as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        rows_pre_filter      as number_materialized_rows,
        rows_pre_user_filter as number_eligible_rows,
        rows                 as number_rows,
        bytes                as number_bytes,
        null                 as number_packets,
        null                 as store_type,
        starttime            as step_start_ts,
        endtime              as step_end_ts,
        tbl                  as table_id
      from
                  stl_scan
        left join base_241129.strings as s1 on s1.system_source = 'stl_scan' and s1.system_number = stl_scan.type
    ),

    cte_step_sort as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'sort'              as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        null                as number_packets,
        s1.my_string        as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
                  stl_sort
        left join base_241129.strings as s1 on s1.system_source = 'store_type' and s1.system_string = stl_sort.is_diskbased
    ),

    cte_step_spatial_join as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'spatial join'      as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        null                as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        tbl                 as table_id
      from
        stl_spatial_join
    ),

    cte_step_unique as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'window'            as step_type,
        case when s1.my_string is not null then s1.my_string else 'undocumented type (' || type || ')' end as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        bytes               as number_bytes,
        null                as number_packets,
        s2.my_string        as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
                  stl_unique
        left join base_241129.strings as s1 on s1.system_source = 'stl_unique' and s1.system_string = stl_unique.type
        left join base_241129.strings as s2 on s2.system_source = 'store_type' and s2.system_string = stl_unique.is_diskbased
    ),

    cte_step_window as 
    (
      select
        query               as query_id,
        segment             as segment_ordinal,
        step                as step_ordinal,
        'window'            as step_type,
        null                as sub_type,
        case when slice >= 12000 then -1 else slice end as slice_ordinal,
        null                as number_materialized_rows,
        null                as number_eligible_rows,
        rows                as number_rows,
        null                as number_bytes,
        null                as number_packets,
        s1.my_string        as store_type,
        starttime           as step_start_ts,
        endtime             as step_end_ts,
        null                as table_id
      from
                  stl_window
        left join base_241129.strings as s1 on s1.system_source = 'store_type' and s1.system_string = stl_window.is_diskbased
    ),

    cte_union_steps as
    (
      select * from cte_step_aggr
      union all
      select * from cte_step_bcast
      union all
      select * from cte_step_compile
      union all
      select * from cte_step_delete
      union all
      select * from cte_step_distribute
      union all
      select * from cte_step_hash
      union all
      select * from cte_step_hashjoin
      union all
      select * from cte_step_insert
      union all
      select * from cte_step_limit
      union all
      select * from cte_step_merge
      union all
      select * from cte_step_mergejoin
      union all
      select * from cte_step_nestloop_op
      union all
      select * from cte_step_parse
      union all
      select * from cte_step_project
      union all
      select * from cte_step_return
      union all
      select * from cte_step_save
      union all
      select * from cte_step_scan
      union all
      select * from cte_step_sort
      union all
      select * from cte_step_spatial_join
      union all
      select * from cte_step_unique
      union all
      select * from cte_step_window
    ),

    cte_query_step_plan as
    (
      select
        stl_stream_segs.query           as query_id,
        stl_stream_segs.stream          as stream_ordinal,
        stl_stream_segs.segment         as segment_ordinal,
        cte_union_steps.step_ordinal,
        base_241129.fixed_stv_slices.node_ordinal,
        cte_union_steps.slice_ordinal,
        base_241129.fixed_stv_slices.slice_type,
        cte_union_steps.step_type,
        cte_union_steps.sub_type,
        cte_union_steps.number_materialized_rows,
        cte_union_steps.number_eligible_rows,
        cte_union_steps.number_rows,
        cte_union_steps.number_bytes,
        cte_union_steps.number_packets,
        cte_union_steps.store_type,
        cte_union_steps.step_start_ts,
        cte_union_steps.step_end_ts,
        (cte_union_steps.step_end_ts - cte_union_steps.step_start_ts) as segment_duration,
        pg_namespace.oid                as schema_id,
        trim( pg_namespace.nspname )    as schema_name,
        cte_union_steps.table_id,
        trim( pg_class.relname )        as table_name
      from
                  stl_stream_segs
             join cte_union_steps          on stl_stream_segs.query   = cte_union_steps.query_id and
                                              stl_stream_segs.segment = cte_union_steps.segment_ordinal
             join base_241129.fixed_stv_slices on cte_union_steps.slice_ordinal = base_241129.fixed_stv_slices.slice_ordinal
        left join pg_class                 on cte_union_steps.table_id      = pg_class.oid
        left join pg_namespace             on pg_class.relnamespace         = pg_namespace.oid
    )

  select
    query_id::int4,
    stream_ordinal::int2,
    segment_ordinal::int2,
    step_ordinal::int2,
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,
    step_type::varchar,
    sub_type::varchar,
    number_materialized_rows::int8,
    number_eligible_rows::int8,
    number_rows::int8,
    number_bytes::int8,
    number_packets::int8,
    store_type::varchar,
    step_start_ts::timestamp,
    step_end_ts::timestamp,
    segment_duration::interval,
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar
  from
    cte_query_step_plan;


create or replace view
  sf_241129.queries_step_plan
as
  select
    query_id::int4                  as qid,
    stream_ordinal::int2            as strm,
    segment_ordinal::int2           as seg,
    step_ordinal::int2              as stp,
    node_ordinal::int2              as node,
    slice_ordinal::int2             as slice,
    left( slice_type, 1 )::varchar  as t,
    step_type::varchar              as ttype,
    number_materialized_rows::int8  as mrows,
    number_eligible_rows::int8      as erows,
    number_rows::int8               as rrows, -- MG2 : keyword doubling of first letter
    number_bytes::int8              as bytes,
    number_packets::int8            as pkts,
    case
      when store_type = 'on disk'   then 'd'
      when store_type = 'in memory' then 'm'
                                    else null
    end::varchar                    as st,
    step_start_ts::timestamp        as start_ts,
    step_end_ts::timestamp          as end_ts,
    extract( epoch from (step_end_ts - step_start_ts) )::numeric(18,6) as dur,
    case
      when schema_name is not null and table_name is not null and sub_type is not null then sub_type || ' ' || schema_name || '.' || table_name
      when schema_name is not null and table_name is not null and sub_type is     null then schema_name || '.' || table_name
      when schema_name is     null and table_name is not null and sub_type is not null then sub_type || ' ' || table_name
      when schema_name is     null and table_name is     null and sub_type is not null then sub_type
      when schema_name is     null and table_name is     null and sub_type is     null then ''
                                                                                       else ''
    end::varchar                    as notes
  from
    lf_241129.queries_step_plan
  order by
    query_id,
    stream_ordinal,
    case when step_type = 'compilation' then 0 else 1 end,
    segment_ordinal,
    step_ordinal,
    node_ordinal,
    slice_ordinal;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_users_by_hour
as
  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    count(*)::int4                       as number_abort
  from
              base_241129.generate_hours
    left join base_241129.fixed_stl_query    on date_trunc( 'hour', query_start_ts ) = base_241129.generate_hours.start_ts
    left join base_241129.fixed_pg_user   using ( user_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    user_id,
    user_name,
    start_ts;



create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_users_by_hour
as
  select
    user_name::varchar as uuser,
    start_ts::timestamp,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_users_by_hour
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_schemas_by_minute
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    start_ts::timestamp,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_scan.sum_scan_rows )::int8     as sum_scan_rows
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_minutes
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null
  group by
    base_241129.fixed_pg_namespace.schema_id,
    base_241129.fixed_pg_namespace.schema_name,
    start_ts;



create or replace view
  lf_241129.analyze_totals_for_tables_by_second
as
  with
    cte_s1 as
    (
      select
        table_id,
        date_trunc( 'second', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            database   = current_database()
        and starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4::int4              as table_id,
    trim( pg_class.relname )::varchar     as table_name,

    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_users_by_all
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        userid      as user_id,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid
    ),

    cte_insert as
    (
      select
        userid      as user_id,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid
    ),

    cte_scan as
    (
      select
        userid      as user_id,
        sum( rows ) as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        userid
    )

  select
    base_241129.fixed_pg_user.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
               base_241129.fixed_pg_user
     left join cte_delete using ( user_id )
     left join cte_insert using ( user_id )
     left join cte_scan   using ( user_id )
  where
       cte_delete.user_id is not null
    or cte_insert.user_id is not null
    or cte_scan.user_id   is not null;


create or replace view
  sf_241129.views_late_binding_columns
as
  select
    schema_name::varchar                as sschema,
    view_name::varchar                  as vview,
    column_name::varchar                as ccolumn,
    column_full_data_type_name::varchar as data_type
  from
    lf_241129.views_late_binding_columns
  order by
    schema_name,
    view_name,
    column_name;


create or replace view
  sf_241129.queries_totals_result_cache_for_all_by_day
as
  select
    start_ts::date,
    number_hit::int8  as hit,
    number_miss::int8 as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_all_by_day
  order by
    start_ts;


create or replace view
  lf_241129.ttables_creation_for_schemas_by_hour
as
  select
    trim( pg_namespace.nspname )::varchar           as schema_name,
    date_trunc( 'hour', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    schema_name,
    start_ts;


create or replace view
  sf_241129.ttables_creation_for_schemas_by_hour
as
  select
    schema_name::varchar as sschema,
    start_ts::timestamp,
    number_tables::int4  as ttables
  from
    lf_241129.ttables_creation_for_schemas_by_hour
  order by
    schema_name,
    start_ts;


create or replace view
  lf_241129.ttables_stats_dists_sorts_for_schemas_by_blocks
as
  select
    base_241129.ttables_basic_info_full.schema_name::varchar,

    -- MG2 : all

    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then number_sorted_blocks else 0 end )::int8    as number_all_compound_sorted_blocks,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then number_sorted_blocks else 0 end )::int8    as number_all_interleaved_sorted_blocks,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then number_sorted_blocks else 0 end )::int8    as number_all_unsorted_sorted_blocks,

    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then number_unsorted_blocks else 0 end )::int8  as number_all_compound_unsorted_blocks,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then number_unsorted_blocks else 0 end )::int8  as number_all_interleaved_unsorted_blocks,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then number_unsorted_blocks else 0 end )::int8  as number_all_unsorted_unsorted_blocks,

    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then number_total_blocks else 0 end )::int8     as number_all_compound_total_blocks,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then number_total_blocks else 0 end )::int8     as number_all_interleaved_total_blocks,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then number_total_blocks else 0 end )::int8     as number_all_unsorted_total_blocks,

    -- MG2 : even

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then number_sorted_blocks else 0 end )::int8    as number_even_compound_sorted_blocks,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then number_sorted_blocks else 0 end )::int8    as number_even_interleaved_sorted_blocks,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then number_sorted_blocks else 0 end )::int8    as number_even_unsorted_sorted_blocks,

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then number_unsorted_blocks else 0 end )::int8  as number_even_compound_unsorted_blocks,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then number_unsorted_blocks else 0 end )::int8  as number_even_interleaved_unsorted_blocks,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then number_unsorted_blocks else 0 end )::int8  as number_even_unsorted_unsorted_blocks,

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then number_total_blocks else 0 end )::int8     as number_even_compound_total_blocks,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then number_total_blocks else 0 end )::int8     as number_even_interleaved_total_blocks,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then number_total_blocks else 0 end )::int8     as number_even_unsorted_total_blocks,

    -- MG2 : key

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then number_sorted_blocks else 0 end )::int8     as number_key_compound_sorted_blocks,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then number_sorted_blocks else 0 end )::int8     as number_key_interleaved_sorted_blocks,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then number_sorted_blocks else 0 end )::int8     as number_key_unsorted_sorted_blocks,

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then number_unsorted_blocks else 0 end )::int8   as number_key_compound_unsorted_blocks,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then number_unsorted_blocks else 0 end )::int8   as number_key_interleaved_unsorted_blocks,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then number_unsorted_blocks else 0 end )::int8   as number_key_unsorted_unsorted_blocks,

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then number_total_blocks else 0 end )::int8      as number_key_compound_total_blocks,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then number_total_blocks else 0 end )::int8      as number_key_interleaved_total_blocks,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then number_total_blocks else 0 end )::int8      as number_key_unsorted_total_blocks
  from
         base_241129.ttables_basic_info_full
    join base_241129.ttables_blocks_rows using ( table_id )
  where
        user_id                 != 1
    and schema_name         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(schema_name,8)     != 'pg_temp_'
  group by
    schema_name;


create or replace view
  sf_241129.ttables_stats_dists_sorts_for_schemas_by_blocks
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,

        -- MG2 : sorted

        base_241129.hr_blks( number_all_compound_sorted_blocks     ) as hr_number_all_compound_sorted_blocks,
        base_241129.hr_blks( number_all_interleaved_sorted_blocks  ) as hr_number_all_interleaved_sorted_blocks,
        base_241129.hr_blks( number_all_unsorted_sorted_blocks     ) as hr_number_all_unsorted_sorted_blocks,

        base_241129.hr_blks( number_even_compound_sorted_blocks    ) as hr_number_even_compound_sorted_blocks,
        base_241129.hr_blks( number_even_interleaved_sorted_blocks ) as hr_number_even_interleaved_sorted_blocks,
        base_241129.hr_blks( number_even_unsorted_sorted_blocks    ) as hr_number_even_unsorted_sorted_blocks,

        base_241129.hr_blks( number_key_compound_sorted_blocks     ) as hr_number_key_compound_sorted_blocks,
        base_241129.hr_blks( number_key_interleaved_sorted_blocks  ) as hr_number_key_interleaved_sorted_blocks,
        base_241129.hr_blks( number_key_unsorted_sorted_blocks     ) as hr_number_key_unsorted_sorted_blocks,

        -- MG2 : unsorted

        base_241129.hr_blks( number_all_compound_unsorted_blocks     ) as hr_number_all_compound_unsorted_blocks,
        base_241129.hr_blks( number_all_interleaved_unsorted_blocks  ) as hr_number_all_interleaved_unsorted_blocks,
        base_241129.hr_blks( number_all_unsorted_unsorted_blocks     ) as hr_number_all_unsorted_unsorted_blocks,

        base_241129.hr_blks( number_even_compound_unsorted_blocks    ) as hr_number_even_compound_unsorted_blocks,
        base_241129.hr_blks( number_even_interleaved_unsorted_blocks ) as hr_number_even_interleaved_unsorted_blocks,
        base_241129.hr_blks( number_even_unsorted_unsorted_blocks    ) as hr_number_even_unsorted_unsorted_blocks,

        base_241129.hr_blks( number_key_compound_unsorted_blocks     ) as hr_number_key_compound_unsorted_blocks,
        base_241129.hr_blks( number_key_interleaved_unsorted_blocks  ) as hr_number_key_interleaved_unsorted_blocks,
        base_241129.hr_blks( number_key_unsorted_unsorted_blocks     ) as hr_number_key_unsorted_unsorted_blocks,

        -- MG2 : sorted lengths

        max( len(hr_number_all_compound_sorted_blocks)     ) over () as max_len_hr_number_all_compound_sorted_blocks,
        max( len(hr_number_all_interleaved_sorted_blocks)  ) over () as max_len_hr_number_all_interleaved_sorted_blocks,
        max( len(hr_number_all_unsorted_sorted_blocks)     ) over () as max_len_hr_number_all_unsorted_sorted_blocks,

        max( len(hr_number_even_compound_sorted_blocks)    ) over () as max_len_hr_number_even_compound_sorted_blocks,
        max( len(hr_number_even_interleaved_sorted_blocks) ) over () as max_len_hr_number_even_interleaved_sorted_blocks,
        max( len(hr_number_even_unsorted_sorted_blocks)    ) over () as max_len_hr_number_even_unsorted_sorted_blocks,

        max( len(hr_number_key_compound_sorted_blocks)     ) over () as max_len_hr_number_key_compound_sorted_blocks,
        max( len(hr_number_key_interleaved_sorted_blocks)  ) over () as max_len_hr_number_key_interleaved_sorted_blocks,
        max( len(hr_number_key_unsorted_sorted_blocks)     ) over () as max_len_hr_number_key_unsorted_sorted_blocks,

        -- MG2 : unsorted lengths

        max( len(hr_number_all_compound_unsorted_blocks)     ) over () as max_len_hr_number_all_compound_unsorted_blocks,
        max( len(hr_number_all_interleaved_unsorted_blocks)  ) over () as max_len_hr_number_all_interleaved_unsorted_blocks,
        max( len(hr_number_all_unsorted_unsorted_blocks)     ) over () as max_len_hr_number_all_unsorted_unsorted_blocks,

        max( len(hr_number_even_compound_unsorted_blocks)    ) over () as max_len_hr_number_even_compound_unsorted_blocks,
        max( len(hr_number_even_interleaved_unsorted_blocks) ) over () as max_len_hr_number_even_interleaved_unsorted_blocks,
        max( len(hr_number_even_unsorted_unsorted_blocks)    ) over () as max_len_hr_number_even_unsorted_unsorted_blocks,

        max( len(hr_number_key_compound_unsorted_blocks)     ) over () as max_len_hr_number_key_compound_unsorted_blocks,
        max( len(hr_number_key_interleaved_unsorted_blocks)  ) over () as max_len_hr_number_key_interleaved_unsorted_blocks,
        max( len(hr_number_key_unsorted_unsorted_blocks)     ) over () as max_len_hr_number_key_unsorted_unsorted_blocks

      from
        lf_241129.ttables_stats_dists_sorts_for_schemas_by_blocks
    )

  select
    schema_name::varchar as sschema,

    ( lpad(hr_number_all_compound_sorted_blocks,  max_len_hr_number_all_compound_sorted_blocks ) || ' / ' || lpad(hr_number_all_compound_unsorted_blocks,  max_len_hr_number_all_compound_unsorted_blocks ) )::varchar as compound_all,
    ( lpad(hr_number_even_compound_sorted_blocks, max_len_hr_number_even_compound_sorted_blocks) || ' / ' || lpad(hr_number_even_compound_unsorted_blocks, max_len_hr_number_even_compound_unsorted_blocks) )::varchar as compound_even,
    ( lpad(hr_number_key_compound_sorted_blocks,  max_len_hr_number_key_compound_sorted_blocks ) || ' / ' || lpad(hr_number_key_compound_unsorted_blocks,  max_len_hr_number_key_compound_unsorted_blocks ) )::varchar as compound_key,

    ( lpad(hr_number_all_interleaved_sorted_blocks,  max_len_hr_number_all_interleaved_sorted_blocks ) || ' / ' || lpad(hr_number_all_interleaved_unsorted_blocks,  max_len_hr_number_all_interleaved_unsorted_blocks ) )::varchar as intlvd_all,
    ( lpad(hr_number_even_interleaved_sorted_blocks, max_len_hr_number_even_interleaved_sorted_blocks) || ' / ' || lpad(hr_number_even_interleaved_unsorted_blocks, max_len_hr_number_even_interleaved_unsorted_blocks) )::varchar as intlvd_even,
    ( lpad(hr_number_key_interleaved_sorted_blocks,  max_len_hr_number_key_interleaved_sorted_blocks ) || ' / ' || lpad(hr_number_key_interleaved_unsorted_blocks,  max_len_hr_number_key_interleaved_unsorted_blocks ) )::varchar as intlvd_key,

    ( lpad(hr_number_all_unsorted_sorted_blocks,  max_len_hr_number_all_unsorted_sorted_blocks ) || ' / ' || lpad(hr_number_all_unsorted_unsorted_blocks,  max_len_hr_number_all_unsorted_unsorted_blocks ) )::varchar as unsorted_all,
    ( lpad(hr_number_even_unsorted_sorted_blocks, max_len_hr_number_even_unsorted_sorted_blocks) || ' / ' || lpad(hr_number_even_unsorted_unsorted_blocks, max_len_hr_number_even_unsorted_unsorted_blocks) )::varchar as unsorted_even,
    ( lpad(hr_number_key_unsorted_sorted_blocks,  max_len_hr_number_key_unsorted_sorted_blocks ) || ' / ' || lpad(hr_number_key_unsorted_unsorted_blocks,  max_len_hr_number_key_unsorted_unsorted_blocks ) )::varchar as unsorted_key
  from
    cte_hr_formatting
  order by
    schema_name;


create or replace view
  sf_241129.analyze_totals_for_all_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_all_by_minute
    )

  select
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_queues_by_day
as
  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_days
    left join base_241129.fixed_stl_query                       on date_trunc( 'day', query_start_ts ) = base_241129.generate_days.start_ts
         join base_241129.fixed_stl_wlm_query                using ( query_id )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    queue_id,
    queue_name,
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_queues_by_day
as
  select
    queue_name::varchar as qname,
    start_ts::date,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_queues_by_day
  order by
    queue_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_slices_by_second
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'second', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal );


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_slices_by_second
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        slice_ordinal,
        slice_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_slices_by_second
    )

  select
    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    slice_ordinal,
    start_ts;


create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_users_by_day
as
  with
    cte_blocks as
    (
      select
        user_id,
        user_name,
        date_trunc( 'day', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        user_name,
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        user_id,
        user_name,
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_users_by_day
as
  select
    user_name::varchar                                                    as uuser,
    start_ts::date,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_users_by_day
  order by
    user_name,
    start_ts;


create table if not exists base_241129.docs
(
  schema_name  varchar(127)    not null encode runlength,
  object_name  varchar(127)    not null encode zstd,
  object_type  varchar(9)      not null encode zstd,
  doc          varchar(65535)  not null encode zstd,

  primary key( schema_name, object_name )
)
diststyle even;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_day_complete
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::date,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_day
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;



create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_all_by_hour
as
  with
    cte_blocks as
    (
      select
        date_trunc( 'hour', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;

create or replace view
  lf_241129.ttables_stats_dists_sorts_for_all_by_count
as
  select
    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then 1 else 0 end )::int4  as all_compound,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then 1 else 0 end )::int4  as all_interleaved,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then 1 else 0 end )::int4  as all_unsorted,

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then 1 else 0 end )::int4 as even_compound,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then 1 else 0 end )::int4 as even_interleaved,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then 1 else 0 end )::int4 as even_unsorted,

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then 1 else 0 end )::int4  as key_compound,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then 1 else 0 end )::int4  as key_interleaved,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then 1 else 0 end )::int4  as key_unsorted
  from
    base_241129.ttables_basic_info_full
  where
        user_id                 != 1
    and schema_name         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(schema_name,8)     != 'pg_temp_';


create or replace view
  sf_241129.ttables_stats_dists_sorts_for_all_by_count
as
  select
    ( all_compound    || ' / ' ||
      all_interleaved || ' / ' ||
      all_unsorted    )::varchar as aall,

    ( even_compound    || ' / ' ||
      even_interleaved || ' / ' ||
      even_unsorted    )::varchar as even,

    ( key_compound    || ' / ' ||
      key_interleaved || ' / ' ||
      key_unsorted    )::varchar as kkey
  from
    lf_241129.ttables_stats_dists_sorts_for_all_by_count;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_queues_by_all
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        queue_name,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_queues_by_all
    )

 select
    node_type::varchar  as node,
    queue_name::varchar as qname,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    queue_name;



create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_all_by_minute
as
  with
    cte_blocks as
    (
      select
        date_trunc( 'minute', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;

create or replace view
  base_241129.fixed_pg_attribute_for_tables
as
  select
    attrelid::int4            as table_id,
    trim( attname ) ::varchar as column_name,
    attnum::int2              as column_ordinal,
    atttypmod::int4,
    attnotnull::bool          as column_notnull_flag
  from
    pg_attribute;


create or replace view
  sf_241129.ttables
as
  select
    schema_name::varchar                     as sschema,
    table_name::varchar                      as ttable,
    user_name::varchar                       as uuser,
    date_trunc( 'day', table_age )::interval as age,
    table_number_columns::int2               as cols,
    table_sort_type::varchar                 as ssort,
    table_number_sortkeys::int2              as skeys,
    case
      when table_distribution_auto_flag = True  then table_distribution_type || ' (auto)'
      when table_distribution_auto_flag = false then table_distribution_type
    end::varchar                             as dist,
    table_stats::varchar                     as stats
  from
    lf_241129.ttables
  where
    user_name != 'rdsdb'
  order by
    schema_name,
    table_name;


create or replace view
  lf_241129.transactions_totals_commits_by_day
as
  select
    date_trunc( 'day', batch_leader_startwork )::timestamp as start_ts,
    sum( batch_leader_duration )::interval                     as sum_batch_leader_duration,
    sum( batch_worker_duration )::interval                     as sum_batch_worker_duration,
    sum( batch_exec_duration - batch_leader_duration - batch_worker_duration )::interval as sum_batch_idle_duration,
    sum( batch_exec_duration  )::interval                      as sum_batch_exec_duration
  from
    lf_241129.batches
  group by
    start_ts;


create or replace view
  sf_241129.transactions_totals_commits_by_day
as
  select
    start_ts::date,
    extract( epoch from sum_batch_leader_duration )::numeric(18,2) as leader,
    extract( epoch from sum_batch_worker_duration )::numeric(18,2) as worker,
    extract( epoch from sum_batch_idle_duration   )::numeric(18,2) as idle,
    extract( epoch from sum_batch_exec_duration   )::numeric(18,2) as exec
  from
    lf_241129.transactions_totals_commits_by_day
  order by
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_tables_by_day
as
  with
    cte_s1 as
    (
      select
        table_id,
        date_trunc( 'day', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            database   = current_database()
        and starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4::int4              as table_id,
    trim( pg_class.relname )::varchar     as table_name,

    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_users_by_second
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'second', query_queue_start_ts )        as start_ts,
        query_wlm_start_ts   - query_start_ts        as leadin_duration,
        query_queue_start_ts - query_wlm_start_ts    as wlm_leadin_duration,
        query_queue_end_ts   - query_queue_start_ts  as query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration   as query_exec_duration,
        query_wlm_end_ts     - query_compile_exec_end_ts as wlm_leadout_duration,
        query_end_ts         - query_wlm_end_ts          as leadout_duration,
        query_total_duration,
        query_total_duration - leadin_duration - wlm_leadin_duration - query_queue_duration + query_compile_duration - query_exec_duration - wlm_leadout_duration - leadout_duration as unaccounted_duration
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query         using ( query_id, user_id )
        left join base_241129.queries_compilation using ( query_id, user_id )
      where
            query_abort_flag    = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        user_id,
        start_ts,

        min( leadin_duration ) over ( partition by user_id, start_ts )                              as min_leadin_duration,
        max( leadin_duration ) over ( partition by user_id, start_ts )                              as max_leadin_duration,
        median( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts )     as med_leadin_duration,
        avg( leadin_duration ) over ( partition by user_id, start_ts )                              as avg_leadin_duration,
        stddev_pop( extract(epoch from (leadin_duration)) ) over ( partition by user_id, start_ts ) as std_leadin_duration,
        sum( leadin_duration ) over ( partition by user_id, start_ts )                              as sum_leadin_duration,

        min( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadin_duration,
        max( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadin_duration,
        median( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadin_duration,
        avg( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadin_duration,
        stddev_pop( extract(epoch from (wlm_leadin_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadin_duration,
        sum( wlm_leadin_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadin_duration,

        min( query_queue_duration ) over ( partition by user_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by user_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by user_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by user_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by user_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by user_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by user_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by user_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by user_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by user_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by user_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by user_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by user_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by user_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by user_id, start_ts )                            as sum_query_exec_duration,

        min( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as min_wlm_leadout_duration,
        max( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as max_wlm_leadout_duration,
        median( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts )     as med_wlm_leadout_duration,
        avg( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as avg_wlm_leadout_duration,
        stddev_pop( extract(epoch from (wlm_leadout_duration)) ) over ( partition by user_id, start_ts ) as std_wlm_leadout_duration,
        sum( wlm_leadout_duration ) over ( partition by user_id, start_ts )                              as sum_wlm_leadout_duration,

        min( leadout_duration ) over ( partition by user_id, start_ts )                              as min_leadout_duration,
        max( leadout_duration ) over ( partition by user_id, start_ts )                              as max_leadout_duration,
        median( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts )     as med_leadout_duration,
        avg( leadout_duration ) over ( partition by user_id, start_ts )                              as avg_leadout_duration,
        stddev_pop( extract(epoch from (leadout_duration)) ) over ( partition by user_id, start_ts ) as std_leadout_duration,
        sum( leadout_duration ) over ( partition by user_id, start_ts )                              as sum_leadout_duration,

        min( query_total_duration ) over ( partition by user_id, start_ts )                              as min_query_total_duration,
        max( query_total_duration ) over ( partition by user_id, start_ts )                              as max_query_total_duration,
        median( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts )     as med_query_total_duration,
        avg( query_total_duration ) over ( partition by user_id, start_ts )                              as avg_query_total_duration,
        stddev_pop( extract(epoch from (query_total_duration)) ) over ( partition by user_id, start_ts ) as std_query_total_duration,
        sum( query_total_duration ) over ( partition by user_id, start_ts )                              as sum_query_total_duration,

        min( unaccounted_duration ) over ( partition by user_id, start_ts )                              as min_unaccounted_duration,
        max( unaccounted_duration ) over ( partition by user_id, start_ts )                              as max_unaccounted_duration,
        median( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts )     as med_unaccounted_duration,
        avg( unaccounted_duration ) over ( partition by user_id, start_ts )                              as avg_unaccounted_duration,
        stddev_pop( extract(epoch from (unaccounted_duration)) ) over ( partition by user_id, start_ts ) as std_unaccounted_duration,
        sum( unaccounted_duration ) over ( partition by user_id, start_ts )                              as sum_unaccounted_duration,

        count(*) over ( partition by user_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    cte_queries_stats.user_id::int4,
    base_241129.fixed_pg_user.user_name::varchar,
    start_ts::timestamp,

    min_leadin_duration::interval,
    max_leadin_duration::interval,
    med_leadin_duration::float4,
    avg_leadin_duration::interval,
    std_leadin_duration::float4,
    sum_leadin_duration::interval,

    min_wlm_leadin_duration::interval,
    max_wlm_leadin_duration::interval,
    med_wlm_leadin_duration::float4,
    avg_wlm_leadin_duration::interval,
    std_wlm_leadin_duration::float4,
    sum_wlm_leadin_duration::interval,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    min_wlm_leadout_duration::interval,
    max_wlm_leadout_duration::interval,
    med_wlm_leadout_duration::float4,
    avg_wlm_leadout_duration::interval,
    std_wlm_leadout_duration::float4,
    sum_wlm_leadout_duration::interval,

    min_leadout_duration::interval,
    max_leadout_duration::interval,
    med_leadout_duration::float4,
    avg_leadout_duration::interval,
    std_leadout_duration::float4,
    sum_leadout_duration::interval,

    min_query_total_duration::interval,
    max_query_total_duration::interval,
    med_query_total_duration::float4,
    avg_query_total_duration::interval,
    std_query_total_duration::float4,
    sum_query_total_duration::interval,

    min_unaccounted_duration::interval,
    max_unaccounted_duration::interval,
    med_unaccounted_duration::float4,
    avg_unaccounted_duration::interval,
    std_unaccounted_duration::float4,
    sum_unaccounted_duration::interval,

    number_queries::int4
  from
              base_241129.generate_seconds
    left join cte_queries_stats     using ( start_ts )
    left join base_241129.fixed_pg_user using ( user_id )
  group by
    cte_queries_stats.user_id,
    user_name,
    start_ts,
    min_leadin_duration,
    max_leadin_duration,
    med_leadin_duration,
    avg_leadin_duration,
    std_leadin_duration,
    sum_leadin_duration,
    min_wlm_leadin_duration,
    max_wlm_leadin_duration,
    med_wlm_leadin_duration,
    avg_wlm_leadin_duration,
    std_wlm_leadin_duration,
    sum_wlm_leadin_duration,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    min_wlm_leadout_duration,
    max_wlm_leadout_duration,
    med_wlm_leadout_duration,
    avg_wlm_leadout_duration,
    std_wlm_leadout_duration,
    sum_wlm_leadout_duration,
    min_leadout_duration,
    max_leadout_duration,
    med_leadout_duration,
    avg_leadout_duration,
    std_leadout_duration,
    sum_leadout_duration,
    min_query_total_duration,
    max_query_total_duration,
    med_query_total_duration,
    avg_query_total_duration,
    std_query_total_duration,
    sum_query_total_duration,
    min_unaccounted_duration,
    max_unaccounted_duration,
    med_unaccounted_duration,
    avg_unaccounted_duration,
    std_unaccounted_duration,
    sum_unaccounted_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_users_by_second
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_leadin_duration)::numeric(18,2)::varchar as hr_min_leadin_duration,
        extract(epoch from max_leadin_duration)::numeric(18,2)::varchar as hr_max_leadin_duration,
        med_leadin_duration::numeric(18,2)::varchar                     as hr_med_leadin_duration,
        extract(epoch from avg_leadin_duration)::numeric(18,2)::varchar as hr_avg_leadin_duration,
        std_leadin_duration::numeric(18,2)::varchar                     as hr_std_leadin_duration,
        extract(epoch from sum_leadin_duration)::numeric(18,2)::varchar as hr_sum_leadin_duration,

        extract(epoch from min_wlm_leadin_duration)::numeric(18,2)::varchar as hr_min_wlm_leadin_duration,
        extract(epoch from max_wlm_leadin_duration)::numeric(18,2)::varchar as hr_max_wlm_leadin_duration,
        med_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadin_duration,
        extract(epoch from avg_wlm_leadin_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadin_duration,
        std_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadin_duration,
        extract(epoch from sum_wlm_leadin_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadin_duration,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        extract(epoch from min_wlm_leadout_duration)::numeric(18,2)::varchar as hr_min_wlm_leadout_duration,
        extract(epoch from max_wlm_leadout_duration)::numeric(18,2)::varchar as hr_max_wlm_leadout_duration,
        med_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadout_duration,
        extract(epoch from avg_wlm_leadout_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadout_duration,
        std_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadout_duration,
        extract(epoch from sum_wlm_leadout_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadout_duration,

        extract(epoch from min_leadout_duration)::numeric(18,2)::varchar as hr_min_leadout_duration,
        extract(epoch from max_leadout_duration)::numeric(18,2)::varchar as hr_max_leadout_duration,
        med_leadout_duration::numeric(18,2)::varchar                     as hr_med_leadout_duration,
        extract(epoch from avg_leadout_duration)::numeric(18,2)::varchar as hr_avg_leadout_duration,
        std_leadout_duration::numeric(18,2)::varchar                     as hr_std_leadout_duration,
        extract(epoch from sum_leadout_duration)::numeric(18,2)::varchar as hr_sum_leadout_duration,

        extract(epoch from min_query_total_duration)::numeric(18,2)::varchar as hr_min_query_total_duration,
        extract(epoch from max_query_total_duration)::numeric(18,2)::varchar as hr_max_query_total_duration,
        med_query_total_duration::numeric(18,2)::varchar                     as hr_med_query_total_duration,
        extract(epoch from avg_query_total_duration)::numeric(18,2)::varchar as hr_avg_query_total_duration,
        std_query_total_duration::numeric(18,2)::varchar                     as hr_std_query_total_duration,
        extract(epoch from sum_query_total_duration)::numeric(18,2)::varchar as hr_sum_query_total_duration,

        extract(epoch from min_unaccounted_duration)::numeric(18,2)::varchar as hr_min_unaccounted_duration,
        extract(epoch from max_unaccounted_duration)::numeric(18,2)::varchar as hr_max_unaccounted_duration,
        med_unaccounted_duration::numeric(18,2)::varchar                     as hr_med_unaccounted_duration,
        extract(epoch from avg_unaccounted_duration)::numeric(18,2)::varchar as hr_avg_unaccounted_duration,
        std_unaccounted_duration::numeric(18,2)::varchar                     as hr_std_unaccounted_duration,
        extract(epoch from sum_unaccounted_duration)::numeric(18,2)::varchar as hr_sum_unaccounted_duration,

        max( len(hr_min_leadin_duration) ) over () as max_len_hr_min_leadin_duration,
        max( len(hr_max_leadin_duration) ) over () as max_len_hr_max_leadin_duration,
        max( len(hr_med_leadin_duration) ) over () as max_len_hr_med_leadin_duration,
        max( len(hr_avg_leadin_duration) ) over () as max_len_hr_avg_leadin_duration,
        max( len(hr_std_leadin_duration) ) over () as max_len_hr_std_leadin_duration,
        max( len(hr_sum_leadin_duration) ) over () as max_len_hr_sum_leadin_duration,

        max( len(hr_min_wlm_leadin_duration) ) over () as max_len_hr_min_wlm_leadin_duration,
        max( len(hr_max_wlm_leadin_duration) ) over () as max_len_hr_max_wlm_leadin_duration,
        max( len(hr_med_wlm_leadin_duration) ) over () as max_len_hr_med_wlm_leadin_duration,
        max( len(hr_avg_wlm_leadin_duration) ) over () as max_len_hr_avg_wlm_leadin_duration,
        max( len(hr_std_wlm_leadin_duration) ) over () as max_len_hr_std_wlm_leadin_duration,
        max( len(hr_sum_wlm_leadin_duration) ) over () as max_len_hr_sum_wlm_leadin_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        max( len(hr_min_wlm_leadout_duration) ) over () as max_len_hr_min_wlm_leadout_duration,
        max( len(hr_max_wlm_leadout_duration) ) over () as max_len_hr_max_wlm_leadout_duration,
        max( len(hr_med_wlm_leadout_duration) ) over () as max_len_hr_med_wlm_leadout_duration,
        max( len(hr_avg_wlm_leadout_duration) ) over () as max_len_hr_avg_wlm_leadout_duration,
        max( len(hr_std_wlm_leadout_duration) ) over () as max_len_hr_std_wlm_leadout_duration,
        max( len(hr_sum_wlm_leadout_duration) ) over () as max_len_hr_sum_wlm_leadout_duration,

        max( len(hr_min_leadout_duration) ) over () as max_len_hr_min_leadout_duration,
        max( len(hr_max_leadout_duration) ) over () as max_len_hr_max_leadout_duration,
        max( len(hr_med_leadout_duration) ) over () as max_len_hr_med_leadout_duration,
        max( len(hr_avg_leadout_duration) ) over () as max_len_hr_avg_leadout_duration,
        max( len(hr_std_leadout_duration) ) over () as max_len_hr_std_leadout_duration,
        max( len(hr_sum_leadout_duration) ) over () as max_len_hr_sum_leadout_duration,

        max( len(hr_min_query_total_duration) ) over () as max_len_hr_min_query_total_duration,
        max( len(hr_max_query_total_duration) ) over () as max_len_hr_max_query_total_duration,
        max( len(hr_med_query_total_duration) ) over () as max_len_hr_med_query_total_duration,
        max( len(hr_avg_query_total_duration) ) over () as max_len_hr_avg_query_total_duration,
        max( len(hr_std_query_total_duration) ) over () as max_len_hr_std_query_total_duration,
        max( len(hr_sum_query_total_duration) ) over () as max_len_hr_sum_query_total_duration,

        max( len(hr_min_unaccounted_duration) ) over () as max_len_hr_min_unaccounted_duration,
        max( len(hr_max_unaccounted_duration) ) over () as max_len_hr_max_unaccounted_duration,
        max( len(hr_med_unaccounted_duration) ) over () as max_len_hr_med_unaccounted_duration,
        max( len(hr_avg_unaccounted_duration) ) over () as max_len_hr_avg_unaccounted_duration,
        max( len(hr_std_unaccounted_duration) ) over () as max_len_hr_std_unaccounted_duration,
        max( len(hr_sum_unaccounted_duration) ) over () as max_len_hr_sum_unaccounted_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_users_by_second
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_min_leadin_duration, max_len_hr_min_leadin_duration ) || ' / ' ||
      lpad( hr_max_leadin_duration, max_len_hr_max_leadin_duration ) || ' / ' ||
      lpad( hr_med_leadin_duration, max_len_hr_med_leadin_duration ) || ' / ' ||
      lpad( hr_avg_leadin_duration, max_len_hr_avg_leadin_duration ) || ' / ' ||
      lpad( hr_std_leadin_duration, max_len_hr_std_leadin_duration ) || ' / ' ||
      lpad( hr_sum_leadin_duration, max_len_hr_sum_leadin_duration ) )::varchar as leadin,

    ( lpad( hr_min_wlm_leadin_duration, max_len_hr_min_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadin_duration, max_len_hr_max_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadin_duration, max_len_hr_med_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadin_duration, max_len_hr_avg_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadin_duration, max_len_hr_std_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_sum_wlm_leadin_duration, max_len_hr_sum_wlm_leadin_duration ) )::varchar as wlm_leadin,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    ( lpad( hr_min_wlm_leadout_duration, max_len_hr_min_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadout_duration, max_len_hr_max_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadout_duration, max_len_hr_med_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadout_duration, max_len_hr_avg_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadout_duration, max_len_hr_std_wlm_leadout_duration ) || ' / ' || 
      lpad( hr_sum_wlm_leadout_duration, max_len_hr_sum_wlm_leadout_duration ) )::varchar as wlm_leadout,

    ( lpad( hr_min_leadout_duration, max_len_hr_min_leadout_duration ) || ' / ' ||
      lpad( hr_max_leadout_duration, max_len_hr_max_leadout_duration ) || ' / ' ||
      lpad( hr_med_leadout_duration, max_len_hr_med_leadout_duration ) || ' / ' ||
      lpad( hr_avg_leadout_duration, max_len_hr_avg_leadout_duration ) || ' / ' ||
      lpad( hr_std_leadout_duration, max_len_hr_std_leadout_duration ) || ' / ' || 
      lpad( hr_sum_leadout_duration, max_len_hr_sum_leadout_duration ) )::varchar as leadout,

    ( lpad( hr_min_query_total_duration, max_len_hr_min_query_total_duration ) || ' / ' ||
      lpad( hr_max_query_total_duration, max_len_hr_max_query_total_duration ) || ' / ' ||
      lpad( hr_med_query_total_duration, max_len_hr_med_query_total_duration ) || ' / ' ||
      lpad( hr_avg_query_total_duration, max_len_hr_avg_query_total_duration ) || ' / ' ||
      lpad( hr_std_query_total_duration, max_len_hr_std_query_total_duration ) || ' / ' || 
      lpad( hr_sum_query_total_duration, max_len_hr_sum_query_total_duration ) )::varchar as total,

    ( lpad( hr_min_unaccounted_duration, max_len_hr_min_unaccounted_duration ) || ' / ' ||
      lpad( hr_max_unaccounted_duration, max_len_hr_max_unaccounted_duration ) || ' / ' ||
      lpad( hr_med_unaccounted_duration, max_len_hr_med_unaccounted_duration ) || ' / ' ||
      lpad( hr_avg_unaccounted_duration, max_len_hr_avg_unaccounted_duration ) || ' / ' ||
      lpad( hr_std_unaccounted_duration, max_len_hr_std_unaccounted_duration ) || ' / ' || 
      lpad( hr_sum_unaccounted_duration, max_len_hr_sum_unaccounted_duration ) )::varchar as unact,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_hour_concise
as
  select
    start_ts::timestamp,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_hour
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_users_by_all
as
  with
    cte_transactions as
    (
      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        user_id,
        transaction_id,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        user_id,
        transaction_id
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                                      as user_name,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
    left join pg_user                        on cte_all_transactions.user_id        = pg_user.usesysid
  group by
    user_id,
    user_name;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_all_by_hour
as
  select
    start_ts::timestamp,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_all_by_hour
  order by
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_node_by_minute
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
             cte_aggr
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
             cte_bcast
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
             cte_delete
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
             cte_dist
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
             cte_hash
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
             cte_hashjoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
             cte_insert
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
             cte_limit
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
             cte_merge
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
             cte_mergejoin
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
             cte_nestloop
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
             cte_parse
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
             cte_project
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
             cte_return
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
             cte_save
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
             cte_sort
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
             cte_spatial_join
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
             cte_unique
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        date_trunc( 'minute', min(starttime) ) as start_ts,
        max(endtime) - min(starttime)       as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        start_ts,
        node_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
             cte_window
        join base_241129.fixed_stv_slices using ( slice_ordinal )
      group by
        start_ts,
        node_ordinal
    )

  select
    base_241129.generate_minutes.start_ts::timestamp as start_ts,
    base_241129.fixed_stv_slices.node_ordinal::int2,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               base_241129.generate_minutes
    cross join base_241129.fixed_stv_slices
     left join cte_aggr_stats             using ( start_ts, node_ordinal )
     left join cte_bcast_stats            using ( start_ts, node_ordinal )
     left join cte_delete_stats           using ( start_ts, node_ordinal )
     left join cte_dist_stats             using ( start_ts, node_ordinal )
     left join cte_hash_stats             using ( start_ts, node_ordinal )
     left join cte_hashjoin_stats         using ( start_ts, node_ordinal )
     left join cte_insert_stats           using ( start_ts, node_ordinal )
     left join cte_limit_stats            using ( start_ts, node_ordinal )
     left join cte_merge_stats            using ( start_ts, node_ordinal )
     left join cte_mergejoin_stats        using ( start_ts, node_ordinal )
     left join cte_nestloop_stats         using ( start_ts, node_ordinal )
     left join cte_parse_stats            using ( start_ts, node_ordinal )
     left join cte_project_stats          using ( start_ts, node_ordinal )
     left join cte_return_stats           using ( start_ts, node_ordinal )
     left join cte_save_stats             using ( start_ts, node_ordinal )
     left join cte_sort_stats             using ( start_ts, node_ordinal )
     left join cte_spatial_join_stats     using ( start_ts, node_ordinal )
     left join cte_unique_stats           using ( start_ts, node_ordinal )
     left join cte_window_stats           using ( start_ts, node_ordinal );


create or replace view
  lf_241129.utils_queries_benchmark
as
  with
    cte_return_times as
    (
      select
        query            as query_id,
        min( starttime ) as min_r2c_start_ts
      from
        stl_return
      where
            slice >= 12811
        and userid = current_user_id
      group by
        query
      order by
        query desc
      limit 5
    ),

    cte_compile_times as
    (
      select
        query                      as query_id,
        sum( endtime - starttime ) as sum_compile_duration
      from
        svl_compile
      where
        userid = current_user_id
      group by
        query
      order by
        query desc
      limit 5
    ),

    cte_exec_start as
    (
      select
        query_id,
        query_start_ts,
        query_compile_exec_start_ts,
        left( trim(query_text), 48 ) as query_text
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id, user_id )
      where
            user_id    = current_user_id
        and session_id = pg_backend_pid()
      order by
        query_id desc
      limit 5
    ),

    cte_durations as
    (
      select
        query_id,
        query_start_ts,
        extract( epoch from ( (min_r2c_start_ts - query_compile_exec_start_ts) - case when sum_compile_duration is null then interval '0 second' else sum_compile_duration end ) ) as exec_duration,
        query_text
      from
                  cte_exec_start
        left join cte_compile_times using ( query_id )
        left join cte_return_times  using ( query_id )
    )

  select
    row_number() over ( order by query_start_ts asc )::int2 as rn,
    query_start_ts::timestamp,
    query_id::int4,
    exec_duration::float4,
    min( exec_duration )        over ( order by query_id desc rows between unbounded preceding and current row )::float4 as min_exec_duration,
    max( exec_duration )        over ( order by query_id desc rows between unbounded preceding and current row )::float4 as max_exec_duration,
    avg( exec_duration )        over ( order by query_id desc rows between unbounded preceding and current row )::float4 as avg_exec_duration,
    stddev_pop( exec_duration ) over ( order by query_id desc rows between unbounded preceding and current row )::float4 as std_exec_duration,
    query_text::varchar
  from
    cte_durations;


create or replace view
  sf_241129.utils_queries_benchmark
as
  select
    rn::int2,
    query_id::int4                   as qid,
    query_start_ts::timestamp        as start_ts,
    exec_duration::numeric(18,2)     as dur,
    min_exec_duration::numeric(18,2) as min,
    max_exec_duration::numeric(18,2) as max,
    avg_exec_duration::numeric(18,2) as avg,
    std_exec_duration::numeric(18,2) as std,
    query_text::varchar              as ttext
  from
    lf_241129.utils_queries_benchmark
  order by
    query_start_ts;


create or replace view
  lf_241129.ttables_creation_for_all_by_day
as
  select
    date_trunc( 'day', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    start_ts;


create or replace view
  sf_241129.ttables_creation_for_all_by_day
as
  select
    start_ts::date,
    number_tables::int4 as ttables
  from
    lf_241129.ttables_creation_for_all_by_day
  order by
    start_ts;


create or replace view
  sf_241129.batches
as
  select
    batch_id::int8                                             as bid,
    batch_startqueue::timestamp                                as start_ts,
    batch_endtime::timestamp                                   as end_ts,
    extract( epoch from batch_queue_duration  )::numeric(18,2) as queue,
    extract( epoch from batch_leader_duration )::numeric(18,2) as leader,
    extract( epoch from batch_worker_duration )::numeric(18,2) as worker,
    extract( epoch from batch_exec_duration  )::numeric(18,2)  as exec, -- exec duration is end of leader node work, and a new batch begins after this
    number_batch_blocks::int8                                  as blks,
    number_batch_headers::int8                                 as hdrs,
    number_batch_transactions::int8                            as xids
  from
    lf_241129.batches
  order by
    batch_startqueue;


create or replace view
  base_241129.fixed_stv_wlm_service_class_state
as
  select
    num_concurrency_scaling_queries::int4,
    num_evicted_queries::int4,
    num_executed_queries::int4,
    num_executing_queries::int4,
    num_queued_queries::int4,
    num_serviced_queries::int4,
    service_class::int2 as queue_id
  from
    stv_wlm_service_class_state;


create or replace view
  lf_241129.wqueues_memory
as
  with
    cte_slices_per_node as
    (
      select
        node     as node_ordinal,
        count(*) as node_number_slices
      from
        stv_slices
      group by
        node
    ),

    cte_slice_info as
    (
      select
        min( node_number_slices ) as min_slices,
        max( node_number_slices ) as max_slices
      from
        cte_slices_per_node
    )

  select
    queue_id::int2,
    queue_name::varchar,
    base_241129.fixed_stv_wlm_service_class_config.num_query_tasks::int2                                                                    as number_slots,
    case
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks = 0 then 0
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks > 0 then (base_241129.fixed_stv_wlm_service_class_config.query_working_mem*base_241129.fixed_stv_wlm_service_class_config.num_query_tasks/2/base_241129.fixed_stv_wlm_service_class_config.num_query_tasks*2) / 2
    end::int4                                                                                                                           as ra3_xlplus_memory_per_slot_per_slice,
    case
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks = 0 then 0
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks > 0 then (base_241129.fixed_stv_wlm_service_class_config.query_working_mem*base_241129.fixed_stv_wlm_service_class_config.num_query_tasks/4/base_241129.fixed_stv_wlm_service_class_config.num_query_tasks*4) / 4
    end::int4                                                                                                                           as ra3_4x_memory_per_slot_per_slice,
    case
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks = 0 then 0
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks > 0 then (base_241129.fixed_stv_wlm_service_class_config.query_working_mem*base_241129.fixed_stv_wlm_service_class_config.num_query_tasks/16/base_241129.fixed_stv_wlm_service_class_config.num_query_tasks*16) / 16
    end::int4                                                                                                                           as ra3_16x_memory_per_slot_per_slice,
    case
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks = 0 then 0
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks > 0 then (base_241129.fixed_stv_wlm_service_class_config.query_working_mem*base_241129.fixed_stv_wlm_service_class_config.num_query_tasks/cte_slice_info.max_slices/base_241129.fixed_stv_wlm_service_class_config.num_query_tasks*cte_slice_info.max_slices) / cte_slice_info.max_slices
    end::int4                                                                                                                           as dc2_min_memory_per_slot_per_slice,
    case
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks = 0 then 0
      when base_241129.fixed_stv_wlm_service_class_config.num_query_tasks > 0 then (base_241129.fixed_stv_wlm_service_class_config.query_working_mem*base_241129.fixed_stv_wlm_service_class_config.num_query_tasks/cte_slice_info.min_slices/base_241129.fixed_stv_wlm_service_class_config.num_query_tasks*cte_slice_info.min_slices) / cte_slice_info.min_slices
    end::int4                                                                                                                           as dc2_max_memory_per_slot_per_slice,
    ( ( base_241129.fixed_stv_wlm_service_class_config.max_execution_time::float8 / 1000 ) * interval '1 second' )::interval                              as timeout
  from
               base_241129.fixed_stv_wlm_service_class_config
          join base_241129.fixed_stv_wlm_service_class_state using ( queue_id )
    cross join cte_slice_info;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_users_by_minute
as
  select
    user_name::varchar                                      as uuser,
    start_ts::timestamp,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_users_by_minute
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_users_by_day
as
  with
    cte_aggr as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                          as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                                               as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        userid as usesysid,
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        usesysid,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    pg_user.usesysid::int4           as usesysid,
    trim( pg_user.usename )::varchar as user_name,
    cte_node_types.node_type::varchar,
    base_241129.generate_days.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               pg_user
    cross join cte_node_types
    cross join base_241129.generate_days
     left join cte_aggr         using( node_type, usesysid, start_ts )
     left join cte_bcast        using( node_type, usesysid, start_ts )
     left join cte_delete       using( node_type, usesysid, start_ts )
     left join cte_dist         using( node_type, usesysid, start_ts )
     left join cte_hash         using( node_type, usesysid, start_ts )
     left join cte_hashjoin     using( node_type, usesysid, start_ts )
     left join cte_insert       using( node_type, usesysid, start_ts )
     left join cte_limit        using( node_type, usesysid, start_ts )
     left join cte_merge        using( node_type, usesysid, start_ts )
     left join cte_mergejoin    using( node_type, usesysid, start_ts )
     left join cte_nestloop     using( node_type, usesysid, start_ts )
     left join cte_parse        using( node_type, usesysid, start_ts )
     left join cte_project      using( node_type, usesysid, start_ts )
     left join cte_return       using( node_type, usesysid, start_ts )
     left join cte_save         using( node_type, usesysid, start_ts )
     left join cte_sort         using( node_type, usesysid, start_ts )
     left join cte_spatial_join using( node_type, usesysid, start_ts )
     left join cte_unique       using( node_type, usesysid, start_ts )
     left join cte_window       using( node_type, usesysid, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.steps_totals_data_on_both_for_users_by_day
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::date,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_users_by_day
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.blocks_rows_totals_for_schema
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_desorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_desorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_desorted_blocks) ) over () as max_len_hr_number_desorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks,

        max( len(hr_number_sorted_rows)   ) over () as max_len_hr_number_sorted_rows,
        max( len(hr_number_desorted_rows) ) over () as max_len_hr_number_desorted_rows,
        max( len(hr_number_unsorted_rows) ) over () as max_len_hr_number_unsorted_rows,
        max( len(hr_number_total_rows)    ) over () as max_len_hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_schema
    )

  select
    schema_name::varchar                                                           as sschema,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_desorted_blocks, max_len_hr_number_desorted_blocks )::varchar  as sdblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks,

    lpad( hr_number_sorted_rows,   max_len_hr_number_sorted_rows   )::varchar      as srows,
    lpad( hr_number_desorted_rows, max_len_hr_number_desorted_rows )::varchar      as sdrops,
    lpad( hr_number_unsorted_rows, max_len_hr_number_unsorted_rows )::varchar      as urows,
    lpad( hr_number_total_rows,    max_len_hr_number_total_rows    )::varchar      as trows
  from
    cte_hr_formatting
  order by
    schema_name;


create or replace view
  lf_241129.ttables_blocks_rows_activity
as
  select
    lf_241129.ttables_blocks_rows.schema_id::int4,
    lf_241129.ttables_blocks_rows.schema_name::varchar,
    lf_241129.ttables_blocks_rows.table_id::int4,
    lf_241129.ttables_blocks_rows.table_name::varchar,
    lf_241129.ttables_blocks_rows.user_id::int4,
    lf_241129.ttables_blocks_rows.user_name::varchar,
    lf_241129.ttables_blocks_rows.table_creation::timestamp,
    lf_241129.ttables_blocks_rows.table_age::interval,
    lf_241129.ttables_blocks_rows.table_number_columns::int2,
    lf_241129.ttables_blocks_rows.table_sort_type::varchar,
    lf_241129.ttables_blocks_rows.table_number_sortkeys::int2,
    lf_241129.ttables_blocks_rows.table_distribution_type::varchar,
    lf_241129.ttables_blocks_rows.table_distribution_auto_flag::bool,
    lf_241129.ttables_blocks_rows.number_slices::int2,
    lf_241129.ttables_blocks_rows.number_sorted_blocks::int8,
    lf_241129.ttables_blocks_rows.number_desorted_blocks::int8,
    lf_241129.ttables_blocks_rows.number_unsorted_blocks::int8,
    lf_241129.ttables_blocks_rows.number_total_blocks::int8,
    lf_241129.ttables_blocks_rows.number_sorted_rows::int8,
    lf_241129.ttables_blocks_rows.number_desorted_rows::int8,
    lf_241129.ttables_blocks_rows.number_unsorted_rows::int8,
    lf_241129.ttables_blocks_rows.number_total_rows::int8,
    base_241129.ttables_activity_fast.number_rows_deleted::int8,
    base_241129.ttables_activity_fast.number_rows_inserted::int8
  from
              lf_241129.ttables_blocks_rows
    left join base_241129.ttables_activity_fast using ( table_id );


create or replace view
  lf_241129.steps_totals_data_on_both_for_statements_by_hour
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'hour', starttime ) as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    statement_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               cte_base
    cross join cte_node_types
    cross join base_241129.generate_hours
     left join cte_aggr         using ( start_ts, query, node_type )
     left join cte_bcast        using ( start_ts, query, node_type )
     left join cte_delete       using ( start_ts, query, node_type )
     left join cte_dist         using ( start_ts, query, node_type )
     left join cte_hash         using ( start_ts, query, node_type )
     left join cte_hashjoin     using ( start_ts, query, node_type )
     left join cte_insert       using ( start_ts, query, node_type )
     left join cte_limit        using ( start_ts, query, node_type )
     left join cte_merge        using ( start_ts, query, node_type )
     left join cte_mergejoin    using ( start_ts, query, node_type )
     left join cte_nestloop     using ( start_ts, query, node_type )
     left join cte_parse        using ( start_ts, query, node_type )
     left join cte_project      using ( start_ts, query, node_type )
     left join cte_return       using ( start_ts, query, node_type )
     left join cte_save         using ( start_ts, query, node_type )
     left join cte_sort         using ( start_ts, query, node_type )
     left join cte_spatial_join using ( start_ts, query, node_type )
     left join cte_unique       using ( start_ts, query, node_type )
     left join cte_window       using ( start_ts, query, node_type )
  group by
    statement_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_users_by_hour
as
  select
    user_name::varchar                 as uuser,
    start_ts::timestamp,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_users_by_hour
  order by
    user_name,
    start_ts;


create or replace view
  base_241129.fixed_stv_wlm_query_state
as
  select
    exec_time::int8,
    query::int4                              as query_id,
    lower( trim( query_priority ) )::varchar as query_priority,
    queue_time::int8,
    service_class::int2                      as queue_id,
    slot_count::int2                         as query_slot_count,
    trim( state )::varchar                   as query_state,
    task::int4                               as task_id,
    wlm_start_time::timestamp,
    xid::int8                                as transaction_id
  from
    stv_wlm_query_state;


create or replace view
  lf_241129.queries_queued
as
  select
    queue_id::int2,
    queue_name::varchar,
    row_number() over ( order by queue_time desc )::int2 as queue_position,
    transaction_id::int8,
    query_id::int4,
    ( ( base_241129.fixed_stv_wlm_query_state.queue_time::float8 / 1000000 ) * interval '1 second' )::interval as queue_duration
  from
         base_241129.fixed_stv_wlm_query_state
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  where
    query_state = 'QueuedWaiting';


create or replace view
  lf_241129.steps_totals_data_on_both_for_slices_by_minute
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    slice_ordinal::int2,
    start_ts::timestamp,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_minutes
     left join cte_aggr         using( start_ts, slice_ordinal )
     left join cte_bcast        using( start_ts, slice_ordinal )
     left join cte_delete       using( start_ts, slice_ordinal )
     left join cte_dist         using( start_ts, slice_ordinal )
     left join cte_hash         using( start_ts, slice_ordinal )
     left join cte_hashjoin     using( start_ts, slice_ordinal )
     left join cte_insert       using( start_ts, slice_ordinal )
     left join cte_limit        using( start_ts, slice_ordinal )
     left join cte_merge        using( start_ts, slice_ordinal )
     left join cte_mergejoin    using( start_ts, slice_ordinal )
     left join cte_nestloop     using( start_ts, slice_ordinal )
     left join cte_parse        using( start_ts, slice_ordinal )
     left join cte_project      using( start_ts, slice_ordinal )
     left join cte_return       using( start_ts, slice_ordinal )
     left join cte_save         using( start_ts, slice_ordinal )
     left join cte_sort         using( start_ts, slice_ordinal )
     left join cte_spatial_join using( start_ts, slice_ordinal )
     left join cte_unique       using( start_ts, slice_ordinal )
     left join cte_window       using( start_ts, slice_ordinal )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.queries_queued
as
  select
    queue_name::varchar  as qname,
    queue_position::int2 as pos,
    transaction_id::int8 as xid,
    query_id::int4       as qid,
    extract( epoch from queue_duration )::numeric(18,2) as dur
  from
    lf_241129.queries_queued
  order by
    qname,
    queue_duration desc;


create or replace view
  base_241129.fixed_stv_inflight
as
  select
    case
      when concurrency_scaling_status = 0 then false
                                          else true
    end::bool              as csc_flag,
    -- insert_pristine::int4
    trim( label )::varchar as label,
    pid::int4              as session_id,
    query::int4            as query_id,
    slice::int2            as slice_ordinal,
    starttime::timestamp   as query_start_ts,
    -- suspended::int4,
    trim( text )::varchar  as text,
    userid::int4           as user_id,
    xid::int8              as transaction_id
  from
    stv_inflight;


create or replace view
  lf_241129.queries_running
as
  with
    cte_worker_queries_data_sessioned as
    (
      select
        query          as query_id,
        sum( bytes )   as sum_data_sessioned_in_bytes,
        sum( workmem ) as sum_memory_used_in_bytes
      from
        stv_exec_state
      group by
        query_id
    ),

    cte_worker_queries_transient_tables as
    (
      select
        query_id,
        count( distinct id ) as number_transient_tables,
        sum( size )          as sum_transient_tables_in_bytes
      from
        stv_tbl_trans
      group by
        query_id
    ),

    cte_exec_state as
    (
      select
        query          as query_id,
        max( segment ) as max_exec_segment
      from
        stv_exec_state
      group by
        query_id
    ),

    cte_worker_queries as
    (
      select
        csc_flag,
        user_id,
        user_name,
        session_id,
        transaction_id,
        -- stv_wlm_query_state.task                         as task_id,
        query_id,
        queue_id,
        queue_name,
        query_slot_count,
        cte_exec_state.max_exec_segment as max_exec_segment,
        ( base_241129.fixed_stv_wlm_query_state.exec_time::float8 / 1000000 ) * interval '1 second' as query_exec_duration,
        ( base_241129.fixed_stv_query_metrics.query_cpu_time::float8 / 1000000 ) * interval '1 second' as query_cpu_duration,
        cte_worker_queries_data_sessioned.sum_data_sessioned_in_bytes,
        cte_worker_queries_data_sessioned.sum_memory_used_in_bytes,
        cte_worker_queries_transient_tables.number_transient_tables,
        cte_worker_queries_transient_tables.sum_transient_tables_in_bytes,
        base_241129.fixed_stv_query_metrics.query_number_blocks_to_disk,
        left( ltrim(base_241129.fixed_stv_inflight.text), 48 ) as query_text
      from
                     base_241129.fixed_stv_inflight
                join base_241129.fixed_stv_wlm_query_state          using ( query_id, transaction_id )
           left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
           left join cte_exec_state                             using ( query_id )
           left join base_241129.fixed_stv_query_metrics            using ( query_id, queue_id, user_id )
           left join base_241129.fixed_pg_user                      using ( user_id )
           left join cte_worker_queries_data_sessioned          using ( query_id )
           left join cte_worker_queries_transient_tables        using ( query_id )
      where
        base_241129.fixed_stv_wlm_query_state.query_state = 'Running'
    )

  select
    queue_id::int2,
    queue_name::varchar,
    csc_flag::bool,
    user_id::int4,
    user_name::varchar,
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    query_slot_count::int2,
    max_exec_segment::int2,
    query_exec_duration::interval,
    query_cpu_duration::interval,
    sum_data_sessioned_in_bytes::int8,
    sum_memory_used_in_bytes::int8,
    number_transient_tables::int4,
    sum_transient_tables_in_bytes::int8,
    query_number_blocks_to_disk::int8,
    query_text::varchar
  from
    cte_worker_queries;


create or replace view
  sf_241129.queries_running
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        session_id,
        transaction_id,
        query_id,
        queue_name,
        query_slot_count,
        csc_flag,
        max_exec_segment,
        query_exec_duration,
        query_cpu_duration,
        base_241129.hr_bytes( sum_data_sessioned_in_bytes ) as hr_sum_data_sessioned_in_bytes,
        base_241129.hr_bytes( sum_memory_used_in_bytes    ) as hr_sum_memory_used_in_bytes,
        number_transient_tables,
        base_241129.hr_bytes( sum_transient_tables_in_bytes ) as hr_sum_transient_tables_in_bytes,
        base_241129.hr_blks( query_number_blocks_to_disk )    as hr_query_number_blocks_to_disk,
        query_text,

        max( len(hr_sum_data_sessioned_in_bytes)   ) over () as max_len_hr_sum_data_sessioned_in_bytes,
        max( len(hr_sum_memory_used_in_bytes)      ) over () as max_len_hr_sum_memory_used_in_bytes,
        max( len(hr_sum_transient_tables_in_bytes) ) over () as max_len_hr_sum_transient_tables_in_bytes,
        max( len(hr_query_number_blocks_to_disk)   ) over () as max_len_hr_query_number_blocks_to_disk
      from
        lf_241129.queries_running
    )

  select
    user_name::varchar                                                                          as uuser,
    session_id::int4                                                                            as pid,
    transaction_id::int8                                                                        as xid,
    query_id::int4                                                                              as qid,
    queue_name::varchar                                                                         as qname,
    query_slot_count::int2                                                                      as s,
    ( case when csc_flag = True then 'C' else ' ' end )::varchar                                as flags,
    max_exec_segment::int2                                                                      as seg,
    extract( epoch from query_exec_duration )::numeric(18,2)                                    as dur,
    extract( epoch from query_cpu_duration  )::numeric(18,2)                                    as cpu,
    lpad( hr_sum_data_sessioned_in_bytes, max_len_hr_sum_data_sessioned_in_bytes )::varchar     as ddata,
    lpad( hr_sum_memory_used_in_bytes, max_len_hr_sum_memory_used_in_bytes )::varchar           as mem,
    number_transient_tables::int4                                                               as num_tt,
    lpad( hr_sum_transient_tables_in_bytes, max_len_hr_sum_transient_tables_in_bytes )::varchar as size_tt,
    lpad( hr_query_number_blocks_to_disk, max_len_hr_query_number_blocks_to_disk )::varchar     as btd,
    ltrim( query_text )::varchar                                                                as ttext
  from
    cte_hr_formatting
  order by
    queue_name,
    query_exec_duration;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_tables_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_tables_by_minute
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_all_by_minute
as
  select
    start_ts::timestamp,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_all_by_minute
  order by
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_queues_by_second
as
  with
    cte_queries as
    (
      select
        queue_id,
        date_trunc( 'second', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
                  base_241129.fixed_stl_wlm_query
        left join base_241129.fixed_stl_query    using ( query_id )
        left join base_241129.queries_compilation using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        queue_id,
        start_ts,

        min( query_queue_duration ) over ( partition by queue_id, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by queue_id, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by queue_id, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by queue_id, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by queue_id, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by queue_id, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by queue_id, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by queue_id, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by queue_id, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by queue_id, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by queue_id, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by queue_id, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by queue_id, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by queue_id, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by queue_id, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by queue_id, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_seconds
    left join cte_queries_stats using ( start_ts )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_queues_by_second
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_queues_by_second
    )

  select
    queue_name::varchar as qname,
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    queue_name,
    start_ts;


create or replace view
  sf_241129.objects_totals_for_all
as
  select
    number_schemas::int2                   as sschemas,
    number_tables::int4                    as ttables,
    number_table_columns::int4             as ttables_cols,
    number_normal_views::int4              as nviews,
    number_normal_view_columns::int4       as nviews_cols,
    number_late_binding_views::int4        as lviews,
    number_materialized_views::int4        as mviews,
    number_materialized_view_columns::int4 as mviews_cols,
    number_functions::int4                 as funcs,
    number_procedures::int4                as procs
  from
    lf_241129.objects_totals_for_all;

create or replace procedure
  ro.sp_grant_versionless_sf_schema_to_user( user_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'grant usage on schema sf to ' || user_name;
      execute sql;

      sql := 'grant select on all tables in schema sf to ' || user_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.analyze_totals_for_database_by_hour
as
  select
    database::varchar                                                                                             as database_name,
    date_trunc( 'hour', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    database,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_statements_by_minute
as

  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                       as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                      as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                        as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                   as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                   as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                   as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                   as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                   as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                   as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                   as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                   as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                      as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                      as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_statements_by_minute
  order by
    statement_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.ttables_creation_for_schemas_by_minute
as
  select
    trim( pg_namespace.nspname )::varchar           as schema_name,
    date_trunc( 'minute', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    schema_name,
    start_ts;


create or replace view
  sf_241129.ttables_creation_for_schemas_by_minute
as
  select
    schema_name::varchar as sschema,
    start_ts::timestamp,
    number_tables::int4  as ttables
  from
    lf_241129.ttables_creation_for_schemas_by_minute
  order by
    schema_name,
    start_ts;


create or replace procedure
  ro.sp_create_versionless_sf_schema_pointed_at_version( version in varchar(6) )
as
  $proc$
    declare
      row  record;
      sql  varchar(4096);

    begin
      -- MG2 : drop existing, generate new

      raise info 'Schema sf might not exist and if so, it''ll be skipped; that''s okay.';

      drop schema if exists sf cascade;

      create schema sf;

      sql := 'select
                trim( pg_namespace.nspname                 ) as schema_name_with_wersion,
                split_part( pg_namespace.nspname, ''_'', 1 ) as schema_name_sans_version,
                trim( pg_class.relname                     ) as object_name
              from
                     pg_class
                join pg_namespace on pg_class.relnamespace = pg_namespace.oid
              where
                    pg_class.relkind     in ( ''r'', ''v'' )
                and pg_namespace.nspname  = ( ' || $$'$$ || 'sf_' || version || $$'$$ || ' );';

      for row in execute sql loop
        sql := 'create view ' || row.schema_name_sans_version || '.' || row.object_name || ' as select * from ' || row.schema_name_with_wersion || '.' || row.object_name;
        execute sql;
      end loop;
    end
  $proc$
language plpgsql;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_statements_by_hour
as

  with
    cte_hr_formatting as
    (
      select
        statement_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_statements_by_hour
    )

  select
    statement_name::varchar as stmt,
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    statement_name,
    start_ts;


create or replace view
  sf_241129.analyze_totals_for_tables_by_all
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_tables_by_day
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_tables_by_all
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    cte_s1.table_id::int4,
    trim( pg_class.relname )::varchar      as table_name,
    node_type::varchar,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r';


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_schemas_by_minute
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'minute', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r'
  group by
    schema_id,
    schema_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_schemas_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_schemas_by_minute
    )

  select
    schema_name::varchar as sschema,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    node_type,
    start_ts;


create or replace view
  base_241129.transactions
as
  with
    cte_transaction_fragments as
    (
      select
        user_id,
        session_id,
        transaction_id,
        min( query_start_ts ) as transaction_fragment_start_ts,
        max( query_end_ts   ) as transaction_fragment_end_ts,
        sum( query_total_duration ) as transaction_fragment_query_total_duration,
        sum( query_wlm_duration ) as transaction_fragment_query_wlm_duration,
        sum( query_queue_duration ) as transaction_fragment_query_queue_duration,
        sum( query_compile_exec_duration ) as transaction_fragment_query_compile_exec_duration,
        0                     as transaction_fragment_number_csc_queries,
        0                     as transaction_fragment_number_rc_queries,
        count(*)              as transaction_fragment_number_leader_queries,
        0                     as transaction_fragment_number_worker_queries,
        false                 as transaction_fragment_abort_flag
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0
      group by
        user_id,
        session_id,
        transaction_id

      union all

      select
        user_id,
        session_id,
        transaction_id,
        min( query_start_ts ) as transaction_fragment_start_ts,
        -- MG2 : base_241129.fixed_stl_utilitytext changes the start time of a 'commit' to be when it is issued
        --       and this is when the query-issuing stage of the transaction has completed
        max( query_end_ts         ) as transaction_fragment_end_ts,
        sum( query_total_duration ) as transaction_fragment_query_total_duration,
        sum( query_wlm_duration   ) as transaction_fragment_query_wlm_duration,
        sum( query_queue_duration ) as transaction_fragment_query_queue_duration,
        sum( query_compile_exec_duration ) as transaction_fragment_query_compile_exec_duration,
        0                     as transaction_fragment_number_csc_queries,
        0                     as transaction_fragment_number_rc_queries,
        count(*)              as transaction_fragment_number_leader_queries,
        0                     as transaction_fragment_number_worker_queries,
        false                 as transaction_fragment_abort_flag
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
      group by
        user_id,
        session_id,
        transaction_id

      union all

      select
        user_id,
        session_id,
        transaction_id,
        min( query_start_ts )              as transaction_fragment_start_ts,
        max( query_end_ts )                as transaction_fragment_end_ts,
        sum( query_total_duration )        as transaction_fragment_query_total_duration,
        sum( query_wlm_duration )          as transaction_fragment_query_wlm_duration,
        sum( query_queue_duration )        as transaction_fragment_query_queue_duration,
        sum( query_compile_exec_duration ) as transaction_fragment_query_compile_exec_duration,
        sum( case when query_csc_flag is true then 1 else 0 end )                       as transaction_fragment_number_csc_queries,
        sum( case when base_241129.fixed_stl_wlm_query.query_id is null then 1 else 0 end ) as transaction_fragment_number_rc_queries,
        0                                  as transaction_fragment_number_leader_queries,
        count(*)                           as transaction_fragment_number_worker_queries,
        case when max( case when query_abort_flag is true then 1 else 0 end ) = 1 then true else false end as transaction_fragment_abort_flag
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, transaction_id, user_id )
      group by
        user_id,
        session_id,
        transaction_id
    ),

    cte_transactions as
    (
      select
        user_id,
        session_id,
        transaction_id,
        min( transaction_fragment_start_ts )                    as transaction_start_ts,
        max( transaction_fragment_end_ts )                      as transaction_end_ts,
        sum( transaction_fragment_query_queue_duration )        as transaction_query_queue_duration,
        sum( transaction_fragment_query_compile_exec_duration ) as transaction_query_compile_exec_duration,
        sum( transaction_fragment_query_total_duration )        as transaction_query_total_duration,
        ( transaction_end_ts - transaction_start_ts ) - transaction_query_total_duration as transaction_idle_duration,
        transaction_end_ts - transaction_start_ts               as transaction_duration,
        sum( transaction_fragment_number_csc_queries )          as transaction_number_csc_queries,
        sum( transaction_fragment_number_rc_queries )           as transaction_number_rc_queries,
        sum( transaction_fragment_number_leader_queries )       as transaction_number_leader_queries,
        sum( transaction_fragment_number_worker_queries )       as transaction_number_worker_queries,
        sum( transaction_fragment_number_leader_queries + transaction_fragment_number_worker_queries ) as transaction_number_total_queries,
        case when max( case when transaction_fragment_abort_flag is true then 1 else 0 end ) = 1 then true else false end as transaction_abort_flag
      from
        cte_transaction_fragments
      where
        transaction_id not in ( select distinct xid from stl_vacuum  )
      group by
        user_id,
        session_id,
        transaction_id
    )

  select
    user_id::int4,
    session_id::int4,
    transaction_id::int8,
    transaction_start_ts::timestamp,
    transaction_end_ts::timestamp,
    transaction_query_queue_duration::interval,
    transaction_query_compile_exec_duration::interval,
    transaction_query_total_duration::interval,
    transaction_idle_duration::interval,
    transaction_duration::interval,
    transaction_number_csc_queries::int4,
    transaction_number_rc_queries::int4,
    transaction_number_leader_queries::int4,
    transaction_number_worker_queries::int4,
    transaction_number_total_queries::int4,
    transaction_abort_flag::bool
  from
     cte_transactions;


create or replace view
  lf_241129.transactions
as
  select
    user_id::int4,
    user_name::varchar,
    session_id::int4,
    transaction_id::int8,
    transaction_start_ts::timestamp,
    transaction_end_ts::timestamp,
    transaction_query_queue_duration::interval,
    transaction_query_compile_exec_duration::interval,
    transaction_query_total_duration::interval,
    transaction_idle_duration::interval,
    transaction_duration::interval,
    transaction_number_csc_queries::int4,
    transaction_number_rc_queries::int4,
    transaction_number_leader_queries::int4,
    transaction_number_worker_queries::int4,
    transaction_number_total_queries::int4,
    transaction_abort_flag::bool
  from
         base_241129.transactions
    join base_241129.fixed_pg_user using ( user_id );


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_all_by_day
as
  select
    case
      when slice_ordinal  = -1 then 'leader'
      when slice_ordinal != -1 then 'worker'
    end::varchar as node_type,

    date_trunc( 'day', scan_start_ts )::timestamp as start_ts,

    sum( case when type =  1 then rows  else 0 end )::int8 as sum_type_1_rows,
    sum( case when type =  1 then bytes else 0 end )::int8 as sum_type_1_bytes,

    sum( case when type =  2 then rows_pre_filter      else 0 end )::int8 as sum_type_2m_rows,
    sum( case when type =  2 then rows_pre_user_filter else 0 end )::int8 as sum_type_2e_rows,
    sum( case when type =  2 then rows                 else 0 end )::int8 as sum_type_2_rows,
    sum( case when type =  2 then bytes                else 0 end )::int8 as sum_type_2_bytes,

    sum( case when type =  3 then rows  else 0 end )::int8 as sum_type_3_rows,
    sum( case when type =  3 then bytes else 0 end )::int8 as sum_type_3_bytes,

    sum( case when type =  9 then rows  else 0 end )::int8 as sum_type_9_rows,
    sum( case when type =  9 then bytes else 0 end )::int8 as sum_type_9_bytes,

    sum( case when type = 16 then rows  else 0 end )::int8 as sum_type_16_rows,
    sum( case when type = 16 then bytes else 0 end )::int8 as sum_type_16_bytes,

    sum( case when type = 17 then rows  else 0 end )::int8 as sum_type_17_rows,
    sum( case when type = 17 then bytes else 0 end )::int8 as sum_type_17_bytes,

    sum( case when type = 18 then rows  else 0 end )::int8 as sum_type_18_rows,
    sum( case when type = 18 then bytes else 0 end )::int8 as sum_type_18_bytes,

    sum( case when type = 21 then rows  else 0 end )::int8 as sum_type_21_rows,
    sum( case when type = 21 then bytes else 0 end )::int8 as sum_type_21_bytes,

    sum( case when type = 22 then rows  else 0 end )::int8 as sum_type_22_rows,
    sum( case when type = 22 then bytes else 0 end )::int8 as sum_type_22_bytes,

    sum( case when type = 23 then rows  else 0 end )::int8 as sum_type_23_rows,
    sum( case when type = 23 then bytes else 0 end )::int8 as sum_type_23_bytes,

    sum( case when type = 24 then rows  else 0 end )::int8 as sum_type_24_rows,
    sum( case when type = 24 then bytes else 0 end )::int8 as sum_type_24_bytes,

    sum( case when type = 25 then rows  else 0 end )::int8 as sum_type_25_rows,
    sum( case when type = 25 then bytes else 0 end )::int8 as sum_type_25_bytes,

    sum( case when type = 28 then rows  else 0 end )::int8 as sum_type_28_rows,
    sum( case when type = 28 then bytes else 0 end )::int8 as sum_type_28_bytes,

    sum( case when type = 29 then rows_pre_filter      else 0 end )::int8 as sum_type_29m_rows,
    sum( case when type = 29 then rows_pre_user_filter else 0 end )::int8 as sum_type_29e_rows,
    sum( case when type = 29 then rows                 else 0 end )::int8 as sum_type_29_rows,
    sum( case when type = 29 then bytes                else 0 end )::int8 as sum_type_29_bytes,

    sum( case when type = 30 then rows  else 0 end )::int8 as sum_type_30_rows,
    sum( case when type = 30 then bytes else 0 end )::int8 as sum_type_30_bytes,

    sum( case when type = 32 then rows  else 0 end )::int8 as sum_type_32_rows,
    sum( case when type = 32 then bytes else 0 end )::int8 as sum_type_32_bytes,

    sum( case when type = 33 then rows  else 0 end )::int8 as sum_type_33_rows,
    sum( case when type = 33 then bytes else 0 end )::int8 as sum_type_33_bytes,

    sum( case when type = 35 then rows  else 0 end )::int8 as sum_type_35_rows,
    sum( case when type = 35 then bytes else 0 end )::int8 as sum_type_35_bytes,

    sum( case when type = 36 then rows  else 0 end )::int8 as sum_type_36_rows,
    sum( case when type = 36 then bytes else 0 end )::int8 as sum_type_36_bytes,

    sum( case when type = 37 then rows  else 0 end )::int8 as sum_type_37_rows,
    sum( case when type = 37 then bytes else 0 end )::int8 as sum_type_37_bytes
  from
    base_241129.fixed_stl_scan
  where
        scan_start_ts >= current_date - interval '6 day'
    and scan_end_ts   <= current_timestamp
  group by
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_all_by_day
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_all_by_day
    )

  select
    node_type::varchar                                                                                                                     as node,
    start_ts::date,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    start_ts;



create or replace procedure
  ro.sp_grant_versionless_sf_schema_to_role( role_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'grant usage on schema sf to role ' || role_name;
      execute sql;

      sql := 'grant select on all tables in schema sf to role ' || role_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.blocks_totals_for_tables_by_column
as
  with
    cte_blocks as
    (
      select
        table_id,
        column_ordinal,
        sum( case when block_sorted_flag = true  then 1 else 0 end ) as number_sorted_blocks,
        sum( case when block_sorted_flag = false then 1 else 0 end ) as number_unsorted_blocks,
        count( * )                                                   as number_total_blocks
      from
        base_241129.fixed_stv_blocklist
      group by
        table_id,
        column_ordinal
    )

    select
      schema_id::int4,
      schema_name::varchar,
      table_id::int4,
      table_name::varchar,
      column_ordinal::int2,
      column_name::varchar,
      cte_blocks.number_sorted_blocks::int8,
      cte_blocks.number_unsorted_blocks::int8,
      cte_blocks.number_total_blocks::int8
    from
           cte_blocks
      join base_241129.fixed_pg_class_for_tables     using ( table_id )
      join base_241129.fixed_pg_namespace            using ( schema_id )
      join base_241129.fixed_pg_attribute_for_tables using ( table_id, column_ordinal );


create or replace view
  sf_241129.blocks_totals_for_tables_by_column
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        column_name,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks
      from
        lf_241129.blocks_totals_for_tables_by_column
    )

  select
    schema_name::varchar                                                           as sschema,
    table_name::varchar                                                            as ttable,
    column_name::varchar                                                           as ccolumn,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    column_name;


create or replace view
  lf_241129.steps_totals_data_on_both_for_slices_by_hour
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'hour', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    slice_ordinal::int2,
    start_ts::timestamp,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_hours
     left join cte_aggr         using( start_ts, slice_ordinal )
     left join cte_bcast        using( start_ts, slice_ordinal )
     left join cte_delete       using( start_ts, slice_ordinal )
     left join cte_dist         using( start_ts, slice_ordinal )
     left join cte_hash         using( start_ts, slice_ordinal )
     left join cte_hashjoin     using( start_ts, slice_ordinal )
     left join cte_insert       using( start_ts, slice_ordinal )
     left join cte_limit        using( start_ts, slice_ordinal )
     left join cte_merge        using( start_ts, slice_ordinal )
     left join cte_mergejoin    using( start_ts, slice_ordinal )
     left join cte_nestloop     using( start_ts, slice_ordinal )
     left join cte_parse        using( start_ts, slice_ordinal )
     left join cte_project      using( start_ts, slice_ordinal )
     left join cte_return       using( start_ts, slice_ordinal )
     left join cte_save         using( start_ts, slice_ordinal )
     left join cte_sort         using( start_ts, slice_ordinal )
     left join cte_spatial_join using( start_ts, slice_ordinal )
     left join cte_unique       using( start_ts, slice_ordinal )
     left join cte_window       using( start_ts, slice_ordinal )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.steps_totals_data_on_both_for_slices_by_hour
as
  select
    slice_ordinal::int2 as slice,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_slices_by_hour
  order by
    slice_ordinal,
    start_ts;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_users_by_minute
as
  with
    cte_transactions as
    (
      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        user_id,
        transaction_id,
        date_trunc( 'minute', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        user_id,
        transaction_id
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                                      as user_name,
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
    left join pg_user                        on cte_all_transactions.user_id        = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_users_by_minute
as
  select
    user_name::varchar                 as uuser,
    start_ts::timestamp,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_users_by_minute
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_all_by_hour
as
  select
    case
      when slice_ordinal  = -1 then 'leader'
      when slice_ordinal != -1 then 'worker'
    end::varchar as node_type,

    date_trunc( 'hour', scan_start_ts )::timestamp as start_ts,

    sum( case when type =  1 then rows  else 0 end )::int8 as sum_type_1_rows,
    sum( case when type =  1 then bytes else 0 end )::int8 as sum_type_1_bytes,

    sum( case when type =  2 then rows_pre_filter      else 0 end )::int8 as sum_type_2m_rows,
    sum( case when type =  2 then rows_pre_user_filter else 0 end )::int8 as sum_type_2e_rows,
    sum( case when type =  2 then rows                 else 0 end )::int8 as sum_type_2_rows,
    sum( case when type =  2 then bytes                else 0 end )::int8 as sum_type_2_bytes,

    sum( case when type =  3 then rows  else 0 end )::int8 as sum_type_3_rows,
    sum( case when type =  3 then bytes else 0 end )::int8 as sum_type_3_bytes,

    sum( case when type =  9 then rows  else 0 end )::int8 as sum_type_9_rows,
    sum( case when type =  9 then bytes else 0 end )::int8 as sum_type_9_bytes,

    sum( case when type = 16 then rows  else 0 end )::int8 as sum_type_16_rows,
    sum( case when type = 16 then bytes else 0 end )::int8 as sum_type_16_bytes,

    sum( case when type = 17 then rows  else 0 end )::int8 as sum_type_17_rows,
    sum( case when type = 17 then bytes else 0 end )::int8 as sum_type_17_bytes,

    sum( case when type = 18 then rows  else 0 end )::int8 as sum_type_18_rows,
    sum( case when type = 18 then bytes else 0 end )::int8 as sum_type_18_bytes,

    sum( case when type = 21 then rows  else 0 end )::int8 as sum_type_21_rows,
    sum( case when type = 21 then bytes else 0 end )::int8 as sum_type_21_bytes,

    sum( case when type = 22 then rows  else 0 end )::int8 as sum_type_22_rows,
    sum( case when type = 22 then bytes else 0 end )::int8 as sum_type_22_bytes,

    sum( case when type = 23 then rows  else 0 end )::int8 as sum_type_23_rows,
    sum( case when type = 23 then bytes else 0 end )::int8 as sum_type_23_bytes,

    sum( case when type = 24 then rows  else 0 end )::int8 as sum_type_24_rows,
    sum( case when type = 24 then bytes else 0 end )::int8 as sum_type_24_bytes,

    sum( case when type = 25 then rows  else 0 end )::int8 as sum_type_25_rows,
    sum( case when type = 25 then bytes else 0 end )::int8 as sum_type_25_bytes,

    sum( case when type = 28 then rows  else 0 end )::int8 as sum_type_28_rows,
    sum( case when type = 28 then bytes else 0 end )::int8 as sum_type_28_bytes,

    sum( case when type = 29 then rows_pre_filter      else 0 end )::int8 as sum_type_29m_rows,
    sum( case when type = 29 then rows_pre_user_filter else 0 end )::int8 as sum_type_29e_rows,
    sum( case when type = 29 then rows                 else 0 end )::int8 as sum_type_29_rows,
    sum( case when type = 29 then bytes                else 0 end )::int8 as sum_type_29_bytes,

    sum( case when type = 30 then rows  else 0 end )::int8 as sum_type_30_rows,
    sum( case when type = 30 then bytes else 0 end )::int8 as sum_type_30_bytes,

    sum( case when type = 32 then rows  else 0 end )::int8 as sum_type_32_rows,
    sum( case when type = 32 then bytes else 0 end )::int8 as sum_type_32_bytes,

    sum( case when type = 33 then rows  else 0 end )::int8 as sum_type_33_rows,
    sum( case when type = 33 then bytes else 0 end )::int8 as sum_type_33_bytes,

    sum( case when type = 35 then rows  else 0 end )::int8 as sum_type_35_rows,
    sum( case when type = 35 then bytes else 0 end )::int8 as sum_type_35_bytes,

    sum( case when type = 36 then rows  else 0 end )::int8 as sum_type_36_rows,
    sum( case when type = 36 then bytes else 0 end )::int8 as sum_type_36_bytes,

    sum( case when type = 37 then rows  else 0 end )::int8 as sum_type_37_rows,
    sum( case when type = 37 then bytes else 0 end )::int8 as sum_type_37_bytes
  from
    base_241129.fixed_stl_scan
  where
        scan_start_ts >= current_date - interval '6 day'
    and scan_end_ts   <= current_timestamp
  group by
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_all_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_all_by_hour
    )

  select
    node_type::varchar                                                                                                                     as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    start_ts;



create or replace view
  sf_241129.ttables_columns_defaults_references
as
  select
    schema_name::varchar                                                                                                             as sschema,
    table_name::varchar                                                                                                              as ttable,
    column_name::varchar                                                                                                             as ccolumn,
    ( column_default_type || '(' || column_default_value || ')' )::varchar                                                           as def,
    ( column_references_schema_name || '.' || column_references_table_name || '(' || column_references_column_name || ')' )::varchar as rref
  from
    lf_241129.ttables_columns
  order by
    schema_name,
    table_name,
    column_name;


create or replace view
  lf_241129.queries_for_leader
as
  select
    user_id::int4,
    user_name::varchar,
    session_id::int4,
    transaction_id::int8,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_total_duration::interval,
    query_leader_node_flag::bool,
    query_text::varchar
  from
         base_241129.queries_for_leader
    join base_241129.fixed_pg_user using ( user_id );


create or replace procedure
  ro.sp_grant_version_to_role( version in varchar(6), role_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'grant usage on schema base_' || version || ' to role ' || role_name;
      execute sql;

      sql := 'grant usage on schema lf_' || version || ' to role ' || role_name;
      execute sql;

      sql := 'grant usage on schema sf_' || version || ' to role ' || role_name;
      execute sql;

      sql := 'grant select on all tables in schema base_' || version || ' to role ' || role_name;
      execute sql;

      sql := 'grant select on all tables in schema lf_' || version || ' to role ' || role_name;
      execute sql;

      sql := 'grant select on all tables in schema sf_' || version || ' to role ' || role_name;
      execute sql;

      sql := 'grant execute on all functions  in schema base_' || version || ' to role ' || role_name;
      execute sql;

      sql := 'grant execute on all procedures in schema lf_' || version || ' to role ' || role_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_all_by_second
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    ),

    cte_scan as
    (
      select
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        start_ts
    )

  select
    start_ts::timestamp,
    case when sum_delete_rows is null then 0 else sum_delete_rows end::int8 as notnulled_sum_delete_rows,
    case when sum_insert_rows is null then 0 else sum_insert_rows end::int8 as notnulled_sum_insert_rows,
    case when sum_scan_rows   is null then 0 else sum_scan_rows   end::int8 as notnulled_sum_scan_rows
  from
         base_241129.generate_seconds
    left join cte_delete using ( start_ts )
    left join cte_insert using ( start_ts )
    left join cte_scan   using ( start_ts );


create or replace view
  lf_241129.steps_stats_durations_on_both_for_all_by_all
as
  with
    cte_aggr as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        node_type
    ),

    cte_bcast as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        node_type
    ),

    cte_delete as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        node_type
    ),

    cte_dist as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        node_type
    ),

    cte_hash as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        node_type
    ),

    cte_hashjoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        node_type
    ),

    cte_insert as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        node_type
    ),

    cte_limit as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        node_type
    ),

    cte_merge as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        node_type
    ),

    cte_mergejoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        node_type
    ),

    cte_nestloop as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        node_type
    ),

    cte_parse as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        node_type
    ),

    cte_project as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        node_type
    ),

    cte_return as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        node_type
    ),

    cte_save as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        node_type
    ),

    cte_sort as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        node_type
    ),

    cte_spatial_join as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        node_type
    ),

    cte_unique as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        node_type
    ),

    cte_window as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        node_type,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
              cte_node_types
    left join cte_aggr_stats         using ( node_type )
    left join cte_bcast_stats        using ( node_type )
    left join cte_delete_stats       using ( node_type )
    left join cte_dist_stats         using ( node_type )
    left join cte_hash_stats         using ( node_type )
    left join cte_hashjoin_stats     using ( node_type )
    left join cte_insert_stats       using ( node_type )
    left join cte_limit_stats        using ( node_type )
    left join cte_merge_stats        using ( node_type )
    left join cte_mergejoin_stats    using ( node_type )
    left join cte_nestloop_stats     using ( node_type )
    left join cte_parse_stats        using ( node_type )
    left join cte_project_stats      using ( node_type )
    left join cte_return_stats       using ( node_type )
    left join cte_save_stats         using ( node_type )
    left join cte_sort_stats         using ( node_type )
    left join cte_spatial_join_stats using ( node_type )
    left join cte_unique_stats       using ( node_type )
    left join cte_window_stats       using ( node_type );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_all_concise
as
  select
    node_type::varchar as node,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_all
  order by
    node_type;


create or replace view
  lf_241129.steps_totals_data_on_both_for_all_by_hour
as
  with
    cte_aggr as
    (
      select
        date_trunc( 'hour', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        date_trunc( 'hour', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        date_trunc( 'hour', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        date_trunc( 'hour', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        date_trunc( 'hour', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        date_trunc( 'hour', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        date_trunc( 'hour', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        date_trunc( 'hour', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        date_trunc( 'hour', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_hours.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               cte_node_types
    cross join base_241129.generate_hours
     left join cte_aggr         using ( node_type, start_ts )
     left join cte_bcast        using ( node_type, start_ts )
     left join cte_delete       using ( node_type, start_ts )
     left join cte_dist         using ( node_type, start_ts )
     left join cte_hash         using ( node_type, start_ts )
     left join cte_hashjoin     using ( node_type, start_ts )
     left join cte_insert       using ( node_type, start_ts )
     left join cte_limit        using ( node_type, start_ts )
     left join cte_merge        using ( node_type, start_ts )
     left join cte_mergejoin    using ( node_type, start_ts )
     left join cte_nestloop     using ( node_type, start_ts )
     left join cte_parse        using ( node_type, start_ts )
     left join cte_project      using ( node_type, start_ts )
     left join cte_return       using ( node_type, start_ts )
     left join cte_save         using ( node_type, start_ts )
     left join cte_sort         using ( node_type, start_ts )
     left join cte_spatial_join using ( node_type, start_ts )
     left join cte_unique       using ( node_type, start_ts )
     left join cte_window       using ( node_type, start_ts );


create or replace view
  lf_241129.ddatabases
as
  select
    pg_database.oid::int4                                   as database_id,
    trim( pg_database.datname )::varchar                    as database_name,
    pg_database.datdba::int4                                as user_id,
    trim( pg_user.usename )::varchar                        as user_name,
    trim( stv_db_isolation_level.isolation_level )::varchar as isolation_method
  from
         pg_database
    join pg_user                on pg_database.datdba  = pg_user.usesysid
    join stv_db_isolation_level on pg_database.datname = stv_db_isolation_level.db_name;


create or replace view
  sf_241129.ddatabases
as
  select
    database_name::varchar,
    user_name::varchar,
    case
      when isolation_method = 'Serializable'       then 'serializable'
      when isolation_method = 'Snapshot Isolation' then 'snapshot'
    end::varchar as iisolation
  from
    lf_241129.ddatabases
  order by
    database_name;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_statements_by_all
as

  with
    cte_hr_formatting as
    (
      select
        statement_name,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_statements_by_all
    )

  select
    statement_name::varchar as stmt,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    statement_name;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_users_by_second
as
  with
    cte_transactions as
    (
      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id

      union all

      select
        user_id,
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        user_id,
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        user_id,
        transaction_id,
        date_trunc( 'second', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        user_id,
        transaction_id
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                                      as user_name,
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
    left join pg_user                        on cte_all_transactions.user_id        = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_users_by_second
as
  select
    user_name::varchar                 as uuser,
    start_ts::timestamp,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_users_by_second
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_day_concise
as
  select
    start_ts::date,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_day
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_schemas_by_hour
as
  select
    schema_name::varchar                                     as sschema,
    start_ts::timestamp,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_schemas_by_hour
  order by
    schema_name,
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_slices_by_all
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)                   as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        slice_ordinal
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        slice_ordinal
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        slice_ordinal
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        slice_ordinal
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        slice_ordinal
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        slice_ordinal
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        slice_ordinal
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        slice_ordinal
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        slice_ordinal
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        slice_ordinal
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        slice_ordinal
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        slice_ordinal
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        slice_ordinal
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        slice_ordinal
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        slice_ordinal
    ),

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        slice_ordinal
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        slice_ordinal
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        slice_ordinal
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end as slice_ordinal,
        max(endtime) - min(starttime)        as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        slice_ordinal,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        slice_ordinal
    )

  select
    base_241129.fixed_stv_slices.node_ordinal::int2,
    base_241129.fixed_stv_slices.slice_ordinal::int2,
    base_241129.fixed_stv_slices.slice_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
              base_241129.fixed_stv_slices
    left join cte_aggr_stats         using ( slice_ordinal )
    left join cte_bcast_stats        using ( slice_ordinal )
    left join cte_delete_stats       using ( slice_ordinal )
    left join cte_dist_stats         using ( slice_ordinal )
    left join cte_hash_stats         using ( slice_ordinal )
    left join cte_hashjoin_stats     using ( slice_ordinal )
    left join cte_insert_stats       using ( slice_ordinal )
    left join cte_limit_stats        using ( slice_ordinal )
    left join cte_merge_stats        using ( slice_ordinal )
    left join cte_mergejoin_stats    using ( slice_ordinal )
    left join cte_nestloop_stats     using ( slice_ordinal )
    left join cte_parse_stats        using ( slice_ordinal )
    left join cte_project_stats      using ( slice_ordinal )
    left join cte_return_stats       using ( slice_ordinal )
    left join cte_save_stats         using ( slice_ordinal )
    left join cte_sort_stats         using ( slice_ordinal )
    left join cte_spatial_join_stats using ( slice_ordinal )
    left join cte_unique_stats       using ( slice_ordinal )
    left join cte_window_stats       using ( slice_ordinal );


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_all_by_day
as
  with
    cte_queries as
    (
      select
        date_trunc( 'day', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        date_trunc( 'day', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    base_241129.generate_days.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_days
    left join cte_queries using ( start_ts )
  group by
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_all_by_day
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_all_by_day
    )

  select
    start_ts::date,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_users_by_all
as
  with
    cte_blocks as
    (
      select
        user_id,
        user_name,
        sum( query_number_blocks_read    ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        user_name
    ),

    cte_nulls_to_values as
    (
      select
        user_id,
        user_name,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    user_id::int4,
    user_name::varchar,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_hour_concise
as
  select
    node_type::varchar as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_hour
  order by
    node_type,
    start_ts;


create or replace view
  lf_241129.analyze_totals_status_for_schema
as
  select
    pg_class.relnamespace::int4                                                                                                                                                   as schema_id,
    trim( pg_namespace.nspname )::varchar                                                                                                                                         as schema_name,

    sum( case when pg_class.reltuples  = pg_statistic_indicator.stairows and pg_statistic_indicator.staiins = 0 and pg_statistic_indicator.staidels = 0 then 1 else 0 end )::int4 as number_good_stats,
    sum( case when pg_class.reltuples != pg_statistic_indicator.stairows or  pg_statistic_indicator.staiins > 0 or  pg_statistic_indicator.staidels > 0 then 1 else 0 end )::int4 as number_stale_stats,
    sum( case when pg_statistic_indicator.stairelid is null or pg_class.reltuples = 0.001::float4 then 1 else 0 end )::int4                                                       as number_missing_stats,

    count(*)::int4                                                                                                                                                                as number_total_stats
  from
              pg_namespace
    left join pg_class               on pg_class.relnamespace = pg_namespace.oid
    left join pg_statistic_indicator on pg_class.oid          = pg_statistic_indicator.stairelid
  where
    relkind = 'r'
  group by
    schema_id,
    schema_name;


create or replace view
  sf_241129.analyze_totals_status_for_schema
as
  select
    schema_name::varchar       as sschema,
    number_good_stats::int4    as good,
    number_stale_stats::int4   as stale,
    number_missing_stats::int4 as missing,
    number_total_stats::int4   as total
  from
    lf_241129.analyze_totals_status_for_schema
  order by
    schema_name;


create or replace view
  lf_241129.vacuum_totals_for_tables_by_day
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        table_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        table_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        table_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        table_id,
        date_trunc( 'day', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        table_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        table_id,
        date_trunc( 'day', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        table_id,
        start_ts
    )

  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    cte_number_vacuums.table_id::int4,
    trim( pg_class.relname )::varchar     as table_name,

    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration                using ( table_id, start_ts )
    left join pg_class     on pg_class.oid          = cte_number_vacuums.table_id
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid;


create or replace view
  base_241129.nodes
as
  with
    cte_nodes_and_slices as
    (
      select
        node                                          as node_ordinal,
        sum( case when type = 'C' then 1 else 0 end ) as number_partial_slices,
        sum( case when type = 'D' then 1 else 0 end ) as number_full_slices,
        count(*)                                      as number_total_slices
      from
        stv_slices
      group by
        node

      union all

      select
        -1 as node_ordinal,
         0 as number_partial_slices,
         0 as number_full_slices,
         0 as number_total_slices
    )

  select
    node_ordinal::int2,
    number_partial_slices::int2,
    number_full_slices::int2,
    number_total_slices::int2
  from
    cte_nodes_and_slices;


create or replace view
  sf_241129.blocks_rows_totals_for_user
as
  with
    cte_hr_formatting as
    (
      select
        user_name,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_desorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_desorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_desorted_blocks) ) over () as max_len_hr_number_desorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks,

        max( len(hr_number_sorted_rows)   ) over () as max_len_hr_number_sorted_rows,
        max( len(hr_number_desorted_rows) ) over () as max_len_hr_number_desorted_rows,
        max( len(hr_number_unsorted_rows) ) over () as max_len_hr_number_unsorted_rows,
        max( len(hr_number_total_rows)    ) over () as max_len_hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_user
    )

  select
    user_name::varchar                                                             as uuser,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_desorted_blocks, max_len_hr_number_desorted_blocks )::varchar  as dsblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks,

    lpad( hr_number_sorted_rows,   max_len_hr_number_sorted_rows   )::varchar      as srows,
    lpad( hr_number_desorted_rows, max_len_hr_number_desorted_rows )::varchar      as dsrows,
    lpad( hr_number_unsorted_rows, max_len_hr_number_unsorted_rows )::varchar      as urows,
    lpad( hr_number_total_rows,    max_len_hr_number_total_rows    )::varchar      as trows
  from
    cte_hr_formatting
  order by
    user_name;


create or replace view
  lf_241129.blocks_rows_totals_for_slice
as
  with
    cte_slices_blocks_rows_s1 as
    (
      select
        stv_blocklist.slice                                                                                            as slice_ordinal,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      where
        tbl != 0
      group by
        slice
    )

  select
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,
    number_sorted_blocks::int8,
    null::int8 as number_desorted_blocks,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    null::int8 as number_desorted_rows,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
              base_241129.fixed_stv_slices
    left join cte_slices_blocks_rows_s1 using ( slice_ordinal );


create or replace view
  sf_241129.queries_text_on_workers_full
as
  select
    session_id::int4       as pid,
    transaction_id::int8   as xid,
    query_id::int4         as qid,
    query_text_order::int4 as n,
    trim( chr(10) || ' ' from query_text )::varchar as ttext
  from
    lf_241129.queries_text_on_workers_full
  order by
    query_id,
    query_text_order;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_query
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        query,
        node_type
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        query,
        node_type
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        query,
        node_type
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        query,
        node_type
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        query,
        node_type
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        query,
        node_type
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        query,
        node_type
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        query,
        node_type
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        query,
        node_type
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        query,
        node_type
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        query,
        node_type
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        query,
        node_type
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        query,
        node_type
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        query,
        node_type
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        query,
        node_type
    ),

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        query,
        node_type
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        query,
        node_type
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        query,
        node_type
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        query,
        node_type,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        query,
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    cte_base.query::int4 as query_id,
    node_type::varchar,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join cte_base
     left join cte_aggr_stats         using ( node_type, query )
     left join cte_bcast_stats        using ( node_type, query )
     left join cte_delete_stats       using ( node_type, query )
     left join cte_dist_stats         using ( node_type, query )
     left join cte_hash_stats         using ( node_type, query )
     left join cte_hashjoin_stats     using ( node_type, query )
     left join cte_insert_stats       using ( node_type, query )
     left join cte_limit_stats        using ( node_type, query )
     left join cte_merge_stats        using ( node_type, query )
     left join cte_mergejoin_stats    using ( node_type, query )
     left join cte_nestloop_stats     using ( node_type, query )
     left join cte_parse_stats        using ( node_type, query )
     left join cte_project_stats      using ( node_type, query )
     left join cte_return_stats       using ( node_type, query )
     left join cte_save_stats         using ( node_type, query )
     left join cte_sort_stats         using ( node_type, query )
     left join cte_spatial_join_stats using ( node_type, query )
     left join cte_unique_stats       using ( node_type, query )
     left join cte_window_stats       using ( node_type, query )
  where
       min_aggr_duration is not null
    or max_aggr_duration is not null
    or med_aggr_duration is not null
    or avg_aggr_duration is not null
    or std_aggr_duration is not null
    or number_aggr_steps is not null
    or min_bcast_duration is not null
    or max_bcast_duration is not null
    or med_bcast_duration is not null
    or avg_bcast_duration is not null
    or std_bcast_duration is not null
    or number_bcast_steps is not null
    or min_delete_duration is not null
    or max_delete_duration is not null
    or med_delete_duration is not null
    or avg_delete_duration is not null
    or std_delete_duration is not null
    or number_delete_steps is not null
    or min_dist_duration is not null
    or max_dist_duration is not null
    or med_dist_duration is not null
    or avg_dist_duration is not null
    or std_dist_duration is not null
    or number_dist_steps is not null
    or min_hash_duration is not null
    or max_hash_duration is not null
    or med_hash_duration is not null
    or avg_hash_duration is not null
    or std_hash_duration is not null
    or number_hash_steps is not null
    or min_hashjoin_duration is not null
    or max_hashjoin_duration is not null
    or med_hashjoin_duration is not null
    or avg_hashjoin_duration is not null
    or std_hashjoin_duration is not null
    or number_hashjoin_steps is not null
    or min_insert_duration is not null
    or max_insert_duration is not null
    or med_insert_duration is not null
    or avg_insert_duration is not null
    or std_insert_duration is not null
    or number_insert_steps is not null
    or min_limit_duration is not null
    or max_limit_duration is not null
    or med_limit_duration is not null
    or avg_limit_duration is not null
    or std_limit_duration is not null
    or number_limit_steps is not null
    or min_merge_duration is not null
    or max_merge_duration is not null
    or med_merge_duration is not null
    or avg_merge_duration is not null
    or std_merge_duration is not null
    or number_merge_steps is not null
    or min_mergejoin_duration is not null
    or max_mergejoin_duration is not null
    or med_mergejoin_duration is not null
    or avg_mergejoin_duration is not null
    or std_mergejoin_duration is not null
    or number_mergejoin_steps is not null
    or min_nestloop_duration is not null
    or max_nestloop_duration is not null
    or med_nestloop_duration is not null
    or avg_nestloop_duration is not null
    or std_nestloop_duration is not null
    or number_nestloop_steps is not null
    or min_parse_duration is not null
    or max_parse_duration is not null
    or med_parse_duration is not null
    or avg_parse_duration is not null
    or std_parse_duration is not null
    or number_parse_steps is not null
    or min_project_duration is not null
    or max_project_duration is not null
    or med_project_duration is not null
    or avg_project_duration is not null
    or std_project_duration is not null
    or number_project_steps is not null
    or min_return_duration is not null
    or max_return_duration is not null
    or med_return_duration is not null
    or avg_return_duration is not null
    or std_return_duration is not null
    or number_return_steps is not null
    or min_save_duration is not null
    or max_save_duration is not null
    or med_save_duration is not null
    or avg_save_duration is not null
    or std_save_duration is not null
    or number_save_steps is not null
    or min_sort_duration is not null
    or max_sort_duration is not null
    or med_sort_duration is not null
    or avg_sort_duration is not null
    or std_sort_duration is not null
    or number_sort_steps is not null
    or min_spatial_join_duration is not null
    or max_spatial_join_duration is not null
    or med_spatial_join_duration is not null
    or avg_spatial_join_duration is not null
    or std_spatial_join_duration is not null
    or number_spatial_join_steps is not null
    or min_unique_duration is not null
    or max_unique_duration is not null
    or med_unique_duration is not null
    or avg_unique_duration is not null
    or std_unique_duration is not null
    or number_unique_steps is not null
    or min_window_duration is not null
    or max_window_duration is not null
    or med_window_duration is not null
    or avg_window_duration is not null
    or std_window_duration is not null
    or number_window_steps is not null;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_node_by_second
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_node_by_second
    )

  select
    node_ordinal::int2  as n,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_users_by_second
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_users_by_second
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_all_by_hour
as
  with
    cte_queries as
    (
      select
        date_trunc( 'hour', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        date_trunc( 'hour', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    base_241129.generate_hours.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_hours
    left join cte_queries using ( start_ts )
  group by
    start_ts;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_users_by_day
as
  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    count(*)::int4                       as number_abort
  from
              base_241129.generate_days
    left join base_241129.fixed_stl_query    on date_trunc( 'day', query_start_ts ) = base_241129.generate_days.start_ts
    left join base_241129.fixed_pg_user   using ( user_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    user_id,
    user_name,
    start_ts;



create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_all_complete
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_all
  order by
    user_name,
    node_type;



create or replace view
  sf_241129.steps_totals_data_on_both_for_slices_by_day
as
  select
    slice_ordinal::int2 as slice,
    start_ts::date,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_slices_by_day
  order by
    slice_ordinal,
    start_ts;


create or replace view
  lf_241129.objects_totals_for_user
as
  with
    cte_pg_class as
    (
      select
        relowner                                                                                                              as user_id,

        -- MG2 : number tables
        sum( case when pg_class.relkind = 'r'                                                             then 1 else 0 end ) as number_tables,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then 1 else 0 end ) as number_normal_views,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts  = 0 and stv_mv_info.name is     null then 1 else 0 end ) as number_late_binding_views,
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then 1 else 0 end ) as number_materialized_views,

        -- MG2 : number columns
        sum( case when pg_class.relkind = 'r'                                                             then pg_class.relnatts + 3 else 0 end ) as number_table_columns,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then pg_class.relnatts     else 0 end ) as number_normal_view_columns,
        -- MG2 : relnatts is 0 for late binding views
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then pg_class.relnatts     else 0 end ) as number_materialized_view_columns
      from
                  pg_class 
             join pg_namespace on pg_class.relnamespace = pg_namespace.oid
        left join stv_mv_info  on stv_mv_info.db_name = current_database()   and
                                  stv_mv_info.schema  = pg_namespace.nspname and
                                  stv_mv_info.name    = pg_class.relname
      where
        pg_class.relkind = 'r' or pg_class.relkind = 'v'
      group by
        pg_class.relowner
    ),

    cte_pg_databases as
    (
      select
        datdba   as user_id,
        count(*) as number_databases
      from
        pg_database
      group by
        datdba
    ),

    cte_pg_proc as
    (
      select
        pg_proc.proowner                                                    as user_id,
        sum( case when pg_language.lanname != 'plpgsql' then 1 else 0 end ) as number_functions,
        sum( case when pg_language.lanname  = 'plpgsql' then 1 else 0 end ) as number_procedures
      from
             pg_proc
        join pg_language on pg_proc.prolang = pg_language.oid
      group by
        pg_proc.proowner
    ),

    cte_pg_namespace as
    (
      select
        nspowner as user_id,
        count(*) as number_schemas
      from
        pg_namespace
      group by
        nspowner
    )

  select
    pg_user.usesysid::int4                                                                                                  as user_id,
    trim( pg_user.usename )::varchar                                                                                        as user_name,
    case when cte_pg_databases.number_databases       is null then 0 else cte_pg_databases.number_databases       end::int2 as number_databases,
    case when cte_pg_namespace.number_schemas         is null then 0 else cte_pg_namespace.number_schemas         end::int2 as number_schemas,
    case when cte_pg_class.number_tables              is null then 0 else cte_pg_class.number_tables              end::int4 as number_tables,
    case when cte_pg_class.number_table_columns       is null then 0 else cte_pg_class.number_table_columns       end::int4 as number_table_columns,
    case when cte_pg_class.number_normal_views        is null then 0 else cte_pg_class.number_normal_views        end::int4 as number_normal_views,
    case when cte_pg_class.number_normal_view_columns is null then 0 else cte_pg_class.number_normal_view_columns end::int4 as number_normal_view_columns,
    case when cte_pg_class.number_late_binding_views  is null then 0 else cte_pg_class.number_late_binding_views  end::int4 as number_late_binding_views,
    case when cte_pg_class.number_materialized_views  is null then 0 else cte_pg_class.number_materialized_views  end::int4 as number_materialized_views,
    case when cte_pg_class.number_materialized_view_columns  is null then 0 else cte_pg_class.number_materialized_view_columns  end::int4 as number_materialized_view_columns,
    case when cte_pg_proc.number_functions            is null then 0 else cte_pg_proc.number_functions            end::int4 as number_functions,
    case when cte_pg_proc.number_procedures           is null then 0 else cte_pg_proc.number_procedures           end::int4 as number_procedures
  from
              pg_user
    left join cte_pg_class     on pg_user.usesysid = cte_pg_class.user_id 
    left join cte_pg_databases on pg_user.usesysid = cte_pg_databases.user_id
    left join cte_pg_proc      on pg_user.usesysid = cte_pg_proc.user_id
    left join cte_pg_namespace on pg_user.usesysid = cte_pg_namespace.user_id;


create or replace view
  sf_241129.ttables_creation_for_schemas_by_second
as
  select
    schema_name::varchar as sschema,
    start_ts::timestamp,
    number_tables::int4  as ttables
  from
    lf_241129.ttables_creation_for_schemas_by_second
  order by
    schema_name,
    start_ts;


create or replace view
  lf_241129.groups_users
as
  -- MG2 : we could just use `any`, but this procuces a cross join under the hood

  with
    cte_groups_users as
    (
      select
        grosysid,
        generate_series( 1, array_upper(grolist,1) ) as user_array_index
      from
        pg_group
    )

  select
    pg_group.grosysid::int4           as group_id,
    trim( pg_group.groname )::varchar as group_name,
    pg_user.usesysid::int4            as user_id,
    trim( pg_user.usename  )::varchar as user_name
  from
              pg_group
         join cte_groups_users using ( grosysid )
    left join pg_user             on pg_user.usesysid = pg_group.grolist[ cte_groups_users.user_array_index ];


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_users_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_leadin_duration)::numeric(18,2)::varchar as hr_min_leadin_duration,
        extract(epoch from max_leadin_duration)::numeric(18,2)::varchar as hr_max_leadin_duration,
        med_leadin_duration::numeric(18,2)::varchar                     as hr_med_leadin_duration,
        extract(epoch from avg_leadin_duration)::numeric(18,2)::varchar as hr_avg_leadin_duration,
        std_leadin_duration::numeric(18,2)::varchar                     as hr_std_leadin_duration,
        extract(epoch from sum_leadin_duration)::numeric(18,2)::varchar as hr_sum_leadin_duration,

        extract(epoch from min_wlm_leadin_duration)::numeric(18,2)::varchar as hr_min_wlm_leadin_duration,
        extract(epoch from max_wlm_leadin_duration)::numeric(18,2)::varchar as hr_max_wlm_leadin_duration,
        med_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadin_duration,
        extract(epoch from avg_wlm_leadin_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadin_duration,
        std_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadin_duration,
        extract(epoch from sum_wlm_leadin_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadin_duration,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        extract(epoch from min_wlm_leadout_duration)::numeric(18,2)::varchar as hr_min_wlm_leadout_duration,
        extract(epoch from max_wlm_leadout_duration)::numeric(18,2)::varchar as hr_max_wlm_leadout_duration,
        med_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadout_duration,
        extract(epoch from avg_wlm_leadout_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadout_duration,
        std_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadout_duration,
        extract(epoch from sum_wlm_leadout_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadout_duration,

        extract(epoch from min_leadout_duration)::numeric(18,2)::varchar as hr_min_leadout_duration,
        extract(epoch from max_leadout_duration)::numeric(18,2)::varchar as hr_max_leadout_duration,
        med_leadout_duration::numeric(18,2)::varchar                     as hr_med_leadout_duration,
        extract(epoch from avg_leadout_duration)::numeric(18,2)::varchar as hr_avg_leadout_duration,
        std_leadout_duration::numeric(18,2)::varchar                     as hr_std_leadout_duration,
        extract(epoch from sum_leadout_duration)::numeric(18,2)::varchar as hr_sum_leadout_duration,

        extract(epoch from min_query_total_duration)::numeric(18,2)::varchar as hr_min_query_total_duration,
        extract(epoch from max_query_total_duration)::numeric(18,2)::varchar as hr_max_query_total_duration,
        med_query_total_duration::numeric(18,2)::varchar                     as hr_med_query_total_duration,
        extract(epoch from avg_query_total_duration)::numeric(18,2)::varchar as hr_avg_query_total_duration,
        std_query_total_duration::numeric(18,2)::varchar                     as hr_std_query_total_duration,
        extract(epoch from sum_query_total_duration)::numeric(18,2)::varchar as hr_sum_query_total_duration,

        extract(epoch from min_unaccounted_duration)::numeric(18,2)::varchar as hr_min_unaccounted_duration,
        extract(epoch from max_unaccounted_duration)::numeric(18,2)::varchar as hr_max_unaccounted_duration,
        med_unaccounted_duration::numeric(18,2)::varchar                     as hr_med_unaccounted_duration,
        extract(epoch from avg_unaccounted_duration)::numeric(18,2)::varchar as hr_avg_unaccounted_duration,
        std_unaccounted_duration::numeric(18,2)::varchar                     as hr_std_unaccounted_duration,
        extract(epoch from sum_unaccounted_duration)::numeric(18,2)::varchar as hr_sum_unaccounted_duration,

        max( len(hr_min_leadin_duration) ) over () as max_len_hr_min_leadin_duration,
        max( len(hr_max_leadin_duration) ) over () as max_len_hr_max_leadin_duration,
        max( len(hr_med_leadin_duration) ) over () as max_len_hr_med_leadin_duration,
        max( len(hr_avg_leadin_duration) ) over () as max_len_hr_avg_leadin_duration,
        max( len(hr_std_leadin_duration) ) over () as max_len_hr_std_leadin_duration,
        max( len(hr_sum_leadin_duration) ) over () as max_len_hr_sum_leadin_duration,

        max( len(hr_min_wlm_leadin_duration) ) over () as max_len_hr_min_wlm_leadin_duration,
        max( len(hr_max_wlm_leadin_duration) ) over () as max_len_hr_max_wlm_leadin_duration,
        max( len(hr_med_wlm_leadin_duration) ) over () as max_len_hr_med_wlm_leadin_duration,
        max( len(hr_avg_wlm_leadin_duration) ) over () as max_len_hr_avg_wlm_leadin_duration,
        max( len(hr_std_wlm_leadin_duration) ) over () as max_len_hr_std_wlm_leadin_duration,
        max( len(hr_sum_wlm_leadin_duration) ) over () as max_len_hr_sum_wlm_leadin_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        max( len(hr_min_wlm_leadout_duration) ) over () as max_len_hr_min_wlm_leadout_duration,
        max( len(hr_max_wlm_leadout_duration) ) over () as max_len_hr_max_wlm_leadout_duration,
        max( len(hr_med_wlm_leadout_duration) ) over () as max_len_hr_med_wlm_leadout_duration,
        max( len(hr_avg_wlm_leadout_duration) ) over () as max_len_hr_avg_wlm_leadout_duration,
        max( len(hr_std_wlm_leadout_duration) ) over () as max_len_hr_std_wlm_leadout_duration,
        max( len(hr_sum_wlm_leadout_duration) ) over () as max_len_hr_sum_wlm_leadout_duration,

        max( len(hr_min_leadout_duration) ) over () as max_len_hr_min_leadout_duration,
        max( len(hr_max_leadout_duration) ) over () as max_len_hr_max_leadout_duration,
        max( len(hr_med_leadout_duration) ) over () as max_len_hr_med_leadout_duration,
        max( len(hr_avg_leadout_duration) ) over () as max_len_hr_avg_leadout_duration,
        max( len(hr_std_leadout_duration) ) over () as max_len_hr_std_leadout_duration,
        max( len(hr_sum_leadout_duration) ) over () as max_len_hr_sum_leadout_duration,

        max( len(hr_min_query_total_duration) ) over () as max_len_hr_min_query_total_duration,
        max( len(hr_max_query_total_duration) ) over () as max_len_hr_max_query_total_duration,
        max( len(hr_med_query_total_duration) ) over () as max_len_hr_med_query_total_duration,
        max( len(hr_avg_query_total_duration) ) over () as max_len_hr_avg_query_total_duration,
        max( len(hr_std_query_total_duration) ) over () as max_len_hr_std_query_total_duration,
        max( len(hr_sum_query_total_duration) ) over () as max_len_hr_sum_query_total_duration,

        max( len(hr_min_unaccounted_duration) ) over () as max_len_hr_min_unaccounted_duration,
        max( len(hr_max_unaccounted_duration) ) over () as max_len_hr_max_unaccounted_duration,
        max( len(hr_med_unaccounted_duration) ) over () as max_len_hr_med_unaccounted_duration,
        max( len(hr_avg_unaccounted_duration) ) over () as max_len_hr_avg_unaccounted_duration,
        max( len(hr_std_unaccounted_duration) ) over () as max_len_hr_std_unaccounted_duration,
        max( len(hr_sum_unaccounted_duration) ) over () as max_len_hr_sum_unaccounted_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_users_by_minute
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_min_leadin_duration, max_len_hr_min_leadin_duration ) || ' / ' ||
      lpad( hr_max_leadin_duration, max_len_hr_max_leadin_duration ) || ' / ' ||
      lpad( hr_med_leadin_duration, max_len_hr_med_leadin_duration ) || ' / ' ||
      lpad( hr_avg_leadin_duration, max_len_hr_avg_leadin_duration ) || ' / ' ||
      lpad( hr_std_leadin_duration, max_len_hr_std_leadin_duration ) || ' / ' ||
      lpad( hr_sum_leadin_duration, max_len_hr_sum_leadin_duration ) )::varchar as leadin,

    ( lpad( hr_min_wlm_leadin_duration, max_len_hr_min_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadin_duration, max_len_hr_max_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadin_duration, max_len_hr_med_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadin_duration, max_len_hr_avg_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadin_duration, max_len_hr_std_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_sum_wlm_leadin_duration, max_len_hr_sum_wlm_leadin_duration ) )::varchar as wlm_leadin,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    ( lpad( hr_min_wlm_leadout_duration, max_len_hr_min_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadout_duration, max_len_hr_max_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadout_duration, max_len_hr_med_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadout_duration, max_len_hr_avg_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadout_duration, max_len_hr_std_wlm_leadout_duration ) || ' / ' || 
      lpad( hr_sum_wlm_leadout_duration, max_len_hr_sum_wlm_leadout_duration ) )::varchar as wlm_leadout,

    ( lpad( hr_min_leadout_duration, max_len_hr_min_leadout_duration ) || ' / ' ||
      lpad( hr_max_leadout_duration, max_len_hr_max_leadout_duration ) || ' / ' ||
      lpad( hr_med_leadout_duration, max_len_hr_med_leadout_duration ) || ' / ' ||
      lpad( hr_avg_leadout_duration, max_len_hr_avg_leadout_duration ) || ' / ' ||
      lpad( hr_std_leadout_duration, max_len_hr_std_leadout_duration ) || ' / ' || 
      lpad( hr_sum_leadout_duration, max_len_hr_sum_leadout_duration ) )::varchar as leadout,

    ( lpad( hr_min_query_total_duration, max_len_hr_min_query_total_duration ) || ' / ' ||
      lpad( hr_max_query_total_duration, max_len_hr_max_query_total_duration ) || ' / ' ||
      lpad( hr_med_query_total_duration, max_len_hr_med_query_total_duration ) || ' / ' ||
      lpad( hr_avg_query_total_duration, max_len_hr_avg_query_total_duration ) || ' / ' ||
      lpad( hr_std_query_total_duration, max_len_hr_std_query_total_duration ) || ' / ' || 
      lpad( hr_sum_query_total_duration, max_len_hr_sum_query_total_duration ) )::varchar as total,

    ( lpad( hr_min_unaccounted_duration, max_len_hr_min_unaccounted_duration ) || ' / ' ||
      lpad( hr_max_unaccounted_duration, max_len_hr_max_unaccounted_duration ) || ' / ' ||
      lpad( hr_med_unaccounted_duration, max_len_hr_med_unaccounted_duration ) || ' / ' ||
      lpad( hr_avg_unaccounted_duration, max_len_hr_avg_unaccounted_duration ) || ' / ' ||
      lpad( hr_std_unaccounted_duration, max_len_hr_std_unaccounted_duration ) || ' / ' || 
      lpad( hr_sum_unaccounted_duration, max_len_hr_sum_unaccounted_duration ) )::varchar as unact,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_statements_by_minute
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'aggr' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'bcast' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'delete' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'dist' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hash' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hashjoin' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'insert' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'limit' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'merge' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'mergejoin' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'nestloop_op' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'parse' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'project' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'return' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'save' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'sort' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'spatial_join' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'unique' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'window' as step_type,
        date_trunc( 'minute', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_steps as
    (
      select * from cte_aggr         union all
      select * from cte_bcast        union all
      select * from cte_delete       union all
      select * from cte_dist         union all
      select * from cte_hash         union all
      select * from cte_hashjoin     union all
      select * from cte_insert       union all
      select * from cte_limit        union all
      select * from cte_merge        union all
      select * from cte_mergejoin    union all
      select * from cte_nestloop     union all
      select * from cte_parse        union all
      select * from cte_project      union all
      select * from cte_return       union all
      select * from cte_save         union all
      select * from cte_sort         union all
      select * from cte_spatial_join union all
      select * from cte_unique       union all
      select * from cte_window
    )

  select
    statement_name::varchar,
    node_type::varchar,
    step_type::varchar,
    start_ts::timestamp,

    min( duration )::interval                          as min_duration,
    max( duration )::interval                          as max_duration,
    median( extract(epoch from duration) )::float4     as med_duration,
    avg( duration )::interval                          as avg_duration,
    stddev_pop( extract(epoch from duration) )::float4 as std_duration,
    count(*)::int8                                     as number_steps
  from
                 cte_base
    natural join cte_steps
  group by
    statement_name,
    node_type,
    step_type,
    start_ts;


create or replace view
  lf_241129.views_normal
as
  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4                    as view_id,
    trim( pg_class.relname )::varchar     as view_name,
    pg_class.relowner::int4               as user_id,
    trim( pg_user.usename )::varchar      as user_name
  from
              pg_class
         join pg_namespace on pg_class.relnamespace = pg_namespace.oid
         join pg_user      on pg_class.relowner     = pg_user.usesysid
    left join stv_mv_info  on     stv_mv_info.db_name = current_database()
                              and stv_mv_info.schema  = pg_namespace.nspname
                              and stv_mv_info.name    = pg_class.relname
  where
        pg_class.relkind   = 'v'
    and pg_class.relnatts >  0     -- MG2 : no late-binding views
    and stv_mv_info.name  is null; -- MG2 : no materialized views


create or replace procedure
  lf_241129.sp_create_group_if_not_exists( group_name in varchar(127) )
as
  $proc$
    declare
      msg  varchar(4096);
      row  record;
      sql  varchar(4096);

    begin
      -- MG2 : check the group exists; if not, make it
      sql := 'select count(*) as number_groups from pg_group where groname = ' || $$'$$ || group_name || $$'$$ || ';';
      execute sql into row;

      if row.number_groups = 0 then
        sql := 'create group ' || group_name;
        execute sql;
        msg := 'Growp "' || group_name || '" created.';
      else
        msg := 'Growp "' || group_name || '" already exists.';
      end if;

      raise info '%', msg;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.queries_totals_result_cache_for_all_by_hour
as
  select
    date_trunc( 'hour', query_start_ts )::timestamp                                                           as start_ts,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
    count(*)::int8                                                                                           as number_total,
    ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
  from
              base_241129.fixed_stl_query
    left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
  where
        query_abort_flag  = false
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_day_complete
as
  select
    start_ts::date,
    node_ordinal::int2 as n,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_day
  order by
    start_ts,
    node_ordinal;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_slices_by_hour
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'hour', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal );


create or replace view
  lf_241129.views_normal_columns
as
  select
    pg_class.relnamespace::int4           as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4                    as view_id,
    trim( pg_class.relname )::varchar     as view_name,
    pg_attribute.attnum::int2             as column_ordinal,
    trim( pg_attribute.attname )::varchar as column_name,
    pg_attribute.atttypid::int4           as column_data_type_id,
    case
      when pg_type.typname in ( 'bpchar', 'char' )  then 'char'
      when left(pg_type.typname, 1) = '_'           then substring(pg_type.typname from 2) || '[]'
                                                    else pg_type.typname::varchar
    end::varchar                          as column_base_data_type_name,
    case
      when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod != -1 then pg_attribute.atttypmod-4
      when pg_type.typname in ( '_bpchar', '_char', '_varchar', 'bpchar', 'char', 'varchar' ) and pg_attribute.atttypmod  = -1 then 1::int4
    end::int4                             as column_octet_length,
    case
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then ( (pg_attribute.atttypmod-4) & 65535 )::int2
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null::int2 -- MG2 : variable length, so unknown
    end::int2                             as column_precision,
    case
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod != -1 then ( (pg_attribute.atttypmod-4) >> 16 )::int2
      when pg_type.typname in ('_numeric', 'numeric' ) and pg_attribute.atttypmod  = -1 then null::int2 -- MG2 : variable length, so unknown
    end::int2                             as column_scale,
    case
      when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
      when pg_type.typname in ( '_bpchar', '_char' ) and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')' || '[]'
      when pg_type.typname = '_numeric'              and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')' || '[]'
      when pg_type.typname = '_numeric'              and pg_attribute.atttypmod  = -1 then 'numeric(variable)[]'
      when pg_type.typname = '_varchar'              and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')' || '[]'
      when pg_type.typname = '_varchar'              and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')' || '[]'

      when pg_type.typname in ( 'bpchar', 'char' ) and pg_attribute.atttypmod != -1 then 'char'    || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      when pg_type.typname in ( 'bpchar', 'char' ) and pg_attribute.atttypmod  = -1 then 'char'    || '(' || '1'                                                                                   || ')'
      when pg_type.typname = 'numeric'             and pg_attribute.atttypmod != -1 then 'numeric' || '(' || ( (pg_attribute.atttypmod-4) >> 16 ) || ',' || ( (pg_attribute.atttypmod-4) & 65535 ) || ')'
      when pg_type.typname = 'numeric'             and pg_attribute.atttypmod  = -1 then 'numeric(variable)'
      when pg_type.typname = 'varchar'             and pg_attribute.atttypmod != -1 then 'varchar' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      when pg_type.typname = 'varchar'             and pg_attribute.atttypmod  = -1 then 'varchar' || '(' || '256'                                                                                 || ')'

      when pg_type.typname = 'varbyte'             and pg_attribute.atttypmod != -1 then 'varbyte' || '(' || (pg_attribute.atttypmod-4)                                                            || ')'
      -- MG2 : not seen pg_attribute.atttypmod = -1 for 'varbyte'

      when left(pg_type.typname, 1) = '_' then substring(trim(pg_type.typname) from 2) || '[]'
                                          else trim(pg_type.typname)::varchar
    end::varchar                          as column_full_data_type_name
  from
              pg_class
         join pg_namespace on pg_namespace.oid      = pg_class.relnamespace
         join pg_attribute on pg_attribute.attrelid = pg_class.oid
         join pg_type      on pg_type.oid           = pg_attribute.atttypid
    left join stv_mv_info  on     stv_mv_info.db_name = current_database()
                              and stv_mv_info.schema  = pg_namespace.nspname
                              and stv_mv_info.name    = pg_class.relname
  where
        pg_class.relkind   = 'v'
    and pg_class.relnatts >  0     -- MG2 : no late-binding views
    and stv_mv_info.name  is null; -- MG2 : no materialized views


create or replace view
  lf_241129.ffunctions
as
  select
    schema_id::int4,
    schema_name::varchar,
    language_id::int4,
    language_name::varchar,
    base_241129.fixed_pg_proc_for_functions.user_id::int4,
    user_name::varchar,
    case
      when provolatile = 'i' then 'immutable'
      when provolatile = 's' then 'stable'
      when provolatile = 'v' then 'volatile'
    end::varchar                          as function_volatility,
    prorettype::int4                      as function_return_data_type_id,
    case
      when proretset = True  then '_' || pg_type_return.typname
      when proretset = False then        pg_type_return.typname
    end::varchar                         as function_return_data_type_name,
    function_id::int4,
    function_name::varchar,
    (
                                                               pg_type_arg_00.oid::varchar     ||
      case when pg_type_arg_01.oid is null then '' else ',' || pg_type_arg_01.oid::varchar end ||
      case when pg_type_arg_02.oid is null then '' else ',' || pg_type_arg_02.oid::varchar end ||
      case when pg_type_arg_03.oid is null then '' else ',' || pg_type_arg_03.oid::varchar end ||
      case when pg_type_arg_04.oid is null then '' else ',' || pg_type_arg_04.oid::varchar end ||
      case when pg_type_arg_05.oid is null then '' else ',' || pg_type_arg_05.oid::varchar end ||
      case when pg_type_arg_06.oid is null then '' else ',' || pg_type_arg_06.oid::varchar end ||
      case when pg_type_arg_07.oid is null then '' else ',' || pg_type_arg_07.oid::varchar end ||
      case when pg_type_arg_08.oid is null then '' else ',' || pg_type_arg_08.oid::varchar end ||
      case when pg_type_arg_09.oid is null then '' else ',' || pg_type_arg_09.oid::varchar end ||
      case when pg_type_arg_10.oid is null then '' else ',' || pg_type_arg_10.oid::varchar end ||
      case when pg_type_arg_11.oid is null then '' else ',' || pg_type_arg_11.oid::varchar end ||
      case when pg_type_arg_12.oid is null then '' else ',' || pg_type_arg_12.oid::varchar end ||
      case when pg_type_arg_13.oid is null then '' else ',' || pg_type_arg_13.oid::varchar end ||
      case when pg_type_arg_14.oid is null then '' else ',' || pg_type_arg_14.oid::varchar end ||
      case when pg_type_arg_15.oid is null then '' else ',' || pg_type_arg_15.oid::varchar end ||
      case when pg_type_arg_16.oid is null then '' else ',' || pg_type_arg_16.oid::varchar end ||
      case when pg_type_arg_17.oid is null then '' else ',' || pg_type_arg_17.oid::varchar end ||
      case when pg_type_arg_18.oid is null then '' else ',' || pg_type_arg_18.oid::varchar end ||
      case when pg_type_arg_19.oid is null then '' else ',' || pg_type_arg_19.oid::varchar end ||
      case when pg_type_arg_20.oid is null then '' else ',' || pg_type_arg_20.oid::varchar end ||
      case when pg_type_arg_21.oid is null then '' else ',' || pg_type_arg_21.oid::varchar end ||
      case when pg_type_arg_22.oid is null then '' else ',' || pg_type_arg_22.oid::varchar end ||
      case when pg_type_arg_23.oid is null then '' else ',' || pg_type_arg_23.oid::varchar end ||
      case when pg_type_arg_24.oid is null then '' else ',' || pg_type_arg_24.oid::varchar end ||
      case when pg_type_arg_25.oid is null then '' else ',' || pg_type_arg_25.oid::varchar end ||
      case when pg_type_arg_26.oid is null then '' else ',' || pg_type_arg_26.oid::varchar end ||
      case when pg_type_arg_27.oid is null then '' else ',' || pg_type_arg_27.oid::varchar end ||
      case when pg_type_arg_28.oid is null then '' else ',' || pg_type_arg_28.oid::varchar end ||
      case when pg_type_arg_29.oid is null then '' else ',' || pg_type_arg_29.oid::varchar end ||
      case when pg_type_arg_30.oid is null then '' else ',' || pg_type_arg_30.oid::varchar end ||
      case when pg_type_arg_31.oid is null then '' else ',' || pg_type_arg_31.oid::varchar end
    )::varchar                            as function_arguments_base_data_type_ids,
    (
                                                                   pg_type_arg_00.typname::varchar     ||
      case when pg_type_arg_01.typname is null then '' else ',' || pg_type_arg_01.typname::varchar end ||
      case when pg_type_arg_02.typname is null then '' else ',' || pg_type_arg_02.typname::varchar end ||
      case when pg_type_arg_03.typname is null then '' else ',' || pg_type_arg_03.typname::varchar end ||
      case when pg_type_arg_04.typname is null then '' else ',' || pg_type_arg_04.typname::varchar end ||
      case when pg_type_arg_05.typname is null then '' else ',' || pg_type_arg_05.typname::varchar end ||
      case when pg_type_arg_06.typname is null then '' else ',' || pg_type_arg_06.typname::varchar end ||
      case when pg_type_arg_07.typname is null then '' else ',' || pg_type_arg_07.typname::varchar end ||
      case when pg_type_arg_08.typname is null then '' else ',' || pg_type_arg_08.typname::varchar end ||
      case when pg_type_arg_09.typname is null then '' else ',' || pg_type_arg_09.typname::varchar end ||
      case when pg_type_arg_10.typname is null then '' else ',' || pg_type_arg_10.typname::varchar end ||
      case when pg_type_arg_11.typname is null then '' else ',' || pg_type_arg_11.typname::varchar end ||
      case when pg_type_arg_12.typname is null then '' else ',' || pg_type_arg_12.typname::varchar end ||
      case when pg_type_arg_13.typname is null then '' else ',' || pg_type_arg_13.typname::varchar end ||
      case when pg_type_arg_14.typname is null then '' else ',' || pg_type_arg_14.typname::varchar end ||
      case when pg_type_arg_15.typname is null then '' else ',' || pg_type_arg_15.typname::varchar end ||
      case when pg_type_arg_16.typname is null then '' else ',' || pg_type_arg_16.typname::varchar end ||
      case when pg_type_arg_17.typname is null then '' else ',' || pg_type_arg_17.typname::varchar end ||
      case when pg_type_arg_18.typname is null then '' else ',' || pg_type_arg_18.typname::varchar end ||
      case when pg_type_arg_19.typname is null then '' else ',' || pg_type_arg_19.typname::varchar end ||
      case when pg_type_arg_20.typname is null then '' else ',' || pg_type_arg_20.typname::varchar end ||
      case when pg_type_arg_21.typname is null then '' else ',' || pg_type_arg_21.typname::varchar end ||
      case when pg_type_arg_22.typname is null then '' else ',' || pg_type_arg_22.typname::varchar end ||
      case when pg_type_arg_23.typname is null then '' else ',' || pg_type_arg_23.typname::varchar end ||
      case when pg_type_arg_24.typname is null then '' else ',' || pg_type_arg_24.typname::varchar end ||
      case when pg_type_arg_25.typname is null then '' else ',' || pg_type_arg_25.typname::varchar end ||
      case when pg_type_arg_26.typname is null then '' else ',' || pg_type_arg_26.typname::varchar end ||
      case when pg_type_arg_27.typname is null then '' else ',' || pg_type_arg_27.typname::varchar end ||
      case when pg_type_arg_28.typname is null then '' else ',' || pg_type_arg_28.typname::varchar end ||
      case when pg_type_arg_29.typname is null then '' else ',' || pg_type_arg_29.typname::varchar end ||
      case when pg_type_arg_30.typname is null then '' else ',' || pg_type_arg_30.typname::varchar end ||
      case when pg_type_arg_31.typname is null then '' else ',' || pg_type_arg_31.typname::varchar end
    )::varchar                            as function_arguments_base_data_type_names
  from
              base_241129.fixed_pg_proc_for_functions
         join base_241129.fixed_pg_user      using ( user_id ) -- MG2 : if this is after the join to namespace, RS throws an poor errow, "user_id" twice - it really means ambiguous
         join base_241129.fixed_pg_namespace using ( schema_id )
         join pg_type as pg_type_return on base_241129.fixed_pg_proc_for_functions.prorettype       = pg_type_return.oid
    -- MG2 : base_241129.fixed_pg_proc_for_functions.proargtypes is an oidvector, which is an array of oids (32 in this case - max function args), when an element is empty, the value is 0
    left join pg_type as pg_type_arg_00 on base_241129.fixed_pg_proc_for_functions.proargtypes[00] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[00] = pg_type_arg_00.oid
    left join pg_type as pg_type_arg_01 on base_241129.fixed_pg_proc_for_functions.proargtypes[01] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[01] = pg_type_arg_01.oid
    left join pg_type as pg_type_arg_02 on base_241129.fixed_pg_proc_for_functions.proargtypes[02] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[02] = pg_type_arg_02.oid
    left join pg_type as pg_type_arg_03 on base_241129.fixed_pg_proc_for_functions.proargtypes[03] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[03] = pg_type_arg_03.oid
    left join pg_type as pg_type_arg_04 on base_241129.fixed_pg_proc_for_functions.proargtypes[04] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[04] = pg_type_arg_04.oid
    left join pg_type as pg_type_arg_05 on base_241129.fixed_pg_proc_for_functions.proargtypes[05] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[05] = pg_type_arg_05.oid
    left join pg_type as pg_type_arg_06 on base_241129.fixed_pg_proc_for_functions.proargtypes[06] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[06] = pg_type_arg_06.oid
    left join pg_type as pg_type_arg_07 on base_241129.fixed_pg_proc_for_functions.proargtypes[07] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[07] = pg_type_arg_07.oid
    left join pg_type as pg_type_arg_08 on base_241129.fixed_pg_proc_for_functions.proargtypes[08] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[08] = pg_type_arg_08.oid
    left join pg_type as pg_type_arg_09 on base_241129.fixed_pg_proc_for_functions.proargtypes[09] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[09] = pg_type_arg_09.oid
    left join pg_type as pg_type_arg_10 on base_241129.fixed_pg_proc_for_functions.proargtypes[10] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[10] = pg_type_arg_10.oid
    left join pg_type as pg_type_arg_11 on base_241129.fixed_pg_proc_for_functions.proargtypes[11] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[11] = pg_type_arg_11.oid
    left join pg_type as pg_type_arg_12 on base_241129.fixed_pg_proc_for_functions.proargtypes[12] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[12] = pg_type_arg_12.oid
    left join pg_type as pg_type_arg_13 on base_241129.fixed_pg_proc_for_functions.proargtypes[13] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[13] = pg_type_arg_13.oid
    left join pg_type as pg_type_arg_14 on base_241129.fixed_pg_proc_for_functions.proargtypes[14] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[14] = pg_type_arg_14.oid
    left join pg_type as pg_type_arg_15 on base_241129.fixed_pg_proc_for_functions.proargtypes[15] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[15] = pg_type_arg_15.oid
    left join pg_type as pg_type_arg_16 on base_241129.fixed_pg_proc_for_functions.proargtypes[16] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[16] = pg_type_arg_16.oid
    left join pg_type as pg_type_arg_17 on base_241129.fixed_pg_proc_for_functions.proargtypes[17] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[17] = pg_type_arg_17.oid
    left join pg_type as pg_type_arg_18 on base_241129.fixed_pg_proc_for_functions.proargtypes[18] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[18] = pg_type_arg_18.oid
    left join pg_type as pg_type_arg_19 on base_241129.fixed_pg_proc_for_functions.proargtypes[19] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[19] = pg_type_arg_19.oid
    left join pg_type as pg_type_arg_20 on base_241129.fixed_pg_proc_for_functions.proargtypes[20] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[20] = pg_type_arg_20.oid
    left join pg_type as pg_type_arg_21 on base_241129.fixed_pg_proc_for_functions.proargtypes[21] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[21] = pg_type_arg_21.oid
    left join pg_type as pg_type_arg_22 on base_241129.fixed_pg_proc_for_functions.proargtypes[22] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[22] = pg_type_arg_22.oid
    left join pg_type as pg_type_arg_23 on base_241129.fixed_pg_proc_for_functions.proargtypes[23] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[23] = pg_type_arg_23.oid
    left join pg_type as pg_type_arg_24 on base_241129.fixed_pg_proc_for_functions.proargtypes[24] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[24] = pg_type_arg_24.oid
    left join pg_type as pg_type_arg_25 on base_241129.fixed_pg_proc_for_functions.proargtypes[25] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[25] = pg_type_arg_25.oid
    left join pg_type as pg_type_arg_26 on base_241129.fixed_pg_proc_for_functions.proargtypes[26] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[26] = pg_type_arg_26.oid
    left join pg_type as pg_type_arg_27 on base_241129.fixed_pg_proc_for_functions.proargtypes[27] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[27] = pg_type_arg_27.oid
    left join pg_type as pg_type_arg_28 on base_241129.fixed_pg_proc_for_functions.proargtypes[28] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[28] = pg_type_arg_28.oid
    left join pg_type as pg_type_arg_29 on base_241129.fixed_pg_proc_for_functions.proargtypes[29] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[29] = pg_type_arg_29.oid
    left join pg_type as pg_type_arg_30 on base_241129.fixed_pg_proc_for_functions.proargtypes[30] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[30] = pg_type_arg_30.oid
    left join pg_type as pg_type_arg_31 on base_241129.fixed_pg_proc_for_functions.proargtypes[31] != 0 and base_241129.fixed_pg_proc_for_functions.proargtypes[31] = pg_type_arg_31.oid;


create or replace procedure
  ro.sp_revoke_versionless_sf_schema_from_group( group_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'revoke usage on schema sf from group ' || group_name;
      execute sql;

      sql := 'revoke select on all tables in schema sf from group ' || group_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.objects
as
  with
    cte_objects_tables_and_views as
    (
      select
        pg_class.relnamespace          as schema_id,
        pg_class.oid                   as object_id,
        case
          when pg_class.relkind = 'r'                                                             then 'table'
          when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then 'view'
          when pg_class.relkind = 'v' and pg_class.relnatts  = 0 and stv_mv_info.name is     null then 'late-binding view'
          when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then 'materialized view'
        end                            as object_type,
        trim( pg_class.relname )       as object_name,
        pg_class.relowner              as object_user_id
      from
                  pg_class
             join pg_namespace on pg_class.relnamespace = pg_namespace.oid
        left join stv_mv_info  on stv_mv_info.db_name = current_database()   and
                                  stv_mv_info.schema  = pg_namespace.nspname and
                                  stv_mv_info.name    = pg_class.relname
      where
        pg_class.relkind in ( 'r', 'v' )
    ),

    cte_objects_funcs_and_procs as
    (
      select
        pg_proc.pronamespace           as schema_id,
        pg_proc.oid                    as object_id,
        case
          when pg_language.lanname = 'c'         then 'C func'
          when pg_language.lanname = 'exfunc'    then 'lambda func'
          when pg_language.lanname = 'internal'  then 'internal func'
          when pg_language.lanname = 'mlfunc'    then 'ML func'
          when pg_language.lanname = 'plpgsql'   then 'procedure'
          when pg_language.lanname = 'plpythonu' then 'python func'
          when pg_language.lanname = 'sql'       then 'SQL func'
                                                 else trim( pg_language.lanname )
        end                            as object_type,
        trim( pg_proc.proname )        as object_name,
        pg_proc.proowner               as object_user_id
      from
             pg_proc
        join pg_language on pg_proc.prolang = pg_language.oid
    ),

    cte_objects_constraints as
    (
      select
        connamespace           as schema_id,
        pg_constraint.oid      as object_id,
        case
          when contype = 'c' then 'check'
          when contype = 'f' then 'foreign key'
          when contype = 'p' then 'primary key'
          when contype = 'u' then 'unique'
                             else 'unknown'
                              -- MG2 : I want to do this, but looks like RS has no way to do it
                              -- else contype::varchar
        end                    as object_type,
        trim( conname )        as object_name,
        pg_class.relowner      as object_user_id
      from
             pg_constraint
        join pg_class on pg_constraint.conrelid = pg_class.oid
    ),

    cte_union_objects as
    (
      select * from cte_objects_tables_and_views
      union all
      select * from cte_objects_funcs_and_procs
      union all
      select * from cte_objects_constraints
    ),

    cte_objects as
    (
      select
        schema_id,
        trim( pg_namespace.nspname ) as schema_name,
        object_id,
        object_name,
        object_type,
        object_user_id,
        trim( pg_user.usename ) as object_user_name
      from
             cte_union_objects
        join pg_namespace on cte_union_objects.schema_id      = pg_namespace.oid
        join pg_user      on cte_union_objects.object_user_id = pg_user.usesysid
    )

  select
    schema_id::int4,
    schema_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_type::varchar,
    object_user_id::int4,
    object_user_name::varchar
  from
    cte_objects;


create or replace view
  lf_241129.wqueues_totals_slot_utilization_by_all
as
  with
    cte_events as
    (
      select
        queue_id,
        query_compile_exec_start_ts as event_ts,
        1                   as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'

      union all

      select
        queue_id,
        query_compile_exec_end_ts as event_ts,
        -1                as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_slot_durations as
    (
      select
        queue_id,
        sum( event_delta ) over ( partition by queue_id order by event_ts rows between unbounded preceding and current row ) as slot_number,
        lead( event_ts   ) over ( partition by queue_id order by event_ts ) - event_ts                                       as duration
      from
        cte_events
    ),

    cte_durations as
    (
      select
        queue_id,
        slot_number,
        sum( duration ) as sum_duration
      from
        cte_slot_durations
      group by
        queue_id,
        slot_number
    )

  select
    queue_id::int2,
    queue_name::varchar,
    slot_number::int2,
    sum_duration::interval as duration
  from
              cte_durations
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  lf_241129.ttables_creation_for_all_by_second
as
  select
    date_trunc( 'second', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    start_ts;


create or replace view
  sf_241129.ttables_creation_for_all_by_second
as
  select
    start_ts::timestamp,
    number_tables::int4 as ttables
  from
    lf_241129.ttables_creation_for_all_by_second
  order by
    start_ts;


create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_query
as
  with
    cte_blocks as
    (
      select
        query_id,
        sum( query_number_blocks_read    ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        query_id
    ),

    cte_nulls_to_values as
    (
      select
        query_id,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    query_id::int4,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_all_by_day
as
  with
    cte_transactions as
    (
      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        transaction_id,
        date_trunc( 'day', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        transaction_id
    )

  select
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
  group by
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_all_by_day
as
  with
    cte_aggr as
    (
      select
        date_trunc( 'day', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        date_trunc( 'day', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        date_trunc( 'day', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        date_trunc( 'day', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        date_trunc( 'day', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        date_trunc( 'day', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        date_trunc( 'day', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        date_trunc( 'day', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        date_trunc( 'day', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_days.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               cte_node_types
    cross join base_241129.generate_days
     left join cte_aggr         using ( node_type, start_ts )
     left join cte_bcast        using ( node_type, start_ts )
     left join cte_delete       using ( node_type, start_ts )
     left join cte_dist         using ( node_type, start_ts )
     left join cte_hash         using ( node_type, start_ts )
     left join cte_hashjoin     using ( node_type, start_ts )
     left join cte_insert       using ( node_type, start_ts )
     left join cte_limit        using ( node_type, start_ts )
     left join cte_merge        using ( node_type, start_ts )
     left join cte_mergejoin    using ( node_type, start_ts )
     left join cte_nestloop     using ( node_type, start_ts )
     left join cte_parse        using ( node_type, start_ts )
     left join cte_project      using ( node_type, start_ts )
     left join cte_return       using ( node_type, start_ts )
     left join cte_save         using ( node_type, start_ts )
     left join cte_sort         using ( node_type, start_ts )
     left join cte_spatial_join using ( node_type, start_ts )
     left join cte_unique       using ( node_type, start_ts )
     left join cte_window       using ( node_type, start_ts );


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_node_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_node_by_hour
    )

  select
    node_ordinal::int2  as n,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_query
as
  select
    query_id::int4                                                        as qid,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_query
  order by
    query_id;


create or replace view
  lf_241129.ttables_stats_dists_sorts_for_schemas_by_count
as
  select
    schema_name::varchar,

    sum( case when table_distribution_type = 'all' and table_sort_type = 'compound'    then 1 else 0 end )::int4  as all_compound,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'interleaved' then 1 else 0 end )::int4  as all_interleaved,
    sum( case when table_distribution_type = 'all' and table_sort_type = 'unsorted'    then 1 else 0 end )::int4  as all_unsorted,

    sum( case when table_distribution_type = 'even' and table_sort_type = 'compound'    then 1 else 0 end )::int4 as even_compound,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'interleaved' then 1 else 0 end )::int4 as even_interleaved,
    sum( case when table_distribution_type = 'even' and table_sort_type = 'unsorted'    then 1 else 0 end )::int4 as even_unsorted,

    sum( case when table_distribution_type = 'key' and table_sort_type = 'compound'    then 1 else 0 end )::int4  as key_compound,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'interleaved' then 1 else 0 end )::int4  as key_interleaved,
    sum( case when table_distribution_type = 'key' and table_sort_type = 'unsorted'    then 1 else 0 end )::int4  as key_unsorted
  from
    base_241129.ttables_basic_info_full
  where
        user_id                 != 1
    and schema_name         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(schema_name,8)     != 'pg_temp_'
  group by
    schema_name;


create or replace view
  sf_241129.ffunctions
as
  select
    schema_name::varchar                                   as sschema,
    case
      when language_name = 'c'         then 'C'
      when language_name = 'exfunc'    then 'lambda'
      when language_name = 'internal'  then 'int'
      when language_name = 'mlfunc'    then 'ml'
      when language_name = 'plpythonu' then 'python'
      when language_name = 'sql'       then 'sql'
                                       else language_name
    end::varchar                                           as ttype,
    user_name::varchar                                     as uuser,
    upper( substring(function_volatility, 1, 1) )::char(1) as vol,
    function_return_data_type_name::varchar                as rreturn,
    function_name::varchar                                 as nname,
    replace
    (
      replace
      (
        replace
        (
          replace
          (
            replace
            (
              replace
              (
                replace
                (
                  replace
                  (
                    replace
                    (
                      replace
                      (
                        function_arguments_base_data_type_names,
                        'internal', 'itl'
                      ),
                      'timestamptz', 'tstz'
                    ),
                    'timestamp', 'ts'
                  ),
                  'cstring', 'cstr'
                ),
                'text', 'txt'
              ),
              'hllsketch', 'hll'
            ),
            'float8', 'flt8'
          ),
          'float4', 'flt4'
        ),
        'geometry', 'geom'
      ),
      'geography', 'geog'
    )::varchar                                             as args
  from
    lf_241129.ffunctions
  where
    user_name != 'rdsdb'
  order by
    schema_name,
    ttype,
    -- MG2 : IDs can be very different to names, so names first, for humans
    function_name,
    function_id;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_node_by_day
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'day', scan_start_ts )::timestamp as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal )
  group by
    node_ordinal,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_users_by_all
as
  with
    cte_queries as
    (
      select
        user_id,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        user_id,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                   as user_name,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
         cte_queries
    join pg_user on cte_queries.user_id = pg_user.usesysid
  group by
    user_id,
    user_name;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_users_by_all
as
  with
    cte_hr_formatting as
    (
      select
        user_name,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_users_by_all
    )

  select
    user_name::varchar as uuser,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_users_by_all
as
  select
    user_name::varchar                                      as uuser,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_users_by_all
  order by
    user_name;


create or replace view
  lf_241129.queries_compilation_stream
as
  with
    cte_queries_compilation_stream as
    (
      select
        svl_compile.pid                   as session_id,
        svl_compile.xid                   as transaction_id,
        svl_compile.query                 as query_id,
        stl_stream_segs.stream            as stream_ordinal,
        sum( 1-compile )                  as stream_number_leader_node_cache_segments,
        sum(   compile )                  as stream_number_offload_segments,
        count(*)                          as stream_number_segments,
        ( max(endtime) - min(starttime) ) as stream_compile_duration
      from
                     svl_compile
        natural join stl_stream_segs
      group by
        svl_compile.pid,
        svl_compile.xid,
        svl_compile.query,
        stl_stream_segs.stream
    )

  select
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    stream_ordinal::int2,
    stream_number_leader_node_cache_segments::int2,
    stream_number_offload_segments::int2,
    stream_number_segments::int2,
    stream_compile_duration::interval
  from
    cte_queries_compilation_stream;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_users_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_leadin_duration)::numeric(18,2)::varchar as hr_min_leadin_duration,
        extract(epoch from max_leadin_duration)::numeric(18,2)::varchar as hr_max_leadin_duration,
        med_leadin_duration::numeric(18,2)::varchar                     as hr_med_leadin_duration,
        extract(epoch from avg_leadin_duration)::numeric(18,2)::varchar as hr_avg_leadin_duration,
        std_leadin_duration::numeric(18,2)::varchar                     as hr_std_leadin_duration,
        extract(epoch from sum_leadin_duration)::numeric(18,2)::varchar as hr_sum_leadin_duration,

        extract(epoch from min_wlm_leadin_duration)::numeric(18,2)::varchar as hr_min_wlm_leadin_duration,
        extract(epoch from max_wlm_leadin_duration)::numeric(18,2)::varchar as hr_max_wlm_leadin_duration,
        med_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadin_duration,
        extract(epoch from avg_wlm_leadin_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadin_duration,
        std_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadin_duration,
        extract(epoch from sum_wlm_leadin_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadin_duration,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        extract(epoch from min_wlm_leadout_duration)::numeric(18,2)::varchar as hr_min_wlm_leadout_duration,
        extract(epoch from max_wlm_leadout_duration)::numeric(18,2)::varchar as hr_max_wlm_leadout_duration,
        med_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadout_duration,
        extract(epoch from avg_wlm_leadout_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadout_duration,
        std_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadout_duration,
        extract(epoch from sum_wlm_leadout_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadout_duration,

        extract(epoch from min_leadout_duration)::numeric(18,2)::varchar as hr_min_leadout_duration,
        extract(epoch from max_leadout_duration)::numeric(18,2)::varchar as hr_max_leadout_duration,
        med_leadout_duration::numeric(18,2)::varchar                     as hr_med_leadout_duration,
        extract(epoch from avg_leadout_duration)::numeric(18,2)::varchar as hr_avg_leadout_duration,
        std_leadout_duration::numeric(18,2)::varchar                     as hr_std_leadout_duration,
        extract(epoch from sum_leadout_duration)::numeric(18,2)::varchar as hr_sum_leadout_duration,

        extract(epoch from min_query_total_duration)::numeric(18,2)::varchar as hr_min_query_total_duration,
        extract(epoch from max_query_total_duration)::numeric(18,2)::varchar as hr_max_query_total_duration,
        med_query_total_duration::numeric(18,2)::varchar                     as hr_med_query_total_duration,
        extract(epoch from avg_query_total_duration)::numeric(18,2)::varchar as hr_avg_query_total_duration,
        std_query_total_duration::numeric(18,2)::varchar                     as hr_std_query_total_duration,
        extract(epoch from sum_query_total_duration)::numeric(18,2)::varchar as hr_sum_query_total_duration,

        extract(epoch from min_unaccounted_duration)::numeric(18,2)::varchar as hr_min_unaccounted_duration,
        extract(epoch from max_unaccounted_duration)::numeric(18,2)::varchar as hr_max_unaccounted_duration,
        med_unaccounted_duration::numeric(18,2)::varchar                     as hr_med_unaccounted_duration,
        extract(epoch from avg_unaccounted_duration)::numeric(18,2)::varchar as hr_avg_unaccounted_duration,
        std_unaccounted_duration::numeric(18,2)::varchar                     as hr_std_unaccounted_duration,
        extract(epoch from sum_unaccounted_duration)::numeric(18,2)::varchar as hr_sum_unaccounted_duration,

        max( len(hr_min_leadin_duration) ) over () as max_len_hr_min_leadin_duration,
        max( len(hr_max_leadin_duration) ) over () as max_len_hr_max_leadin_duration,
        max( len(hr_med_leadin_duration) ) over () as max_len_hr_med_leadin_duration,
        max( len(hr_avg_leadin_duration) ) over () as max_len_hr_avg_leadin_duration,
        max( len(hr_std_leadin_duration) ) over () as max_len_hr_std_leadin_duration,
        max( len(hr_sum_leadin_duration) ) over () as max_len_hr_sum_leadin_duration,

        max( len(hr_min_wlm_leadin_duration) ) over () as max_len_hr_min_wlm_leadin_duration,
        max( len(hr_max_wlm_leadin_duration) ) over () as max_len_hr_max_wlm_leadin_duration,
        max( len(hr_med_wlm_leadin_duration) ) over () as max_len_hr_med_wlm_leadin_duration,
        max( len(hr_avg_wlm_leadin_duration) ) over () as max_len_hr_avg_wlm_leadin_duration,
        max( len(hr_std_wlm_leadin_duration) ) over () as max_len_hr_std_wlm_leadin_duration,
        max( len(hr_sum_wlm_leadin_duration) ) over () as max_len_hr_sum_wlm_leadin_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        max( len(hr_min_wlm_leadout_duration) ) over () as max_len_hr_min_wlm_leadout_duration,
        max( len(hr_max_wlm_leadout_duration) ) over () as max_len_hr_max_wlm_leadout_duration,
        max( len(hr_med_wlm_leadout_duration) ) over () as max_len_hr_med_wlm_leadout_duration,
        max( len(hr_avg_wlm_leadout_duration) ) over () as max_len_hr_avg_wlm_leadout_duration,
        max( len(hr_std_wlm_leadout_duration) ) over () as max_len_hr_std_wlm_leadout_duration,
        max( len(hr_sum_wlm_leadout_duration) ) over () as max_len_hr_sum_wlm_leadout_duration,

        max( len(hr_min_leadout_duration) ) over () as max_len_hr_min_leadout_duration,
        max( len(hr_max_leadout_duration) ) over () as max_len_hr_max_leadout_duration,
        max( len(hr_med_leadout_duration) ) over () as max_len_hr_med_leadout_duration,
        max( len(hr_avg_leadout_duration) ) over () as max_len_hr_avg_leadout_duration,
        max( len(hr_std_leadout_duration) ) over () as max_len_hr_std_leadout_duration,
        max( len(hr_sum_leadout_duration) ) over () as max_len_hr_sum_leadout_duration,

        max( len(hr_min_query_total_duration) ) over () as max_len_hr_min_query_total_duration,
        max( len(hr_max_query_total_duration) ) over () as max_len_hr_max_query_total_duration,
        max( len(hr_med_query_total_duration) ) over () as max_len_hr_med_query_total_duration,
        max( len(hr_avg_query_total_duration) ) over () as max_len_hr_avg_query_total_duration,
        max( len(hr_std_query_total_duration) ) over () as max_len_hr_std_query_total_duration,
        max( len(hr_sum_query_total_duration) ) over () as max_len_hr_sum_query_total_duration,

        max( len(hr_min_unaccounted_duration) ) over () as max_len_hr_min_unaccounted_duration,
        max( len(hr_max_unaccounted_duration) ) over () as max_len_hr_max_unaccounted_duration,
        max( len(hr_med_unaccounted_duration) ) over () as max_len_hr_med_unaccounted_duration,
        max( len(hr_avg_unaccounted_duration) ) over () as max_len_hr_avg_unaccounted_duration,
        max( len(hr_std_unaccounted_duration) ) over () as max_len_hr_std_unaccounted_duration,
        max( len(hr_sum_unaccounted_duration) ) over () as max_len_hr_sum_unaccounted_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_users_by_hour
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_min_leadin_duration, max_len_hr_min_leadin_duration ) || ' / ' ||
      lpad( hr_max_leadin_duration, max_len_hr_max_leadin_duration ) || ' / ' ||
      lpad( hr_med_leadin_duration, max_len_hr_med_leadin_duration ) || ' / ' ||
      lpad( hr_avg_leadin_duration, max_len_hr_avg_leadin_duration ) || ' / ' ||
      lpad( hr_std_leadin_duration, max_len_hr_std_leadin_duration ) || ' / ' ||
      lpad( hr_sum_leadin_duration, max_len_hr_sum_leadin_duration ) )::varchar as leadin,

    ( lpad( hr_min_wlm_leadin_duration, max_len_hr_min_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadin_duration, max_len_hr_max_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadin_duration, max_len_hr_med_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadin_duration, max_len_hr_avg_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadin_duration, max_len_hr_std_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_sum_wlm_leadin_duration, max_len_hr_sum_wlm_leadin_duration ) )::varchar as wlm_leadin,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    ( lpad( hr_min_wlm_leadout_duration, max_len_hr_min_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadout_duration, max_len_hr_max_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadout_duration, max_len_hr_med_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadout_duration, max_len_hr_avg_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadout_duration, max_len_hr_std_wlm_leadout_duration ) || ' / ' || 
      lpad( hr_sum_wlm_leadout_duration, max_len_hr_sum_wlm_leadout_duration ) )::varchar as wlm_leadout,

    ( lpad( hr_min_leadout_duration, max_len_hr_min_leadout_duration ) || ' / ' ||
      lpad( hr_max_leadout_duration, max_len_hr_max_leadout_duration ) || ' / ' ||
      lpad( hr_med_leadout_duration, max_len_hr_med_leadout_duration ) || ' / ' ||
      lpad( hr_avg_leadout_duration, max_len_hr_avg_leadout_duration ) || ' / ' ||
      lpad( hr_std_leadout_duration, max_len_hr_std_leadout_duration ) || ' / ' || 
      lpad( hr_sum_leadout_duration, max_len_hr_sum_leadout_duration ) )::varchar as leadout,

    ( lpad( hr_min_query_total_duration, max_len_hr_min_query_total_duration ) || ' / ' ||
      lpad( hr_max_query_total_duration, max_len_hr_max_query_total_duration ) || ' / ' ||
      lpad( hr_med_query_total_duration, max_len_hr_med_query_total_duration ) || ' / ' ||
      lpad( hr_avg_query_total_duration, max_len_hr_avg_query_total_duration ) || ' / ' ||
      lpad( hr_std_query_total_duration, max_len_hr_std_query_total_duration ) || ' / ' || 
      lpad( hr_sum_query_total_duration, max_len_hr_sum_query_total_duration ) )::varchar as total,

    ( lpad( hr_min_unaccounted_duration, max_len_hr_min_unaccounted_duration ) || ' / ' ||
      lpad( hr_max_unaccounted_duration, max_len_hr_max_unaccounted_duration ) || ' / ' ||
      lpad( hr_med_unaccounted_duration, max_len_hr_med_unaccounted_duration ) || ' / ' ||
      lpad( hr_avg_unaccounted_duration, max_len_hr_avg_unaccounted_duration ) || ' / ' ||
      lpad( hr_std_unaccounted_duration, max_len_hr_std_unaccounted_duration ) || ' / ' || 
      lpad( hr_sum_unaccounted_duration, max_len_hr_sum_unaccounted_duration ) )::varchar as unact,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.wqueues
as
  with
    cte_queries as
    (
      select
        queue_id,
        query_state,
        ( ( queue_time::float8 / 1000000 ) * interval '1 second' )::interval as queue_duration,
        ( ( exec_time::float8  / 1000000 ) * interval '1 second' )::interval as exec_duration
      from
        base_241129.fixed_stv_wlm_query_state
      where
        query_state in ( 'QueuedWaiting', 'Running' )
    ),

    cte_queries_stats as
    (
      select
        queue_id,
        query_state,

        min( queue_duration ) over ( partition by queue_id, 'QueuedWaiting' )                              as min_queue_duration,
        max( queue_duration ) over ( partition by queue_id, 'QueuedWaiting' )                              as max_queue_duration,
        median( extract(epoch from (queue_duration)) ) over ( partition by queue_id, 'QueuedWaiting' )     as med_queue_duration,
        avg( queue_duration ) over ( partition by queue_id, 'QueuedWaiting' )                              as avg_queue_duration,
        stddev_pop( extract(epoch from (queue_duration)) ) over ( partition by queue_id, 'QueuedWaiting' ) as std_queue_duration,

        min( exec_duration ) over ( partition by queue_id, 'Running' )                            as min_exec_duration,
        max( exec_duration ) over ( partition by queue_id, 'Running' )                            as max_exec_duration,
        median( extract(epoch from exec_duration) ) over ( partition by queue_id, 'Running' )     as med_exec_duration,
        avg( exec_duration ) over ( partition by queue_id, 'Running' )                            as avg_exec_duration,
        stddev_pop( extract(epoch from exec_duration) ) over ( partition by queue_id, 'Running' ) as std_exec_duration,

        count(*) over ( partition by queue_id, query_state ) as number_queries
      from
        cte_queries
    )

  select
    queue_id::int2,
    queue_name::varchar,
    base_241129.fixed_stv_wlm_service_class_config.num_query_tasks::int2                as number_slots,
    base_241129.fixed_stv_wlm_service_class_state.num_queued_queries::int2              as number_queued_queries,
    base_241129.fixed_stv_wlm_service_class_state.num_executing_queries::int2           as number_running_queries,
    base_241129.fixed_stv_wlm_service_class_state.num_executed_queries::int4            as number_completed_queries,
    cte_queries_stats.min_queue_duration::interval,
    cte_queries_stats.max_queue_duration::interval,
    cte_queries_stats.med_queue_duration::float4,
    cte_queries_stats.avg_queue_duration::interval,
    cte_queries_stats.std_queue_duration::float4,
    cte_queries_stats.min_exec_duration::interval,
    cte_queries_stats.max_exec_duration::interval,
    cte_queries_stats.med_exec_duration::float4,
    cte_queries_stats.avg_exec_duration::interval,
    cte_queries_stats.std_exec_duration::float4,
    base_241129.fixed_stv_wlm_service_class_config.queue_csc_flag::bool,
    base_241129.fixed_stv_wlm_service_class_state.num_concurrency_scaling_queries::int4 as number_csc_completed_queries
  from
               base_241129.fixed_stv_wlm_service_class_config
     left join base_241129.fixed_stv_wlm_service_class_state using ( queue_id )
     left join cte_queries_stats                      using ( queue_id )
  where
    -- MG2 : no leader node queue
    queue_id != -1
  group by
    queue_id,
    queue_name,
    number_slots,
    number_queued_queries,
    number_running_queries,
    number_completed_queries,
    min_queue_duration,
    max_queue_duration,
    med_queue_duration,
    avg_queue_duration,
    std_queue_duration,
    min_exec_duration,
    max_exec_duration,
    med_exec_duration,
    avg_exec_duration,
    std_exec_duration,
    queue_csc_flag,
    number_csc_completed_queries;


create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_users_by_minute
as
  with
    cte_blocks as
    (
      select
        user_id,
        user_name,
        date_trunc( 'minute', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        user_name,
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        user_id,
        user_name,
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_users_by_minute
as
  select
    user_name::varchar                                                    as uuser,
    start_ts::timestamp,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_users_by_minute
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_all_complete
as
  select
    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_all
  order by
    node_ordinal,
    slice_ordinal;


create or replace view
  lf_241129.steps_totals_data_on_both_for_statements_by_day
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    statement_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               cte_base
    cross join cte_node_types
    cross join base_241129.generate_days
     left join cte_aggr         using ( start_ts, query, node_type )
     left join cte_bcast        using ( start_ts, query, node_type )
     left join cte_delete       using ( start_ts, query, node_type )
     left join cte_dist         using ( start_ts, query, node_type )
     left join cte_hash         using ( start_ts, query, node_type )
     left join cte_hashjoin     using ( start_ts, query, node_type )
     left join cte_insert       using ( start_ts, query, node_type )
     left join cte_limit        using ( start_ts, query, node_type )
     left join cte_merge        using ( start_ts, query, node_type )
     left join cte_mergejoin    using ( start_ts, query, node_type )
     left join cte_nestloop     using ( start_ts, query, node_type )
     left join cte_parse        using ( start_ts, query, node_type )
     left join cte_project      using ( start_ts, query, node_type )
     left join cte_return       using ( start_ts, query, node_type )
     left join cte_save         using ( start_ts, query, node_type )
     left join cte_sort         using ( start_ts, query, node_type )
     left join cte_spatial_join using ( start_ts, query, node_type )
     left join cte_unique       using ( start_ts, query, node_type )
     left join cte_window       using ( start_ts, query, node_type )
  group by
    statement_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_all_by_all
as
  with
    cte_queries as
    (
      select
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
    cte_queries;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_all_by_all
as
  select
    ( extract(epoch from min_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_duration)::numeric(18,2) || ' / ' ||
      med_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_duration)::numeric(18,2) || ' / ' ||
      std_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from sum_duration)::numeric(18,2) )::varchar as duration,

      number_queries::int4 as queries
    from
      lf_241129.queries_stats_durations_on_leader_for_all_by_all;


create or replace view
  sf_241129.steps_totals_data_on_both_for_node_by_second
as
  select
    node_ordinal::int2 as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_node_by_second
  order by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.privileges_granted_to_users
as
  select
    user_name::varchar as uuser,

    object_type::varchar as ttype,
    schema_name::varchar as sschema,
    object_name::varchar as oobject,

    object_return_base_data_type_name::varchar    as rt,
    object_argument_base_data_type_names::varchar as args,

    object_owner_user_name::varchar as oowner,

    (
      case
        when priv_database_grant_create = true then 'C'
        when priv_database_create       = true then 'c'
                                               else ' '
      end
      ||
      case
        when priv_database_grant_temp = true then 'T'
        when priv_database_temp       = true then 't'
                                             else ' '
      end
      ||
      case
        when priv_language_grant_usage = true then 'U'
        when priv_language_usage       = true then 'u'
                                              else ' '
      end
      ||
      '.'
      ||
      case
        when priv_schema_grant_create = true then 'C'
        when priv_schema_create       = true then 'c'
                                             else ' '
      end
      ||
      case
        when priv_schema_grant_usage = true then 'U'
        when priv_schema_usage       = true then 'u'
                                            else ' '
      end
      ||
      '.'
      ||
      case
        when priv_table_grant_delete = true then 'D'
        when priv_table_delete       = true then 'd'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_reference = true then 'F'
        when priv_table_reference       = true then 'f'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_insert = true then 'I'
        when priv_table_insert       = true then 'i'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_delete = true then 'D'
        when priv_table_delete       = true then 'd'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_drop = true then 'P'
        when priv_table_drop       = true then 'p'
                                          else ' '
      end
      ||
      case
        when priv_table_grant_select = true or priv_view_grant_select = true then 'S'
        when priv_table_select       = true or priv_view_select       = true then 's'
                                                                        else ' '
      end
      ||
      case
        when priv_table_grant_update = true then 'U'
        when priv_table_update       = true then 'u'
                                            else ' '
      end
      ||
      case
        when priv_table_grant_delete = true then 'D'
        when priv_table_delete       = true then 'd'
                                            else ' '
      end
      ||
      case
        when priv_function_grant_execute = true or priv_procedure_grant_execute = true then 'X'
        when priv_function_execute       = true or priv_procedure_execute       = true then 'x'
                                                                                  else ' '
      end
    )::varchar as privs
  from
    lf_241129.privileges_granted_to_users
  order by
    user_name,
    schema_name,
    case when object_type = 'schema' then 0 else 1 end,
    object_type,
    object_name;


create or replace view
  lf_241129.vacuum_totals_for_all_by_second
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        date_trunc( 'second', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        date_trunc( 'second', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        start_ts
    )

  select
    start_ts::timestamp,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( start_ts );


create or replace view
  sf_241129.vacuum_totals_for_all_by_second
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_all_by_second
    )

  select
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_node_by_minute
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'minute', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_minutes
     left join cte_aggr         using( slice_ordinal, start_ts )
     left join cte_bcast        using( slice_ordinal, start_ts )
     left join cte_delete       using( slice_ordinal, start_ts )
     left join cte_dist         using( slice_ordinal, start_ts )
     left join cte_hash         using( slice_ordinal, start_ts )
     left join cte_hashjoin     using( slice_ordinal, start_ts )
     left join cte_insert       using( slice_ordinal, start_ts )
     left join cte_limit        using( slice_ordinal, start_ts )
     left join cte_merge        using( slice_ordinal, start_ts )
     left join cte_mergejoin    using( slice_ordinal, start_ts )
     left join cte_nestloop     using( slice_ordinal, start_ts )
     left join cte_parse        using( slice_ordinal, start_ts )
     left join cte_project      using( slice_ordinal, start_ts )
     left join cte_return       using( slice_ordinal, start_ts )
     left join cte_save         using( slice_ordinal, start_ts )
     left join cte_sort         using( slice_ordinal, start_ts )
     left join cte_spatial_join using( slice_ordinal, start_ts )
     left join cte_unique       using( slice_ordinal, start_ts )
     left join cte_window       using( slice_ordinal, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null
  group by
    node_ordinal,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_all_by_day
as
  with
    cte_queries as
    (
      select
        date_trunc( 'day', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        ( query_compile_exec_duration - query_compile_duration ) as query_exec_duration
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id )
        left join base_241129.queries_compilation  using ( query_id )
      where
            query_abort_flag      = false
        and query_csc_flag          = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts           <= current_timestamp
        and query_start_ts         >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        start_ts,

        min( query_queue_duration ) over ( partition by start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by start_ts ) as number_queries
      from
        cte_queries
    )

  select
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_days
    left join cte_queries_stats using ( start_ts )
  group by
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_all_by_day
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_all_by_day
    )

  select
    start_ts::date,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_users_by_hour
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'hour', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        user_id,
        date_trunc( 'hour', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                   as user_name,
    base_241129.generate_hours.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_hours
    left join cte_queries using ( start_ts )
    left join pg_user on cte_queries.user_id = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_users_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_users_by_hour
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_all_by_second
as
  select
    start_ts::timestamp,
    number_abort::int4 as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_all_by_second
  order by
    start_ts;


create or replace view
  lf_241129.vacuum_stats_for_all_by_minute
as
  with
    cte_xids as 
    (
      select
        sum( num_new_blocks      ) as sum_number_new_blocks,
        sum( num_blocks_replaced ) as sum_number_replaced_blocks,

        sum( num_new_rows        ) as sum_number_new_rows,
        sum( num_deleted_rows    ) as sum_number_deleted_rows,

        count(*)                   as number_qids,

        min( query_start_ts ) as min_query_start_ts,
        max( query_end_ts   ) as max_query_end_ts,

        max_query_end_ts - min_query_start_ts as duration
      from
             base_241129.fixed_stl_vacuum_detail
        join base_241129.fixed_stl_query using( query_id, transaction_id )
      where
            -- MG2 : procably we should include aborted - need a general approcch to this
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_end_ts     <  current_timestamp
      group by
        transaction_id
      having
        -- MG2 : ignore xids in partial first start_ts
        min_query_start_ts >= current_date - interval '6 day'
    ),

    cte_xids_stats as
    (
      select
        date_trunc( 'minute', min_query_start_ts ) as start_ts,

        min( sum_number_new_blocks )        over () as min_sum_number_new_blocks,
        max( sum_number_new_blocks )        over () as max_sum_number_new_blocks,
        median( sum_number_new_blocks )     over () as med_sum_number_new_blocks,
        avg( sum_number_new_blocks )        over () as avg_sum_number_new_blocks,
        stddev_pop( sum_number_new_blocks ) over () as std_sum_number_new_blocks,
        sum( sum_number_new_blocks )        over () as sum_sum_number_new_blocks,

        min( sum_number_replaced_blocks )        over () as min_sum_number_replaced_blocks,
        max( sum_number_replaced_blocks )        over () as max_sum_number_replaced_blocks,
        median( sum_number_replaced_blocks )     over () as med_sum_number_replaced_blocks,
        avg( sum_number_replaced_blocks )        over () as avg_sum_number_replaced_blocks,
        stddev_pop( sum_number_replaced_blocks ) over () as std_sum_number_replaced_blocks,
        sum( sum_number_replaced_blocks )        over () as sum_sum_number_replaced_blocks,

        min( sum_number_new_rows )        over () as min_sum_number_new_rows,
        max( sum_number_new_rows )        over () as max_sum_number_new_rows,
        median( sum_number_new_rows )     over () as med_sum_number_new_rows,
        avg( sum_number_new_rows )        over () as avg_sum_number_new_rows,
        stddev_pop( sum_number_new_rows ) over () as std_sum_number_new_rows,
        sum( sum_number_new_rows )        over () as sum_sum_number_new_rows,

        min( sum_number_deleted_rows )        over () as min_sum_number_deleted_rows,
        max( sum_number_deleted_rows )        over () as max_sum_number_deleted_rows,
        median( sum_number_deleted_rows )     over () as med_sum_number_deleted_rows,
        avg( sum_number_deleted_rows )        over () as avg_sum_number_deleted_rows,
        stddev_pop( sum_number_deleted_rows ) over () as std_sum_number_deleted_rows,
        sum( sum_number_deleted_rows )        over () as sum_sum_number_deleted_rows,

        min( duration ) over ()                              as min_duration,
        max( duration ) over ()                              as max_duration,
        median( extract(epoch from (duration)) ) over ()     as med_duration,
        avg( duration ) over ()                              as avg_duration,
        stddev_pop( extract(epoch from (duration)) ) over () as std_duration,
        sum( duration ) over ()                              as sum_duration,

        sum( number_qids ) over () as number_qids,
        count(*) over ()           as number_xids
      from
        cte_xids
    )

  select
    start_ts::timestamp,

    min_sum_number_new_blocks::int8,
    max_sum_number_new_blocks::int8,
    med_sum_number_new_blocks::int8,
    avg_sum_number_new_blocks::float4,
    std_sum_number_new_blocks::float4,
    sum_sum_number_new_blocks::float4,

    min_sum_number_replaced_blocks::int8,
    max_sum_number_replaced_blocks::int8,
    med_sum_number_replaced_blocks::int8,
    avg_sum_number_replaced_blocks::float4,
    std_sum_number_replaced_blocks::float4,
    sum_sum_number_replaced_blocks::float4,

    min_sum_number_new_rows::int8,
    max_sum_number_new_rows::int8,
    med_sum_number_new_rows::int8,
    avg_sum_number_new_rows::float4,
    std_sum_number_new_rows::float4,
    sum_sum_number_new_rows::float4,

    min_sum_number_deleted_rows::int8,
    max_sum_number_deleted_rows::int8,
    med_sum_number_deleted_rows::int8,
    avg_sum_number_deleted_rows::float4,
    std_sum_number_deleted_rows::float4,
    sum_sum_number_deleted_rows::float4,

    min_duration::interval,
    max_duration::interval,
    med_duration::float4,
    avg_duration::interval,
    std_duration::float4,
    sum_duration::interval,

    number_qids::int8,
    number_xids::int8
  from
              base_241129.generate_minutes
    left join cte_xids_stats using ( start_ts )
  group by
    start_ts,
    min_sum_number_new_blocks,
    max_sum_number_new_blocks,
    med_sum_number_new_blocks,
    avg_sum_number_new_blocks,
    std_sum_number_new_blocks,
    sum_sum_number_new_blocks,
    min_sum_number_replaced_blocks,
    max_sum_number_replaced_blocks,
    med_sum_number_replaced_blocks,
    avg_sum_number_replaced_blocks,
    std_sum_number_replaced_blocks,
    sum_sum_number_replaced_blocks,
    min_sum_number_new_rows,
    max_sum_number_new_rows,
    med_sum_number_new_rows,
    avg_sum_number_new_rows,
    std_sum_number_new_rows,
    sum_sum_number_new_rows,
    min_sum_number_deleted_rows,
    max_sum_number_deleted_rows,
    med_sum_number_deleted_rows,
    avg_sum_number_deleted_rows,
    std_sum_number_deleted_rows,
    sum_sum_number_deleted_rows,
    min_duration,
    max_duration,
    med_duration,
    avg_duration,
    std_duration,
    sum_duration,
    number_qids,
    number_xids;


create or replace view
  lf_241129.ttables_stats_datatypes_encodings_by_count_and_blocks
as
  with
    cte_datatype_encoding_column_counts as 
    (
      select
        pg_attribute.atttypid::int4                                           as base_data_type_id,
        sum( case when attencodingtype in (20, 148) then 1 else 0 end )::int4 as number_az64_columns,
        sum( case when attencodingtype =  1         then 1 else 0 end )::int4 as number_bytedict_columns,
        sum( case when attencodingtype =  2         then 1 else 0 end )::int4 as number_delta_columns,
        sum( case when attencodingtype =  5         then 1 else 0 end )::int4 as number_delta32k_columns,
        sum( case when attencodingtype in (3, 131)  then 1 else 0 end )::int4 as number_lzo_columns,
        sum( case when attencodingtype =  15        then 1 else 0 end )::int4 as number_mostly8_columns,
        sum( case when attencodingtype =  16        then 1 else 0 end )::int4 as number_mostly16_columns,
        sum( case when attencodingtype =  17        then 1 else 0 end )::int4 as number_mostly32_columns,
        sum( case when attencodingtype in (0, 128)  then 1 else 0 end )::int4 as number_raw_columns,
        sum( case when attencodingtype =  4         then 1 else 0 end )::int4 as number_runlength_columns,
        sum( case when attencodingtype =  7         then 1 else 0 end )::int4 as number_text255_columns,
        sum( case when attencodingtype =  18        then 1 else 0 end )::int4 as number_text32k_columns,
        sum( case when attencodingtype in (19, 147) then 1 else 0 end )::int4 as number_zstd_columns,
        count(*)::int4                                                        as number_total_columns
      from
                  pg_class
             join pg_namespace  on pg_namespace.oid = pg_class.relnamespace
             join pg_attribute  on pg_class.oid     = pg_attribute.attrelid
             join pg_type       on pg_type.oid      = pg_attribute.atttypid
      where
            pg_class.relowner                != 1
        and pg_class.relkind                  = 'r'
        and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
        and left(pg_namespace.nspname,8)     != 'pg_temp_'
        and pg_attribute.attnum              >= 0
      group by
        base_data_type_id
    ),

    cte_datatype_encoding_block_counts as
    (
      select
        pg_attribute.atttypid::int4                                           as base_data_type_id,
        sum( case when attencodingtype in (20, 148) then 1 else 0 end )::int8 as number_az64_blocks,
        sum( case when attencodingtype =  1         then 1 else 0 end )::int8 as number_bytedict_blocks,
        sum( case when attencodingtype =  2         then 1 else 0 end )::int8 as number_delta_blocks,
        sum( case when attencodingtype =  5         then 1 else 0 end )::int8 as number_delta32k_blocks,
        sum( case when attencodingtype in (3, 131)  then 1 else 0 end )::int8 as number_lzo_blocks,
        sum( case when attencodingtype =  15        then 1 else 0 end )::int8 as number_mostly8_blocks,
        sum( case when attencodingtype =  16        then 1 else 0 end )::int8 as number_mostly16_blocks,
        sum( case when attencodingtype =  17        then 1 else 0 end )::int8 as number_mostly32_blocks,
        sum( case when attencodingtype in (0, 128)  then 1 else 0 end )::int8 as number_raw_blocks,
        sum( case when attencodingtype =  4         then 1 else 0 end )::int8 as number_runlength_blocks,
        sum( case when attencodingtype =  7         then 1 else 0 end )::int8 as number_text255_blocks,
        sum( case when attencodingtype =  18        then 1 else 0 end )::int8 as number_text32k_blocks,
        sum( case when attencodingtype in (19, 147) then 1 else 0 end )::int8 as number_zstd_blocks,
        count(*)::int8                                                        as number_total_blocks
      from
                  pg_class
             join pg_namespace  on pg_namespace.oid = pg_class.relnamespace
             join pg_attribute  on pg_class.oid     = pg_attribute.attrelid
             join pg_type       on pg_type.oid      = pg_attribute.atttypid
        left join stv_blocklist on pg_class.oid        = stv_blocklist.tbl and
                                   pg_attribute.attnum = stv_blocklist.col + 1
      where
            pg_class.relowner                != 1
        and pg_class.relkind                  = 'r'
        and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
        and left(pg_namespace.nspname,8)     != 'pg_temp_'
        and pg_attribute.attnum              >= 0
      group by
        base_data_type_id
    ),

    cte_data_types as
    (
      select base_data_type_id from cte_datatype_encoding_column_counts
      union
      select base_data_type_id from cte_datatype_encoding_block_counts
    )

  select
    base_data_type_id::int4,
    case
      when pg_type.typname = 'bpchar' then 'char'
                                      else trim( pg_type.typname )
    end::varchar as base_data_type_name,

    number_az64_columns::int4,
    number_bytedict_columns::int4,
    number_delta_columns::int4,
    number_delta32k_columns::int4,
    number_lzo_columns::int4,
    number_mostly8_columns::int4,
    number_mostly16_columns::int4,
    number_mostly32_columns::int4,
    number_raw_columns::int4,
    number_runlength_columns::int4,
    number_text255_columns::int4,
    number_text32k_columns::int4,
    number_zstd_columns::int4,
    number_total_columns::int4,

    number_az64_blocks::int8,
    number_bytedict_blocks::int8,
    number_delta_blocks::int8,
    number_delta32k_blocks::int8,
    number_lzo_blocks::int8,
    number_mostly8_blocks::int8,
    number_mostly16_blocks::int8,
    number_mostly32_blocks::int8,
    number_raw_blocks::int8,
    number_runlength_blocks::int8,
    number_text255_blocks::int8,
    number_text32k_blocks::int8,
    number_zstd_blocks::int8,
    number_total_blocks::int8
  from
              cte_data_types
         join pg_type on cte_data_types.base_data_type_id = pg_type.oid
    left join cte_datatype_encoding_column_counts using ( base_data_type_id )
    left join cte_datatype_encoding_block_counts  using ( base_data_type_id );


create or replace view
  sf_241129.ttables_stats_datatypes_encodings_by_count_and_blocks
as
  with
    cte_hr_formatting as
    (
      select
        base_data_type_name,

        number_az64_columns::varchar      as hr_number_az64_columns,
        number_bytedict_columns::varchar  as hr_number_bytedict_columns,
        number_delta_columns::varchar     as hr_number_delta_columns,
        number_delta32k_columns::varchar  as hr_number_delta32k_columns,
        number_lzo_columns::varchar       as hr_number_lzo_columns,
        number_mostly8_columns::varchar   as hr_number_mostly8_columns,
        number_mostly16_columns::varchar  as hr_number_mostly16_columns,
        number_mostly32_columns::varchar  as hr_number_mostly32_columns,
        number_raw_columns::varchar       as hr_number_raw_columns,
        number_runlength_columns::varchar as hr_number_runlength_columns,
        number_text255_columns::varchar   as hr_number_text255_columns,
        number_text32k_columns::varchar   as hr_number_text32k_columns,
        number_zstd_columns::varchar      as hr_number_zstd_columns,
        number_total_columns::varchar     as hr_number_total_columns,

        base_241129.hr_blks( number_az64_blocks      ) as hr_number_az64_blocks,
        base_241129.hr_blks( number_bytedict_blocks  ) as hr_number_bytedict_blocks,
        base_241129.hr_blks( number_delta_blocks     ) as hr_number_delta_blocks,
        base_241129.hr_blks( number_delta32k_blocks  ) as hr_number_delta32k_blocks,
        base_241129.hr_blks( number_lzo_blocks       ) as hr_number_lzo_blocks,
        base_241129.hr_blks( number_mostly8_blocks   ) as hr_number_mostly8_blocks,
        base_241129.hr_blks( number_mostly16_blocks  ) as hr_number_mostly16_blocks,
        base_241129.hr_blks( number_mostly32_blocks  ) as hr_number_mostly32_blocks,
        base_241129.hr_blks( number_raw_blocks       ) as hr_number_raw_blocks,
        base_241129.hr_blks( number_runlength_blocks ) as hr_number_runlength_blocks,
        base_241129.hr_blks( number_text255_blocks   ) as hr_number_text255_blocks,
        base_241129.hr_blks( number_text32k_blocks   ) as hr_number_text32k_blocks,
        base_241129.hr_blks( number_zstd_blocks      ) as hr_number_zstd_blocks,
        base_241129.hr_blks( number_total_blocks     ) as hr_number_total_blocks,

        max( len(hr_number_az64_columns)      ) over () as max_len_hr_number_az64_columns_width,
        max( len(hr_number_bytedict_columns)  ) over () as max_len_hr_number_bytedict_columns_width,
        max( len(hr_number_delta_columns)     ) over () as max_len_hr_number_delta_columns_width,
        max( len(hr_number_delta32k_columns)  ) over () as max_len_hr_number_delta32k_columns_width,
        max( len(hr_number_lzo_columns)       ) over () as max_len_hr_number_lzo_columns_width,
        max( len(hr_number_mostly8_columns)   ) over () as max_len_hr_number_mostly8_columns_width,
        max( len(hr_number_mostly16_columns)  ) over () as max_len_hr_number_mostly16_columns_width,
        max( len(hr_number_mostly32_columns)  ) over () as max_len_hr_number_mostly32_columns_width,
        max( len(hr_number_raw_columns)       ) over () as max_len_hr_number_raw_columns_width,
        max( len(hr_number_runlength_columns) ) over () as max_len_hr_number_runlength_columns_width,
        max( len(hr_number_text255_columns)   ) over () as max_len_hr_number_text255_columns_width,
        max( len(hr_number_text32k_columns)   ) over () as max_len_hr_number_text32k_columns_width,
        max( len(hr_number_zstd_columns)      ) over () as max_len_hr_number_zstd_columns_width,
        max( len(hr_number_total_columns)     ) over () as max_len_hr_number_total_columns_width,

        max( len(hr_number_az64_blocks)      ) over () as max_len_hr_number_az64_blocks_width,
        max( len(hr_number_bytedict_blocks)  ) over () as max_len_hr_number_bytedict_blocks_width,
        max( len(hr_number_delta_blocks)     ) over () as max_len_hr_number_delta_blocks_width,
        max( len(hr_number_delta32k_blocks)  ) over () as max_len_hr_number_delta32k_blocks_width,
        max( len(hr_number_lzo_blocks)       ) over () as max_len_hr_number_lzo_blocks_width,
        max( len(hr_number_mostly8_blocks)   ) over () as max_len_hr_number_mostly8_blocks_width,
        max( len(hr_number_mostly16_blocks)  ) over () as max_len_hr_number_mostly16_blocks_width,
        max( len(hr_number_mostly32_blocks)  ) over () as max_len_hr_number_mostly32_blocks_width,
        max( len(hr_number_raw_blocks)       ) over () as max_len_hr_number_raw_blocks_width,
        max( len(hr_number_runlength_blocks) ) over () as max_len_hr_number_runlength_blocks_width,
        max( len(hr_number_text255_blocks)   ) over () as max_len_hr_number_text255_blocks_width,
        max( len(hr_number_text32k_blocks)   ) over () as max_len_hr_number_text32k_blocks_width,
        max( len(hr_number_zstd_blocks)      ) over () as max_len_hr_number_zstd_blocks_width,
        max( len(hr_number_total_blocks)     ) over () as max_len_hr_number_total_blocks_width
      from
        lf_241129.ttables_stats_datatypes_encodings_by_count_and_blocks
    )

  select
    base_data_type_name::varchar as data_type,

    ( lpad( hr_number_az64_columns,      max_len_hr_number_az64_columns_width      ) || ' / ' || lpad( hr_number_az64_blocks,      max_len_hr_number_az64_blocks_width      ) )::varchar as "az64",
    ( lpad( hr_number_bytedict_columns,  max_len_hr_number_bytedict_columns_width  ) || ' / ' || lpad( hr_number_bytedict_blocks,  max_len_hr_number_bytedict_blocks_width  ) )::varchar as "bytedict",
    ( lpad( hr_number_delta_columns,     max_len_hr_number_delta_columns_width     ) || ' / ' || lpad( hr_number_delta_blocks,     max_len_hr_number_delta_blocks_width     ) )::varchar as "delta",
    ( lpad( hr_number_delta32k_columns,  max_len_hr_number_delta32k_columns_width  ) || ' / ' || lpad( hr_number_delta32k_blocks,  max_len_hr_number_delta32k_blocks_width  ) )::varchar as "delta32k",
    ( lpad( hr_number_lzo_columns,       max_len_hr_number_lzo_columns_width       ) || ' / ' || lpad( hr_number_lzo_blocks,       max_len_hr_number_lzo_blocks_width       ) )::varchar as "lzo",
    ( lpad( hr_number_mostly8_columns,   max_len_hr_number_mostly8_columns_width   ) || ' / ' || lpad( hr_number_mostly8_blocks,   max_len_hr_number_mostly8_blocks_width   ) )::varchar as "mostly8",
    ( lpad( hr_number_mostly16_columns,  max_len_hr_number_mostly16_columns_width  ) || ' / ' || lpad( hr_number_mostly16_blocks,  max_len_hr_number_mostly16_blocks_width  ) )::varchar as "mostly16",
    ( lpad( hr_number_mostly32_columns,  max_len_hr_number_mostly32_columns_width  ) || ' / ' || lpad( hr_number_mostly32_blocks,  max_len_hr_number_mostly32_blocks_width  ) )::varchar as "mostly32",
    ( lpad( hr_number_raw_columns,       max_len_hr_number_raw_columns_width       ) || ' / ' || lpad( hr_number_raw_blocks,       max_len_hr_number_raw_blocks_width       ) )::varchar as "raw",
    ( lpad( hr_number_runlength_columns, max_len_hr_number_runlength_columns_width ) || ' / ' || lpad( hr_number_runlength_blocks, max_len_hr_number_runlength_blocks_width ) )::varchar as "runlength",
    ( lpad( hr_number_text255_columns,   max_len_hr_number_text255_columns_width   ) || ' / ' || lpad( hr_number_text255_blocks,   max_len_hr_number_text255_blocks_width   ) )::varchar as "text255",
    ( lpad( hr_number_text32k_columns,   max_len_hr_number_text32k_columns_width   ) || ' / ' || lpad( hr_number_text32k_blocks,   max_len_hr_number_text32k_blocks_width   ) )::varchar as "text32k",
    ( lpad( hr_number_zstd_columns,      max_len_hr_number_zstd_columns_width      ) || ' / ' || lpad( hr_number_zstd_blocks,      max_len_hr_number_zstd_blocks_width      ) )::varchar as "zstd",
    ( lpad( hr_number_total_columns,     max_len_hr_number_total_columns_width     ) || ' / ' || lpad( hr_number_total_blocks,     max_len_hr_number_total_blocks_width     ) )::varchar as "total"
  from
    cte_hr_formatting
  order by
    data_type;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_minute_complete
as
  select
    start_ts::timestamp,
    node_ordinal::int2 as n,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_minute
  order by
    start_ts,
    node_ordinal;


create or replace view
  sf_241129.ttables_blocks_rows
as
  select
    schema_name::varchar          as sschema,
    table_name::varchar           as ttable,
    user_name::varchar            as uuser,
    ( extract( epoch from table_age ) / (60*60*24) )::int4 as age,
    table_number_columns::int2    as cols,
    table_sort_type::varchar      as ssort,
    table_number_sortkeys::int2   as skeys,
    case
      when table_distribution_auto_flag = true  then table_distribution_type || ' (auto)'
      when table_distribution_auto_flag = false then table_distribution_type
    end::varchar                  as dist,
    number_slices::int2           as slices,
    number_sorted_blocks::int8    as sblks,
    number_desorted_blocks::int8  as dsblks,
    number_unsorted_blocks::int8  as ublks,
    number_total_blocks::int8     as tblks,
    number_sorted_rows::int8      as srows,
    number_desorted_rows::int8    as dsrows,
    number_unsorted_rows::int8    as urows,
    number_total_rows::int8       as trows
  from
    lf_241129.ttables_blocks_rows
  order by
    schema_name,
    table_name;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_query
as
  with
    cte_queries as
    (
      select
        query_id,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id )
        left join base_241129.queries_compilation  using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        query_id,

        min( query_queue_duration ) over ( partition by query_id )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by query_id )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by query_id )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by query_id )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by query_id ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by query_id )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by query_id )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by query_id )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by query_id )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by query_id )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by query_id ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by query_id )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by query_id )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by query_id )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by query_id )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by query_id )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by query_id ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by query_id )                            as sum_query_exec_duration
      from
        cte_queries
    )

  select
    query_id::int4,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval
  from
    cte_queries_stats
  group by
    query_id,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration;



create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_all_concise
as
  select
    node_ordinal::int2 as n,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_all
  order by
    node_ordinal;


create or replace view
  sf_241129.wqueues
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        number_slots,
        number_queued_queries,
        number_running_queries,
        number_completed_queries,

        extract(epoch from min_queue_duration)::numeric(18,2)::varchar as hr_min_queue_duration,
        extract(epoch from max_queue_duration)::numeric(18,2)::varchar as hr_max_queue_duration,
        med_queue_duration::numeric(18,2)::varchar                     as hr_med_queue_duration,
        extract(epoch from avg_queue_duration)::numeric(18,2)::varchar as hr_avg_queue_duration,
        std_queue_duration::numeric(18,2)::varchar                     as hr_std_queue_duration,

        extract(epoch from min_exec_duration)::numeric(18,2)::varchar as hr_min_exec_duration,
        extract(epoch from max_exec_duration)::numeric(18,2)::varchar as hr_max_exec_duration,
        med_exec_duration::numeric(18,2)::varchar                     as hr_med_exec_duration,
        extract(epoch from avg_exec_duration)::numeric(18,2)::varchar as hr_avg_exec_duration,
        std_exec_duration::numeric(18,2)::varchar                     as hr_std_exec_duration,

        max( len(hr_min_queue_duration) ) over () as max_len_hr_min_queue_duration,
        max( len(hr_max_queue_duration) ) over () as max_len_hr_max_queue_duration,
        max( len(hr_med_queue_duration) ) over () as max_len_hr_med_queue_duration,
        max( len(hr_avg_queue_duration) ) over () as max_len_hr_avg_queue_duration,
        max( len(hr_std_queue_duration) ) over () as max_len_hr_std_queue_duration,

        max( len(hr_min_exec_duration) ) over () as max_len_hr_min_exec_duration,
        max( len(hr_max_exec_duration) ) over () as max_len_hr_max_exec_duration,
        max( len(hr_med_exec_duration) ) over () as max_len_hr_med_exec_duration,
        max( len(hr_avg_exec_duration) ) over () as max_len_hr_avg_exec_duration,
        max( len(hr_std_exec_duration) ) over () as max_len_hr_std_exec_duration,

        queue_csc_flag,
        number_csc_completed_queries
      from
        lf_241129.wqueues
    )

  select
    queue_name::varchar                                   as qname,
    number_slots::int2                                    as slots,
    number_queued_queries::int2                           as queued,
    number_running_queries::int2                          as running,
    number_completed_queries::int4                        as completed,

    ( lpad( hr_min_queue_duration, max_len_hr_min_queue_duration ) || ' / ' ||
      lpad( hr_max_queue_duration, max_len_hr_max_queue_duration ) || ' / ' ||
      lpad( hr_med_queue_duration, max_len_hr_med_queue_duration ) || ' / ' ||
      lpad( hr_avg_queue_duration, max_len_hr_avg_queue_duration ) || ' / ' ||
      lpad( hr_std_queue_duration, max_len_hr_std_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_exec_duration, max_len_hr_min_exec_duration ) || ' / ' ||
      lpad( hr_max_exec_duration, max_len_hr_max_exec_duration ) || ' / ' ||
      lpad( hr_med_exec_duration, max_len_hr_med_exec_duration ) || ' / ' ||
      lpad( hr_avg_exec_duration, max_len_hr_avg_exec_duration ) || ' / ' ||
      lpad( hr_std_exec_duration, max_len_hr_std_exec_duration ) )::varchar as exec,

    case when queue_csc_flag = true then '✔' else '' end::varchar  as csc,
    number_csc_completed_queries::int4                    as csc_comp
  from
    cte_hr_formatting
  order by
    queue_name;


create or replace view
  lf_241129.queries_for_all
as
  select
    user_id::int4,
    user_name::varchar,
    session_id::int4,
    transaction_id::int8,
    query_id::int4,
    queue_id::int2,
    queue_name::varchar,
    query_start_ts::timestamp,
    query_end_ts::timestamp,
    query_queue_duration::interval,
    query_compile_exec_duration::interval,
    query_wlm_duration::interval,
    query_total_duration::interval,
    query_abort_flag::bool,
    query_csc_flag::bool,
    query_leader_node_flag::bool,
    query_rc_flag::bool,
    query_text::varchar
  from
              base_241129.queries_for_all
         join base_241129.fixed_pg_user using ( user_id )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  sf_241129.queries_for_all
as
  select
    user_name::varchar                                               as uuser,
    session_id::int4                                                 as pid,
    transaction_id::int8                                             as xid,
    query_id::int4                                                   as qid,
    queue_name::varchar                                              as qname,
    query_start_ts::timestamp                                        as start_ts,
    query_end_ts::timestamp                                          as end_ts,
    extract( epoch from query_queue_duration )::numeric(18,2)        as queue,
    extract( epoch from query_compile_exec_duration )::numeric(18,2) as cexec,
    extract( epoch from query_total_duration )::numeric(18,2)        as total,
    (
      case when query_abort_flag       = true then 'A' else ' ' end || 
      case when query_csc_flag         = true then 'C' else ' ' end ||
      case when query_leader_node_flag = true then 'L' else ' ' end ||
      case when query_rc_flag          = true then 'R' else ' ' end 
    )::varchar                                                       as flags,
    left( ltrim(query_text), 48 )::varchar                           as ttext
  from
    lf_241129.queries_for_all
  order by
    query_start_ts;


create or replace view
  base_241129.views_tables
as
  with recursive
    flat_dependencies as
    (
      select
        case
          when pg_class_parent_type.relname = 'pg_class'     and pg_class_parent_class.relkind = 'r'                    then 'table'
          when pg_class_parent_type.relname = 'pg_class'     and pg_class_parent_class.relkind = 'v'                    then 'view'
                                                                                                                        else null
        end                                        as parent_object_type,
        case
          when pg_class_parent_type.relname = 'pg_class'                                                                then pg_class_parent_class.oid
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then pg_namespace_parent_class.oid
        end                                        as parent_object_id,

        case when pg_class_parent_type.relname = 'pg_class'     and pg_class_parent_class.relkind = 'r'                 then pg_class_parent_class.oid         else null end as parent_table_id,
        case when pg_class_parent_type.relname = 'pg_class'     and pg_class_parent_class.relkind = 'v'                 then pg_class_parent_class.oid         else null end as parent_view_id,

        case
          when pg_class_child_type.relname = 'pg_class'     and pg_class_child_class_table_or_view.relkind = 'r'        then 'table'     -- MG2 : unnecessary; tables can't be children
          when pg_class_child_type.relname = 'pg_class'     and pg_class_child_class_table_or_view.relkind = 'v'        then 'view'      -- MG2 : when the parent is not a table or view (i.e. is a namespace, and maybe also a function)
          when pg_class_child_type.relname = 'pg_rewrite'   and pg_class_child_class_view.relkind          = 'v'        then 'view'      -- MG2 : note "pg_rewrite" is correct, and different to parents
                                                                                                                        else null
        end                                        as child_object_type,
        case
          when pg_class_child_type.relname = 'pg_class'                                                                 then pg_class_child_class_table_or_view.oid
          when pg_class_child_type.relname = 'pg_rewrite'                                                               then pg_class_child_class_view.oid
        end                                        as child_object_id,

        case when pg_class_child_type.relname = 'pg_class'                                                              then pg_class_child_class_table_or_view.oid else null end as child_table_id,
        case when pg_class_child_type.relname = 'pg_rewrite'                                                            then pg_class_child_class_view.oid          else null end as child_view_id,
        null                                                                                                                                                                      as child_database_id
      from
             -- MG2 : distinct causes an assert; group by is a workaround
             --       normal dependency are the only type we care about

             ( select classid, objid, refclassid, refobjid from pg_depend where deptype = 'n' group by classid, objid, refclassid, refobjid ) as pg_depend_distinct

             -- MG2 : first, parents
             join pg_class      as pg_class_parent_type       on pg_class_parent_type.oid = pg_depend_distinct.refclassid

        -- MG2 : parents who are in pg_class (and by the where clause will be tables or views)
        left join pg_class      as pg_class_parent_class      on pg_class_parent_type.relname = 'pg_class' and pg_depend_distinct.refobjid        = pg_class_parent_class.oid
        left join pg_namespace  as pg_namespace_parent_class  on pg_class_parent_type.relname = 'pg_class' and pg_class_parent_class.relnamespace = pg_namespace_parent_class.oid
        left join ( select usesysid, usename from pg_user ) as pg_user_parent_class on pg_class_parent_type.relname = 'pg_class' and pg_class_parent_class.relowner = pg_user_parent_class.usesysid

              -- MG2 : next, children
             join pg_class      as pg_class_child_type        on pg_class_child_type.oid = pg_depend_distinct.classid

        -- MG2 : children who are in pg_class (and which surprisingly can be both tables or views - turns out not all views go via pg_rewrite)
        left join pg_class      as pg_class_child_class_table_or_view      on pg_class_child_type.relname = 'pg_class' and pg_depend_distinct.objid                   = pg_class_child_class_table_or_view.oid
        left join pg_namespace  as pg_namespace_child_class_table_or_view  on pg_class_child_type.relname = 'pg_class' and pg_namespace_child_class_table_or_view.oid = pg_class_child_class_table_or_view.relnamespace
        left join ( select usesysid, usename from pg_user ) as pg_user_child_class_table_or_view on pg_class_child_type.relname = 'pg_class' and pg_user_child_class_table_or_view.usesysid = pg_class_child_class_table_or_view.relowner

        -- MG2 : children who are in pg_rewrite (and so are really in pg_class, and are views)
        left join pg_rewrite    as pg_rewrite_child_class          on pg_class_child_type.relname = 'pg_rewrite' and pg_rewrite_child_class.oid         = pg_depend_distinct.objid
        left join pg_class      as pg_class_child_class_view       on pg_class_child_type.relname = 'pg_rewrite' and pg_class_child_class_view.oid      = pg_rewrite_child_class.ev_class
        left join pg_namespace  as pg_namespace_child_class_view   on pg_class_child_type.relname = 'pg_rewrite' and pg_namespace_child_class_view.oid  = pg_class_child_class_view.relnamespace
        left join ( select usesysid, usename from pg_user ) as pg_user_child_class_view on pg_class_child_type.relname = 'pg_rewrite' and pg_class_child_class_view.relowner = pg_user_child_class_view.usesysid

      where
        -- MG2 : ignore pg_type and pg_conversion
        pg_class_parent_type.relname not in ('pg_conversion', 'pg_type') and
        pg_class_child_type.relname  not in ('pg_conversion', 'pg_type') and
        -- MG2 : ignore all the (certainly seemingly) strange and (definitely) unexpected dependencies where a view depends upon itself (it does not in fact, at least not in the SQL of the view)
        not
        (
          pg_rewrite_child_class.ev_class is not null                                             and
          pg_class_parent_type.relname = 'pg_class'   and pg_class_parent_class.relkind     = 'v' and
          pg_class_child_type.relname  = 'pg_rewrite' and pg_class_child_class_view.relkind = 'v' and
          pg_depend_distinct.refobjid  = pg_rewrite_child_class.ev_class
        )
    ),

    dependencies( original_table_id, depth, 
                  parent_object_type, parent_object_id, parent_table_id, parent_view_id,
                  child_object_type, child_object_id, child_table_id, child_view_id ) as 
    (
      select
        flat_dependencies.parent_object_id as original_table_id,
        0 as depth,
        flat_dependencies.parent_object_type,
        flat_dependencies.parent_object_id,
        flat_dependencies.parent_table_id,
        flat_dependencies.parent_view_id,
        flat_dependencies.child_object_type,
        flat_dependencies.child_object_id,
        flat_dependencies.child_table_id,
        flat_dependencies.child_view_id
      from
        flat_dependencies
      where
        parent_object_type = 'table'

      union all

      select
        dependencies.original_table_id,
        dependencies.depth + 1 as depth,
        flat_dependencies.parent_object_type,
        flat_dependencies.parent_object_id,
        flat_dependencies.parent_table_id,
        flat_dependencies.parent_view_id,
        flat_dependencies.child_object_type,
        flat_dependencies.child_object_id,
        flat_dependencies.child_table_id,
        flat_dependencies.child_view_id
      from
             dependencies
        join flat_dependencies on dependencies.child_object_id = flat_dependencies.parent_object_id
    )

  select distinct
    child_view_id::int4      as view_id,
    original_table_id::int4  as table_id
  from
    dependencies;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_schemas_by_all
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl
    ),

    cte_scan as
    (
      select
        tbl         as table_id,
        sum( rows ) as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_scan.sum_scan_rows )::int8     as sum_scan_rows
  from
              base_241129.fixed_pg_class_for_tables
         join base_241129.fixed_pg_namespace using ( schema_id )
    left join cte_delete                 using ( table_id )
    left join cte_insert                 using ( table_id )
    left join cte_scan                   using ( table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null
  group by
    base_241129.fixed_pg_namespace.schema_id,
    base_241129.fixed_pg_namespace.schema_name;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_schemas_by_all
as
  select
    schema_name::varchar                                    as sschema,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_schemas_by_all
  order by
    schema_name;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_slices_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        slice_ordinal,
        slice_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_slices_by_hour
    )

  select
    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    slice_ordinal,
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_all_by_second
as
  with
    cte_aggr as
    (
      select
        date_trunc( 'second', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        date_trunc( 'second', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        date_trunc( 'second', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        date_trunc( 'second', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        date_trunc( 'second', starttime )                          as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        date_trunc( 'second', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        date_trunc( 'second', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        date_trunc( 'second', starttime )                                               as start_ts,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        date_trunc( 'second', starttime )                       as start_ts,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_seconds.start_ts::timestamp as start_ts,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               cte_node_types
    cross join base_241129.generate_seconds
     left join cte_aggr         using ( node_type, start_ts )
     left join cte_bcast        using ( node_type, start_ts )
     left join cte_delete       using ( node_type, start_ts )
     left join cte_dist         using ( node_type, start_ts )
     left join cte_hash         using ( node_type, start_ts )
     left join cte_hashjoin     using ( node_type, start_ts )
     left join cte_insert       using ( node_type, start_ts )
     left join cte_limit        using ( node_type, start_ts )
     left join cte_merge        using ( node_type, start_ts )
     left join cte_mergejoin    using ( node_type, start_ts )
     left join cte_nestloop     using ( node_type, start_ts )
     left join cte_parse        using ( node_type, start_ts )
     left join cte_project      using ( node_type, start_ts )
     left join cte_return       using ( node_type, start_ts )
     left join cte_save         using ( node_type, start_ts )
     left join cte_sort         using ( node_type, start_ts )
     left join cte_spatial_join using ( node_type, start_ts )
     left join cte_unique       using ( node_type, start_ts )
     left join cte_window       using ( node_type, start_ts );


create or replace view
  lf_241129.analyze_totals_for_users_by_second
as
  with
    cte_s1 as
    (
      select
        userid                         as user_id,
        date_trunc( 'second', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        userid,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
             cte_s1
   left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  sf_241129.analyze_totals_for_users_by_second
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_users_by_second
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;



create or replace view
  sf_241129.steps_totals_rows_on_workers_for_schemas_by_minute
as
  select
    schema_name::varchar                                     as sschema,
    start_ts::timestamp,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_schemas_by_minute
  order by
    schema_name,
    start_ts;


create or replace procedure
  ro.sp_revoke_versionless_sf_schema_from_user( user_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'revoke usage on schema sf from ' || user_name;
      execute sql;

      sql := 'revoke select on all tables in schema sf from ' || user_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.ttables_columns_blocks
as
  with
    cte_column_blocks as
    (
      -- MG2 : no excluding tables; we want to see everything

      select
        tbl                             as table_id,
        col                             as column_ordinal,
        sum( 1-stv_blocklist.unsorted ) as number_sorted_blocks,
        sum(   stv_blocklist.unsorted ) as number_unsorted_blocks,
        count( * )                      as number_total_blocks
      from
        stv_blocklist
      group by
        tbl,
        col
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    column_ordinal::int2,
    column_name::varchar,
    column_data_type_id::int4,
    column_base_data_type_name::varchar,
    column_octet_length::int4,
    column_precision::int2,
    column_scale::int2,
    column_full_data_type_name::varchar,
    column_encoding_id::int4,
    column_encoding_name::varchar,
    column_sortkey_ordinal::int2,
    column_distkey::bool,
    column_notnull::bool,
    number_sorted_blocks::int8,
    number_unsorted_blocks::int8,
    number_total_blocks::int8
  from
         base_241129.ttables_columns_basic_info
    join cte_column_blocks using ( table_id, column_ordinal );


create or replace view
  sf_241129.ttables_columns_blocks
as
  select
    schema_name::varchar                                                                        as sschema,
    table_name::varchar                                                                         as ttable,
    column_name::varchar                                                                        as ccolumn,
    column_full_data_type_name::varchar                                                         as ttype,
    column_encoding_name::varchar                                                               as enc,
    case when column_sortkey_ordinal >  0    then column_sortkey_ordinal else null end::int2    as sk,
    case when column_distkey          = true then 'dk'                   else null end::varchar as dk,
    case when column_notnull          = true then 'nn'                   else null end::varchar as nn,
    base_241129.hr_blks( number_sorted_blocks   )::varchar                                          as sblks,
    base_241129.hr_blks( number_unsorted_blocks )::varchar                                          as ublks,
    base_241129.hr_blks( number_total_blocks    )::varchar                                          as tblks
  from
    lf_241129.ttables_columns_blocks
  order by
    schema_name,
    table_name,
    column_name;


create or replace view
  sf_241129.views_normal_columns
as
  select
    schema_name::varchar                as sschema,
    view_name::varchar                  as vview,
    column_name::varchar                as ccolumn,
    column_full_data_type_name::varchar as data_type
  from
    lf_241129.views_normal_columns
  order by
    schema_name,
    view_name,
    column_name;


create or replace view
  lf_241129.queries_totals_result_cache_for_users_by_second
as
  with
    cte_s1 as
    (
      select
        user_id,
        date_trunc( 'second', query_start_ts )                                                                      as start_ts,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
        count(*)                                                                                                 as number_total,
        ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_start_ts   >= current_date - interval '6 day'
        and query_end_ts     <= current_timestamp
      group by
        user_id,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    number_hit::int8,
    number_miss::int8,
    number_total::int8,
    percentage::float4
  from
              cte_s1
    left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  lf_241129.transactions_totals_commits_by_hour
as
  select
    date_trunc( 'hour', batch_leader_startwork )::timestamp as start_ts,
    sum( batch_leader_duration )::interval                     as sum_batch_leader_duration,
    sum( batch_worker_duration )::interval                     as sum_batch_worker_duration,
    sum( batch_exec_duration - batch_leader_duration - batch_worker_duration )::interval as sum_batch_idle_duration,
    sum( batch_exec_duration  )::interval                      as sum_batch_exec_duration
  from
    lf_241129.batches
  group by
    start_ts;


create or replace view
  sf_241129.transactions_totals_commits_by_hour
as
  select
    start_ts::timestamp,
    extract( epoch from sum_batch_leader_duration )::numeric(18,2) as leader,
    extract( epoch from sum_batch_worker_duration )::numeric(18,2) as worker,
    extract( epoch from sum_batch_idle_duration   )::numeric(18,2) as idle,
    extract( epoch from sum_batch_exec_duration   )::numeric(18,2) as exec
  from
    lf_241129.transactions_totals_commits_by_hour
  order by
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_second_complete
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_second
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;



create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_node_by_minute
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'minute', scan_start_ts )::timestamp as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal )
  group by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_node_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_node_by_minute
    )

  select
    node_ordinal::int2  as n,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_slices_by_all
as
  select
    slice_ordinal::int2 as slice,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_slices_by_all
  order by
    slice_ordinal;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_statements_by_minute
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query as query_id
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query as query_id
      from
        stl_unload_log
    ),

    cte_queries as
    (
      select
        cte_base.statement_name,
        date_trunc( 'minute', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
             cte_base
        join base_241129.fixed_stl_wlm_query using ( query_id )
        join base_241129.fixed_stl_query     using ( query_id )
        join base_241129.queries_compilation   using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        statement_name,
        start_ts,

        min( query_queue_duration ) over ( partition by statement_name, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by statement_name, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by statement_name, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by statement_name, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by statement_name, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by statement_name, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by statement_name, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by statement_name, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by statement_name, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by statement_name, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by statement_name, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by statement_name, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by statement_name, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    statement_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_minutes
    left join cte_queries_stats using ( start_ts )
  group by
    statement_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_statements_by_minute
as

  with
    cte_hr_formatting as
    (
      select
        statement_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_statements_by_minute
    )

  select
    statement_name::varchar as stmt,
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    statement_name,
    start_ts;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_all_by_second
as
  with
    cte_aggr as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_aggr_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_aggr_duration,
        max( duration )::interval                          as max_aggr_duration,
        median( extract(epoch from duration) )::float4     as med_aggr_duration,
        avg( duration )::interval                          as avg_aggr_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_aggr_duration,
        count(*)                                           as number_aggr_steps
      from
        cte_aggr
      group by
        node_type,
        start_ts
    ),

    cte_bcast as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_bcast_duration,
        max( duration )::interval                          as max_bcast_duration,
        median( extract(epoch from duration) )::float4     as med_bcast_duration,
        avg( duration )::interval                          as avg_bcast_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_bcast_duration,
        count(*)                                           as number_bcast_steps
      from
        cte_bcast
      group by
        node_type,
        start_ts
    ),

    cte_delete as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_delete_duration,
        max( duration )::interval                          as max_delete_duration,
        median( extract(epoch from duration) )::float4     as med_delete_duration,
        avg( duration )::interval                          as avg_delete_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_delete_duration,
        count(*)                                           as number_delete_steps
      from
        cte_delete
      group by
        node_type,
        start_ts
    ),

    cte_dist as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_dist_duration,
        max( duration )::interval                          as max_dist_duration,
        median( extract(epoch from duration) )::float4     as med_dist_duration,
        avg( duration )::interval                          as avg_dist_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_dist_duration,
        count(*)                                           as number_dist_steps
      from
        cte_dist
      group by
        node_type,
        start_ts
    ),

    cte_hash as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hash_duration,
        max( duration )::interval                          as max_hash_duration,
        median( extract(epoch from duration) )::float4     as med_hash_duration,
        avg( duration )::interval                          as avg_hash_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hash_duration,
        count(*)                                           as number_hash_steps
      from
        cte_hash
      group by
        node_type,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_hashjoin_duration,
        max( duration )::interval                          as max_hashjoin_duration,
        median( extract(epoch from duration) )::float4     as med_hashjoin_duration,
        avg( duration )::interval                          as avg_hashjoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_hashjoin_duration,
        count(*)                                           as number_hashjoin_steps
      from
        cte_hashjoin
      group by
        node_type,
        start_ts
    ),

    cte_insert as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_insert_duration,
        max( duration )::interval                          as max_insert_duration,
        median( extract(epoch from duration) )::float4     as med_insert_duration,
        avg( duration )::interval                          as avg_insert_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_insert_duration,
        count(*)                                           as number_insert_steps
      from
        cte_insert
      group by
        node_type,
        start_ts
    ),

    cte_limit as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_limit_duration,
        max( duration )::interval                          as max_limit_duration,
        median( extract(epoch from duration) )::float4     as med_limit_duration,
        avg( duration )::interval                          as avg_limit_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_limit_duration,
        count(*)                                           as number_limit_steps
      from
        cte_limit
      group by
        node_type,
        start_ts
    ),

    cte_merge as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_merge_duration,
        max( duration )::interval                          as max_merge_duration,
        median( extract(epoch from duration) )::float4     as med_merge_duration,
        avg( duration )::interval                          as avg_merge_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_merge_duration,
        count(*)                                           as number_merge_steps
      from
        cte_merge
      group by
        node_type,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_mergejoin_duration,
        max( duration )::interval                          as max_mergejoin_duration,
        median( extract(epoch from duration) )::float4     as med_mergejoin_duration,
        avg( duration )::interval                          as avg_mergejoin_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_mergejoin_duration,
        count(*)                                           as number_mergejoin_steps
      from
        cte_mergejoin
      group by
        node_type,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_nestloop_duration,
        max( duration )::interval                          as max_nestloop_duration,
        median( extract(epoch from duration) )::float4     as med_nestloop_duration,
        avg( duration )::interval                          as avg_nestloop_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_nestloop_duration,
        count(*)                                           as number_nestloop_steps
      from
        cte_nestloop
      group by
        node_type,
        start_ts
    ),

    cte_parse as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_parse_duration,
        max( duration )::interval                          as max_parse_duration,
        median( extract(epoch from duration) )::float4     as med_parse_duration,
        avg( duration )::interval                          as avg_parse_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_parse_duration,
        count(*)                                           as number_parse_steps
      from
        cte_parse
      group by
        node_type,
        start_ts
    ),

    cte_project as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_project_duration,
        max( duration )::interval                          as max_project_duration,
        median( extract(epoch from duration) )::float4     as med_project_duration,
        avg( duration )::interval                          as avg_project_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_project_duration,
        count(*)                                           as number_project_steps
      from
        cte_project
      group by
        node_type,
        start_ts
    ),

    cte_return as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime)  as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_return_duration,
        max( duration )::interval                          as max_return_duration,
        median( extract(epoch from duration) )::float4     as med_return_duration,
        avg( duration )::interval                          as avg_return_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_return_duration,
        count(*)                                           as number_return_steps
      from
        cte_return
      group by
        node_type,
        start_ts
    ),

    cte_save as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_save_duration,
        max( duration )::interval                          as max_save_duration,
        median( extract(epoch from duration) )::float4     as med_save_duration,
        avg( duration )::interval                          as avg_save_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_save_duration,
        count(*)                                           as number_save_steps
      from
        cte_save
      group by
        node_type,
        start_ts
    ),

    cte_sort as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_sort_duration,
        max( duration )::interval                          as max_sort_duration,
        median( extract(epoch from duration) )::float4     as med_sort_duration,
        avg( duration )::interval                          as avg_sort_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_sort_duration,
        count(*)                                           as number_sort_steps
      from
        cte_sort
      group by
        node_type,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_spatial_join_duration,
        max( duration )::interval                          as max_spatial_join_duration,
        median( extract(epoch from duration) )::float4     as med_spatial_join_duration,
        avg( duration )::interval                          as avg_spatial_join_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_spatial_join_duration,
        count(*)                                           as number_spatial_join_steps
      from
        cte_spatial_join
      group by
        node_type,
        start_ts
    ),

    cte_unique as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_unique_duration,
        max( duration )::interval                          as max_unique_duration,
        median( extract(epoch from duration) )::float4     as med_unique_duration,
        avg( duration )::interval                          as avg_unique_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_unique_duration,
        count(*)                                           as number_unique_steps
      from
        cte_unique
      group by
        node_type,
        start_ts
    ),

    cte_window as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window_stats as
    (
      select
        node_type,
        start_ts,
        min( duration )::interval                          as min_window_duration,
        max( duration )::interval                          as max_window_duration,
        median( extract(epoch from duration) )::float4     as med_window_duration,
        avg( duration )::interval                          as avg_window_duration,
        stddev_pop( extract(epoch from duration) )::float4 as std_window_duration,
        count(*)                                           as number_window_steps
      from
        cte_window
      group by
        node_type,
        start_ts
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,
    base_241129.generate_seconds.start_ts::timestamp as start_ts,

    min_aggr_duration::interval,
    max_aggr_duration::interval,
    med_aggr_duration::float4,
    avg_aggr_duration::interval,
    std_aggr_duration::float4,
    number_aggr_steps::int8,

    min_bcast_duration::interval,
    max_bcast_duration::interval,
    med_bcast_duration::float4,
    avg_bcast_duration::interval,
    std_bcast_duration::float4,
    number_bcast_steps::int8,

    min_delete_duration::interval,
    max_delete_duration::interval,
    med_delete_duration::float4,
    avg_delete_duration::interval,
    std_delete_duration::float4,
    number_delete_steps::int8,

    min_dist_duration::interval,
    max_dist_duration::interval,
    med_dist_duration::float4,
    avg_dist_duration::interval,
    std_dist_duration::float4,
    number_dist_steps::int8,

    min_hash_duration::interval,
    max_hash_duration::interval,
    med_hash_duration::float4,
    avg_hash_duration::interval,
    std_hash_duration::float4,
    number_hash_steps::int8,

    min_hashjoin_duration::interval,
    max_hashjoin_duration::interval,
    med_hashjoin_duration::float4,
    avg_hashjoin_duration::interval,
    std_hashjoin_duration::float4,
    number_hashjoin_steps::int8,

    min_insert_duration::interval,
    max_insert_duration::interval,
    med_insert_duration::float4,
    avg_insert_duration::interval,
    std_insert_duration::float4,
    number_insert_steps::int8,

    min_limit_duration::interval,
    max_limit_duration::interval,
    med_limit_duration::float4,
    avg_limit_duration::interval,
    std_limit_duration::float4,
    number_limit_steps::int8,

    min_merge_duration::interval,
    max_merge_duration::interval,
    med_merge_duration::float4,
    avg_merge_duration::interval,
    std_merge_duration::float4,
    number_merge_steps::int8,

    min_mergejoin_duration::interval,
    max_mergejoin_duration::interval,
    med_mergejoin_duration::float4,
    avg_mergejoin_duration::interval,
    std_mergejoin_duration::float4,
    number_mergejoin_steps::int8,

    min_nestloop_duration::interval,
    max_nestloop_duration::interval,
    med_nestloop_duration::float4,
    avg_nestloop_duration::interval,
    std_nestloop_duration::float4,
    number_nestloop_steps::int8,

    min_parse_duration::interval,
    max_parse_duration::interval,
    med_parse_duration::float4,
    avg_parse_duration::interval,
    std_parse_duration::float4,
    number_parse_steps::int8,

    min_project_duration::interval,
    max_project_duration::interval,
    med_project_duration::float4,
    avg_project_duration::interval,
    std_project_duration::float4,
    number_project_steps::int8,

    min_return_duration::interval,
    max_return_duration::interval,
    med_return_duration::float4,
    avg_return_duration::interval,
    std_return_duration::float4,
    number_return_steps::int8,

    min_save_duration::interval,
    max_save_duration::interval,
    med_save_duration::float4,
    avg_save_duration::interval,
    std_save_duration::float4,
    number_save_steps::int8,

    min_sort_duration::interval,
    max_sort_duration::interval,
    med_sort_duration::float4,
    avg_sort_duration::interval,
    std_sort_duration::float4,
    number_sort_steps::int8,

    min_spatial_join_duration::interval,
    max_spatial_join_duration::interval,
    med_spatial_join_duration::float4,
    avg_spatial_join_duration::interval,
    std_spatial_join_duration::float4,
    number_spatial_join_steps::int8,

    min_unique_duration::interval,
    max_unique_duration::interval,
    med_unique_duration::float4,
    avg_unique_duration::interval,
    std_unique_duration::float4,
    number_unique_steps::int8,

    min_window_duration::interval,
    max_window_duration::interval,
    med_window_duration::float4,
    avg_window_duration::interval,
    std_window_duration::float4,
    number_window_steps::int8
  from
               cte_node_types
    cross join base_241129.generate_seconds
     left join cte_aggr_stats             using ( node_type, start_ts )
     left join cte_bcast_stats            using ( node_type, start_ts )
     left join cte_delete_stats           using ( node_type, start_ts )
     left join cte_dist_stats             using ( node_type, start_ts )
     left join cte_hash_stats             using ( node_type, start_ts )
     left join cte_hashjoin_stats         using ( node_type, start_ts )
     left join cte_insert_stats           using ( node_type, start_ts )
     left join cte_limit_stats            using ( node_type, start_ts )
     left join cte_merge_stats            using ( node_type, start_ts )
     left join cte_mergejoin_stats        using ( node_type, start_ts )
     left join cte_nestloop_stats         using ( node_type, start_ts )
     left join cte_parse_stats            using ( node_type, start_ts )
     left join cte_project_stats          using ( node_type, start_ts )
     left join cte_return_stats           using ( node_type, start_ts )
     left join cte_save_stats             using ( node_type, start_ts )
     left join cte_sort_stats             using ( node_type, start_ts )
     left join cte_spatial_join_stats     using ( node_type, start_ts )
     left join cte_unique_stats           using ( node_type, start_ts )
     left join cte_window_stats           using ( node_type, start_ts );


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_second_complete
as
  select
    node_type::varchar,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_second
  order by
    node_type,
    start_ts;



create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_all_complete
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_all
  order by
    schema_name,
    table_name,
    node_type;



create or replace view
  lf_241129.queries_totals_result_cache_for_all_by_second
as
  select
    date_trunc( 'second', query_start_ts )::timestamp                                                           as start_ts,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
    sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
    count(*)::int8                                                                                           as number_total,
    ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
  from
              base_241129.fixed_stl_query
    left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
  where
        query_abort_flag  = false
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    start_ts;


create or replace view
  sf_241129.queries_totals_result_cache_for_all_by_second
as
  select
    start_ts::timestamp,
    number_hit::int8  as hit,
    number_miss::int8 as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_all_by_second
  order by
    start_ts;


create or replace view
  lf_241129.transactions_stats_commits_by_hour
as
  with
    cte_transaction_queuing_stats as
    (
      select
        date_trunc( 'hour', transaction_batch_queue_start )                                         as start_ts,
        min( batch_leader_startwork - transaction_batch_queue_start )                              as min_transaction_batch_queue_start,
        max( batch_leader_startwork - transaction_batch_queue_start )                              as max_transaction_batch_queue_start,
        median( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) )     as med_transaction_batch_queue_start,
        avg( batch_leader_startwork - transaction_batch_queue_start )                              as avg_transaction_batch_queue_start,
        stddev_pop( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) ) as std_transaction_batch_queue_start,
        sum( batch_leader_startwork - transaction_batch_queue_start )                              as sum_transaction_batch_queue_start
      from
             base_241129.transactions_in_batches
        join lf_241129.batches using ( batch_id )
      group by
        start_ts
    ),

    cte_batch_stats as
    (
      select
        date_trunc( 'hour', batch_leader_startwork ) as start_ts,

        min( batch_queue_duration )                            over ( partition by start_ts ) as min_batch_queue_duration,
        max( batch_queue_duration )                            over ( partition by start_ts ) as max_batch_queue_duration,
        median( extract(epoch from batch_queue_duration) )     over ( partition by start_ts ) as med_batch_queue_duration,
        avg( batch_queue_duration )                            over ( partition by start_ts ) as avg_batch_queue_duration,
        stddev_pop( extract(epoch from batch_queue_duration) ) over ( partition by start_ts ) as std_batch_queue_duration,
        sum( batch_queue_duration )                            over ( partition by start_ts ) as sum_batch_queue_duration,

        min( batch_leader_duration )                            over ( partition by start_ts ) as min_batch_leader_duration,
        max( batch_leader_duration )                            over ( partition by start_ts ) as max_batch_leader_duration,
        median( extract(epoch from batch_leader_duration) )     over ( partition by start_ts ) as med_batch_leader_duration,
        avg( batch_leader_duration )                            over ( partition by start_ts ) as avg_batch_leader_duration,
        stddev_pop( extract(epoch from batch_leader_duration) ) over ( partition by start_ts ) as std_batch_leader_duration,
        sum( batch_leader_duration )                            over ( partition by start_ts ) as sum_batch_leader_duration,

        min( batch_worker_duration )                            over ( partition by start_ts ) as min_batch_worker_duration,
        max( batch_worker_duration )                            over ( partition by start_ts ) as max_batch_worker_duration,
        median( extract(epoch from batch_worker_duration) )     over ( partition by start_ts ) as med_batch_worker_duration,
        avg( batch_worker_duration )                            over ( partition by start_ts ) as avg_batch_worker_duration,
        stddev_pop( extract(epoch from batch_worker_duration) ) over ( partition by start_ts ) as std_batch_worker_duration,
        sum( batch_worker_duration )                            over ( partition by start_ts ) as sum_batch_worker_duration,

        min( number_batch_transactions )         over ( partition by start_ts ) as min_number_batch_transactions,
        max( number_batch_transactions )         over ( partition by start_ts ) as max_number_batch_transactions,
        median( number_batch_transactions )      over ( partition by start_ts ) as med_number_batch_transactions,
        avg( number_batch_transactions::float4 ) over ( partition by start_ts ) as avg_number_batch_transactions,
        stddev_pop( number_batch_transactions )  over ( partition by start_ts ) as std_number_batch_transactions,
        sum( number_batch_transactions )         over ( partition by start_ts ) as sum_number_batch_transactions,

        min( number_batch_blocks )         over ( partition by start_ts ) as min_number_batch_blocks,
        max( number_batch_blocks )         over ( partition by start_ts ) as max_number_batch_blocks,
        median( number_batch_blocks )      over ( partition by start_ts ) as med_number_batch_blocks,
        avg( number_batch_blocks::float4 ) over ( partition by start_ts ) as avg_number_batch_blocks,
        stddev_pop( number_batch_blocks )  over ( partition by start_ts ) as std_number_batch_blocks,
        sum( number_batch_blocks )         over ( partition by start_ts ) as sum_number_batch_blocks,

        count(*) over ( partition by start_ts ) as number_batches
      from
        lf_241129.batches
    )

  select
    cte_batch_stats.start_ts::timestamp,

    cte_transaction_queuing_stats.min_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start::interval,

    cte_batch_stats.min_batch_queue_duration::interval,
    cte_batch_stats.max_batch_queue_duration::interval,
    cte_batch_stats.med_batch_queue_duration::float4,
    cte_batch_stats.avg_batch_queue_duration::interval,
    cte_batch_stats.std_batch_queue_duration::float4,
    cte_batch_stats.sum_batch_queue_duration::interval,

    cte_batch_stats.min_batch_leader_duration::interval,
    cte_batch_stats.max_batch_leader_duration::interval,
    cte_batch_stats.med_batch_leader_duration::float4,
    cte_batch_stats.avg_batch_leader_duration::interval,
    cte_batch_stats.std_batch_leader_duration::float4,
    cte_batch_stats.sum_batch_leader_duration::interval,

    cte_batch_stats.min_batch_worker_duration::interval,
    cte_batch_stats.max_batch_worker_duration::interval,
    cte_batch_stats.med_batch_worker_duration::float4,
    cte_batch_stats.avg_batch_worker_duration::interval,
    cte_batch_stats.std_batch_worker_duration::float4,
    cte_batch_stats.sum_batch_worker_duration::interval,

    cte_batch_stats.min_number_batch_transactions::int8,
    cte_batch_stats.max_number_batch_transactions::int8,
    cte_batch_stats.med_number_batch_transactions::int8,
    cte_batch_stats.avg_number_batch_transactions::float4,
    cte_batch_stats.std_number_batch_transactions::float4,
    cte_batch_stats.sum_number_batch_transactions::int8,

    cte_batch_stats.min_number_batch_blocks::int8,
    cte_batch_stats.max_number_batch_blocks::int8,
    cte_batch_stats.med_number_batch_blocks::int8,
    cte_batch_stats.avg_number_batch_blocks::float4,
    cte_batch_stats.std_number_batch_blocks::float4,
    cte_batch_stats.sum_number_batch_blocks::int8,

    cte_batch_stats.number_batches::int8
  from
              cte_batch_stats
    left join cte_transaction_queuing_stats using ( start_ts )
  group by
    start_ts,
    cte_transaction_queuing_stats.min_transaction_batch_queue_start,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start,
    cte_batch_stats.min_batch_queue_duration,
    cte_batch_stats.max_batch_queue_duration,
    cte_batch_stats.med_batch_queue_duration,
    cte_batch_stats.avg_batch_queue_duration,
    cte_batch_stats.std_batch_queue_duration,
    cte_batch_stats.sum_batch_queue_duration,
    cte_batch_stats.min_batch_leader_duration,
    cte_batch_stats.max_batch_leader_duration,
    cte_batch_stats.med_batch_leader_duration,
    cte_batch_stats.avg_batch_leader_duration,
    cte_batch_stats.std_batch_leader_duration,
    cte_batch_stats.sum_batch_leader_duration,
    cte_batch_stats.min_batch_worker_duration,
    cte_batch_stats.max_batch_worker_duration,
    cte_batch_stats.med_batch_worker_duration,
    cte_batch_stats.avg_batch_worker_duration,
    cte_batch_stats.std_batch_worker_duration,
    cte_batch_stats.sum_batch_worker_duration,
    cte_batch_stats.min_number_batch_transactions,
    cte_batch_stats.max_number_batch_transactions,
    cte_batch_stats.med_number_batch_transactions,
    cte_batch_stats.avg_number_batch_transactions,
    cte_batch_stats.std_number_batch_transactions,
    cte_batch_stats.sum_number_batch_transactions,
    cte_batch_stats.min_number_batch_blocks,
    cte_batch_stats.max_number_batch_blocks,
    cte_batch_stats.med_number_batch_blocks,
    cte_batch_stats.avg_number_batch_blocks,
    cte_batch_stats.std_number_batch_blocks,
    cte_batch_stats.sum_number_batch_blocks,
    cte_batch_stats.number_batches;


create or replace view
  sf_241129.transactions_stats_commits_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_min_transaction_batch_queue_start,
        extract(epoch from max_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_max_transaction_batch_queue_start,
        med_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_med_transaction_batch_queue_start,
        extract(epoch from avg_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_avg_transaction_batch_queue_start,
        std_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_std_transaction_batch_queue_start,
        extract(epoch from sum_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_sum_transaction_batch_queue_start,

        extract(epoch from min_batch_queue_duration)::numeric(18,2)::varchar as hr_min_batch_queue_duration,
        extract(epoch from max_batch_queue_duration)::numeric(18,2)::varchar as hr_max_batch_queue_duration,
        med_batch_queue_duration::numeric(18,2)::varchar                     as hr_med_batch_queue_duration,
        extract(epoch from avg_batch_queue_duration)::numeric(18,2)::varchar as hr_avg_batch_queue_duration,
        std_batch_queue_duration::numeric(18,2)::varchar                     as hr_std_batch_queue_duration,
        extract(epoch from sum_batch_queue_duration)::numeric(18,2)::varchar as hr_sum_batch_queue_duration,

        extract(epoch from min_batch_leader_duration)::numeric(18,2)::varchar as hr_min_batch_leader_duration,
        extract(epoch from max_batch_leader_duration)::numeric(18,2)::varchar as hr_max_batch_leader_duration,
        med_batch_leader_duration::numeric(18,2)::varchar                     as hr_med_batch_leader_duration,
        extract(epoch from avg_batch_leader_duration)::numeric(18,2)::varchar as hr_avg_batch_leader_duration,
        std_batch_leader_duration::numeric(18,2)::varchar                     as hr_std_batch_leader_duration,
        extract(epoch from sum_batch_leader_duration)::numeric(18,2)::varchar as hr_sum_batch_leader_duration,

        extract(epoch from min_batch_worker_duration)::numeric(18,2)::varchar as hr_min_batch_worker_duration,
        extract(epoch from max_batch_worker_duration)::numeric(18,2)::varchar as hr_max_batch_worker_duration,
        med_batch_worker_duration::numeric(18,2)::varchar                     as hr_med_batch_worker_duration,
        extract(epoch from avg_batch_worker_duration)::numeric(18,2)::varchar as hr_avg_batch_worker_duration,
        std_batch_worker_duration::numeric(18,2)::varchar                     as hr_std_batch_worker_duration,
        extract(epoch from sum_batch_worker_duration)::numeric(18,2)::varchar as hr_sum_batch_worker_duration,

        min_number_batch_transactions::varchar                as hr_min_number_batch_transactions,
        max_number_batch_transactions::varchar                as hr_max_number_batch_transactions,
        med_number_batch_transactions::varchar                as hr_med_number_batch_transactions,
        avg_number_batch_transactions::numeric(18,2)::varchar as hr_avg_number_batch_transactions,
        std_number_batch_transactions::numeric(18,2)::varchar as hr_std_number_batch_transactions,
        sum_number_batch_transactions::varchar                as hr_sum_number_batch_transactions,

        base_241129.hr_number( min_number_batch_blocks ) as hr_min_number_batch_blocks,
        base_241129.hr_number( max_number_batch_blocks ) as hr_max_number_batch_blocks,
        base_241129.hr_number( med_number_batch_blocks ) as hr_med_number_batch_blocks,
        base_241129.hr_number( avg_number_batch_blocks ) as hr_avg_number_batch_blocks,
        base_241129.hr_number( std_number_batch_blocks ) as hr_std_number_batch_blocks,
        base_241129.hr_number( sum_number_batch_blocks ) as hr_sum_number_batch_blocks,

        number_batches,

        max( len(hr_min_transaction_batch_queue_start) ) over () as max_len_hr_min_transaction_batch_queue_start,
        max( len(hr_max_transaction_batch_queue_start) ) over () as max_len_hr_max_transaction_batch_queue_start,
        max( len(hr_med_transaction_batch_queue_start) ) over () as max_len_hr_med_transaction_batch_queue_start,
        max( len(hr_avg_transaction_batch_queue_start) ) over () as max_len_hr_avg_transaction_batch_queue_start,
        max( len(hr_std_transaction_batch_queue_start) ) over () as max_len_hr_std_transaction_batch_queue_start,
        max( len(hr_sum_transaction_batch_queue_start) ) over () as max_len_hr_sum_transaction_batch_queue_start,

        max( len(hr_min_batch_queue_duration) ) over () as max_len_hr_min_batch_queue_duration,
        max( len(hr_max_batch_queue_duration) ) over () as max_len_hr_max_batch_queue_duration,
        max( len(hr_med_batch_queue_duration) ) over () as max_len_hr_med_batch_queue_duration,
        max( len(hr_avg_batch_queue_duration) ) over () as max_len_hr_avg_batch_queue_duration,
        max( len(hr_std_batch_queue_duration) ) over () as max_len_hr_std_batch_queue_duration,
        max( len(hr_sum_batch_queue_duration) ) over () as max_len_hr_sum_batch_queue_duration,

        max( len(hr_min_batch_leader_duration) ) over () as max_len_hr_min_batch_leader_duration,
        max( len(hr_max_batch_leader_duration) ) over () as max_len_hr_max_batch_leader_duration,
        max( len(hr_med_batch_leader_duration) ) over () as max_len_hr_med_batch_leader_duration,
        max( len(hr_avg_batch_leader_duration) ) over () as max_len_hr_avg_batch_leader_duration,
        max( len(hr_std_batch_leader_duration) ) over () as max_len_hr_std_batch_leader_duration,
        max( len(hr_sum_batch_leader_duration) ) over () as max_len_hr_sum_batch_leader_duration,

        max( len(hr_min_batch_worker_duration) ) over () as max_len_hr_min_batch_worker_duration,
        max( len(hr_max_batch_worker_duration) ) over () as max_len_hr_max_batch_worker_duration,
        max( len(hr_med_batch_worker_duration) ) over () as max_len_hr_med_batch_worker_duration,
        max( len(hr_avg_batch_worker_duration) ) over () as max_len_hr_avg_batch_worker_duration,
        max( len(hr_std_batch_worker_duration) ) over () as max_len_hr_std_batch_worker_duration,
        max( len(hr_sum_batch_worker_duration) ) over () as max_len_hr_sum_batch_worker_duration,

        max( len(hr_min_number_batch_transactions) ) over () as max_len_hr_min_number_batch_transactions,
        max( len(hr_max_number_batch_transactions) ) over () as max_len_hr_max_number_batch_transactions,
        max( len(hr_med_number_batch_transactions) ) over () as max_len_hr_med_number_batch_transactions,
        max( len(hr_avg_number_batch_transactions) ) over () as max_len_hr_avg_number_batch_transactions,
        max( len(hr_std_number_batch_transactions) ) over () as max_len_hr_std_number_batch_transactions,
        max( len(hr_sum_number_batch_transactions) ) over () as max_len_hr_sum_number_batch_transactions,

        max( len(hr_min_number_batch_blocks) ) over () as max_len_hr_min_number_batch_blocks,
        max( len(hr_max_number_batch_blocks) ) over () as max_len_hr_max_number_batch_blocks,
        max( len(hr_med_number_batch_blocks) ) over () as max_len_hr_med_number_batch_blocks,
        max( len(hr_avg_number_batch_blocks) ) over () as max_len_hr_avg_number_batch_blocks,
        max( len(hr_std_number_batch_blocks) ) over () as max_len_hr_std_number_batch_blocks,
        max( len(hr_sum_number_batch_blocks) ) over () as max_len_hr_sum_number_batch_blocks
      from
        lf_241129.transactions_stats_commits_by_hour
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_transaction_batch_queue_start, max_len_hr_min_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_max_transaction_batch_queue_start, max_len_hr_max_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_med_transaction_batch_queue_start, max_len_hr_med_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_avg_transaction_batch_queue_start, max_len_hr_avg_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_std_transaction_batch_queue_start, max_len_hr_std_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_sum_transaction_batch_queue_start, max_len_hr_sum_transaction_batch_queue_start ) )::varchar              as "xid queuing",

    ( lpad( hr_min_batch_queue_duration, max_len_hr_min_batch_queue_duration ) || ' / ' ||
      lpad( hr_max_batch_queue_duration, max_len_hr_max_batch_queue_duration ) || ' / ' ||
      lpad( hr_med_batch_queue_duration, max_len_hr_med_batch_queue_duration ) || ' / ' ||
      lpad( hr_avg_batch_queue_duration, max_len_hr_avg_batch_queue_duration ) || ' / ' ||
      lpad( hr_std_batch_queue_duration, max_len_hr_std_batch_queue_duration ) || ' / ' ||
      lpad( hr_sum_batch_queue_duration, max_len_hr_sum_batch_queue_duration ) )::varchar          as "bid queuing",

    ( lpad( hr_min_batch_leader_duration, max_len_hr_min_batch_leader_duration ) || ' / ' ||
      lpad( hr_max_batch_leader_duration, max_len_hr_max_batch_leader_duration ) || ' / ' ||
      lpad( hr_med_batch_leader_duration, max_len_hr_med_batch_leader_duration ) || ' / ' ||
      lpad( hr_avg_batch_leader_duration, max_len_hr_avg_batch_leader_duration ) || ' / ' ||
      lpad( hr_std_batch_leader_duration, max_len_hr_std_batch_leader_duration ) || ' / ' ||
      lpad( hr_sum_batch_leader_duration, max_len_hr_sum_batch_leader_duration ) )::varchar          as "ldr duration",

    ( lpad( hr_min_batch_worker_duration, max_len_hr_min_batch_worker_duration ) || ' / ' ||
      lpad( hr_max_batch_worker_duration, max_len_hr_max_batch_worker_duration ) || ' / ' ||
      lpad( hr_med_batch_worker_duration, max_len_hr_med_batch_worker_duration ) || ' / ' ||
      lpad( hr_avg_batch_worker_duration, max_len_hr_avg_batch_worker_duration ) || ' / ' ||
      lpad( hr_std_batch_worker_duration, max_len_hr_std_batch_worker_duration ) || ' / ' ||
      lpad( hr_sum_batch_worker_duration, max_len_hr_sum_batch_worker_duration ) )::varchar          as "wrk duration",

    ( lpad( hr_min_number_batch_transactions, max_len_hr_min_number_batch_transactions ) || ' / ' ||
      lpad( hr_max_number_batch_transactions, max_len_hr_max_number_batch_transactions ) || ' / ' ||
      lpad( hr_med_number_batch_transactions, max_len_hr_med_number_batch_transactions ) || ' / ' ||
      lpad( hr_avg_number_batch_transactions, max_len_hr_avg_number_batch_transactions ) || ' / ' ||
      lpad( hr_std_number_batch_transactions, max_len_hr_std_number_batch_transactions ) || ' / ' ||
      lpad( hr_sum_number_batch_transactions, max_len_hr_sum_number_batch_transactions ) )::varchar  as "xids/bid",

    ( lpad( hr_min_number_batch_blocks, max_len_hr_min_number_batch_blocks ) || ' / ' ||
      lpad( hr_max_number_batch_blocks, max_len_hr_max_number_batch_blocks ) || ' / ' ||
      lpad( hr_med_number_batch_blocks, max_len_hr_med_number_batch_blocks ) || ' / ' ||
      lpad( hr_avg_number_batch_blocks, max_len_hr_avg_number_batch_blocks ) || ' / ' ||
      lpad( hr_std_number_batch_blocks, max_len_hr_std_number_batch_blocks ) || ' / ' ||
      lpad( hr_sum_number_batch_blocks, max_len_hr_sum_number_batch_blocks ) )::varchar              as "blks/bid",

    number_batches::int8                                                                                    as bids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.blocks_rows_totals_for_node
as
  with
    cte_blocks_and_rows_by_slice as
    (
      select
        slice                                                                                                          as slice_ordinal,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      where
        tbl != 0
      group by
        slice
    ),

    cte_blocks_and_rows_by_node as
    (
      select
        base_241129.nodes.node_ordinal,
        base_241129.nodes.number_partial_slices,
        base_241129.nodes.number_full_slices,
        base_241129.nodes.number_total_slices,
        sum( cte_blocks_and_rows_by_slice.number_sorted_blocks   ) as number_sorted_blocks,
        sum( cte_blocks_and_rows_by_slice.number_unsorted_blocks ) as number_unsorted_blocks,
        sum( cte_blocks_and_rows_by_slice.number_total_blocks    ) as number_total_blocks,
        sum( cte_blocks_and_rows_by_slice.number_sorted_rows     ) as number_sorted_rows,
        sum( cte_blocks_and_rows_by_slice.number_unsorted_rows   ) as number_unsorted_rows,
        sum( cte_blocks_and_rows_by_slice.number_total_rows      ) as number_total_rows
      from
             base_241129.fixed_stv_slices
        join cte_blocks_and_rows_by_slice using ( slice_ordinal )
        join base_241129.nodes                using ( node_ordinal )
      group by
        base_241129.nodes.node_ordinal,
        base_241129.nodes.number_partial_slices,
        base_241129.nodes.number_full_slices,
        base_241129.nodes.number_total_slices
    )

  select
    node_ordinal::int2,
    number_partial_slices::int2,
    number_full_slices::int2,
    number_total_slices::int2,
    number_sorted_blocks::int8,
    null::int8 as number_desorted_blocks,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    null::int8 as number_desorted_rows,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
    cte_blocks_and_rows_by_node;


create or replace view
  lf_241129.wqueues_totals_slot_utilization_by_minute
as
  with
    cte_events as
    (
      select
        queue_id,
        query_compile_exec_start_ts as event_ts,
        1                   as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'

      union all

      select
        queue_id,
        query_compile_exec_end_ts as event_ts,
        -1                as event_delta
      from
             base_241129.fixed_stl_wlm_query
        join base_241129.fixed_stl_query     using ( query_id ) -- MG2 : exclude queries serviced by result cache
      where
            query_abort_flag      = false
        and query_csc_flag        = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts         <= current_timestamp
        and query_start_ts       >= current_date - interval '6 day'
    ),

    cte_slot_durations as
    (
      select
        queue_id,
        sum( event_delta ) over ( partition by queue_id, start_ts order by event_ts rows between unbounded preceding and current row ) as slot_number,
        date_trunc( 'minute', event_ts )                                                                                             as start_ts,
        lead( event_ts   ) over ( partition by queue_id, start_ts order by event_ts ) - event_ts                                       as duration
      from
        cte_events
    ),

    cte_durations as
    (
      select
        queue_id,
        slot_number,
        start_ts,
        sum( duration ) as sum_duration
      from
        cte_slot_durations
      group by
        queue_id,
        slot_number,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    slot_number::int2,
    start_ts::timestamp,
    sum_duration::interval as duration
  from
              cte_durations
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  sf_241129.wqueues_totals_slot_utilization_by_minute
as
  select
    queue_name::varchar                           as qname,
    slot_number::int2                             as slot,
    start_ts::timestamp,
    extract( epoch from duration )::numeric(18,2) as dur
  from
    lf_241129.wqueues_totals_slot_utilization_by_minute
  order by
    queue_name,
    slot_number,
    start_ts;


create or replace view
  lf_241129.vacuum_totals_for_users_by_hour
as
  with
    cte_fixed_stl_vacuum as
    (
      select
        xid             as transaction_id,
        userid          as user_id,
        eventtime,
        status,
        lower( status ) as lcstatus
      from
        stl_vacuum
      where
            eventtime <= current_timestamp
        and eventtime >= current_date - interval '6 day'
    ),

    cte_durations as
    (
      select
        transaction_id,
        user_id,
        min( eventtime ) as min_eventtime,
        max( eventtime ) as max_eventtime
      from
        cte_fixed_stl_vacuum
      group by
        transaction_id,
        user_id
    ),

    cte_finished_vacuums as
    (
      select
        transaction_id
      from
        cte_fixed_stl_vacuum
      where
        0 != position( 'Finished' in status )
    ),

    cte_total_duration as
    (
      select
        user_id,
        date_trunc( 'hour', min_eventtime )   as start_ts,
        sum( max_eventtime - min_eventtime ) as total_duration
      from
                     cte_finished_vacuums
        natural join cte_durations
      group by
        user_id,
        start_ts
    ),

    cte_number_vacuums as
    (
      select
        user_id,
        date_trunc( 'hour', eventtime ) as start_ts,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_delete_skipped,

        sum( case when left(status,10) != '[VacuumBG]' and status = 'Started' then 1 else 0 end ) as number_manual_full,
        sum( case when left(status,10) != '[VacuumBG]' and status = 'Skipped' then 1 else 0 end ) as number_manual_full_skipped,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_recluster,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_reindex,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort,

        sum( case when     left(status,10) != '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_manual_sort_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0 != position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_delete_skipped,

        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Started' then 1 else 0 end ) as number_auto_full,
        sum( case when left(status,10)  = '[VacuumBG]' and substring(status from 12) = 'Skipped' then 1 else 0 end ) as number_auto_full_skipped,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0 != position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_recluster,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0 != position('reindex' in lcstatus)
                  and 0  = position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_reindex,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0  = position('skipped' in lcstatus) 
                  and 0 != position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort,

        sum( case when     left(status,10) = '[VacuumBG]'
                  and 0 != position('skipped' in lcstatus) 
                  and 0  = position('started' in lcstatus) 
                  and 0  = position('delete'  in lcstatus)
                  and 0  = position('range'   in lcstatus)
                  and 0  = position('reindex' in lcstatus)
                  and 0 != position('sort'    in lcstatus) then 1 else 0 end ) as number_auto_sort_skipped
      from
        cte_fixed_stl_vacuum
      group by
        user_id,
        start_ts
    )

  select
    cte_number_vacuums.user_id::int4,
    trim( pg_user.usename )::varchar as user_name,
    cte_number_vacuums.start_ts::timestamp as start_ts,

    number_manual_delete::int4,
    number_manual_delete_skipped::int4,
    number_manual_full::int4,
    number_manual_full_skipped::int4,
    number_manual_recluster::int4,
    number_manual_reindex::int4,
    number_manual_sort::int4,
    number_manual_sort_skipped::int4,
    number_auto_delete::int4,
    number_auto_delete_skipped::int4,
    number_auto_full::int4,
    number_auto_full_skipped::int4,
    number_auto_recluster::int4,
    number_auto_reindex::int4,
    number_auto_sort::int4,
    number_auto_sort_skipped::int4,

    ( number_manual_delete + number_manual_full + number_manual_recluster + number_manual_reindex + number_manual_sort + number_auto_delete + number_auto_full + number_auto_recluster + number_auto_reindex + number_auto_sort )::int4 as number_total_vacuum,
    total_duration::interval
  from
              cte_number_vacuums
    left join cte_total_duration using ( user_id, start_ts )
         join pg_user on pg_user.usesysid = cte_number_vacuums.user_id;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_schemas_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_schemas_by_hour
    )

  select
    schema_name::varchar as sschema,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_all_by_all
as
  with
    cte_aggr as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_bcast as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_delete as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_dist as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_hash as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_hashjoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_insert as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_limit as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_merge as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_mergejoin as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_nestloop as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_parse as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_project as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_return as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_save as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    -- scan

    cte_sort as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_spatial_join as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_unique as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_window as
    (
      select
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    node_type::varchar,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
              cte_node_types
    left join cte_aggr         using ( node_type )
    left join cte_bcast        using ( node_type )
    left join cte_delete       using ( node_type )
    left join cte_dist         using ( node_type )
    left join cte_hash         using ( node_type )
    left join cte_hashjoin     using ( node_type )
    left join cte_insert       using ( node_type )
    left join cte_limit        using ( node_type )
    left join cte_merge        using ( node_type )
    left join cte_mergejoin    using ( node_type )
    left join cte_nestloop     using ( node_type )
    left join cte_parse        using ( node_type )
    left join cte_project      using ( node_type )
    left join cte_return       using ( node_type )
    left join cte_save         using ( node_type )
    left join cte_sort         using ( node_type )
    left join cte_spatial_join using ( node_type )
    left join cte_unique       using ( node_type )
    left join cte_window       using ( node_type );


create or replace view
  sf_241129.analyze_totals_for_users_by_all
as
  with
    cte_hr_formatting as
    (
      select
        user_name,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_users_by_day
    )

  select
    user_name::varchar as uuser,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    user_name;


create or replace view
  lf_241129.ttables_creation_for_all_by_minute
as
  select
    date_trunc( 'minute', relcreationtime )::timestamp as start_ts,
    count( * )::int4                                as number_tables
  from
         pg_class_info
    join pg_namespace on pg_class_info.relnamespace = pg_namespace.oid
  where
        pg_class_info.relkind             = 'r'
    and pg_namespace.nspname         not in ( 'information_schema', 'pg_catalog', 'pg_internal', 'sf', 'lf', 'base' )
    and left(pg_namespace.nspname,8)     != 'pg_temp_'
  group by
    start_ts;


create or replace view
  sf_241129.ttables_creation_for_all_by_minute
as
  select
    start_ts::timestamp,
    number_tables::int4 as ttables
  from
    lf_241129.ttables_creation_for_all_by_minute
  order by
    start_ts;


create or replace view
  lf_241129.transactions_batches
as
  select
    user_id::int4,
    user_name::varchar,
    session_id::int4,

    transaction_id::int8,
    transaction_start_ts::timestamp,
    transaction_end_ts::timestamp,
    transaction_query_queue_duration::interval,
    transaction_query_compile_exec_duration::interval,
    transaction_query_total_duration::interval,
    transaction_idle_duration::interval,
    transaction_duration::interval,
    transaction_number_csc_queries::int4,
    transaction_number_rc_queries::int4,
    transaction_number_leader_queries::int4,
    transaction_number_worker_queries::int4,
    transaction_number_total_queries::int4,
    transaction_abort_flag::bool,

    batch_id::int8,
    base_241129.transactions_in_batches.transaction_batch_queue_start::timestamp,
    ( batch_startqueue - base_241129.transactions_in_batches.transaction_batch_queue_start )::interval as transaction_batch_queue_duration,
    batch_leader_startwork::timestamp as batch_start_ts,
    batch_endtime::timestamp as batch_end_ts,
    batch_leader_duration::interval,
    batch_worker_duration::interval,
    batch_queue_duration::interval,
    batch_exec_duration::interval,
    batch_total_duration::interval,
    number_batch_blocks::int8,
    number_batch_headers::int8,
    number_batch_transactions::int8
  from
              base_241129.transactions
    left join base_241129.fixed_pg_user           using ( user_id )
    left join base_241129.transactions_in_batches using ( transaction_id )
    left join lf_241129.batches                   using ( batch_id );


create or replace view
  sf_241129.transactions_batches
as
  select
    user_name::varchar   as uuser,
    session_id::int4     as pid,
    transaction_id::int8 as xid,
    batch_id::int8       as bid,

    transaction_start_ts::timestamp                                                            as xid_start_ts,
    extract( epoch from transaction_query_queue_duration        )::numeric(18,2)               as queue,
    extract( epoch from transaction_query_compile_exec_duration )::numeric(18,2)               as cexec,
    extract( epoch from transaction_idle_duration               )::numeric(18,2)               as idle,
    -- transaction_end_ts::timestamp                                                           as xid_end_ts,
    ( transaction_number_csc_queries    || '/' || transaction_number_rc_queries     )::varchar as "c/r",
    ( transaction_number_leader_queries || '/' || transaction_number_worker_queries )::varchar as "l/w",
    case when transaction_abort_flag = true then '✔' else '' end::varchar as a,

    extract( epoch from transaction_batch_queue_duration )::numeric(18,2)                      as cqueue,
    extract( epoch from batch_leader_duration )::numeric(18,2)                                 as ldr,
    extract( epoch from batch_worker_duration )::numeric(18,2)                                 as wrk,
    batch_end_ts::timestamp   as bid_end_ts,

    number_batch_blocks::int8                                                                  as blks,
    number_batch_transactions::int8                                                            as xids
  from
    lf_241129.transactions_batches
  order by
    transaction_start_ts;


create or replace view
  lf_241129.queries_totals_result_cache_for_users_by_day
as
  with
    cte_s1 as
    (
      select
        user_id,
        date_trunc( 'day', query_start_ts )                                                                      as start_ts,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
        count(*)                                                                                                 as number_total,
        ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_start_ts   >= current_date - interval '6 day'
        and query_end_ts     <= current_timestamp
      group by
        user_id,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    number_hit::int8,
    number_miss::int8,
    number_total::int8,
    percentage::float4
  from
              cte_s1
    left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  sf_241129.queries_totals_result_cache_for_users_by_day
as
  select
    user_name::varchar as uuser,
    start_ts::date,
    number_hit::int8    as hit,
    number_miss::int8   as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_users_by_day
  order by
    user_name,
    start_ts;


create or replace view
  base_241129.fixed_stv_wlm_qmr_config
as
  select
    trim( action          )::varchar as rule_action,
    trim( action_value    )::varchar as action_value,
    trim( metric_name     )::varchar as metric_name,
    trim( metric_operator )::varchar as metric_operator,
    metric_value::float8,
    trim( rule_name       )::varchar as rule_name,
    service_class::int2              as queue_id
  from
    stv_wlm_qmr_config;


create or replace view
  lf_241129.wqueues_qmr_rules
as
  select
    ( dense_rank() over ( order by rule_name asc ) )::int2 as rule_ordinal,
    rule_name::varchar,
    queue_id::int2,
    queue_name::varchar,
    rule_action::varchar,
    metric_name::varchar,
    metric_operator::varchar,
    metric_value::float8
  from
         base_241129.fixed_stv_wlm_qmr_config
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id );


create or replace view
  lf_241129.blocks_rows_totals_for_cluster
as
  with
    cte_slices_blocks_rows as
    (
      select
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      where
        tbl != 0
    ),

    cte_nodes_and_slices as
    (
      select
        count( distinct node )                                   as number_nodes,
        sum( case when stv_slices.type = 'C' then 1 else 0 end ) as number_partial_slices,
        sum( case when stv_slices.type = 'D' then 1 else 0 end ) as number_full_slices
      from
        stv_slices
    )

  select
    number_nodes::int2,
    number_partial_slices::int2,
    number_full_slices::int2,
    number_sorted_blocks::int8,
    null::int8 as number_desorted_blocks,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    null::int8 as number_desorted_rows,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
               cte_slices_blocks_rows
    cross join cte_nodes_and_slices;


create or replace view
  sf_241129.blocks_rows_totals_for_cluster
as
  with
    cte_hr_formatting as
    (
      select
        number_nodes,
        number_partial_slices,
        number_full_slices,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_desorted_blocks ) as hr_number_desorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_desorted_rows ) as hr_number_desorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_cluster
    )

  select
    number_nodes::int2                 as nodes,
    number_partial_slices::int2        as partial_slices,
    number_full_slices::varchar        as full_slices,

    hr_number_sorted_blocks::varchar   as sblks,
    hr_number_desorted_blocks::varchar as dsblks,
    hr_number_unsorted_blocks::varchar as ublks,
    hr_number_total_blocks::varchar    as tblks,
    hr_number_sorted_rows::varchar     as srows,
    hr_number_desorted_rows::varchar   as dsrows,
    hr_number_unsorted_rows::varchar   as urows,
    hr_number_total_rows::varchar      as trows
  from
    cte_hr_formatting;


create or replace view
  sf_241129.steps_totals_data_on_both_for_all_by_all
as
  select
    node_type::varchar as node,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_all_by_all
  order by
    node_type;


create or replace view
  sf_241129.vacuum_totals_for_tables_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_tables_by_minute
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_users_by_minute
as
  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    count(*)::int4                       as number_abort
  from
              base_241129.generate_minutes
    left join base_241129.fixed_stl_query    on date_trunc( 'minute', query_start_ts ) = base_241129.generate_minutes.start_ts
    left join base_241129.fixed_pg_user   using ( user_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    user_id,
    user_name,
    start_ts;



create or replace view
  sf_241129.steps_stats_durations_on_both_for_statements_by_minute
as
  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    step_type::varchar      as step,
    start_ts::timestamp,

    extract( epoch from min_duration )::numeric(18,2) as min,
    extract( epoch from max_duration )::numeric(18,2) as max,
    med_duration::numeric(18,2)                       as med,
    extract( epoch from avg_duration )::numeric(18,2) as avg,
    std_duration::numeric(18,2)                       as std,
    number_steps::int8                                as num
  from
    lf_241129.steps_stats_durations_on_both_for_statements_by_minute
  order by
    statement_name,
    node_type,
    step_type,
    start_ts;



create or replace view
  sf_241129.analyze_totals_for_database_by_day
as
  with
    cte_hr_formatting as
    (
      select
        database_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_database_by_day
    )

  select
    database_name::varchar as dbname,
    start_ts::date,

    ( hr_number_manual_full_analyze             || ' / ' ||
      hr_number_manual_predictatecolumn_analyze || ' / ' ||
      hr_number_manual_skipped_analyze          )::varchar as manual,

    ( hr_number_automatic_full_analyze             || ' / ' ||
      hr_number_automatic_predictatecolumn_analyze || ' / ' ||
      hr_number_automatic_skipped_analyze          )::varchar as automatic,

    ( hr_number_autoanalyze_full_analyze             || ' / ' ||
      hr_number_autoanalyze_predictatecolumn_analyze || ' / ' ||
      hr_number_autoanalyze_skipped_analyze          )::varchar as autoanalyze,

    ( hr_number_total_full_analyze             || ' / ' ||
      hr_number_total_predictatecolumn_analyze || ' / ' ||
      hr_number_total_skipped_analyze          )::varchar as total
  from
    cte_hr_formatting
  order by
    database_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_tables_by_minute
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'minute', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    start_ts::timestamp,
    cte_delete.sum_delete_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_scan.sum_scan_rows::int8
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_minutes
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_tables_by_minute
as
  select
    schema_name::varchar                       as sschema,
    table_name::varchar                        as ttable,
    start_ts::timestamp,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_tables_by_minute
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  sf_241129.queries_compilation_stream
as
  select
    session_id::int4                                             as pid,
    transaction_id::int8                                         as xid,
    query_id::int4                                               as qid,
    stream_ordinal::int2                                         as stream,
    stream_number_leader_node_cache_segments::int2               as lncache,
    stream_number_offload_segments::int2                         as offload,
    stream_number_segments::int2                                 as total,
    extract( epoch from stream_compile_duration )::numeric(18,6) as dur
  from
    lf_241129.queries_compilation_stream
  order by
    session_id,
    transaction_id,
    query_id,
    stream_ordinal;


create or replace view
  sf_241129.vacuum_totals_for_tables_by_second
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_tables_by_second
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace procedure
  ro.sp_grant_versionless_sf_schema_to_group( group_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'grant usage on schema sf to group ' || group_name;
      execute sql;

      sql := 'grant select on all tables in schema sf to group ' || group_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  sf_241129.objects_totals_for_user
as
  select
    user_name::varchar                     as uuser,
    number_databases::int2                 as dbs,
    number_schemas::int2                   as sschemas,
    number_tables::int4                    as ttables,
    number_table_columns::int4             as ttables_cols,
    number_normal_views::int4              as nviews,
    number_normal_view_columns::int4       as nviews_cols,
    number_late_binding_views::int4        as lviews,
    number_materialized_views::int4        as mviews,
    number_materialized_view_columns::int4 as mviews_cols,
    number_functions::int4                 as funcs,
    number_procedures::int4                as procs
  from
    lf_241129.objects_totals_for_user
  order by
    user_name;


create or replace view
  lf_241129.transactions_stats_commits_by_minute
as
  with
    cte_transaction_queuing_stats as
    (
      select
        date_trunc( 'minute', transaction_batch_queue_start )                                         as start_ts,
        min( batch_leader_startwork - transaction_batch_queue_start )                              as min_transaction_batch_queue_start,
        max( batch_leader_startwork - transaction_batch_queue_start )                              as max_transaction_batch_queue_start,
        median( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) )     as med_transaction_batch_queue_start,
        avg( batch_leader_startwork - transaction_batch_queue_start )                              as avg_transaction_batch_queue_start,
        stddev_pop( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) ) as std_transaction_batch_queue_start,
        sum( batch_leader_startwork - transaction_batch_queue_start )                              as sum_transaction_batch_queue_start
      from
             base_241129.transactions_in_batches
        join lf_241129.batches using ( batch_id )
      group by
        start_ts
    ),

    cte_batch_stats as
    (
      select
        date_trunc( 'minute', batch_leader_startwork ) as start_ts,

        min( batch_queue_duration )                            over ( partition by start_ts ) as min_batch_queue_duration,
        max( batch_queue_duration )                            over ( partition by start_ts ) as max_batch_queue_duration,
        median( extract(epoch from batch_queue_duration) )     over ( partition by start_ts ) as med_batch_queue_duration,
        avg( batch_queue_duration )                            over ( partition by start_ts ) as avg_batch_queue_duration,
        stddev_pop( extract(epoch from batch_queue_duration) ) over ( partition by start_ts ) as std_batch_queue_duration,
        sum( batch_queue_duration )                            over ( partition by start_ts ) as sum_batch_queue_duration,

        min( batch_leader_duration )                            over ( partition by start_ts ) as min_batch_leader_duration,
        max( batch_leader_duration )                            over ( partition by start_ts ) as max_batch_leader_duration,
        median( extract(epoch from batch_leader_duration) )     over ( partition by start_ts ) as med_batch_leader_duration,
        avg( batch_leader_duration )                            over ( partition by start_ts ) as avg_batch_leader_duration,
        stddev_pop( extract(epoch from batch_leader_duration) ) over ( partition by start_ts ) as std_batch_leader_duration,
        sum( batch_leader_duration )                            over ( partition by start_ts ) as sum_batch_leader_duration,

        min( batch_worker_duration )                            over ( partition by start_ts ) as min_batch_worker_duration,
        max( batch_worker_duration )                            over ( partition by start_ts ) as max_batch_worker_duration,
        median( extract(epoch from batch_worker_duration) )     over ( partition by start_ts ) as med_batch_worker_duration,
        avg( batch_worker_duration )                            over ( partition by start_ts ) as avg_batch_worker_duration,
        stddev_pop( extract(epoch from batch_worker_duration) ) over ( partition by start_ts ) as std_batch_worker_duration,
        sum( batch_worker_duration )                            over ( partition by start_ts ) as sum_batch_worker_duration,

        min( number_batch_transactions )         over ( partition by start_ts ) as min_number_batch_transactions,
        max( number_batch_transactions )         over ( partition by start_ts ) as max_number_batch_transactions,
        median( number_batch_transactions )      over ( partition by start_ts ) as med_number_batch_transactions,
        avg( number_batch_transactions::float4 ) over ( partition by start_ts ) as avg_number_batch_transactions,
        stddev_pop( number_batch_transactions )  over ( partition by start_ts ) as std_number_batch_transactions,
        sum( number_batch_transactions )         over ( partition by start_ts ) as sum_number_batch_transactions,

        min( number_batch_blocks )         over ( partition by start_ts ) as min_number_batch_blocks,
        max( number_batch_blocks )         over ( partition by start_ts ) as max_number_batch_blocks,
        median( number_batch_blocks )      over ( partition by start_ts ) as med_number_batch_blocks,
        avg( number_batch_blocks::float4 ) over ( partition by start_ts ) as avg_number_batch_blocks,
        stddev_pop( number_batch_blocks )  over ( partition by start_ts ) as std_number_batch_blocks,
        sum( number_batch_blocks )         over ( partition by start_ts ) as sum_number_batch_blocks,

        count(*) over ( partition by start_ts ) as number_batches
      from
        lf_241129.batches
    )

  select
    cte_batch_stats.start_ts::timestamp,

    cte_transaction_queuing_stats.min_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start::interval,

    cte_batch_stats.min_batch_queue_duration::interval,
    cte_batch_stats.max_batch_queue_duration::interval,
    cte_batch_stats.med_batch_queue_duration::float4,
    cte_batch_stats.avg_batch_queue_duration::interval,
    cte_batch_stats.std_batch_queue_duration::float4,
    cte_batch_stats.sum_batch_queue_duration::interval,

    cte_batch_stats.min_batch_leader_duration::interval,
    cte_batch_stats.max_batch_leader_duration::interval,
    cte_batch_stats.med_batch_leader_duration::float4,
    cte_batch_stats.avg_batch_leader_duration::interval,
    cte_batch_stats.std_batch_leader_duration::float4,
    cte_batch_stats.sum_batch_leader_duration::interval,

    cte_batch_stats.min_batch_worker_duration::interval,
    cte_batch_stats.max_batch_worker_duration::interval,
    cte_batch_stats.med_batch_worker_duration::float4,
    cte_batch_stats.avg_batch_worker_duration::interval,
    cte_batch_stats.std_batch_worker_duration::float4,
    cte_batch_stats.sum_batch_worker_duration::interval,

    cte_batch_stats.min_number_batch_transactions::int8,
    cte_batch_stats.max_number_batch_transactions::int8,
    cte_batch_stats.med_number_batch_transactions::int8,
    cte_batch_stats.avg_number_batch_transactions::float4,
    cte_batch_stats.std_number_batch_transactions::float4,
    cte_batch_stats.sum_number_batch_transactions::int8,

    cte_batch_stats.min_number_batch_blocks::int8,
    cte_batch_stats.max_number_batch_blocks::int8,
    cte_batch_stats.med_number_batch_blocks::int8,
    cte_batch_stats.avg_number_batch_blocks::float4,
    cte_batch_stats.std_number_batch_blocks::float4,
    cte_batch_stats.sum_number_batch_blocks::int8,

    cte_batch_stats.number_batches::int8
  from
              cte_batch_stats
    left join cte_transaction_queuing_stats using ( start_ts )
  group by
    start_ts,
    cte_transaction_queuing_stats.min_transaction_batch_queue_start,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start,
    cte_batch_stats.min_batch_queue_duration,
    cte_batch_stats.max_batch_queue_duration,
    cte_batch_stats.med_batch_queue_duration,
    cte_batch_stats.avg_batch_queue_duration,
    cte_batch_stats.std_batch_queue_duration,
    cte_batch_stats.sum_batch_queue_duration,
    cte_batch_stats.min_batch_leader_duration,
    cte_batch_stats.max_batch_leader_duration,
    cte_batch_stats.med_batch_leader_duration,
    cte_batch_stats.avg_batch_leader_duration,
    cte_batch_stats.std_batch_leader_duration,
    cte_batch_stats.sum_batch_leader_duration,
    cte_batch_stats.min_batch_worker_duration,
    cte_batch_stats.max_batch_worker_duration,
    cte_batch_stats.med_batch_worker_duration,
    cte_batch_stats.avg_batch_worker_duration,
    cte_batch_stats.std_batch_worker_duration,
    cte_batch_stats.sum_batch_worker_duration,
    cte_batch_stats.min_number_batch_transactions,
    cte_batch_stats.max_number_batch_transactions,
    cte_batch_stats.med_number_batch_transactions,
    cte_batch_stats.avg_number_batch_transactions,
    cte_batch_stats.std_number_batch_transactions,
    cte_batch_stats.sum_number_batch_transactions,
    cte_batch_stats.min_number_batch_blocks,
    cte_batch_stats.max_number_batch_blocks,
    cte_batch_stats.med_number_batch_blocks,
    cte_batch_stats.avg_number_batch_blocks,
    cte_batch_stats.std_number_batch_blocks,
    cte_batch_stats.sum_number_batch_blocks,
    cte_batch_stats.number_batches;


create or replace view
  sf_241129.transactions_stats_commits_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_min_transaction_batch_queue_start,
        extract(epoch from max_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_max_transaction_batch_queue_start,
        med_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_med_transaction_batch_queue_start,
        extract(epoch from avg_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_avg_transaction_batch_queue_start,
        std_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_std_transaction_batch_queue_start,
        extract(epoch from sum_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_sum_transaction_batch_queue_start,

        extract(epoch from min_batch_queue_duration)::numeric(18,2)::varchar as hr_min_batch_queue_duration,
        extract(epoch from max_batch_queue_duration)::numeric(18,2)::varchar as hr_max_batch_queue_duration,
        med_batch_queue_duration::numeric(18,2)::varchar                     as hr_med_batch_queue_duration,
        extract(epoch from avg_batch_queue_duration)::numeric(18,2)::varchar as hr_avg_batch_queue_duration,
        std_batch_queue_duration::numeric(18,2)::varchar                     as hr_std_batch_queue_duration,
        extract(epoch from sum_batch_queue_duration)::numeric(18,2)::varchar as hr_sum_batch_queue_duration,

        extract(epoch from min_batch_leader_duration)::numeric(18,2)::varchar as hr_min_batch_leader_duration,
        extract(epoch from max_batch_leader_duration)::numeric(18,2)::varchar as hr_max_batch_leader_duration,
        med_batch_leader_duration::numeric(18,2)::varchar                     as hr_med_batch_leader_duration,
        extract(epoch from avg_batch_leader_duration)::numeric(18,2)::varchar as hr_avg_batch_leader_duration,
        std_batch_leader_duration::numeric(18,2)::varchar                     as hr_std_batch_leader_duration,
        extract(epoch from sum_batch_leader_duration)::numeric(18,2)::varchar as hr_sum_batch_leader_duration,

        extract(epoch from min_batch_worker_duration)::numeric(18,2)::varchar as hr_min_batch_worker_duration,
        extract(epoch from max_batch_worker_duration)::numeric(18,2)::varchar as hr_max_batch_worker_duration,
        med_batch_worker_duration::numeric(18,2)::varchar                     as hr_med_batch_worker_duration,
        extract(epoch from avg_batch_worker_duration)::numeric(18,2)::varchar as hr_avg_batch_worker_duration,
        std_batch_worker_duration::numeric(18,2)::varchar                     as hr_std_batch_worker_duration,
        extract(epoch from sum_batch_worker_duration)::numeric(18,2)::varchar as hr_sum_batch_worker_duration,

        min_number_batch_transactions::varchar                as hr_min_number_batch_transactions,
        max_number_batch_transactions::varchar                as hr_max_number_batch_transactions,
        med_number_batch_transactions::varchar                as hr_med_number_batch_transactions,
        avg_number_batch_transactions::numeric(18,2)::varchar as hr_avg_number_batch_transactions,
        std_number_batch_transactions::numeric(18,2)::varchar as hr_std_number_batch_transactions,
        sum_number_batch_transactions::varchar                as hr_sum_number_batch_transactions,

        base_241129.hr_number( min_number_batch_blocks ) as hr_min_number_batch_blocks,
        base_241129.hr_number( max_number_batch_blocks ) as hr_max_number_batch_blocks,
        base_241129.hr_number( med_number_batch_blocks ) as hr_med_number_batch_blocks,
        base_241129.hr_number( avg_number_batch_blocks ) as hr_avg_number_batch_blocks,
        base_241129.hr_number( std_number_batch_blocks ) as hr_std_number_batch_blocks,
        base_241129.hr_number( sum_number_batch_blocks ) as hr_sum_number_batch_blocks,

        number_batches,

        max( len(hr_min_transaction_batch_queue_start) ) over () as max_len_hr_min_transaction_batch_queue_start,
        max( len(hr_max_transaction_batch_queue_start) ) over () as max_len_hr_max_transaction_batch_queue_start,
        max( len(hr_med_transaction_batch_queue_start) ) over () as max_len_hr_med_transaction_batch_queue_start,
        max( len(hr_avg_transaction_batch_queue_start) ) over () as max_len_hr_avg_transaction_batch_queue_start,
        max( len(hr_std_transaction_batch_queue_start) ) over () as max_len_hr_std_transaction_batch_queue_start,
        max( len(hr_sum_transaction_batch_queue_start) ) over () as max_len_hr_sum_transaction_batch_queue_start,

        max( len(hr_min_batch_queue_duration) ) over () as max_len_hr_min_batch_queue_duration,
        max( len(hr_max_batch_queue_duration) ) over () as max_len_hr_max_batch_queue_duration,
        max( len(hr_med_batch_queue_duration) ) over () as max_len_hr_med_batch_queue_duration,
        max( len(hr_avg_batch_queue_duration) ) over () as max_len_hr_avg_batch_queue_duration,
        max( len(hr_std_batch_queue_duration) ) over () as max_len_hr_std_batch_queue_duration,
        max( len(hr_sum_batch_queue_duration) ) over () as max_len_hr_sum_batch_queue_duration,

        max( len(hr_min_batch_leader_duration) ) over () as max_len_hr_min_batch_leader_duration,
        max( len(hr_max_batch_leader_duration) ) over () as max_len_hr_max_batch_leader_duration,
        max( len(hr_med_batch_leader_duration) ) over () as max_len_hr_med_batch_leader_duration,
        max( len(hr_avg_batch_leader_duration) ) over () as max_len_hr_avg_batch_leader_duration,
        max( len(hr_std_batch_leader_duration) ) over () as max_len_hr_std_batch_leader_duration,
        max( len(hr_sum_batch_leader_duration) ) over () as max_len_hr_sum_batch_leader_duration,

        max( len(hr_min_batch_worker_duration) ) over () as max_len_hr_min_batch_worker_duration,
        max( len(hr_max_batch_worker_duration) ) over () as max_len_hr_max_batch_worker_duration,
        max( len(hr_med_batch_worker_duration) ) over () as max_len_hr_med_batch_worker_duration,
        max( len(hr_avg_batch_worker_duration) ) over () as max_len_hr_avg_batch_worker_duration,
        max( len(hr_std_batch_worker_duration) ) over () as max_len_hr_std_batch_worker_duration,
        max( len(hr_sum_batch_worker_duration) ) over () as max_len_hr_sum_batch_worker_duration,

        max( len(hr_min_number_batch_transactions) ) over () as max_len_hr_min_number_batch_transactions,
        max( len(hr_max_number_batch_transactions) ) over () as max_len_hr_max_number_batch_transactions,
        max( len(hr_med_number_batch_transactions) ) over () as max_len_hr_med_number_batch_transactions,
        max( len(hr_avg_number_batch_transactions) ) over () as max_len_hr_avg_number_batch_transactions,
        max( len(hr_std_number_batch_transactions) ) over () as max_len_hr_std_number_batch_transactions,
        max( len(hr_sum_number_batch_transactions) ) over () as max_len_hr_sum_number_batch_transactions,

        max( len(hr_min_number_batch_blocks) ) over () as max_len_hr_min_number_batch_blocks,
        max( len(hr_max_number_batch_blocks) ) over () as max_len_hr_max_number_batch_blocks,
        max( len(hr_med_number_batch_blocks) ) over () as max_len_hr_med_number_batch_blocks,
        max( len(hr_avg_number_batch_blocks) ) over () as max_len_hr_avg_number_batch_blocks,
        max( len(hr_std_number_batch_blocks) ) over () as max_len_hr_std_number_batch_blocks,
        max( len(hr_sum_number_batch_blocks) ) over () as max_len_hr_sum_number_batch_blocks
      from
        lf_241129.transactions_stats_commits_by_minute
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_transaction_batch_queue_start, max_len_hr_min_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_max_transaction_batch_queue_start, max_len_hr_max_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_med_transaction_batch_queue_start, max_len_hr_med_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_avg_transaction_batch_queue_start, max_len_hr_avg_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_std_transaction_batch_queue_start, max_len_hr_std_transaction_batch_queue_start ) || ' / ' ||
      lpad( hr_sum_transaction_batch_queue_start, max_len_hr_sum_transaction_batch_queue_start ) )::varchar              as "xid queuing",

    ( lpad( hr_min_batch_queue_duration, max_len_hr_min_batch_queue_duration ) || ' / ' ||
      lpad( hr_max_batch_queue_duration, max_len_hr_max_batch_queue_duration ) || ' / ' ||
      lpad( hr_med_batch_queue_duration, max_len_hr_med_batch_queue_duration ) || ' / ' ||
      lpad( hr_avg_batch_queue_duration, max_len_hr_avg_batch_queue_duration ) || ' / ' ||
      lpad( hr_std_batch_queue_duration, max_len_hr_std_batch_queue_duration ) || ' / ' ||
      lpad( hr_sum_batch_queue_duration, max_len_hr_sum_batch_queue_duration ) )::varchar          as "bid queuing",

    ( lpad( hr_min_batch_leader_duration, max_len_hr_min_batch_leader_duration ) || ' / ' ||
      lpad( hr_max_batch_leader_duration, max_len_hr_max_batch_leader_duration ) || ' / ' ||
      lpad( hr_med_batch_leader_duration, max_len_hr_med_batch_leader_duration ) || ' / ' ||
      lpad( hr_avg_batch_leader_duration, max_len_hr_avg_batch_leader_duration ) || ' / ' ||
      lpad( hr_std_batch_leader_duration, max_len_hr_std_batch_leader_duration ) || ' / ' ||
      lpad( hr_sum_batch_leader_duration, max_len_hr_sum_batch_leader_duration ) )::varchar          as "ldr duration",

    ( lpad( hr_min_batch_worker_duration, max_len_hr_min_batch_worker_duration ) || ' / ' ||
      lpad( hr_max_batch_worker_duration, max_len_hr_max_batch_worker_duration ) || ' / ' ||
      lpad( hr_med_batch_worker_duration, max_len_hr_med_batch_worker_duration ) || ' / ' ||
      lpad( hr_avg_batch_worker_duration, max_len_hr_avg_batch_worker_duration ) || ' / ' ||
      lpad( hr_std_batch_worker_duration, max_len_hr_std_batch_worker_duration ) || ' / ' ||
      lpad( hr_sum_batch_worker_duration, max_len_hr_sum_batch_worker_duration ) )::varchar          as "wrk duration",

    ( lpad( hr_min_number_batch_transactions, max_len_hr_min_number_batch_transactions ) || ' / ' ||
      lpad( hr_max_number_batch_transactions, max_len_hr_max_number_batch_transactions ) || ' / ' ||
      lpad( hr_med_number_batch_transactions, max_len_hr_med_number_batch_transactions ) || ' / ' ||
      lpad( hr_avg_number_batch_transactions, max_len_hr_avg_number_batch_transactions ) || ' / ' ||
      lpad( hr_std_number_batch_transactions, max_len_hr_std_number_batch_transactions ) || ' / ' ||
      lpad( hr_sum_number_batch_transactions, max_len_hr_sum_number_batch_transactions ) )::varchar  as "xids/bid",

    ( lpad( hr_min_number_batch_blocks, max_len_hr_min_number_batch_blocks ) || ' / ' ||
      lpad( hr_max_number_batch_blocks, max_len_hr_max_number_batch_blocks ) || ' / ' ||
      lpad( hr_med_number_batch_blocks, max_len_hr_med_number_batch_blocks ) || ' / ' ||
      lpad( hr_avg_number_batch_blocks, max_len_hr_avg_number_batch_blocks ) || ' / ' ||
      lpad( hr_std_number_batch_blocks, max_len_hr_std_number_batch_blocks ) || ' / ' ||
      lpad( hr_sum_number_batch_blocks, max_len_hr_sum_number_batch_blocks ) )::varchar              as "blks/bid",

    number_batches::int8                                                                                    as bids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_slices_by_second
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'second', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    slice_ordinal::int2,
    start_ts::timestamp,

    cte_aggr.sum_aggr_on_disk_bytes::int8,
    cte_aggr.sum_aggr_in_memory_bytes::int8,
    cte_aggr.sum_aggr_on_disk_rows::int8,
    cte_aggr.sum_aggr_in_memory_rows::int8,
    cte_bcast.sum_bcast_bytes::int8,
    cte_bcast.sum_bcast_packets::int8,
    cte_bcast.sum_bcast_rows::int8,
    cte_delete.sum_delete_rows::int8,
    cte_dist.sum_dist_bytes::int8,
    cte_dist.sum_dist_packets::int8,
    cte_dist.sum_dist_rows::int8,
    cte_hash.sum_hash_on_disk_bytes::int8,
    cte_hash.sum_hash_in_memory_bytes::int8,
    cte_hash.sum_hash_on_disk_rows::int8,
    cte_hash.sum_hash_in_memory_rows::int8,
    cte_hashjoin.sum_hashjoin_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_limit.sum_limit_rows::int8,
    cte_merge.sum_merge_rows::int8,
    cte_mergejoin.sum_mergejoin_rows::int8,
    cte_nestloop.sum_nestloop_rows::int8,
    cte_parse.sum_parse_rows::int8,
    cte_project.sum_project_rows::int8,
    cte_return.sum_return_bytes::int8,
    cte_return.sum_return_packets::int8,
    cte_return.sum_return_rows::int8,
    cte_save.sum_save_on_disk_bytes::int8,
    cte_save.sum_save_in_memory_bytes::int8,
    cte_save.sum_save_on_disk_rows::int8,
    cte_save.sum_save_in_memory_rows::int8,
    cte_sort.sum_sort_on_disk_bytes::int8,
    cte_sort.sum_sort_in_memory_bytes::int8,
    cte_sort.sum_sort_on_disk_rows::int8,
    cte_sort.sum_sort_in_memory_rows::int8,
    cte_spatial_join.sum_spatial_join_rows::int8,
    cte_unique.sum_unique_on_disk_bytes::int8,
    cte_unique.sum_unique_in_memory_bytes::int8,
    cte_unique.sum_unique_on_disk_rows::int8,
    cte_unique.sum_unique_in_memory_rows::int8,
    cte_window.sum_window_rows::int8
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_seconds
     left join cte_aggr         using( start_ts, slice_ordinal )
     left join cte_bcast        using( start_ts, slice_ordinal )
     left join cte_delete       using( start_ts, slice_ordinal )
     left join cte_dist         using( start_ts, slice_ordinal )
     left join cte_hash         using( start_ts, slice_ordinal )
     left join cte_hashjoin     using( start_ts, slice_ordinal )
     left join cte_insert       using( start_ts, slice_ordinal )
     left join cte_limit        using( start_ts, slice_ordinal )
     left join cte_merge        using( start_ts, slice_ordinal )
     left join cte_mergejoin    using( start_ts, slice_ordinal )
     left join cte_nestloop     using( start_ts, slice_ordinal )
     left join cte_parse        using( start_ts, slice_ordinal )
     left join cte_project      using( start_ts, slice_ordinal )
     left join cte_return       using( start_ts, slice_ordinal )
     left join cte_save         using( start_ts, slice_ordinal )
     left join cte_sort         using( start_ts, slice_ordinal )
     left join cte_spatial_join using( start_ts, slice_ordinal )
     left join cte_unique       using( start_ts, slice_ordinal )
     left join cte_window       using( start_ts, slice_ordinal )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null;


create or replace view
  sf_241129.steps_totals_data_on_both_for_slices_by_second
as
  select
    slice_ordinal::int2 as slice,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_slices_by_second
  order by
    slice_ordinal,
    start_ts;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_all_by_all
as
  with
    cte_transactions as
    (
      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        transaction_id,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        transaction_id
    )

  select
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_all_by_all
as
  select
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_all_by_all;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_hour_concise
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_hour
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_day_concise
as
  select
    node_type::varchar as node,
    start_ts::date,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_day
  order by
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_all_by_second
as
  select
    case
      when slice_ordinal  = -1 then 'leader'
      when slice_ordinal != -1 then 'worker'
    end::varchar as node_type,

    date_trunc( 'second', scan_start_ts )::timestamp as start_ts,

    sum( case when type =  1 then rows  else 0 end )::int8 as sum_type_1_rows,
    sum( case when type =  1 then bytes else 0 end )::int8 as sum_type_1_bytes,

    sum( case when type =  2 then rows_pre_filter      else 0 end )::int8 as sum_type_2m_rows,
    sum( case when type =  2 then rows_pre_user_filter else 0 end )::int8 as sum_type_2e_rows,
    sum( case when type =  2 then rows                 else 0 end )::int8 as sum_type_2_rows,
    sum( case when type =  2 then bytes                else 0 end )::int8 as sum_type_2_bytes,

    sum( case when type =  3 then rows  else 0 end )::int8 as sum_type_3_rows,
    sum( case when type =  3 then bytes else 0 end )::int8 as sum_type_3_bytes,

    sum( case when type =  9 then rows  else 0 end )::int8 as sum_type_9_rows,
    sum( case when type =  9 then bytes else 0 end )::int8 as sum_type_9_bytes,

    sum( case when type = 16 then rows  else 0 end )::int8 as sum_type_16_rows,
    sum( case when type = 16 then bytes else 0 end )::int8 as sum_type_16_bytes,

    sum( case when type = 17 then rows  else 0 end )::int8 as sum_type_17_rows,
    sum( case when type = 17 then bytes else 0 end )::int8 as sum_type_17_bytes,

    sum( case when type = 18 then rows  else 0 end )::int8 as sum_type_18_rows,
    sum( case when type = 18 then bytes else 0 end )::int8 as sum_type_18_bytes,

    sum( case when type = 21 then rows  else 0 end )::int8 as sum_type_21_rows,
    sum( case when type = 21 then bytes else 0 end )::int8 as sum_type_21_bytes,

    sum( case when type = 22 then rows  else 0 end )::int8 as sum_type_22_rows,
    sum( case when type = 22 then bytes else 0 end )::int8 as sum_type_22_bytes,

    sum( case when type = 23 then rows  else 0 end )::int8 as sum_type_23_rows,
    sum( case when type = 23 then bytes else 0 end )::int8 as sum_type_23_bytes,

    sum( case when type = 24 then rows  else 0 end )::int8 as sum_type_24_rows,
    sum( case when type = 24 then bytes else 0 end )::int8 as sum_type_24_bytes,

    sum( case when type = 25 then rows  else 0 end )::int8 as sum_type_25_rows,
    sum( case when type = 25 then bytes else 0 end )::int8 as sum_type_25_bytes,

    sum( case when type = 28 then rows  else 0 end )::int8 as sum_type_28_rows,
    sum( case when type = 28 then bytes else 0 end )::int8 as sum_type_28_bytes,

    sum( case when type = 29 then rows_pre_filter      else 0 end )::int8 as sum_type_29m_rows,
    sum( case when type = 29 then rows_pre_user_filter else 0 end )::int8 as sum_type_29e_rows,
    sum( case when type = 29 then rows                 else 0 end )::int8 as sum_type_29_rows,
    sum( case when type = 29 then bytes                else 0 end )::int8 as sum_type_29_bytes,

    sum( case when type = 30 then rows  else 0 end )::int8 as sum_type_30_rows,
    sum( case when type = 30 then bytes else 0 end )::int8 as sum_type_30_bytes,

    sum( case when type = 32 then rows  else 0 end )::int8 as sum_type_32_rows,
    sum( case when type = 32 then bytes else 0 end )::int8 as sum_type_32_bytes,

    sum( case when type = 33 then rows  else 0 end )::int8 as sum_type_33_rows,
    sum( case when type = 33 then bytes else 0 end )::int8 as sum_type_33_bytes,

    sum( case when type = 35 then rows  else 0 end )::int8 as sum_type_35_rows,
    sum( case when type = 35 then bytes else 0 end )::int8 as sum_type_35_bytes,

    sum( case when type = 36 then rows  else 0 end )::int8 as sum_type_36_rows,
    sum( case when type = 36 then bytes else 0 end )::int8 as sum_type_36_bytes,

    sum( case when type = 37 then rows  else 0 end )::int8 as sum_type_37_rows,
    sum( case when type = 37 then bytes else 0 end )::int8 as sum_type_37_bytes
  from
    base_241129.fixed_stl_scan
  where
        scan_start_ts >= current_date - interval '6 day'
    and scan_end_ts   <= current_timestamp
  group by
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_all_by_second
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_all_by_second
    )

  select
    node_type::varchar                                                                                                                     as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    start_ts;



create or replace view
  sf_241129.blocks_rows_totals_for_node
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        number_partial_slices,
        number_full_slices,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_desorted_blocks ) as hr_number_desorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_desorted_rows ) as hr_number_desorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_desorted_blocks) ) over () as max_len_hr_number_desorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks,

        max( len(hr_number_sorted_rows)   ) over () as max_len_hr_number_sorted_rows,
        max( len(hr_number_desorted_rows) ) over () as max_len_hr_number_desorted_rows,
        max( len(hr_number_unsorted_rows) ) over () as max_len_hr_number_unsorted_rows,
        max( len(hr_number_total_rows)    ) over () as max_len_hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_node
    )

  select
    node_ordinal::int2                                                 as node,
    ( number_partial_slices || ' / ' || number_full_slices )::varchar  as slices,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_desorted_blocks, max_len_hr_number_desorted_blocks )::varchar  as dsblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks,

    lpad( hr_number_sorted_rows,   max_len_hr_number_sorted_rows   )::varchar      as srows,
    lpad( hr_number_desorted_rows, max_len_hr_number_desorted_rows )::varchar      as dsrows,
    lpad( hr_number_unsorted_rows, max_len_hr_number_unsorted_rows )::varchar      as urows,
    lpad( hr_number_total_rows,    max_len_hr_number_total_rows    )::varchar      as trows
  from
    cte_hr_formatting
  order by
    node_ordinal;


create or replace view
  sf_241129.views_normal
as
  select
    schema_name::varchar as sschema,
    view_name::varchar   as vview,
    user_name::varchar   as uuser
  from
    lf_241129.views_normal
  where
    user_name != 'rdsdb'
  order by
    schema_name,
    view_name;


create or replace procedure
  ro.sp_grant_version_to_user( version in varchar(6), user_name in varchar(127) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'grant usage on schema base_' || version || ' to ' || user_name;
      execute sql;

      sql := 'grant usage on schema lf_' || version || ' to ' || user_name;
      execute sql;

      sql := 'grant usage on schema sf_' || version || ' to ' || user_name;
      execute sql;

      sql := 'grant select on all tables in schema base_' || version || ' to ' || user_name;
      execute sql;

      sql := 'grant select on all tables in schema lf_' || version || ' to ' || user_name;
      execute sql;

      sql := 'grant select on all tables in schema sf_' || version || ' to ' || user_name;
      execute sql;

      sql := 'grant execute on all functions  in schema base_' || version || ' to ' || user_name;
      execute sql;

      sql := 'grant execute on all procedures in schema lf_' || version || ' to ' || user_name;
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_users_by_minute
as
  with
    cte_s1 as
    (
      select
        user_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'minute', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        user_id,
        node_type,
        start_ts
    )

  select
    user_id::int4,
    node_type::varchar,
    trim( pg_user.usename )::varchar as user_name,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_user on cte_s1.user_id = pg_user.usesysid;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_tables_by_second
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'second', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    base_241129.fixed_pg_class_for_tables.table_id::int4,
    base_241129.fixed_pg_class_for_tables.table_name::varchar,
    start_ts::timestamp,
    cte_delete.sum_delete_rows::int8,
    cte_insert.sum_insert_rows::int8,
    cte_scan.sum_scan_rows::int8
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_seconds
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null;


create or replace view
  lf_241129.steps_stats_durations_on_both_for_statements_by_second
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'aggr' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_bcast as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'bcast' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_delete as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'delete' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_dist as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'dist' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hash as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hash' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'hashjoin' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'insert' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'limit' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'merge' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'mergejoin' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'nestloop_op' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'parse' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'project' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'return' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'save' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'sort' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'spatial_join' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'unique' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        'window' as step_type,
        date_trunc( 'second', starttime ) as start_ts,
        max(endtime) - min(starttime) as duration
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type,
        step_type,
        start_ts,
        query,
        segment,
        step
    ),

    cte_steps as
    (
      select * from cte_aggr         union all
      select * from cte_bcast        union all
      select * from cte_delete       union all
      select * from cte_dist         union all
      select * from cte_hash         union all
      select * from cte_hashjoin     union all
      select * from cte_insert       union all
      select * from cte_limit        union all
      select * from cte_merge        union all
      select * from cte_mergejoin    union all
      select * from cte_nestloop     union all
      select * from cte_parse        union all
      select * from cte_project      union all
      select * from cte_return       union all
      select * from cte_save         union all
      select * from cte_sort         union all
      select * from cte_spatial_join union all
      select * from cte_unique       union all
      select * from cte_window
    )

  select
    statement_name::varchar,
    node_type::varchar,
    step_type::varchar,
    start_ts::timestamp,

    min( duration )::interval                          as min_duration,
    max( duration )::interval                          as max_duration,
    median( extract(epoch from duration) )::float4     as med_duration,
    avg( duration )::interval                          as avg_duration,
    stddev_pop( extract(epoch from duration) )::float4 as std_duration,
    count(*)::int8                                     as number_steps
  from
                 cte_base
    natural join cte_steps
  group by
    statement_name,
    node_type,
    step_type,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_statements_by_second
as
  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    step_type::varchar      as step,
    start_ts::timestamp,

    extract( epoch from min_duration )::numeric(18,2) as min,
    extract( epoch from max_duration )::numeric(18,2) as max,
    med_duration::numeric(18,2)                       as med,
    extract( epoch from avg_duration )::numeric(18,2) as avg,
    std_duration::numeric(18,2)                       as std,
    number_steps::int8                                as num
  from
    lf_241129.steps_stats_durations_on_both_for_statements_by_second
  order by
    statement_name,
    node_type,
    step_type,
    start_ts;



create or replace view
  lf_241129.objects_dependencies
as
  -- MG2 : so, with regard to naming, children depend on parents (as in real life)

  with recursive
    flat_dependencies as
    (
      select
        case
          when pg_class_parent_type.relname = 'pg_class'                                                                then pg_class_parent_class.relowner
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then pg_namespace_parent_namespace.nspowner
          when pg_class_parent_type.relname = 'pg_proc'                                                                 then pg_procparent_proc.proowner
          when pg_class_parent_type.relname = 'pg_language'                                                             then null
        end                                        as parent_owner_id,
        case
          when pg_class_parent_type.relname = 'pg_class'                                                                then trim(pg_user_parent_class.usename)
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then trim(pg_user_parent_namespace.usename)
          when pg_class_parent_type.relname = 'pg_proc'                                                                 then trim(pg_user_parent_proc.usename)
          when pg_class_parent_type.relname = 'pg_language'                                                             then null
        end                                        as parent_owner_name,
        case
          when pg_class_parent_type.relname = 'pg_class'                                                                then pg_class_parent_class.relnamespace
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then null
          when pg_class_parent_type.relname = 'pg_proc'                                                                 then pg_procparent_proc.pronamespace
          when pg_class_parent_type.relname = 'pg_language'                                                             then null
        end                                        as parent_schema_id,
        case
          when pg_class_parent_type.relname = 'pg_class'                                                                then trim(pg_namespace_parent_class.nspname)
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then null
          when pg_class_parent_type.relname = 'pg_proc'                                                                 then trim(pg_namespace_parent_proc.nspname)
          when pg_class_parent_type.relname = 'pg_language'                                                             then null
        end                                        as parent_schema_name,
        case
          when pg_class_parent_type.relname = 'pg_class'     and pg_class_parent_class.relkind = 'r'                    then 'table'
          when pg_class_parent_type.relname = 'pg_class'     and pg_class_parent_class.relkind = 'v'                    then 'view'
          when pg_class_parent_type.relname = 'pg_proc'      and pg_language_parent_proc.lanname  = 'plpgsql'           then 'procedure' -- MG2 : actually no procedure info in pg_depend
          when pg_class_parent_type.relname = 'pg_proc'      and pg_language_parent_proc.lanname != 'plpgsql'           then 'function'
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then 'schema'
          when pg_class_parent_type.relname = 'pg_language'                                                             then 'language'
                                                                                                                        else null
        end                                        as parent_object_type,
        case
          when pg_class_parent_type.relname = 'pg_class'                                                                then pg_class_parent_class.oid
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then pg_namespace_parent_namespace.oid
          when pg_class_parent_type.relname = 'pg_proc'                                                                 then pg_procparent_proc.oid
          when pg_class_parent_type.relname = 'pg_language'                                                             then pg_language_parent_language.oid
        end                                        as parent_object_id,
        case
          when pg_class_parent_type.relname = 'pg_class'                                                                then trim(pg_class_parent_class.relname)
          when pg_class_parent_type.relname = 'pg_namespace'                                                            then trim(pg_namespace_parent_namespace.nspname)
          when pg_class_parent_type.relname = 'pg_proc'                                                                 then trim(pg_procparent_proc.proname)
          when pg_class_parent_type.relname = 'pg_language'                                                             then trim(pg_language_parent_language.lanname)
        end                                        as parent_object_name,

        case
          when pg_class_child_type.relname = 'pg_class'                                                                 then pg_class_child_class_table_or_view.relowner
          when pg_class_child_type.relname = 'pg_rewrite'                                                               then pg_class_child_class_view.relowner
          when pg_class_child_type.relname = 'pg_namespace'                                                             then null
          when pg_class_child_type.relname = 'pg_proc'                                                                  then pg_procparent_proc.proowner
          when pg_class_child_type.relname = 'pg_language'                                                              then null
        end                                        as child_owner_id,
        case
          when pg_class_child_type.relname = 'pg_class'                                                                 then trim(pg_user_child_class_table_or_view.usename)
          when pg_class_child_type.relname = 'pg_rewrite'                                                               then trim(pg_user_child_class_view.usename)
          when pg_class_child_type.relname = 'pg_namespace'                                                             then null
          when pg_class_child_type.relname = 'pg_proc'                                                                  then trim(pg_user_parent_proc.usename)
          when pg_class_child_type.relname = 'pg_language'                                                              then null
        end                                        as child_owner_name,
        case
          when pg_class_child_type.relname = 'pg_class'                                                                 then pg_class_child_class_table_or_view.relnamespace
          when pg_class_child_type.relname = 'pg_rewrite'                                                               then pg_class_child_class_view.relnamespace
          when pg_class_child_type.relname = 'pg_namespace'                                                             then null
          when pg_class_child_type.relname = 'pg_proc'                                                                  then pg_procchild_proc.pronamespace
          when pg_class_child_type.relname = 'pg_language'                                                              then null
        end                                        as child_schema_id,
        case
          when pg_class_child_type.relname = 'pg_class'                                                                 then trim(pg_namespace_child_class_table_or_view.nspname)
          when pg_class_child_type.relname = 'pg_rewrite'                                                               then trim(pg_namespace_child_class_view.nspname)
          when pg_class_child_type.relname = 'pg_namespace'                                                             then null
          when pg_class_child_type.relname = 'pg_proc'                                                                  then trim(pg_namespace_child_proc.nspname)
          when pg_class_child_type.relname = 'pg_language'                                                              then null
        end                                        as child_schema_name,
        case
          when pg_class_child_type.relname = 'pg_class'     and pg_class_child_class_table_or_view.relkind  = 'r'       then 'table'     -- MG2 : unnecessary; tables can't be children
          when pg_class_child_type.relname = 'pg_class'     and pg_class_child_class_table_or_view.relkind  = 'v'       then 'view'      -- MG2 : when the parent is not a table or view (i.e. is a namespace, and maybe also a function)
          when pg_class_child_type.relname = 'pg_rewrite'   and pg_class_child_class_view.relkind           = 'v'       then 'view'      -- MG2 : note "pg_rewrite" is correct, and different to parents
          when pg_class_child_type.relname = 'pg_proc'      and pg_language_child_proc.lanname              = 'plpgsql' then 'procedure' -- MG2 : actually no procedure info in pg_depend
          when pg_class_child_type.relname = 'pg_proc'      and pg_language_child_proc.lanname             != 'plpgsql' then 'function'
          when pg_class_child_type.relname = 'pg_namespace'                                                             then 'schema'    -- MG2 : unnecessary; pg_depends does not record namespaces being owned by users, we do that manually later
          when pg_class_child_type.relname = 'pg_language'                                                              then 'language'
                                                                                                                        else null
        end                                        as child_object_type,
        case
          when pg_class_child_type.relname = 'pg_class'                                                                 then pg_class_child_class_table_or_view.oid
          when pg_class_child_type.relname = 'pg_rewrite'                                                               then pg_class_child_class_view.oid
          when pg_class_child_type.relname = 'pg_namespace'                                                             then pg_namespace_child_namespace.oid
          when pg_class_child_type.relname = 'pg_proc'                                                                  then pg_procchild_proc.oid
          when pg_class_child_type.relname = 'pg_language'                                                              then pg_language_child_language.oid
        end                                        as child_object_id,

        case
          when pg_class_child_type.relname = 'pg_class'                                                                 then trim(pg_class_child_class_table_or_view.relname)
          when pg_class_child_type.relname = 'pg_rewrite'                                                               then trim(pg_class_child_class_view.relname)
          when pg_class_child_type.relname = 'pg_namespace'                                                             then trim(pg_namespace_child_namespace.nspname)
          when pg_class_child_type.relname = 'pg_proc'                                                                  then trim(pg_procchild_proc.proname)
          when pg_class_child_type.relname = 'pg_language'                                                              then trim(pg_language_child_language.lanname)
        end                                        as child_object_name

      from
             -- MG2 : distinct causes an assert; group by is a workaround
             --       normal dependency are the only type we care about

             ( select classid, objid, refclassid, refobjid from pg_depend where deptype = 'n' group by classid, objid, refclassid, refobjid ) as pg_depend_distinct

             -- MG2 : first, parents
             join pg_class      as pg_class_parent_type       on pg_class_parent_type.oid = pg_depend_distinct.refclassid

        -- MG2 : parents who are in pg_class (and by the where clause will be tables or views)
        left join pg_class      as pg_class_parent_class      on pg_class_parent_type.relname = 'pg_class'  and pg_depend_distinct.refobjid        = pg_class_parent_class.oid
        left join pg_namespace  as pg_namespace_parent_class  on pg_class_parent_type.relname = 'pg_class'  and pg_class_parent_class.relnamespace = pg_namespace_parent_class.oid
        left join ( select usesysid, usename from pg_user ) as pg_user_parent_class on pg_class_parent_type.relname = 'pg_class' and pg_class_parent_class.relowner = pg_user_parent_class.usesysid

        -- MG2 : parents who are in pg_proc (and by the where clause will be procedures or.functions)
        left join pg_proc       as pg_procparent_proc        on pg_class_parent_type.relname = 'pg_proc'   and pg_depend_distinct.refobjid        = pg_procparent_proc.oid
        left join pg_namespace  as pg_namespace_parent_proc   on pg_class_parent_type.relname = 'pg_proc'   and pg_procparent_proc.pronamespace   = pg_namespace_parent_proc.oid
        left join pg_language   as pg_language_parent_proc    on pg_class_parent_type.relname = 'pg_proc'   and pg_procparent_proc.prolang        = pg_language_parent_proc.oid
        left join ( select usesysid, usename from pg_user ) as pg_user_parent_proc on pg_class_parent_type.relname = 'pg_proc' and pg_procparent_proc.proowner = pg_user_parent_proc.usesysid

        -- MG2 : parents who are in pg_namespace (and will naturally enough be namespaces)
        left join pg_namespace  as pg_namespace_parent_namespace  on pg_class_parent_type.relname = 'pg_namespace' and pg_depend_distinct.refobjid       = pg_namespace_parent_namespace.oid
        left join ( select usesysid, usename from pg_user ) as pg_user_parent_namespace on pg_class_parent_type.relname = 'pg_namespace' and pg_user_parent_namespace.usesysid = pg_namespace_parent_namespace.nspowner

        -- MG2 : parents who are a language
        left join pg_language   as pg_language_parent_language on pg_class_parent_type.relname = 'pg_language'  and pg_depend_distinct.refobjid = pg_language_parent_language.oid

              -- MG2 : next, children
             join pg_class      as pg_class_child_type         on pg_class_child_type.oid = pg_depend_distinct.classid

        -- MG2 : children who are in pg_class (and which surprisingly can be both tables or views - turns out not all views go via pg_rewrite)
        left join pg_class      as pg_class_child_class_table_or_view      on pg_class_child_type.relname = 'pg_class' and pg_depend_distinct.objid                   = pg_class_child_class_table_or_view.oid
        left join pg_namespace  as pg_namespace_child_class_table_or_view  on pg_class_child_type.relname = 'pg_class' and pg_namespace_child_class_table_or_view.oid = pg_class_child_class_table_or_view.relnamespace
        left join ( select usesysid, usename from pg_user ) as pg_user_child_class_table_or_view on pg_class_child_type.relname = 'pg_class' and pg_user_child_class_table_or_view.usesysid = pg_class_child_class_table_or_view.relowner

        -- MG2 : children who are in pg_rewrite (and so are really in pg_class, and are views)
        left join pg_rewrite    as pg_rewrite_child_class          on pg_class_child_type.relname = 'pg_rewrite' and pg_rewrite_child_class.oid         = pg_depend_distinct.objid
        left join pg_class      as pg_class_child_class_view       on pg_class_child_type.relname = 'pg_rewrite' and pg_class_child_class_view.oid      = pg_rewrite_child_class.ev_class
        left join pg_namespace  as pg_namespace_child_class_view   on pg_class_child_type.relname = 'pg_rewrite' and pg_namespace_child_class_view.oid  = pg_class_child_class_view.relnamespace
        left join ( select usesysid, usename from pg_user ) as pg_user_child_class_view on pg_class_child_type.relname = 'pg_rewrite' and pg_class_child_class_view.relowner = pg_user_child_class_view.usesysid

        -- MG2 : children who are in pg_proc (and by the where clause will be procedures or.functions)
        left join pg_proc       as pg_procchild_proc         on pg_class_child_type.relname = 'pg_proc'    and pg_depend_distinct.objid           = pg_procchild_proc.oid
        left join pg_namespace  as pg_namespace_child_proc    on pg_class_child_type.relname = 'pg_proc'    and pg_procchild_proc.pronamespace    = pg_namespace_child_proc.oid
        left join pg_language   as pg_language_child_proc     on pg_class_child_type.relname = 'pg_proc'    and pg_procchild_proc.prolang         = pg_language_child_proc.oid
        left join ( select usesysid, usename from pg_user ) as pg_user_child_proc on pg_class_child_type.relname = 'pg_proc' and pg_procchild_proc.proowner = pg_user_child_proc.usesysid

        -- MG2 : children who are in pg_namespace (and will naturally enough be namespaces) (but with the missing info from pg_depend, namespaces are never children)
        left join pg_namespace  as pg_namespace_child_namespace  on pg_class_child_type.relname = 'pg_namespace' and pg_depend_distinct.objid = pg_namespace_child_namespace.oid

        -- MG2 : children who are a language (yes, it happens!  for the language invoking.functions)
        left join pg_language  as pg_language_child_language     on pg_class_child_type.relname = 'pg_language'  and pg_depend_distinct.objid = pg_language_child_language.oid

      where
        -- MG2 : ignore pg_type and pg_conversion
        pg_class_parent_type.relname not in ('pg_conversion', 'pg_type') and
        pg_class_child_type.relname  not in ('pg_conversion', 'pg_type') and

        -- MG2 : ignore all the (certainly seemingly) strange and (definitely) unexpected dependencies where a view depends upon itself (it does not in fact, at least not in the SQL of the view)
        not
        (
          pg_rewrite_child_class.ev_class is not null                                                  and
          pg_class_parent_type.relname     = 'pg_class'   and pg_class_parent_class.relkind      = 'v' and
          pg_class_child_type.relname      = 'pg_rewrite' and pg_class_child_class_view.relkind  = 'v' and
          pg_depend_distinct.refobjid      = pg_rewrite_child_class.ev_class
        )

      union all

      -- MG2 : tables and views depend on their owners, and this is not in pg_depends

      select
        null                          as parent_owner_id,
        null::varchar                 as parent_owner_name,
        null                          as parent_schema_id,
        null::varchar                 as parent_schema_name,
        'user'                        as parent_object_type,
        pg_user.usesysid              as parent_object_id,
        trim( pg_user.usename )       as parent_object_name,
        pg_class.relowner             as child_owner_id,
        trim( pg_user.usename )       as child_owner_name,
        pg_namespace.oid              as child_schema_id,
        trim( pg_namespace.nspname )  as child_schema_name,
        case
          when pg_class.relkind = 'r' then 'table'
          when pg_class.relkind = 'v' then 'view'
        end                           as child_object_type,
        pg_class.oid                  as child_object_id,
        trim( pg_class.relname )      as child_object_name
      from
        pg_class,
        pg_namespace,
        pg_user
      where
        pg_class.relnamespace =  pg_namespace.oid and
        pg_class.relowner     =  pg_user.usesysid and
        pg_class.relkind      in ('r', 'v')

      union all

      -- MG2 :.functions and procedures depend on their owners, and this is not in pg_depends

      select
        null                          as parent_owner_id,
        null::varchar                 as parent_owner_name,
        null                          as parent_schema_id,
        null::varchar                 as parent_schema_name,
        'user'                        as parent_object_type,
        pg_user.usesysid              as parent_object_id,
        trim( pg_user.usename )       as parent_object_name,
        pg_namespace.nspowner         as child_owner_id,
        trim( pg_user.usename )       as child_owner_name,
        pg_namespace.oid              as child_schema_id,
        trim( pg_namespace.nspname )  as child_schema_name,
        case
          when pg_language.lanname  = 'pplpgsql' then 'procedure'
          when pg_language.lanname != 'pplpgsql' then 'function'
        end                           as child_object_type,
        pg_proc.oid                   as child_object_id,
        trim( pg_proc.proname )       as child_object_name
      from
        pg_proc,
        pg_language,
        pg_namespace,
        pg_user
      where
        pg_proc.prolang      = pg_language.oid and
        pg_proc.pronamespace = pg_namespace.oid and
        pg_proc.proowner     = pg_user.usesysid

      union all

      -- MG2 :.functions and procedures depend on their language, and this is not in pg_depends

      select
        null                          as parent_owner_id,
        null::varchar                 as parent_owner_name,
        null                          as parent_schema_id,
        null::varchar                 as parent_schema_name,
        'language'                    as parent_object_type,
        pg_language.oid               as parent_object_id,
        trim( pg_language.lanname )   as parent_object_name,
        pg_proc.proowner              as child_owner_id,
        trim( pg_user.usename )       as child_owner_name,
        pg_namespace.oid              as child_schema_id,
        trim( pg_namespace.nspname )  as child_schema_name,
        case
          when pg_language.lanname  = 'pplpgsql' then 'procedure'
          when pg_language.lanname != 'pplpgsql' then 'function'
        end                           as child_object_type,
        pg_proc.oid                   as child_object_id,
        trim( pg_proc.proname )       as child_object_name
      from
        pg_proc,
        pg_namespace,
        pg_language,
        pg_user
      where
        pg_proc.pronamespace = pg_namespace.oid and
        pg_proc.prolang      = pg_language.oid  and
        pg_proc.proowner     = pg_user.usesysid

      union all

      -- MG2 : languages depends on their owners, and this is not in pg_depends
      --       the only record of this is in the ACL in pg_language, PITA to session
      --       just going to make the records manually, since all languages are owned by rdsdb, user ID 1

      select
        null                         as parent_owner_id,
        null::varchar                as parent_owner_name,
        null                         as parent_schema_id,
        null::varchar                as parent_schema_name,
        'user'                       as parent_object_type,
        pg_user.usesysid             as parent_object_id,
        'rdsdb'                      as parent_object_name,
        pg_user.usesysid             as child_owner_id,
        'rdsdb'                      as child_owner_name,
        null                         as child_schema_id,
        null::varchar                as child_schema_name,
        'language'                   as child_object_type,
        pg_language.oid              as child_object_id,
        trim( pg_language.lanname )  as child_object_name
      from
        pg_language,
        pg_user
      where
        pg_user.usename = 'rdsdb'

      union all

      -- MG2 : namespaces depend on their owners, and this is not in pg_depends

      select
        null                          as parent_owner_id,
        null::varchar                 as parent_owner_name,
        null                          as parent_schema_id,
        null::varchar                 as parent_schema_name,
        'user'                        as parent_object_type,
        pg_user.usesysid              as parent_object_id,
        trim( pg_user.usename )       as parent_object_name,
        pg_namespace.nspowner         as child_owner_id,
        trim( pg_user.usename )       as child_owner_name,
        null                          as child_schema_id,
        null::varchar                 as child_schema_name,
        'schema'                      as child_object_type,
        pg_namespace.oid              as child_object_id,
        trim( pg_namespace.nspname )  as child_object_name
      from
        pg_namespace,
        pg_user
      where
        pg_namespace.nspowner = pg_user.usesysid

      union all

      -- MG2 : databases depend on their owners, and this is not in pg_depends

      select
        null                         as parent_owner_id,
        null::varchar                as parent_owner_name,
        null                         as parent_schema_id,
        null::varchar                as parent_schema_name,
        'user'                       as parent_object_type,
        pg_user.usesysid             as parent_object_id,
        trim( pg_user.usename )      as parent_object_name,
        pg_database.datdba           as child_owner_id,
        trim( pg_user.usename )      as child_owner_name,
        null                         as child_schema_id,
        null::varchar                as child_schema_name,
        'database'                   as child_object_type,
        pg_database.oid              as child_object_id,
        trim( pg_database.datname )  as child_object_name
      from
        pg_database,
        pg_user
      where
        pg_database.datdba = pg_user.usesysid
    ),

    dependencies( dependency_depth, parent_owner_id, parent_owner_name, parent_schema_id, parent_schema_name, parent_object_type, parent_object_id, parent_object_name, child_owner_id, child_owner_name, child_schema_id, child_schema_name, child_object_type, child_object_id, child_object_name ) as 
    (
      select
        0 as dependency_depth,
        flat_dependencies.parent_owner_id,
        flat_dependencies.parent_owner_name,
        flat_dependencies.parent_schema_id,
        flat_dependencies.parent_schema_name,
        flat_dependencies.parent_object_type,
        flat_dependencies.parent_object_id,
        flat_dependencies.parent_object_name,
        flat_dependencies.child_owner_id,
        flat_dependencies.child_owner_name,
        flat_dependencies.child_schema_id,
        flat_dependencies.child_schema_name,
        flat_dependencies.child_object_type,
        flat_dependencies.child_object_id,
        flat_dependencies.child_object_name
      from
        flat_dependencies
      where
        parent_object_type = 'user'

      union all

      select
        dependencies.dependency_depth + 1 as dependency_depth,
        flat_dependencies.parent_owner_id,
        flat_dependencies.parent_owner_name,
        flat_dependencies.parent_schema_id,
        flat_dependencies.parent_schema_name,
        flat_dependencies.parent_object_type,
        flat_dependencies.parent_object_id,
        flat_dependencies.parent_object_name,
        flat_dependencies.child_owner_id,
        flat_dependencies.child_owner_name,
        flat_dependencies.child_schema_id,
        flat_dependencies.child_schema_name,
        flat_dependencies.child_object_type,
        flat_dependencies.child_object_id,
        flat_dependencies.child_object_name
      from
        dependencies,
        flat_dependencies
      where
            dependencies.child_object_type = flat_dependencies.parent_object_type
        and dependencies.child_object_id   = flat_dependencies.parent_object_id
    )

  select
    dependency_depth::int2,
    parent_owner_id::int4,
    parent_owner_name::varchar,
    parent_schema_id::int4,
    parent_schema_name::varchar,
    parent_object_type::varchar,
    parent_object_id::int4,
    parent_object_name::varchar,

    child_owner_id::int4,
    child_owner_name::varchar,
    child_schema_id::int4,
    child_schema_name::varchar,
    child_object_type::varchar,
    child_object_id::int4,
    child_object_name::varchar
  from
    dependencies;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_all_by_all
as
  select
    case
      when slice_ordinal  = -1 then 'leader'
      when slice_ordinal != -1 then 'worker'
    end::varchar as node_type,

    sum( case when type =  1 then rows  else 0 end )::int8 as sum_type_1_rows,
    sum( case when type =  1 then bytes else 0 end )::int8 as sum_type_1_bytes,

    sum( case when type =  2 then rows_pre_filter      else 0 end )::int8 as sum_type_2m_rows,
    sum( case when type =  2 then rows_pre_user_filter else 0 end )::int8 as sum_type_2e_rows,
    sum( case when type =  2 then rows                 else 0 end )::int8 as sum_type_2_rows,
    sum( case when type =  2 then bytes                else 0 end )::int8 as sum_type_2_bytes,

    sum( case when type =  3 then rows  else 0 end )::int8 as sum_type_3_rows,
    sum( case when type =  3 then bytes else 0 end )::int8 as sum_type_3_bytes,

    sum( case when type =  9 then rows  else 0 end )::int8 as sum_type_9_rows,
    sum( case when type =  9 then bytes else 0 end )::int8 as sum_type_9_bytes,

    sum( case when type = 16 then rows  else 0 end )::int8 as sum_type_16_rows,
    sum( case when type = 16 then bytes else 0 end )::int8 as sum_type_16_bytes,

    sum( case when type = 17 then rows  else 0 end )::int8 as sum_type_17_rows,
    sum( case when type = 17 then bytes else 0 end )::int8 as sum_type_17_bytes,

    sum( case when type = 18 then rows  else 0 end )::int8 as sum_type_18_rows,
    sum( case when type = 18 then bytes else 0 end )::int8 as sum_type_18_bytes,

    sum( case when type = 21 then rows  else 0 end )::int8 as sum_type_21_rows,
    sum( case when type = 21 then bytes else 0 end )::int8 as sum_type_21_bytes,

    sum( case when type = 22 then rows  else 0 end )::int8 as sum_type_22_rows,
    sum( case when type = 22 then bytes else 0 end )::int8 as sum_type_22_bytes,

    sum( case when type = 23 then rows  else 0 end )::int8 as sum_type_23_rows,
    sum( case when type = 23 then bytes else 0 end )::int8 as sum_type_23_bytes,

    sum( case when type = 24 then rows  else 0 end )::int8 as sum_type_24_rows,
    sum( case when type = 24 then bytes else 0 end )::int8 as sum_type_24_bytes,

    sum( case when type = 25 then rows  else 0 end )::int8 as sum_type_25_rows,
    sum( case when type = 25 then bytes else 0 end )::int8 as sum_type_25_bytes,

    sum( case when type = 28 then rows  else 0 end )::int8 as sum_type_28_rows,
    sum( case when type = 28 then bytes else 0 end )::int8 as sum_type_28_bytes,

    sum( case when type = 29 then rows_pre_filter      else 0 end )::int8 as sum_type_29m_rows,
    sum( case when type = 29 then rows_pre_user_filter else 0 end )::int8 as sum_type_29e_rows,
    sum( case when type = 29 then rows                 else 0 end )::int8 as sum_type_29_rows,
    sum( case when type = 29 then bytes                else 0 end )::int8 as sum_type_29_bytes,

    sum( case when type = 30 then rows  else 0 end )::int8 as sum_type_30_rows,
    sum( case when type = 30 then bytes else 0 end )::int8 as sum_type_30_bytes,

    sum( case when type = 32 then rows  else 0 end )::int8 as sum_type_32_rows,
    sum( case when type = 32 then bytes else 0 end )::int8 as sum_type_32_bytes,

    sum( case when type = 33 then rows  else 0 end )::int8 as sum_type_33_rows,
    sum( case when type = 33 then bytes else 0 end )::int8 as sum_type_33_bytes,

    sum( case when type = 35 then rows  else 0 end )::int8 as sum_type_35_rows,
    sum( case when type = 35 then bytes else 0 end )::int8 as sum_type_35_bytes,

    sum( case when type = 36 then rows  else 0 end )::int8 as sum_type_36_rows,
    sum( case when type = 36 then bytes else 0 end )::int8 as sum_type_36_bytes,

    sum( case when type = 37 then rows  else 0 end )::int8 as sum_type_37_rows,
    sum( case when type = 37 then bytes else 0 end )::int8 as sum_type_37_bytes
  from
    base_241129.fixed_stl_scan
  where
        scan_start_ts >= current_date - interval '6 day'
    and scan_end_ts   <= current_timestamp
  group by
    node_type;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_all_by_all
as
  with
    cte_hr_formatting as
    (
      select
        node_type,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_all_by_all
    )

 select
    node_type::varchar                                                                                                                     as node,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_statements_by_all
as
  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    step_type::varchar      as step,

    extract( epoch from min_duration )::numeric(18,2) as min,
    extract( epoch from max_duration )::numeric(18,2) as max,
    med_duration::numeric(18,2)                       as med,
    extract( epoch from avg_duration )::numeric(18,2) as avg,
    std_duration::numeric(18,2)                       as std,
    number_steps::int8                                as num
  from
    lf_241129.steps_stats_durations_on_both_for_statements_by_all
  order by
    statement_name,
    node_type,
    step_type;


create or replace view
  sf_241129.uusers
as
  select
    user_name::varchar              as uuser,
    user_type::varchar              as ttype,
    user_syslog::varchar            as ssyslog,
    number_databases::int2          as dbs,
    number_functions::int4          as funcs,
    number_procedures::int4         as procs,
    number_schemas::int2            as sschemas,
    number_tables::int4             as ttables,
    number_normal_views::int4       as nviews,
    number_late_binding_views::int4 as lviews,
    number_materialized_views::int4 as mviews
  from
    lf_241129.uusers
  order by
    user_name;


create or replace view
  lf_241129.privileges_granted_to_groups
as
  select
    acl_recipient_group_id::int4      as group_id,
    acl_recipient_group_name::varchar as group_name,

    object_owner_user_id::int4,
    object_owner_user_name::varchar,

    schema_id::int4,
    schema_name::varchar,
    object_type::varchar,
    object_return_base_data_type_id::int4,
    object_return_base_data_type_name::varchar,
    object_id::int4,
    object_name::varchar,
    object_argument_base_data_type_ids::varchar,
    object_argument_base_data_type_names::varchar,

    case
      when object_type = 'database'  and strpos(acl_privileges,'C') != 0 then true else false
    end::bool as priv_database_create,

    case
      when object_type = 'database'  and strpos(acl_privileges,'T') != 0 then true else false
    end::bool as priv_database_temp,

    case
      when object_type = 'function'  and strpos(acl_privileges,'X') != 0 then true else false
    end::bool as priv_function_execute,

    case
      when object_type = 'language'  and strpos(acl_privileges,'U') != 0 then true else false
    end::bool as priv_language_usage,

    case
      when object_type = 'procedure' and strpos(acl_privileges,'X') != 0 then true else false
    end::bool as priv_procedure_execute,

    case
      when object_type = 'schema'    and strpos(acl_privileges,'C') != 0 then true else false
    end::bool as priv_schema_create,

    case
      when object_type = 'schema'    and strpos(acl_privileges,'U') != 0 then true else false
    end::bool as priv_schema_usage,

    case
      when object_type = 'table'     and strpos(acl_privileges,'d') != 0 then true else false
    end::bool as priv_table_delete,

    case
      when object_type = 'table'     and strpos(acl_privileges,'D') != 0 then true else false
    end::bool as priv_table_drop,

    case
      when object_type = 'table'     and strpos(acl_privileges,'a') != 0 then true else false
    end::bool as priv_table_insert,

    case
      when object_type = 'table'     and strpos(acl_privileges,'x') != 0 then true else false
    end::bool as priv_table_reference,

    case
      when object_type = 'table'     and strpos(acl_privileges,'r') != 0 then true else false
    end::bool as priv_table_select,

    case
      when object_type = 'table'     and strpos(acl_privileges,'w') != 0 then true else false
    end::bool as priv_table_update,

    case
      when object_type = 'view'      and strpos(acl_privileges,'D') != 0 then true else false
    end::bool as priv_view_drop,

    case
      when object_type = 'view'      and strpos(acl_privileges,'r') != 0 then true else false
    end::bool as priv_view_select
  from
    base_241129.privileges_groups;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_queues_by_second
as
  with
    cte_s1 as
    (
      select
        query_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'second', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        query_id,
        node_type,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
         cte_s1
    join base_241129.fixed_stl_wlm_query                using ( query_id )
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.analyze_totals_status_for_all
as
  select
    sum( case when pg_class.reltuples  = pg_statistic_indicator.stairows and pg_statistic_indicator.staiins = 0 and pg_statistic_indicator.staidels = 0 then 1 else 0 end )::int4 as number_good_stats,
    sum( case when pg_class.reltuples != pg_statistic_indicator.stairows or  pg_statistic_indicator.staiins > 0 or  pg_statistic_indicator.staidels > 0 then 1 else 0 end )::int4 as number_stale_stats,
    sum( case when pg_statistic_indicator.stairelid is null or pg_class.reltuples = 0.001::float4 then 1 else 0 end )::int4                                                       as number_missing_stats,
    count(*)::int4                                                                                                                                                                as number_total_stats
  from
              pg_class
    left join pg_statistic_indicator on pg_class.oid = pg_statistic_indicator.stairelid
  where
        relkind   = 'r'
    and relowner != 1;


create or replace view
  sf_241129.analyze_totals_status_for_all
as
  select
    number_good_stats::int4    as good,
    number_stale_stats::int4   as stale,
    number_missing_stats::int4 as missing,
    number_total_stats::int4   as total
  from
    lf_241129.analyze_totals_status_for_all;


create or replace view
  sf_241129.sschemas
as
  select
    schema_name::varchar            as sschema,
    user_name::varchar              as uuser,
    number_tables::int4             as ttables,
    number_normal_views::int4       as nviews,
    number_late_binding_views::int4 as lviews,
    number_materialized_views::int4 as mviews,
    number_functions::int4          as funcs,
    number_procedures::int4         as procs
  from
    lf_241129.sschemas
  order by
    schema_name;


create or replace view
  sf_241129.analyze_totals_for_tables_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_tables_by_minute
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_all_by_all
as
  select
    number_abort::int4 as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_all_by_all;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_users_by_minute
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'minute', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        user_id,
        date_trunc( 'minute', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                   as user_name,
    base_241129.generate_minutes.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_minutes
    left join cte_queries using ( start_ts )
    left join pg_user on cte_queries.user_id = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_users_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_users_by_minute
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.wqueues_totals_slot_utilization_by_all
as
  select
    queue_name::varchar                           as qname,
    slot_number::int2                             as slot,
    extract( epoch from duration )::numeric(18,2) as dur
  from
    lf_241129.wqueues_totals_slot_utilization_by_all
  order by
    queue_name,
    slot_number;


create or replace view
  lf_241129.pprocedures
as
  select
    pg_namespace.oid::int4                 as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    pg_proc.oid::int4                      as procid,
    trim( pg_proc.proname )::varchar       as procname,
    (
                                                               pg_type_arg_00.oid::varchar     ||
      case when pg_type_arg_01.oid is null then '' else ',' || pg_type_arg_01.oid::varchar end ||
      case when pg_type_arg_02.oid is null then '' else ',' || pg_type_arg_02.oid::varchar end ||
      case when pg_type_arg_03.oid is null then '' else ',' || pg_type_arg_03.oid::varchar end ||
      case when pg_type_arg_04.oid is null then '' else ',' || pg_type_arg_04.oid::varchar end ||
      case when pg_type_arg_05.oid is null then '' else ',' || pg_type_arg_05.oid::varchar end ||
      case when pg_type_arg_06.oid is null then '' else ',' || pg_type_arg_06.oid::varchar end ||
      case when pg_type_arg_07.oid is null then '' else ',' || pg_type_arg_07.oid::varchar end ||
      case when pg_type_arg_08.oid is null then '' else ',' || pg_type_arg_08.oid::varchar end ||
      case when pg_type_arg_09.oid is null then '' else ',' || pg_type_arg_09.oid::varchar end ||
      case when pg_type_arg_10.oid is null then '' else ',' || pg_type_arg_10.oid::varchar end ||
      case when pg_type_arg_11.oid is null then '' else ',' || pg_type_arg_11.oid::varchar end ||
      case when pg_type_arg_12.oid is null then '' else ',' || pg_type_arg_12.oid::varchar end ||
      case when pg_type_arg_13.oid is null then '' else ',' || pg_type_arg_13.oid::varchar end ||
      case when pg_type_arg_14.oid is null then '' else ',' || pg_type_arg_14.oid::varchar end ||
      case when pg_type_arg_15.oid is null then '' else ',' || pg_type_arg_15.oid::varchar end ||
      case when pg_type_arg_16.oid is null then '' else ',' || pg_type_arg_16.oid::varchar end ||
      case when pg_type_arg_17.oid is null then '' else ',' || pg_type_arg_17.oid::varchar end ||
      case when pg_type_arg_18.oid is null then '' else ',' || pg_type_arg_18.oid::varchar end ||
      case when pg_type_arg_19.oid is null then '' else ',' || pg_type_arg_19.oid::varchar end ||
      case when pg_type_arg_20.oid is null then '' else ',' || pg_type_arg_20.oid::varchar end ||
      case when pg_type_arg_21.oid is null then '' else ',' || pg_type_arg_21.oid::varchar end ||
      case when pg_type_arg_22.oid is null then '' else ',' || pg_type_arg_22.oid::varchar end ||
      case when pg_type_arg_23.oid is null then '' else ',' || pg_type_arg_23.oid::varchar end ||
      case when pg_type_arg_24.oid is null then '' else ',' || pg_type_arg_24.oid::varchar end ||
      case when pg_type_arg_25.oid is null then '' else ',' || pg_type_arg_25.oid::varchar end ||
      case when pg_type_arg_26.oid is null then '' else ',' || pg_type_arg_26.oid::varchar end ||
      case when pg_type_arg_27.oid is null then '' else ',' || pg_type_arg_27.oid::varchar end ||
      case when pg_type_arg_28.oid is null then '' else ',' || pg_type_arg_28.oid::varchar end ||
      case when pg_type_arg_29.oid is null then '' else ',' || pg_type_arg_29.oid::varchar end ||
      case when pg_type_arg_30.oid is null then '' else ',' || pg_type_arg_30.oid::varchar end ||
      case when pg_type_arg_31.oid is null then '' else ',' || pg_type_arg_31.oid::varchar end
    )::varchar                             as argument_base_data_type_ids,
    (
                                                                   pg_type_arg_00.typname::varchar     ||
      case when pg_type_arg_01.typname is null then '' else ',' || pg_type_arg_01.typname::varchar end ||
      case when pg_type_arg_02.typname is null then '' else ',' || pg_type_arg_02.typname::varchar end ||
      case when pg_type_arg_03.typname is null then '' else ',' || pg_type_arg_03.typname::varchar end ||
      case when pg_type_arg_04.typname is null then '' else ',' || pg_type_arg_04.typname::varchar end ||
      case when pg_type_arg_05.typname is null then '' else ',' || pg_type_arg_05.typname::varchar end ||
      case when pg_type_arg_06.typname is null then '' else ',' || pg_type_arg_06.typname::varchar end ||
      case when pg_type_arg_07.typname is null then '' else ',' || pg_type_arg_07.typname::varchar end ||
      case when pg_type_arg_08.typname is null then '' else ',' || pg_type_arg_08.typname::varchar end ||
      case when pg_type_arg_09.typname is null then '' else ',' || pg_type_arg_09.typname::varchar end ||
      case when pg_type_arg_10.typname is null then '' else ',' || pg_type_arg_10.typname::varchar end ||
      case when pg_type_arg_11.typname is null then '' else ',' || pg_type_arg_11.typname::varchar end ||
      case when pg_type_arg_12.typname is null then '' else ',' || pg_type_arg_12.typname::varchar end ||
      case when pg_type_arg_13.typname is null then '' else ',' || pg_type_arg_13.typname::varchar end ||
      case when pg_type_arg_14.typname is null then '' else ',' || pg_type_arg_14.typname::varchar end ||
      case when pg_type_arg_15.typname is null then '' else ',' || pg_type_arg_15.typname::varchar end ||
      case when pg_type_arg_16.typname is null then '' else ',' || pg_type_arg_16.typname::varchar end ||
      case when pg_type_arg_17.typname is null then '' else ',' || pg_type_arg_17.typname::varchar end ||
      case when pg_type_arg_18.typname is null then '' else ',' || pg_type_arg_18.typname::varchar end ||
      case when pg_type_arg_19.typname is null then '' else ',' || pg_type_arg_19.typname::varchar end ||
      case when pg_type_arg_20.typname is null then '' else ',' || pg_type_arg_20.typname::varchar end ||
      case when pg_type_arg_21.typname is null then '' else ',' || pg_type_arg_21.typname::varchar end ||
      case when pg_type_arg_22.typname is null then '' else ',' || pg_type_arg_22.typname::varchar end ||
      case when pg_type_arg_23.typname is null then '' else ',' || pg_type_arg_23.typname::varchar end ||
      case when pg_type_arg_24.typname is null then '' else ',' || pg_type_arg_24.typname::varchar end ||
      case when pg_type_arg_25.typname is null then '' else ',' || pg_type_arg_25.typname::varchar end ||
      case when pg_type_arg_26.typname is null then '' else ',' || pg_type_arg_26.typname::varchar end ||
      case when pg_type_arg_27.typname is null then '' else ',' || pg_type_arg_27.typname::varchar end ||
      case when pg_type_arg_28.typname is null then '' else ',' || pg_type_arg_28.typname::varchar end ||
      case when pg_type_arg_29.typname is null then '' else ',' || pg_type_arg_29.typname::varchar end ||
      case when pg_type_arg_30.typname is null then '' else ',' || pg_type_arg_30.typname::varchar end ||
      case when pg_type_arg_31.typname is null then '' else ',' || pg_type_arg_31.typname::varchar end
    )::varchar                             as argument_base_data_type_names,

    pg_proc.proowner::int4                 as user_id,
    trim( pg_user.usename )::varchar       as user_name,
    pg_proc.prosecdef::bool                as run_as_owner
  from
              pg_proc
         join pg_language               on pg_proc.prolang          = pg_language.oid
         join pg_namespace              on pg_proc.pronamespace     = pg_namespace.oid
         join pg_user                   on pg_proc.proowner         = pg_user.usesysid
    -- MG2 : pg_proc.proargtypes is an oidvector, which is an array of oids (32 in this case - max proc input args, output args are not included), when an element is empty, the value is 0
    left join pg_type as pg_type_arg_00 on pg_proc.proargtypes[00] != 0 and pg_proc.proargtypes[00] = pg_type_arg_00.oid
    left join pg_type as pg_type_arg_01 on pg_proc.proargtypes[01] != 0 and pg_proc.proargtypes[01] = pg_type_arg_01.oid
    left join pg_type as pg_type_arg_02 on pg_proc.proargtypes[02] != 0 and pg_proc.proargtypes[02] = pg_type_arg_02.oid
    left join pg_type as pg_type_arg_03 on pg_proc.proargtypes[03] != 0 and pg_proc.proargtypes[03] = pg_type_arg_03.oid
    left join pg_type as pg_type_arg_04 on pg_proc.proargtypes[04] != 0 and pg_proc.proargtypes[04] = pg_type_arg_04.oid
    left join pg_type as pg_type_arg_05 on pg_proc.proargtypes[05] != 0 and pg_proc.proargtypes[05] = pg_type_arg_05.oid
    left join pg_type as pg_type_arg_06 on pg_proc.proargtypes[06] != 0 and pg_proc.proargtypes[06] = pg_type_arg_06.oid
    left join pg_type as pg_type_arg_07 on pg_proc.proargtypes[07] != 0 and pg_proc.proargtypes[07] = pg_type_arg_07.oid
    left join pg_type as pg_type_arg_08 on pg_proc.proargtypes[08] != 0 and pg_proc.proargtypes[08] = pg_type_arg_08.oid
    left join pg_type as pg_type_arg_09 on pg_proc.proargtypes[09] != 0 and pg_proc.proargtypes[09] = pg_type_arg_09.oid
    left join pg_type as pg_type_arg_10 on pg_proc.proargtypes[10] != 0 and pg_proc.proargtypes[10] = pg_type_arg_10.oid
    left join pg_type as pg_type_arg_11 on pg_proc.proargtypes[11] != 0 and pg_proc.proargtypes[11] = pg_type_arg_11.oid
    left join pg_type as pg_type_arg_12 on pg_proc.proargtypes[12] != 0 and pg_proc.proargtypes[12] = pg_type_arg_12.oid
    left join pg_type as pg_type_arg_13 on pg_proc.proargtypes[13] != 0 and pg_proc.proargtypes[13] = pg_type_arg_13.oid
    left join pg_type as pg_type_arg_14 on pg_proc.proargtypes[14] != 0 and pg_proc.proargtypes[14] = pg_type_arg_14.oid
    left join pg_type as pg_type_arg_15 on pg_proc.proargtypes[15] != 0 and pg_proc.proargtypes[15] = pg_type_arg_15.oid
    left join pg_type as pg_type_arg_16 on pg_proc.proargtypes[16] != 0 and pg_proc.proargtypes[16] = pg_type_arg_16.oid
    left join pg_type as pg_type_arg_17 on pg_proc.proargtypes[17] != 0 and pg_proc.proargtypes[17] = pg_type_arg_17.oid
    left join pg_type as pg_type_arg_18 on pg_proc.proargtypes[18] != 0 and pg_proc.proargtypes[18] = pg_type_arg_18.oid
    left join pg_type as pg_type_arg_19 on pg_proc.proargtypes[19] != 0 and pg_proc.proargtypes[19] = pg_type_arg_19.oid
    left join pg_type as pg_type_arg_20 on pg_proc.proargtypes[20] != 0 and pg_proc.proargtypes[20] = pg_type_arg_20.oid
    left join pg_type as pg_type_arg_21 on pg_proc.proargtypes[21] != 0 and pg_proc.proargtypes[21] = pg_type_arg_21.oid
    left join pg_type as pg_type_arg_22 on pg_proc.proargtypes[22] != 0 and pg_proc.proargtypes[22] = pg_type_arg_22.oid
    left join pg_type as pg_type_arg_23 on pg_proc.proargtypes[23] != 0 and pg_proc.proargtypes[23] = pg_type_arg_23.oid
    left join pg_type as pg_type_arg_24 on pg_proc.proargtypes[24] != 0 and pg_proc.proargtypes[24] = pg_type_arg_24.oid
    left join pg_type as pg_type_arg_25 on pg_proc.proargtypes[25] != 0 and pg_proc.proargtypes[25] = pg_type_arg_25.oid
    left join pg_type as pg_type_arg_26 on pg_proc.proargtypes[26] != 0 and pg_proc.proargtypes[26] = pg_type_arg_26.oid
    left join pg_type as pg_type_arg_27 on pg_proc.proargtypes[27] != 0 and pg_proc.proargtypes[27] = pg_type_arg_27.oid
    left join pg_type as pg_type_arg_28 on pg_proc.proargtypes[28] != 0 and pg_proc.proargtypes[28] = pg_type_arg_28.oid
    left join pg_type as pg_type_arg_29 on pg_proc.proargtypes[29] != 0 and pg_proc.proargtypes[29] = pg_type_arg_29.oid
    left join pg_type as pg_type_arg_30 on pg_proc.proargtypes[30] != 0 and pg_proc.proargtypes[30] = pg_type_arg_30.oid
    left join pg_type as pg_type_arg_31 on pg_proc.proargtypes[31] != 0 and pg_proc.proargtypes[31] = pg_type_arg_31.oid
  where
    pg_language.lanname = 'plpgsql';


create or replace view
  sf_241129.pprocedures
as
  select
    schema_name::varchar                   as sschema,
    procname::varchar                     as proc,
    argument_base_data_type_names::varchar as args,
    user_name::varchar                     as uuser,
    case when run_as_owner = true then '✔' else '' end::varchar as rao
  from
    lf_241129.pprocedures
  order by
    schema_name,
    procname,
    argument_base_data_type_names;

create or replace view
  sf_241129.steps_totals_data_on_both_for_all_by_day
as
  select
    node_type::varchar as node,
    start_ts::date,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_all_by_day
  order by
    node_type,
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_database_by_minute
as
  select
    database::varchar                                                                                             as database_name,
    date_trunc( 'minute', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    database,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_queues_by_minute
as
  with
    cte_s1 as
    (
      select
        query_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'minute', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        query_id,
        node_type,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
         cte_s1
    join base_241129.fixed_stl_wlm_query                using ( query_id )
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_queues_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_queues_by_minute
    )

  select
    queue_name::varchar as qname,
    node_type::varchar  as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    queue_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.groups_users
as
  select
    group_name::varchar,
    user_name::varchar
  from
    lf_241129.groups_users
  order by
    group_name,
    user_name;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_queues_by_hour
as
  select
    queue_name::varchar as qname,
    start_ts::timestamp,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_queues_by_hour
  order by
    queue_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_data_on_both_for_node_by_day
as
  with
    cte_aggr as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_bcast as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                          as start_ts,
        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_delete as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_dist as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                          as start_ts,
        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hash as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_hashjoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_insert as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_limit as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_merge as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_mergejoin as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_nestloop as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_parse as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_project as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_return as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                          as start_ts,
        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_save as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    -- scan

    cte_sort as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_spatial_join as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                       as start_ts,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_unique as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                                               as start_ts,
        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    ),

    cte_window as
    (
      select
        case when slice >= 12811 then -1 else slice end::int2 as slice_ordinal,
        date_trunc( 'day', starttime )                        as start_ts,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    start_ts::timestamp,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               base_241129.fixed_stv_slices
    cross join base_241129.generate_days
     left join cte_aggr         using( slice_ordinal, start_ts )
     left join cte_bcast        using( slice_ordinal, start_ts )
     left join cte_delete       using( slice_ordinal, start_ts )
     left join cte_dist         using( slice_ordinal, start_ts )
     left join cte_hash         using( slice_ordinal, start_ts )
     left join cte_hashjoin     using( slice_ordinal, start_ts )
     left join cte_insert       using( slice_ordinal, start_ts )
     left join cte_limit        using( slice_ordinal, start_ts )
     left join cte_merge        using( slice_ordinal, start_ts )
     left join cte_mergejoin    using( slice_ordinal, start_ts )
     left join cte_nestloop     using( slice_ordinal, start_ts )
     left join cte_parse        using( slice_ordinal, start_ts )
     left join cte_project      using( slice_ordinal, start_ts )
     left join cte_return       using( slice_ordinal, start_ts )
     left join cte_save         using( slice_ordinal, start_ts )
     left join cte_sort         using( slice_ordinal, start_ts )
     left join cte_spatial_join using( slice_ordinal, start_ts )
     left join cte_unique       using( slice_ordinal, start_ts )
     left join cte_window       using( slice_ordinal, start_ts )
  where
       cte_aggr.sum_aggr_on_disk_bytes        is not null
    or cte_aggr.sum_aggr_in_memory_bytes      is not null
    or cte_aggr.sum_aggr_on_disk_rows         is not null
    or cte_aggr.sum_aggr_in_memory_rows       is not null
    or cte_bcast.sum_bcast_bytes              is not null
    or cte_bcast.sum_bcast_packets            is not null
    or cte_bcast.sum_bcast_rows               is not null
    or cte_delete.sum_delete_rows             is not null
    or cte_dist.sum_dist_bytes                is not null
    or cte_dist.sum_dist_packets              is not null
    or cte_dist.sum_dist_rows                 is not null
    or cte_hash.sum_hash_on_disk_bytes        is not null
    or cte_hash.sum_hash_in_memory_bytes      is not null
    or cte_hash.sum_hash_on_disk_rows         is not null
    or cte_hash.sum_hash_in_memory_rows       is not null
    or cte_hashjoin.sum_hashjoin_rows         is not null
    or cte_insert.sum_insert_rows             is not null
    or cte_limit.sum_limit_rows               is not null
    or cte_merge.sum_merge_rows               is not null
    or cte_mergejoin.sum_mergejoin_rows       is not null
    or cte_nestloop.sum_nestloop_rows         is not null
    or cte_parse.sum_parse_rows               is not null
    or cte_project.sum_project_rows           is not null
    or cte_return.sum_return_bytes            is not null
    or cte_return.sum_return_packets          is not null
    or cte_return.sum_return_rows             is not null
    or cte_save.sum_save_on_disk_bytes        is not null
    or cte_save.sum_save_in_memory_bytes      is not null
    or cte_save.sum_save_on_disk_rows         is not null
    or cte_save.sum_save_in_memory_rows       is not null
    or cte_sort.sum_sort_on_disk_bytes        is not null
    or cte_sort.sum_sort_in_memory_bytes      is not null
    or cte_sort.sum_sort_on_disk_rows         is not null
    or cte_sort.sum_sort_in_memory_rows       is not null
    or cte_spatial_join.sum_spatial_join_rows is not null
    or cte_unique.sum_unique_on_disk_bytes    is not null
    or cte_unique.sum_unique_in_memory_bytes  is not null
    or cte_unique.sum_unique_on_disk_rows     is not null
    or cte_unique.sum_unique_in_memory_rows   is not null
    or cte_window.sum_window_rows             is not null
  group by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_node_by_day
as
  select
    node_ordinal::int2 as node,
    start_ts::date,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_node_by_day
  order by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_all_by_day
as
  select
    start_ts::date,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_all_by_day
  order by
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_all_by_second
as
  with
    cte_queries as
    (
      select
        date_trunc( 'second', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        date_trunc( 'second', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    base_241129.generate_seconds.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_seconds
    left join cte_queries using ( start_ts )
  group by
    start_ts;


create or replace view
  sf_241129.wqueues_memory
as
  select
    queue_name::varchar                                   as qname,
    number_slots::int2                                    as slots,
    ra3_xlplus_memory_per_slot_per_slice::int4            as ra3_xlplus,
    ra3_4x_memory_per_slot_per_slice::int4                as ra3_4x,
    ra3_16x_memory_per_slot_per_slice::int4               as ra3_16x,
    dc2_min_memory_per_slot_per_slice::int4               as dc2_min,
    dc2_min_memory_per_slot_per_slice::int4               as dc2_max
  from
    lf_241129.wqueues_memory
  order by
    queue_name;


create or replace view
  base_241129.fixed_pg_language
as
  select
    oid::int4                as language_id,
    trim( lanname )::varchar as language_name,
    -- MG2 : all languages owned by `rdsdb`
    1::int4                  as user_id
  from
    pg_language;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_all_by_second
as
  select
    start_ts::timestamp,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_all_by_second
  order by
    start_ts;



create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_node_by_all
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal
    )

  select
    node_ordinal::int2,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal )
  group by
    node_ordinal;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_node_by_all
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_node_by_all
    )

 select
    node_ordinal::int2 as n,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_users_by_day
as
  with
    cte_s1 as
    (
      select
        user_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'day', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        user_id,
        node_type,
        start_ts
    )

  select
    user_id::int4,
    node_type::varchar,
    trim( pg_user.usename )::varchar as user_name,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_user on cte_s1.user_id = pg_user.usesysid;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_all_concise
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_all
  order by
    user_name,
    node_type;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_tables_by_minute_complete
as
  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort
  from
    lf_241129.steps_stats_durations_on_both_for_tables_by_minute
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;



create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_users_by_day
as
  select
    user_name::varchar as uuser,
    start_ts::date,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_users_by_day
  order by
    user_name,
    start_ts;


create or replace view
  base_241129.fixed_pg_attribute_for_views
as
  select
    attrelid::int4            as view_id,
    trim( attname ) ::varchar as column_name,
    attnum::int2              as column_ordinal,
    atttypmod::int4,
    attnotnull::bool          as column_notnull_flag
  from
    pg_attribute;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_schemas_by_day
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        tbl         as table_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_insert as
    (
      select
        tbl         as table_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    ),

    cte_scan as
    (
      select
        tbl                            as table_id,
        date_trunc( 'day', starttime ) as start_ts,
        sum( rows )                    as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
      group by
        tbl,
        start_ts
    )

  select
    base_241129.fixed_pg_namespace.schema_id::int4,
    base_241129.fixed_pg_namespace.schema_name::varchar,
    start_ts::timestamp,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_scan.sum_scan_rows )::int8     as sum_scan_rows
  from
               base_241129.fixed_pg_class_for_tables
          join base_241129.fixed_pg_namespace using ( schema_id )
    cross join base_241129.generate_days
     left join cte_delete using ( start_ts, table_id )
     left join cte_insert using ( start_ts, table_id )
     left join cte_scan   using ( start_ts, table_id )
  where
       cte_delete.table_id is not null
    or cte_insert.table_id is not null
    or cte_scan.table_id   is not null
  group by
    base_241129.fixed_pg_namespace.schema_id,
    base_241129.fixed_pg_namespace.schema_name,
    start_ts;



create or replace view
  base_241129.transactions_compilation
as
  select
    user_id::int4,
    transaction_id::int8,
    sum( segment_compile_duration )::interval as transaction_compile_duration
  from
    base_241129.fixed_svl_compile
  group by
    user_id,
    transaction_id;


create or replace view
  lf_241129.transactions_stats_commits_by_all
as
  with
    cte_transaction_queuing_stats as
    (
      select
        min( batch_leader_startwork - transaction_batch_queue_start )                              as min_transaction_batch_queue_start,
        max( batch_leader_startwork - transaction_batch_queue_start )                              as max_transaction_batch_queue_start,
        median( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) )     as med_transaction_batch_queue_start,
        avg( batch_leader_startwork - transaction_batch_queue_start )                              as avg_transaction_batch_queue_start,
        stddev_pop( extract(epoch from (batch_leader_startwork - transaction_batch_queue_start)) ) as std_transaction_batch_queue_start,
        sum( batch_leader_startwork - transaction_batch_queue_start )                              as sum_transaction_batch_queue_start
      from
             base_241129.transactions_in_batches
        join lf_241129.batches using ( batch_id )
    ),

    cte_batch_stats as
    (
      select
        min( batch_queue_duration )                            over () as min_batch_queue_duration,
        max( batch_queue_duration )                            over () as max_batch_queue_duration,
        median( extract(epoch from batch_queue_duration) )     over () as med_batch_queue_duration,
        avg( batch_queue_duration )                            over () as avg_batch_queue_duration,
        stddev_pop( extract(epoch from batch_queue_duration) ) over () as std_batch_queue_duration,
        sum( batch_queue_duration )                            over () as sum_batch_queue_duration,

        min( batch_leader_duration )                            over () as min_batch_leader_duration,
        max( batch_leader_duration )                            over () as max_batch_leader_duration,
        median( extract(epoch from batch_leader_duration) )     over () as med_batch_leader_duration,
        avg( batch_leader_duration )                            over () as avg_batch_leader_duration,
        stddev_pop( extract(epoch from batch_leader_duration) ) over () as std_batch_leader_duration,
        sum( batch_leader_duration )                            over () as sum_batch_leader_duration,

        min( batch_worker_duration )                            over () as min_batch_worker_duration,
        max( batch_worker_duration )                            over () as max_batch_worker_duration,
        median( extract(epoch from batch_worker_duration) )     over () as med_batch_worker_duration,
        avg( batch_worker_duration )                            over () as avg_batch_worker_duration,
        stddev_pop( extract(epoch from batch_worker_duration) ) over () as std_batch_worker_duration,
        sum( batch_worker_duration )                            over () as sum_batch_worker_duration,

        min( number_batch_transactions )         over () as min_number_batch_transactions,
        max( number_batch_transactions )         over () as max_number_batch_transactions,
        median( number_batch_transactions )      over () as med_number_batch_transactions,
        avg( number_batch_transactions::float4 ) over () as avg_number_batch_transactions,
        stddev_pop( number_batch_transactions )  over () as std_number_batch_transactions,
        sum( number_batch_transactions )         over () as sum_number_batch_transactions,

        min( number_batch_blocks )         over () as min_number_batch_blocks,
        max( number_batch_blocks )         over () as max_number_batch_blocks,
        median( number_batch_blocks )      over () as med_number_batch_blocks,
        avg( number_batch_blocks::float4 ) over () as avg_number_batch_blocks,
        stddev_pop( number_batch_blocks )  over () as std_number_batch_blocks,
        sum( number_batch_blocks )         over () as sum_number_batch_blocks,

        count(*) over () as number_batches
      from
        lf_241129.batches
    )

  select
    cte_transaction_queuing_stats.min_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start::interval,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start::float4,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start::interval,

    cte_batch_stats.min_batch_queue_duration::interval,
    cte_batch_stats.max_batch_queue_duration::interval,
    cte_batch_stats.med_batch_queue_duration::float4,
    cte_batch_stats.avg_batch_queue_duration::interval,
    cte_batch_stats.std_batch_queue_duration::float4,
    cte_batch_stats.sum_batch_queue_duration::interval,

    cte_batch_stats.min_batch_leader_duration::interval,
    cte_batch_stats.max_batch_leader_duration::interval,
    cte_batch_stats.med_batch_leader_duration::float4,
    cte_batch_stats.avg_batch_leader_duration::interval,
    cte_batch_stats.std_batch_leader_duration::float4,
    cte_batch_stats.sum_batch_leader_duration::interval,

    cte_batch_stats.min_batch_worker_duration::interval,
    cte_batch_stats.max_batch_worker_duration::interval,
    cte_batch_stats.med_batch_worker_duration::float4,
    cte_batch_stats.avg_batch_worker_duration::interval,
    cte_batch_stats.std_batch_worker_duration::float4,
    cte_batch_stats.sum_batch_worker_duration::interval,

    cte_batch_stats.min_number_batch_transactions::int8,
    cte_batch_stats.max_number_batch_transactions::int8,
    cte_batch_stats.med_number_batch_transactions::int8,
    cte_batch_stats.avg_number_batch_transactions::float4,
    cte_batch_stats.std_number_batch_transactions::float4,
    cte_batch_stats.sum_number_batch_transactions::int8,

    cte_batch_stats.min_number_batch_blocks::int8,
    cte_batch_stats.max_number_batch_blocks::int8,
    cte_batch_stats.med_number_batch_blocks::int8,
    cte_batch_stats.avg_number_batch_blocks::float4,
    cte_batch_stats.std_number_batch_blocks::float4,
    cte_batch_stats.sum_number_batch_blocks::int8,

    cte_batch_stats.number_batches::int8
  from
               cte_batch_stats
    cross join cte_transaction_queuing_stats
  group by
    cte_transaction_queuing_stats.min_transaction_batch_queue_start,
    cte_transaction_queuing_stats.max_transaction_batch_queue_start,
    cte_transaction_queuing_stats.med_transaction_batch_queue_start,
    cte_transaction_queuing_stats.avg_transaction_batch_queue_start,
    cte_transaction_queuing_stats.std_transaction_batch_queue_start,
    cte_transaction_queuing_stats.sum_transaction_batch_queue_start,
    cte_batch_stats.min_batch_queue_duration,
    cte_batch_stats.max_batch_queue_duration,
    cte_batch_stats.med_batch_queue_duration,
    cte_batch_stats.avg_batch_queue_duration,
    cte_batch_stats.std_batch_queue_duration,
    cte_batch_stats.sum_batch_queue_duration,
    cte_batch_stats.min_batch_leader_duration,
    cte_batch_stats.max_batch_leader_duration,
    cte_batch_stats.med_batch_leader_duration,
    cte_batch_stats.avg_batch_leader_duration,
    cte_batch_stats.std_batch_leader_duration,
    cte_batch_stats.sum_batch_leader_duration,
    cte_batch_stats.min_batch_worker_duration,
    cte_batch_stats.max_batch_worker_duration,
    cte_batch_stats.med_batch_worker_duration,
    cte_batch_stats.avg_batch_worker_duration,
    cte_batch_stats.std_batch_worker_duration,
    cte_batch_stats.sum_batch_worker_duration,
    cte_batch_stats.min_number_batch_transactions,
    cte_batch_stats.max_number_batch_transactions,
    cte_batch_stats.med_number_batch_transactions,
    cte_batch_stats.avg_number_batch_transactions,
    cte_batch_stats.std_number_batch_transactions,
    cte_batch_stats.sum_number_batch_transactions,
    cte_batch_stats.min_number_batch_blocks,
    cte_batch_stats.max_number_batch_blocks,
    cte_batch_stats.med_number_batch_blocks,
    cte_batch_stats.avg_number_batch_blocks,
    cte_batch_stats.std_number_batch_blocks,
    cte_batch_stats.sum_number_batch_blocks,
    cte_batch_stats.number_batches;


create or replace view
  lf_241129.steps_totals_data_on_both_for_statements_by_all
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query
      from
        stl_unload_log
    ),

    cte_aggr as
    (
      select
        query,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_aggr_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_aggr_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_aggr_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_aggr_in_memory_rows
      from
        stl_aggr
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_bcast as
    (
      select
        query,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_bcast_bytes,
        sum( packets ) as sum_bcast_packets,
        sum( rows    ) as sum_bcast_rows
      from
        stl_bcast
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_delete as
    (
      select
        query,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_dist as
    (
      select
        query,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_dist_bytes,
        sum( packets ) as sum_dist_packets,
        sum( rows    ) as sum_dist_rows
      from
        stl_dist
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_hash as
    (
      select
        query,

        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_hash_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_hash_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_hash_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_hash_in_memory_rows
      from
        stl_hash
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_hashjoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_hashjoin_rows
      from
        stl_hashjoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_insert as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_limit as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_limit_rows
      from
        stl_limit
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_merge as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_merge_rows
      from
        stl_merge
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_mergejoin as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_mergejoin_rows
      from
        stl_mergejoin
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_nestloop as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_nestloop_rows
      from
        stl_nestloop
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_parse as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_parse_rows
      from
        stl_parse
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_project as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_project_rows
      from
        stl_project
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_return as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( bytes   ) as sum_return_bytes,
        sum( packets ) as sum_return_packets,
        sum( rows    ) as sum_return_rows
      from
        stl_return
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_save as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_save_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_save_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_save_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_save_in_memory_rows
      from
        stl_save
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    -- scan

    cte_sort as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_sort_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_sort_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_sort_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_sort_in_memory_rows
      from
        stl_sort
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_spatial_join as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_spatial_join_rows
      from
        stl_spatial_join
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_unique as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,

        sum( case when is_diskbased = 't' then bytes else 0 end ) as sum_unique_on_disk_bytes,
        sum( case when is_diskbased = 'f' then bytes else 0 end ) as sum_unique_in_memory_bytes,
        sum( case when is_diskbased = 't' then rows  else 0 end ) as sum_unique_on_disk_rows,
        sum( case when is_diskbased = 'f' then rows  else 0 end ) as sum_unique_in_memory_rows
      from
        stl_unique
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_window as
    (
      select
        query,
        case
          when slice >= 12811 then 'leader'
          when slice <  12811 then 'worker'
        end as node_type,
        sum( rows ) as sum_window_rows
      from
        stl_window
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
      group by
        query,
        node_type
    ),

    cte_node_types as
    (
      select 'leader' as node_type
      union all
      select 'worker' as node_type
    )

  select
    statement_name::varchar,
    node_type::varchar,

    sum( cte_aggr.sum_aggr_on_disk_bytes )::int8 as sum_aggr_on_disk_bytes,
    sum( cte_aggr.sum_aggr_in_memory_bytes )::int8 as sum_aggr_in_memory_bytes,
    sum( cte_aggr.sum_aggr_on_disk_rows )::int8 as sum_aggr_on_disk_rows,
    sum( cte_aggr.sum_aggr_in_memory_rows )::int8 as sum_aggr_in_memory_rows,
    sum( cte_bcast.sum_bcast_bytes )::int8 as sum_bcast_bytes,
    sum( cte_bcast.sum_bcast_packets )::int8 as sum_bcast_packets,
    sum( cte_bcast.sum_bcast_rows )::int8 as sum_bcast_rows,
    sum( cte_delete.sum_delete_rows )::int8 as sum_delete_rows,
    sum( cte_dist.sum_dist_bytes )::int8 as sum_dist_bytes,
    sum( cte_dist.sum_dist_packets )::int8 as sum_dist_packets,
    sum( cte_dist.sum_dist_rows )::int8 as sum_dist_rows,
    sum( cte_hash.sum_hash_on_disk_bytes )::int8 as sum_hash_on_disk_bytes,
    sum( cte_hash.sum_hash_in_memory_bytes )::int8 as sum_hash_in_memory_bytes,
    sum( cte_hash.sum_hash_on_disk_rows )::int8 as sum_hash_on_disk_rows,
    sum( cte_hash.sum_hash_in_memory_rows )::int8 as sum_hash_in_memory_rows,
    sum( cte_hashjoin.sum_hashjoin_rows )::int8 as sum_hashjoin_rows,
    sum( cte_insert.sum_insert_rows )::int8 as sum_insert_rows,
    sum( cte_limit.sum_limit_rows )::int8 as sum_limit_rows,
    sum( cte_merge.sum_merge_rows )::int8 as sum_merge_rows,
    sum( cte_mergejoin.sum_mergejoin_rows )::int8 as sum_mergejoin_rows,
    sum( cte_nestloop.sum_nestloop_rows )::int8 as sum_nestloop_rows,
    sum( cte_parse.sum_parse_rows )::int8 as sum_parse_rows,
    sum( cte_project.sum_project_rows )::int8 as sum_project_rows,
    sum( cte_return.sum_return_bytes )::int8 as sum_return_bytes,
    sum( cte_return.sum_return_packets )::int8 as sum_return_packets,
    sum( cte_return.sum_return_rows )::int8 as sum_return_rows,
    sum( cte_save.sum_save_on_disk_bytes )::int8 as sum_save_on_disk_bytes,
    sum( cte_save.sum_save_in_memory_bytes )::int8 as sum_save_in_memory_bytes,
    sum( cte_save.sum_save_on_disk_rows )::int8 as sum_save_on_disk_rows,
    sum( cte_save.sum_save_in_memory_rows )::int8 as sum_save_in_memory_rows,
    sum( cte_sort.sum_sort_on_disk_bytes )::int8 as sum_sort_on_disk_bytes,
    sum( cte_sort.sum_sort_in_memory_bytes )::int8 as sum_sort_in_memory_bytes,
    sum( cte_sort.sum_sort_on_disk_rows )::int8 as sum_sort_on_disk_rows,
    sum( cte_sort.sum_sort_in_memory_rows )::int8 as sum_sort_in_memory_rows,
    sum( cte_spatial_join.sum_spatial_join_rows )::int8 as sum_spatial_join_rows,
    sum( cte_unique.sum_unique_on_disk_bytes )::int8 as sum_unique_on_disk_bytes,
    sum( cte_unique.sum_unique_in_memory_bytes )::int8 as sum_unique_in_memory_bytes,
    sum( cte_unique.sum_unique_on_disk_rows )::int8 as sum_unique_on_disk_rows,
    sum( cte_unique.sum_unique_in_memory_rows )::int8 as sum_unique_in_memory_rows,
    sum( cte_window.sum_window_rows )::int8 as sum_window_rows
  from
               cte_base
    cross join cte_node_types
     left join cte_aggr         using ( query, node_type )
     left join cte_bcast        using ( query, node_type )
     left join cte_delete       using ( query, node_type )
     left join cte_dist         using ( query, node_type )
     left join cte_hash         using ( query, node_type )
     left join cte_hashjoin     using ( query, node_type )
     left join cte_insert       using ( query, node_type )
     left join cte_limit        using ( query, node_type )
     left join cte_merge        using ( query, node_type )
     left join cte_mergejoin    using ( query, node_type )
     left join cte_nestloop     using ( query, node_type )
     left join cte_parse        using ( query, node_type )
     left join cte_project      using ( query, node_type )
     left join cte_return       using ( query, node_type )
     left join cte_save         using ( query, node_type )
     left join cte_sort         using ( query, node_type )
     left join cte_spatial_join using ( query, node_type )
     left join cte_unique       using ( query, node_type )
     left join cte_window       using ( query, node_type )
  group by
    statement_name,
    node_type;


create or replace view
  sf_241129.steps_totals_data_on_both_for_statements_by_all
as

  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                       as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                      as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                        as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                   as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                   as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                   as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                   as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                   as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                   as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                   as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                   as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                      as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                      as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_statements_by_all
  order by
    statement_name,
    node_type;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_all_by_hour
as
  with
    cte_queries as
    (
      select
        date_trunc( 'hour', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        ( query_compile_exec_duration - query_compile_duration ) as query_exec_duration
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id )
        left join base_241129.queries_compilation  using ( query_id )
      where
            query_abort_flag      = false
        and query_csc_flag          = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts           <= current_timestamp
        and query_start_ts         >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        start_ts,

        min( query_queue_duration ) over ( partition by start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by start_ts ) as number_queries
      from
        cte_queries
    )

  select
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_hours
    left join cte_queries_stats using ( start_ts )
  group by
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.objects
as
  select
    object_user_name::varchar as uuser,
    schema_name::varchar      as sschema,
    object_name::varchar      as oobject,
    object_type::varchar      as ttype
  from
    lf_241129.objects
  where
    object_user_name != 'rdsdb'
  order by
    schema_name,
    object_name,
    object_type;


create or replace view
  lf_241129.blocks_rows_totals_for_table
as
  with
    cte_table_sort_type as
    (
      select
        pg_class.oid as table_id,
        case
          when ( min(pg_attribute.attsortkeyord) >= 0 and max(pg_attribute.attsortkeyord) >  0 ) then 'compound'
          when ( min(pg_attribute.attsortkeyord) <  0                                          ) then 'interleaved'
          when ( min(pg_attribute.attsortkeyord)  = 0 and max(pg_attribute.attsortkeyord)  = 0 ) then 'unsorted'
        end          as table_sort_type
      from
             pg_class
        join pg_attribute on pg_class.oid = pg_attribute.attrelid
      where
        pg_class.relkind = 'r'
      group by
        pg_class.oid
    ),

    cte_blocks_and_rows_s1 as
    (
      select
        stv_blocklist.tbl                                                                                              as table_id,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      where
        tbl != 0
      group by
        stv_blocklist.tbl
    ),

    cte_nodes as
    (
      -- MG2 : needed to get row counts correct for `all` distribution

      select
        count( distinct node ) as number_nodes
      from
        stv_slices
    ),

    cte_blocks_and_rows_s2 as
    (
      select
        pg_class_info.relnamespace    as schema_id,
        trim( pg_namespace.nspname )  as schema_name,        
        table_id,
        trim( pg_class_info.relname ) as table_name,
        cte_table_sort_type.table_sort_type,
        number_sorted_blocks,
        case when cte_table_sort_type.table_sort_type = 'unsorted' then 0                      else number_unsorted_blocks end as number_desorted_blocks,
        case when cte_table_sort_type.table_sort_type = 'unsorted' then number_unsorted_blocks else 0                      end as number_unsorted_blocks,
        number_total_blocks,

        case when (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_sorted_rows   / cte_nodes.number_nodes else cte_blocks_and_rows_s1.number_sorted_rows   end as number_sorted_rows,
        case
          when cte_table_sort_type.table_sort_type  = 'unsorted'                                                                                                                       then 0
          when cte_table_sort_type.table_sort_type != 'unsorted' and (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                                                                                       else cte_blocks_and_rows_s1.number_unsorted_rows
        end as number_desorted_rows,
        case
          when cte_table_sort_type.table_sort_type != 'unsorted'                                                                                                                       then 0
          when cte_table_sort_type.table_sort_type  = 'unsorted' and (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_unsorted_rows / cte_nodes.number_nodes
                                                                                                                                                                                       else cte_blocks_and_rows_s1.number_unsorted_rows
        end as number_unsorted_rows,
        case when (pg_class_info.reldiststyle = 8) or (pg_class_info.reldiststyle = 9 and pg_class_info.releffectivediststyle = 10) then cte_blocks_and_rows_s1.number_total_rows    / cte_nodes.number_nodes else cte_blocks_and_rows_s1.number_total_rows    end as number_total_rows
      from
                   cte_blocks_and_rows_s1
        cross join cte_nodes
              join cte_table_sort_type using ( table_id )
              join pg_class_info on cte_blocks_and_rows_s1.table_id = pg_class_info.reloid
              join pg_namespace  on pg_class_info.relnamespace      = pg_namespace.oid
      where
        pg_class_info.relkind = 'r'
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    table_sort_type::varchar,
    number_sorted_blocks::int8,
    number_desorted_blocks::int8,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    number_desorted_rows::int8,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
    cte_blocks_and_rows_s2;


create or replace view
  sf_241129.blocks_rows_totals_for_table
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_desorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_desorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_desorted_blocks) ) over () as max_len_hr_number_desorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks,

        max( len(hr_number_sorted_rows)   ) over () as max_len_hr_number_sorted_rows,
        max( len(hr_number_desorted_rows) ) over () as max_len_hr_number_desorted_rows,
        max( len(hr_number_unsorted_rows) ) over () as max_len_hr_number_unsorted_rows,
        max( len(hr_number_total_rows)    ) over () as max_len_hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_table
    )

  select
    schema_name::varchar                                                           as sschema,
    table_name::varchar                                                            as ttable,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_desorted_blocks, max_len_hr_number_desorted_blocks )::varchar  as dsblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks,

    lpad( hr_number_sorted_rows,   max_len_hr_number_sorted_rows   )::varchar      as srows,
    lpad( hr_number_desorted_rows, max_len_hr_number_desorted_rows )::varchar      as dsrows,
    lpad( hr_number_unsorted_rows, max_len_hr_number_unsorted_rows )::varchar      as urows,
    lpad( hr_number_total_rows,    max_len_hr_number_total_rows    )::varchar      as trows
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_second_concise
as
  select
    start_ts::timestamp,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_second
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  sf_241129.steps_totals_data_on_both_for_slices_by_minute
as
  select
    slice_ordinal::int2 as slice,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_slices_by_minute
  order by
    slice_ordinal,
    start_ts;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_users_by_hour
as
  select
    user_name::varchar                                      as uuser,
    start_ts::timestamp,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_users_by_hour
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_all_by_hour
as
  select
    date_trunc( 'hour', starttime )::timestamp                                                                     as start_ts,

    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_manual_full_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_manual_predictatecolumn_analyze,
    sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_manual_skipped_analyze,

    sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end )::int4 as number_automatic_full_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_automatic_predictatecolumn_analyze,
    sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end )::int4 as number_automatic_skipped_analyze,

    sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end )::int4 as number_autoanalyze_full_analyze,
    sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end )::int4 as number_autoanalyze_predictatecolumn_analyze,
    sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end )::int4 as number_autoanalyze_skipped_analyze,

    sum( case when                                           status = 'Full'            then 1 else 0 end )::int4 as number_total_full_analyze,
    sum( case when                                           status = 'PredicateColumn' then 1 else 0 end )::int4 as number_total_predictatecolumn_analyze,
    sum( case when                                           status = 'Skipped'         then 1 else 0 end )::int4 as number_total_skipped_analyze
  from
    stl_analyze
  where
        starttime <= current_timestamp
    and starttime >= current_date - interval '6 day'
  group by
    start_ts;


create or replace view
  sf_241129.analyze_totals_for_database_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        database_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_database_by_minute
    )

  select
    database_name::varchar as dbname,
    start_ts::timestamp,

    ( hr_number_manual_full_analyze             || ' / ' ||
      hr_number_manual_predictatecolumn_analyze || ' / ' ||
      hr_number_manual_skipped_analyze          )::varchar as manual,

    ( hr_number_automatic_full_analyze             || ' / ' ||
      hr_number_automatic_predictatecolumn_analyze || ' / ' ||
      hr_number_automatic_skipped_analyze          )::varchar as automatic,

    ( hr_number_autoanalyze_full_analyze             || ' / ' ||
      hr_number_autoanalyze_predictatecolumn_analyze || ' / ' ||
      hr_number_autoanalyze_skipped_analyze          )::varchar as autoanalyze,

    ( hr_number_total_full_analyze             || ' / ' ||
      hr_number_total_predictatecolumn_analyze || ' / ' ||
      hr_number_total_skipped_analyze          )::varchar as total
  from
    cte_hr_formatting
  order by
    database_name,
    start_ts;


create or replace view
  lf_241129.transactions_totals_query_counts_on_both_for_all_by_hour
as
  with
    cte_transactions as
    (
      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_ddltext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        count(*)               as number_leader_queries,
        0                      as number_worker_queries
      from
        base_241129.fixed_stl_utilitytext
      where
            sequence        = 0
        and transaction_id >= 1
        and query_end_ts   <= current_timestamp
        and query_start_ts >= current_date - interval '6 day'
      group by
        transaction_id

      union all

      select
        transaction_id,
        min( query_start_ts )  as min_starttime,
        0                      as number_leader_queries,
        count(*)               as number_worker_queries
      from
        base_241129.fixed_stl_query
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and transaction_id   >= 1
        and query_end_ts     <= current_timestamp
        and query_start_ts   >= current_date - interval '6 day'
      group by
        transaction_id
    ),

    cte_all_transactions as
    (
      select
        transaction_id,
        date_trunc( 'hour', min(min_starttime) ) as start_ts,
        sum( number_leader_queries ) as sum_number_leader_queries,
        sum( number_worker_queries ) as sum_number_worker_queries
      from
        cte_transactions
      group by
        transaction_id
    )

  select
    start_ts::timestamp,
    count( distinct cte_all_transactions.transaction_id           )::int8 as number_transactions,
    count( distinct base_241129.fixed_stl_commit_stats.transaction_id )::int8 as number_writing_transactions,
    ( number_transactions - number_writing_transactions           )::int8 as number_readonly_transactions,
    sum( sum_number_leader_queries )::int8                                as number_leader_queries,
    sum( sum_number_worker_queries )::int8                                as number_worker_queries,
    sum( sum_number_leader_queries + sum_number_worker_queries )::int8    as number_total_queries
  from
              cte_all_transactions
    left join base_241129.fixed_stl_commit_stats on cte_all_transactions.transaction_id = base_241129.fixed_stl_commit_stats.transaction_id and base_241129.fixed_stl_commit_stats.node_ordinal = -1
  group by
    start_ts;


create or replace view
  lf_241129.analyze_totals_for_users_by_hour
as
  with
    cte_s1 as
    (
      select
        userid                         as user_id,
        date_trunc( 'hour', starttime ) as start_ts,

        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_manual_full_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_manual_predictatecolumn_analyze,
        sum( case when is_auto = 'f' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_manual_skipped_analyze,

        sum( case when is_auto = 't' and is_background = 'f' and status = 'Full'            then 1 else 0 end ) as number_automatic_full_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'PredicateColumn' then 1 else 0 end ) as number_automatic_predictatecolumn_analyze,
        sum( case when is_auto = 't' and is_background = 'f' and status = 'Skipped'         then 1 else 0 end ) as number_automatic_skipped_analyze,

        sum( case when                   is_background = 't' and status = 'Full'            then 1 else 0 end ) as number_autoanalyze_full_analyze,
        sum( case when                   is_background = 't' and status = 'PredicateColumn' then 1 else 0 end ) as number_autoanalyze_predictatecolumn_analyze,
        sum( case when                   is_background = 't' and status = 'Skipped'         then 1 else 0 end ) as number_autoanalyze_skipped_analyze,

        sum( case when                                           status = 'Full'            then 1 else 0 end ) as number_total_full_analyze,
        sum( case when                                           status = 'PredicateColumn' then 1 else 0 end ) as number_total_predictatecolumn_analyze,
        sum( case when                                           status = 'Skipped'         then 1 else 0 end ) as number_total_skipped_analyze
      from
        stl_analyze
      where
            starttime <= current_timestamp
        and starttime >= current_date - interval '6 day'
      group by
        userid,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,

    number_manual_full_analyze::int4,
    number_manual_predictatecolumn_analyze::int4,
    number_manual_skipped_analyze::int4,

    number_automatic_full_analyze::int4,
    number_automatic_predictatecolumn_analyze::int4,
    number_automatic_skipped_analyze::int4,

    number_autoanalyze_full_analyze::int4,
    number_autoanalyze_predictatecolumn_analyze::int4,
    number_autoanalyze_skipped_analyze::int4,

    number_total_full_analyze::int4,
    number_total_predictatecolumn_analyze::int4,
    number_total_skipped_analyze::int4
  from
             cte_s1
   left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  sf_241129.analyze_totals_for_users_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_users_by_hour
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;



create or replace view
  sf_241129.blocks_rows_totals_for_slice
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        slice_ordinal,
        slice_type,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_desorted_blocks ) as hr_number_desorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_desorted_rows ) as hr_number_desorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_desorted_blocks) ) over () as max_len_hr_number_desorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks,

        max( len(hr_number_sorted_rows)   ) over () as max_len_hr_number_sorted_rows,
        max( len(hr_number_desorted_rows) ) over () as max_len_hr_number_desorted_rows,
        max( len(hr_number_unsorted_rows) ) over () as max_len_hr_number_unsorted_rows,
        max( len(hr_number_total_rows)    ) over () as max_len_hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_slice
    )

  select
    node_ordinal::int2                                                             as node,
    slice_ordinal::int2                                                            as slice,
    slice_type::varchar                                                            as ttype,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_desorted_blocks, max_len_hr_number_desorted_blocks )::varchar  as dsblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks,

    lpad( hr_number_sorted_rows,   max_len_hr_number_sorted_rows   )::varchar      as srows,
    lpad( hr_number_desorted_rows, max_len_hr_number_desorted_rows )::varchar      as dsrows,
    lpad( hr_number_unsorted_rows, max_len_hr_number_unsorted_rows )::varchar      as urows,
    lpad( hr_number_total_rows,    max_len_hr_number_total_rows    )::varchar      as trows
  from
    cte_hr_formatting
  order by
    node_ordinal,
    slice_ordinal;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_all_complete
as
  select
    node_type::varchar,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_all
  order by
    node_type;


create or replace view
  sf_241129.transactions
as
  select
    user_name::varchar                                                           as uuser,
    session_id::int4                                                             as pid,
    transaction_id::int8                                                         as xid,
    transaction_start_ts::timestamp                                              as start_ts,
    transaction_end_ts::timestamp                                                as end_ts,
    extract( epoch from transaction_query_queue_duration )::numeric(18,2)        as qry_queue,
    extract( epoch from transaction_query_compile_exec_duration )::numeric(18,2) as qry_cexec,
    extract( epoch from transaction_query_total_duration )::numeric(18,2)        as qry_total,
    extract( epoch from transaction_idle_duration )::numeric(18,2)               as xid_idle,
    extract( epoch from transaction_duration )::numeric(18,2)                    as xid_total,
    transaction_number_csc_queries::int4                                         as csc,
    transaction_number_rc_queries::int4                                          as rc,
    transaction_number_leader_queries::int4                                      as ldr,
    transaction_number_worker_queries::int4                                      as wrk,
    case when transaction_abort_flag = true then '✔' else '' end::varchar       as abt
  from
    lf_241129.transactions
  order by
    transaction_start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_tables_by_all
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        schema_name,
        table_name,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_tables_by_all
    )

 select
    node_type::varchar   as node,
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    schema_name,
    table_name;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_users_by_hour
as
  with
    cte_s1 as
    (
      select
        user_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'hour', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        user_id,
        node_type,
        start_ts
    )

  select
    user_id::int4,
    node_type::varchar,
    trim( pg_user.usename )::varchar as user_name,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join pg_user on cte_s1.user_id = pg_user.usesysid;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_statements_by_day
as
  with
    cte_base as
    (
      -- MG2 : both of these are slice-based

      select distinct
        'copy' as statement_name,
        query as query_id
      from
        stl_load_commits

      union all

      select distinct
        'unload' as statement_name,
        query as query_id
      from
        stl_unload_log
    ),

    cte_queries as
    (
      select
        cte_base.statement_name,
        date_trunc( 'day', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        query_compile_exec_duration - query_compile_duration as query_exec_duration
      from
             cte_base
        join base_241129.fixed_stl_wlm_query using ( query_id )
        join base_241129.fixed_stl_query     using ( query_id )
        join base_241129.queries_compilation   using ( query_id )
      where
            query_abort_flag             = false
        and query_csc_flag               = false
        and query_queue_start_ts        is not null
        and query_queue_end_ts          is not null
        and query_compile_exec_end_ts   is not null
        and query_compile_exec_start_ts is not null
        and query_end_ts                <= current_timestamp
        and query_start_ts              >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        statement_name,
        start_ts,

        min( query_queue_duration ) over ( partition by statement_name, start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by statement_name, start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by statement_name, start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by statement_name, start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by statement_name, start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by statement_name, start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by statement_name, start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by statement_name, start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by statement_name, start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by statement_name, start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by statement_name, start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by statement_name, start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by statement_name, start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by statement_name, start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by statement_name, start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by statement_name, start_ts ) as number_queries
      from
        cte_queries
    )

  select
    statement_name::varchar,
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_days
    left join cte_queries_stats using ( start_ts )
  group by
    statement_name,
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  lf_241129.pprocedures_text
as
  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_proc.oid::int4                     as procedure_id,
    trim( pg_proc.proname )::varchar      as procedure_name,
    pg_proc.prosrc::varchar               as procedure_text
  from
         pg_proc
    join pg_language  on pg_proc.prolang      = pg_language.oid
    join pg_namespace on pg_proc.pronamespace = pg_namespace.oid
  where
    pg_language.lanname = 'plpgsql';


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_all_by_second
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_all_by_second
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace procedure
  ro.sp_uninstall_version( version in varchar(6) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'drop schema base_' || version || ' cascade';
      execute sql;

      sql := 'drop schema lf_' || version || ' cascade';
      execute sql;

      sql := 'drop schema sf_' || version || ' cascade';
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_tables_by_day
as
  select
    schema_name::varchar                       as sschema,
    table_name::varchar                        as ttable,
    start_ts::date,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_tables_by_day
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_all_by_hour
as
  select
    node_type::varchar as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_all_by_hour
  order by
    node_type,
    start_ts;


create or replace view
  lf_241129.objects_totals_for_schema
as
  with
    cte_pg_class as
    (
      select
        relnamespace                                                                                                          as schema_id,

        -- MG2 : number tables
        sum( case when pg_class.relkind = 'r'                                                             then 1 else 0 end ) as number_tables,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then 1 else 0 end ) as number_normal_views,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts  = 0 and stv_mv_info.name is     null then 1 else 0 end ) as number_late_binding_views,
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then 1 else 0 end ) as number_materialized_views,

        -- MG2 : number columns
        sum( case when pg_class.relkind = 'r'                                                             then pg_class.relnatts + 3 else 0 end ) as number_table_columns,
        sum( case when pg_class.relkind = 'v' and pg_class.relnatts >  0 and stv_mv_info.name is     null then pg_class.relnatts     else 0 end ) as number_normal_view_columns,
        -- MG2 : relnatts is 0 for late binding views
        sum( case when pg_class.relkind = 'v'                            and stv_mv_info.name is not null then pg_class.relnatts     else 0 end ) as number_materialized_view_columns
      from
                  pg_class
             join pg_namespace on pg_class.relnamespace = pg_namespace.oid
        left join stv_mv_info  on stv_mv_info.db_name = current_database()   and
                                  stv_mv_info.schema  = pg_namespace.nspname and
                                  stv_mv_info.name    = pg_class.relname
      where
        pg_class.relkind = 'r' or pg_class.relkind = 'v'
      group by
        schema_id
    ),

    cte_pg_proc as
    (
      select
        pg_proc.pronamespace as schema_id,
        sum( case when pg_language.lanname != 'plpgsql' then 1 else 0 end ) as number_functions,
        sum( case when pg_language.lanname  = 'plpgsql' then 1 else 0 end ) as number_procedures
      from
             pg_proc
        join pg_language on pg_proc.prolang = pg_language.oid
      group by
        pg_proc.pronamespace
    )

  select
    pg_namespace.oid::int4                                                                                                  as schema_id,
    trim( pg_namespace.nspname )::varchar                                                                                   as schema_name,
    case when cte_pg_class.number_tables              is null then 0 else cte_pg_class.number_tables              end::int4 as number_tables,
    case when cte_pg_class.number_table_columns       is null then 0 else cte_pg_class.number_table_columns       end::int4 as number_table_columns,
    case when cte_pg_class.number_normal_views        is null then 0 else cte_pg_class.number_normal_views        end::int4 as number_normal_views,
    case when cte_pg_class.number_normal_view_columns is null then 0 else cte_pg_class.number_normal_view_columns end::int4 as number_normal_view_columns,
    case when cte_pg_class.number_late_binding_views  is null then 0 else cte_pg_class.number_late_binding_views  end::int4 as number_late_binding_views,
    case when cte_pg_class.number_materialized_views  is null then 0 else cte_pg_class.number_materialized_views  end::int4 as number_materialized_views,
    case when cte_pg_class.number_materialized_view_columns  is null then 0 else cte_pg_class.number_materialized_view_columns  end::int4 as number_materialized_view_columns,
    case when cte_pg_proc.number_functions            is null then 0 else cte_pg_proc.number_functions            end::int4 as number_functions,
    case when cte_pg_proc.number_procedures           is null then 0 else cte_pg_proc.number_procedures           end::int4 as number_procedures
  from
              pg_namespace
    left join cte_pg_class on pg_namespace.oid = cte_pg_class.schema_id
    left join cte_pg_proc  on pg_namespace.oid = cte_pg_proc.schema_id;


create or replace view
  sf_241129.objects_totals_for_schema
as
  select
    schema_name::varchar                   as sschema,
    number_tables::int4                    as ttables,
    number_table_columns::int4             as ttables_cols,
    number_normal_views::int4              as nviews,
    number_normal_view_columns::int4       as nviews_cols,
    number_late_binding_views::int4        as lviews,
    number_materialized_views::int4        as mviews,
    number_materialized_view_columns::int4 as mviews_cols,
    number_functions::int4                 as funcs,
    number_procedures::int4                as procs
  from
    lf_241129.objects_totals_for_schema
  order by
    schema_name;


create or replace view
  lf_241129.queries_stats_durations_on_leader_for_users_by_day
as
  with
    cte_queries as
    (
      select
        user_id,
        date_trunc( 'day', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_ddltext
      where
        sequence = 0

      union all

      select
        user_id,
        date_trunc( 'day', query_start_ts ) as start_ts,
        query_total_duration
      from
        base_241129.fixed_stl_utilitytext
      where
        sequence = 0
    )

  select
    user_id::int4,
    trim( pg_user.usename )::varchar                   as user_name,
    base_241129.generate_days.start_ts::timestamp             as start_ts,
    min( query_total_duration )::interval              as min_duration,
    max( query_total_duration )::interval              as max_duration,
    median( extract(epoch from query_total_duration) )::float4     as med_duration,
    avg( query_total_duration )::interval              as avg_duration,
    stddev_pop( extract(epoch from query_total_duration) )::float4 as std_duration,
    sum( query_total_duration )::interval              as sum_duration,
    count(*)::int4                                     as number_queries
  from
              base_241129.generate_days
    left join cte_queries using ( start_ts )
    left join pg_user on cte_queries.user_id = pg_user.usesysid
  group by
    user_id,
    user_name,
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_users_by_day
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_users_by_day
    )

  select
    user_name::varchar as uuser,
    start_ts::date,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.vacuum_totals_for_users_by_day
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_users_by_day
    )

  select
    user_name::varchar as uuser,
    start_ts::date,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_rows_on_workers_for_all_by_all
as
  -- MG2 : leader nodes do not perform delete or insert steps

  with
    cte_delete as
    (
      select
        sum( rows ) as sum_delete_rows
      from
        stl_delete
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
    ),

    cte_insert as
    (
      select
        sum( rows ) as sum_insert_rows
      from
        stl_insert
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
    ),

    cte_scan as
    (
      select
        sum( rows ) as sum_scan_rows
      from
        stl_scan
      where
            starttime >= current_date - interval '6 day'
        and endtime   <= current_timestamp
        and starttime != '2000-01-01 00:00:00'
        and endtime   != '2000-01-01 00:00:00'
        and slice     <  12811
    ),

    cte_nulls_to_values as
    (
      select
        case when sum_delete_rows is null then 0 else sum_delete_rows end as notnulled_sum_delete_rows,
        case when sum_insert_rows is null then 0 else sum_insert_rows end as notnulled_sum_insert_rows,
        case when sum_scan_rows   is null then 0 else sum_scan_rows   end as notnulled_sum_scan_rows
      from
                   cte_delete
        cross join cte_insert
        cross join cte_scan
    )

  select
    notnulled_sum_delete_rows::int8,
    notnulled_sum_insert_rows::int8,
    notnulled_sum_scan_rows::int8
  from
    cte_nulls_to_values;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_all_by_all
as
  select
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_all_by_all;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_all_concise
as
  select
    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_all
  order by
    node_ordinal,
    slice_ordinal;


create or replace view
  sf_241129.wqueues_totals_slot_utilization_by_hour
as
  select
    queue_name::varchar                           as qname,
    slot_number::int2                             as slot,
    start_ts::timestamp,
    extract( epoch from duration )::numeric(18,2) as dur
  from
    lf_241129.wqueues_totals_slot_utilization_by_hour
  order by
    queue_name,
    slot_number,
    start_ts;


create or replace view
  lf_241129.queries_totals_result_cache_for_users_by_hour
as
  with
    cte_s1 as
    (
      select
        user_id,
        date_trunc( 'hour', query_start_ts )                                                                      as start_ts,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is     null then 1 else 0 end )::int8 as number_hit,
        sum( case when base_241129.fixed_stl_query.query_id is not null and base_241129.fixed_stl_wlm_query.query_id is not null then 1 else 0 end )::int8 as number_miss,
        count(*)                                                                                                 as number_total,
        ( number_hit::float4 / (number_hit+number_miss)::float4 )::float4                                        as percentage
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id, user_id )
      where
            query_abort_flag  = false
        and query_csc_flag    = false
        and query_start_ts   >= current_date - interval '6 day'
        and query_end_ts     <= current_timestamp
      group by
        user_id,
        start_ts
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    number_hit::int8,
    number_miss::int8,
    number_total::int8,
    percentage::float4
  from
              cte_s1
    left join base_241129.fixed_pg_user using ( user_id );


create or replace view
  sf_241129.queries_totals_result_cache_for_users_by_hour
as
  select
    user_name::varchar as uuser,
    start_ts::timestamp,
    number_hit::int8    as hit,
    number_miss::int8   as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_users_by_hour
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.objects_dependencies
as
  select
    dependency_depth::int2 as ddepth,
    parent_owner_name::varchar,
    parent_schema_name::varchar,
    parent_object_type::varchar,
    parent_object_name::varchar,
    child_owner_name::varchar,
    child_schema_name::varchar,
    child_object_type::varchar,
    child_object_name::varchar
  from
    lf_241129.objects_dependencies
  order by
    dependency_depth,
    parent_schema_name,
    parent_object_name,
    parent_object_type,
    child_schema_name,
    child_object_name,
    child_object_type;


create or replace view
  sf_241129.views_materialized
as
  select
    schema_name::varchar as sschema,
    view_name::varchar   as vview,
    user_name::varchar   as uuser,
    (
      case when failures     != 'none'        then 'f' else ' ' end
      ||
      case when autorefresh   = 'automatic'   then 'a' else 'm' end
      ||
      case when refresh_type  = 'unavailable' then 'u' else ' ' end
      ||
      case when refresh_type  = 'full'        then 'f' else 'i' end
    )::varchar as status
  from
    lf_241129.views_materialized
  order by
    schema_name,
    view_name;


create or replace view
  sf_241129.ttables_blocks_rows_activity
as
  select
    schema_name::varchar          as sschema,
    table_name::varchar           as ttable,
    user_name::varchar            as uuser,
    ( extract( epoch from table_age ) / (60*60*24) )::int4 as age,
    table_number_columns::int2    as cols,
    table_sort_type::varchar      as ssort,
    table_number_sortkeys::int2   as skeys,
    case
      when table_distribution_auto_flag = True  then table_distribution_type || ' (auto)'
      when table_distribution_auto_flag = false then table_distribution_type
    end::varchar                  as dist,
    number_slices::int2           as slices,
    number_sorted_blocks::int8    as sblks,
    number_desorted_blocks::int8  as dsblks,
    number_unsorted_blocks::int8  as ublks,
    number_total_blocks::int8     as tblks,
    number_sorted_rows::int8      as srows,
    number_desorted_rows::int8    as dsrows,
    number_unsorted_rows::int8    as urows,
    number_total_rows::int8       as trows,
    number_rows_deleted::int8     as drops,
    number_rows_inserted::int8    as irows
  from
    lf_241129.ttables_blocks_rows_activity
  order by
    schema_name,
    table_name;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_users_by_hour
as
  select
    user_name::varchar                                                    as uuser,
    start_ts::timestamp,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_users_by_hour
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_second_concise
as
  select
    node_type::varchar as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_second
  order by
    node_type,
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_statements_by_day
as

  with
    cte_hr_formatting as
    (
      select
        statement_name,
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_statements_by_day
    )

  select
    statement_name::varchar as stmt,
    start_ts::date,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    statement_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_tables_by_second
as
  select
    schema_name::varchar                       as sschema,
    table_name::varchar                        as ttable,
    start_ts::timestamp,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_tables_by_second
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_statements_by_day
as

  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    start_ts::date,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                       as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                      as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                        as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                   as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                   as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                   as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                   as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                   as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                   as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                   as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                   as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                      as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                      as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_statements_by_day
  order by
    statement_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_users_by_hour
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_users_by_hour
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.views_late_binding
as
  select
    pg_namespace.oid::int4                as schema_id,
    trim( pg_namespace.nspname )::varchar as schema_name,
    pg_class.oid::int4                    as view_id,
    trim( pg_class.relname )::varchar     as view_name,
    pg_class.relowner::int4               as user_id,
    trim( pg_user.usename )::varchar      as user_name
  from
              pg_class
         join pg_namespace on pg_class.relnamespace = pg_namespace.oid
         join pg_user      on pg_class.relowner     = pg_user.usesysid
    left join stv_mv_info  on     stv_mv_info.db_name = current_database()
                              and stv_mv_info.schema  = pg_namespace.nspname
                              and stv_mv_info.name    = pg_class.relname
  where
        pg_class.relkind   = 'v'
    and pg_class.relnatts  = 0     -- MG2 : only late-binding views
    and stv_mv_info.name  is null; -- MG2 : no materialized views


create or replace view
  sf_241129.vacuum_totals_for_tables_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_tables_by_hour
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_users_by_second
as
  select
    user_name::varchar                                      as uuser,
    start_ts::timestamp,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_users_by_second
  order by
    user_name,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_queues_by_day
as
  with
    cte_s1 as
    (
      select
        query_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'day', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        query_id,
        node_type,
        start_ts
    )

  select
    queue_id::int2,
    queue_name::varchar,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
         cte_s1
    join base_241129.fixed_stl_wlm_query                using ( query_id )
    join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  group by
    queue_id,
    queue_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.analyze_totals_for_tables_by_day
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_tables_by_day
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::date,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  lf_241129.transactions_totals_commits_by_all
as
  select
    sum( batch_leader_duration )::interval               as sum_batch_leader_duration,
    sum( batch_worker_duration )::interval               as sum_batch_worker_duration,
    sum( batch_exec_duration - batch_leader_duration - batch_worker_duration )::interval as sum_batch_idle_duration,
    sum( batch_exec_duration  )::interval                as sum_batch_exec_duration
  from
    lf_241129.batches;


create or replace procedure
  ro.sp_revoke_version_from_public( version in varchar(6) )
as
  $proc$
    declare
      sql  varchar(4096);

    begin
      sql := 'revoke usage on schema base_' || version || ' from public';
      execute sql;

      sql := 'revoke usage on schema lf_' || version || ' from public';
      execute sql;

      sql := 'revoke usage on schema sf_' || version || ' from public';
      execute sql;

      sql := 'revoke select on all tables in schema base_' || version || ' from public';
      execute sql;

      sql := 'revoke select on all tables in schema lf_' || version || ' from public';
      execute sql;

      sql := 'revoke select on all tables in schema sf_' || version || ' from public';
      execute sql;

      sql := 'revoke execute on all functions  in schema base_' || version || ' from public';
      execute sql;

      sql := 'revoke execute on all procedures in schema lf_' || version || ' from public';
      execute sql;
    end
  $proc$
language plpgsql;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_users_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_users_by_hour
    )

  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.vacuum_totals_for_all_by_all
as
  with
    cte_hr_formatting as
    (
      select
        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_all_by_all
    )

  select
    ( hr_number_manual_delete || ' / ' || hr_number_manual_full )::varchar as manual,
    ( hr_number_auto_delete   || ' / ' || hr_number_auto_full   )::varchar as auto,

    number_total_vacuum::int4                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                    as dur
  from
    cte_hr_formatting;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_schemas_by_second
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'second', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r'
  group by
    schema_id,
    schema_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_schemas_by_second
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_schemas_by_second
    )

  select
    schema_name::varchar as sschema,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_users_by_second
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_users_by_second
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.queries_totals_result_cache_for_all_by_hour
as
  select
    start_ts::timestamp,
    number_hit::int8  as hit,
    number_miss::int8 as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_all_by_hour
  order by
    start_ts;


create or replace view
  lf_241129.blocks_rows_totals_for_tables_by_slice
as
  with
    cte_blocks_and_rows as
    (
      select
        stv_blocklist.tbl                                                                                              as table_id,
        stv_blocklist.slice                                                                                            as slice_ordinal,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      where
        tbl != 0
      group by
        stv_blocklist.tbl,
        stv_blocklist.slice
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    table_distribution_type::varchar,
    table_distribution_auto_flag::bool,
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,
    number_sorted_blocks::int8,
    number_unsorted_blocks::int8,
    number_total_blocks::int8,
    number_sorted_rows::int8,
    number_unsorted_rows::int8,
    number_total_rows::int8
  from
              cte_blocks_and_rows
    left join base_241129.ttables_basic_info_fast using ( table_id )
    left join base_241129.fixed_stv_slices           using ( slice_ordinal );


create or replace view
  sf_241129.blocks_rows_totals_for_tables_by_node
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        table_distribution_type,
        table_distribution_auto_flag,
        node_ordinal,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks,

        max( len(hr_number_sorted_rows)   ) over () as max_len_hr_number_sorted_rows,
        max( len(hr_number_unsorted_rows) ) over () as max_len_hr_number_unsorted_rows,
        max( len(hr_number_total_rows)    ) over () as max_len_hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_tables_by_slice
    )

  select
    schema_name::varchar              as sschema,
    table_name::varchar               as ttable,
    table_distribution_type::varchar  as dist,
    case
      when table_distribution_auto_flag = True  then '✔'
      when table_distribution_auto_flag = false then ''
    end::varchar                      as a,
    node_ordinal::int2                as n,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks,

    lpad( hr_number_sorted_rows,   max_len_hr_number_sorted_rows   )::varchar      as srows,
    lpad( hr_number_unsorted_rows, max_len_hr_number_unsorted_rows )::varchar      as urows,
    lpad( hr_number_total_rows,    max_len_hr_number_total_rows    )::varchar      as trows
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    node_ordinal;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_minute_concise
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_minute
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.queries_totals_blocks_on_workers_for_users_by_second
as
  with
    cte_blocks as
    (
      select
        user_id,
        user_name,
        date_trunc( 'second', query_start_ts ) as start_ts,
        sum( query_number_blocks_read     ) as sum_query_number_blocks_read,
        sum( query_number_blocks_to_disk  ) as sum_query_number_blocks_to_disk
      from
                  base_241129.fixed_stl_query_metrics
        left join base_241129.fixed_pg_user using ( user_id )
      where
        query_start_ts >= current_date - interval '6 day'
      group by
        user_id,
        user_name,
        start_ts
    ),

    cte_nulls_to_values as
    (
      select
        user_id,
        user_name,
        start_ts,
        case when sum_query_number_blocks_read    is null then 0 else sum_query_number_blocks_read    end as notnulled_sum_query_number_blocks_read,
        case when sum_query_number_blocks_to_disk is null then 0 else sum_query_number_blocks_to_disk end as notnulled_sum_query_number_blocks_to_disk
      from
        cte_blocks
    )

  select
    user_id::int4,
    user_name::varchar,
    start_ts::timestamp,
    notnulled_sum_query_number_blocks_read::int8,
    notnulled_sum_query_number_blocks_to_disk::int8
  from
    cte_nulls_to_values;


create or replace view
  sf_241129.steps_totals_data_on_both_for_statements_by_hour
as

  select
    statement_name::varchar as stmt,
    node_type::varchar      as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                       as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                      as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                        as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                   as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                   as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                   as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                   as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                   as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                   as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                   as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                   as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                      as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                      as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_statements_by_hour
  order by
    statement_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.queries_stats_durations_on_workers_for_all_by_minute
as
  with
    cte_queries as
    (
      select
        date_trunc( 'minute', query_queue_start_ts ) as start_ts,
        query_queue_duration,
        query_compile_duration,
        ( query_compile_exec_duration - query_compile_duration ) as query_exec_duration
      from
                  base_241129.fixed_stl_query
        left join base_241129.fixed_stl_wlm_query using ( query_id )
        left join base_241129.queries_compilation  using ( query_id )
      where
            query_abort_flag      = false
        and query_csc_flag          = false
        and query_queue_start_ts is not null
        and query_queue_end_ts   is not null
        and query_compile_exec_end_ts    is not null
        and query_compile_exec_start_ts  is not null
        and query_end_ts           <= current_timestamp
        and query_start_ts         >= current_date - interval '6 day'
    ),

    cte_queries_stats as
    (
      select
        start_ts,

        min( query_queue_duration ) over ( partition by start_ts )                              as min_query_queue_duration,
        max( query_queue_duration ) over ( partition by start_ts )                              as max_query_queue_duration,
        median( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts )     as med_query_queue_duration,
        avg( query_queue_duration ) over ( partition by start_ts )                              as avg_query_queue_duration,
        stddev_pop( extract(epoch from (query_queue_duration)) ) over ( partition by start_ts ) as std_query_queue_duration,
        sum( query_queue_duration ) over ( partition by start_ts )                              as sum_query_queue_duration,

        min( query_compile_duration ) over ( partition by start_ts )                              as min_query_compile_duration,
        max( query_compile_duration ) over ( partition by start_ts )                              as max_query_compile_duration,
        median( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts )     as med_query_compile_duration,
        avg( query_compile_duration ) over ( partition by start_ts )                              as avg_query_compile_duration,
        stddev_pop( extract(epoch from (query_compile_duration)) ) over ( partition by start_ts ) as std_query_compile_duration,
        sum( query_compile_duration ) over ( partition by start_ts )                              as sum_query_compile_duration,

        min( query_exec_duration ) over ( partition by start_ts )                            as min_query_exec_duration,
        max( query_exec_duration ) over ( partition by start_ts )                            as max_query_exec_duration,
        median( extract(epoch from query_exec_duration) ) over ( partition by start_ts )     as med_query_exec_duration,
        avg( query_exec_duration ) over ( partition by start_ts )                            as avg_query_exec_duration,
        stddev_pop( extract(epoch from query_exec_duration) ) over ( partition by start_ts ) as std_query_exec_duration,
        sum( query_exec_duration ) over ( partition by start_ts )                            as sum_query_exec_duration,

        count(*) over ( partition by start_ts ) as number_queries
      from
        cte_queries
    )

  select
    start_ts::timestamp,

    min_query_queue_duration::interval,
    max_query_queue_duration::interval,
    med_query_queue_duration::float4,
    avg_query_queue_duration::interval,
    std_query_queue_duration::float4,
    sum_query_queue_duration::interval,

    min_query_compile_duration::interval,
    max_query_compile_duration::interval,
    med_query_compile_duration::float4,
    avg_query_compile_duration::interval,
    std_query_compile_duration::float4,
    sum_query_compile_duration::interval,

    min_query_exec_duration::interval,
    max_query_exec_duration::interval,
    med_query_exec_duration::float4,
    avg_query_exec_duration::interval,
    std_query_exec_duration::float4,
    sum_query_exec_duration::interval,

    number_queries::int4
  from
              base_241129.generate_minutes
    left join cte_queries_stats using ( start_ts )
  group by
    start_ts,
    min_query_queue_duration,
    max_query_queue_duration,
    med_query_queue_duration,
    avg_query_queue_duration,
    std_query_queue_duration,
    sum_query_queue_duration,
    min_query_compile_duration,
    max_query_compile_duration,
    med_query_compile_duration,
    avg_query_compile_duration,
    std_query_compile_duration,
    sum_query_compile_duration,
    min_query_exec_duration,
    max_query_exec_duration,
    med_query_exec_duration,
    avg_query_exec_duration,
    std_query_exec_duration,
    sum_query_exec_duration,
    number_queries;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_all_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_all_by_minute
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_hour_complete
as
  select
    start_ts::timestamp,
    node_ordinal::int2 as n,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_hour
  order by
    start_ts,
    node_ordinal;


create or replace view
  sf_241129.privileges_granted_to_groups
as
  select
    group_name::varchar as ggroup,

    object_type::varchar as ttype,
    schema_name::varchar as sschema,
    object_name::varchar as oobject,

    object_return_base_data_type_name::varchar    as rt,
    object_argument_base_data_type_names::varchar as args,

    object_owner_user_name::varchar as oowner,

    (
      case when priv_database_create = true then 'c' else ' ' end
      ||
      case when priv_database_temp   = true then 't' else ' ' end
      ||
      case when priv_language_usage  = true then 'u' else ' ' end
      ||
      '.'
      ||
      case when priv_schema_create   = true then 'c' else ' ' end
      ||
      case when priv_schema_usage    = true then 'u' else ' ' end
      ||
      '.'
      ||
      case when priv_table_delete    = true then 'd' else ' ' end
      ||
      case when priv_table_reference = true then 'f' else ' ' end
      ||
      case when priv_table_insert    = true then 'i' else ' ' end
      ||
      case when priv_table_drop      = true then 'p' else ' ' end
      ||
      case when priv_table_select = true or priv_view_select = true then 's' else ' ' end
      ||
      case when priv_table_update    = true then 'u' else ' ' end
      ||
      case when priv_function_execute = true or priv_procedure_execute = true then 'x' else ' ' end
    )::varchar as privs
  from
    lf_241129.privileges_granted_to_groups
  order by
    group_name,
    schema_name,
    case when object_type = 'schema' then 0 else 1 end,
    object_type,
    object_name;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_slices_by_minute_complete
as
  select
    start_ts::timestamp,

    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_slices_by_minute
  order by
    start_ts,
    node_ordinal,
    slice_ordinal;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_users_by_minute
as
  select
    user_name::varchar as uuser,
    start_ts::timestamp,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_users_by_minute
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_data_on_both_for_node_by_minute
as
  select
    node_ordinal::int2 as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_node_by_minute
  order by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.analyze_totals_for_database_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        database_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_database_by_hour
    )

  select
    database_name::varchar as dbname,
    start_ts::timestamp,

    ( hr_number_manual_full_analyze             || ' / ' ||
      hr_number_manual_predictatecolumn_analyze || ' / ' ||
      hr_number_manual_skipped_analyze          )::varchar as manual,

    ( hr_number_automatic_full_analyze             || ' / ' ||
      hr_number_automatic_predictatecolumn_analyze || ' / ' ||
      hr_number_automatic_skipped_analyze          )::varchar as automatic,

    ( hr_number_autoanalyze_full_analyze             || ' / ' ||
      hr_number_autoanalyze_predictatecolumn_analyze || ' / ' ||
      hr_number_autoanalyze_skipped_analyze          )::varchar as autoanalyze,

    ( hr_number_total_full_analyze             || ' / ' ||
      hr_number_total_predictatecolumn_analyze || ' / ' ||
      hr_number_total_skipped_analyze          )::varchar as total
  from
    cte_hr_formatting
  order by
    database_name,
    start_ts;


create or replace procedure
  ro.sp_revoke_versionless_sf_schema_from_public()
as
  $proc$
    begin
      revoke usage on schema sf from public;
      revoke select on all tables in schema sf from public;
    end
  $proc$
language plpgsql;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_all_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_all_by_minute
    )

  select
    node_type::varchar                                                                                                                     as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    start_ts;



create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_users_by_all
as
  select
    user_name::varchar                                                    as uuser,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_users_by_all
  order by
    user_name;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_schemas_by_day
as
  select
    schema_name::varchar                                     as sschema,
    start_ts::date,
    base_241129.hr_number( sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_schemas_by_day
  order by
    schema_name,
    start_ts;


create or replace view
  sf_241129.ttables_creation_for_all_by_hour
as
  select
    start_ts::timestamp,
    number_tables::int4 as ttables
  from
    lf_241129.ttables_creation_for_all_by_hour
  order by
    start_ts;


create or replace view
  sf_241129.vacuum_stats_for_all_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        base_241129.hr_number( min_sum_number_new_blocks ) as hr_min_sum_number_new_blocks,
        base_241129.hr_number( max_sum_number_new_blocks ) as hr_max_sum_number_new_blocks,
        base_241129.hr_number( med_sum_number_new_blocks ) as hr_med_sum_number_new_blocks,
        base_241129.hr_number( avg_sum_number_new_blocks ) as hr_avg_sum_number_new_blocks,
        base_241129.hr_number( std_sum_number_new_blocks ) as hr_std_sum_number_new_blocks,
        base_241129.hr_number( sum_sum_number_new_blocks ) as hr_sum_sum_number_new_blocks,

        base_241129.hr_number( min_sum_number_replaced_blocks ) as hr_min_sum_number_replaced_blocks,
        base_241129.hr_number( max_sum_number_replaced_blocks ) as hr_max_sum_number_replaced_blocks,
        base_241129.hr_number( med_sum_number_replaced_blocks ) as hr_med_sum_number_replaced_blocks,
        base_241129.hr_number( avg_sum_number_replaced_blocks ) as hr_avg_sum_number_replaced_blocks,
        base_241129.hr_number( std_sum_number_replaced_blocks ) as hr_std_sum_number_replaced_blocks,
        base_241129.hr_number( sum_sum_number_replaced_blocks ) as hr_sum_sum_number_replaced_blocks,

        base_241129.hr_number( min_sum_number_new_rows ) as hr_min_sum_number_new_rows,
        base_241129.hr_number( max_sum_number_new_rows ) as hr_max_sum_number_new_rows,
        base_241129.hr_number( med_sum_number_new_rows ) as hr_med_sum_number_new_rows,
        base_241129.hr_number( avg_sum_number_new_rows ) as hr_avg_sum_number_new_rows,
        base_241129.hr_number( std_sum_number_new_rows ) as hr_std_sum_number_new_rows,
        base_241129.hr_number( sum_sum_number_new_rows ) as hr_sum_sum_number_new_rows,

        base_241129.hr_number( min_sum_number_deleted_rows ) as hr_min_sum_number_deleted_rows,
        base_241129.hr_number( max_sum_number_deleted_rows ) as hr_max_sum_number_deleted_rows,
        base_241129.hr_number( med_sum_number_deleted_rows ) as hr_med_sum_number_deleted_rows,
        base_241129.hr_number( avg_sum_number_deleted_rows ) as hr_avg_sum_number_deleted_rows,
        base_241129.hr_number( std_sum_number_deleted_rows ) as hr_std_sum_number_deleted_rows,
        base_241129.hr_number( sum_sum_number_deleted_rows ) as hr_sum_sum_number_deleted_rows,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_sum_number_new_blocks) ) over () as max_len_hr_min_sum_number_new_blocks,
        max( len(hr_max_sum_number_new_blocks) ) over () as max_len_hr_max_sum_number_new_blocks,
        max( len(hr_med_sum_number_new_blocks) ) over () as max_len_hr_med_sum_number_new_blocks,
        max( len(hr_avg_sum_number_new_blocks) ) over () as max_len_hr_avg_sum_number_new_blocks,
        max( len(hr_std_sum_number_new_blocks) ) over () as max_len_hr_std_sum_number_new_blocks,
        max( len(hr_sum_sum_number_new_blocks) ) over () as max_len_hr_sum_sum_number_new_blocks,

        max( len(hr_min_sum_number_replaced_blocks) ) over () as max_len_hr_min_sum_number_replaced_blocks,
        max( len(hr_max_sum_number_replaced_blocks) ) over () as max_len_hr_max_sum_number_replaced_blocks,
        max( len(hr_med_sum_number_replaced_blocks) ) over () as max_len_hr_med_sum_number_replaced_blocks,
        max( len(hr_avg_sum_number_replaced_blocks) ) over () as max_len_hr_avg_sum_number_replaced_blocks,
        max( len(hr_std_sum_number_replaced_blocks) ) over () as max_len_hr_std_sum_number_replaced_blocks,
        max( len(hr_sum_sum_number_replaced_blocks) ) over () as max_len_hr_sum_sum_number_replaced_blocks,

        max( len(hr_min_sum_number_new_rows) ) over () as max_len_hr_min_sum_number_new_rows,
        max( len(hr_max_sum_number_new_rows) ) over () as max_len_hr_max_sum_number_new_rows,
        max( len(hr_med_sum_number_new_rows) ) over () as max_len_hr_med_sum_number_new_rows,
        max( len(hr_avg_sum_number_new_rows) ) over () as max_len_hr_avg_sum_number_new_rows,
        max( len(hr_std_sum_number_new_rows) ) over () as max_len_hr_std_sum_number_new_rows,
        max( len(hr_sum_sum_number_new_rows) ) over () as max_len_hr_sum_sum_number_new_rows,

        max( len(hr_min_sum_number_deleted_rows) ) over () as max_len_hr_min_sum_number_deleted_rows,
        max( len(hr_max_sum_number_deleted_rows) ) over () as max_len_hr_max_sum_number_deleted_rows,
        max( len(hr_med_sum_number_deleted_rows) ) over () as max_len_hr_med_sum_number_deleted_rows,
        max( len(hr_avg_sum_number_deleted_rows) ) over () as max_len_hr_avg_sum_number_deleted_rows,
        max( len(hr_std_sum_number_deleted_rows) ) over () as max_len_hr_std_sum_number_deleted_rows,
        max( len(hr_sum_sum_number_deleted_rows) ) over () as max_len_hr_sum_sum_number_deleted_rows,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_qids,
        number_xids
      from
        lf_241129.vacuum_stats_for_all_by_minute
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_sum_number_new_blocks, max_len_hr_min_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_new_blocks, max_len_hr_max_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_new_blocks, max_len_hr_med_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_new_blocks, max_len_hr_avg_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_new_blocks, max_len_hr_std_sum_number_new_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_new_blocks, max_len_hr_sum_sum_number_new_blocks ) )::varchar as "new blocks",

    ( lpad( hr_min_sum_number_replaced_blocks, max_len_hr_min_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_max_sum_number_replaced_blocks, max_len_hr_max_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_med_sum_number_replaced_blocks, max_len_hr_med_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_avg_sum_number_replaced_blocks, max_len_hr_avg_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_std_sum_number_replaced_blocks, max_len_hr_std_sum_number_replaced_blocks ) || ' / ' ||
      lpad( hr_sum_sum_number_replaced_blocks, max_len_hr_sum_sum_number_replaced_blocks ) )::varchar as "replaced blocks",

    ( lpad( hr_min_sum_number_new_rows, max_len_hr_min_sum_number_new_rows ) || ' / ' ||
      lpad( hr_max_sum_number_new_rows, max_len_hr_max_sum_number_new_rows ) || ' / ' ||
      lpad( hr_med_sum_number_new_rows, max_len_hr_med_sum_number_new_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_new_rows, max_len_hr_avg_sum_number_new_rows ) || ' / ' ||
      lpad( hr_std_sum_number_new_rows, max_len_hr_std_sum_number_new_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_new_rows, max_len_hr_sum_sum_number_new_rows ) )::varchar as "new rows",

    ( lpad( hr_min_sum_number_deleted_rows, max_len_hr_min_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_max_sum_number_deleted_rows, max_len_hr_max_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_med_sum_number_deleted_rows, max_len_hr_med_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_avg_sum_number_deleted_rows, max_len_hr_avg_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_std_sum_number_deleted_rows, max_len_hr_std_sum_number_deleted_rows ) || ' / ' ||
      lpad( hr_sum_sum_number_deleted_rows, max_len_hr_sum_sum_number_deleted_rows ) )::varchar as "deleted rows",

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as "xid duration",

    number_qids::int8 as qids,
    number_xids::int8 as xids
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  sf_241129.queries_for_leader
as
  select
    user_name::varchar             as uuser,
    session_id::int4               as pid,
    transaction_id::int8           as xid,
    query_start_ts::timestamp      as start_ts,
    query_end_ts::timestamp        as end_ts,
    extract( epoch from query_total_duration )::numeric(18,2) as dur,
    left( ltrim(query_text), 48 )::varchar as ttext
  from
    lf_241129.queries_for_leader
  order by
    query_start_ts;


create or replace view
  base_241129.fixed_pg_proc_for_procedures
as
  select
    object_id::int4      as procedure_id,
    object_name::varchar as procedure_name,
    schema_id::int4,
    user_id::int4,
    language_id::int4,
    language_name::varchar,
    prosrc::text as procedure_text,
    proargtypes::oidvector
  from
    base_241129.fixed_pg_proc
  where
    language_name = 'plpgsql';


create or replace view
  sf_241129.analyze_totals_for_all_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_all_by_hour
    )

  select
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_all_by_hour
as
  select
    start_ts::timestamp,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_all_by_hour
  order by
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_query
as
  with
    cte_hr_formatting as
    (
      select
        query_id,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration
      from
        lf_241129.queries_stats_durations_on_workers_for_query
    )

  select
    query_id::int4 as qid,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec
  from
    cte_hr_formatting
  order by
    query_id;


create or replace view
  sf_241129.analyze_totals_for_tables_by_second
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_full_analyze::varchar                  as hr_number_manual_full_analyze,
        number_manual_predictatecolumn_analyze::varchar      as hr_number_manual_predictatecolumn_analyze,
        number_manual_skipped_analyze::varchar               as hr_number_manual_skipped_analyze,

        number_automatic_full_analyze::varchar               as hr_number_automatic_full_analyze,
        number_automatic_predictatecolumn_analyze::varchar   as hr_number_automatic_predictatecolumn_analyze,
        number_automatic_skipped_analyze::varchar            as hr_number_automatic_skipped_analyze,

        number_autoanalyze_full_analyze::varchar             as hr_number_autoanalyze_full_analyze,
        number_autoanalyze_predictatecolumn_analyze::varchar as hr_number_autoanalyze_predictatecolumn_analyze,
        number_autoanalyze_skipped_analyze::varchar          as hr_number_autoanalyze_skipped_analyze,

        number_total_full_analyze::varchar                   as hr_number_total_full_analyze,
        number_total_predictatecolumn_analyze::varchar       as hr_number_total_predictatecolumn_analyze,
        number_total_skipped_analyze::varchar                as hr_number_total_skipped_analyze,

        max( len(hr_number_manual_full_analyze)             ) over () as max_len_hr_number_manual_full_analyze,
        max( len(hr_number_manual_predictatecolumn_analyze) ) over () as max_len_hr_number_manual_predictatecolumn_analyze,
        max( len(hr_number_manual_skipped_analyze)          ) over () as max_len_hr_number_manual_skipped_analyze,

        max( len(hr_number_automatic_full_analyze)             ) over () as max_len_hr_number_automatic_full_analyze,
        max( len(hr_number_automatic_predictatecolumn_analyze) ) over () as max_len_hr_number_automatic_predictatecolumn_analyze,
        max( len(hr_number_automatic_skipped_analyze)          ) over () as max_len_hr_number_automatic_skipped_analyze,

        max( len(hr_number_autoanalyze_full_analyze)             ) over () as max_len_hr_number_autoanalyze_full_analyze,
        max( len(hr_number_autoanalyze_predictatecolumn_analyze) ) over () as max_len_hr_number_autoanalyze_predictatecolumn_analyze,
        max( len(hr_number_autoanalyze_skipped_analyze)          ) over () as max_len_hr_number_autoanalyze_skipped_analyze,

        max( len(hr_number_total_full_analyze)             ) over () as max_len_hr_number_total_full_analyze,
        max( len(hr_number_total_predictatecolumn_analyze) ) over () as max_len_hr_number_total_predictatecolumn_analyze,
        max( len(hr_number_total_skipped_analyze)          ) over () as max_len_hr_number_total_skipped_analyze
      from
        lf_241129.analyze_totals_for_tables_by_second
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::timestamp,

    ( lpad( hr_number_manual_full_analyze, max_len_hr_number_manual_full_analyze                         ) || ' / ' ||
      lpad( hr_number_manual_predictatecolumn_analyze, max_len_hr_number_manual_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_manual_skipped_analyze, max_len_hr_number_manual_skipped_analyze                   ) )::varchar as manual,

    ( lpad( hr_number_automatic_full_analyze, max_len_hr_number_automatic_full_analyze                         ) || ' / ' ||
      lpad( hr_number_automatic_predictatecolumn_analyze, max_len_hr_number_automatic_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_automatic_skipped_analyze, max_len_hr_number_automatic_skipped_analyze                   ) )::varchar as automatic,

    ( lpad( hr_number_autoanalyze_full_analyze, max_len_hr_number_autoanalyze_full_analyze                         ) || ' / ' ||
      lpad( hr_number_autoanalyze_predictatecolumn_analyze, max_len_hr_number_autoanalyze_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_autoanalyze_skipped_analyze, max_len_hr_number_autoanalyze_skipped_analyze                   ) )::varchar as autoanalyze,

    ( lpad( hr_number_total_full_analyze, max_len_hr_number_total_full_analyze                         ) || ' / ' ||
      lpad( hr_number_total_predictatecolumn_analyze, max_len_hr_number_total_predictatecolumn_analyze ) || ' / ' ||
      lpad( hr_number_total_skipped_analyze, max_len_hr_number_total_skipped_analyze                   ) )::varchar as total
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  sf_241129.locks_by_table
as
  select
    user_name::varchar          as uuser,
    session_id::int4::int4      as pid,
    transaction_id::int8::int8  as xid,
    schema_name::varchar        as sschema,
    table_name::varchar         as ttable,
    extract( epoch from duration )::numeric(18,2) as dur,
    lock_type::varchar          as ttype,
    case
      when lock_state = 'granted'   then '✔'
      when lock_state = 'requested' then ''
    end::varchar                as s,
    query_text::varchar         as ttext
  from
    lf_241129.locks
  order by
    schema_name,
    table_name,
    duration desc,
    ttype;


create or replace view
  sf_241129.pprocedures_text
as
  select
    procedure_id::int4,
    procedure_text::varchar
  from
    lf_241129.pprocedures_text
  order by
    procedure_id;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_users_by_all
as
  with
    cte_hr_formatting as
    (
      select
        node_type,
        user_name,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_users_by_all
    )

 select
    node_type::varchar as node,
    user_name::varchar as uuser,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_type,
    user_name;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_all_by_all
as
  select
    ( extract(epoch from min_query_queue_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_query_queue_duration)::numeric(18,2) || ' / ' ||
      med_query_queue_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_query_queue_duration)::numeric(18,2) || ' / ' ||
      std_query_queue_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from sum_query_queue_duration)::numeric(18,2) )::varchar as queue,

    ( extract(epoch from min_query_compile_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_query_compile_duration)::numeric(18,2) || ' / ' ||
      med_query_compile_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_query_compile_duration)::numeric(18,2) || ' / ' ||
      std_query_compile_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from sum_query_compile_duration)::numeric(18,2) )::varchar as compile,

    ( extract(epoch from min_query_exec_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_query_exec_duration)::numeric(18,2) || ' / ' ||
      med_query_exec_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_query_exec_duration)::numeric(18,2) || ' / ' ||
      std_query_exec_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from sum_query_exec_duration)::numeric(18,2) )::varchar as exec,

    number_queries::int4 as queries
  from
    lf_241129.queries_stats_durations_on_workers_for_all_by_all;


create or replace view
  lf_241129.blocks_rows_totals_for_tables_by_node
as
  with
    cte_blocks_and_rows as
    (
      select
        stv_blocklist.tbl                                                                                              as table_id,
        stv_blocklist.slice                                                                                            as slice_ordinal,
        sum( 1-stv_blocklist.unsorted )                                                                                as number_sorted_blocks,
        sum(   stv_blocklist.unsorted )                                                                                as number_unsorted_blocks,
        count( * )                                                                                                     as number_total_blocks,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 0 then stv_blocklist.num_values else 0 end ) as number_sorted_rows,
        sum( case when stv_blocklist.col = 0 and stv_blocklist.unsorted = 1 then stv_blocklist.num_values else 0 end ) as number_unsorted_rows,
        sum( case when stv_blocklist.col = 0                                then stv_blocklist.num_values else 0 end ) as number_total_rows
      from
        stv_blocklist
      where
        tbl != 0
      group by
        stv_blocklist.tbl,
        stv_blocklist.slice
    )

  select
    schema_id::int4,
    schema_name::varchar,
    table_id::int4,
    table_name::varchar,
    table_distribution_type::varchar,
    table_distribution_auto_flag::bool,
    node_ordinal::int2,
    sum( number_sorted_blocks   )::int8 as number_sorted_blocks,
    sum( number_unsorted_blocks )::int8 as number_unsorted_blocks,
    sum( number_total_blocks    )::int8 as number_total_blocks,
    sum( number_sorted_rows     )::int8 as number_sorted_rows,
    sum( number_unsorted_rows   )::int8 as number_unsorted_rows,
    sum( number_total_rows      )::int8 as number_total_rows
  from
              cte_blocks_and_rows
    left join base_241129.ttables_basic_info_fast using ( table_id )
    left join base_241129.fixed_stv_slices           using ( slice_ordinal )
  group by
    schema_id,
    schema_name,
    table_id,
    table_name,
    table_distribution_type,
    table_distribution_auto_flag,
    node_ordinal;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_schemas_by_day
as
  with
    cte_s1 as
    (
      select
        table_id,
        case
          when slice_ordinal  = -1 then 'leader'
          when slice_ordinal != -1 then 'worker'
        end::varchar                                     as node_type,
        date_trunc( 'day', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        table_id,
        node_type,
        start_ts
    )

  select
    pg_class.relnamespace::int4            as schema_id,
    trim( pg_namespace.nspname )::varchar  as schema_name,
    node_type::varchar,
    start_ts::timestamp,

    sum( sum_type_1_rows  )::int8 as sum_type_1_rows,
    sum( sum_type_1_bytes )::int8 as sum_type_1_bytes,

    sum( sum_type_2m_rows )::int8 as sum_type_2m_rows,
    sum( sum_type_2e_rows )::int8 as sum_type_2e_rows,
    sum( sum_type_2_rows  )::int8 as sum_type_2_rows,
    sum( sum_type_2_bytes )::int8 as sum_type_2_bytes,

    sum( sum_type_3_rows  )::int8 as sum_type_3_rows,
    sum( sum_type_3_bytes )::int8 as sum_type_3_bytes,

    sum( sum_type_9_rows  )::int8 as sum_type_9_rows,
    sum( sum_type_9_bytes )::int8 as sum_type_9_bytes,

    sum( sum_type_16_rows  )::int8 as sum_type_16_rows,
    sum( sum_type_16_bytes )::int8 as sum_type_16_bytes,

    sum( sum_type_17_rows  )::int8 as sum_type_17_rows,
    sum( sum_type_17_bytes )::int8 as sum_type_17_bytes,

    sum( sum_type_18_rows  )::int8 as sum_type_18_rows,
    sum( sum_type_18_bytes )::int8 as sum_type_18_bytes,

    sum( sum_type_21_rows  )::int8 as sum_type_21_rows,
    sum( sum_type_21_bytes )::int8 as sum_type_21_bytes,

    sum( sum_type_22_rows  )::int8 as sum_type_22_rows,
    sum( sum_type_22_bytes )::int8 as sum_type_22_bytes,

    sum( sum_type_23_rows  )::int8 as sum_type_23_rows,
    sum( sum_type_23_bytes )::int8 as sum_type_23_bytes,

    sum( sum_type_24_rows  )::int8 as sum_type_24_rows,
    sum( sum_type_24_bytes )::int8 as sum_type_24_bytes,

    sum( sum_type_25_rows  )::int8 as sum_type_25_rows,
    sum( sum_type_25_bytes )::int8 as sum_type_25_bytes,

    sum( sum_type_28_rows  )::int8 as sum_type_28_rows,
    sum( sum_type_28_bytes )::int8 as sum_type_28_bytes,

    sum( sum_type_29m_rows )::int8 as sum_type_29m_rows,
    sum( sum_type_29e_rows )::int8 as sum_type_29e_rows,
    sum( sum_type_29_rows  )::int8 as sum_type_29_rows,
    sum( sum_type_29_bytes )::int8 as sum_type_29_bytes,

    sum( sum_type_30_rows  )::int8 as sum_type_30_rows,
    sum( sum_type_30_bytes )::int8 as sum_type_30_bytes,

    sum( sum_type_32_rows  )::int8 as sum_type_32_rows,
    sum( sum_type_32_bytes )::int8 as sum_type_32_bytes,

    sum( sum_type_33_rows  )::int8 as sum_type_33_rows,
    sum( sum_type_33_bytes )::int8 as sum_type_33_bytes,

    sum( sum_type_35_rows  )::int8 as sum_type_35_rows,
    sum( sum_type_35_bytes )::int8 as sum_type_35_bytes,

    sum( sum_type_36_rows  )::int8 as sum_type_36_rows,
    sum( sum_type_36_bytes )::int8 as sum_type_36_bytes,

    sum( sum_type_37_rows  )::int8 as sum_type_37_rows,
    sum( sum_type_37_bytes )::int8 as sum_type_37_bytes
  from
              cte_s1
    left join pg_class     on cte_s1.table_id       = pg_class.oid
    left join pg_namespace on pg_class.relnamespace = pg_namespace.oid
  where
    pg_class.relkind  = 'r'
  group by
    schema_id,
    schema_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_schemas_by_day
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_schemas_by_day
    )

  select
    schema_name::varchar as sschema,
    node_type::varchar   as node,
    start_ts::date,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_queues_by_day
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_queues_by_day
    )

  select
    queue_name::varchar as qname,
    node_type::varchar  as node,
    start_ts::date,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    queue_name,
    node_type,
    start_ts;


create or replace view
  lf_241129.steps_totals_scan_data_on_both_for_slices_by_minute
as
  with
    cte_s1 as
    (
      select
        slice_ordinal,
        date_trunc( 'minute', scan_start_ts )               as start_ts,

        sum( case when type =  1 then rows  else 0 end ) as sum_type_1_rows,
        sum( case when type =  1 then bytes else 0 end ) as sum_type_1_bytes,

        sum( case when type =  2 then rows_pre_filter      else 0 end ) as sum_type_2m_rows,
        sum( case when type =  2 then rows_pre_user_filter else 0 end ) as sum_type_2e_rows,
        sum( case when type =  2 then rows                 else 0 end ) as sum_type_2_rows,
        sum( case when type =  2 then bytes                else 0 end ) as sum_type_2_bytes,

        sum( case when type =  3 then rows  else 0 end ) as sum_type_3_rows,
        sum( case when type =  3 then bytes else 0 end ) as sum_type_3_bytes,

        sum( case when type =  9 then rows  else 0 end ) as sum_type_9_rows,
        sum( case when type =  9 then bytes else 0 end ) as sum_type_9_bytes,

        sum( case when type = 16 then rows  else 0 end ) as sum_type_16_rows,
        sum( case when type = 16 then bytes else 0 end ) as sum_type_16_bytes,

        sum( case when type = 17 then rows  else 0 end ) as sum_type_17_rows,
        sum( case when type = 17 then bytes else 0 end ) as sum_type_17_bytes,

        sum( case when type = 18 then rows  else 0 end ) as sum_type_18_rows,
        sum( case when type = 18 then bytes else 0 end ) as sum_type_18_bytes,

        sum( case when type = 21 then rows  else 0 end ) as sum_type_21_rows,
        sum( case when type = 21 then bytes else 0 end ) as sum_type_21_bytes,

        sum( case when type = 22 then rows  else 0 end ) as sum_type_22_rows,
        sum( case when type = 22 then bytes else 0 end ) as sum_type_22_bytes,

        sum( case when type = 23 then rows  else 0 end ) as sum_type_23_rows,
        sum( case when type = 23 then bytes else 0 end ) as sum_type_23_bytes,

        sum( case when type = 24 then rows  else 0 end ) as sum_type_24_rows,
        sum( case when type = 24 then bytes else 0 end ) as sum_type_24_bytes,

        sum( case when type = 25 then rows  else 0 end ) as sum_type_25_rows,
        sum( case when type = 25 then bytes else 0 end ) as sum_type_25_bytes,

        sum( case when type = 28 then rows  else 0 end ) as sum_type_28_rows,
        sum( case when type = 28 then bytes else 0 end ) as sum_type_28_bytes,

        sum( case when type = 29 then rows_pre_filter      else 0 end ) as sum_type_29m_rows,
        sum( case when type = 29 then rows_pre_user_filter else 0 end ) as sum_type_29e_rows,
        sum( case when type = 29 then rows                 else 0 end ) as sum_type_29_rows,
        sum( case when type = 29 then bytes                else 0 end ) as sum_type_29_bytes,

        sum( case when type = 30 then rows  else 0 end ) as sum_type_30_rows,
        sum( case when type = 30 then bytes else 0 end ) as sum_type_30_bytes,

        sum( case when type = 32 then rows  else 0 end ) as sum_type_32_rows,
        sum( case when type = 32 then bytes else 0 end ) as sum_type_32_bytes,

        sum( case when type = 33 then rows  else 0 end ) as sum_type_33_rows,
        sum( case when type = 33 then bytes else 0 end ) as sum_type_33_bytes,

        sum( case when type = 35 then rows  else 0 end ) as sum_type_35_rows,
        sum( case when type = 35 then bytes else 0 end ) as sum_type_35_bytes,

        sum( case when type = 36 then rows  else 0 end ) as sum_type_36_rows,
        sum( case when type = 36 then bytes else 0 end ) as sum_type_36_bytes,

        sum( case when type = 37 then rows  else 0 end ) as sum_type_37_rows,
        sum( case when type = 37 then bytes else 0 end ) as sum_type_37_bytes
      from
        base_241129.fixed_stl_scan
      where
            scan_start_ts >= current_date - interval '6 day'
        and scan_end_ts   <= current_timestamp
      group by
        slice_ordinal,
        start_ts
    )

  select
    node_ordinal::int2,
    slice_ordinal::int2,
    slice_type::varchar,
    start_ts::timestamp,

    sum_type_1_rows::int8,
    sum_type_1_bytes::int8,

    sum_type_2m_rows::int8,
    sum_type_2e_rows::int8,
    sum_type_2_rows::int8,
    sum_type_2_bytes::int8,

    sum_type_3_rows::int8,
    sum_type_3_bytes::int8,

    sum_type_9_rows::int8,
    sum_type_9_bytes::int8,

    sum_type_16_rows::int8,
    sum_type_16_bytes::int8,

    sum_type_17_rows::int8,
    sum_type_17_bytes::int8,

    sum_type_18_rows::int8,
    sum_type_18_bytes::int8,

    sum_type_21_rows::int8,
    sum_type_21_bytes::int8,

    sum_type_22_rows::int8,
    sum_type_22_bytes::int8,

    sum_type_23_rows::int8,
    sum_type_23_bytes::int8,

    sum_type_24_rows::int8,
    sum_type_24_bytes::int8,

    sum_type_25_rows::int8,
    sum_type_25_bytes::int8,

    sum_type_28_rows::int8,
    sum_type_28_bytes::int8,

    sum_type_29m_rows::int8,
    sum_type_29e_rows::int8,
    sum_type_29_rows::int8,
    sum_type_29_bytes::int8,

    sum_type_30_rows::int8,
    sum_type_30_bytes::int8,

    sum_type_32_rows::int8,
    sum_type_32_bytes::int8,

    sum_type_33_rows::int8,
    sum_type_33_bytes::int8,

    sum_type_35_rows::int8,
    sum_type_35_bytes::int8,

    sum_type_36_rows::int8,
    sum_type_36_bytes::int8,

    sum_type_37_rows::int8,
    sum_type_37_bytes::int8
  from
              cte_s1
    left join base_241129.fixed_stv_slices using ( slice_ordinal );


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_slices_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        slice_ordinal,
        slice_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_slices_by_minute
    )

  select
    node_ordinal::int2  as n,
    slice_ordinal::int2 as s,
    left(slice_type,1)::varchar as t,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    slice_ordinal,
    start_ts;


create or replace view
  lf_241129.queries_compilation_stream_segment
as
  with
    cte_queries_compilation_stream_segment as
    (
      select
        svl_compile.query                                       as query_id,
        stl_stream_segs.stream                                  as stream_ordinal,
        svl_compile.segment                                     as segment_ordinal,
        svl_compile.starttime                                   as compilation_start,
        svl_compile.endtime                                     as compilation_end,
        ( svl_compile.endtime - svl_compile.starttime )         as compilation_duration,
        case
          when svl_compile.compile = 0 then 'leader-node cache'
          when svl_compile.compile = 1 then 'compile off-load'
                                       else 'unknown'
        end                                                     as compilation_status
      from
        -- MG2 : no stream in svl_compile
                     svl_compile
        natural join stl_stream_segs
    )

  select
    query_id::int4,
    stream_ordinal::int2,
    segment_ordinal::int2,
    compilation_start::timestamp,
    compilation_end::timestamp,
    compilation_duration::interval,
    compilation_status::varchar
  from
    cte_queries_compilation_stream_segment;


create or replace view
  sf_241129.queries_compilation_stream_segment
as
  select
    query_id::int4                as qid,
    stream_ordinal::int2          as stream,
    segment_ordinal::int2         as segment,
    compilation_start::timestamp  as start_ts,
    compilation_end::timestamp    as end_ts,
    extract( epoch from compilation_duration )::numeric(18,6) as dur,
    compilation_status::varchar   as status
  from
    lf_241129.queries_compilation_stream_segment
  order by
    query_id,
    stream_ordinal,
    segment_ordinal;


create or replace view
  sf_241129.transactions_totals_commits_by_all
as
  select
    extract( epoch from sum_batch_leader_duration )::numeric(18,2) as leader,
    extract( epoch from sum_batch_worker_duration )::numeric(18,2) as worker,
    extract( epoch from sum_batch_idle_duration   )::numeric(18,2) as idle,
    extract( epoch from sum_batch_exec_duration   )::numeric(18,2) as exec
  from
    lf_241129.transactions_totals_commits_by_all;


create or replace view
  lf_241129.queries_totals_aborts_on_workers_for_queues_by_second
as
  select
    queue_id::int2,
    queue_name::varchar,
    start_ts::timestamp,
    count(*)::int4 as number_abort
  from
              base_241129.generate_seconds
    left join base_241129.fixed_stl_query                       on date_trunc( 'second', query_start_ts ) = base_241129.generate_seconds.start_ts
         join base_241129.fixed_stl_wlm_query                using ( query_id )
    left join base_241129.fixed_stv_wlm_service_class_config using ( queue_id )
  where
        query_abort_flag  = true
    and query_csc_flag    = false
    and query_start_ts   >= current_date - interval '6 day'
    and query_end_ts     <= current_timestamp
  group by
    queue_id,
    queue_name,
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_queues_by_second
as
  select
    queue_name::varchar as qname,
    start_ts::timestamp,
    number_abort::int4  as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_queues_by_second
  order by
    queue_name,
    start_ts;


create or replace view
  sf_241129.blocks_rows_totals_for_tables_by_slice
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        table_distribution_type,
        table_distribution_auto_flag,
        node_ordinal,
        slice_ordinal,
        slice_type,

        base_241129.hr_blks( number_sorted_blocks   ) as hr_number_sorted_blocks,
        base_241129.hr_blks( number_unsorted_blocks ) as hr_number_unsorted_blocks,
        base_241129.hr_blks( number_total_blocks    ) as hr_number_total_blocks,

        base_241129.hr_number( number_sorted_rows   ) as hr_number_sorted_rows,
        base_241129.hr_number( number_unsorted_rows ) as hr_number_unsorted_rows,
        base_241129.hr_number( number_total_rows    ) as hr_number_total_rows,

        max( len(hr_number_sorted_blocks)   ) over () as max_len_hr_number_sorted_blocks,
        max( len(hr_number_unsorted_blocks) ) over () as max_len_hr_number_unsorted_blocks,
        max( len(hr_number_total_blocks)    ) over () as max_len_hr_number_total_blocks,

        max( len(hr_number_sorted_rows)   ) over () as max_len_hr_number_sorted_rows,
        max( len(hr_number_unsorted_rows) ) over () as max_len_hr_number_unsorted_rows,
        max( len(hr_number_total_rows)    ) over () as max_len_hr_number_total_rows
      from
        lf_241129.blocks_rows_totals_for_tables_by_slice
    )

  select
    schema_name::varchar              as sschema,
    table_name::varchar               as ttable,
    table_distribution_type::varchar  as dist,
    case
      when table_distribution_auto_flag = True  then '✔'
      when table_distribution_auto_flag = false then ''
    end::varchar                      as a,
    node_ordinal::int2                as n,
    slice_ordinal::int2               as s,
    slice_type::varchar               as t,

    lpad( hr_number_sorted_blocks,   max_len_hr_number_sorted_blocks   )::varchar  as sblks,
    lpad( hr_number_unsorted_blocks, max_len_hr_number_unsorted_blocks )::varchar  as ublks,
    lpad( hr_number_total_blocks,    max_len_hr_number_total_blocks    )::varchar  as tblks,

    lpad( hr_number_sorted_rows,   max_len_hr_number_sorted_rows   )::varchar      as srows,
    lpad( hr_number_unsorted_rows, max_len_hr_number_unsorted_rows )::varchar      as urows,
    lpad( hr_number_total_rows,    max_len_hr_number_total_rows    )::varchar      as trows
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    node_ordinal,
    slice_ordinal;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_minute_concise
as
  select
    node_type::varchar as node,
    start_ts::timestamp,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_minute
  order by
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_day_concise
as
  select
    start_ts::date,
    node_ordinal::int2 as n,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_day
  order by
    start_ts,
    node_ordinal;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_users_by_day
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        extract(epoch from min_leadin_duration)::numeric(18,2)::varchar as hr_min_leadin_duration,
        extract(epoch from max_leadin_duration)::numeric(18,2)::varchar as hr_max_leadin_duration,
        med_leadin_duration::numeric(18,2)::varchar                     as hr_med_leadin_duration,
        extract(epoch from avg_leadin_duration)::numeric(18,2)::varchar as hr_avg_leadin_duration,
        std_leadin_duration::numeric(18,2)::varchar                     as hr_std_leadin_duration,
        extract(epoch from sum_leadin_duration)::numeric(18,2)::varchar as hr_sum_leadin_duration,

        extract(epoch from min_wlm_leadin_duration)::numeric(18,2)::varchar as hr_min_wlm_leadin_duration,
        extract(epoch from max_wlm_leadin_duration)::numeric(18,2)::varchar as hr_max_wlm_leadin_duration,
        med_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadin_duration,
        extract(epoch from avg_wlm_leadin_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadin_duration,
        std_wlm_leadin_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadin_duration,
        extract(epoch from sum_wlm_leadin_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadin_duration,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        extract(epoch from min_wlm_leadout_duration)::numeric(18,2)::varchar as hr_min_wlm_leadout_duration,
        extract(epoch from max_wlm_leadout_duration)::numeric(18,2)::varchar as hr_max_wlm_leadout_duration,
        med_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_med_wlm_leadout_duration,
        extract(epoch from avg_wlm_leadout_duration)::numeric(18,2)::varchar as hr_avg_wlm_leadout_duration,
        std_wlm_leadout_duration::numeric(18,2)::varchar                     as hr_std_wlm_leadout_duration,
        extract(epoch from sum_wlm_leadout_duration)::numeric(18,2)::varchar as hr_sum_wlm_leadout_duration,

        extract(epoch from min_leadout_duration)::numeric(18,2)::varchar as hr_min_leadout_duration,
        extract(epoch from max_leadout_duration)::numeric(18,2)::varchar as hr_max_leadout_duration,
        med_leadout_duration::numeric(18,2)::varchar                     as hr_med_leadout_duration,
        extract(epoch from avg_leadout_duration)::numeric(18,2)::varchar as hr_avg_leadout_duration,
        std_leadout_duration::numeric(18,2)::varchar                     as hr_std_leadout_duration,
        extract(epoch from sum_leadout_duration)::numeric(18,2)::varchar as hr_sum_leadout_duration,

        extract(epoch from min_query_total_duration)::numeric(18,2)::varchar as hr_min_query_total_duration,
        extract(epoch from max_query_total_duration)::numeric(18,2)::varchar as hr_max_query_total_duration,
        med_query_total_duration::numeric(18,2)::varchar                     as hr_med_query_total_duration,
        extract(epoch from avg_query_total_duration)::numeric(18,2)::varchar as hr_avg_query_total_duration,
        std_query_total_duration::numeric(18,2)::varchar                     as hr_std_query_total_duration,
        extract(epoch from sum_query_total_duration)::numeric(18,2)::varchar as hr_sum_query_total_duration,

        extract(epoch from min_unaccounted_duration)::numeric(18,2)::varchar as hr_min_unaccounted_duration,
        extract(epoch from max_unaccounted_duration)::numeric(18,2)::varchar as hr_max_unaccounted_duration,
        med_unaccounted_duration::numeric(18,2)::varchar                     as hr_med_unaccounted_duration,
        extract(epoch from avg_unaccounted_duration)::numeric(18,2)::varchar as hr_avg_unaccounted_duration,
        std_unaccounted_duration::numeric(18,2)::varchar                     as hr_std_unaccounted_duration,
        extract(epoch from sum_unaccounted_duration)::numeric(18,2)::varchar as hr_sum_unaccounted_duration,

        max( len(hr_min_leadin_duration) ) over () as max_len_hr_min_leadin_duration,
        max( len(hr_max_leadin_duration) ) over () as max_len_hr_max_leadin_duration,
        max( len(hr_med_leadin_duration) ) over () as max_len_hr_med_leadin_duration,
        max( len(hr_avg_leadin_duration) ) over () as max_len_hr_avg_leadin_duration,
        max( len(hr_std_leadin_duration) ) over () as max_len_hr_std_leadin_duration,
        max( len(hr_sum_leadin_duration) ) over () as max_len_hr_sum_leadin_duration,

        max( len(hr_min_wlm_leadin_duration) ) over () as max_len_hr_min_wlm_leadin_duration,
        max( len(hr_max_wlm_leadin_duration) ) over () as max_len_hr_max_wlm_leadin_duration,
        max( len(hr_med_wlm_leadin_duration) ) over () as max_len_hr_med_wlm_leadin_duration,
        max( len(hr_avg_wlm_leadin_duration) ) over () as max_len_hr_avg_wlm_leadin_duration,
        max( len(hr_std_wlm_leadin_duration) ) over () as max_len_hr_std_wlm_leadin_duration,
        max( len(hr_sum_wlm_leadin_duration) ) over () as max_len_hr_sum_wlm_leadin_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        max( len(hr_min_wlm_leadout_duration) ) over () as max_len_hr_min_wlm_leadout_duration,
        max( len(hr_max_wlm_leadout_duration) ) over () as max_len_hr_max_wlm_leadout_duration,
        max( len(hr_med_wlm_leadout_duration) ) over () as max_len_hr_med_wlm_leadout_duration,
        max( len(hr_avg_wlm_leadout_duration) ) over () as max_len_hr_avg_wlm_leadout_duration,
        max( len(hr_std_wlm_leadout_duration) ) over () as max_len_hr_std_wlm_leadout_duration,
        max( len(hr_sum_wlm_leadout_duration) ) over () as max_len_hr_sum_wlm_leadout_duration,

        max( len(hr_min_leadout_duration) ) over () as max_len_hr_min_leadout_duration,
        max( len(hr_max_leadout_duration) ) over () as max_len_hr_max_leadout_duration,
        max( len(hr_med_leadout_duration) ) over () as max_len_hr_med_leadout_duration,
        max( len(hr_avg_leadout_duration) ) over () as max_len_hr_avg_leadout_duration,
        max( len(hr_std_leadout_duration) ) over () as max_len_hr_std_leadout_duration,
        max( len(hr_sum_leadout_duration) ) over () as max_len_hr_sum_leadout_duration,

        max( len(hr_min_query_total_duration) ) over () as max_len_hr_min_query_total_duration,
        max( len(hr_max_query_total_duration) ) over () as max_len_hr_max_query_total_duration,
        max( len(hr_med_query_total_duration) ) over () as max_len_hr_med_query_total_duration,
        max( len(hr_avg_query_total_duration) ) over () as max_len_hr_avg_query_total_duration,
        max( len(hr_std_query_total_duration) ) over () as max_len_hr_std_query_total_duration,
        max( len(hr_sum_query_total_duration) ) over () as max_len_hr_sum_query_total_duration,

        max( len(hr_min_unaccounted_duration) ) over () as max_len_hr_min_unaccounted_duration,
        max( len(hr_max_unaccounted_duration) ) over () as max_len_hr_max_unaccounted_duration,
        max( len(hr_med_unaccounted_duration) ) over () as max_len_hr_med_unaccounted_duration,
        max( len(hr_avg_unaccounted_duration) ) over () as max_len_hr_avg_unaccounted_duration,
        max( len(hr_std_unaccounted_duration) ) over () as max_len_hr_std_unaccounted_duration,
        max( len(hr_sum_unaccounted_duration) ) over () as max_len_hr_sum_unaccounted_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_users_by_day
    )

  select
    user_name::varchar as uuser,
    start_ts::date,

    ( lpad( hr_min_leadin_duration, max_len_hr_min_leadin_duration ) || ' / ' ||
      lpad( hr_max_leadin_duration, max_len_hr_max_leadin_duration ) || ' / ' ||
      lpad( hr_med_leadin_duration, max_len_hr_med_leadin_duration ) || ' / ' ||
      lpad( hr_avg_leadin_duration, max_len_hr_avg_leadin_duration ) || ' / ' ||
      lpad( hr_std_leadin_duration, max_len_hr_std_leadin_duration ) || ' / ' ||
      lpad( hr_sum_leadin_duration, max_len_hr_sum_leadin_duration ) )::varchar as leadin,

    ( lpad( hr_min_wlm_leadin_duration, max_len_hr_min_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadin_duration, max_len_hr_max_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadin_duration, max_len_hr_med_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadin_duration, max_len_hr_avg_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadin_duration, max_len_hr_std_wlm_leadin_duration ) || ' / ' ||
      lpad( hr_sum_wlm_leadin_duration, max_len_hr_sum_wlm_leadin_duration ) )::varchar as wlm_leadin,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    ( lpad( hr_min_wlm_leadout_duration, max_len_hr_min_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_max_wlm_leadout_duration, max_len_hr_max_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_med_wlm_leadout_duration, max_len_hr_med_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_avg_wlm_leadout_duration, max_len_hr_avg_wlm_leadout_duration ) || ' / ' ||
      lpad( hr_std_wlm_leadout_duration, max_len_hr_std_wlm_leadout_duration ) || ' / ' || 
      lpad( hr_sum_wlm_leadout_duration, max_len_hr_sum_wlm_leadout_duration ) )::varchar as wlm_leadout,

    ( lpad( hr_min_leadout_duration, max_len_hr_min_leadout_duration ) || ' / ' ||
      lpad( hr_max_leadout_duration, max_len_hr_max_leadout_duration ) || ' / ' ||
      lpad( hr_med_leadout_duration, max_len_hr_med_leadout_duration ) || ' / ' ||
      lpad( hr_avg_leadout_duration, max_len_hr_avg_leadout_duration ) || ' / ' ||
      lpad( hr_std_leadout_duration, max_len_hr_std_leadout_duration ) || ' / ' || 
      lpad( hr_sum_leadout_duration, max_len_hr_sum_leadout_duration ) )::varchar as leadout,

    ( lpad( hr_min_query_total_duration, max_len_hr_min_query_total_duration ) || ' / ' ||
      lpad( hr_max_query_total_duration, max_len_hr_max_query_total_duration ) || ' / ' ||
      lpad( hr_med_query_total_duration, max_len_hr_med_query_total_duration ) || ' / ' ||
      lpad( hr_avg_query_total_duration, max_len_hr_avg_query_total_duration ) || ' / ' ||
      lpad( hr_std_query_total_duration, max_len_hr_std_query_total_duration ) || ' / ' || 
      lpad( hr_sum_query_total_duration, max_len_hr_sum_query_total_duration ) )::varchar as total,

    ( lpad( hr_min_unaccounted_duration, max_len_hr_min_unaccounted_duration ) || ' / ' ||
      lpad( hr_max_unaccounted_duration, max_len_hr_max_unaccounted_duration ) || ' / ' ||
      lpad( hr_med_unaccounted_duration, max_len_hr_med_unaccounted_duration ) || ' / ' ||
      lpad( hr_avg_unaccounted_duration, max_len_hr_avg_unaccounted_duration ) || ' / ' ||
      lpad( hr_std_unaccounted_duration, max_len_hr_std_unaccounted_duration ) || ' / ' || 
      lpad( hr_sum_unaccounted_duration, max_len_hr_sum_unaccounted_duration ) )::varchar as unact,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_users_by_minute
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_users_by_minute
    )

  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_all_by_day_complete
as
  select
    node_type::varchar,
    start_ts::date,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                  || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_all_by_day
  order by
    node_type,
    start_ts;



create or replace view
  sf_241129.views_late_binding
as
  select
    schema_name::varchar as sschema,
    view_name::varchar   as vview,
    user_name::varchar   as uuser
  from
    lf_241129.views_late_binding
  order by
    schema_name,
    view_name;


create or replace view
  sf_241129.vacuum_totals_for_users_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_users_by_hour
    )

  select
    user_name::varchar as uuser,
    start_ts::timestamp,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.queries_totals_result_cache_for_all_by_minute
as
  select
    start_ts::timestamp,
    number_hit::int8  as hit,
    number_miss::int8 as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_all_by_minute
  order by
    start_ts;


create or replace view
  sf_241129.queries_totals_aborts_on_workers_for_all_by_hour
as
  select
    start_ts::timestamp,
    number_abort::int4 as aborts
  from
    lf_241129.queries_totals_aborts_on_workers_for_all_by_hour
  order by
    start_ts;


create or replace view
  sf_241129.docs
as
  select
    schema_name::varchar as sschema,
    object_name::varchar as oobject,
    object_type::varchar as ttype,
    doc::varchar         as doc
  from
    base_241129.docs
  order by
    schema_name,
    object_name,
    object_type;


create or replace view
  sf_241129.wqueues_qmr_rules
as
  select
    rule_ordinal::int2   as oorder,
    rule_name::varchar   as rrule,
    queue_name::varchar  as qname,
    rule_action::varchar as aaction,
    listagg( metric_name || ' ' || metric_operator || ' ' || metric_value, '\n' ) within group ( order by metric_name )::varchar as conditions
  from
    lf_241129.wqueues_qmr_rules
  group by
    rule_ordinal,
    rule_name,
    queue_name,
    rule_action
  order by
    rule_ordinal;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_queues_by_second
as
  with
    cte_hr_formatting as
    (
      select
        queue_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_queues_by_second
    )

  select
    queue_name::varchar as qname,
    node_type::varchar  as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    queue_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.queries_stats_durations_on_both_by_transaction
as
  with
    cte_hr_formatting as
    (
      select
        transaction_id,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        extract(epoch from min_idle_duration)::numeric(18,2)::varchar as hr_min_idle_duration,
        extract(epoch from max_idle_duration)::numeric(18,2)::varchar as hr_max_idle_duration,
        med_idle_duration::numeric(18,2)::varchar                     as hr_med_idle_duration,
        extract(epoch from avg_idle_duration)::numeric(18,2)::varchar as hr_avg_idle_duration,
        std_idle_duration::numeric(18,2)::varchar                     as hr_std_idle_duration,
        extract(epoch from sum_idle_duration)::numeric(18,2)::varchar as hr_sum_idle_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        max( len(hr_min_idle_duration) ) over () as max_len_hr_min_idle_duration,
        max( len(hr_max_idle_duration) ) over () as max_len_hr_max_idle_duration,
        max( len(hr_med_idle_duration) ) over () as max_len_hr_med_idle_duration,
        max( len(hr_avg_idle_duration) ) over () as max_len_hr_avg_idle_duration,
        max( len(hr_std_idle_duration) ) over () as max_len_hr_std_idle_duration,
        max( len(hr_sum_idle_duration) ) over () as max_len_hr_sum_idle_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_both_by_transaction
    )

  select
    transaction_id::int8 as xid,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    ( lpad( hr_min_idle_duration, max_len_hr_min_idle_duration ) || ' / ' ||
      lpad( hr_max_idle_duration, max_len_hr_max_idle_duration ) || ' / ' ||
      lpad( hr_med_idle_duration, max_len_hr_med_idle_duration ) || ' / ' ||
      lpad( hr_avg_idle_duration, max_len_hr_avg_idle_duration ) || ' / ' ||
      lpad( hr_std_idle_duration, max_len_hr_std_idle_duration ) || ' / ' || 
      lpad( hr_sum_idle_duration, max_len_hr_sum_idle_duration ) )::varchar as idle,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    transaction_id;


create or replace view
  sf_241129.queries_stats_durations_on_leader_for_all_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_duration)::numeric(18,2)::varchar as hr_min_duration,
        extract(epoch from max_duration)::numeric(18,2)::varchar as hr_max_duration,
        med_duration::numeric(18,2)::varchar                     as hr_med_duration,
        extract(epoch from avg_duration)::numeric(18,2)::varchar as hr_avg_duration,
        std_duration::numeric(18,2)::varchar                     as hr_std_duration,
        extract(epoch from sum_duration)::numeric(18,2)::varchar as hr_sum_duration,

        max( len(hr_min_duration) ) over () as max_len_hr_min_duration,
        max( len(hr_max_duration) ) over () as max_len_hr_max_duration,
        max( len(hr_med_duration) ) over () as max_len_hr_med_duration,
        max( len(hr_avg_duration) ) over () as max_len_hr_avg_duration,
        max( len(hr_std_duration) ) over () as max_len_hr_std_duration,
        max( len(hr_sum_duration) ) over () as max_len_hr_sum_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_leader_for_all_by_hour
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_duration, max_len_hr_min_duration ) || ' / ' ||
      lpad( hr_max_duration, max_len_hr_max_duration ) || ' / ' ||
      lpad( hr_med_duration, max_len_hr_med_duration ) || ' / ' ||
      lpad( hr_avg_duration, max_len_hr_avg_duration ) || ' / ' ||
      lpad( hr_std_duration, max_len_hr_std_duration ) || ' / ' ||
      lpad( hr_sum_duration, max_len_hr_sum_duration ) )::varchar as duration,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


create or replace view
  sf_241129.steps_totals_rows_on_workers_for_users_by_day
as
  select
    user_name::varchar                                      as uuser,
    start_ts::date,
    base_241129.hr_number( notnulled_sum_delete_rows )::varchar as ddelete,
    base_241129.hr_number( notnulled_sum_insert_rows )::varchar as iinsert,
    base_241129.hr_number( notnulled_sum_scan_rows )::varchar   as scan
  from
    lf_241129.steps_totals_rows_on_workers_for_users_by_day
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.ttables_stats_dists_sorts_for_schemas_by_count
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,

        all_compound::varchar     as hr_all_compound,
        all_interleaved::varchar  as hr_all_interleaved,
        all_unsorted::varchar     as hr_all_unsorted,

        even_compound::varchar    as hr_even_compound,
        even_interleaved::varchar as hr_even_interleaved,
        even_unsorted::varchar    as hr_even_unsorted,

        key_compound::varchar     as hr_key_compound,
        key_interleaved::varchar  as hr_key_interleaved,
        key_unsorted::varchar     as hr_key_unsorted,

        max( len(hr_all_compound)     ) over () as max_len_hr_all_compound,
        max( len(hr_all_interleaved)  ) over () as max_len_hr_all_interleaved,
        max( len(hr_all_unsorted)     ) over () as max_len_hr_all_unsorted,

        max( len(hr_even_compound)    ) over () as max_len_hr_even_compound,
        max( len(hr_even_interleaved) ) over () as max_len_hr_even_interleaved,
        max( len(hr_even_unsorted)    ) over () as max_len_hr_even_unsorted,

        max( len(hr_key_compound)     ) over () as max_len_hr_key_compound,
        max( len(hr_key_interleaved)  ) over () as max_len_hr_key_interleaved,
        max( len(hr_key_unsorted)     ) over () as max_len_hr_key_unsorted
      from
        lf_241129.ttables_stats_dists_sorts_for_schemas_by_count
    )

  select
    schema_name::varchar as sschema,

    ( lpad( hr_all_compound,     max_len_hr_all_compound    ) || ' / ' ||
      lpad( hr_all_interleaved,  max_len_hr_all_interleaved ) || ' / ' ||
      lpad( hr_all_unsorted,     max_len_hr_all_unsorted    )  )::varchar as aall,

    ( lpad( hr_even_compound,    max_len_hr_even_compound    ) || ' / ' ||
      lpad( hr_even_interleaved, max_len_hr_even_interleaved ) || ' / ' ||
      lpad( hr_even_unsorted,    max_len_hr_even_unsorted    )  )::varchar as even,

    ( lpad( hr_key_compound,     max_len_hr_key_compound    ) || ' / ' ||
      lpad( hr_key_interleaved,  max_len_hr_key_interleaved ) || ' / ' ||
      lpad( hr_key_unsorted,     max_len_hr_key_unsorted    )  )::varchar as kkey
  from
    cte_hr_formatting
  order by
    schema_name;


create or replace view
  sf_241129.steps_totals_data_on_both_for_all_by_second
as
  select
    node_type::varchar as node,
    start_ts::timestamp,

    ( base_241129.hr_bytes( sum_aggr_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_aggr_on_disk_bytes ) )::varchar     as aggr,
    base_241129.hr_bytes( sum_bcast_bytes )::varchar                                                                              as bcast,
    base_241129.hr_number( sum_delete_rows )::varchar                                                                             as ddelete,
    base_241129.hr_bytes( sum_dist_bytes )::varchar                                                                               as dist,
    ( base_241129.hr_bytes( sum_hash_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_hash_on_disk_bytes ) )::varchar     as hash,
    base_241129.hr_number( sum_hashjoin_rows  )::varchar                                                                          as hjoin,
    base_241129.hr_number( sum_insert_rows    )::varchar                                                                          as iinsert,
    base_241129.hr_number( sum_limit_rows     )::varchar                                                                          as llimit,
    base_241129.hr_number( sum_merge_rows     )::varchar                                                                          as mmerge,
    base_241129.hr_number( sum_mergejoin_rows )::varchar                                                                          as mjoin,
    base_241129.hr_number( sum_nestloop_rows  )::varchar                                                                          as nloop,
    -- base_241129.hr_number( sum_parse_rows     )::varchar                                                                          as parse,
    -- base_241129.hr_number( sum_project_rows   )::varchar                                                                          as project,
    base_241129.hr_bytes( sum_return_bytes )::varchar                                                                             as rreturn,
    ( base_241129.hr_bytes( sum_save_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_save_on_disk_bytes ) )::varchar     as save,
    ( base_241129.hr_bytes( sum_sort_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_sort_on_disk_bytes ) )::varchar     as ssort,
    -- base_241129.hr_number( sum_spatial_join_rows )::varchar                                                                       as sjoin,
    ( base_241129.hr_bytes( sum_unique_in_memory_bytes ) || '/' || base_241129.hr_bytes( sum_unique_on_disk_bytes ) )::varchar as uunique,
    base_241129.hr_number( sum_window_rows )::varchar                                                                             as wwindow
  from
    lf_241129.steps_totals_data_on_both_for_all_by_second
  order by
    node_type,
    start_ts;


create or replace view
  sf_241129.ttables_zonemap
as
  select
    schema_name::varchar                as sschema,
    table_name::varchar                 as ttable,
    column_name::varchar                as ccolumn,
    column_full_data_type_name::varchar as data_type,
    column_encoding_name::varchar       as eencoding,
    number_00_20_blocks::int8           as "0-20",
    number_20_40_blocks::int8           as "20-40",
    number_40_60_blocks::int8           as "40-60",
    number_60_80_blocks::int8           as "60-80",
    number_80_100_blocks::int8          as "80-100",
    number_overflow_blocks::int8        as oover,
    number_underflow_blocks::int8       as under,
    number_saturated_blocks::int8       as sat
  from
    lf_241129.ttables_zonemap
  order by
    schema_name,
    table_name,
    column_name;


create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_all_by_minute
as
  select
    start_ts::timestamp,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_all_by_minute
  order by
    start_ts;


create or replace view
  base_241129.fixed_pg_group
as
  select
    grosysid::int4           as group_id,
    trim( groname )::varchar as group_name
  from
    pg_group;


create or replace view
  sf_241129.vacuum_totals_for_tables_by_day
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        start_ts,

        number_manual_delete::varchar          as hr_number_manual_delete,
        number_manual_delete_skipped::varchar  as hr_number_manual_delete_skipped,
        number_manual_full::varchar            as hr_number_manual_full,
        number_manual_full_skipped::varchar    as hr_number_manual_full_skipped,
        number_manual_recluster,
        number_manual_reindex,
        number_manual_sort::varchar            as hr_number_manual_sort,
        number_manual_sort_skipped::varchar    as hr_number_manual_sort_skipped,

        max( len(hr_number_manual_delete)         ) over () as max_len_hr_number_manual_delete,
        max( len(hr_number_manual_delete_skipped) ) over () as max_len_hr_number_manual_delete_skipped,
        max( len(hr_number_manual_full)           ) over () as max_len_hr_number_manual_full,
        max( len(hr_number_manual_full_skipped)   ) over () as max_len_hr_number_manual_full_skipped,
        max( len(hr_number_manual_sort)           ) over () as max_len_hr_number_manual_sort,
        max( len(hr_number_manual_sort_skipped)   ) over () as max_len_hr_number_manual_sort_skipped,

        number_auto_delete::varchar          as hr_number_auto_delete,
        number_auto_delete_skipped::varchar  as hr_number_auto_delete_skipped,
        number_auto_full::varchar            as hr_number_auto_full,
        number_auto_full_skipped::varchar    as hr_number_auto_full_skipped,
        number_auto_recluster,
        number_auto_reindex,
        number_auto_sort::varchar            as hr_number_auto_sort,
        number_auto_sort_skipped::varchar    as hr_number_auto_sort_skipped,

        max( len(hr_number_auto_delete)         ) over () as max_len_hr_number_auto_delete,
        max( len(hr_number_auto_delete_skipped) ) over () as max_len_hr_number_auto_delete_skipped,
        max( len(hr_number_auto_full)           ) over () as max_len_hr_number_auto_full,
        max( len(hr_number_auto_full_skipped)   ) over () as max_len_hr_number_auto_full_skipped,
        max( len(hr_number_auto_sort)           ) over () as max_len_hr_number_auto_sort,
        max( len(hr_number_auto_sort_skipped)   ) over () as max_len_hr_number_auto_sort_skipped,

        number_total_vacuum,
        total_duration
      from
        lf_241129.vacuum_totals_for_tables_by_day
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    start_ts::date,

    ( lpad(hr_number_manual_delete, max_len_hr_number_manual_delete) || ' / ' || lpad(hr_number_manual_full, max_len_hr_number_manual_full ) )::varchar as manual,
    ( lpad(hr_number_auto_delete, max_len_hr_number_auto_delete    ) || ' / ' || lpad(hr_number_auto_full, max_len_hr_number_auto_full     ) )::varchar as auto,

    number_total_vacuum::int4                                                              as total,
    extract( epoch from total_duration )::numeric(18,2)                                    as dur
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    start_ts;


create or replace view
  sf_241129.queries_totals_result_cache_for_users_by_second
as
  select
    user_name::varchar as uuser,
    start_ts::timestamp,
    number_hit::int8    as hit,
    number_miss::int8   as miss,
    number_total::int8  as total,
    percentage::numeric(18, 2) as "%"
  from
    lf_241129.queries_totals_result_cache_for_users_by_second
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_users_by_day
as
  with
    cte_hr_formatting as
    (
      select
        user_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_users_by_day
    )

  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::date,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    user_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_node_by_day
as
  with
    cte_hr_formatting as
    (
      select
        node_ordinal,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_node_by_day
    )

  select
    node_ordinal::int2  as n,
    start_ts::date,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    node_ordinal,
    start_ts;


create or replace view
  sf_241129.transactions_totals_commits_by_minute
as
  select
    start_ts::timestamp,
    extract( epoch from sum_batch_leader_duration )::numeric(18,2) as leader,
    extract( epoch from sum_batch_worker_duration )::numeric(18,2) as worker,
    extract( epoch from sum_batch_idle_duration   )::numeric(18,2) as idle,
    extract( epoch from sum_batch_exec_duration   )::numeric(18,2) as exec
  from
    lf_241129.transactions_totals_commits_by_minute
  order by
    start_ts;


create or replace view
  sf_241129.queries_totals_blocks_on_workers_for_users_by_second
as
  select
    user_name::varchar                                                    as uuser,
    start_ts::timestamp,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_read    )::varchar as rread,
    base_241129.hr_blks( notnulled_sum_query_number_blocks_to_disk )::varchar as to_disk
  from
    lf_241129.queries_totals_blocks_on_workers_for_users_by_second
  order by
    user_name,
    start_ts;


create or replace view
  sf_241129.steps_totals_scan_data_on_both_for_tables_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        schema_name,
        table_name,
        node_type,
        start_ts,

        base_241129.hr_bytes(  sum_type_1_bytes ) as hr_sum_type_1_bytes,
        base_241129.hr_number( sum_type_1_rows  ) as hr_sum_type_1_rows,

        base_241129.hr_bytes(  sum_type_2_bytes ) as hr_sum_type_2_bytes,
        base_241129.hr_number( sum_type_2m_rows ) as hr_sum_type_2m_rows,
        base_241129.hr_number( sum_type_2e_rows ) as hr_sum_type_2e_rows,
        base_241129.hr_number( sum_type_2_rows  ) as hr_sum_type_2_rows,

        base_241129.hr_bytes(  sum_type_3_bytes ) as hr_sum_type_3_bytes,
        base_241129.hr_number( sum_type_3_rows  ) as hr_sum_type_3_rows,

        base_241129.hr_bytes(  sum_type_9_bytes ) as hr_sum_type_9_bytes,
        base_241129.hr_number( sum_type_9_rows  ) as hr_sum_type_9_rows,

        base_241129.hr_bytes(  sum_type_16_bytes ) as hr_sum_type_16_bytes,
        base_241129.hr_number( sum_type_16_rows  ) as hr_sum_type_16_rows,

        base_241129.hr_bytes(  sum_type_17_bytes ) as hr_sum_type_17_bytes,
        base_241129.hr_number( sum_type_17_rows  ) as hr_sum_type_17_rows,

        base_241129.hr_bytes(  sum_type_18_bytes ) as hr_sum_type_18_bytes,
        base_241129.hr_number( sum_type_18_rows  ) as hr_sum_type_18_rows,

        base_241129.hr_bytes(  sum_type_21_bytes ) as hr_sum_type_21_bytes,
        base_241129.hr_number( sum_type_21_rows  ) as hr_sum_type_21_rows,

        base_241129.hr_bytes(  sum_type_22_bytes ) as hr_sum_type_22_bytes,
        base_241129.hr_number( sum_type_22_rows  ) as hr_sum_type_22_rows,

        base_241129.hr_bytes(  sum_type_23_bytes ) as hr_sum_type_23_bytes,
        base_241129.hr_number( sum_type_23_rows  ) as hr_sum_type_23_rows,

        base_241129.hr_bytes(  sum_type_24_bytes ) as hr_sum_type_24_bytes,
        base_241129.hr_number( sum_type_24_rows  ) as hr_sum_type_24_rows,

        base_241129.hr_bytes(  sum_type_25_bytes ) as hr_sum_type_25_bytes,
        base_241129.hr_number( sum_type_25_rows  ) as hr_sum_type_25_rows,

        base_241129.hr_bytes(  sum_type_28_bytes ) as hr_sum_type_28_bytes,
        base_241129.hr_number( sum_type_28_rows  ) as hr_sum_type_28_rows,

        base_241129.hr_bytes(  sum_type_29_bytes ) as hr_sum_type_29_bytes,
        base_241129.hr_number( sum_type_29m_rows ) as hr_sum_type_29m_rows,
        base_241129.hr_number( sum_type_29e_rows ) as hr_sum_type_29e_rows,
        base_241129.hr_number( sum_type_29_rows  ) as hr_sum_type_29_rows,

        base_241129.hr_bytes(  sum_type_30_bytes ) as hr_sum_type_30_bytes,
        base_241129.hr_number( sum_type_30_rows  ) as hr_sum_type_30_rows,

        base_241129.hr_bytes(  sum_type_32_bytes ) as hr_sum_type_32_bytes,
        base_241129.hr_number( sum_type_32_rows  ) as hr_sum_type_32_rows,

        base_241129.hr_bytes(  sum_type_33_bytes ) as hr_sum_type_33_bytes,
        base_241129.hr_number( sum_type_33_rows  ) as hr_sum_type_33_rows,

        base_241129.hr_bytes(  sum_type_35_bytes ) as hr_sum_type_35_bytes,
        base_241129.hr_number( sum_type_35_rows  ) as hr_sum_type_35_rows,

        base_241129.hr_bytes(  sum_type_36_bytes ) as hr_sum_type_36_bytes,
        base_241129.hr_number( sum_type_36_rows  ) as hr_sum_type_36_rows,

        base_241129.hr_bytes(  sum_type_37_bytes ) as hr_sum_type_37_bytes,
        base_241129.hr_number( sum_type_37_rows  ) as hr_sum_type_37_rows,

        max( len(hr_sum_type_1_bytes) ) over () as max_len_hr_sum_type_1_bytes,
        max( len(hr_sum_type_1_rows)  ) over () as max_len_hr_sum_type_1_rows,

        max( len(hr_sum_type_2_bytes) ) over () as max_len_hr_sum_type_2_bytes,
        max( len(hr_sum_type_2m_rows) ) over () as max_len_hr_sum_type_2m_rows,
        max( len(hr_sum_type_2e_rows) ) over () as max_len_hr_sum_type_2e_rows,
        max( len(hr_sum_type_2_rows)  ) over () as max_len_hr_sum_type_2_rows,

        max( len(hr_sum_type_3_bytes) ) over () as max_len_hr_sum_type_3_bytes,
        max( len(hr_sum_type_3_rows)  ) over () as max_len_hr_sum_type_3_rows,

        max( len(hr_sum_type_9_bytes) ) over () as max_len_hr_sum_type_9_bytes,
        max( len(hr_sum_type_9_rows)  ) over () as max_len_hr_sum_type_9_rows,

        max( len(hr_sum_type_16_bytes) ) over () as max_len_hr_sum_type_16_bytes,
        max( len(hr_sum_type_16_rows)  ) over () as max_len_hr_sum_type_16_rows,

        max( len(hr_sum_type_17_bytes) ) over () as max_len_hr_sum_type_17_bytes,
        max( len(hr_sum_type_17_rows)  ) over () as max_len_hr_sum_type_17_rows,

        max( len(hr_sum_type_18_bytes) ) over () as max_len_hr_sum_type_18_bytes,
        max( len(hr_sum_type_18_rows)  ) over () as max_len_hr_sum_type_18_rows,

        max( len(hr_sum_type_21_bytes) ) over () as max_len_hr_sum_type_21_bytes,
        max( len(hr_sum_type_21_rows)  ) over () as max_len_hr_sum_type_21_rows,

        max( len(hr_sum_type_22_bytes) ) over () as max_len_hr_sum_type_22_bytes,
        max( len(hr_sum_type_22_rows)  ) over () as max_len_hr_sum_type_22_rows,

        max( len(hr_sum_type_23_bytes) ) over () as max_len_hr_sum_type_23_bytes,
        max( len(hr_sum_type_23_rows)  ) over () as max_len_hr_sum_type_23_rows,

        max( len(hr_sum_type_24_bytes) ) over () as max_len_hr_sum_type_24_bytes,
        max( len(hr_sum_type_24_rows)  ) over () as max_len_hr_sum_type_24_rows,

        max( len(hr_sum_type_25_bytes) ) over () as max_len_hr_sum_type_25_bytes,
        max( len(hr_sum_type_25_rows)  ) over () as max_len_hr_sum_type_25_rows,

        max( len(hr_sum_type_28_bytes) ) over () as max_len_hr_sum_type_28_bytes,
        max( len(hr_sum_type_28_rows)  ) over () as max_len_hr_sum_type_28_rows,

        max( len(hr_sum_type_29_bytes) ) over () as max_len_hr_sum_type_29_bytes,
        max( len(hr_sum_type_29m_rows) ) over () as max_len_hr_sum_type_29m_rows,
        max( len(hr_sum_type_29e_rows) ) over () as max_len_hr_sum_type_29e_rows,
        max( len(hr_sum_type_29_rows)  ) over () as max_len_hr_sum_type_29_rows,

        max( len(hr_sum_type_30_bytes) ) over () as max_len_hr_sum_type_30_bytes,
        max( len(hr_sum_type_30_rows)  ) over () as max_len_hr_sum_type_30_rows,

        max( len(hr_sum_type_32_bytes) ) over () as max_len_hr_sum_type_32_bytes,
        max( len(hr_sum_type_32_rows)  ) over () as max_len_hr_sum_type_32_rows,

        max( len(hr_sum_type_33_bytes) ) over () as max_len_hr_sum_type_33_bytes,
        max( len(hr_sum_type_33_rows)  ) over () as max_len_hr_sum_type_33_rows,

        max( len(hr_sum_type_35_bytes) ) over () as max_len_hr_sum_type_35_bytes,
        max( len(hr_sum_type_35_rows)  ) over () as max_len_hr_sum_type_35_rows,

        max( len(hr_sum_type_36_bytes) ) over () as max_len_hr_sum_type_36_bytes,
        max( len(hr_sum_type_36_rows)  ) over () as max_len_hr_sum_type_36_rows,

        max( len(hr_sum_type_37_bytes) ) over () as max_len_hr_sum_type_37_bytes,
        max( len(hr_sum_type_37_rows)  ) over () as max_len_hr_sum_type_37_rows
      from
        lf_241129.steps_totals_scan_data_on_both_for_tables_by_hour
    )

  select
    schema_name::varchar as sschema,
    table_name::varchar  as ttable,
    node_type::varchar   as node,
    start_ts::timestamp,

    ( lpad(hr_sum_type_1_bytes, max_len_hr_sum_type_1_bytes) || '/' || lpad(hr_sum_type_1_rows, max_len_hr_sum_type_1_rows) )::varchar     as t1,
    ( lpad(hr_sum_type_2_bytes, max_len_hr_sum_type_2_bytes) || '/' || lpad(hr_sum_type_2m_rows, max_len_hr_sum_type_2m_rows) || '/' || lpad(hr_sum_type_2e_rows, max_len_hr_sum_type_2e_rows) || '/' || lpad(hr_sum_type_2_rows, max_len_hr_sum_type_2_rows) )::varchar as t2bmer,
    ( lpad(hr_sum_type_3_bytes, max_len_hr_sum_type_3_bytes) || '/' || lpad(hr_sum_type_3_rows, max_len_hr_sum_type_3_rows) )::varchar     as t3,
    ( lpad(hr_sum_type_9_bytes, max_len_hr_sum_type_9_bytes) || '/' || lpad(hr_sum_type_9_rows, max_len_hr_sum_type_9_rows) )::varchar     as t9,
    ( lpad(hr_sum_type_16_bytes, max_len_hr_sum_type_16_bytes) || '/' || lpad(hr_sum_type_16_rows, max_len_hr_sum_type_16_rows) )::varchar as t16,
    ( lpad(hr_sum_type_17_bytes, max_len_hr_sum_type_17_bytes) || '/' || lpad(hr_sum_type_17_rows, max_len_hr_sum_type_17_rows) )::varchar as t17,
    ( lpad(hr_sum_type_18_bytes, max_len_hr_sum_type_18_bytes) || '/' || lpad(hr_sum_type_18_rows, max_len_hr_sum_type_18_rows) )::varchar as t18,
    ( lpad(hr_sum_type_21_bytes, max_len_hr_sum_type_21_bytes) || '/' || lpad(hr_sum_type_21_rows, max_len_hr_sum_type_21_rows) )::varchar as t21,
    ( lpad(hr_sum_type_22_bytes, max_len_hr_sum_type_22_bytes) || '/' || lpad(hr_sum_type_22_rows, max_len_hr_sum_type_22_rows) )::varchar as t22,
    ( lpad(hr_sum_type_23_bytes, max_len_hr_sum_type_23_bytes) || '/' || lpad(hr_sum_type_23_rows, max_len_hr_sum_type_23_rows) )::varchar as t23,
    ( lpad(hr_sum_type_24_bytes, max_len_hr_sum_type_24_bytes) || '/' || lpad(hr_sum_type_24_rows, max_len_hr_sum_type_24_rows) )::varchar as t24,
    ( lpad(hr_sum_type_25_bytes, max_len_hr_sum_type_25_bytes) || '/' || lpad(hr_sum_type_25_rows, max_len_hr_sum_type_25_rows) )::varchar as t25,
    ( lpad(hr_sum_type_28_bytes, max_len_hr_sum_type_28_bytes) || '/' || lpad(hr_sum_type_28_rows, max_len_hr_sum_type_28_rows) )::varchar as t28,
    ( lpad(hr_sum_type_29_bytes, max_len_hr_sum_type_29_bytes) || '/' || lpad(hr_sum_type_29m_rows, max_len_hr_sum_type_29m_rows) || '/' || lpad(hr_sum_type_29e_rows, max_len_hr_sum_type_29e_rows) || '/' || lpad(hr_sum_type_29_rows, max_len_hr_sum_type_29_rows) )::varchar as t29bmer,
    ( lpad(hr_sum_type_30_bytes, max_len_hr_sum_type_30_bytes) || '/' || lpad(hr_sum_type_30_rows, max_len_hr_sum_type_30_rows) )::varchar as t30,
    ( lpad(hr_sum_type_32_bytes, max_len_hr_sum_type_32_bytes) || '/' || lpad(hr_sum_type_32_rows, max_len_hr_sum_type_32_rows) )::varchar as t32,
    ( lpad(hr_sum_type_33_bytes, max_len_hr_sum_type_33_bytes) || '/' || lpad(hr_sum_type_33_rows, max_len_hr_sum_type_33_rows) )::varchar as t33,
    ( lpad(hr_sum_type_35_bytes, max_len_hr_sum_type_35_bytes) || '/' || lpad(hr_sum_type_35_rows, max_len_hr_sum_type_35_rows) )::varchar as t35,
    ( lpad(hr_sum_type_36_bytes, max_len_hr_sum_type_36_bytes) || '/' || lpad(hr_sum_type_36_rows, max_len_hr_sum_type_36_rows) )::varchar as t36,
    ( lpad(hr_sum_type_37_bytes, max_len_hr_sum_type_37_bytes) || '/' || lpad(hr_sum_type_37_rows, max_len_hr_sum_type_37_rows) )::varchar as t37
  from
    cte_hr_formatting
  order by
    schema_name,
    table_name,
    node_type,
    start_ts;


create or replace view
  sf_241129.transactions_stats_commits_by_all
as
  with
    cte_hr_formatting as
    (
      select
        extract(epoch from min_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_min_transaction_batch_queue_start,
        extract(epoch from max_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_max_transaction_batch_queue_start,
        med_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_med_transaction_batch_queue_start,
        extract(epoch from avg_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_avg_transaction_batch_queue_start,
        std_transaction_batch_queue_start::numeric(18,2)::varchar                     as hr_std_transaction_batch_queue_start,
        extract(epoch from sum_transaction_batch_queue_start)::numeric(18,2)::varchar as hr_sum_transaction_batch_queue_start,

        extract(epoch from min_batch_queue_duration)::numeric(18,2)::varchar as hr_min_batch_queue_duration,
        extract(epoch from max_batch_queue_duration)::numeric(18,2)::varchar as hr_max_batch_queue_duration,
        med_batch_queue_duration::numeric(18,2)::varchar                     as hr_med_batch_queue_duration,
        extract(epoch from avg_batch_queue_duration)::numeric(18,2)::varchar as hr_avg_batch_queue_duration,
        std_batch_queue_duration::numeric(18,2)::varchar                     as hr_std_batch_queue_duration,
        extract(epoch from sum_batch_queue_duration)::numeric(18,2)::varchar as hr_sum_batch_queue_duration,

        extract(epoch from min_batch_leader_duration)::numeric(18,2)::varchar as hr_min_batch_leader_duration,
        extract(epoch from max_batch_leader_duration)::numeric(18,2)::varchar as hr_max_batch_leader_duration,
        med_batch_leader_duration::numeric(18,2)::varchar                     as hr_med_batch_leader_duration,
        extract(epoch from avg_batch_leader_duration)::numeric(18,2)::varchar as hr_avg_batch_leader_duration,
        std_batch_leader_duration::numeric(18,2)::varchar                     as hr_std_batch_leader_duration,
        extract(epoch from sum_batch_leader_duration)::numeric(18,2)::varchar as hr_sum_batch_leader_duration,

        extract(epoch from min_batch_worker_duration)::numeric(18,2)::varchar as hr_min_batch_worker_duration,
        extract(epoch from max_batch_worker_duration)::numeric(18,2)::varchar as hr_max_batch_worker_duration,
        med_batch_worker_duration::numeric(18,2)::varchar                     as hr_med_batch_worker_duration,
        extract(epoch from avg_batch_worker_duration)::numeric(18,2)::varchar as hr_avg_batch_worker_duration,
        std_batch_worker_duration::numeric(18,2)::varchar                     as hr_std_batch_worker_duration,
        extract(epoch from sum_batch_worker_duration)::numeric(18,2)::varchar as hr_sum_batch_worker_duration,

        min_number_batch_transactions::varchar                as hr_min_number_batch_transactions,
        max_number_batch_transactions::varchar                as hr_max_number_batch_transactions,
        med_number_batch_transactions::varchar                as hr_med_number_batch_transactions,
        avg_number_batch_transactions::numeric(18,2)::varchar as hr_avg_number_batch_transactions,
        std_number_batch_transactions::numeric(18,2)::varchar as hr_std_number_batch_transactions,
        sum_number_batch_transactions::varchar                as hr_sum_number_batch_transactions,

        base_241129.hr_number( min_number_batch_blocks ) as hr_min_number_batch_blocks,
        base_241129.hr_number( max_number_batch_blocks ) as hr_max_number_batch_blocks,
        base_241129.hr_number( med_number_batch_blocks ) as hr_med_number_batch_blocks,
        base_241129.hr_number( avg_number_batch_blocks ) as hr_avg_number_batch_blocks,
        base_241129.hr_number( std_number_batch_blocks ) as hr_std_number_batch_blocks,
        base_241129.hr_number( sum_number_batch_blocks ) as hr_sum_number_batch_blocks,

        number_batches
      from
        lf_241129.transactions_stats_commits_by_all
    )

  select
    ( hr_min_transaction_batch_queue_start || ' / ' ||
      hr_max_transaction_batch_queue_start || ' / ' ||
      hr_med_transaction_batch_queue_start || ' / ' ||
      hr_avg_transaction_batch_queue_start || ' / ' ||
      hr_std_transaction_batch_queue_start || ' / ' ||
      hr_sum_transaction_batch_queue_start )::varchar             as "xid queuing",

    ( hr_min_batch_queue_duration || ' / ' ||
      hr_max_batch_queue_duration || ' / ' ||
      hr_med_batch_queue_duration || ' / ' ||
      hr_avg_batch_queue_duration || ' / ' ||
      hr_std_batch_queue_duration || ' / ' ||
      hr_sum_batch_queue_duration          )::varchar  as "bid queuing",

    ( hr_min_batch_leader_duration || ' / ' ||
      hr_max_batch_leader_duration || ' / ' ||
      hr_med_batch_leader_duration || ' / ' ||
      hr_avg_batch_leader_duration || ' / ' ||
      hr_std_batch_leader_duration || ' / ' ||
      hr_sum_batch_leader_duration          )::varchar  as "ldr duration",

    ( hr_min_batch_worker_duration || ' / ' ||
      hr_max_batch_worker_duration || ' / ' ||
      hr_med_batch_worker_duration || ' / ' ||
      hr_avg_batch_worker_duration || ' / ' ||
      hr_std_batch_worker_duration || ' / ' ||
      hr_sum_batch_worker_duration          )::varchar  as "wrk duration",

    ( hr_min_number_batch_transactions || ' / ' ||
      hr_max_number_batch_transactions || ' / ' ||
      hr_med_number_batch_transactions || ' / ' ||
      hr_avg_number_batch_transactions || ' / ' ||
      hr_std_number_batch_transactions || ' / ' ||
      hr_sum_number_batch_transactions )::varchar       as "xids/bid",

    ( hr_min_number_batch_blocks || ' / ' ||
      hr_max_number_batch_blocks || ' / ' ||
      hr_med_number_batch_blocks || ' / ' ||
      hr_avg_number_batch_blocks || ' / ' ||
      hr_std_number_batch_blocks || ' / ' ||
      hr_sum_number_batch_blocks )::varchar             as "blks/bid",

    number_batches::int8                                as bids
  from
    cte_hr_formatting;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_node_by_minute_concise
as
  select
    start_ts::timestamp,
    node_ordinal::int2 as n,

    extract( epoch from avg_aggr_duration      )::numeric(18,2) as aggr,
    extract( epoch from avg_bcast_duration     )::numeric(18,2) as bcast,
    extract( epoch from avg_delete_duration    )::numeric(18,2) as del,
    extract( epoch from avg_dist_duration      )::numeric(18,2) as dist,
    extract( epoch from avg_hash_duration      )::numeric(18,2) as hash,
    extract( epoch from avg_hashjoin_duration  )::numeric(18,2) as hjoin,
    extract( epoch from avg_insert_duration    )::numeric(18,2) as iinsert,
    extract( epoch from avg_limit_duration     )::numeric(18,2) as llimit,
    extract( epoch from avg_merge_duration     )::numeric(18,2) as mmerge,
    extract( epoch from avg_mergejoin_duration )::numeric(18,2) as mjoin,
    extract( epoch from avg_nestloop_duration  )::numeric(18,2) as nloop,
    extract( epoch from avg_return_duration    )::numeric(18,2) as rreturn,
    extract( epoch from avg_save_duration      )::numeric(18,2) as save,
    extract( epoch from avg_sort_duration      )::numeric(18,2) as ssort,
    extract( epoch from avg_unique_duration    )::numeric(18,2) as uunique,
    extract( epoch from avg_window_duration    )::numeric(18,2) as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_node_by_minute
  order by
    start_ts,
    node_ordinal;


create or replace view
  sf_241129.steps_stats_durations_on_both_for_users_by_second_complete
as
  select
    user_name::varchar as uuser,
    node_type::varchar as node,
    start_ts::timestamp,

    ( extract(epoch from min_aggr_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_aggr_duration)::numeric(18,2) || ' / ' ||
      med_aggr_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_aggr_duration)::numeric(18,2) || ' / ' ||
      std_aggr_duration::numeric(18,2) )::varchar as aggr,

    ( extract(epoch from min_bcast_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_bcast_duration)::numeric(18,2) || ' / ' ||
      med_bcast_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_bcast_duration)::numeric(18,2) || ' / ' ||
      std_bcast_duration::numeric(18,2) )::varchar as bcast,

    ( extract(epoch from min_delete_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_delete_duration)::numeric(18,2) || ' / ' ||
      med_delete_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_delete_duration)::numeric(18,2) || ' / ' ||
      std_delete_duration::numeric(18,2) )::varchar as ddelete,

    ( extract(epoch from min_dist_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_dist_duration)::numeric(18,2) || ' / ' ||
      med_dist_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_dist_duration)::numeric(18,2) || ' / ' ||
      std_dist_duration::numeric(18,2) )::varchar as dist,

    ( extract(epoch from min_hash_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hash_duration)::numeric(18,2) || ' / ' ||
      med_hash_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hash_duration)::numeric(18,2) || ' / ' ||
      std_hash_duration::numeric(18,2) )::varchar as hash,

    ( extract(epoch from min_hashjoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_hashjoin_duration)::numeric(18,2) || ' / ' ||
      med_hashjoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_hashjoin_duration)::numeric(18,2) || ' / ' ||
      std_hashjoin_duration::numeric(18,2) )::varchar as hashjoin,

    ( extract(epoch from min_insert_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_insert_duration)::numeric(18,2) || ' / ' ||
      med_insert_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_insert_duration)::numeric(18,2) || ' / ' ||
      std_insert_duration::numeric(18,2) )::varchar as iinsert,

    ( extract(epoch from min_limit_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_limit_duration)::numeric(18,2) || ' / ' ||
      med_limit_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_limit_duration)::numeric(18,2) || ' / ' ||
      std_limit_duration::numeric(18,2) )::varchar as llimit,

    ( extract(epoch from min_merge_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_merge_duration)::numeric(18,2) || ' / ' ||
      med_merge_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_merge_duration)::numeric(18,2) || ' / ' ||
      std_merge_duration::numeric(18,2) )::varchar as mmerge,

    ( extract(epoch from min_mergejoin_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_mergejoin_duration)::numeric(18,2) || ' / ' ||
      med_mergejoin_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_mergejoin_duration)::numeric(18,2) || ' / ' ||
      std_mergejoin_duration::numeric(18,2) )::varchar as mergejoin,

    ( extract(epoch from min_nestloop_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_nestloop_duration)::numeric(18,2) || ' / ' ||
      med_nestloop_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_nestloop_duration)::numeric(18,2) || ' / ' ||
      std_nestloop_duration::numeric(18,2) )::varchar as nestloop_op,

    ( extract(epoch from min_parse_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_parse_duration)::numeric(18,2) || ' / ' ||
      med_parse_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_parse_duration)::numeric(18,2) || ' / ' ||
      std_parse_duration::numeric(18,2) )::varchar as parse,

    ( extract(epoch from min_project_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_project_duration)::numeric(18,2) || ' / ' ||
      med_project_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_project_duration)::numeric(18,2) || ' / ' ||
      std_project_duration::numeric(18,2) )::varchar as project,

    ( extract(epoch from min_return_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_return_duration)::numeric(18,2) || ' / ' ||
      med_return_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_return_duration)::numeric(18,2) || ' / ' ||
      std_return_duration::numeric(18,2) )::varchar as rreturn,

    ( extract(epoch from min_save_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_save_duration)::numeric(18,2) || ' / ' ||
      med_save_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_save_duration)::numeric(18,2) || ' / ' ||
      std_save_duration::numeric(18,2) )::varchar as save,

    ( extract(epoch from min_sort_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_sort_duration)::numeric(18,2) || ' / ' ||
      med_sort_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_sort_duration)::numeric(18,2) || ' / ' ||
      std_sort_duration::numeric(18,2) )::varchar as ssort,

    ( extract(epoch from min_spatial_join_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_spatial_join_duration)::numeric(18,2) || ' / ' ||
      med_spatial_join_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_spatial_join_duration)::numeric(18,2) || ' / ' ||
      std_spatial_join_duration::numeric(18,2) )::varchar as spatial_join,

    ( extract(epoch from min_unique_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_unique_duration)::numeric(18,2) || ' / ' ||
      med_unique_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_unique_duration)::numeric(18,2) || ' / ' ||
      std_unique_duration::numeric(18,2) )::varchar as uunique,

    ( extract(epoch from min_window_duration)::numeric(18,2) || ' / ' ||
      extract(epoch from max_window_duration)::numeric(18,2) || ' / ' ||
      med_window_duration::numeric(18,2)                     || ' / ' ||
      extract(epoch from avg_window_duration)::numeric(18,2) || ' / ' ||
      std_window_duration::numeric(18,2) )::varchar as wwindow
  from
    lf_241129.steps_stats_durations_on_both_for_users_by_second
  order by
    user_name,
    node_type,
    start_ts;



create or replace view
  sf_241129.transactions_totals_query_counts_on_both_for_users_by_all
as
  select
    user_name::varchar                 as uuser,
    number_readonly_transactions::int8 as "rowxids",
    number_writing_transactions::int8  as "wr xids",
    number_transactions::int8          as xids,
    number_leader_queries::int8        as "ldr qids",
    number_worker_queries::int8        as "wrk qids",
    number_total_queries::int8         as qids
  from
    lf_241129.transactions_totals_query_counts_on_both_for_users_by_all
  order by
    user_name;


create or replace view
  sf_241129.wqueues_qmr_log
as
  select
    event_ts::timestamp,
    rule_name::varchar   as rrule,
    rule_action::varchar as aaction,
    queue_name::varchar  as qname,
    user_name::varchar   as uuser,
    query_id::int4       as qid
  from
    lf_241129.wqueues_qmr_log
  order by
    event_ts;


create or replace view
  sf_241129.queries_stats_durations_on_workers_for_all_by_hour
as
  with
    cte_hr_formatting as
    (
      select
        start_ts,

        extract(epoch from min_query_queue_duration)::numeric(18,2)::varchar as hr_min_query_queue_duration,
        extract(epoch from max_query_queue_duration)::numeric(18,2)::varchar as hr_max_query_queue_duration,
        med_query_queue_duration::numeric(18,2)::varchar                     as hr_med_query_queue_duration,
        extract(epoch from avg_query_queue_duration)::numeric(18,2)::varchar as hr_avg_query_queue_duration,
        std_query_queue_duration::numeric(18,2)::varchar                     as hr_std_query_queue_duration,
        extract(epoch from sum_query_queue_duration)::numeric(18,2)::varchar as hr_sum_query_queue_duration,

        extract(epoch from min_query_compile_duration)::numeric(18,2)::varchar as hr_min_query_compile_duration,
        extract(epoch from max_query_compile_duration)::numeric(18,2)::varchar as hr_max_query_compile_duration,
        med_query_compile_duration::numeric(18,2)::varchar                     as hr_med_query_compile_duration,
        extract(epoch from avg_query_compile_duration)::numeric(18,2)::varchar as hr_avg_query_compile_duration,
        std_query_compile_duration::numeric(18,2)::varchar                     as hr_std_query_compile_duration,
        extract(epoch from sum_query_compile_duration)::numeric(18,2)::varchar as hr_sum_query_compile_duration,

        extract(epoch from min_query_exec_duration)::numeric(18,2)::varchar as hr_min_query_exec_duration,
        extract(epoch from max_query_exec_duration)::numeric(18,2)::varchar as hr_max_query_exec_duration,
        med_query_exec_duration::numeric(18,2)::varchar                     as hr_med_query_exec_duration,
        extract(epoch from avg_query_exec_duration)::numeric(18,2)::varchar as hr_avg_query_exec_duration,
        std_query_exec_duration::numeric(18,2)::varchar                     as hr_std_query_exec_duration,
        extract(epoch from sum_query_exec_duration)::numeric(18,2)::varchar as hr_sum_query_exec_duration,

        max( len(hr_min_query_queue_duration) ) over () as max_len_hr_min_query_queue_duration,
        max( len(hr_max_query_queue_duration) ) over () as max_len_hr_max_query_queue_duration,
        max( len(hr_med_query_queue_duration) ) over () as max_len_hr_med_query_queue_duration,
        max( len(hr_avg_query_queue_duration) ) over () as max_len_hr_avg_query_queue_duration,
        max( len(hr_std_query_queue_duration) ) over () as max_len_hr_std_query_queue_duration,
        max( len(hr_sum_query_queue_duration) ) over () as max_len_hr_sum_query_queue_duration,

        max( len(hr_min_query_compile_duration) ) over () as max_len_hr_min_query_compile_duration,
        max( len(hr_max_query_compile_duration) ) over () as max_len_hr_max_query_compile_duration,
        max( len(hr_med_query_compile_duration) ) over () as max_len_hr_med_query_compile_duration,
        max( len(hr_avg_query_compile_duration) ) over () as max_len_hr_avg_query_compile_duration,
        max( len(hr_std_query_compile_duration) ) over () as max_len_hr_std_query_compile_duration,
        max( len(hr_sum_query_compile_duration) ) over () as max_len_hr_sum_query_compile_duration,

        max( len(hr_min_query_exec_duration) ) over () as max_len_hr_min_query_exec_duration,
        max( len(hr_max_query_exec_duration) ) over () as max_len_hr_max_query_exec_duration,
        max( len(hr_med_query_exec_duration) ) over () as max_len_hr_med_query_exec_duration,
        max( len(hr_avg_query_exec_duration) ) over () as max_len_hr_avg_query_exec_duration,
        max( len(hr_std_query_exec_duration) ) over () as max_len_hr_std_query_exec_duration,
        max( len(hr_sum_query_exec_duration) ) over () as max_len_hr_sum_query_exec_duration,

        number_queries
      from
        lf_241129.queries_stats_durations_on_workers_for_all_by_hour
    )

  select
    start_ts::timestamp,

    ( lpad( hr_min_query_queue_duration, max_len_hr_min_query_queue_duration ) || ' / ' ||
      lpad( hr_max_query_queue_duration, max_len_hr_max_query_queue_duration ) || ' / ' ||
      lpad( hr_med_query_queue_duration, max_len_hr_med_query_queue_duration ) || ' / ' ||
      lpad( hr_avg_query_queue_duration, max_len_hr_avg_query_queue_duration ) || ' / ' ||
      lpad( hr_std_query_queue_duration, max_len_hr_std_query_queue_duration ) || ' / ' ||
      lpad( hr_sum_query_queue_duration, max_len_hr_sum_query_queue_duration ) )::varchar as queue,

    ( lpad( hr_min_query_compile_duration, max_len_hr_min_query_compile_duration ) || ' / ' ||
      lpad( hr_max_query_compile_duration, max_len_hr_max_query_compile_duration ) || ' / ' ||
      lpad( hr_med_query_compile_duration, max_len_hr_med_query_compile_duration ) || ' / ' ||
      lpad( hr_avg_query_compile_duration, max_len_hr_avg_query_compile_duration ) || ' / ' ||
      lpad( hr_std_query_compile_duration, max_len_hr_std_query_compile_duration ) || ' / ' ||
      lpad( hr_sum_query_compile_duration, max_len_hr_sum_query_compile_duration ) )::varchar as compile,

    ( lpad( hr_min_query_exec_duration, max_len_hr_min_query_exec_duration ) || ' / ' ||
      lpad( hr_max_query_exec_duration, max_len_hr_max_query_exec_duration ) || ' / ' ||
      lpad( hr_med_query_exec_duration, max_len_hr_med_query_exec_duration ) || ' / ' ||
      lpad( hr_avg_query_exec_duration, max_len_hr_avg_query_exec_duration ) || ' / ' ||
      lpad( hr_std_query_exec_duration, max_len_hr_std_query_exec_duration ) || ' / ' || 
      lpad( hr_sum_query_exec_duration, max_len_hr_sum_query_exec_duration ) )::varchar as exec,

    number_queries::int4 as queries
  from
    cte_hr_formatting
  order by
    start_ts;


delete from base_241129.strings; -- MG2 : protect in case user installs twice; we vacuum later

insert into
  base_241129.strings( system_source, system_number, system_string, my_string )
values
  ( 'stl_aggr', null, 'HASHED', 'grouped, unsorted' ),
  ( 'stl_aggr', null, 'PLAIN',  'ungrouped, scalar' ),
  ( 'stl_aggr', null, 'SORTED', 'grouped, sorted'   ),

  ( 'stl_hashjoin', 0, null, 'inner join'  ),
  ( 'stl_hashjoin', 1, null, 'left join'   ),
  ( 'stl_hashjoin', 2, null, 'full join'   ),
  ( 'stl_hashjoin', 3, null, 'right join'  ),
  ( 'stl_hashjoin', 4, null, 'union'       ),
  ( 'stl_hashjoin', 5, null, 'in operator' ),

  ( 'stl_scan',  1, null, 'network to row-store table' ),
  ( 'stl_scan',  2, null, 'user table' ),
  ( 'stl_scan',  3, null, 'row-store table' ),
  ( 'stl_scan',  9, null, 'undocumented type' ),
  ( 'stl_scan', 16, null, 'leader node table' ),
  ( 'stl_scan', 17, null, 'UNION ALL' ),
  ( 'stl_scan', 18, null, 'undocumented type' ),
  ( 'stl_scan', 21, null, 'S3' ),
  ( 'stl_scan', 22, null, 'DynamoDB' ),
  ( 'stl_scan', 23, null, 'SSH' ),
  ( 'stl_scan', 24, null, 'sorted data from remote cluster' ),
  ( 'stl_scan', 25, null, 'unsorted data from remote cluster' ),
  ( 'stl_scan', 28, null, 'time series UNION ALL view' ),
  ( 'stl_scan', 29, null, 'Spectrum S3 scan' ),
  ( 'stl_scan', 30, null, 'partition data from external database' ),
  ( 'stl_scan', 32, null, 'runtime filter' ),
  ( 'stl_scan', 33, null, 'Postgres table' ),
  ( 'stl_scan', 35, null, 'AQUA scan' ),
  ( 'stl_scan', 36, null, 'mySQL table' ),
  ( 'stl_scan', 37, null, 'Kinesis stream' ),

  ( 'stl_unique', null, 'HASHED', 'grouped, unsorted' ),
  ( 'stl_unique', null, 'PLAIN',  'ungrouped, scalar' ),
  ( 'stl_unique', null, 'SORTED', 'grouped, sorted'   ),

  ( 'store_type', null, 't', 'on disk' ),
  ( 'store_type', null, 'f', 'in memory' );

delete from base_241129.numbers; -- MG2 : protect in case user installs twice; we vacuum later

insert into
  base_241129.numbers( number )
with
  cte_base as
  (
    select 1 as number
    union all
    select 2 as number
    union all
    select 3 as number
    union all
    select 4 as number
  )

select
  -- MG2 : I need to count from 0, so -1
  row_number() over () - 1 as number
from
             cte_base as t0 -- 4
  cross join cte_base as t1 -- 16
  cross join cte_base as t2 -- 64
  cross join cte_base as t3 -- 256
  cross join cte_base as t4 -- 1024
  cross join cte_base as t5 -- 4096
  cross join cte_base as t6 -- 16384
  cross join cte_base as t7 -- 65536
  cross join cte_base as t8 -- 262144
  cross join cte_base as t9 -- 1048576
limit
  ( 7 * 24 * 60 * 60 ); -- 604800

insert into
  base_241129.docs( schema_name, object_name, object_type, doc )
values  ('lf_241129', 'analyze_totals_for_users_by_minute', 'view', '# `lf_241129.analyze_totals_for_users_by_minute`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `user_id`                                       `int4`
  `user_name`                                     `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('sf_241129', 'analyze_totals_for_users_by_day', 'view', '# `sf_241129.analyze_totals_for_users_by_day`

## Description

## Columns

  column          data type
  --------------- -----------
  `uuser`         `varchar`
  `start_ts`      `date`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### uuser

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'ttables_creation_for_all_by_hour', 'view', '# `lf_241129.ttables_creation_for_all_by_hour`

## Description

## Columns

  column            data type
  ----------------- -------------
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### start_ts

### number_tables
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_all_by_day', 'view', '# `lf_241129.steps_stats_durations_on_both_for_all_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'analyze_totals_for_database_by_day', 'view', '# `lf_241129.analyze_totals_for_database_by_day`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `database_name`                                 `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### database_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('sf_241129', 'vacuum_totals_for_users_by_second', 'view', '# `sf_241129.vacuum_totals_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### uuser

### start_ts

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'vacuum_stats_for_all_by_hour', 'view', '# `sf_241129.vacuum_stats_for_all_by_hour`

## Description

## Columns

  column              data type
  ------------------- -------------
  `start_ts`          `timestamp`
  `new blocks`        `varchar`
  `replaced blocks`   `varchar`
  `new rows`          `varchar`
  `deleted rows`      `varchar`
  `xid duration`      `varchar`
  `qids`              `int8`
  `xids`              `int8`

## Column Descriptions

### start_ts

### new blocks

### replaced blocks

### new rows

### deleted rows

### xid duration

### qids

### xids
'),
  ('lf_241129', 'sessions_fast', 'view', '# `lf_241129.sessions_fast`

## Description

## Columns

  column                      data type
  --------------------------- ------------
  `user_id`                   `int4`
  `user_name`                 `varchar`
  `session_id`                `int4`
  `truncated_database_name`   `varchar`
  `session_duration`          `interval`

## Column Descriptions

### user_id

### user_name

### session_id

### truncated_database_name

### session_duration
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_minute_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_minute_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `del`        `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `save`       `numeric`
  `ssort`      `numeric`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### del

### hjoin

### iinsert

### mjoin

### nloop

### save

### ssort
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_all_by_second', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### queue

### compile

### exec

### queries
'),
  ('base_241129', 'fixed_pg_attribute', 'view', '# `base_241129.fixed_pg_attribute`

## Description

## Columns

  column                  data type
  ----------------------- -----------
  `object_id`             `int4`
  `column_name`           `varchar`
  `column_ordinal`        `int2`
  `atttypmod`             `int4`
  `column_notnull_flag`   `bool`

## Column Descriptions

### object_id

### column_name

### column_ordinal

### atttypmod

### column_notnull_flag
'),
  ('lf_241129', 'steps_totals_data_on_both_for_node_by_hour', 'view', '# `lf_241129.steps_totals_data_on_both_for_node_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_ordinal`                 `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('ro_241129', 'sp_drop_versionless_sf_schema', 'procedure', '# `ro_241129.sp_drop_versionless_sf_schema`

## Description

## Arguments

No arguments.
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_all', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_schemas_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `varchar`
  `sschema`   `varchar`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### node

### sschema

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_queues_by_hour', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_queues_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### qname

### start_ts

### queue

### compile

### exec

### queries
'),
  ('lf_241129', 'wqueues_routing_rules', 'view', '# `lf_241129.wqueues_routing_rules`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `rule_order`                  `int2`
  `query_group_wildcard_flag`   `bool`
  `user_group_wildcard_flag`    `bool`
  `condition`                   `varchar`
  `queue_id`                    `int2`
  `queue_name`                  `varchar`

## Column Descriptions

### rule_order

### query_group_wildcard_flag

### user_group_wildcard_flag

### condition

### queue_id

### queue_name
'),
  ('sf_241129', 'sessions_fast', 'view', '# `sf_241129.sessions_fast`

## Description

## Columns

  column    data type
  --------- -----------
  `uuser`   `varchar`
  `pid`     `int4`
  `tdb`     `varchar`
  `dur`     `numeric`

## Column Descriptions

### uuser

### pid

### tdb

### dur
'),
  ('sf_241129', 'queries_transactions_batches', 'view', '# `sf_241129.queries_transactions_batches`

## Description

## Columns

  column           data type
  ---------------- -------------
  `uuser`          `varchar`
  `pid`            `int4`
  `xid`            `int8`
  `qid`            `int4`
  `qname`          `varchar`
  `qid_start_ts`   `timestamp`
  `queue`          `numeric`
  `compile`        `numeric`
  `exec`           `numeric`
  `total`          `numeric`
  `xid_end`        `numeric`
  `bid_end`        `numeric`
  `flags`          `varchar`
  `br`             `int8`
  `btd`            `int8`
  `bid`            `int8`
  `query_text`     `varchar`

## Column Descriptions

### uuser

### pid

### xid

### qid

### qname

### qid_start_ts

### queue

### compile

### exec

### total

### xid_end

### bid_end

### flags

### br

### btd

### bid

### query_text
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_slices_by_day', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_slices_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `start_ts`   `date`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### s

### t

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_all_by_minute', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### start_ts

### aborts
'),
  ('sf_241129', 'queries_totals_result_cache_for_users_by_minute', 'view', '# `sf_241129.queries_totals_result_cache_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### uuser

### start_ts

### hit

### miss

### total

### %
'),
  ('lf_241129', 'queries_compilation', 'view', '# `lf_241129.queries_compilation`

## Description

## Columns

  column                                      data type
  ------------------------------------------- ------------
  `session_id`                                `int4`
  `transaction_id`                            `int8`
  `query_id`                                  `int4`
  `query_number_leader_node_cache_segments`   `int2`
  `query_number_offload_segments`             `int2`
  `query_number_segments`                     `int2`
  `query_compile_duration`                    `interval`

## Column Descriptions

### session_id

### transaction_id

### query_id

### query_number_leader_node_cache_segments

### query_number_offload_segments

### query_number_segments

### query_compile_duration
'),
  ('lf_241129', 'vacuum_totals_for_tables_by_hour', 'view', '# `lf_241129.vacuum_totals_for_tables_by_hour`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'locks_by_transaction', 'view', '# `sf_241129.locks_by_transaction`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `pid`       `int4`
  `xid`       `int8`
  `sschema`   `varchar`
  `ttable`    `varchar`
  `dur`       `numeric`
  `ttype`     `varchar`
  `s`         `varchar`
  `ttext`     `varchar`

## Column Descriptions

### uuser

### pid

### xid

### sschema

### ttable

### dur

### ttype

### s

### ttext
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_schemas_by_second', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_schemas_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### start_ts

### ddelete

### iinsert

### scan
'),
  ('sf_241129', 'steps_totals_data_on_both_for_node_by_all', 'view', '# `sf_241129.steps_totals_data_on_both_for_node_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `int2`
  `aggr`      `varchar`
  `bcast`     `varchar`
  `ddelete`   `varchar`
  `dist`      `varchar`
  `hash`      `varchar`
  `hjoin`     `varchar`
  `iinsert`   `varchar`
  `llimit`    `varchar`
  `mmerge`    `varchar`
  `mjoin`     `varchar`
  `nloop`     `varchar`
  `rreturn`   `varchar`
  `save`      `varchar`
  `ssort`     `varchar`
  `uunique`   `varchar`
  `wwindow`   `varchar`

## Column Descriptions

### node

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'ttables_creation_for_schemas_by_day', 'view', '# `sf_241129.ttables_creation_for_schemas_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `sschema`    `varchar`
  `start_ts`   `date`
  `ttables`    `int4`

## Column Descriptions

### sschema

### start_ts

### ttables
'),
  ('sf_241129', 'uusers_default_privileges', 'view', '# `sf_241129.uusers_default_privileges`

## Description

## Columns

  column             data type
  ------------------ -----------
  `uuser`            `varchar`
  `sschema`          `varchar`
  `ttype`            `varchar`
  `recipient_type`   `varchar`
  `recipient_name`   `varchar`
  `privs`            `varchar`

## Column Descriptions

### uuser

### sschema

### ttype

### recipient_type

### recipient_name

### privs
'),
  ('sf_241129', 'utils_most_recent_copy_error_for_current_user', 'view', '# `sf_241129.utils_most_recent_copy_error_for_current_user`

## Description

## Columns

  column    data type
  --------- -----------
  `kkey`    `varchar`
  `value`   `varchar`

## Column Descriptions

### kkey

### value
'),
  ('sf_241129', 'analyze_totals_for_users_by_minute', 'view', '# `sf_241129.analyze_totals_for_users_by_minute`

## Description

## Columns

  column          data type
  --------------- -------------
  `uuser`         `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### uuser

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_hour_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_hour_complete`

## Description

## Columns

  column          data type
  --------------- -------------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `node`          `varchar`
  `start_ts`      `timestamp`
  `aggr`          `varchar`
  `ddelete`       `varchar`
  `hashjoin`      `varchar`
  `iinsert`       `varchar`
  `mergejoin`     `varchar`
  `nestloop_op`   `varchar`
  `save`          `varchar`
  `ssort`         `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### ddelete

### hashjoin

### iinsert

### mergejoin

### nestloop_op

### save

### ssort
'),
  ('lf_241129', 'sp_create_role_if_not_exists', 'procedure', '# `lf_241129.sp_create_role_if_not_exists`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `role_name`   `varchar( 127 )`

## Argument Descriptions

### role_name
'),
  ('base_241129', 'fixed_pg_user', 'view', '# `base_241129.fixed_pg_user`

## Description

## Columns

  column        data type
  ------------- -----------
  `user_id`     `int4`
  `user_name`   `varchar`

## Column Descriptions

### user_id

### user_name
'),
  ('sf_241129', 'steps_totals_data_on_both_for_users_by_minute', 'view', '# `sf_241129.steps_totals_data_on_both_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('base_241129', 'ttables_activity_full', 'view', '# `base_241129.ttables_activity_full`

## Description

## Columns

  column                   data type
  ------------------------ -----------
  `table_id`               `int4`
  `number_rows_deleted`    `int8`
  `number_rows_inserted`   `int8`
  `number_rows_scanned`    `int8`

## Column Descriptions

### table_id

### number_rows_deleted

### number_rows_inserted

### number_rows_scanned
'),
  ('lf_241129', 'queries_text_on_workers_fast', 'view', '# `lf_241129.queries_text_on_workers_fast`

## Description

## Columns

  column             data type
  ------------------ -------------
  `session_id`       `int4`
  `transaction_id`   `int8`
  `query_id`         `int4`
  `query_start_ts`   `timestamp`
  `query_text`       `varchar`

## Column Descriptions

### session_id

### transaction_id

### query_id

### query_start_ts

### query_text
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_all_by_second', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### start_ts

### rread

### to_disk
'),
  ('lf_241129', 'steps_totals_data_on_both_for_users_by_second', 'view', '# `lf_241129.steps_totals_data_on_both_for_users_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `usesysid`                     `int4`
  `user_name`                    `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### usesysid

### user_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'analyze_totals_for_tables_by_hour', 'view', '# `lf_241129.analyze_totals_for_tables_by_hour`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `schema_id`                                     `int4`
  `schema_name`                                   `varchar`
  `table_id`                                      `int4`
  `table_name`                                    `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_statements_by_all', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_statements_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ ------------
  `statement_name`               `varchar`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### statement_name

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('ro_241129', 'sp_revoke_version_from_group', 'procedure', '# `ro_241129.sp_revoke_version_from_group`

## Description

## Arguments

  argument       data type
  -------------- ------------------
  `version`      `varchar( 6 )`
  `group_name`   `varchar( 127 )`

## Argument Descriptions

### version

### group_name
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_statements_by_second', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_statements_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### stmt

### start_ts

### queue

### compile

### exec

### queries
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_second_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_second_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `n`          `int2`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_users_by_hour', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_users_by_hour`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `user_id`                                     `int4`
  `user_name`                                   `varchar`
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_hour_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_hour_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `del`        `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `save`       `numeric`
  `ssort`      `numeric`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### del

### hjoin

### iinsert

### mjoin

### nloop

### save

### ssort
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_hour', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_schemas_by_hour`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'queries_text_for_leader', 'view', '# `sf_241129.queries_text_for_leader`

## Description

## Columns

  column       data type
  ------------ -------------
  `pid`        `int4`
  `xid`        `int8`
  `start_ts`   `timestamp`
  `n`          `int4`
  `ttext`      `varchar`

## Column Descriptions

### pid

### xid

### start_ts

### n

### ttext
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_all_by_hour', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### start_ts

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'queries_transactions_batches', 'view', '# `lf_241129.queries_transactions_batches`

## Description

## Columns

  column                                data type
  ------------------------------------- -------------
  `user_id`                             `int4`
  `user_name`                           `varchar`
  `session_id`                          `int4`
  `transaction_id`                      `int8`
  `transaction_start_ts`                `timestamp`
  `query_id`                            `int4`
  `queue_id`                            `int2`
  `queue_name`                          `varchar`
  `query_start_ts`                      `timestamp`
  `query_end_ts`                        `timestamp`
  `query_queue_duration`                `interval`
  `query_compile_duration`              `interval`
  `query_exec_duration`                 `interval`
  `query_wlm_duration`                  `interval`
  `query_total_duration`                `interval`
  `query_abort_flag`                    `bool`
  `query_csc_flag`                      `bool`
  `query_leader_node_flag`              `bool`
  `query_rc_flag`                       `bool`
  `query_text`                          `varchar`
  `query_number_blocks_read`            `int8`
  `query_number_blocks_to_disk`         `int8`
  `transaction_end_ts`                  `timestamp`
  `transaction_duration`                `interval`
  `transaction_idle_duration`           `interval`
  `transaction_number_leader_queries`   `int4`
  `transaction_number_worker_queries`   `int4`
  `transaction_number_total_queries`    `int4`
  `transaction_abort_flag`              `bool`
  `batch_id`                            `int8`
  `transaction_batch_queue_start`       `timestamp`
  `transaction_batch_queue_duration`    `interval`
  `batch_start_ts`                      `timestamp`
  `batch_end_ts`                        `timestamp`
  `batch_leader_duration`               `interval`
  `batch_worker_duration`               `interval`
  `batch_queue_duration`                `interval`
  `batch_exec_duration`                 `interval`
  `batch_total_duration`                `interval`
  `number_batch_blocks`                 `int8`
  `number_batch_headers`                `int8`
  `number_batch_transactions`           `int8`

## Column Descriptions

### user_id

### user_name

### session_id

### transaction_id

### transaction_start_ts

### query_id

### queue_id

### queue_name

### query_start_ts

### query_end_ts

### query_queue_duration

### query_compile_duration

### query_exec_duration

### query_wlm_duration

### query_total_duration

### query_abort_flag

### query_csc_flag

### query_leader_node_flag

### query_rc_flag

### query_text

### query_number_blocks_read

### query_number_blocks_to_disk

### transaction_end_ts

### transaction_duration

### transaction_idle_duration

### transaction_number_leader_queries

### transaction_number_worker_queries

### transaction_number_total_queries

### transaction_abort_flag

### batch_id

### transaction_batch_queue_start

### transaction_batch_queue_duration

### batch_start_ts

### batch_end_ts

### batch_leader_duration

### batch_worker_duration

### batch_queue_duration

### batch_exec_duration

### batch_total_duration

### number_batch_blocks

### number_batch_headers

### number_batch_transactions
'),
  ('lf_241129', 'vacuum_totals_for_users_by_second', 'view', '# `lf_241129.vacuum_totals_for_users_by_second`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_tables_by_second', 'view', '# `lf_241129.steps_stats_durations_on_both_for_tables_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `table_id`                    `int4`
  `table_name`                  `varchar`
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_queues_by_all', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_queues_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `qname`     `varchar`
  `queue`     `varchar`
  `compile`   `varchar`
  `exec`      `varchar`
  `queries`   `int4`

## Column Descriptions

### qname

### queue

### compile

### exec

### queries
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_hour_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_hour_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `n`          `int2`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_all_by_minute', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_all_by_minute`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_slices_by_day', 'view', '# `lf_241129.steps_stats_durations_on_both_for_slices_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `slice_ordinal`               `int2`
  `slice_type`                  `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### slice_ordinal

### slice_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_users_by_second', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_users_by_second`

## Description

## Columns

  column                data type
  --------------------- -------------
  `user_id`             `int4`
  `node_type`           `varchar`
  `user_name`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### user_id

### node_type

### user_name

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'privileges_granted_to_users', 'view', '# `lf_241129.privileges_granted_to_users`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `user_id`                                `int4`
  `user_name`                              `varchar`
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `priv_database_create`                   `bool`
  `priv_database_grant_create`             `bool`
  `priv_database_temp`                     `bool`
  `priv_database_grant_temp`               `bool`
  `priv_function_execute`                  `bool`
  `priv_function_grant_execute`            `bool`
  `priv_language_usage`                    `bool`
  `priv_language_grant_usage`              `bool`
  `priv_procedure_execute`                 `bool`
  `priv_procedure_grant_execute`           `bool`
  `priv_schema_create`                     `bool`
  `priv_schema_grant_create`               `bool`
  `priv_schema_usage`                      `bool`
  `priv_schema_grant_usage`                `bool`
  `priv_table_delete`                      `bool`
  `priv_table_grant_delete`                `bool`
  `priv_table_drop`                        `bool`
  `priv_table_grant_drop`                  `bool`
  `priv_table_insert`                      `bool`
  `priv_table_grant_insert`                `bool`
  `priv_table_reference`                   `bool`
  `priv_table_grant_reference`             `bool`
  `priv_table_select`                      `bool`
  `priv_table_grant_select`                `bool`
  `priv_table_update`                      `bool`
  `priv_table_grant_update`                `bool`
  `priv_view_drop`                         `bool`
  `priv_view_grant_drop`                   `bool`
  `priv_view_select`                       `bool`
  `priv_view_grant_select`                 `bool`

## Column Descriptions

### user_id

### user_name

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### priv_database_create

### priv_database_grant_create

### priv_database_temp

### priv_database_grant_temp

### priv_function_execute

### priv_function_grant_execute

### priv_language_usage

### priv_language_grant_usage

### priv_procedure_execute

### priv_procedure_grant_execute

### priv_schema_create

### priv_schema_grant_create

### priv_schema_usage

### priv_schema_grant_usage

### priv_table_delete

### priv_table_grant_delete

### priv_table_drop

### priv_table_grant_drop

### priv_table_insert

### priv_table_grant_insert

### priv_table_reference

### priv_table_grant_reference

### priv_table_select

### priv_table_grant_select

### priv_table_update

### priv_table_grant_update

### priv_view_drop

### priv_view_grant_drop

### priv_view_select

### priv_view_grant_select
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_users_by_second', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_users_by_second`

## Description

## Columns

  column             data type
  ------------------ -------------
  `user_id`          `int4`
  `user_name`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('sf_241129', 'analyze_totals_for_tables_by_hour', 'view', '# `sf_241129.analyze_totals_for_tables_by_hour`

## Description

## Columns

  column          data type
  --------------- -------------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('sf_241129', 'uusers_privileges', 'view', '# `sf_241129.uusers_privileges`

## Description

## Columns

  column               data type
  -------------------- -----------
  `uuser`              `varchar`
  `ttype`              `varchar`
  `sschema`            `varchar`
  `oobject`            `varchar`
  `rt`                 `varchar`
  `args`               `varchar`
  `oowner`             `varchar`
  `priv_origin_type`   `varchar`
  `priv_origin_name`   `varchar`
  `privs`              `varchar`

## Column Descriptions

### uuser

### ttype

### sschema

### oobject

### rt

### args

### oowner

### priv_origin_type

### priv_origin_name

### privs
'),
  ('sf_241129', 'queries_totals_result_cache_for_users_by_all', 'view', '# `sf_241129.queries_totals_result_cache_for_users_by_all`

## Description

## Columns

  column    data type
  --------- -----------
  `uuser`   `varchar`
  `hit`     `int8`
  `miss`    `int8`
  `total`   `int8`
  `%`       `numeric`

## Column Descriptions

### uuser

### hit

### miss

### total

### %
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_users_by_hour', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_users_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `user_id`                      `int4`
  `user_name`                    `varchar`
  `start_ts`                     `timestamp`
  `min_leadin_duration`          `interval`
  `max_leadin_duration`          `interval`
  `med_leadin_duration`          `float4`
  `avg_leadin_duration`          `interval`
  `std_leadin_duration`          `float4`
  `sum_leadin_duration`          `interval`
  `min_wlm_leadin_duration`      `interval`
  `max_wlm_leadin_duration`      `interval`
  `med_wlm_leadin_duration`      `float4`
  `avg_wlm_leadin_duration`      `interval`
  `std_wlm_leadin_duration`      `float4`
  `sum_wlm_leadin_duration`      `interval`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `min_wlm_leadout_duration`     `interval`
  `max_wlm_leadout_duration`     `interval`
  `med_wlm_leadout_duration`     `float4`
  `avg_wlm_leadout_duration`     `interval`
  `std_wlm_leadout_duration`     `float4`
  `sum_wlm_leadout_duration`     `interval`
  `min_leadout_duration`         `interval`
  `max_leadout_duration`         `interval`
  `med_leadout_duration`         `float4`
  `avg_leadout_duration`         `interval`
  `std_leadout_duration`         `float4`
  `sum_leadout_duration`         `interval`
  `min_query_total_duration`     `interval`
  `max_query_total_duration`     `interval`
  `med_query_total_duration`     `float4`
  `avg_query_total_duration`     `interval`
  `std_query_total_duration`     `float4`
  `sum_query_total_duration`     `interval`
  `min_unaccounted_duration`     `interval`
  `max_unaccounted_duration`     `interval`
  `med_unaccounted_duration`     `float4`
  `avg_unaccounted_duration`     `interval`
  `std_unaccounted_duration`     `float4`
  `sum_unaccounted_duration`     `interval`
  `number_queries`               `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_leadin_duration

### max_leadin_duration

### med_leadin_duration

### avg_leadin_duration

### std_leadin_duration

### sum_leadin_duration

### min_wlm_leadin_duration

### max_wlm_leadin_duration

### med_wlm_leadin_duration

### avg_wlm_leadin_duration

### std_wlm_leadin_duration

### sum_wlm_leadin_duration

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### min_wlm_leadout_duration

### max_wlm_leadout_duration

### med_wlm_leadout_duration

### avg_wlm_leadout_duration

### std_wlm_leadout_duration

### sum_wlm_leadout_duration

### min_leadout_duration

### max_leadout_duration

### med_leadout_duration

### avg_leadout_duration

### std_leadout_duration

### sum_leadout_duration

### min_query_total_duration

### max_query_total_duration

### med_query_total_duration

### avg_query_total_duration

### std_query_total_duration

### sum_query_total_duration

### min_unaccounted_duration

### max_unaccounted_duration

### med_unaccounted_duration

### avg_unaccounted_duration

### std_unaccounted_duration

### sum_unaccounted_duration

### number_queries
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_slices_by_second', 'view', '# `lf_241129.steps_stats_durations_on_both_for_slices_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `slice_ordinal`               `int2`
  `slice_type`                  `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### slice_ordinal

### slice_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_all_by_hour', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_all_by_hour`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### start_ts

### number_abort
'),
  ('base_241129', 'fixed_stl_query', 'view', '# `base_241129.fixed_stl_query`

## Description

## Columns

  column                    data type
  ------------------------- -------------
  `query_abort_flag`        `bool`
  `query_csc_flag`          `bool`
  `database_partial_name`   `varchar`
  `query_end_ts`            `timestamp`
  `session_id`              `int4`
  `query_id`                `int4`
  `query_text`              `varchar`
  `query_start_ts`          `timestamp`
  `user_id`                 `int4`
  `transaction_id`          `int8`
  `query_total_duration`    `interval`

## Column Descriptions

### query_abort_flag

### query_csc_flag

### database_partial_name

### query_end_ts

### session_id

### query_id

### query_text

### query_start_ts

### user_id

### transaction_id

### query_total_duration
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_day_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_day_concise`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_tables_by_minute', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_tables_by_minute`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `table_id`            `int4`
  `table_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'analyze_totals_for_tables_by_all', 'view', '# `lf_241129.analyze_totals_for_tables_by_all`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -----------
  `schema_id`                                     `int4`
  `schema_name`                                   `varchar`
  `table_id`                                      `int4`
  `table_name`                                    `varchar`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_minute_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_minute_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_tables_by_day', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_tables_by_day`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `table_id`            `int4`
  `table_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_users_by_second', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### uuser

### start_ts

### aborts
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_all_by_minute', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### start_ts

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'blocks_rows_totals_for_schema', 'view', '# `lf_241129.blocks_rows_totals_for_schema`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `schema_id`                `int4`
  `schema_name`              `varchar`
  `number_sorted_blocks`     `int8`
  `number_desorted_blocks`   `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`
  `number_sorted_rows`       `int8`
  `number_desorted_rows`     `int8`
  `number_unsorted_rows`     `int8`
  `number_total_rows`        `int8`

## Column Descriptions

### schema_id

### schema_name

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_statements_by_all', 'view', '# `lf_241129.steps_stats_durations_on_both_for_statements_by_all`

## Description

## Columns

  column             data type
  ------------------ ------------
  `statement_name`   `varchar`
  `node_type`        `varchar`
  `step_type`        `varchar`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `number_steps`     `int8`

## Column Descriptions

### statement_name

### node_type

### step_type

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### number_steps
'),
  ('sf_241129', 'wqueues_totals_slot_utilization_by_day', 'view', '# `sf_241129.wqueues_totals_slot_utilization_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `qname`      `varchar`
  `slot`       `int2`
  `start_ts`   `date`
  `dur`        `numeric`

## Column Descriptions

### qname

### slot

### start_ts

### dur
'),
  ('sf_241129', 'analyze_totals_for_database_by_all', 'view', '# `sf_241129.analyze_totals_for_database_by_all`

## Description

## Columns

  column          data type
  --------------- -----------
  `dbname`        `varchar`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### dbname

### manual

### automatic

### autoanalyze

### total
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_hour_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_hour_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `node_type`      `varchar`
  `start_ts`       `timestamp`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### node_type

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_all_by_day', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### start_ts

### rread

### to_disk
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_all_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_all_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `n`              `int2`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### n

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_data_on_both_for_node_by_second', 'view', '# `lf_241129.steps_totals_data_on_both_for_node_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_ordinal`                 `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_schemas_by_hour', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_schemas_by_hour`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_all_by_all', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_all_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `rread`     `varchar`
  `to_disk`   `varchar`

## Column Descriptions

### rread

### to_disk
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_queues_by_all', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_queues_by_all`

## Description

## Columns

  column     data type
  ---------- -----------
  `qname`    `varchar`
  `aborts`   `int4`

## Column Descriptions

### qname

### aborts
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_all_by_second', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_all_by_second`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### start_ts

### number_abort
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_node_by_day', 'view', '# `lf_241129.steps_stats_durations_on_both_for_node_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_all_by_second', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_all_by_second`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('lf_241129', 'objects_totals_for_all', 'view', '# `lf_241129.objects_totals_for_all`

## Description

## Columns

  column                               data type
  ------------------------------------ -----------
  `number_schemas`                     `int2`
  `number_tables`                      `int4`
  `number_table_columns`               `int4`
  `number_normal_views`                `int4`
  `number_normal_view_columns`         `int4`
  `number_late_binding_views`          `int4`
  `number_materialized_views`          `int4`
  `number_materialized_view_columns`   `int4`
  `number_functions`                   `int4`
  `number_procedures`                  `int4`

## Column Descriptions

### number_schemas

### number_tables

### number_table_columns

### number_normal_views

### number_normal_view_columns

### number_late_binding_views

### number_materialized_views

### number_materialized_view_columns

### number_functions

### number_procedures
'),
  ('base_241129', 'fixed_pg_proc_for_functions', 'view', '# `base_241129.fixed_pg_proc_for_functions`

## Description

## Columns

  column            data type
  ----------------- -------------
  `function_id`     `int4`
  `function_name`   `varchar`
  `schema_id`       `int4`
  `user_id`         `int4`
  `language_id`     `int4`
  `language_name`   `varchar`
  `provolatile`     `bpchar`
  `prorettype`      `oid`
  `proretset`       `bool`
  `function_text`   `text`
  `proargtypes`     `oidvector`

## Column Descriptions

### function_id

### function_name

### schema_id

### user_id

### language_id

### language_name

### provolatile

### prorettype

### proretset

### function_text

### proargtypes
'),
  ('sf_241129', 'transactions_stats_commits_by_day', 'view', '# `sf_241129.transactions_stats_commits_by_day`

## Description

## Columns

  column           data type
  ---------------- -----------
  `start_ts`       `date`
  `xid queuing`    `varchar`
  `bid queuing`    `varchar`
  `ldr duration`   `varchar`
  `wrk duration`   `varchar`
  `xids/bid`       `varchar`
  `blks/bid`       `varchar`
  `bids`           `int8`

## Column Descriptions

### start_ts

### xid queuing

### bid queuing

### ldr duration

### wrk duration

### xids/bid

### blks/bid

### bids
'),
  ('lf_241129', 'locks', 'view', '# `lf_241129.locks`

## Description

## Columns

  column             data type
  ------------------ ------------
  `schema_id`        `int4`
  `schema_name`      `varchar`
  `table_id`         `int4`
  `table_name`       `varchar`
  `duration`         `interval`
  `lock_type`        `varchar`
  `lock_state`       `varchar`
  `session_id`       `int4`
  `transaction_id`   `int8`
  `user_id`          `int4`
  `user_name`        `varchar`
  `query_text`       `varchar`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### duration

### lock_type

### lock_state

### session_id

### transaction_id

### user_id

### user_name

### query_text
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_all_by_day', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### start_ts

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_users_by_minute', 'view', '# `lf_241129.steps_stats_durations_on_both_for_users_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'vacuum_totals_for_tables_by_all', 'view', '# `sf_241129.vacuum_totals_for_tables_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `manual`    `varchar`
  `auto`      `varchar`
  `total`     `int4`
  `dur`       `numeric`

## Column Descriptions

### sschema

### ttable

### manual

### auto

### total

### dur
'),
  ('lf_241129', 'ttables_blocks_rows', 'view', '# `lf_241129.ttables_blocks_rows`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `table_creation`                 `timestamp`
  `table_age`                      `interval`
  `table_number_columns`           `int2`
  `table_sort_type`                `varchar`
  `table_number_sortkeys`          `int2`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`
  `number_slices`                  `int2`
  `number_sorted_blocks`           `int8`
  `number_desorted_blocks`         `int8`
  `number_unsorted_blocks`         `int8`
  `number_total_blocks`            `int8`
  `number_sorted_rows`             `int8`
  `number_desorted_rows`           `int8`
  `number_unsorted_rows`           `int8`
  `number_total_rows`              `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### user_id

### user_name

### table_creation

### table_age

### table_number_columns

### table_sort_type

### table_number_sortkeys

### table_distribution_type

### table_distribution_auto_flag

### number_slices

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('lf_241129', 'vacuum_totals_for_tables_by_all', 'view', '# `lf_241129.vacuum_totals_for_tables_by_all`

## Description

## Columns

  column                           data type
  -------------------------------- ------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('lf_241129', 'wqueues_qmr_log', 'view', '# `lf_241129.wqueues_qmr_log`

## Description

## Columns

  column          data type
  --------------- -------------
  `event_ts`      `timestamp`
  `rule_name`     `varchar`
  `rule_action`   `varchar`
  `queue_id`      `int2`
  `queue_name`    `varchar`
  `user_id`       `int4`
  `user_name`     `varchar`
  `query_id`      `int4`

## Column Descriptions

### event_ts

### rule_name

### rule_action

### queue_id

### queue_name

### user_id

### user_name

### query_id
'),
  ('sf_241129', 'vacuum_totals_for_all_by_minute', 'view', '# `sf_241129.vacuum_totals_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### start_ts

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'ttables_stats_dists_sorts_for_all_by_blocks', 'view', '# `sf_241129.ttables_stats_dists_sorts_for_all_by_blocks`

## Description

## Columns

  column   data type
  -------- -----------
  `aall`   `varchar`
  `even`   `varchar`
  `kkey`   `varchar`

## Column Descriptions

### aall

### even

### kkey
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_query', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_query`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `query_id`                    `int4`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### query_id

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('base_241129', 'ttables_basic_info_full', 'view', '# `base_241129.ttables_basic_info_full`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `user_id`                        `int4`
  `table_creation`                 `timestamp`
  `table_age`                      `interval`
  `table_number_columns`           `int2`
  `table_sort_type`                `varchar`
  `table_number_sortkeys`          `int2`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### user_id

### table_creation

### table_age

### table_number_columns

### table_sort_type

### table_number_sortkeys

### table_distribution_type

### table_distribution_auto_flag
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_hour_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_hour_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `n`              `int2`
  `s`              `int2`
  `t`              `varchar`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_second_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_second_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `del`        `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `save`       `numeric`
  `ssort`      `numeric`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### del

### hjoin

### iinsert

### mjoin

### nloop

### save

### ssort
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_all_by_minute', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### duration

### queries
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_second_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_second_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `n`              `int2`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('base_241129', 'queries_for_leader', 'view', '# `base_241129.queries_for_leader`

## Description

## Columns

  column                     data type
  -------------------------- -------------
  `user_id`                  `int4`
  `session_id`               `int4`
  `transaction_id`           `int8`
  `queue_id`                 `int2`
  `query_start_ts`           `timestamp`
  `query_end_ts`             `timestamp`
  `query_total_duration`     `interval`
  `query_leader_node_flag`   `bool`
  `query_text`               `varchar`

## Column Descriptions

### user_id

### session_id

### transaction_id

### queue_id

### query_start_ts

### query_end_ts

### query_total_duration

### query_leader_node_flag

### query_text
'),
  ('sf_241129', 'groups_privileges', 'view', '# `sf_241129.groups_privileges`

## Description

## Columns

  column      data type
  ----------- -----------
  `ggroup`    `varchar`
  `ttype`     `varchar`
  `sschema`   `varchar`
  `oobject`   `varchar`
  `rt`        `varchar`
  `args`      `varchar`
  `oowner`    `varchar`
  `privs`     `varchar`

## Column Descriptions

### ggroup

### ttype

### sschema

### oobject

### rt

### args

### oowner

### privs
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_tables_by_all', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_tables_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `ddelete`   `varchar`
  `iinsert`   `varchar`
  `scan`      `varchar`

## Column Descriptions

### sschema

### ttable

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'analyze_totals_for_users_by_all', 'view', '# `lf_241129.analyze_totals_for_users_by_all`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -----------
  `user_id`                                       `int4`
  `user_name`                                     `varchar`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### user_id

### user_name

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_all', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_schemas_by_all`

## Description

## Columns

  column                data type
  --------------------- -----------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `node_type`           `varchar`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### node_type

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'queries_text_on_both_full', 'view', '# `sf_241129.queries_text_on_both_full`

## Description

## Columns

  column       data type
  ------------ -------------
  `pid`        `int4`
  `xid`        `int8`
  `qid`        `int4`
  `start_ts`   `timestamp`
  `n`          `int4`
  `ttext`      `varchar`

## Column Descriptions

### pid

### xid

### qid

### start_ts

### n

### ttext
'),
  ('sf_241129', 'privileges_granted_to_public', 'view', '# `sf_241129.privileges_granted_to_public`

## Description

## Columns

  column      data type
  ----------- -----------
  `ttype`     `varchar`
  `sschema`   `varchar`
  `oobject`   `varchar`
  `rt`        `varchar`
  `args`      `varchar`
  `oowner`    `varchar`
  `privs`     `varchar`

## Column Descriptions

### ttype

### sschema

### oobject

### rt

### args

### oowner

### privs
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_query', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_query`

## Description

## Columns

  column      data type
  ----------- -----------
  `qid`       `int4`
  `node`      `varchar`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### qid

### node

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_tables_by_hour', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_tables_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### ddelete

### iinsert

### scan
'),
  ('base_241129', 'queries_for_all', 'view', '# `base_241129.queries_for_all`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `user_id`                       `int4`
  `session_id`                    `int4`
  `transaction_id`                `int8`
  `query_id`                      `int4`
  `queue_id`                      `int2`
  `query_start_ts`                `timestamp`
  `query_end_ts`                  `timestamp`
  `query_queue_duration`          `interval`
  `query_compile_exec_duration`   `interval`
  `query_wlm_duration`            `interval`
  `query_total_duration`          `interval`
  `query_abort_flag`              `bool`
  `query_csc_flag`                `bool`
  `query_leader_node_flag`        `bool`
  `query_rc_flag`                 `bool`
  `query_text`                    `varchar`

## Column Descriptions

### user_id

### session_id

### transaction_id

### query_id

### queue_id

### query_start_ts

### query_end_ts

### query_queue_duration

### query_compile_exec_duration

### query_wlm_duration

### query_total_duration

### query_abort_flag

### query_csc_flag

### query_leader_node_flag

### query_rc_flag

### query_text
'),
  ('lf_241129', 'analyze_totals_for_database_by_second', 'view', '# `lf_241129.analyze_totals_for_database_by_second`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `database_name`                                 `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### database_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'analyze_totals_for_users_by_day', 'view', '# `lf_241129.analyze_totals_for_users_by_day`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `user_id`                                       `int4`
  `user_name`                                     `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'uusers', 'view', '# `lf_241129.uusers`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `user_type`                   `varchar`
  `user_syslog`                 `varchar`
  `number_databases`            `int2`
  `number_functions`            `int4`
  `number_procedures`           `int4`
  `number_schemas`              `int2`
  `number_tables`               `int4`
  `number_normal_views`         `int4`
  `number_late_binding_views`   `int4`
  `number_materialized_views`   `int4`

## Column Descriptions

### user_id

### user_name

### user_type

### user_syslog

### number_databases

### number_functions

### number_procedures

### number_schemas

### number_tables

### number_normal_views

### number_late_binding_views

### number_materialized_views
'),
  ('ro_241129', 'sp_grant_version_to_public', 'procedure', '# `ro_241129.sp_grant_version_to_public`

## Description

## Arguments

  argument    data type
  ----------- ----------------
  `version`   `varchar( 6 )`

## Argument Descriptions

### version
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_node_by_second', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_node_by_second`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_second_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_second_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('base_241129', 'ttables_basic_info_fast', 'view', '# `base_241129.ttables_basic_info_fast`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `user_id`                        `int4`
  `table_creation`                 `timestamp`
  `table_age`                      `interval`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### user_id

### table_creation

### table_age

### table_distribution_type

### table_distribution_auto_flag
'),
  ('sf_241129', 'wqueues_totals_slot_utilization_by_second', 'view', '# `sf_241129.wqueues_totals_slot_utilization_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `slot`       `int2`
  `start_ts`   `timestamp`
  `dur`        `numeric`

## Column Descriptions

### qname

### slot

### start_ts

### dur
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_queues_by_day', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_queues_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `qname`      `varchar`
  `start_ts`   `date`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### qname

### start_ts

### queue

### compile

### exec

### queries
'),
  ('base_241129', 'fixed_stv_query_metrics', 'view', '# `base_241129.fixed_stv_query_metrics`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `query_number_blocks_read`      `int8`
  `query_number_blocks_to_disk`   `int8`
  `query_cpu_time`                `int8`
  `query_id`                      `int4`
  `query_number_rows`             `int8`
  `queue_id`                      `int2`
  `query_start_ts`                `timestamp`
  `user_id`                       `int4`

## Column Descriptions

### query_number_blocks_read

### query_number_blocks_to_disk

### query_cpu_time

### query_id

### query_number_rows

### queue_id

### query_start_ts

### user_id
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_queues_by_hour', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_queues_by_hour`

## Description

## Columns

  column           data type
  ---------------- -------------
  `queue_id`       `int2`
  `queue_name`     `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### number_abort
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_queues_by_minute', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_queues_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `queue_id`                     `int2`
  `queue_name`                   `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('base_241129', 'fixed_stl_querytext', 'view', '# `base_241129.fixed_stl_querytext`

## Description

## Columns

  column             data type
  ------------------ -----------
  `user_id`          `int4`
  `transaction_id`   `int8`
  `session_id`       `int4`
  `query_id`         `int4`
  `sequence`         `int4`
  `query_text`       `varchar`

## Column Descriptions

### user_id

### transaction_id

### session_id

### query_id

### sequence

### query_text
'),
  ('lf_241129', 'sschemas_blocks_rows', 'view', '# `lf_241129.sschemas_blocks_rows`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `number_tables`               `int4`
  `number_normal_views`         `int4`
  `number_late_binding_views`   `int4`
  `number_materialized_views`   `int4`
  `number_functions`            `int4`
  `number_procedures`           `int4`
  `number_sorted_blocks`        `int8`
  `number_desorted_blocks`      `int8`
  `number_unsorted_blocks`      `int8`
  `number_total_blocks`         `int8`
  `number_sorted_rows`          `int8`
  `number_desorted_rows`        `int8`
  `number_unsorted_rows`        `int8`
  `number_total_rows`           `int8`

## Column Descriptions

### schema_id

### schema_name

### user_id

### user_name

### number_tables

### number_normal_views

### number_late_binding_views

### number_materialized_views

### number_functions

### number_procedures

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_users_by_all', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_users_by_all`

## Description

## Columns

  column          data type
  --------------- -----------
  `uuser`         `varchar`
  `leadin`        `varchar`
  `wlm_leadin`    `varchar`
  `queue`         `varchar`
  `compile`       `varchar`
  `exec`          `varchar`
  `wlm_leadout`   `varchar`
  `leadout`       `varchar`
  `total`         `varchar`
  `unact`         `varchar`
  `queries`       `int4`

## Column Descriptions

### uuser

### leadin

### wlm_leadin

### queue

### compile

### exec

### wlm_leadout

### leadout

### total

### unact

### queries
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_users_by_second', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_all_by_day', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_all_by_day`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### start_ts

### number_abort
'),
  ('base_241129', 'fixed_pg_get_late_binding_view_cols', 'view', '# `base_241129.fixed_pg_get_late_binding_view_cols`

## Description

## Columns

  column        data type
  ------------- -----------
  `attnum`      `int2`
  `attname`     `varchar`
  `attrelid`    `int4`
  `atttypmod`   `int4`
  `atttypid`    `int4`

## Column Descriptions

### attnum

### attname

### attrelid

### atttypmod

### atttypid
'),
  ('ro_241129', 'sp_grant_version_to_group', 'procedure', '# `ro_241129.sp_grant_version_to_group`

## Description

## Arguments

  argument       data type
  -------------- ------------------
  `version`      `varchar( 6 )`
  `group_name`   `varchar( 127 )`

## Argument Descriptions

### version

### group_name
'),
  ('sf_241129', 'queries_text_on_both_fast', 'view', '# `sf_241129.queries_text_on_both_fast`

## Description

## Columns

  column       data type
  ------------ -------------
  `pid`        `int4`
  `xid`        `int8`
  `qid`        `int4`
  `start_ts`   `timestamp`
  `n`          `int4`
  `ttext`      `varchar`

## Column Descriptions

### pid

### xid

### qid

### start_ts

### n

### ttext
'),
  ('sf_241129', 'vacuum_stats_for_all_by_all', 'view', '# `sf_241129.vacuum_stats_for_all_by_all`

## Description

## Columns

  column              data type
  ------------------- -----------
  `new blocks`        `varchar`
  `replaced blocks`   `varchar`
  `new rows`          `varchar`
  `deleted rows`      `varchar`
  `xid duration`      `varchar`
  `qids`              `int8`
  `xids`              `int8`

## Column Descriptions

### new blocks

### replaced blocks

### new rows

### deleted rows

### xid duration

### qids

### xids
'),
  ('base_241129', 'ttables_columns_basic_info', 'view', '# `base_241129.ttables_columns_basic_info`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `schema_id`                    `int4`
  `schema_name`                  `varchar`
  `table_id`                     `int4`
  `table_name`                   `varchar`
  `column_ordinal`               `int2`
  `column_name`                  `varchar`
  `column_data_type_id`          `int4`
  `column_base_data_type_name`   `varchar`
  `column_octet_length`          `int4`
  `column_precision`             `int2`
  `column_scale`                 `int2`
  `column_full_data_type_name`   `varchar`
  `column_encoding_id`           `int4`
  `column_encoding_name`         `varchar`
  `column_sortkey_ordinal`       `int2`
  `column_distkey`               `bool`
  `column_notnull`               `bool`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### column_ordinal

### column_name

### column_data_type_id

### column_base_data_type_name

### column_octet_length

### column_precision

### column_scale

### column_full_data_type_name

### column_encoding_id

### column_encoding_name

### column_sortkey_ordinal

### column_distkey

### column_notnull
'),
  ('lf_241129', 'transactions_totals_commits_by_minute', 'view', '# `lf_241129.transactions_totals_commits_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `sum_batch_leader_duration`   `interval`
  `sum_batch_worker_duration`   `interval`
  `sum_batch_idle_duration`     `interval`
  `sum_batch_exec_duration`     `interval`

## Column Descriptions

### start_ts

### sum_batch_leader_duration

### sum_batch_worker_duration

### sum_batch_idle_duration

### sum_batch_exec_duration
'),
  ('sf_241129', 'analyze_totals_for_all_by_day', 'view', '# `sf_241129.analyze_totals_for_all_by_day`

## Description

## Columns

  column          data type
  --------------- -----------
  `start_ts`      `date`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_node_by_hour', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_node_by_hour`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'analyze_totals_for_tables_by_minute', 'view', '# `lf_241129.analyze_totals_for_tables_by_minute`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `schema_id`                                     `int4`
  `schema_name`                                   `varchar`
  `table_id`                                      `int4`
  `table_name`                                    `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_tables_by_day', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_tables_by_day`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `table_id`          `int4`
  `table_name`        `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('sf_241129', 'views_materialized_table', 'view', '# `sf_241129.views_materialized_table`

## Description

## Columns

  column      data type
  ----------- ------------
  `sschema`   `varchar`
  `vview`     `varchar`
  `uuser`     `varchar`
  `status`    `varchar`
  `ttable`    `varchar`
  `age`       `interval`
  `cols`      `int2`
  `ssort`     `varchar`
  `skeys`     `int2`
  `dist`      `varchar`

## Column Descriptions

### sschema

### vview

### uuser

### status

### ttable

### age

### cols

### ssort

### skeys

### dist
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_users_by_day', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `start_ts`   `date`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### uuser

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('sf_241129', 'steps_totals_data_on_both_for_query', 'view', '# `sf_241129.steps_totals_data_on_both_for_query`

## Description

## Columns

  column      data type
  ----------- -----------
  `qid`       `int4`
  `node`      `varchar`
  `aggr`      `varchar`
  `bcast`     `varchar`
  `ddelete`   `varchar`
  `dist`      `varchar`
  `hash`      `varchar`
  `hjoin`     `varchar`
  `iinsert`   `varchar`
  `llimit`    `varchar`
  `mmerge`    `varchar`
  `mjoin`     `varchar`
  `nloop`     `varchar`
  `rreturn`   `varchar`
  `save`      `varchar`
  `ssort`     `varchar`
  `uunique`   `varchar`
  `wwindow`   `varchar`

## Column Descriptions

### qid

### node

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'blocks_rows_totals_for_user', 'view', '# `lf_241129.blocks_rows_totals_for_user`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `user_id`                  `int4`
  `user_name`                `varchar`
  `number_sorted_blocks`     `int8`
  `number_desorted_blocks`   `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`
  `number_sorted_rows`       `int8`
  `number_desorted_rows`     `int8`
  `number_unsorted_rows`     `int8`
  `number_total_rows`        `int8`

## Column Descriptions

### user_id

### user_name

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('sf_241129', 'views_text', 'view', '# `sf_241129.views_text`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `vview`     `varchar`
  `ttext`     `varchar`

## Column Descriptions

### sschema

### vview

### ttext
'),
  ('lf_241129', 'steps_totals_data_on_both_for_users_by_hour', 'view', '# `lf_241129.steps_totals_data_on_both_for_users_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `usesysid`                     `int4`
  `user_name`                    `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### usesysid

### user_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'wqueues_routing_rules', 'view', '# `sf_241129.wqueues_routing_rules`

## Description

## Columns

  column        data type
  ------------- -----------
  `oorder`      `int2`
  `qgwc`        `varchar`
  `ugwc`        `varchar`
  `condition`   `varchar`
  `qname`       `varchar`

## Column Descriptions

### oorder

### qgwc

### ugwc

### condition

### qname
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_all_by_all', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_all_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ ------------
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('ro_241129', 'installed_versions', 'view', '# `ro_241129.installed_versions`

## Description

## Columns

  column       data type
  ------------ -----------
  `vversion`   `varchar`

## Column Descriptions

### vversion
'),
  ('sf_241129', 'vacuum_stats_for_all_by_day', 'view', '# `sf_241129.vacuum_stats_for_all_by_day`

## Description

## Columns

  column              data type
  ------------------- -----------
  `start_ts`          `date`
  `new blocks`        `varchar`
  `replaced blocks`   `varchar`
  `new rows`          `varchar`
  `deleted rows`      `varchar`
  `xid duration`      `varchar`
  `qids`              `int8`
  `xids`              `int8`

## Column Descriptions

### start_ts

### new blocks

### replaced blocks

### new rows

### deleted rows

### xid duration

### qids

### xids
'),
  ('base_241129', 'transactions_blocks', 'view', '# `base_241129.transactions_blocks`

## Description

## Columns

  column                                data type
  ------------------------------------- -----------
  `transaction_id`                      `int8`
  `transaction_number_blocks_read`      `int8`
  `transaction_number_blocks_to_disk`   `int8`

## Column Descriptions

### transaction_id

### transaction_number_blocks_read

### transaction_number_blocks_to_disk
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_hour_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_hour_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `uuser`          `varchar`
  `node`           `varchar`
  `start_ts`       `timestamp`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('base_241129', 'fixed_stv_blocklist', 'view', '# `base_241129.fixed_stv_blocklist`

## Description

## Columns

  column                  data type
  ----------------------- -----------
  `slice_ordinal`         `int2`
  `column_ordinal`        `int2`
  `table_id`              `int4`
  `block_ordinal`         `int4`
  `block_number_values`   `int4`
  `block_min_value`       `int8`
  `block_max_value`       `int8`
  `block_sorted_flag`     `bool`

## Column Descriptions

### slice_ordinal

### column_ordinal

### table_id

### block_ordinal

### block_number_values

### block_min_value

### block_max_value

### block_sorted_flag
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_day_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_day_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `start_ts`       `date`
  `n`              `int2`
  `s`              `int2`
  `t`              `varchar`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_tables_by_hour', 'view', '# `lf_241129.steps_stats_durations_on_both_for_tables_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `table_id`                    `int4`
  `table_name`                  `varchar`
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_day_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_day_concise`

## Description

## Columns

  column       data type
  ------------ -----------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `aggr`       `numeric`
  `del`        `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `save`       `numeric`
  `ssort`      `numeric`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### del

### hjoin

### iinsert

### mjoin

### nloop

### save

### ssort
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_tables_by_all', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_tables_by_all`

## Description

## Columns

  column              data type
  ------------------- -----------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `table_id`          `int4`
  `table_name`        `varchar`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('sf_241129', 'analyze_totals_for_all_by_second', 'view', '# `sf_241129.analyze_totals_for_all_by_second`

## Description

## Columns

  column          data type
  --------------- -------------
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'vacuum_totals_for_tables_by_minute', 'view', '# `lf_241129.vacuum_totals_for_tables_by_minute`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'queries_totals_result_cache_for_query', 'view', '# `sf_241129.queries_totals_result_cache_for_query`

## Description

## Columns

  column    data type
  --------- -----------
  `qid`     `int4`
  `hit`     `int8`
  `miss`    `int8`
  `total`   `int8`
  `%`       `numeric`

## Column Descriptions

### qid

### hit

### miss

### total

### %
'),
  ('lf_241129', 'steps_totals_data_on_both_for_statements_by_second', 'view', '# `lf_241129.steps_totals_data_on_both_for_statements_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### statement_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'transactions_totals_commits_by_second', 'view', '# `sf_241129.transactions_totals_commits_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `leader`     `numeric`
  `worker`     `numeric`
  `idle`       `numeric`
  `exec`       `numeric`

## Column Descriptions

### start_ts

### leader

### worker

### idle

### exec
'),
  ('lf_241129', 'ttables_creation_for_schemas_by_second', 'view', '# `lf_241129.ttables_creation_for_schemas_by_second`

## Description

## Columns

  column            data type
  ----------------- -------------
  `schema_name`     `varchar`
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### schema_name

### start_ts

### number_tables
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_statements_by_hour', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_statements_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### statement_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('ro_241129', 'sp_revoke_versionless_sf_schema_from_role', 'procedure', '# `ro_241129.sp_revoke_versionless_sf_schema_from_role`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `role_name`   `varchar( 127 )`

## Argument Descriptions

### role_name
'),
  ('lf_241129', 'wqueues_totals_slot_utilization_by_hour', 'view', '# `lf_241129.wqueues_totals_slot_utilization_by_hour`

## Description

## Columns

  column          data type
  --------------- -------------
  `queue_id`      `int2`
  `queue_name`    `varchar`
  `slot_number`   `int2`
  `start_ts`      `timestamp`
  `duration`      `interval`

## Column Descriptions

### queue_id

### queue_name

### slot_number

### start_ts

### duration
'),
  ('lf_241129', 'analyze_totals_for_all_by_minute', 'view', '# `lf_241129.analyze_totals_for_all_by_minute`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_tables_by_second', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_tables_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_all_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_all_concise`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `node`      `varchar`
  `aggr`      `numeric`
  `del`       `numeric`
  `hjoin`     `numeric`
  `iinsert`   `numeric`
  `mjoin`     `numeric`
  `nloop`     `numeric`
  `save`      `numeric`
  `ssort`     `numeric`

## Column Descriptions

### sschema

### ttable

### node

### aggr

### del

### hjoin

### iinsert

### mjoin

### nloop

### save

### ssort
'),
  ('sf_241129', 'queries_compilation', 'view', '# `sf_241129.queries_compilation`

## Description

## Columns

  column      data type
  ----------- -----------
  `pid`       `int4`
  `xid`       `int8`
  `qid`       `int4`
  `lncache`   `int2`
  `offload`   `int2`
  `total`     `int2`
  `dur`       `numeric`

## Column Descriptions

### pid

### xid

### qid

### lncache

### offload

### total

### dur
'),
  ('sf_241129', 'transactions_stats_commits_by_second', 'view', '# `sf_241129.transactions_stats_commits_by_second`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `xid queuing`    `varchar`
  `bid queuing`    `varchar`
  `ldr duration`   `varchar`
  `wrk duration`   `varchar`
  `xids/bid`       `varchar`
  `blks/bid`       `varchar`
  `bids`           `int8`

## Column Descriptions

### start_ts

### xid queuing

### bid queuing

### ldr duration

### wrk duration

### xids/bid

### blks/bid

### bids
'),
  ('lf_241129', 'steps_totals_data_on_both_for_slices_by_day', 'view', '# `lf_241129.steps_totals_data_on_both_for_slices_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `slice_ordinal`                `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### slice_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_minute_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_minute_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `node_type`      `varchar`
  `start_ts`       `timestamp`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### node_type

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_node_by_all', 'view', '# `lf_241129.steps_stats_durations_on_both_for_node_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- ------------
  `node_ordinal`                `int2`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### node_ordinal

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('ro_241129', 'sp_revoke_version_from_user', 'procedure', '# `ro_241129.sp_revoke_version_from_user`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `version`     `varchar( 6 )`
  `user_name`   `varchar( 127 )`

## Argument Descriptions

### version

### user_name
'),
  ('sf_241129', 'queries_for_worker', 'view', '# `sf_241129.queries_for_worker`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `pid`        `int4`
  `xid`        `int8`
  `qid`        `int4`
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `end_ts`     `timestamp`
  `queue`      `numeric`
  `cexec`      `numeric`
  `total`      `numeric`
  `flags`      `varchar`
  `ttext`      `varchar`

## Column Descriptions

### uuser

### pid

### xid

### qid

### qname

### start_ts

### end_ts

### queue

### cexec

### total

### flags

### ttext
'),
  ('base_241129', 'privileges_all', 'view', '# `base_241129.privileges_all`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `acl_recipient_type`                     `varchar`
  `acl_recipient_id`                       `int4`
  `acl_recipient_name`                     `varchar`
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `acl_privileges`                         `varchar`

## Column Descriptions

### acl_recipient_type

### acl_recipient_id

### acl_recipient_name

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### acl_privileges
'),
  ('base_241129', 'generate_days', 'view', '# `base_241129.generate_days`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`

## Column Descriptions

### start_ts
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_second_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_second_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `n`              `int2`
  `s`              `int2`
  `t`              `varchar`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('sf_241129', 'ffunctions_text', 'view', '# `sf_241129.ffunctions_text`

## Description

## Columns

  column    data type
  --------- -----------
  `fid`     `int4`
  `ttext`   `varchar`

## Column Descriptions

### fid

### ttext
'),
  ('base_241129', 'privileges_groups', 'view', '# `base_241129.privileges_groups`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `acl_recipient_group_id`                 `int4`
  `acl_recipient_group_name`               `varchar`
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `acl_privileges`                         `varchar`

## Column Descriptions

### acl_recipient_group_id

### acl_recipient_group_name

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### acl_privileges
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_all_by_all', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_all_by_all`

## Description

## Columns

  column           data type
  ---------------- -----------
  `number_abort`   `int4`

## Column Descriptions

### number_abort
'),
  ('base_241129', 'fixed_stl_query_metrics', 'view', '# `base_241129.fixed_stl_query_metrics`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `query_number_blocks_read`      `int8`
  `query_number_blocks_to_disk`   `int8`
  `query_cpu_time`                `int8`
  `query_id`                      `int4`
  `query_number_rows`             `int8`
  `queue_id`                      `int2`
  `query_start_ts`                `timestamp`
  `user_id`                       `int4`

## Column Descriptions

### query_number_blocks_read

### query_number_blocks_to_disk

### query_cpu_time

### query_id

### query_number_rows

### queue_id

### query_start_ts

### user_id
'),
  ('lf_241129', 'analyze_totals_for_all_by_second', 'view', '# `lf_241129.analyze_totals_for_all_by_second`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_users_by_day', 'view', '# `lf_241129.steps_stats_durations_on_both_for_users_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_users_by_minute', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_users_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `user_id`                      `int4`
  `user_name`                    `varchar`
  `start_ts`                     `timestamp`
  `min_leadin_duration`          `interval`
  `max_leadin_duration`          `interval`
  `med_leadin_duration`          `float4`
  `avg_leadin_duration`          `interval`
  `std_leadin_duration`          `float4`
  `sum_leadin_duration`          `interval`
  `min_wlm_leadin_duration`      `interval`
  `max_wlm_leadin_duration`      `interval`
  `med_wlm_leadin_duration`      `float4`
  `avg_wlm_leadin_duration`      `interval`
  `std_wlm_leadin_duration`      `float4`
  `sum_wlm_leadin_duration`      `interval`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `min_wlm_leadout_duration`     `interval`
  `max_wlm_leadout_duration`     `interval`
  `med_wlm_leadout_duration`     `float4`
  `avg_wlm_leadout_duration`     `interval`
  `std_wlm_leadout_duration`     `float4`
  `sum_wlm_leadout_duration`     `interval`
  `min_leadout_duration`         `interval`
  `max_leadout_duration`         `interval`
  `med_leadout_duration`         `float4`
  `avg_leadout_duration`         `interval`
  `std_leadout_duration`         `float4`
  `sum_leadout_duration`         `interval`
  `min_query_total_duration`     `interval`
  `max_query_total_duration`     `interval`
  `med_query_total_duration`     `float4`
  `avg_query_total_duration`     `interval`
  `std_query_total_duration`     `float4`
  `sum_query_total_duration`     `interval`
  `min_unaccounted_duration`     `interval`
  `max_unaccounted_duration`     `interval`
  `med_unaccounted_duration`     `float4`
  `avg_unaccounted_duration`     `interval`
  `std_unaccounted_duration`     `float4`
  `sum_unaccounted_duration`     `interval`
  `number_queries`               `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_leadin_duration

### max_leadin_duration

### med_leadin_duration

### avg_leadin_duration

### std_leadin_duration

### sum_leadin_duration

### min_wlm_leadin_duration

### max_wlm_leadin_duration

### med_wlm_leadin_duration

### avg_wlm_leadin_duration

### std_wlm_leadin_duration

### sum_wlm_leadin_duration

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### min_wlm_leadout_duration

### max_wlm_leadout_duration

### med_wlm_leadout_duration

### avg_wlm_leadout_duration

### std_wlm_leadout_duration

### sum_wlm_leadout_duration

### min_leadout_duration

### max_leadout_duration

### med_leadout_duration

### avg_leadout_duration

### std_leadout_duration

### sum_leadout_duration

### min_query_total_duration

### max_query_total_duration

### med_query_total_duration

### avg_query_total_duration

### std_query_total_duration

### sum_query_total_duration

### min_unaccounted_duration

### max_unaccounted_duration

### med_unaccounted_duration

### avg_unaccounted_duration

### std_unaccounted_duration

### sum_unaccounted_duration

### number_queries
'),
  ('lf_241129', 'vacuum_totals_for_all_by_all', 'view', '# `lf_241129.vacuum_totals_for_all_by_all`

## Description

## Columns

  column                           data type
  -------------------------------- ------------
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('lf_241129', 'ttables_zonemap', 'view', '# `lf_241129.ttables_zonemap`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `schema_id`                    `int4`
  `schema_name`                  `varchar`
  `table_id`                     `int4`
  `table_name`                   `varchar`
  `column_ordinal`               `int2`
  `column_name`                  `varchar`
  `column_full_data_type_name`   `varchar`
  `column_encoding_name`         `varchar`
  `number_00_20_blocks`          `int8`
  `number_20_40_blocks`          `int8`
  `number_40_60_blocks`          `int8`
  `number_60_80_blocks`          `int8`
  `number_80_100_blocks`         `int8`
  `number_overflow_blocks`       `int8`
  `number_underflow_blocks`      `int8`
  `number_saturated_blocks`      `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### column_ordinal

### column_name

### column_full_data_type_name

### column_encoding_name

### number_00_20_blocks

### number_20_40_blocks

### number_40_60_blocks

### number_60_80_blocks

### number_80_100_blocks

### number_overflow_blocks

### number_underflow_blocks

### number_saturated_blocks
'),
  ('lf_241129', 'queries_stats_durations_on_both_by_transaction', 'view', '# `lf_241129.queries_stats_durations_on_both_by_transaction`

## Description

## Columns

  column                         data type
  ------------------------------ ------------
  `transaction_id`               `int8`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `min_idle_duration`            `interval`
  `max_idle_duration`            `interval`
  `med_idle_duration`            `float4`
  `avg_idle_duration`            `interval`
  `std_idle_duration`            `float4`
  `sum_idle_duration`            `interval`
  `number_queries`               `int4`

## Column Descriptions

### transaction_id

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### min_idle_duration

### max_idle_duration

### med_idle_duration

### avg_idle_duration

### std_idle_duration

### sum_idle_duration

### number_queries
'),
  ('sf_241129', 'groups', 'view', '# `sf_241129.groups`

## Description

## Columns

  column     data type
  ---------- -----------
  `ggroup`   `varchar`
  `users`    `int4`

## Column Descriptions

### ggroup

### users
'),
  ('sf_241129', 'nodes', 'view', '# `sf_241129.nodes`

## Description

## Columns

  column           data type
  ---------------- -----------
  `node`           `int2`
  `slices`         `varchar`
  `disks`          `int2`
  `total_blocks`   `varchar`
  `disk_used`      `varchar`
  `rms_used`       `varchar`
  `total_used`     `varchar`

## Column Descriptions

### node

### slices

### disks

### total_blocks

### disk_used

### rms_used

### total_used
'),
  ('sf_241129', 'sessions_full', 'view', '# `sf_241129.sessions_full`

## Description

## Columns

  column    data type
  --------- -----------
  `uuser`   `varchar`
  `pid`     `int4`
  `tdb`     `varchar`
  `dur`     `numeric`
  `locks`   `int4`
  `ip`      `varchar`

## Column Descriptions

### uuser

### pid

### tdb

### dur

### locks

### ip
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_queues_by_minute', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_queues_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### qname

### start_ts

### queue

### compile

### exec

### queries
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_users_by_second', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_users_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_query', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_query`

## Description

## Columns

  column             data type
  ------------------ -------------
  `qid`              `int4`
  `query_start_ts`   `timestamp`
  `aborts`           `int4`

## Column Descriptions

### qid

### query_start_ts

### aborts
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_statements_by_second', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_statements_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### statement_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('sf_241129', 'steps_totals_data_on_both_for_all_by_minute', 'view', '# `sf_241129.steps_totals_data_on_both_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_users_by_all', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_users_by_all`

## Description

## Columns

  column     data type
  ---------- -----------
  `uuser`    `varchar`
  `aborts`   `int4`

## Column Descriptions

### uuser

### aborts
'),
  ('lf_241129', 'steps_totals_data_on_both_for_slices_by_all', 'view', '# `lf_241129.steps_totals_data_on_both_for_slices_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `slice_ordinal`                `int2`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### slice_ordinal

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('base_241129', 'privileges_public', 'view', '# `base_241129.privileges_public`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `acl_privileges`                         `varchar`

## Column Descriptions

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### acl_privileges
'),
  ('sf_241129', 'analyze_totals_for_database_by_second', 'view', '# `sf_241129.analyze_totals_for_database_by_second`

## Description

## Columns

  column          data type
  --------------- -------------
  `dbname`        `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### dbname

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'ttables', 'view', '# `lf_241129.ttables`

## Description

This view provides information about normal (non-external) Redshift
tables, both on the leader node and on the worker nodes, one row per
table.

(I did think about separate views for leader node tables and worker node
tables, but I know of no canonical indication of whether a table is
leader or worker, and of no way to tell the difference between a leader
node table and a brand new worker node table.)

Note that the column count does *not* include the three system columns.

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `table_creation`                 `timestamp`
  `table_age`                      `interval`
  `table_number_columns`           `int2`
  `table_sort_type`                `varchar`
  `table_number_sortkeys`          `int2`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`
  `table_stats`                    `varchar`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### user_id

### user_name

### table_creation

### table_age

### table_number_columns

### table_sort_type

### table_number_sortkeys

### table_distribution_type

### table_distribution_auto_flag

### table_stats
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_users_by_day', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_users_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `user_id`                      `int4`
  `user_name`                    `varchar`
  `start_ts`                     `timestamp`
  `min_leadin_duration`          `interval`
  `max_leadin_duration`          `interval`
  `med_leadin_duration`          `float4`
  `avg_leadin_duration`          `interval`
  `std_leadin_duration`          `float4`
  `sum_leadin_duration`          `interval`
  `min_wlm_leadin_duration`      `interval`
  `max_wlm_leadin_duration`      `interval`
  `med_wlm_leadin_duration`      `float4`
  `avg_wlm_leadin_duration`      `interval`
  `std_wlm_leadin_duration`      `float4`
  `sum_wlm_leadin_duration`      `interval`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `min_wlm_leadout_duration`     `interval`
  `max_wlm_leadout_duration`     `interval`
  `med_wlm_leadout_duration`     `float4`
  `avg_wlm_leadout_duration`     `interval`
  `std_wlm_leadout_duration`     `float4`
  `sum_wlm_leadout_duration`     `interval`
  `min_leadout_duration`         `interval`
  `max_leadout_duration`         `interval`
  `med_leadout_duration`         `float4`
  `avg_leadout_duration`         `interval`
  `std_leadout_duration`         `float4`
  `sum_leadout_duration`         `interval`
  `min_query_total_duration`     `interval`
  `max_query_total_duration`     `interval`
  `med_query_total_duration`     `float4`
  `avg_query_total_duration`     `interval`
  `std_query_total_duration`     `float4`
  `sum_query_total_duration`     `interval`
  `min_unaccounted_duration`     `interval`
  `max_unaccounted_duration`     `interval`
  `med_unaccounted_duration`     `float4`
  `avg_unaccounted_duration`     `interval`
  `std_unaccounted_duration`     `float4`
  `sum_unaccounted_duration`     `interval`
  `number_queries`               `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_leadin_duration

### max_leadin_duration

### med_leadin_duration

### avg_leadin_duration

### std_leadin_duration

### sum_leadin_duration

### min_wlm_leadin_duration

### max_wlm_leadin_duration

### med_wlm_leadin_duration

### avg_wlm_leadin_duration

### std_wlm_leadin_duration

### sum_wlm_leadin_duration

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### min_wlm_leadout_duration

### max_wlm_leadout_duration

### med_wlm_leadout_duration

### avg_wlm_leadout_duration

### std_wlm_leadout_duration

### sum_wlm_leadout_duration

### min_leadout_duration

### max_leadout_duration

### med_leadout_duration

### avg_leadout_duration

### std_leadout_duration

### sum_leadout_duration

### min_query_total_duration

### max_query_total_duration

### med_query_total_duration

### avg_query_total_duration

### std_query_total_duration

### sum_query_total_duration

### min_unaccounted_duration

### max_unaccounted_duration

### med_unaccounted_duration

### avg_unaccounted_duration

### std_unaccounted_duration

### sum_unaccounted_duration

### number_queries
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_query', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_query`

## Description

## Columns

  column      data type
  ----------- -----------
  `qid`       `int4`
  `ddelete`   `varchar`
  `iinsert`   `varchar`
  `scan`      `varchar`

## Column Descriptions

### qid

### ddelete

### iinsert

### scan
'),
  ('ro_241129', 'sp_grant_versionless_sf_schema_to_public', 'procedure', '# `ro_241129.sp_grant_versionless_sf_schema_to_public`

## Description

## Arguments

No arguments.
'),
  ('lf_241129', 'queries_totals_result_cache_for_all_by_minute', 'view', '# `lf_241129.queries_totals_result_cache_for_all_by_minute`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_users_by_minute', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_users_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('lf_241129', 'views_late_binding_columns', 'view', '# `lf_241129.views_late_binding_columns`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `schema_id`                    `int4`
  `schema_name`                  `varchar`
  `view_id`                      `int4`
  `view_name`                    `varchar`
  `column_ordinal`               `int2`
  `column_name`                  `varchar`
  `column_data_type_id`          `int4`
  `column_base_data_type_name`   `varchar`
  `column_octet_length`          `int4`
  `column_precision`             `int2`
  `column_scale`                 `int2`
  `column_full_data_type_name`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### column_ordinal

### column_name

### column_data_type_id

### column_base_data_type_name

### column_octet_length

### column_precision

### column_scale

### column_full_data_type_name
'),
  ('sf_241129', 'steps_totals_data_on_both_for_statements_by_second', 'view', '# `sf_241129.steps_totals_data_on_both_for_statements_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### stmt

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_users_by_hour', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_users_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_minute_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_minute_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `uuser`          `varchar`
  `node`           `varchar`
  `start_ts`       `timestamp`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('sf_241129', 'views_materialized_table_blocks_rows', 'view', '# `sf_241129.views_materialized_table_blocks_rows`

## Description

## Columns

  column      data type
  ----------- ------------
  `sschema`   `varchar`
  `vview`     `varchar`
  `uuser`     `varchar`
  `status`    `varchar`
  `ttable`    `varchar`
  `age`       `interval`
  `cols`      `int2`
  `ssort`     `varchar`
  `skeys`     `int2`
  `dist`      `varchar`
  `slices`    `int2`
  `sblks`     `int8`
  `ublks`     `int8`
  `tblks`     `int8`
  `srows`     `int8`
  `urows`     `int8`
  `trows`     `int8`

## Column Descriptions

### sschema

### vview

### uuser

### status

### ttable

### age

### cols

### ssort

### skeys

### dist

### slices

### sblks

### ublks

### tblks

### srows

### urows

### trows
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_users_by_day', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_users_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_queues_by_hour', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_queues_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### qname

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_queues_by_all', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_queues_by_all`

## Description

## Columns

  column                data type
  --------------------- -----------
  `queue_id`            `int2`
  `queue_name`          `varchar`
  `node_type`           `varchar`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### queue_id

### queue_name

### node_type

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_day_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_day_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `uuser`          `varchar`
  `node`           `varchar`
  `start_ts`       `date`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('sf_241129', 'wqueues_totals_slot_utilization_by_day_as_table', 'view', '# `sf_241129.wqueues_totals_slot_utilization_by_day_as_table`

## Description

## Columns

  column      data type
  ----------- -----------
  `qname`     `varchar`
  `slot`      `int2`
  `today`     `numeric`
  `today-1`   `numeric`
  `today-2`   `numeric`
  `today-3`   `numeric`
  `today-4`   `numeric`
  `today-5`   `numeric`

## Column Descriptions

### qname

### slot

### today

### today-1

### today-2

### today-3

### today-4

### today-5
'),
  ('lf_241129', 'transactions_stats_commits_by_day', 'view', '# `lf_241129.transactions_stats_commits_by_day`

## Description

## Columns

  column                                data type
  ------------------------------------- -------------
  `start_ts`                            `timestamp`
  `min_transaction_batch_queue_start`   `interval`
  `max_transaction_batch_queue_start`   `interval`
  `med_transaction_batch_queue_start`   `float4`
  `avg_transaction_batch_queue_start`   `interval`
  `std_transaction_batch_queue_start`   `float4`
  `sum_transaction_batch_queue_start`   `interval`
  `min_batch_queue_duration`            `interval`
  `max_batch_queue_duration`            `interval`
  `med_batch_queue_duration`            `float4`
  `avg_batch_queue_duration`            `interval`
  `std_batch_queue_duration`            `float4`
  `sum_batch_queue_duration`            `interval`
  `min_batch_leader_duration`           `interval`
  `max_batch_leader_duration`           `interval`
  `med_batch_leader_duration`           `float4`
  `avg_batch_leader_duration`           `interval`
  `std_batch_leader_duration`           `float4`
  `sum_batch_leader_duration`           `interval`
  `min_batch_worker_duration`           `interval`
  `max_batch_worker_duration`           `interval`
  `med_batch_worker_duration`           `float4`
  `avg_batch_worker_duration`           `interval`
  `std_batch_worker_duration`           `float4`
  `sum_batch_worker_duration`           `interval`
  `min_number_batch_transactions`       `int8`
  `max_number_batch_transactions`       `int8`
  `med_number_batch_transactions`       `int8`
  `avg_number_batch_transactions`       `float4`
  `std_number_batch_transactions`       `float4`
  `sum_number_batch_transactions`       `int8`
  `min_number_batch_blocks`             `int8`
  `max_number_batch_blocks`             `int8`
  `med_number_batch_blocks`             `int8`
  `avg_number_batch_blocks`             `float4`
  `std_number_batch_blocks`             `float4`
  `sum_number_batch_blocks`             `int8`
  `number_batches`                      `int8`

## Column Descriptions

### start_ts

### min_transaction_batch_queue_start

### max_transaction_batch_queue_start

### med_transaction_batch_queue_start

### avg_transaction_batch_queue_start

### std_transaction_batch_queue_start

### sum_transaction_batch_queue_start

### min_batch_queue_duration

### max_batch_queue_duration

### med_batch_queue_duration

### avg_batch_queue_duration

### std_batch_queue_duration

### sum_batch_queue_duration

### min_batch_leader_duration

### max_batch_leader_duration

### med_batch_leader_duration

### avg_batch_leader_duration

### std_batch_leader_duration

### sum_batch_leader_duration

### min_batch_worker_duration

### max_batch_worker_duration

### med_batch_worker_duration

### avg_batch_worker_duration

### std_batch_worker_duration

### sum_batch_worker_duration

### min_number_batch_transactions

### max_number_batch_transactions

### med_number_batch_transactions

### avg_number_batch_transactions

### std_number_batch_transactions

### sum_number_batch_transactions

### min_number_batch_blocks

### max_number_batch_blocks

### med_number_batch_blocks

### avg_number_batch_blocks

### std_number_batch_blocks

### sum_number_batch_blocks

### number_batches
'),
  ('sf_241129', 'vacuum_stats_for_all_by_second', 'view', '# `sf_241129.vacuum_stats_for_all_by_second`

## Description

## Columns

  column              data type
  ------------------- -------------
  `start_ts`          `timestamp`
  `new blocks`        `varchar`
  `replaced blocks`   `varchar`
  `new rows`          `varchar`
  `deleted rows`      `varchar`
  `xid duration`      `varchar`
  `qids`              `int8`
  `xids`              `int8`

## Column Descriptions

### start_ts

### new blocks

### replaced blocks

### new rows

### deleted rows

### xid duration

### qids

### xids
'),
  ('lf_241129', 'privileges_granted_to_public', 'view', '# `lf_241129.privileges_granted_to_public`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `priv_database_create`                   `bool`
  `priv_database_temp`                     `bool`
  `priv_function_execute`                  `bool`
  `priv_language_usage`                    `bool`
  `priv_procedure_execute`                 `bool`
  `priv_schema_create`                     `bool`
  `priv_schema_usage`                      `bool`
  `priv_table_delete`                      `bool`
  `priv_table_drop`                        `bool`
  `priv_table_insert`                      `bool`
  `priv_table_reference`                   `bool`
  `priv_table_select`                      `bool`
  `priv_table_update`                      `bool`
  `priv_view_drop`                         `bool`
  `priv_view_select`                       `bool`

## Column Descriptions

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### priv_database_create

### priv_database_temp

### priv_function_execute

### priv_language_usage

### priv_procedure_execute

### priv_schema_create

### priv_schema_usage

### priv_table_delete

### priv_table_drop

### priv_table_insert

### priv_table_reference

### priv_table_select

### priv_table_update

### priv_view_drop

### priv_view_select
'),
  ('base_241129', 'queries_blocks', 'view', '# `base_241129.queries_blocks`

## Description

## Columns

  column                          data type
  ------------------------------- -----------
  `query_id`                      `int4`
  `query_number_blocks_read`      `int8`
  `query_number_blocks_to_disk`   `int8`

## Column Descriptions

### query_id

### query_number_blocks_read

### query_number_blocks_to_disk
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_all_by_second', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('lf_241129', 'views_materialized_columns', 'view', '# `lf_241129.views_materialized_columns`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `schema_id`                    `int4`
  `schema_name`                  `varchar`
  `view_id`                      `int4`
  `view_name`                    `varchar`
  `column_ordinal`               `int2`
  `column_name`                  `varchar`
  `column_data_type_id`          `int4`
  `column_base_data_type_name`   `varchar`
  `column_octet_length`          `int4`
  `column_precision`             `int2`
  `column_scale`                 `int2`
  `column_full_data_type_name`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### column_ordinal

### column_name

### column_data_type_id

### column_base_data_type_name

### column_octet_length

### column_precision

### column_scale

### column_full_data_type_name
'),
  ('sf_241129', 'sschemas_blocks_rows_activity', 'view', '# `sf_241129.sschemas_blocks_rows_activity`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `uuser`     `varchar`
  `ttables`   `int4`
  `nviews`    `int4`
  `lviews`    `int4`
  `mviews`    `int4`
  `funcs`     `int4`
  `procs`     `int4`
  `sblks`     `int8`
  `dsblks`    `int8`
  `ublks`     `int8`
  `tblks`     `int8`
  `srows`     `int8`
  `dsrows`    `int8`
  `urows`     `int8`
  `trows`     `int8`
  `drops`     `int8`
  `irows`     `int8`

## Column Descriptions

### sschema

### uuser

### ttables

### nviews

### lviews

### mviews

### funcs

### procs

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows

### drops

### irows
'),
  ('base_241129', 'generate_hours', 'view', '# `base_241129.generate_hours`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`

## Column Descriptions

### start_ts
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_queues_by_day', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_queues_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `queue_id`                     `int2`
  `queue_name`                   `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('lf_241129', 'queries_totals_result_cache_for_all_by_day', 'view', '# `lf_241129.queries_totals_result_cache_for_all_by_day`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_statements_by_day', 'view', '# `sf_241129.steps_stats_durations_on_both_for_statements_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `stmt`       `varchar`
  `node`       `varchar`
  `step`       `varchar`
  `start_ts`   `date`
  `min`        `numeric`
  `max`        `numeric`
  `med`        `numeric`
  `avg`        `numeric`
  `std`        `numeric`
  `num`        `int8`

## Column Descriptions

### stmt

### node

### step

### start_ts

### min

### max

### med

### avg

### std

### num
'),
  ('lf_241129', 'views_text', 'view', '# `lf_241129.views_text`

## Description

## Columns

  column          data type
  --------------- -----------
  `schema_id`     `int4`
  `schema_name`   `varchar`
  `view_id`       `int4`
  `view_name`     `varchar`
  `view_text`     `varchar`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### view_text
'),
  ('ro_241129', 'sp_revoke_version_from_role', 'procedure', '# `ro_241129.sp_revoke_version_from_role`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `version`     `varchar( 6 )`
  `role_name`   `varchar( 127 )`

## Argument Descriptions

### version

### role_name
'),
  ('lf_241129', 'steps_totals_data_on_both_for_statements_by_minute', 'view', '# `lf_241129.steps_totals_data_on_both_for_statements_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### statement_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'views_materialized_columns', 'view', '# `sf_241129.views_materialized_columns`

## Description

## Columns

  column        data type
  ------------- -----------
  `sschema`     `varchar`
  `vview`       `varchar`
  `ccolumn`     `varchar`
  `data_type`   `varchar`

## Column Descriptions

### sschema

### vview

### ccolumn

### data_type
'),
  ('lf_241129', 'vacuum_totals_for_tables_by_second', 'view', '# `lf_241129.vacuum_totals_for_tables_by_second`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'vacuum_totals_for_all_by_hour', 'view', '# `sf_241129.vacuum_totals_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### start_ts

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'queries_totals_result_cache_for_all_by_all', 'view', '# `sf_241129.queries_totals_result_cache_for_all_by_all`

## Description

## Columns

  column    data type
  --------- -----------
  `hit`     `int8`
  `miss`    `int8`
  `total`   `int8`
  `%`       `numeric`

## Column Descriptions

### hit

### miss

### total

### %
'),
  ('sf_241129', 'vacuum_totals_for_users_by_all', 'view', '# `sf_241129.vacuum_totals_for_users_by_all`

## Description

## Columns

  column     data type
  ---------- -----------
  `uuser`    `varchar`
  `manual`   `varchar`
  `auto`     `varchar`
  `total`    `int4`
  `dur`      `numeric`

## Column Descriptions

### uuser

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'steps_totals_data_on_both_for_users_by_all', 'view', '# `sf_241129.steps_totals_data_on_both_for_users_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `node`      `varchar`
  `aggr`      `varchar`
  `bcast`     `varchar`
  `ddelete`   `varchar`
  `dist`      `varchar`
  `hash`      `varchar`
  `hjoin`     `varchar`
  `iinsert`   `varchar`
  `llimit`    `varchar`
  `mmerge`    `varchar`
  `mjoin`     `varchar`
  `nloop`     `varchar`
  `rreturn`   `varchar`
  `save`      `varchar`
  `ssort`     `varchar`
  `uunique`   `varchar`
  `wwindow`   `varchar`

## Column Descriptions

### uuser

### node

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_tables_by_hour', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_tables_by_hour`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `table_id`            `int4`
  `table_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_totals_data_on_both_for_node_by_hour', 'view', '# `sf_241129.steps_totals_data_on_both_for_node_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `int2`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'vacuum_totals_for_all_by_day', 'view', '# `sf_241129.vacuum_totals_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### start_ts

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_all_by_day', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `aborts`     `int4`

## Column Descriptions

### start_ts

### aborts
'),
  ('sf_241129', 'queries_text_on_workers_fast', 'view', '# `sf_241129.queries_text_on_workers_fast`

## Description

## Columns

  column       data type
  ------------ -------------
  `pid`        `int4`
  `xid`        `int8`
  `qid`        `int4`
  `start_ts`   `timestamp`
  `ttext`      `varchar`

## Column Descriptions

### pid

### xid

### qid

### start_ts

### ttext
'),
  ('base_241129', 'fixed_pg_class_for_views', 'view', '# `base_241129.fixed_pg_class_for_views`

## Description

## Columns

  column        data type
  ------------- -----------
  `view_id`     `int4`
  `view_name`   `varchar`
  `schema_id`   `int4`
  `user_id`     `int4`
  `relnatts`    `int2`

## Column Descriptions

### view_id

### view_name

### schema_id

### user_id

### relnatts
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_users_by_second', 'view', '# `lf_241129.steps_stats_durations_on_both_for_users_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'vacuum_stats_for_all_by_day', 'view', '# `lf_241129.vacuum_stats_for_all_by_day`

## Description

## Columns

  column                             data type
  ---------------------------------- -------------
  `start_ts`                         `timestamp`
  `min_sum_number_new_blocks`        `int8`
  `max_sum_number_new_blocks`        `int8`
  `med_sum_number_new_blocks`        `int8`
  `avg_sum_number_new_blocks`        `float4`
  `std_sum_number_new_blocks`        `float4`
  `sum_sum_number_new_blocks`        `float4`
  `min_sum_number_replaced_blocks`   `int8`
  `max_sum_number_replaced_blocks`   `int8`
  `med_sum_number_replaced_blocks`   `int8`
  `avg_sum_number_replaced_blocks`   `float4`
  `std_sum_number_replaced_blocks`   `float4`
  `sum_sum_number_replaced_blocks`   `float4`
  `min_sum_number_new_rows`          `int8`
  `max_sum_number_new_rows`          `int8`
  `med_sum_number_new_rows`          `int8`
  `avg_sum_number_new_rows`          `float4`
  `std_sum_number_new_rows`          `float4`
  `sum_sum_number_new_rows`          `float4`
  `min_sum_number_deleted_rows`      `int8`
  `max_sum_number_deleted_rows`      `int8`
  `med_sum_number_deleted_rows`      `int8`
  `avg_sum_number_deleted_rows`      `float4`
  `std_sum_number_deleted_rows`      `float4`
  `sum_sum_number_deleted_rows`      `float4`
  `min_duration`                     `interval`
  `max_duration`                     `interval`
  `med_duration`                     `float4`
  `avg_duration`                     `interval`
  `std_duration`                     `float4`
  `sum_duration`                     `interval`
  `number_qids`                      `int8`
  `number_xids`                      `int8`

## Column Descriptions

### start_ts

### min_sum_number_new_blocks

### max_sum_number_new_blocks

### med_sum_number_new_blocks

### avg_sum_number_new_blocks

### std_sum_number_new_blocks

### sum_sum_number_new_blocks

### min_sum_number_replaced_blocks

### max_sum_number_replaced_blocks

### med_sum_number_replaced_blocks

### avg_sum_number_replaced_blocks

### std_sum_number_replaced_blocks

### sum_sum_number_replaced_blocks

### min_sum_number_new_rows

### max_sum_number_new_rows

### med_sum_number_new_rows

### avg_sum_number_new_rows

### std_sum_number_new_rows

### sum_sum_number_new_rows

### min_sum_number_deleted_rows

### max_sum_number_deleted_rows

### med_sum_number_deleted_rows

### avg_sum_number_deleted_rows

### std_sum_number_deleted_rows

### sum_sum_number_deleted_rows

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_qids

### number_xids
'),
  ('lf_241129', 'steps_totals_data_on_both_for_all_by_minute', 'view', '# `lf_241129.steps_totals_data_on_both_for_all_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_users_by_all', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_users_by_all`

## Description

## Columns

  column                data type
  --------------------- -----------
  `user_id`             `int4`
  `node_type`           `varchar`
  `user_name`           `varchar`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### user_id

### node_type

### user_name

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'ttables_columns', 'view', '# `sf_241129.ttables_columns`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `ccolumn`   `varchar`
  `ttype`     `varchar`
  `enc`       `varchar`
  `sk`        `int2`
  `dk`        `varchar`
  `nn`        `varchar`
  `pk`        `varchar`
  `u`         `varchar`

## Column Descriptions

### sschema

### ttable

### ccolumn

### ttype

### enc

### sk

### dk

### nn

### pk

### u
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_users_by_hour', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_users_by_hour`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_tables_by_day', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_tables_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'analyze_totals_for_all_by_all', 'view', '# `sf_241129.analyze_totals_for_all_by_all`

## Description

## Columns

  column          data type
  --------------- -----------
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'uusers_default_privileges', 'view', '# `lf_241129.uusers_default_privileges`

## Description

## Columns

  column                           data type
  -------------------------------- -----------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `object_type`                    `varchar`
  `priv_recipient_type`            `varchar`
  `priv_recipient_id`              `int4`
  `priv_recipient_name`            `varchar`
  `priv_function_execute`          `bool`
  `priv_function_grant_execute`    `bool`
  `priv_procedure_execute`         `bool`
  `priv_procedure_grant_execute`   `bool`
  `priv_table_delete`              `bool`
  `priv_table_grant_delete`        `bool`
  `priv_table_drop`                `bool`
  `priv_table_grant_drop`          `bool`
  `priv_table_insert`              `bool`
  `priv_table_grant_insert`        `bool`
  `priv_table_reference`           `bool`
  `priv_table_grant_reference`     `bool`
  `priv_table_select`              `bool`
  `priv_table_grant_select`        `bool`
  `priv_table_update`              `bool`
  `priv_table_grant_update`        `bool`

## Column Descriptions

### user_id

### user_name

### schema_id

### schema_name

### object_type

### priv_recipient_type

### priv_recipient_id

### priv_recipient_name

### priv_function_execute

### priv_function_grant_execute

### priv_procedure_execute

### priv_procedure_grant_execute

### priv_table_delete

### priv_table_grant_delete

### priv_table_drop

### priv_table_grant_drop

### priv_table_insert

### priv_table_grant_insert

### priv_table_reference

### priv_table_grant_reference

### priv_table_select

### priv_table_grant_select

### priv_table_update

### priv_table_grant_update
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_all_by_minute', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_all_by_minute`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('base_241129', 'hr_number', 'function', '# `base_241129.hr_number`

## Description

## Returns

`varchar`

## Arguments

  argument   data type
  ---------- -----------
  `number`   `int8`

## Argument Descriptions

### number
'),
  ('base_241129', 'generate_seconds', 'view', '# `base_241129.generate_seconds`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`

## Column Descriptions

### start_ts
'),
  ('sf_241129', 'vacuum_totals_for_users_by_minute', 'view', '# `sf_241129.vacuum_totals_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### uuser

### start_ts

### manual

### auto

### total

### dur
'),
  ('lf_241129', 'vacuum_totals_for_users_by_day', 'view', '# `lf_241129.vacuum_totals_for_users_by_day`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_queues_by_minute', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_queues_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### qname

### start_ts

### aborts
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_users_by_all', 'view', '# `lf_241129.steps_stats_durations_on_both_for_users_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- ------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### user_id

### user_name

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'sschemas_blocks_rows', 'view', '# `sf_241129.sschemas_blocks_rows`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `uuser`     `varchar`
  `ttables`   `int4`
  `nviews`    `int4`
  `lviews`    `int4`
  `mviews`    `int4`
  `funcs`     `int4`
  `procs`     `int4`
  `sblks`     `int8`
  `dsblks`    `int8`
  `ublks`     `int8`
  `tblks`     `int8`
  `srows`     `int8`
  `dsrows`    `int8`
  `urows`     `int8`
  `trows`     `int8`

## Column Descriptions

### sschema

### uuser

### ttables

### nviews

### lviews

### mviews

### funcs

### procs

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_slices_by_all', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_slices_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `n`         `int2`
  `s`         `int2`
  `t`         `varchar`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### n

### s

### t

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_all_by_minute', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_all_by_minute`

## Description

## Columns

  column             data type
  ------------------ -------------
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_statements_by_hour', 'view', '# `sf_241129.steps_stats_durations_on_both_for_statements_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `node`       `varchar`
  `step`       `varchar`
  `start_ts`   `timestamp`
  `min`        `numeric`
  `max`        `numeric`
  `med`        `numeric`
  `avg`        `numeric`
  `std`        `numeric`
  `num`        `int8`

## Column Descriptions

### stmt

### node

### step

### start_ts

### min

### max

### med

### avg

### std

### num
'),
  ('sf_241129', 'queries_step_plan', 'view', '# `sf_241129.queries_step_plan`

## Description

## Columns

  column       data type
  ------------ -------------
  `qid`        `int4`
  `strm`       `int2`
  `seg`        `int2`
  `stp`        `int2`
  `node`       `int2`
  `slice`      `int2`
  `t`          `varchar`
  `ttype`      `varchar`
  `mrows`      `int8`
  `erows`      `int8`
  `rrows`      `int8`
  `bytes`      `int8`
  `pkts`       `int8`
  `st`         `varchar`
  `start_ts`   `timestamp`
  `end_ts`     `timestamp`
  `dur`        `numeric`
  `notes`      `varchar`

## Column Descriptions

### qid

### strm

### seg

### stp

### node

### slice

### t

### ttype

### mrows

### erows

### rrows

### bytes

### pkts

### st

### start_ts

### end_ts

### dur

### notes
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_users_by_hour', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### uuser

### start_ts

### aborts
'),
  ('lf_241129', 'nodes', 'view', '# `lf_241129.nodes`

## Description

## Columns

  column                       data type
  ---------------------------- -----------
  `node_ordinal`               `int2`
  `number_partial_slices`      `int2`
  `number_full_slices`         `int2`
  `number_total_slices`        `int2`
  `number_local_disks`         `int2`
  `total_disk_blocks`          `int8`
  `number_used_local_blocks`   `int8`
  `number_used_rms_blocks`     `int8`
  `number_total_blocks_used`   `int8`

## Column Descriptions

### node_ordinal

### number_partial_slices

### number_full_slices

### number_total_slices

### number_local_disks

### total_disk_blocks

### number_used_local_blocks

### number_used_rms_blocks

### number_total_blocks_used
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_schemas_by_minute', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_schemas_by_minute`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('lf_241129', 'analyze_totals_for_tables_by_second', 'view', '# `lf_241129.analyze_totals_for_tables_by_second`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `schema_id`                                     `int4`
  `schema_name`                                   `varchar`
  `table_id`                                      `int4`
  `table_name`                                    `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_all_by_hour', 'view', '# `lf_241129.steps_stats_durations_on_both_for_all_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_users_by_all', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_users_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### user_id

### user_name

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('sf_241129', 'views_late_binding_columns', 'view', '# `sf_241129.views_late_binding_columns`

## Description

## Columns

  column        data type
  ------------- -----------
  `sschema`     `varchar`
  `vview`       `varchar`
  `ccolumn`     `varchar`
  `data_type`   `varchar`

## Column Descriptions

### sschema

### vview

### ccolumn

### data_type
'),
  ('sf_241129', 'queries_totals_result_cache_for_all_by_day', 'view', '# `sf_241129.queries_totals_result_cache_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### start_ts

### hit

### miss

### total

### %
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_slices_by_day', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_slices_by_day`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `slice_ordinal`       `int2`
  `slice_type`          `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'ttables_creation_for_schemas_by_hour', 'view', '# `sf_241129.ttables_creation_for_schemas_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `start_ts`   `timestamp`
  `ttables`    `int4`

## Column Descriptions

### sschema

### start_ts

### ttables
'),
  ('sf_241129', 'ttables_stats_dists_sorts_for_schemas_by_blocks', 'view', '# `sf_241129.ttables_stats_dists_sorts_for_schemas_by_blocks`

## Description

## Columns

  column            data type
  ----------------- -----------
  `sschema`         `varchar`
  `compound_all`    `varchar`
  `compound_even`   `varchar`
  `compound_key`    `varchar`
  `intlvd_all`      `varchar`
  `intlvd_even`     `varchar`
  `intlvd_key`      `varchar`
  `unsorted_all`    `varchar`
  `unsorted_even`   `varchar`
  `unsorted_key`    `varchar`

## Column Descriptions

### sschema

### compound_all

### compound_even

### compound_key

### intlvd_all

### intlvd_even

### intlvd_key

### unsorted_all

### unsorted_even

### unsorted_key
'),
  ('sf_241129', 'analyze_totals_for_all_by_minute', 'view', '# `sf_241129.analyze_totals_for_all_by_minute`

## Description

## Columns

  column          data type
  --------------- -------------
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_queues_by_day', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_queues_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `qname`      `varchar`
  `start_ts`   `date`
  `aborts`     `int4`

## Column Descriptions

### qname

### start_ts

### aborts
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_slices_by_second', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_slices_by_second`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `slice_ordinal`       `int2`
  `slice_type`          `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_tables_by_all', 'view', '# `lf_241129.steps_stats_durations_on_both_for_tables_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- ------------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `table_id`                    `int4`
  `table_name`                  `varchar`
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_slices_by_second', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_slices_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### s

### t

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_users_by_day', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `start_ts`   `date`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### uuser

### start_ts

### rread

### to_disk
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_users_by_hour', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_users_by_hour`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_abort
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_day_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_day_complete`

## Description

## Columns

  column          data type
  --------------- -----------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `node`          `varchar`
  `start_ts`      `date`
  `aggr`          `varchar`
  `ddelete`       `varchar`
  `hashjoin`      `varchar`
  `iinsert`       `varchar`
  `mergejoin`     `varchar`
  `nestloop_op`   `varchar`
  `save`          `varchar`
  `ssort`         `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### ddelete

### hashjoin

### iinsert

### mergejoin

### nestloop_op

### save

### ssort
'),
  ('base_241129', 'generate_minutes', 'view', '# `base_241129.generate_minutes`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`

## Column Descriptions

### start_ts
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_all_by_hour', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_all_by_hour`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('sf_241129', 'ttables_stats_dists_sorts_for_all_by_count', 'view', '# `sf_241129.ttables_stats_dists_sorts_for_all_by_count`

## Description

## Columns

  column   data type
  -------- -----------
  `aall`   `varchar`
  `even`   `varchar`
  `kkey`   `varchar`

## Column Descriptions

### aall

### even

### kkey
'),
  ('lf_241129', 'steps_totals_data_on_both_for_users_by_minute', 'view', '# `lf_241129.steps_totals_data_on_both_for_users_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `usesysid`                     `int4`
  `user_name`                    `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### usesysid

### user_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_queues_by_all', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_queues_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `varchar`
  `qname`     `varchar`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### node

### qname

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_all_by_minute', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_all_by_minute`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_all_by_hour', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_all_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('base_241129', 'fixed_pg_attribute_for_tables', 'view', '# `base_241129.fixed_pg_attribute_for_tables`

## Description

## Columns

  column                  data type
  ----------------------- -----------
  `table_id`              `int4`
  `column_name`           `varchar`
  `column_ordinal`        `int2`
  `atttypmod`             `int4`
  `column_notnull_flag`   `bool`

## Column Descriptions

### table_id

### column_name

### column_ordinal

### atttypmod

### column_notnull_flag
'),
  ('sf_241129', 'ttables', 'view', '# `sf_241129.ttables`

## Description

This view provides information about normal (non-external) Redshift
tables, both on the leader node and on the worker nodes, one row per
table.

(I did think about separate views for leader node tables and worker node
tables, but I know of no canonical indication of whether a table is
leader or worker, and of no way to tell the difference between a leader
node table and a brand new worker node table.)

Note that the column count does *not* include the three system columns.

## Columns

  column      data type
  ----------- ------------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `uuser`     `varchar`
  `age`       `interval`
  `cols`      `int2`
  `ssort`     `varchar`
  `skeys`     `int2`
  `dist`      `varchar`
  `stats`     `varchar`

## Column Descriptions

### sschema

### ttable

### uuser

### age

### cols

### ssort

### skeys

### dist

### stats
'),
  ('sf_241129', 'transactions_totals_commits_by_day', 'view', '# `sf_241129.transactions_totals_commits_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `leader`     `numeric`
  `worker`     `numeric`
  `idle`       `numeric`
  `exec`       `numeric`

## Column Descriptions

### start_ts

### leader

### worker

### idle

### exec
'),
  ('lf_241129', 'analyze_totals_for_tables_by_day', 'view', '# `lf_241129.analyze_totals_for_tables_by_day`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `schema_id`                                     `int4`
  `schema_name`                                   `varchar`
  `table_id`                                      `int4`
  `table_name`                                    `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('base_241129', 'fixed_stl_wlm_query', 'view', '# `base_241129.fixed_stl_wlm_query`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `query_compile_exec_end_ts`     `timestamp`
  `query_compile_exec_start_ts`   `timestamp`
  `final_state`                   `bpchar`
  `query_id`                      `int4`
  `query_queue_end_ts`            `timestamp`
  `query_queue_start_ts`          `timestamp`
  `queue_id`                      `int2`
  `query_wlm_end_ts`              `timestamp`
  `query_wlm_start_ts`            `timestamp`
  `slot_count`                    `int2`
  `task_id`                       `int4`
  `user_id`                       `int4`
  `transaction_id`                `int8`
  `query_queue_duration`          `interval`
  `query_compile_exec_duration`   `interval`
  `query_wlm_duration`            `interval`

## Column Descriptions

### query_compile_exec_end_ts

### query_compile_exec_start_ts

### final_state

### query_id

### query_queue_end_ts

### query_queue_start_ts

### queue_id

### query_wlm_end_ts

### query_wlm_start_ts

### slot_count

### task_id

### user_id

### transaction_id

### query_queue_duration

### query_compile_exec_duration

### query_wlm_duration
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_users_by_second', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_users_by_second`

## Description

## Columns

  column          data type
  --------------- -------------
  `uuser`         `varchar`
  `start_ts`      `timestamp`
  `leadin`        `varchar`
  `wlm_leadin`    `varchar`
  `queue`         `varchar`
  `compile`       `varchar`
  `exec`          `varchar`
  `wlm_leadout`   `varchar`
  `leadout`       `varchar`
  `total`         `varchar`
  `unact`         `varchar`
  `queries`       `int4`

## Column Descriptions

### uuser

### start_ts

### leadin

### wlm_leadin

### queue

### compile

### exec

### wlm_leadout

### leadout

### total

### unact

### queries
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_hour_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_hour_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_users_by_all', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_users_by_all`

## Description

## Columns

  column                           data type
  -------------------------------- -----------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### user_id

### user_name

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_all_by_hour', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### start_ts

### rread

### to_disk
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_node_by_minute', 'view', '# `lf_241129.steps_stats_durations_on_both_for_node_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('base_241129', 'fixed_stl_scan', 'view', '# `base_241129.fixed_stl_scan`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `query_id`                    `int4`
  `slice_ordinal`               `int2`
  `segment_ordinal`             `int2`
  `step_ordinal`                `int2`
  `scan_start_ts`               `timestamp`
  `scan_end_ts`                 `timestamp`
  `tasknum`                     `int4`
  `rows`                        `int8`
  `bytes`                       `int8`
  `fetches`                     `int8`
  `type`                        `int4`
  `table_id`                    `int4`
  `is_rrscan`                   `bpchar`
  `is_delayed_scan`             `bpchar`
  `rows_pre_filter`             `int8`
  `rows_pre_user_filter`        `int8`
  `perm_table_name`             `bpchar`
  `is_rlf_scan`                 `bpchar`
  `is_rlf_scan_reason`          `int4`
  `num_em_blocks`               `int4`
  `checksum`                    `int8`
  `runtime_filtering`           `bpchar`
  `scan_region`                 `int4`
  `num_sortkey_as_predicate`    `int4`
  `row_fetcher_state`           `int8`
  `consumed_scan_ranges`        `int8`
  `work_stealing_reason`        `int8`
  `is_vectorized_scan`          `bpchar`
  `is_vectorized_scan_reason`   `int4`
  `row_fetcher_reason`          `int8`
  `topology_signature`          `int8`
  `use_tpm_partition`           `bpchar`
  `is_rrscan_expr`              `bpchar`
  `use_fwd_mtx_opt`             `bpchar`
  `scanned_mega_value`          `bpchar`

## Column Descriptions

### user_id

### query_id

### slice_ordinal

### segment_ordinal

### step_ordinal

### scan_start_ts

### scan_end_ts

### tasknum

### rows

### bytes

### fetches

### type

### table_id

### is_rrscan

### is_delayed_scan

### rows_pre_filter

### rows_pre_user_filter

### perm_table_name

### is_rlf_scan

### is_rlf_scan_reason

### num_em_blocks

### checksum

### runtime_filtering

### scan_region

### num_sortkey_as_predicate

### row_fetcher_state

### consumed_scan_ranges

### work_stealing_reason

### is_vectorized_scan

### is_vectorized_scan_reason

### row_fetcher_reason

### topology_signature

### use_tpm_partition

### is_rrscan_expr

### use_fwd_mtx_opt

### scanned_mega_value
'),
  ('sf_241129', 'utils_queries_benchmark', 'view', '# `sf_241129.utils_queries_benchmark`

## Description

This view provides an easy-to-use method to correctly and accurately
benchmark queries.

First, be aware that computing the execution time taken by a query is
*not* straightforward - I\'ll go into how it\'s done, but for now just
say - it is here done as correctly as is possible in Redshift (I say "as
possible", because in fact, right now, it is not possible to canonically
know the execution time of a query, due to flaws in the system tables).

Usage is as with all `sf` views, simple;

> `select * from sf.queries_benchmark;`

You then get five lines of output - for your five most recent queries.

     rn |    qid    |          start_ts          |  dur  |  min  |  max  |  avg  |  std  |                  text                   
    ----+-----------+----------------------------+-------+-------+-------+-------+-------+-----------------------------------------
      1 | 740502166 | 2024-10-29 20:58:08.085074 |  1.87 |  1.87 | 96.72 | 75.66 | 36.93 | select * from sf.vacuum_by_user_by_day;
      2 | 741840588 | 2024-10-30 07:53:59.909107 | 96.72 | 92.31 | 96.72 | 94.11 |  1.68 | select * from sf.queries_benchmark;
      3 | 741894382 | 2024-10-30 08:17:15.535093 | 93.05 | 92.31 | 94.36 | 93.24 |  0.85 | select * from sf.queries_benchmark;
      4 | 741909558 | 2024-10-30 08:23:32.480694 | 92.31 | 92.31 | 94.36 | 93.34 |  1.02 | select * from sf.queries_benchmark;
      5 | 741929643 | 2024-10-30 08:32:17.004436 | 94.36 | 94.36 | 94.36 | 94.36 |  0.00 | select * from sf.queries_benchmark;
    (5 rows)

So, what\'s this all about?

It\'s about statistics.

Each row is for one query, of the five most queries you have issued.

The `dur`, which is `duration`, column tells you how long each query
took to execute.

The other columns however are the stats for that query *and every query
before it*.

So you now have the most easy way to simply run a given query five times
in a row, and get the `min / man / avg / std` stats for all five.

Or maybe you just run three, and that\'s enough.

There\'s no median, because median turns out to be problematic to compute
in this situation.

Now I need to explain how query execution time is computed.

Query duration is hard to get right in Redshift. The docs do provide
anything like the information to get it right, and getting it right has
some complexity, and in fact in principle is currently impossible,
although in practise it will be normally be fine.

There are three basic problems.

1.  Redshift includes query compilation time in execution time.
2.  We want to avoid including the time it takes return rows from the
    query to the client.
3.  We want to avoid query queuing time.

So I handle these. The query execution time is from the moment the query
begins to run (rather than when it arrives at Redshift), compilation is
subtracted from execution time, and the end of the timed period is the
very beginning of when Redshift starts to return rows to the user.

here is one, and unfixable, problem, but I think it is rare.

When Redshift is compiling query A, if it sees query B, which is
compiling at the same time and which will produce the output query A
needs, it will stall query A compilation until query B finishes, and
then use the output from query B. The problem is there is absolutely no
way to know when this happens, and it looks like execution time.

## Columns

  column       data type
  ------------ -------------
  `rn`         `int2`
  `qid`        `int4`
  `start_ts`   `timestamp`
  `dur`        `numeric`
  `min`        `numeric`
  `max`        `numeric`
  `avg`        `numeric`
  `std`        `numeric`
  `ttext`      `varchar`

## Column Descriptions

### rn

### qid

### start_ts

### dur

### min

### max

### avg

### std

### ttext
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_users_by_hour', 'view', '# `lf_241129.steps_stats_durations_on_both_for_users_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `start_ts`                    `timestamp`
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'ttables_creation_for_all_by_day', 'view', '# `sf_241129.ttables_creation_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `ttables`    `int4`

## Column Descriptions

### start_ts

### ttables
'),
  ('sf_241129', 'batches', 'view', '# `sf_241129.batches`

## Description

## Columns

  column       data type
  ------------ -------------
  `bid`        `int8`
  `start_ts`   `timestamp`
  `end_ts`     `timestamp`
  `queue`      `numeric`
  `leader`     `numeric`
  `worker`     `numeric`
  `exec`       `numeric`
  `blks`       `int8`
  `hdrs`       `int8`
  `xids`       `int8`

## Column Descriptions

### bid

### start_ts

### end_ts

### queue

### leader

### worker

### exec

### blks

### hdrs

### xids
'),
  ('lf_241129', 'wqueues_memory', 'view', '# `lf_241129.wqueues_memory`

## Description

## Columns

  column                                   data type
  ---------------------------------------- ------------
  `queue_id`                               `int2`
  `queue_name`                             `varchar`
  `number_slots`                           `int2`
  `ra3_xlplus_memory_per_slot_per_slice`   `int4`
  `ra3_4x_memory_per_slot_per_slice`       `int4`
  `ra3_16x_memory_per_slot_per_slice`      `int4`
  `dc2_min_memory_per_slot_per_slice`      `int4`
  `dc2_max_memory_per_slot_per_slice`      `int4`
  `timeout`                                `interval`

## Column Descriptions

### queue_id

### queue_name

### number_slots

### ra3_xlplus_memory_per_slot_per_slice

### ra3_4x_memory_per_slot_per_slice

### ra3_16x_memory_per_slot_per_slice

### dc2_min_memory_per_slot_per_slice

### dc2_max_memory_per_slot_per_slice

### timeout
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_users_by_minute', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### uuser

### start_ts

### ddelete

### iinsert

### scan
'),
  ('base_241129', 'fixed_pg_proc', 'view', '# `base_241129.fixed_pg_proc`

## Description

## Columns

  column            data type
  ----------------- -------------
  `object_id`       `int4`
  `object_name`     `varchar`
  `schema_id`       `int4`
  `user_id`         `int4`
  `language_id`     `int4`
  `language_name`   `varchar`
  `provolatile`     `bpchar`
  `prorettype`      `oid`
  `proretset`       `bool`
  `prosrc`          `text`
  `proargtypes`     `oidvector`

## Column Descriptions

### object_id

### object_name

### schema_id

### user_id

### language_id

### language_name

### provolatile

### prorettype

### proretset

### prosrc

### proargtypes
'),
  ('sf_241129', 'steps_totals_data_on_both_for_users_by_day', 'view', '# `sf_241129.steps_totals_data_on_both_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'blocks_rows_totals_for_schema', 'view', '# `sf_241129.blocks_rows_totals_for_schema`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `sblks`     `varchar`
  `sdblks`    `varchar`
  `ublks`     `varchar`
  `tblks`     `varchar`
  `srows`     `varchar`
  `sdrops`    `varchar`
  `urows`     `varchar`
  `trows`     `varchar`

## Column Descriptions

### sschema

### sblks

### sdblks

### ublks

### tblks

### srows

### sdrops

### urows

### trows
'),
  ('lf_241129', 'ttables_blocks_rows_activity', 'view', '# `lf_241129.ttables_blocks_rows_activity`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `table_creation`                 `timestamp`
  `table_age`                      `interval`
  `table_number_columns`           `int2`
  `table_sort_type`                `varchar`
  `table_number_sortkeys`          `int2`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`
  `number_slices`                  `int2`
  `number_sorted_blocks`           `int8`
  `number_desorted_blocks`         `int8`
  `number_unsorted_blocks`         `int8`
  `number_total_blocks`            `int8`
  `number_sorted_rows`             `int8`
  `number_desorted_rows`           `int8`
  `number_unsorted_rows`           `int8`
  `number_total_rows`              `int8`
  `number_rows_deleted`            `int8`
  `number_rows_inserted`           `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### user_id

### user_name

### table_creation

### table_age

### table_number_columns

### table_sort_type

### table_number_sortkeys

### table_distribution_type

### table_distribution_auto_flag

### number_slices

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows

### number_rows_deleted

### number_rows_inserted
'),
  ('lf_241129', 'steps_totals_data_on_both_for_statements_by_hour', 'view', '# `lf_241129.steps_totals_data_on_both_for_statements_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### statement_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_users_by_hour', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### uuser

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('lf_241129', 'queries_totals_result_cache_for_query', 'view', '# `lf_241129.queries_totals_result_cache_for_query`

## Description

## Columns

  column           data type
  ---------------- -----------
  `query_id`       `int4`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### query_id

### number_hit

### number_miss

### number_total

### percentage
'),
  ('base_241129', 'fixed_stv_slices', 'view', '# `base_241129.fixed_stv_slices`

## Description

## Columns

  column            data type
  ----------------- -----------
  `node_ordinal`    `int2`
  `slice_ordinal`   `int2`
  `slice_type`      `varchar`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type
'),
  ('lf_241129', 'queries_queued', 'view', '# `lf_241129.queries_queued`

## Description

## Columns

  column             data type
  ------------------ ------------
  `queue_id`         `int2`
  `queue_name`       `varchar`
  `queue_position`   `int2`
  `transaction_id`   `int8`
  `query_id`         `int4`
  `queue_duration`   `interval`

## Column Descriptions

### queue_id

### queue_name

### queue_position

### transaction_id

### query_id

### queue_duration
'),
  ('lf_241129', 'steps_totals_data_on_both_for_slices_by_minute', 'view', '# `lf_241129.steps_totals_data_on_both_for_slices_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `slice_ordinal`                `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### slice_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'queries_queued', 'view', '# `sf_241129.queries_queued`

## Description

## Columns

  column    data type
  --------- -----------
  `qname`   `varchar`
  `pos`     `int2`
  `xid`     `int8`
  `qid`     `int4`
  `dur`     `numeric`

## Column Descriptions

### qname

### pos

### xid

### qid

### dur
'),
  ('sf_241129', 'queries_running', 'view', '# `sf_241129.queries_running`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `pid`       `int4`
  `xid`       `int8`
  `qid`       `int4`
  `qname`     `varchar`
  `s`         `int2`
  `flags`     `varchar`
  `seg`       `int2`
  `dur`       `numeric`
  `cpu`       `numeric`
  `ddata`     `varchar`
  `mem`       `varchar`
  `num_tt`    `int4`
  `size_tt`   `varchar`
  `btd`       `varchar`
  `ttext`     `varchar`

## Column Descriptions

### uuser

### pid

### xid

### qid

### qname

### s

### flags

### seg

### dur

### cpu

### ddata

### mem

### num_tt

### size_tt

### btd

### ttext
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_tables_by_minute', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_tables_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_all_by_minute', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### start_ts

### rread

### to_disk
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_queues_by_second', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_queues_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### qname

### start_ts

### queue

### compile

### exec

### queries
'),
  ('sf_241129', 'objects_totals_for_all', 'view', '# `sf_241129.objects_totals_for_all`

## Description

## Columns

  column           data type
  ---------------- -----------
  `sschemas`       `int2`
  `ttables`        `int4`
  `ttables_cols`   `int4`
  `nviews`         `int4`
  `nviews_cols`    `int4`
  `lviews`         `int4`
  `mviews`         `int4`
  `mviews_cols`    `int4`
  `funcs`          `int4`
  `procs`          `int4`

## Column Descriptions

### sschemas

### ttables

### ttables_cols

### nviews

### nviews_cols

### lviews

### mviews

### mviews_cols

### funcs

### procs
'),
  ('ro_241129', 'sp_grant_versionless_sf_schema_to_user', 'procedure', '# `ro_241129.sp_grant_versionless_sf_schema_to_user`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `user_name`   `varchar( 127 )`

## Argument Descriptions

### user_name
'),
  ('base_241129', 'fixed_stl_vacuum_detail', 'view', '# `base_241129.fixed_stl_vacuum_detail`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `compact`                      `int4`
  `empty_space_size`             `int8`
  `end_row`                      `int8`
  `insufficient_local_blk_cnt`   `int4`
  `local_block_count`            `int8`
  `local_to_total_ratio`         `float8`
  `num_blocks_replaced`          `int8`
  `num_deleted_rows`             `int8`
  `num_new_blocks`               `int8`
  `num_new_rows`                 `int8`
  `part`                         `int4`
  `session_id`                   `int4`
  `query_id`                     `int4`
  `slice_ordinal`                `int2`
  `start_row`                    `int8`
  `table_id`                     `int4`
  `table_partition_number`       `int4`
  `total_block_io_estimate`      `int8`
  `user_id`                      `int4`
  `window_num`                   `int4`
  `transaction_id`               `int8`

## Column Descriptions

### compact

### empty_space_size

### end_row

### insufficient_local_blk_cnt

### local_block_count

### local_to_total_ratio

### num_blocks_replaced

### num_deleted_rows

### num_new_blocks

### num_new_rows

### part

### session_id

### query_id

### slice_ordinal

### start_row

### table_id

### table_partition_number

### total_block_io_estimate

### user_id

### window_num

### transaction_id
'),
  ('lf_241129', 'analyze_totals_for_database_by_hour', 'view', '# `lf_241129.analyze_totals_for_database_by_hour`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `database_name`                                 `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### database_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('sf_241129', 'steps_totals_data_on_both_for_statements_by_minute', 'view', '# `sf_241129.steps_totals_data_on_both_for_statements_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### stmt

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'ttables_creation_for_schemas_by_minute', 'view', '# `sf_241129.ttables_creation_for_schemas_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `start_ts`   `timestamp`
  `ttables`    `int4`

## Column Descriptions

### sschema

### start_ts

### ttables
'),
  ('ro_241129', 'sp_create_versionless_sf_schema_pointed_at_version', 'procedure', '# `ro_241129.sp_create_versionless_sf_schema_pointed_at_version`

## Description

## Arguments

  argument    data type
  ----------- ----------------
  `version`   `varchar( 6 )`

## Argument Descriptions

### version
'),
  ('lf_241129', 'uusers_privileges', 'view', '# `lf_241129.uusers_privileges`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `user_id`                                `int4`
  `user_name`                              `varchar`
  `user_owner`                             `bool`
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `priv_origin_type`                       `varchar`
  `priv_origin_id`                         `int4`
  `priv_origin_name`                       `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `priv_database_create`                   `bool`
  `priv_database_grant_create`             `bool`
  `priv_database_temp`                     `bool`
  `priv_database_grant_temp`               `bool`
  `priv_function_execute`                  `bool`
  `priv_function_grant_execute`            `bool`
  `priv_language_usage`                    `bool`
  `priv_language_grant_usage`              `bool`
  `priv_procedure_execute`                 `bool`
  `priv_procedure_grant_execute`           `bool`
  `priv_schema_create`                     `bool`
  `priv_schema_grant_create`               `bool`
  `priv_schema_usage`                      `bool`
  `priv_schema_grant_usage`                `bool`
  `priv_table_delete`                      `bool`
  `priv_table_grant_delete`                `bool`
  `priv_table_drop`                        `bool`
  `priv_table_grant_drop`                  `bool`
  `priv_table_insert`                      `bool`
  `priv_table_grant_insert`                `bool`
  `priv_table_reference`                   `bool`
  `priv_table_grant_reference`             `bool`
  `priv_table_select`                      `bool`
  `priv_table_grant_select`                `bool`
  `priv_table_update`                      `bool`
  `priv_table_grant_update`                `bool`
  `priv_view_drop`                         `bool`
  `priv_view_grant_drop`                   `bool`
  `priv_view_select`                       `bool`
  `priv_view_grant_select`                 `bool`

## Column Descriptions

### user_id

### user_name

### user_owner

### object_owner_user_id

### object_owner_user_name

### priv_origin_type

### priv_origin_id

### priv_origin_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### priv_database_create

### priv_database_grant_create

### priv_database_temp

### priv_database_grant_temp

### priv_function_execute

### priv_function_grant_execute

### priv_language_usage

### priv_language_grant_usage

### priv_procedure_execute

### priv_procedure_grant_execute

### priv_schema_create

### priv_schema_grant_create

### priv_schema_usage

### priv_schema_grant_usage

### priv_table_delete

### priv_table_grant_delete

### priv_table_drop

### priv_table_grant_drop

### priv_table_insert

### priv_table_grant_insert

### priv_table_reference

### priv_table_grant_reference

### priv_table_select

### priv_table_grant_select

### priv_table_update

### priv_table_grant_update

### priv_view_drop

### priv_view_grant_drop

### priv_view_select

### priv_view_grant_select
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_statements_by_hour', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_statements_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### stmt

### start_ts

### queue

### compile

### exec

### queries
'),
  ('sf_241129', 'analyze_totals_for_tables_by_all', 'view', '# `sf_241129.analyze_totals_for_tables_by_all`

## Description

## Columns

  column          data type
  --------------- -----------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### sschema

### ttable

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_tables_by_all', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_tables_by_all`

## Description

## Columns

  column                data type
  --------------------- -----------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `table_id`            `int4`
  `table_name`          `varchar`
  `node_type`           `varchar`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_minute', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_schemas_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'transactions', 'view', '# `lf_241129.transactions`

## Description

## Columns

  column                                      data type
  ------------------------------------------- -------------
  `user_id`                                   `int4`
  `user_name`                                 `varchar`
  `session_id`                                `int4`
  `transaction_id`                            `int8`
  `transaction_start_ts`                      `timestamp`
  `transaction_end_ts`                        `timestamp`
  `transaction_query_queue_duration`          `interval`
  `transaction_query_compile_exec_duration`   `interval`
  `transaction_query_total_duration`          `interval`
  `transaction_idle_duration`                 `interval`
  `transaction_duration`                      `interval`
  `transaction_number_csc_queries`            `int4`
  `transaction_number_rc_queries`             `int4`
  `transaction_number_leader_queries`         `int4`
  `transaction_number_worker_queries`         `int4`
  `transaction_number_total_queries`          `int4`
  `transaction_abort_flag`                    `bool`

## Column Descriptions

### user_id

### user_name

### session_id

### transaction_id

### transaction_start_ts

### transaction_end_ts

### transaction_query_queue_duration

### transaction_query_compile_exec_duration

### transaction_query_total_duration

### transaction_idle_duration

### transaction_duration

### transaction_number_csc_queries

### transaction_number_rc_queries

### transaction_number_leader_queries

### transaction_number_worker_queries

### transaction_number_total_queries

### transaction_abort_flag
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_users_by_day', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_users_by_day`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_all_by_day', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `node`       `varchar`
  `start_ts`   `date`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('ro_241129', 'sp_grant_versionless_sf_schema_to_role', 'procedure', '# `ro_241129.sp_grant_versionless_sf_schema_to_role`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `role_name`   `varchar( 127 )`

## Argument Descriptions

### role_name
'),
  ('sf_241129', 'blocks_totals_for_tables_by_column', 'view', '# `sf_241129.blocks_totals_for_tables_by_column`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `ccolumn`   `varchar`
  `sblks`     `varchar`
  `ublks`     `varchar`
  `tblks`     `varchar`

## Column Descriptions

### sschema

### ttable

### ccolumn

### sblks

### ublks

### tblks
'),
  ('sf_241129', 'steps_totals_data_on_both_for_slices_by_hour', 'view', '# `sf_241129.steps_totals_data_on_both_for_slices_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `slice`      `int2`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### slice

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_users_by_minute', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### uuser

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_all_by_hour', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'ttables_columns_defaults_references', 'view', '# `sf_241129.ttables_columns_defaults_references`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `ccolumn`   `varchar`
  `def`       `varchar`
  `rref`      `varchar`

## Column Descriptions

### sschema

### ttable

### ccolumn

### def

### rref
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_all_by_day', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_all_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('lf_241129', 'queries_for_leader', 'view', '# `lf_241129.queries_for_leader`

## Description

## Columns

  column                     data type
  -------------------------- -------------
  `user_id`                  `int4`
  `user_name`                `varchar`
  `session_id`               `int4`
  `transaction_id`           `int8`
  `query_start_ts`           `timestamp`
  `query_end_ts`             `timestamp`
  `query_total_duration`     `interval`
  `query_leader_node_flag`   `bool`
  `query_text`               `varchar`

## Column Descriptions

### user_id

### user_name

### session_id

### transaction_id

### query_start_ts

### query_end_ts

### query_total_duration

### query_leader_node_flag

### query_text
'),
  ('ro_241129', 'sp_grant_version_to_role', 'procedure', '# `ro_241129.sp_grant_version_to_role`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `version`     `varchar( 6 )`
  `role_name`   `varchar( 127 )`

## Argument Descriptions

### version

### role_name
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_all_by_second', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_all_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('lf_241129', 'vacuum_stats_for_all_by_hour', 'view', '# `lf_241129.vacuum_stats_for_all_by_hour`

## Description

## Columns

  column                             data type
  ---------------------------------- -------------
  `start_ts`                         `timestamp`
  `min_sum_number_new_blocks`        `int8`
  `max_sum_number_new_blocks`        `int8`
  `med_sum_number_new_blocks`        `int8`
  `avg_sum_number_new_blocks`        `float4`
  `std_sum_number_new_blocks`        `float4`
  `sum_sum_number_new_blocks`        `float4`
  `min_sum_number_replaced_blocks`   `int8`
  `max_sum_number_replaced_blocks`   `int8`
  `med_sum_number_replaced_blocks`   `int8`
  `avg_sum_number_replaced_blocks`   `float4`
  `std_sum_number_replaced_blocks`   `float4`
  `sum_sum_number_replaced_blocks`   `float4`
  `min_sum_number_new_rows`          `int8`
  `max_sum_number_new_rows`          `int8`
  `med_sum_number_new_rows`          `int8`
  `avg_sum_number_new_rows`          `float4`
  `std_sum_number_new_rows`          `float4`
  `sum_sum_number_new_rows`          `float4`
  `min_sum_number_deleted_rows`      `int8`
  `max_sum_number_deleted_rows`      `int8`
  `med_sum_number_deleted_rows`      `int8`
  `avg_sum_number_deleted_rows`      `float4`
  `std_sum_number_deleted_rows`      `float4`
  `sum_sum_number_deleted_rows`      `float4`
  `min_duration`                     `interval`
  `max_duration`                     `interval`
  `med_duration`                     `float4`
  `avg_duration`                     `interval`
  `std_duration`                     `float4`
  `sum_duration`                     `interval`
  `number_qids`                      `int8`
  `number_xids`                      `int8`

## Column Descriptions

### start_ts

### min_sum_number_new_blocks

### max_sum_number_new_blocks

### med_sum_number_new_blocks

### avg_sum_number_new_blocks

### std_sum_number_new_blocks

### sum_sum_number_new_blocks

### min_sum_number_replaced_blocks

### max_sum_number_replaced_blocks

### med_sum_number_replaced_blocks

### avg_sum_number_replaced_blocks

### std_sum_number_replaced_blocks

### sum_sum_number_replaced_blocks

### min_sum_number_new_rows

### max_sum_number_new_rows

### med_sum_number_new_rows

### avg_sum_number_new_rows

### std_sum_number_new_rows

### sum_sum_number_new_rows

### min_sum_number_deleted_rows

### max_sum_number_deleted_rows

### med_sum_number_deleted_rows

### avg_sum_number_deleted_rows

### std_sum_number_deleted_rows

### sum_sum_number_deleted_rows

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_qids

### number_xids
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_all_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_all_concise`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `varchar`
  `aggr`      `numeric`
  `bcast`     `numeric`
  `del`       `numeric`
  `dist`      `numeric`
  `hash`      `numeric`
  `hjoin`     `numeric`
  `iinsert`   `numeric`
  `llimit`    `numeric`
  `mmerge`    `numeric`
  `mjoin`     `numeric`
  `nloop`     `numeric`
  `rreturn`   `numeric`
  `save`      `numeric`
  `ssort`     `numeric`
  `uunique`   `numeric`
  `wwindow`   `numeric`

## Column Descriptions

### node

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_data_on_both_for_all_by_hour', 'view', '# `lf_241129.steps_totals_data_on_both_for_all_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'ddatabases', 'view', '# `sf_241129.ddatabases`

## Description

## Columns

  column            data type
  ----------------- -----------
  `database_name`   `varchar`
  `user_name`       `varchar`
  `iisolation`      `varchar`

## Column Descriptions

### database_name

### user_name

### iisolation
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_statements_by_all', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_statements_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `stmt`      `varchar`
  `queue`     `varchar`
  `compile`   `varchar`
  `exec`      `varchar`
  `queries`   `int4`

## Column Descriptions

### stmt

### queue

### compile

### exec

### queries
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_users_by_second', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### uuser

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_day_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_day_concise`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_schemas_by_hour', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_schemas_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### start_ts

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_query', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_query`

## Description

## Columns

  column             data type
  ------------------ -------------
  `query_id`         `int4`
  `query_start_ts`   `timestamp`
  `number_abort`     `int4`

## Column Descriptions

### query_id

### query_start_ts

### number_abort
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_slices_by_all', 'view', '# `lf_241129.steps_stats_durations_on_both_for_slices_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- ------------
  `node_ordinal`                `int2`
  `slice_ordinal`               `int2`
  `slice_type`                  `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_all_by_day', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### duration

### queries
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_users_by_all', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_users_by_all`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -----------
  `user_id`                                     `int4`
  `user_name`                                   `varchar`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### user_id

### user_name

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_hour_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_hour_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_all_by_day', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_all_by_day`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('sf_241129', 'analyze_totals_status_for_schema', 'view', '# `sf_241129.analyze_totals_status_for_schema`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `good`      `int4`
  `stale`     `int4`
  `missing`   `int4`
  `total`     `int4`

## Column Descriptions

### sschema

### good

### stale

### missing

### total
'),
  ('lf_241129', 'vacuum_totals_for_tables_by_day', 'view', '# `lf_241129.vacuum_totals_for_tables_by_day`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('lf_241129', 'ttables_creation_for_schemas_by_hour', 'view', '# `lf_241129.ttables_creation_for_schemas_by_hour`

## Description

## Columns

  column            data type
  ----------------- -------------
  `schema_name`     `varchar`
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### schema_name

### start_ts

### number_tables
'),
  ('base_241129', 'nodes', 'view', '# `base_241129.nodes`

## Description

## Columns

  column                    data type
  ------------------------- -----------
  `node_ordinal`            `int2`
  `number_partial_slices`   `int2`
  `number_full_slices`      `int2`
  `number_total_slices`     `int2`

## Column Descriptions

### node_ordinal

### number_partial_slices

### number_full_slices

### number_total_slices
'),
  ('sf_241129', 'blocks_rows_totals_for_user', 'view', '# `sf_241129.blocks_rows_totals_for_user`

## Description

## Columns

  column     data type
  ---------- -----------
  `uuser`    `varchar`
  `sblks`    `varchar`
  `dsblks`   `varchar`
  `ublks`    `varchar`
  `tblks`    `varchar`
  `srows`    `varchar`
  `dsrows`   `varchar`
  `urows`    `varchar`
  `trows`    `varchar`

## Column Descriptions

### uuser

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows
'),
  ('lf_241129', 'blocks_rows_totals_for_slice', 'view', '# `lf_241129.blocks_rows_totals_for_slice`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `node_ordinal`             `int2`
  `slice_ordinal`            `int2`
  `slice_type`               `varchar`
  `number_sorted_blocks`     `int8`
  `number_desorted_blocks`   `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`
  `number_sorted_rows`       `int8`
  `number_desorted_rows`     `int8`
  `number_unsorted_rows`     `int8`
  `number_total_rows`        `int8`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('sf_241129', 'queries_text_on_workers_full', 'view', '# `sf_241129.queries_text_on_workers_full`

## Description

## Columns

  column    data type
  --------- -----------
  `pid`     `int4`
  `xid`     `int8`
  `qid`     `int4`
  `n`       `int4`
  `ttext`   `varchar`

## Column Descriptions

### pid

### xid

### qid

### n

### ttext
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_query', 'view', '# `lf_241129.steps_stats_durations_on_both_for_query`

## Description

## Columns

  column                        data type
  ----------------------------- ------------
  `query_id`                    `int4`
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### query_id

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'sschemas', 'view', '# `lf_241129.sschemas`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `number_tables`               `int4`
  `number_normal_views`         `int4`
  `number_late_binding_views`   `int4`
  `number_materialized_views`   `int4`
  `number_functions`            `int4`
  `number_procedures`           `int4`

## Column Descriptions

### schema_id

### schema_name

### user_id

### user_name

### number_tables

### number_normal_views

### number_late_binding_views

### number_materialized_views

### number_functions

### number_procedures
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_node_by_second', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_node_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `n`          `int2`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_users_by_second', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### uuser

### start_ts

### duration

### queries
'),
  ('base_241129', 'fixed_stl_wlm_rule_action', 'view', '# `base_241129.fixed_stl_wlm_rule_action`

## Description

## Columns

  column          data type
  --------------- -------------
  `action`        `varchar`
  `rule_action`   `varchar`
  `query_id`      `int4`
  `event_ts`      `timestamp`
  `rule_name`     `varchar`
  `queue_id`      `int2`
  `user_id`       `int4`

## Column Descriptions

### action

### rule_action

### query_id

### event_ts

### rule_name

### queue_id

### user_id
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_all_by_hour', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_all_by_hour`

## Description

## Columns

  column             data type
  ------------------ -------------
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_users_by_day', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_users_by_day`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_abort
'),
  ('base_241129', 'queries_on_leader_as_for_worker', 'view', '# `base_241129.queries_on_leader_as_for_worker`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `user_id`                       `int4`
  `session_id`                    `int4`
  `transaction_id`                `int8`
  `query_id`                      `int4`
  `queue_id`                      `int2`
  `query_start_ts`                `timestamp`
  `query_end_ts`                  `timestamp`
  `query_queue_duration`          `interval`
  `query_compile_exec_duration`   `interval`
  `query_wlm_duration`            `interval`
  `query_total_duration`          `interval`
  `query_abort_flag`              `bool`
  `query_csc_flag`                `bool`
  `query_leader_node_flag`        `bool`
  `query_rc_flag`                 `bool`
  `query_text`                    `varchar`

## Column Descriptions

### user_id

### session_id

### transaction_id

### query_id

### queue_id

### query_start_ts

### query_end_ts

### query_queue_duration

### query_compile_exec_duration

### query_wlm_duration

### query_total_duration

### query_abort_flag

### query_csc_flag

### query_leader_node_flag

### query_rc_flag

### query_text
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_all_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_all_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `uuser`          `varchar`
  `node`           `varchar`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### uuser

### node

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('base_241129', 'fixed_stv_inflight', 'view', '# `base_241129.fixed_stv_inflight`

## Description

## Columns

  column             data type
  ------------------ -------------
  `csc_flag`         `bool`
  `label`            `varchar`
  `session_id`       `int4`
  `query_id`         `int4`
  `slice_ordinal`    `int2`
  `query_start_ts`   `timestamp`
  `text`             `varchar`
  `user_id`          `int4`
  `transaction_id`   `int8`

## Column Descriptions

### csc_flag

### label

### session_id

### query_id

### slice_ordinal

### query_start_ts

### text

### user_id

### transaction_id
'),
  ('sf_241129', 'steps_totals_data_on_both_for_slices_by_day', 'view', '# `sf_241129.steps_totals_data_on_both_for_slices_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `slice`      `int2`
  `start_ts`   `date`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### slice

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'objects_totals_for_user', 'view', '# `lf_241129.objects_totals_for_user`

## Description

## Columns

  column                               data type
  ------------------------------------ -----------
  `user_id`                            `int4`
  `user_name`                          `varchar`
  `number_databases`                   `int2`
  `number_schemas`                     `int2`
  `number_tables`                      `int4`
  `number_table_columns`               `int4`
  `number_normal_views`                `int4`
  `number_normal_view_columns`         `int4`
  `number_late_binding_views`          `int4`
  `number_materialized_views`          `int4`
  `number_materialized_view_columns`   `int4`
  `number_functions`                   `int4`
  `number_procedures`                  `int4`

## Column Descriptions

### user_id

### user_name

### number_databases

### number_schemas

### number_tables

### number_table_columns

### number_normal_views

### number_normal_view_columns

### number_late_binding_views

### number_materialized_views

### number_materialized_view_columns

### number_functions

### number_procedures
'),
  ('sf_241129', 'ttables_creation_for_schemas_by_second', 'view', '# `sf_241129.ttables_creation_for_schemas_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `start_ts`   `timestamp`
  `ttables`    `int4`

## Column Descriptions

### sschema

### start_ts

### ttables
'),
  ('lf_241129', 'groups', 'view', '# `lf_241129.groups`

## Description

## Columns

  column                 data type
  ---------------------- -----------
  `group_id`             `int4`
  `group_name`           `varchar`
  `group_number_users`   `int4`

## Column Descriptions

### group_id

### group_name

### group_number_users
'),
  ('lf_241129', 'groups_users', 'view', '# `lf_241129.groups_users`

## Description

## Columns

  column         data type
  -------------- -----------
  `group_id`     `int4`
  `group_name`   `varchar`
  `user_id`      `int4`
  `user_name`    `varchar`

## Column Descriptions

### group_id

### group_name

### user_id

### user_name
'),
  ('lf_241129', 'vacuum_totals_for_users_by_minute', 'view', '# `lf_241129.vacuum_totals_for_users_by_minute`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_users_by_minute', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_users_by_minute`

## Description

## Columns

  column          data type
  --------------- -------------
  `uuser`         `varchar`
  `start_ts`      `timestamp`
  `leadin`        `varchar`
  `wlm_leadin`    `varchar`
  `queue`         `varchar`
  `compile`       `varchar`
  `exec`          `varchar`
  `wlm_leadout`   `varchar`
  `leadout`       `varchar`
  `total`         `varchar`
  `unact`         `varchar`
  `queries`       `int4`

## Column Descriptions

### uuser

### start_ts

### leadin

### wlm_leadin

### queue

### compile

### exec

### wlm_leadout

### leadout

### total

### unact

### queries
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_statements_by_minute', 'view', '# `lf_241129.steps_stats_durations_on_both_for_statements_by_minute`

## Description

## Columns

  column             data type
  ------------------ -------------
  `statement_name`   `varchar`
  `node_type`        `varchar`
  `step_type`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `number_steps`     `int8`

## Column Descriptions

### statement_name

### node_type

### step_type

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### number_steps
'),
  ('lf_241129', 'wqueues_totals_slot_utilization_by_second', 'view', '# `lf_241129.wqueues_totals_slot_utilization_by_second`

## Description

## Columns

  column          data type
  --------------- -------------
  `queue_id`      `int2`
  `queue_name`    `varchar`
  `slot_number`   `int2`
  `start_ts`      `timestamp`
  `duration`      `interval`

## Column Descriptions

### queue_id

### queue_name

### slot_number

### start_ts

### duration
'),
  ('lf_241129', 'queries_for_worker', 'view', '# `lf_241129.queries_for_worker`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `user_id`                       `int4`
  `user_name`                     `varchar`
  `session_id`                    `int4`
  `transaction_id`                `int8`
  `query_id`                      `int4`
  `queue_id`                      `int2`
  `queue_name`                    `varchar`
  `query_start_ts`                `timestamp`
  `query_end_ts`                  `timestamp`
  `query_queue_duration`          `interval`
  `query_compile_exec_duration`   `interval`
  `query_wlm_duration`            `interval`
  `query_total_duration`          `interval`
  `query_abort_flag`              `bool`
  `query_csc_flag`                `bool`
  `query_leader_node_flag`        `bool`
  `query_rc_flag`                 `bool`
  `query_text`                    `varchar`

## Column Descriptions

### user_id

### user_name

### session_id

### transaction_id

### query_id

### queue_id

### queue_name

### query_start_ts

### query_end_ts

### query_queue_duration

### query_compile_exec_duration

### query_wlm_duration

### query_total_duration

### query_abort_flag

### query_csc_flag

### query_leader_node_flag

### query_rc_flag

### query_text
'),
  ('base_241129', 'fixed_stl_load_errors', 'view', '# `base_241129.fixed_stl_load_errors`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `column_name`                  `varchar`
  `reason`                       `varchar`
  `file_name`                    `varchar`
  `line_number`                  `int8`
  `query_id`                     `int4`
  `field_value`                  `varchar`
  `slice_ordinal`                `int2`
  `event_ts`                     `timestamp`
  `table_id`                     `int4`
  `column_full_data_type_name`   `varchar`
  `user_id`                      `int4`

## Column Descriptions

### column_name

### reason

### file_name

### line_number

### query_id

### field_value

### slice_ordinal

### event_ts

### table_id

### column_full_data_type_name

### user_id
'),
  ('lf_241129', 'views_normal', 'view', '# `lf_241129.views_normal`

## Description

## Columns

  column          data type
  --------------- -----------
  `schema_id`     `int4`
  `schema_name`   `varchar`
  `view_id`       `int4`
  `view_name`     `varchar`
  `user_id`       `int4`
  `user_name`     `varchar`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### user_id

### user_name
'),
  ('lf_241129', 'sp_create_group_if_not_exists', 'procedure', '# `lf_241129.sp_create_group_if_not_exists`

## Description

## Arguments

  argument       data type
  -------------- ------------------
  `group_name`   `varchar( 127 )`

## Argument Descriptions

### group_name
'),
  ('lf_241129', 'queries_totals_result_cache_for_all_by_hour', 'view', '# `lf_241129.queries_totals_result_cache_for_all_by_hour`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_queues_by_hour', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_queues_by_hour`

## Description

## Columns

  column                data type
  --------------------- -------------
  `queue_id`            `int2`
  `queue_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### queue_id

### queue_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_statements_by_hour', 'view', '# `lf_241129.steps_stats_durations_on_both_for_statements_by_hour`

## Description

## Columns

  column             data type
  ------------------ -------------
  `statement_name`   `varchar`
  `node_type`        `varchar`
  `step_type`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `number_steps`     `int8`

## Column Descriptions

### statement_name

### node_type

### step_type

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### number_steps
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_day_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_day_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `start_ts`       `date`
  `n`              `int2`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_slices_by_hour', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_slices_by_hour`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `slice_ordinal`       `int2`
  `slice_type`          `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('base_241129', 'fixed_stl_utilitytext', 'view', '# `base_241129.fixed_stl_utilitytext`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `user_id`                       `int4`
  `transaction_id`                `int8`
  `session_id`                    `int4`
  `queue_id`                      `int2`
  `query_start_ts`                `timestamp`
  `query_end_ts`                  `timestamp`
  `sequence`                      `int4`
  `query_text`                    `varchar`
  `query_queue_duration`          `interval`
  `query_wlm_duration`            `interval`
  `query_compile_exec_duration`   `interval`
  `query_total_duration`          `interval`

## Column Descriptions

### user_id

### transaction_id

### session_id

### queue_id

### query_start_ts

### query_end_ts

### sequence

### query_text

### query_queue_duration

### query_wlm_duration

### query_compile_exec_duration

### query_total_duration
'),
  ('lf_241129', 'views_normal_columns', 'view', '# `lf_241129.views_normal_columns`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `schema_id`                    `int4`
  `schema_name`                  `varchar`
  `view_id`                      `int4`
  `view_name`                    `varchar`
  `column_ordinal`               `int2`
  `column_name`                  `varchar`
  `column_data_type_id`          `int4`
  `column_base_data_type_name`   `varchar`
  `column_octet_length`          `int4`
  `column_precision`             `int2`
  `column_scale`                 `int2`
  `column_full_data_type_name`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### column_ordinal

### column_name

### column_data_type_id

### column_base_data_type_name

### column_octet_length

### column_precision

### column_scale

### column_full_data_type_name
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_all_by_minute', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_all_by_minute`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### start_ts

### number_abort
'),
  ('lf_241129', 'ffunctions', 'view', '# `lf_241129.ffunctions`

## Description

## Columns

  column                                      data type
  ------------------------------------------- -----------
  `schema_id`                                 `int4`
  `schema_name`                               `varchar`
  `language_id`                               `int4`
  `language_name`                             `varchar`
  `user_id`                                   `int4`
  `user_name`                                 `varchar`
  `function_volatility`                       `varchar`
  `function_return_data_type_id`              `int4`
  `function_return_data_type_name`            `varchar`
  `function_id`                               `int4`
  `function_name`                             `varchar`
  `function_arguments_base_data_type_ids`     `varchar`
  `function_arguments_base_data_type_names`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### language_id

### language_name

### user_id

### user_name

### function_volatility

### function_return_data_type_id

### function_return_data_type_name

### function_id

### function_name

### function_arguments_base_data_type_ids

### function_arguments_base_data_type_names
'),
  ('ro_241129', 'sp_revoke_versionless_sf_schema_from_group', 'procedure', '# `ro_241129.sp_revoke_versionless_sf_schema_from_group`

## Description

## Arguments

  argument       data type
  -------------- ------------------
  `group_name`   `varchar( 127 )`

## Argument Descriptions

### group_name
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_all_by_day', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_all_by_day`

## Description

## Columns

  column             data type
  ------------------ -------------
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_users_by_all', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_users_by_all`

## Description

## Columns

  column           data type
  ---------------- -----------
  `user_id`        `int4`
  `user_name`      `varchar`
  `number_abort`   `int4`

## Column Descriptions

### user_id

### user_name

### number_abort
'),
  ('lf_241129', 'objects', 'view', '# `lf_241129.objects`

## Description

## Columns

  column               data type
  -------------------- -----------
  `schema_id`          `int4`
  `schema_name`        `varchar`
  `object_id`          `int4`
  `object_name`        `varchar`
  `object_type`        `varchar`
  `object_user_id`     `int4`
  `object_user_name`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### object_id

### object_name

### object_type

### object_user_id

### object_user_name
'),
  ('lf_241129', 'wqueues_totals_slot_utilization_by_all', 'view', '# `lf_241129.wqueues_totals_slot_utilization_by_all`

## Description

## Columns

  column          data type
  --------------- ------------
  `queue_id`      `int2`
  `queue_name`    `varchar`
  `slot_number`   `int2`
  `duration`      `interval`

## Column Descriptions

### queue_id

### queue_name

### slot_number

### duration
'),
  ('sf_241129', 'ttables_creation_for_all_by_second', 'view', '# `sf_241129.ttables_creation_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `ttables`    `int4`

## Column Descriptions

### start_ts

### ttables
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_query', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_query`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -----------
  `query_id`                                    `int4`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### query_id

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_all_by_day', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_all_by_day`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('lf_241129', 'steps_totals_data_on_both_for_all_by_day', 'view', '# `lf_241129.steps_totals_data_on_both_for_all_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_users_by_minute', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_users_by_minute`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_node_by_hour', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_node_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `n`          `int2`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('base_241129', 'queries_transactions', 'view', '# `base_241129.queries_transactions`

## Description

## Columns

  column                                      data type
  ------------------------------------------- -------------
  `user_id`                                   `int4`
  `session_id`                                `int4`
  `transaction_id`                            `int8`
  `query_id`                                  `int4`
  `queue_id`                                  `int2`
  `query_start_ts`                            `timestamp`
  `query_end_ts`                              `timestamp`
  `query_queue_duration`                      `interval`
  `query_compile_exec_duration`               `interval`
  `query_wlm_duration`                        `interval`
  `query_total_duration`                      `interval`
  `query_abort_flag`                          `bool`
  `query_csc_flag`                            `bool`
  `query_leader_node_flag`                    `bool`
  `query_rc_flag`                             `bool`
  `query_text`                                `varchar`
  `transaction_start_ts`                      `timestamp`
  `transaction_end_ts`                        `timestamp`
  `transaction_query_total_duration`          `interval`
  `transaction_query_wlm_duration`            `interval`
  `transaction_query_queue_duration`          `interval`
  `transaction_query_compile_exec_duration`   `interval`
  `transaction_number_rc_queries`             `int4`
  `transaction_number_leader_queries`         `int4`
  `transaction_number_worker_queries`         `int4`
  `transaction_number_total_queries`          `int4`
  `transaction_abort_flag`                    `bool`
  `transaction_duration`                      `interval`
  `transaction_idle_duration`                 `interval`

## Column Descriptions

### user_id

### session_id

### transaction_id

### query_id

### queue_id

### query_start_ts

### query_end_ts

### query_queue_duration

### query_compile_exec_duration

### query_wlm_duration

### query_total_duration

### query_abort_flag

### query_csc_flag

### query_leader_node_flag

### query_rc_flag

### query_text

### transaction_start_ts

### transaction_end_ts

### transaction_query_total_duration

### transaction_query_wlm_duration

### transaction_query_queue_duration

### transaction_query_compile_exec_duration

### transaction_number_rc_queries

### transaction_number_leader_queries

### transaction_number_worker_queries

### transaction_number_total_queries

### transaction_abort_flag

### transaction_duration

### transaction_idle_duration
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_query', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_query`

## Description

## Columns

  column      data type
  ----------- -----------
  `qid`       `int4`
  `rread`     `varchar`
  `to_disk`   `varchar`

## Column Descriptions

### qid

### rread

### to_disk
'),
  ('lf_241129', 'ttables_stats_dists_sorts_for_schemas_by_count', 'view', '# `lf_241129.ttables_stats_dists_sorts_for_schemas_by_count`

## Description

## Columns

  column               data type
  -------------------- -----------
  `schema_name`        `varchar`
  `all_compound`       `int4`
  `all_interleaved`    `int4`
  `all_unsorted`       `int4`
  `even_compound`      `int4`
  `even_interleaved`   `int4`
  `even_unsorted`      `int4`
  `key_compound`       `int4`
  `key_interleaved`    `int4`
  `key_unsorted`       `int4`

## Column Descriptions

### schema_name

### all_compound

### all_interleaved

### all_unsorted

### even_compound

### even_interleaved

### even_unsorted

### key_compound

### key_interleaved

### key_unsorted
'),
  ('base_241129', 'ttables_blocks_rows', 'view', '# `base_241129.ttables_blocks_rows`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `schema_id`                `int4`
  `table_id`                 `int4`
  `number_slices`            `int2`
  `number_sorted_blocks`     `int8`
  `number_desorted_blocks`   `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`
  `number_sorted_rows`       `int8`
  `number_desorted_rows`     `int8`
  `number_unsorted_rows`     `int8`
  `number_total_rows`        `int8`

## Column Descriptions

### schema_id

### table_id

### number_slices

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('sf_241129', 'ffunctions', 'view', '# `sf_241129.ffunctions`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttype`     `varchar`
  `uuser`     `varchar`
  `vol`       `bpchar`
  `rreturn`   `varchar`
  `nname`     `varchar`
  `args`      `varchar`

## Column Descriptions

### sschema

### ttype

### uuser

### vol

### rreturn

### nname

### args
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_node_by_day', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_node_by_day`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'analyze_totals_for_all_by_all', 'view', '# `lf_241129.analyze_totals_for_all_by_all`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -----------
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'vacuum_totals_for_all_by_hour', 'view', '# `lf_241129.vacuum_totals_for_all_by_hour`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_users_by_all', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_users_by_all`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### uuser

### duration

### queries
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_users_by_all', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_users_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `ddelete`   `varchar`
  `iinsert`   `varchar`
  `scan`      `varchar`

## Column Descriptions

### uuser

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'queries_running', 'view', '# `lf_241129.queries_running`

## Description

## Columns

  column                            data type
  --------------------------------- ------------
  `queue_id`                        `int2`
  `queue_name`                      `varchar`
  `csc_flag`                        `bool`
  `user_id`                         `int4`
  `user_name`                       `varchar`
  `session_id`                      `int4`
  `transaction_id`                  `int8`
  `query_id`                        `int4`
  `query_slot_count`                `int2`
  `max_exec_segment`                `int2`
  `query_exec_duration`             `interval`
  `query_cpu_duration`              `interval`
  `sum_data_sessioned_in_bytes`     `int8`
  `sum_memory_used_in_bytes`        `int8`
  `number_transient_tables`         `int4`
  `sum_transient_tables_in_bytes`   `int8`
  `query_number_blocks_to_disk`     `int8`
  `query_text`                      `varchar`

## Column Descriptions

### queue_id

### queue_name

### csc_flag

### user_id

### user_name

### session_id

### transaction_id

### query_id

### query_slot_count

### max_exec_segment

### query_exec_duration

### query_cpu_duration

### sum_data_sessioned_in_bytes

### sum_memory_used_in_bytes

### number_transient_tables

### sum_transient_tables_in_bytes

### query_number_blocks_to_disk

### query_text
'),
  ('base_241129', 'hr_blks', 'function', '# `base_241129.hr_blks`

## Description

## Returns

`varchar`

## Arguments

  argument   data type
  ---------- -----------
  `blocks`   `int8`

## Argument Descriptions

### blocks
'),
  ('lf_241129', 'queries_compilation_stream', 'view', '# `lf_241129.queries_compilation_stream`

## Description

## Columns

  column                                       data type
  -------------------------------------------- ------------
  `session_id`                                 `int4`
  `transaction_id`                             `int8`
  `query_id`                                   `int4`
  `stream_ordinal`                             `int2`
  `stream_number_leader_node_cache_segments`   `int2`
  `stream_number_offload_segments`             `int2`
  `stream_number_segments`                     `int2`
  `stream_compile_duration`                    `interval`

## Column Descriptions

### session_id

### transaction_id

### query_id

### stream_ordinal

### stream_number_leader_node_cache_segments

### stream_number_offload_segments

### stream_number_segments

### stream_compile_duration
'),
  ('base_241129', 'fixed_stv_wlm_service_class_state', 'view', '# `base_241129.fixed_stv_wlm_service_class_state`

## Description

## Columns

  column                              data type
  ----------------------------------- -----------
  `num_concurrency_scaling_queries`   `int4`
  `num_evicted_queries`               `int4`
  `num_executed_queries`              `int4`
  `num_executing_queries`             `int4`
  `num_queued_queries`                `int4`
  `num_serviced_queries`              `int4`
  `queue_id`                          `int2`

## Column Descriptions

### num_concurrency_scaling_queries

### num_evicted_queries

### num_executed_queries

### num_executing_queries

### num_queued_queries

### num_serviced_queries

### queue_id
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_users_by_hour', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_users_by_hour`

## Description

## Columns

  column          data type
  --------------- -------------
  `uuser`         `varchar`
  `start_ts`      `timestamp`
  `leadin`        `varchar`
  `wlm_leadin`    `varchar`
  `queue`         `varchar`
  `compile`       `varchar`
  `exec`          `varchar`
  `wlm_leadout`   `varchar`
  `leadout`       `varchar`
  `total`         `varchar`
  `unact`         `varchar`
  `queries`       `int4`

## Column Descriptions

### uuser

### start_ts

### leadin

### wlm_leadin

### queue

### compile

### exec

### wlm_leadout

### leadout

### total

### unact

### queries
'),
  ('lf_241129', 'wqueues', 'view', '# `lf_241129.wqueues`

## Description

## Columns

  column                           data type
  -------------------------------- ------------
  `queue_id`                       `int2`
  `queue_name`                     `varchar`
  `number_slots`                   `int2`
  `number_queued_queries`          `int2`
  `number_running_queries`         `int2`
  `number_completed_queries`       `int4`
  `min_queue_duration`             `interval`
  `max_queue_duration`             `interval`
  `med_queue_duration`             `float4`
  `avg_queue_duration`             `interval`
  `std_queue_duration`             `float4`
  `min_exec_duration`              `interval`
  `max_exec_duration`              `interval`
  `med_exec_duration`              `float4`
  `avg_exec_duration`              `interval`
  `std_exec_duration`              `float4`
  `queue_csc_flag`                 `bool`
  `number_csc_completed_queries`   `int4`

## Column Descriptions

### queue_id

### queue_name

### number_slots

### number_queued_queries

### number_running_queries

### number_completed_queries

### min_queue_duration

### max_queue_duration

### med_queue_duration

### avg_queue_duration

### std_queue_duration

### min_exec_duration

### max_exec_duration

### med_exec_duration

### avg_exec_duration

### std_exec_duration

### queue_csc_flag

### number_csc_completed_queries
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_users_by_minute', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### uuser

### start_ts

### rread

### to_disk
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_all_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_all_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `n`              `int2`
  `s`              `int2`
  `t`              `varchar`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### n

### s

### t

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_data_on_both_for_statements_by_day', 'view', '# `lf_241129.steps_totals_data_on_both_for_statements_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### statement_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('base_241129', 'fixed_stv_wlm_classification_config', 'view', '# `base_241129.fixed_stv_wlm_classification_config`

## Description

## Columns

  column         data type
  -------------- -----------
  `action`       `varchar`
  `action_seq`   `int2`
  `queue_id`     `int2`
  `condition`    `varchar`
  `id`           `int2`

## Column Descriptions

### action

### action_seq

### queue_id

### condition

### id
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_all_by_all', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_all_by_all`

## Description

## Columns

  column       data type
  ------------ -----------
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### duration

### queries
'),
  ('sf_241129', 'steps_totals_data_on_both_for_node_by_second', 'view', '# `sf_241129.steps_totals_data_on_both_for_node_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `int2`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'groups_privileges', 'view', '# `lf_241129.groups_privileges`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `group_id`                               `int4`
  `group_name`                             `varchar`
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `priv_database_create`                   `bool`
  `priv_database_temp`                     `bool`
  `priv_function_execute`                  `bool`
  `priv_language_usage`                    `bool`
  `priv_procedure_execute`                 `bool`
  `priv_schema_create`                     `bool`
  `priv_schema_usage`                      `bool`
  `priv_table_delete`                      `bool`
  `priv_table_drop`                        `bool`
  `priv_table_insert`                      `bool`
  `priv_table_reference`                   `bool`
  `priv_table_select`                      `bool`
  `priv_table_update`                      `bool`
  `priv_view_drop`                         `bool`
  `priv_view_select`                       `bool`

## Column Descriptions

### group_id

### group_name

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### priv_database_create

### priv_database_temp

### priv_function_execute

### priv_language_usage

### priv_procedure_execute

### priv_schema_create

### priv_schema_usage

### priv_table_delete

### priv_table_drop

### priv_table_insert

### priv_table_reference

### priv_table_select

### priv_table_update

### priv_view_drop

### priv_view_select
'),
  ('sf_241129', 'privileges_granted_to_users', 'view', '# `sf_241129.privileges_granted_to_users`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `ttype`     `varchar`
  `sschema`   `varchar`
  `oobject`   `varchar`
  `rt`        `varchar`
  `args`      `varchar`
  `oowner`    `varchar`
  `privs`     `varchar`

## Column Descriptions

### uuser

### ttype

### sschema

### oobject

### rt

### args

### oowner

### privs
'),
  ('sf_241129', 'vacuum_totals_for_all_by_second', 'view', '# `sf_241129.vacuum_totals_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### start_ts

### manual

### auto

### total

### dur
'),
  ('lf_241129', 'steps_totals_data_on_both_for_node_by_minute', 'view', '# `lf_241129.steps_totals_data_on_both_for_node_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_ordinal`                 `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_all_by_day', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_all_by_day`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_all_by_day', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### queue

### compile

### exec

### queries
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_users_by_hour', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### uuser

### start_ts

### duration

### queries
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_all_by_second', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### start_ts

### aborts
'),
  ('lf_241129', 'vacuum_stats_for_all_by_minute', 'view', '# `lf_241129.vacuum_stats_for_all_by_minute`

## Description

## Columns

  column                             data type
  ---------------------------------- -------------
  `start_ts`                         `timestamp`
  `min_sum_number_new_blocks`        `int8`
  `max_sum_number_new_blocks`        `int8`
  `med_sum_number_new_blocks`        `int8`
  `avg_sum_number_new_blocks`        `float4`
  `std_sum_number_new_blocks`        `float4`
  `sum_sum_number_new_blocks`        `float4`
  `min_sum_number_replaced_blocks`   `int8`
  `max_sum_number_replaced_blocks`   `int8`
  `med_sum_number_replaced_blocks`   `int8`
  `avg_sum_number_replaced_blocks`   `float4`
  `std_sum_number_replaced_blocks`   `float4`
  `sum_sum_number_replaced_blocks`   `float4`
  `min_sum_number_new_rows`          `int8`
  `max_sum_number_new_rows`          `int8`
  `med_sum_number_new_rows`          `int8`
  `avg_sum_number_new_rows`          `float4`
  `std_sum_number_new_rows`          `float4`
  `sum_sum_number_new_rows`          `float4`
  `min_sum_number_deleted_rows`      `int8`
  `max_sum_number_deleted_rows`      `int8`
  `med_sum_number_deleted_rows`      `int8`
  `avg_sum_number_deleted_rows`      `float4`
  `std_sum_number_deleted_rows`      `float4`
  `sum_sum_number_deleted_rows`      `float4`
  `min_duration`                     `interval`
  `max_duration`                     `interval`
  `med_duration`                     `float4`
  `avg_duration`                     `interval`
  `std_duration`                     `float4`
  `sum_duration`                     `interval`
  `number_qids`                      `int8`
  `number_xids`                      `int8`

## Column Descriptions

### start_ts

### min_sum_number_new_blocks

### max_sum_number_new_blocks

### med_sum_number_new_blocks

### avg_sum_number_new_blocks

### std_sum_number_new_blocks

### sum_sum_number_new_blocks

### min_sum_number_replaced_blocks

### max_sum_number_replaced_blocks

### med_sum_number_replaced_blocks

### avg_sum_number_replaced_blocks

### std_sum_number_replaced_blocks

### sum_sum_number_replaced_blocks

### min_sum_number_new_rows

### max_sum_number_new_rows

### med_sum_number_new_rows

### avg_sum_number_new_rows

### std_sum_number_new_rows

### sum_sum_number_new_rows

### min_sum_number_deleted_rows

### max_sum_number_deleted_rows

### med_sum_number_deleted_rows

### avg_sum_number_deleted_rows

### std_sum_number_deleted_rows

### sum_sum_number_deleted_rows

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_qids

### number_xids
'),
  ('sf_241129', 'ttables_stats_datatypes_encodings_by_count_and_blocks', 'view', '# `sf_241129.ttables_stats_datatypes_encodings_by_count_and_blocks`

## Description

## Columns

  column        data type
  ------------- -----------
  `data_type`   `varchar`
  `az64`        `varchar`
  `bytedict`    `varchar`
  `delta`       `varchar`
  `delta32k`    `varchar`
  `lzo`         `varchar`
  `mostly8`     `varchar`
  `mostly16`    `varchar`
  `mostly32`    `varchar`
  `raw`         `varchar`
  `runlength`   `varchar`
  `text255`     `varchar`
  `text32k`     `varchar`
  `zstd`        `varchar`
  `total`       `varchar`

## Column Descriptions

### data_type

### az64

### bytedict

### delta

### delta32k

### lzo

### mostly8

### mostly16

### mostly32

### raw

### runlength

### text255

### text32k

### zstd

### total
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_minute_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_minute_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `n`              `int2`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('sf_241129', 'ttables_blocks_rows', 'view', '# `sf_241129.ttables_blocks_rows`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `uuser`     `varchar`
  `age`       `int4`
  `cols`      `int2`
  `ssort`     `varchar`
  `skeys`     `int2`
  `dist`      `varchar`
  `slices`    `int2`
  `sblks`     `int8`
  `dsblks`    `int8`
  `ublks`     `int8`
  `tblks`     `int8`
  `srows`     `int8`
  `dsrows`    `int8`
  `urows`     `int8`
  `trows`     `int8`

## Column Descriptions

### sschema

### ttable

### uuser

### age

### cols

### ssort

### skeys

### dist

### slices

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_query', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_query`

## Description

## Columns

  column                         data type
  ------------------------------ ------------
  `query_id`                     `int4`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`

## Column Descriptions

### query_id

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_all_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_all_concise`

## Description

## Columns

  column      data type
  ----------- -----------
  `n`         `int2`
  `aggr`      `numeric`
  `bcast`     `numeric`
  `del`       `numeric`
  `dist`      `numeric`
  `hash`      `numeric`
  `hjoin`     `numeric`
  `iinsert`   `numeric`
  `llimit`    `numeric`
  `mmerge`    `numeric`
  `mjoin`     `numeric`
  `nloop`     `numeric`
  `rreturn`   `numeric`
  `save`      `numeric`
  `ssort`     `numeric`
  `uunique`   `numeric`
  `wwindow`   `numeric`

## Column Descriptions

### n

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_tables_by_second', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_tables_by_second`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `table_id`            `int4`
  `table_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'wqueues', 'view', '# `sf_241129.wqueues`

## Description

## Columns

  column        data type
  ------------- -----------
  `qname`       `varchar`
  `slots`       `int2`
  `queued`      `int2`
  `running`     `int2`
  `completed`   `int4`
  `queue`       `varchar`
  `exec`        `varchar`
  `csc`         `varchar`
  `csc_comp`    `int4`

## Column Descriptions

### qname

### slots

### queued

### running

### completed

### queue

### exec

### csc

### csc_comp
'),
  ('sf_241129', 'queries_for_all', 'view', '# `sf_241129.queries_for_all`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `pid`        `int4`
  `xid`        `int8`
  `qid`        `int4`
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `end_ts`     `timestamp`
  `queue`      `numeric`
  `cexec`      `numeric`
  `total`      `numeric`
  `flags`      `varchar`
  `ttext`      `varchar`

## Column Descriptions

### uuser

### pid

### xid

### qid

### qname

### start_ts

### end_ts

### queue

### cexec

### total

### flags

### ttext
'),
  ('base_241129', 'views_tables', 'view', '# `base_241129.views_tables`

## Description

## Columns

  column       data type
  ------------ -----------
  `view_id`    `int4`
  `table_id`   `int4`

## Column Descriptions

### view_id

### table_id
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_schemas_by_all', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_schemas_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ddelete`   `varchar`
  `iinsert`   `varchar`
  `scan`      `varchar`

## Column Descriptions

### sschema

### ddelete

### iinsert

### scan
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_slices_by_hour', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_slices_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### s

### t

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('base_241129', 'fixed_stv_wlm_service_class_config', 'view', '# `base_241129.fixed_stv_wlm_service_class_config`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `queue_csc_flag`              `bool`
  `max_execution_time`          `int4`
  `queue_name`                  `varchar`
  `num_query_tasks`             `int2`
  `query_group_wildcard_flag`   `bool`
  `query_working_mem`           `int4`
  `queue_id`                    `int2`
  `user_group_wildcard_flag`    `bool`
  `user_role_wildcard_flag`     `bool`

## Column Descriptions

### queue_csc_flag

### max_execution_time

### queue_name

### num_query_tasks

### query_group_wildcard_flag

### query_working_mem

### queue_id

### user_group_wildcard_flag

### user_role_wildcard_flag
'),
  ('lf_241129', 'steps_totals_data_on_both_for_all_by_second', 'view', '# `lf_241129.steps_totals_data_on_both_for_all_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'analyze_totals_for_users_by_second', 'view', '# `sf_241129.analyze_totals_for_users_by_second`

## Description

## Columns

  column          data type
  --------------- -------------
  `uuser`         `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### uuser

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('base_241129', 'privileges_users', 'view', '# `base_241129.privileges_users`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `acl_recipient_user_id`                  `int4`
  `acl_recipient_user_name`                `varchar`
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `acl_privileges`                         `varchar`

## Column Descriptions

### acl_recipient_user_id

### acl_recipient_user_name

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### acl_privileges
'),
  ('lf_241129', 'views_materialized_table', 'view', '# `lf_241129.views_materialized_table`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `view_id`                        `int4`
  `view_name`                      `varchar`
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `failures`                       `varchar`
  `autorefresh`                    `varchar`
  `refresh_type`                   `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `table_creation`                 `timestamp`
  `table_age`                      `interval`
  `table_number_columns`           `int2`
  `table_sort_type`                `varchar`
  `table_number_sortkeys`          `int2`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### user_id

### user_name

### failures

### autorefresh

### refresh_type

### table_id

### table_name

### table_creation

### table_age

### table_number_columns

### table_sort_type

### table_number_sortkeys

### table_distribution_type

### table_distribution_auto_flag
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_schemas_by_minute', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_schemas_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### start_ts

### ddelete

### iinsert

### scan
'),
  ('ro_241129', 'sp_revoke_versionless_sf_schema_from_user', 'procedure', '# `ro_241129.sp_revoke_versionless_sf_schema_from_user`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `user_name`   `varchar( 127 )`

## Argument Descriptions

### user_name
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_users_by_all', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_users_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ ------------
  `user_id`                      `int4`
  `user_name`                    `varchar`
  `min_leadin_duration`          `interval`
  `max_leadin_duration`          `interval`
  `med_leadin_duration`          `float4`
  `avg_leadin_duration`          `interval`
  `std_leadin_duration`          `float4`
  `sum_leadin_duration`          `interval`
  `min_wlm_leadin_duration`      `interval`
  `max_wlm_leadin_duration`      `interval`
  `med_wlm_leadin_duration`      `float4`
  `avg_wlm_leadin_duration`      `interval`
  `std_wlm_leadin_duration`      `float4`
  `sum_wlm_leadin_duration`      `interval`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `min_wlm_leadout_duration`     `interval`
  `max_wlm_leadout_duration`     `interval`
  `med_wlm_leadout_duration`     `float4`
  `avg_wlm_leadout_duration`     `interval`
  `std_wlm_leadout_duration`     `float4`
  `sum_wlm_leadout_duration`     `interval`
  `min_leadout_duration`         `interval`
  `max_leadout_duration`         `interval`
  `med_leadout_duration`         `float4`
  `avg_leadout_duration`         `interval`
  `std_leadout_duration`         `float4`
  `sum_leadout_duration`         `interval`
  `min_query_total_duration`     `interval`
  `max_query_total_duration`     `interval`
  `med_query_total_duration`     `float4`
  `avg_query_total_duration`     `interval`
  `std_query_total_duration`     `float4`
  `sum_query_total_duration`     `interval`
  `min_unaccounted_duration`     `interval`
  `max_unaccounted_duration`     `interval`
  `med_unaccounted_duration`     `float4`
  `avg_unaccounted_duration`     `interval`
  `std_unaccounted_duration`     `float4`
  `sum_unaccounted_duration`     `interval`
  `number_queries`               `int4`

## Column Descriptions

### user_id

### user_name

### min_leadin_duration

### max_leadin_duration

### med_leadin_duration

### avg_leadin_duration

### std_leadin_duration

### sum_leadin_duration

### min_wlm_leadin_duration

### max_wlm_leadin_duration

### med_wlm_leadin_duration

### avg_wlm_leadin_duration

### std_wlm_leadin_duration

### sum_wlm_leadin_duration

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### min_wlm_leadout_duration

### max_wlm_leadout_duration

### med_wlm_leadout_duration

### avg_wlm_leadout_duration

### std_wlm_leadout_duration

### sum_wlm_leadout_duration

### min_leadout_duration

### max_leadout_duration

### med_leadout_duration

### avg_leadout_duration

### std_leadout_duration

### sum_leadout_duration

### min_query_total_duration

### max_query_total_duration

### med_query_total_duration

### avg_query_total_duration

### std_query_total_duration

### sum_query_total_duration

### min_unaccounted_duration

### max_unaccounted_duration

### med_unaccounted_duration

### avg_unaccounted_duration

### std_unaccounted_duration

### sum_unaccounted_duration

### number_queries
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_queues_by_hour', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_queues_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `queue_id`                     `int2`
  `queue_name`                   `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('sf_241129', 'ttables_columns_blocks', 'view', '# `sf_241129.ttables_columns_blocks`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `ccolumn`   `varchar`
  `ttype`     `varchar`
  `enc`       `varchar`
  `sk`        `int2`
  `dk`        `varchar`
  `nn`        `varchar`
  `sblks`     `varchar`
  `ublks`     `varchar`
  `tblks`     `varchar`

## Column Descriptions

### sschema

### ttable

### ccolumn

### ttype

### enc

### sk

### dk

### nn

### sblks

### ublks

### tblks
'),
  ('sf_241129', 'views_normal_columns', 'view', '# `sf_241129.views_normal_columns`

## Description

## Columns

  column        data type
  ------------- -----------
  `sschema`     `varchar`
  `vview`       `varchar`
  `ccolumn`     `varchar`
  `data_type`   `varchar`

## Column Descriptions

### sschema

### vview

### ccolumn

### data_type
'),
  ('lf_241129', 'queries_totals_result_cache_for_users_by_second', 'view', '# `lf_241129.queries_totals_result_cache_for_users_by_second`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('sf_241129', 'transactions_totals_commits_by_hour', 'view', '# `sf_241129.transactions_totals_commits_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `leader`     `numeric`
  `worker`     `numeric`
  `idle`       `numeric`
  `exec`       `numeric`

## Column Descriptions

### start_ts

### leader

### worker

### idle

### exec
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_second_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_second_complete`

## Description

## Columns

  column          data type
  --------------- -------------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `node`          `varchar`
  `start_ts`      `timestamp`
  `aggr`          `varchar`
  `ddelete`       `varchar`
  `hashjoin`      `varchar`
  `iinsert`       `varchar`
  `mergejoin`     `varchar`
  `nestloop_op`   `varchar`
  `save`          `varchar`
  `ssort`         `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### ddelete

### hashjoin

### iinsert

### mergejoin

### nestloop_op

### save

### ssort
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_node_by_minute', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_node_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `n`          `int2`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'ttables_columns', 'view', '# `lf_241129.ttables_columns`

## Description

## Columns

  column                               data type
  ------------------------------------ -----------
  `schema_id`                          `int4`
  `schema_name`                        `varchar`
  `table_id`                           `int4`
  `table_name`                         `varchar`
  `column_ordinal`                     `int2`
  `column_name`                        `varchar`
  `column_data_type_id`                `int4`
  `column_base_data_type_name`         `varchar`
  `column_octet_length`                `int4`
  `column_precision`                   `int2`
  `column_scale`                       `int2`
  `column_full_data_type_name`         `varchar`
  `column_encoding_id`                 `int4`
  `column_encoding_name`               `varchar`
  `column_sortkey_ordinal`             `int2`
  `column_distkey`                     `bool`
  `column_notnull`                     `bool`
  `column_primary_key`                 `bool`
  `column_unique`                      `bool`
  `column_foreign_key`                 `bool`
  `column_default`                     `bool`
  `column_default_type`                `varchar`
  `column_default_value`               `varchar`
  `column_references_schema_id`        `int4`
  `column_references_schema_name`      `varchar`
  `column_references_table_id`         `int4`
  `column_references_table_name`       `varchar`
  `column_references_column_ordinal`   `int2`
  `column_references_column_name`      `varchar`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### column_ordinal

### column_name

### column_data_type_id

### column_base_data_type_name

### column_octet_length

### column_precision

### column_scale

### column_full_data_type_name

### column_encoding_id

### column_encoding_name

### column_sortkey_ordinal

### column_distkey

### column_notnull

### column_primary_key

### column_unique

### column_foreign_key

### column_default

### column_default_type

### column_default_value

### column_references_schema_id

### column_references_schema_name

### column_references_table_id

### column_references_table_name

### column_references_column_ordinal

### column_references_column_name
'),
  ('sf_241129', 'steps_totals_data_on_both_for_slices_by_all', 'view', '# `sf_241129.steps_totals_data_on_both_for_slices_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `slice`     `int2`
  `aggr`      `varchar`
  `bcast`     `varchar`
  `ddelete`   `varchar`
  `dist`      `varchar`
  `hash`      `varchar`
  `hjoin`     `varchar`
  `iinsert`   `varchar`
  `llimit`    `varchar`
  `mmerge`    `varchar`
  `mjoin`     `varchar`
  `nloop`     `varchar`
  `rreturn`   `varchar`
  `save`      `varchar`
  `ssort`     `varchar`
  `uunique`   `varchar`
  `wwindow`   `varchar`

## Column Descriptions

### slice

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_statements_by_minute', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_statements_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### stmt

### start_ts

### queue

### compile

### exec

### queries
'),
  ('lf_241129', 'views_materialized', 'view', '# `lf_241129.views_materialized`

## Description

## Columns

  column           data type
  ---------------- -----------
  `schema_id`      `int4`
  `schema_name`    `varchar`
  `view_id`        `int4`
  `view_name`      `varchar`
  `user_id`        `int4`
  `user_name`      `varchar`
  `failures`       `varchar`
  `autorefresh`    `varchar`
  `refresh_type`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### user_id

### user_name

### failures

### autorefresh

### refresh_type
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_second_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_second_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `node_type`      `varchar`
  `start_ts`       `timestamp`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### node_type

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'ttables_columns_blocks', 'view', '# `lf_241129.ttables_columns_blocks`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `schema_id`                    `int4`
  `schema_name`                  `varchar`
  `table_id`                     `int4`
  `table_name`                   `varchar`
  `column_ordinal`               `int2`
  `column_name`                  `varchar`
  `column_data_type_id`          `int4`
  `column_base_data_type_name`   `varchar`
  `column_octet_length`          `int4`
  `column_precision`             `int2`
  `column_scale`                 `int2`
  `column_full_data_type_name`   `varchar`
  `column_encoding_id`           `int4`
  `column_encoding_name`         `varchar`
  `column_sortkey_ordinal`       `int2`
  `column_distkey`               `bool`
  `column_notnull`               `bool`
  `number_sorted_blocks`         `int8`
  `number_unsorted_blocks`       `int8`
  `number_total_blocks`          `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### column_ordinal

### column_name

### column_data_type_id

### column_base_data_type_name

### column_octet_length

### column_precision

### column_scale

### column_full_data_type_name

### column_encoding_id

### column_encoding_name

### column_sortkey_ordinal

### column_distkey

### column_notnull

### number_sorted_blocks

### number_unsorted_blocks

### number_total_blocks
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_all_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_all_complete`

## Description

## Columns

  column          data type
  --------------- -----------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `node`          `varchar`
  `aggr`          `varchar`
  `ddelete`       `varchar`
  `hashjoin`      `varchar`
  `iinsert`       `varchar`
  `mergejoin`     `varchar`
  `nestloop_op`   `varchar`
  `save`          `varchar`
  `ssort`         `varchar`

## Column Descriptions

### sschema

### ttable

### node

### aggr

### ddelete

### hashjoin

### iinsert

### mergejoin

### nestloop_op

### save

### ssort
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_node_by_second', 'view', '# `lf_241129.steps_stats_durations_on_both_for_node_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'queries_totals_result_cache_for_all_by_second', 'view', '# `sf_241129.queries_totals_result_cache_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### start_ts

### hit

### miss

### total

### %
'),
  ('sf_241129', 'transactions_stats_commits_by_hour', 'view', '# `sf_241129.transactions_stats_commits_by_hour`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `xid queuing`    `varchar`
  `bid queuing`    `varchar`
  `ldr duration`   `varchar`
  `wrk duration`   `varchar`
  `xids/bid`       `varchar`
  `blks/bid`       `varchar`
  `bids`           `int8`

## Column Descriptions

### start_ts

### xid queuing

### bid queuing

### ldr duration

### wrk duration

### xids/bid

### blks/bid

### bids
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_all_by_all', 'view', '# `lf_241129.steps_stats_durations_on_both_for_all_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- ------------
  `node_type`                   `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### node_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'blocks_rows_totals_for_node', 'view', '# `lf_241129.blocks_rows_totals_for_node`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `node_ordinal`             `int2`
  `number_partial_slices`    `int2`
  `number_full_slices`       `int2`
  `number_total_slices`      `int2`
  `number_sorted_blocks`     `int8`
  `number_desorted_blocks`   `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`
  `number_sorted_rows`       `int8`
  `number_desorted_rows`     `int8`
  `number_unsorted_rows`     `int8`
  `number_total_rows`        `int8`

## Column Descriptions

### node_ordinal

### number_partial_slices

### number_full_slices

### number_total_slices

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('lf_241129', 'ttables_stats_dists_sorts_for_schemas_by_blocks', 'view', '# `lf_241129.ttables_stats_dists_sorts_for_schemas_by_blocks`

## Description

## Columns

  column                                      data type
  ------------------------------------------- -----------
  `schema_name`                               `varchar`
  `number_all_compound_sorted_blocks`         `int8`
  `number_all_interleaved_sorted_blocks`      `int8`
  `number_all_unsorted_sorted_blocks`         `int8`
  `number_all_compound_unsorted_blocks`       `int8`
  `number_all_interleaved_unsorted_blocks`    `int8`
  `number_all_unsorted_unsorted_blocks`       `int8`
  `number_all_compound_total_blocks`          `int8`
  `number_all_interleaved_total_blocks`       `int8`
  `number_all_unsorted_total_blocks`          `int8`
  `number_even_compound_sorted_blocks`        `int8`
  `number_even_interleaved_sorted_blocks`     `int8`
  `number_even_unsorted_sorted_blocks`        `int8`
  `number_even_compound_unsorted_blocks`      `int8`
  `number_even_interleaved_unsorted_blocks`   `int8`
  `number_even_unsorted_unsorted_blocks`      `int8`
  `number_even_compound_total_blocks`         `int8`
  `number_even_interleaved_total_blocks`      `int8`
  `number_even_unsorted_total_blocks`         `int8`
  `number_key_compound_sorted_blocks`         `int8`
  `number_key_interleaved_sorted_blocks`      `int8`
  `number_key_unsorted_sorted_blocks`         `int8`
  `number_key_compound_unsorted_blocks`       `int8`
  `number_key_interleaved_unsorted_blocks`    `int8`
  `number_key_unsorted_unsorted_blocks`       `int8`
  `number_key_compound_total_blocks`          `int8`
  `number_key_interleaved_total_blocks`       `int8`
  `number_key_unsorted_total_blocks`          `int8`

## Column Descriptions

### schema_name

### number_all_compound_sorted_blocks

### number_all_interleaved_sorted_blocks

### number_all_unsorted_sorted_blocks

### number_all_compound_unsorted_blocks

### number_all_interleaved_unsorted_blocks

### number_all_unsorted_unsorted_blocks

### number_all_compound_total_blocks

### number_all_interleaved_total_blocks

### number_all_unsorted_total_blocks

### number_even_compound_sorted_blocks

### number_even_interleaved_sorted_blocks

### number_even_unsorted_sorted_blocks

### number_even_compound_unsorted_blocks

### number_even_interleaved_unsorted_blocks

### number_even_unsorted_unsorted_blocks

### number_even_compound_total_blocks

### number_even_interleaved_total_blocks

### number_even_unsorted_total_blocks

### number_key_compound_sorted_blocks

### number_key_interleaved_sorted_blocks

### number_key_unsorted_sorted_blocks

### number_key_compound_unsorted_blocks

### number_key_interleaved_unsorted_blocks

### number_key_unsorted_unsorted_blocks

### number_key_compound_total_blocks

### number_key_interleaved_total_blocks

### number_key_unsorted_total_blocks
'),
  ('sf_241129', 'wqueues_totals_slot_utilization_by_minute', 'view', '# `sf_241129.wqueues_totals_slot_utilization_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `slot`       `int2`
  `start_ts`   `timestamp`
  `dur`        `numeric`

## Column Descriptions

### qname

### slot

### start_ts

### dur
'),
  ('lf_241129', 'vacuum_totals_for_users_by_hour', 'view', '# `lf_241129.vacuum_totals_for_users_by_hour`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_hour', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_schemas_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'steps_totals_data_on_both_for_all_by_all', 'view', '# `lf_241129.steps_totals_data_on_both_for_all_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `node_type`                    `varchar`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_type

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'analyze_totals_for_users_by_all', 'view', '# `sf_241129.analyze_totals_for_users_by_all`

## Description

## Columns

  column          data type
  --------------- -----------
  `uuser`         `varchar`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### uuser

### manual

### automatic

### autoanalyze

### total
'),
  ('sf_241129', 'ttables_creation_for_all_by_minute', 'view', '# `sf_241129.ttables_creation_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `ttables`    `int4`

## Column Descriptions

### start_ts

### ttables
'),
  ('sf_241129', 'transactions_batches', 'view', '# `sf_241129.transactions_batches`

## Description

## Columns

  column           data type
  ---------------- -------------
  `uuser`          `varchar`
  `pid`            `int4`
  `xid`            `int8`
  `bid`            `int8`
  `xid_start_ts`   `timestamp`
  `queue`          `numeric`
  `cexec`          `numeric`
  `idle`           `numeric`
  `c/r`            `varchar`
  `l/w`            `varchar`
  `a`              `varchar`
  `cqueue`         `numeric`
  `ldr`            `numeric`
  `wrk`            `numeric`
  `bid_end_ts`     `timestamp`
  `blks`           `int8`
  `xids`           `int8`

## Column Descriptions

### uuser

### pid

### xid

### bid

### xid_start_ts

### queue

### cexec

### idle

### c/r

### l/w

### a

### cqueue

### ldr

### wrk

### bid_end_ts

### blks

### xids
'),
  ('sf_241129', 'queries_totals_result_cache_for_users_by_day', 'view', '# `sf_241129.queries_totals_result_cache_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `start_ts`   `date`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### uuser

### start_ts

### hit

### miss

### total

### %
'),
  ('lf_241129', 'wqueues_qmr_rules', 'view', '# `lf_241129.wqueues_qmr_rules`

## Description

## Columns

  column              data type
  ------------------- -----------
  `rule_ordinal`      `int2`
  `rule_name`         `varchar`
  `queue_id`          `int2`
  `queue_name`        `varchar`
  `rule_action`       `varchar`
  `metric_name`       `varchar`
  `metric_operator`   `varchar`
  `metric_value`      `float8`

## Column Descriptions

### rule_ordinal

### rule_name

### queue_id

### queue_name

### rule_action

### metric_name

### metric_operator

### metric_value
'),
  ('sf_241129', 'blocks_rows_totals_for_cluster', 'view', '# `sf_241129.blocks_rows_totals_for_cluster`

## Description

## Columns

  column             data type
  ------------------ -----------
  `nodes`            `int2`
  `partial_slices`   `int2`
  `full_slices`      `varchar`
  `sblks`            `varchar`
  `dsblks`           `varchar`
  `ublks`            `varchar`
  `tblks`            `varchar`
  `srows`            `varchar`
  `dsrows`           `varchar`
  `urows`            `varchar`
  `trows`            `varchar`

## Column Descriptions

### nodes

### partial_slices

### full_slices

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows
'),
  ('sf_241129', 'steps_totals_data_on_both_for_all_by_all', 'view', '# `sf_241129.steps_totals_data_on_both_for_all_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `varchar`
  `aggr`      `varchar`
  `bcast`     `varchar`
  `ddelete`   `varchar`
  `dist`      `varchar`
  `hash`      `varchar`
  `hjoin`     `varchar`
  `iinsert`   `varchar`
  `llimit`    `varchar`
  `mmerge`    `varchar`
  `mjoin`     `varchar`
  `nloop`     `varchar`
  `rreturn`   `varchar`
  `save`      `varchar`
  `ssort`     `varchar`
  `uunique`   `varchar`
  `wwindow`   `varchar`

## Column Descriptions

### node

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'wqueues_totals_slot_utilization_by_day', 'view', '# `lf_241129.wqueues_totals_slot_utilization_by_day`

## Description

## Columns

  column          data type
  --------------- -------------
  `queue_id`      `int2`
  `queue_name`    `varchar`
  `slot_number`   `int2`
  `start_ts`      `timestamp`
  `duration`      `interval`

## Column Descriptions

### queue_id

### queue_name

### slot_number

### start_ts

### duration
'),
  ('sf_241129', 'vacuum_totals_for_tables_by_minute', 'view', '# `sf_241129.vacuum_totals_for_tables_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### auto

### total

### dur
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_users_by_minute', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_users_by_minute`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_abort
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_statements_by_minute', 'view', '# `sf_241129.steps_stats_durations_on_both_for_statements_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `node`       `varchar`
  `step`       `varchar`
  `start_ts`   `timestamp`
  `min`        `numeric`
  `max`        `numeric`
  `med`        `numeric`
  `avg`        `numeric`
  `std`        `numeric`
  `num`        `int8`

## Column Descriptions

### stmt

### node

### step

### start_ts

### min

### max

### med

### avg

### std

### num
'),
  ('sf_241129', 'analyze_totals_for_database_by_day', 'view', '# `sf_241129.analyze_totals_for_database_by_day`

## Description

## Columns

  column          data type
  --------------- -----------
  `dbname`        `varchar`
  `start_ts`      `date`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### dbname

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_tables_by_minute', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_tables_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### ddelete

### iinsert

### scan
'),
  ('sf_241129', 'queries_compilation_stream', 'view', '# `sf_241129.queries_compilation_stream`

## Description

## Columns

  column      data type
  ----------- -----------
  `pid`       `int4`
  `xid`       `int8`
  `qid`       `int4`
  `stream`    `int2`
  `lncache`   `int2`
  `offload`   `int2`
  `total`     `int2`
  `dur`       `numeric`

## Column Descriptions

### pid

### xid

### qid

### stream

### lncache

### offload

### total

### dur
'),
  ('sf_241129', 'vacuum_totals_for_tables_by_second', 'view', '# `sf_241129.vacuum_totals_for_tables_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### auto

### total

### dur
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_schemas_by_all', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_schemas_by_all`

## Description

## Columns

  column              data type
  ------------------- -----------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('lf_241129', 'analyze_totals_for_all_by_day', 'view', '# `lf_241129.analyze_totals_for_all_by_day`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('ro_241129', 'sp_grant_versionless_sf_schema_to_group', 'procedure', '# `ro_241129.sp_grant_versionless_sf_schema_to_group`

## Description

## Arguments

  argument       data type
  -------------- ------------------
  `group_name`   `varchar( 127 )`

## Argument Descriptions

### group_name
'),
  ('sf_241129', 'objects_totals_for_user', 'view', '# `sf_241129.objects_totals_for_user`

## Description

## Columns

  column           data type
  ---------------- -----------
  `uuser`          `varchar`
  `dbs`            `int2`
  `sschemas`       `int2`
  `ttables`        `int4`
  `ttables_cols`   `int4`
  `nviews`         `int4`
  `nviews_cols`    `int4`
  `lviews`         `int4`
  `mviews`         `int4`
  `mviews_cols`    `int4`
  `funcs`          `int4`
  `procs`          `int4`

## Column Descriptions

### uuser

### dbs

### sschemas

### ttables

### ttables_cols

### nviews

### nviews_cols

### lviews

### mviews

### mviews_cols

### funcs

### procs
'),
  ('base_241129', 'fixed_pg_class', 'view', '# `base_241129.fixed_pg_class`

## Description

## Columns

  column          data type
  --------------- -----------
  `object_id`     `int4`
  `object_name`   `varchar`
  `schema_id`     `int4`
  `user_id`       `int4`
  `relnatts`      `int2`
  `relkind`       `bpchar`

## Column Descriptions

### object_id

### object_name

### schema_id

### user_id

### relnatts

### relkind
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_users_by_second', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_users_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `user_id`                      `int4`
  `user_name`                    `varchar`
  `start_ts`                     `timestamp`
  `min_leadin_duration`          `interval`
  `max_leadin_duration`          `interval`
  `med_leadin_duration`          `float4`
  `avg_leadin_duration`          `interval`
  `std_leadin_duration`          `float4`
  `sum_leadin_duration`          `interval`
  `min_wlm_leadin_duration`      `interval`
  `max_wlm_leadin_duration`      `interval`
  `med_wlm_leadin_duration`      `float4`
  `avg_wlm_leadin_duration`      `interval`
  `std_wlm_leadin_duration`      `float4`
  `sum_wlm_leadin_duration`      `interval`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `min_wlm_leadout_duration`     `interval`
  `max_wlm_leadout_duration`     `interval`
  `med_wlm_leadout_duration`     `float4`
  `avg_wlm_leadout_duration`     `interval`
  `std_wlm_leadout_duration`     `float4`
  `sum_wlm_leadout_duration`     `interval`
  `min_leadout_duration`         `interval`
  `max_leadout_duration`         `interval`
  `med_leadout_duration`         `float4`
  `avg_leadout_duration`         `interval`
  `std_leadout_duration`         `float4`
  `sum_leadout_duration`         `interval`
  `min_query_total_duration`     `interval`
  `max_query_total_duration`     `interval`
  `med_query_total_duration`     `float4`
  `avg_query_total_duration`     `interval`
  `std_query_total_duration`     `float4`
  `sum_query_total_duration`     `interval`
  `min_unaccounted_duration`     `interval`
  `max_unaccounted_duration`     `interval`
  `med_unaccounted_duration`     `float4`
  `avg_unaccounted_duration`     `interval`
  `std_unaccounted_duration`     `float4`
  `sum_unaccounted_duration`     `interval`
  `number_queries`               `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_leadin_duration

### max_leadin_duration

### med_leadin_duration

### avg_leadin_duration

### std_leadin_duration

### sum_leadin_duration

### min_wlm_leadin_duration

### max_wlm_leadin_duration

### med_wlm_leadin_duration

### avg_wlm_leadin_duration

### std_wlm_leadin_duration

### sum_wlm_leadin_duration

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### min_wlm_leadout_duration

### max_wlm_leadout_duration

### med_wlm_leadout_duration

### avg_wlm_leadout_duration

### std_wlm_leadout_duration

### sum_wlm_leadout_duration

### min_leadout_duration

### max_leadout_duration

### med_leadout_duration

### avg_leadout_duration

### std_leadout_duration

### sum_leadout_duration

### min_query_total_duration

### max_query_total_duration

### med_query_total_duration

### avg_query_total_duration

### std_query_total_duration

### sum_query_total_duration

### min_unaccounted_duration

### max_unaccounted_duration

### med_unaccounted_duration

### avg_unaccounted_duration

### std_unaccounted_duration

### sum_unaccounted_duration

### number_queries
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_all_by_minute', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_all_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### start_ts

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('sf_241129', 'transactions_stats_commits_by_minute', 'view', '# `sf_241129.transactions_stats_commits_by_minute`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `xid queuing`    `varchar`
  `bid queuing`    `varchar`
  `ldr duration`   `varchar`
  `wrk duration`   `varchar`
  `xids/bid`       `varchar`
  `blks/bid`       `varchar`
  `bids`           `int8`

## Column Descriptions

### start_ts

### xid queuing

### bid queuing

### ldr duration

### wrk duration

### xids/bid

### blks/bid

### bids
'),
  ('sf_241129', 'steps_totals_data_on_both_for_slices_by_second', 'view', '# `sf_241129.steps_totals_data_on_both_for_slices_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `slice`      `int2`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### slice

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('base_241129', 'fixed_stl_commit_stats', 'view', '# `base_241129.fixed_stl_commit_stats`

## Description

## Columns

  column             data type
  ------------------ -------------
  `batch_id`         `int8`
  `transaction_id`   `int8`
  `node_ordinal`     `int2`
  `startqueue`       `timestamp`
  `startwork`        `timestamp`
  `endflush`         `timestamp`
  `endstage`         `timestamp`
  `endlocal`         `timestamp`
  `startglobal`      `timestamp`
  `endtime`          `timestamp`
  `newblocks`        `int8`
  `dirtyblocks`      `int8`
  `headers`          `int8`

## Column Descriptions

### batch_id

### transaction_id

### node_ordinal

### startqueue

### startwork

### endflush

### endstage

### endlocal

### startglobal

### endtime

### newblocks

### dirtyblocks

### headers
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_slices_by_hour', 'view', '# `lf_241129.steps_stats_durations_on_both_for_slices_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `slice_ordinal`               `int2`
  `slice_type`                  `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### slice_ordinal

### slice_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('lf_241129', 'vacuum_stats_for_all_by_second', 'view', '# `lf_241129.vacuum_stats_for_all_by_second`

## Description

## Columns

  column                             data type
  ---------------------------------- -------------
  `start_ts`                         `timestamp`
  `min_sum_number_new_blocks`        `int8`
  `max_sum_number_new_blocks`        `int8`
  `med_sum_number_new_blocks`        `int8`
  `avg_sum_number_new_blocks`        `float4`
  `std_sum_number_new_blocks`        `float4`
  `sum_sum_number_new_blocks`        `float4`
  `min_sum_number_replaced_blocks`   `int8`
  `max_sum_number_replaced_blocks`   `int8`
  `med_sum_number_replaced_blocks`   `int8`
  `avg_sum_number_replaced_blocks`   `float4`
  `std_sum_number_replaced_blocks`   `float4`
  `sum_sum_number_replaced_blocks`   `float4`
  `min_sum_number_new_rows`          `int8`
  `max_sum_number_new_rows`          `int8`
  `med_sum_number_new_rows`          `int8`
  `avg_sum_number_new_rows`          `float4`
  `std_sum_number_new_rows`          `float4`
  `sum_sum_number_new_rows`          `float4`
  `min_sum_number_deleted_rows`      `int8`
  `max_sum_number_deleted_rows`      `int8`
  `med_sum_number_deleted_rows`      `int8`
  `avg_sum_number_deleted_rows`      `float4`
  `std_sum_number_deleted_rows`      `float4`
  `sum_sum_number_deleted_rows`      `float4`
  `min_duration`                     `interval`
  `max_duration`                     `interval`
  `med_duration`                     `float4`
  `avg_duration`                     `interval`
  `std_duration`                     `float4`
  `sum_duration`                     `interval`
  `number_qids`                      `int8`
  `number_xids`                      `int8`

## Column Descriptions

### start_ts

### min_sum_number_new_blocks

### max_sum_number_new_blocks

### med_sum_number_new_blocks

### avg_sum_number_new_blocks

### std_sum_number_new_blocks

### sum_sum_number_new_blocks

### min_sum_number_replaced_blocks

### max_sum_number_replaced_blocks

### med_sum_number_replaced_blocks

### avg_sum_number_replaced_blocks

### std_sum_number_replaced_blocks

### sum_sum_number_replaced_blocks

### min_sum_number_new_rows

### max_sum_number_new_rows

### med_sum_number_new_rows

### avg_sum_number_new_rows

### std_sum_number_new_rows

### sum_sum_number_new_rows

### min_sum_number_deleted_rows

### max_sum_number_deleted_rows

### med_sum_number_deleted_rows

### avg_sum_number_deleted_rows

### std_sum_number_deleted_rows

### sum_sum_number_deleted_rows

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_qids

### number_xids
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_all_by_all', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_all_by_all`

## Description

## Columns

  column       data type
  ------------ -----------
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('lf_241129', 'ttables_creation_for_schemas_by_day', 'view', '# `lf_241129.ttables_creation_for_schemas_by_day`

## Description

## Columns

  column            data type
  ----------------- -------------
  `schema_name`     `varchar`
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### schema_name

### start_ts

### number_tables
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_hour_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_hour_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_day_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_day_concise`

## Description

## Columns

  column       data type
  ------------ -----------
  `node`       `varchar`
  `start_ts`   `date`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'blocks_totals_for_tables_by_column', 'view', '# `lf_241129.blocks_totals_for_tables_by_column`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `schema_id`                `int4`
  `schema_name`              `varchar`
  `table_id`                 `int4`
  `table_name`               `varchar`
  `column_ordinal`           `int2`
  `column_name`              `varchar`
  `number_sorted_blocks`     `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### column_ordinal

### column_name

### number_sorted_blocks

### number_unsorted_blocks

### number_total_blocks
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_all_by_second', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'blocks_rows_totals_for_node', 'view', '# `sf_241129.blocks_rows_totals_for_node`

## Description

## Columns

  column     data type
  ---------- -----------
  `node`     `int2`
  `slices`   `varchar`
  `sblks`    `varchar`
  `dsblks`   `varchar`
  `ublks`    `varchar`
  `tblks`    `varchar`
  `srows`    `varchar`
  `dsrows`   `varchar`
  `urows`    `varchar`
  `trows`    `varchar`

## Column Descriptions

### node

### slices

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows
'),
  ('sf_241129', 'views_normal', 'view', '# `sf_241129.views_normal`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `vview`     `varchar`
  `uuser`     `varchar`

## Column Descriptions

### sschema

### vview

### uuser
'),
  ('ro_241129', 'sp_grant_version_to_user', 'procedure', '# `ro_241129.sp_grant_version_to_user`

## Description

## Arguments

  argument      data type
  ------------- ------------------
  `version`     `varchar( 6 )`
  `user_name`   `varchar( 127 )`

## Argument Descriptions

### version

### user_name
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_users_by_minute', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_users_by_minute`

## Description

## Columns

  column                data type
  --------------------- -------------
  `user_id`             `int4`
  `node_type`           `varchar`
  `user_name`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### user_id

### node_type

### user_name

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_tables_by_second', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_tables_by_second`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `table_id`          `int4`
  `table_name`        `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('lf_241129', 'steps_totals_data_on_both_for_users_by_all', 'view', '# `lf_241129.steps_totals_data_on_both_for_users_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `user_id`                      `int4`
  `user_name`                    `varchar`
  `node_type`                    `varchar`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### user_id

### user_name

### node_type

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_statements_by_second', 'view', '# `sf_241129.steps_stats_durations_on_both_for_statements_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `node`       `varchar`
  `step`       `varchar`
  `start_ts`   `timestamp`
  `min`        `numeric`
  `max`        `numeric`
  `med`        `numeric`
  `avg`        `numeric`
  `std`        `numeric`
  `num`        `int8`

## Column Descriptions

### stmt

### node

### step

### start_ts

### min

### max

### med

### avg

### std

### num
'),
  ('lf_241129', 'objects_dependencies', 'view', '# `lf_241129.objects_dependencies`

## Description

## Columns

  column                 data type
  ---------------------- -----------
  `dependency_depth`     `int2`
  `parent_owner_id`      `int4`
  `parent_owner_name`    `varchar`
  `parent_schema_id`     `int4`
  `parent_schema_name`   `varchar`
  `parent_object_type`   `varchar`
  `parent_object_id`     `int4`
  `parent_object_name`   `varchar`
  `child_owner_id`       `int4`
  `child_owner_name`     `varchar`
  `child_schema_id`      `int4`
  `child_schema_name`    `varchar`
  `child_object_type`    `varchar`
  `child_object_id`      `int4`
  `child_object_name`    `varchar`

## Column Descriptions

### dependency_depth

### parent_owner_id

### parent_owner_name

### parent_schema_id

### parent_schema_name

### parent_object_type

### parent_object_id

### parent_object_name

### child_owner_id

### child_owner_name

### child_schema_id

### child_schema_name

### child_object_type

### child_object_id

### child_object_name
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_all_by_all', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_all_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `varchar`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### node

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_statements_by_all', 'view', '# `sf_241129.steps_stats_durations_on_both_for_statements_by_all`

## Description

## Columns

  column   data type
  -------- -----------
  `stmt`   `varchar`
  `node`   `varchar`
  `step`   `varchar`
  `min`    `numeric`
  `max`    `numeric`
  `med`    `numeric`
  `avg`    `numeric`
  `std`    `numeric`
  `num`    `int8`

## Column Descriptions

### stmt

### node

### step

### min

### max

### med

### avg

### std

### num
'),
  ('lf_241129', 'queries_step_plan', 'view', '# `lf_241129.queries_step_plan`

## Description

## Columns

  column                       data type
  ---------------------------- -------------
  `query_id`                   `int4`
  `stream_ordinal`             `int2`
  `segment_ordinal`            `int2`
  `step_ordinal`               `int2`
  `node_ordinal`               `int2`
  `slice_ordinal`              `int2`
  `slice_type`                 `varchar`
  `step_type`                  `varchar`
  `sub_type`                   `varchar`
  `number_materialized_rows`   `int8`
  `number_eligible_rows`       `int8`
  `number_rows`                `int8`
  `number_bytes`               `int8`
  `number_packets`             `int8`
  `store_type`                 `varchar`
  `step_start_ts`              `timestamp`
  `step_end_ts`                `timestamp`
  `segment_duration`           `interval`
  `schema_id`                  `int4`
  `schema_name`                `varchar`
  `table_id`                   `int4`
  `table_name`                 `varchar`

## Column Descriptions

### query_id

### stream_ordinal

### segment_ordinal

### step_ordinal

### node_ordinal

### slice_ordinal

### slice_type

### step_type

### sub_type

### number_materialized_rows

### number_eligible_rows

### number_rows

### number_bytes

### number_packets

### store_type

### step_start_ts

### step_end_ts

### segment_duration

### schema_id

### schema_name

### table_id

### table_name
'),
  ('base_241129', 'fixed_stv_wlm_qmr_config', 'view', '# `base_241129.fixed_stv_wlm_qmr_config`

## Description

## Columns

  column              data type
  ------------------- -----------
  `rule_action`       `varchar`
  `action_value`      `varchar`
  `metric_name`       `varchar`
  `metric_operator`   `varchar`
  `metric_value`      `float8`
  `rule_name`         `varchar`
  `queue_id`          `int2`

## Column Descriptions

### rule_action

### action_value

### metric_name

### metric_operator

### metric_value

### rule_name

### queue_id
'),
  ('lf_241129', 'vacuum_stats_for_all_by_all', 'view', '# `lf_241129.vacuum_stats_for_all_by_all`

## Description

## Columns

  column                             data type
  ---------------------------------- ------------
  `min_sum_number_new_blocks`        `int8`
  `max_sum_number_new_blocks`        `int8`
  `med_sum_number_new_blocks`        `int8`
  `avg_sum_number_new_blocks`        `float4`
  `std_sum_number_new_blocks`        `float4`
  `sum_sum_number_new_blocks`        `float4`
  `min_sum_number_replaced_blocks`   `int8`
  `max_sum_number_replaced_blocks`   `int8`
  `med_sum_number_replaced_blocks`   `int8`
  `avg_sum_number_replaced_blocks`   `float4`
  `std_sum_number_replaced_blocks`   `float4`
  `sum_sum_number_replaced_blocks`   `float4`
  `min_sum_number_new_rows`          `int8`
  `max_sum_number_new_rows`          `int8`
  `med_sum_number_new_rows`          `int8`
  `avg_sum_number_new_rows`          `float4`
  `std_sum_number_new_rows`          `float4`
  `sum_sum_number_new_rows`          `float4`
  `min_sum_number_deleted_rows`      `int8`
  `max_sum_number_deleted_rows`      `int8`
  `med_sum_number_deleted_rows`      `int8`
  `avg_sum_number_deleted_rows`      `float4`
  `std_sum_number_deleted_rows`      `float4`
  `sum_sum_number_deleted_rows`      `float4`
  `min_duration`                     `interval`
  `max_duration`                     `interval`
  `med_duration`                     `float4`
  `avg_duration`                     `interval`
  `std_duration`                     `float4`
  `sum_duration`                     `interval`
  `number_qids`                      `int8`
  `number_xids`                      `int8`

## Column Descriptions

### min_sum_number_new_blocks

### max_sum_number_new_blocks

### med_sum_number_new_blocks

### avg_sum_number_new_blocks

### std_sum_number_new_blocks

### sum_sum_number_new_blocks

### min_sum_number_replaced_blocks

### max_sum_number_replaced_blocks

### med_sum_number_replaced_blocks

### avg_sum_number_replaced_blocks

### std_sum_number_replaced_blocks

### sum_sum_number_replaced_blocks

### min_sum_number_new_rows

### max_sum_number_new_rows

### med_sum_number_new_rows

### avg_sum_number_new_rows

### std_sum_number_new_rows

### sum_sum_number_new_rows

### min_sum_number_deleted_rows

### max_sum_number_deleted_rows

### med_sum_number_deleted_rows

### avg_sum_number_deleted_rows

### std_sum_number_deleted_rows

### sum_sum_number_deleted_rows

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_qids

### number_xids
'),
  ('sf_241129', 'uusers', 'view', '# `sf_241129.uusers`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `ttype`      `varchar`
  `ssyslog`    `varchar`
  `dbs`        `int2`
  `funcs`      `int4`
  `procs`      `int4`
  `sschemas`   `int2`
  `ttables`    `int4`
  `nviews`     `int4`
  `lviews`     `int4`
  `mviews`     `int4`

## Column Descriptions

### uuser

### ttype

### ssyslog

### dbs

### funcs

### procs

### sschemas

### ttables

### nviews

### lviews

### mviews
'),
  ('lf_241129', 'analyze_totals_for_database_by_all', 'view', '# `lf_241129.analyze_totals_for_database_by_all`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -----------
  `database_name`                                 `varchar`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### database_name

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'privileges_granted_to_groups', 'view', '# `lf_241129.privileges_granted_to_groups`

## Description

## Columns

  column                                   data type
  ---------------------------------------- -----------
  `group_id`                               `int4`
  `group_name`                             `varchar`
  `object_owner_user_id`                   `int4`
  `object_owner_user_name`                 `varchar`
  `schema_id`                              `int4`
  `schema_name`                            `varchar`
  `object_type`                            `varchar`
  `object_return_base_data_type_id`        `int4`
  `object_return_base_data_type_name`      `varchar`
  `object_id`                              `int4`
  `object_name`                            `varchar`
  `object_argument_base_data_type_ids`     `varchar`
  `object_argument_base_data_type_names`   `varchar`
  `priv_database_create`                   `bool`
  `priv_database_temp`                     `bool`
  `priv_function_execute`                  `bool`
  `priv_language_usage`                    `bool`
  `priv_procedure_execute`                 `bool`
  `priv_schema_create`                     `bool`
  `priv_schema_usage`                      `bool`
  `priv_table_delete`                      `bool`
  `priv_table_drop`                        `bool`
  `priv_table_insert`                      `bool`
  `priv_table_reference`                   `bool`
  `priv_table_select`                      `bool`
  `priv_table_update`                      `bool`
  `priv_view_drop`                         `bool`
  `priv_view_select`                       `bool`

## Column Descriptions

### group_id

### group_name

### object_owner_user_id

### object_owner_user_name

### schema_id

### schema_name

### object_type

### object_return_base_data_type_id

### object_return_base_data_type_name

### object_id

### object_name

### object_argument_base_data_type_ids

### object_argument_base_data_type_names

### priv_database_create

### priv_database_temp

### priv_function_execute

### priv_language_usage

### priv_procedure_execute

### priv_schema_create

### priv_schema_usage

### priv_table_delete

### priv_table_drop

### priv_table_insert

### priv_table_reference

### priv_table_select

### priv_table_update

### priv_view_drop

### priv_view_select
'),
  ('lf_241129', 'transactions_totals_commits_by_hour', 'view', '# `lf_241129.transactions_totals_commits_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `sum_batch_leader_duration`   `interval`
  `sum_batch_worker_duration`   `interval`
  `sum_batch_idle_duration`     `interval`
  `sum_batch_exec_duration`     `interval`

## Column Descriptions

### start_ts

### sum_batch_leader_duration

### sum_batch_worker_duration

### sum_batch_idle_duration

### sum_batch_exec_duration
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_queues_by_second', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_queues_by_second`

## Description

## Columns

  column                data type
  --------------------- -------------
  `queue_id`            `int2`
  `queue_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### queue_id

### queue_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_all_by_second', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_all_by_second`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_users_by_all', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_users_by_all`

## Description

## Columns

  column             data type
  ------------------ ------------
  `user_id`          `int4`
  `user_name`        `varchar`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### user_id

### user_name

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('sf_241129', 'analyze_totals_status_for_all', 'view', '# `sf_241129.analyze_totals_status_for_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `good`      `int4`
  `stale`     `int4`
  `missing`   `int4`
  `total`     `int4`

## Column Descriptions

### good

### stale

### missing

### total
'),
  ('sf_241129', 'sschemas', 'view', '# `sf_241129.sschemas`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `uuser`     `varchar`
  `ttables`   `int4`
  `nviews`    `int4`
  `lviews`    `int4`
  `mviews`    `int4`
  `funcs`     `int4`
  `procs`     `int4`

## Column Descriptions

### sschema

### uuser

### ttables

### nviews

### lviews

### mviews

### funcs

### procs
'),
  ('sf_241129', 'analyze_totals_for_tables_by_minute', 'view', '# `sf_241129.analyze_totals_for_tables_by_minute`

## Description

## Columns

  column          data type
  --------------- -------------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_users_by_hour', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_users_by_hour`

## Description

## Columns

  column             data type
  ------------------ -------------
  `user_id`          `int4`
  `user_name`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_all_by_all', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_all_by_all`

## Description

## Columns

  column     data type
  ---------- -----------
  `aborts`   `int4`

## Column Descriptions

### aborts
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_users_by_minute', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### uuser

### start_ts

### duration

### queries
'),
  ('sf_241129', 'wqueues_totals_slot_utilization_by_all', 'view', '# `sf_241129.wqueues_totals_slot_utilization_by_all`

## Description

## Columns

  column    data type
  --------- -----------
  `qname`   `varchar`
  `slot`    `int2`
  `dur`     `numeric`

## Column Descriptions

### qname

### slot

### dur
'),
  ('lf_241129', 'vacuum_totals_for_all_by_second', 'view', '# `lf_241129.vacuum_totals_for_all_by_second`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('lf_241129', 'queries_for_all', 'view', '# `lf_241129.queries_for_all`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `user_id`                       `int4`
  `user_name`                     `varchar`
  `session_id`                    `int4`
  `transaction_id`                `int8`
  `query_id`                      `int4`
  `queue_id`                      `int2`
  `queue_name`                    `varchar`
  `query_start_ts`                `timestamp`
  `query_end_ts`                  `timestamp`
  `query_queue_duration`          `interval`
  `query_compile_exec_duration`   `interval`
  `query_wlm_duration`            `interval`
  `query_total_duration`          `interval`
  `query_abort_flag`              `bool`
  `query_csc_flag`                `bool`
  `query_leader_node_flag`        `bool`
  `query_rc_flag`                 `bool`
  `query_text`                    `varchar`

## Column Descriptions

### user_id

### user_name

### session_id

### transaction_id

### query_id

### queue_id

### queue_name

### query_start_ts

### query_end_ts

### query_queue_duration

### query_compile_exec_duration

### query_wlm_duration

### query_total_duration

### query_abort_flag

### query_csc_flag

### query_leader_node_flag

### query_rc_flag

### query_text
'),
  ('sf_241129', 'pprocedures', 'view', '# `sf_241129.pprocedures`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `proc`      `varchar`
  `args`      `varchar`
  `uuser`     `varchar`
  `rao`       `varchar`

## Column Descriptions

### sschema

### proc

### args

### uuser

### rao
'),
  ('sf_241129', 'steps_totals_data_on_both_for_all_by_day', 'view', '# `sf_241129.steps_totals_data_on_both_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `node`       `varchar`
  `start_ts`   `date`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'analyze_totals_for_database_by_minute', 'view', '# `lf_241129.analyze_totals_for_database_by_minute`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `database_name`                                 `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### database_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'sschemas_blocks_rows_activity', 'view', '# `lf_241129.sschemas_blocks_rows_activity`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `user_id`                     `int4`
  `user_name`                   `varchar`
  `number_tables`               `int4`
  `number_normal_views`         `int4`
  `number_late_binding_views`   `int4`
  `number_materialized_views`   `int4`
  `number_functions`            `int4`
  `number_procedures`           `int4`
  `number_sorted_blocks`        `int8`
  `number_desorted_blocks`      `int8`
  `number_unsorted_blocks`      `int8`
  `number_total_blocks`         `int8`
  `number_sorted_rows`          `int8`
  `number_desorted_rows`        `int8`
  `number_unsorted_rows`        `int8`
  `number_total_rows`           `int8`
  `number_rows_deleted`         `int8`
  `number_rows_inserted`        `int8`

## Column Descriptions

### schema_id

### schema_name

### user_id

### user_name

### number_tables

### number_normal_views

### number_late_binding_views

### number_materialized_views

### number_functions

### number_procedures

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows

### number_rows_deleted

### number_rows_inserted
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_all_by_all', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_all_by_all`

## Description

## Columns

  column             data type
  ------------------ ------------
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_queues_by_minute', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_queues_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### qname

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'groups_users', 'view', '# `sf_241129.groups_users`

## Description

## Columns

  column         data type
  -------------- -----------
  `group_name`   `varchar`
  `user_name`    `varchar`

## Column Descriptions

### group_name

### user_name
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_all_by_second', 'view', '# `lf_241129.steps_stats_durations_on_both_for_all_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_queues_by_hour', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_queues_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### qname

### start_ts

### aborts
'),
  ('sf_241129', 'steps_totals_data_on_both_for_node_by_day', 'view', '# `sf_241129.steps_totals_data_on_both_for_node_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `node`       `int2`
  `start_ts`   `date`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_queues_by_minute', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_queues_by_minute`

## Description

## Columns

  column                data type
  --------------------- -------------
  `queue_id`            `int2`
  `queue_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### queue_id

### queue_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_all_by_all', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_all_by_all`

## Description

## Columns

  column                           data type
  -------------------------------- -----------
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('lf_241129', 'queries_totals_result_cache_for_users_by_day', 'view', '# `lf_241129.queries_totals_result_cache_for_users_by_day`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_all_by_day', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_all_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('lf_241129', 'queries_text_on_both_full', 'view', '# `lf_241129.queries_text_on_both_full`

## Description

## Columns

  column               data type
  -------------------- -------------
  `session_id`         `int4`
  `transaction_id`     `int8`
  `query_id`           `int4`
  `query_start_ts`     `timestamp`
  `query_text_order`   `int4`
  `query_text`         `varchar`

## Column Descriptions

### session_id

### transaction_id

### query_id

### query_start_ts

### query_text_order

### query_text
'),
  ('lf_241129', 'utils_queries_benchmark', 'view', '# `lf_241129.utils_queries_benchmark`

## Description

This view provides an easy-to-use method to correctly and accurately
benchmark queries.

First, be aware that computing the execution time taken by a query is
*not* straightforward - I\'ll go into how it\'s done, but for now just
say - it is here done as correctly as is possible in Redshift (I say "as
possible", because in fact, right now, it is not possible to canonically
know the execution time of a query, due to flaws in the system tables).

Usage is as with all `sf` views, simple;

> `select * from sf.queries_benchmark;`

You then get five lines of output - for your five most recent queries.

     rn |    qid    |          start_ts          |  dur  |  min  |  max  |  avg  |  std  |                  text                   
    ----+-----------+----------------------------+-------+-------+-------+-------+-------+-----------------------------------------
      1 | 740502166 | 2024-10-29 20:58:08.085074 |  1.87 |  1.87 | 96.72 | 75.66 | 36.93 | select * from sf.vacuum_by_user_by_day;
      2 | 741840588 | 2024-10-30 07:53:59.909107 | 96.72 | 92.31 | 96.72 | 94.11 |  1.68 | select * from sf.queries_benchmark;
      3 | 741894382 | 2024-10-30 08:17:15.535093 | 93.05 | 92.31 | 94.36 | 93.24 |  0.85 | select * from sf.queries_benchmark;
      4 | 741909558 | 2024-10-30 08:23:32.480694 | 92.31 | 92.31 | 94.36 | 93.34 |  1.02 | select * from sf.queries_benchmark;
      5 | 741929643 | 2024-10-30 08:32:17.004436 | 94.36 | 94.36 | 94.36 | 94.36 |  0.00 | select * from sf.queries_benchmark;
    (5 rows)

So, what\'s this all about?

It\'s about statistics.

Each row is for one query, of the five most queries you have issued.

The `dur`, which is `duration`, column tells you how long each query
took to execute.

The other columns however are the stats for that query *and every query
before it*.

So you now have the most easy way to simply run a given query five times
in a row, and get the `min / man / avg / std` stats for all five.

Or maybe you just run three, and that\'s enough.

There\'s no median, because median turns out to be problematic to compute
in this situation.

Now I need to explain how query execution time is computed.

Query duration is hard to get right in Redshift. The docs do provide
anything like the information to get it right, and getting it right has
some complexity, and in fact in principle is currently impossible,
although in practise it will be normally be fine.

There are three basic problems.

1.  Redshift includes query compilation time in execution time.
2.  We want to avoid including the time it takes return rows from the
    query to the client.
3.  We want to avoid query queuing time.

So I handle these. The query execution time is from the moment the query
begins to run (rather than when it arrives at Redshift), compilation is
subtracted from execution time, and the end of the timed period is the
very beginning of when Redshift starts to return rows to the user.

here is one, and unfixable, problem, but I think it is rare.

When Redshift is compiling query A, if it sees query B, which is
compiling at the same time and which will produce the output query A
needs, it will stall query A compilation until query B finishes, and
then use the output from query B. The problem is there is absolutely no
way to know when this happens, and it looks like execution time.

## Columns

  column                data type
  --------------------- -------------
  `rn`                  `int2`
  `query_start_ts`      `timestamp`
  `query_id`            `int4`
  `exec_duration`       `float4`
  `min_exec_duration`   `float4`
  `max_exec_duration`   `float4`
  `avg_exec_duration`   `float4`
  `std_exec_duration`   `float4`
  `query_text`          `varchar`

## Column Descriptions

### rn

### query_start_ts

### query_id

### exec_duration

### min_exec_duration

### max_exec_duration

### avg_exec_duration

### std_exec_duration

### query_text
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_all_by_second', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_all_by_second`

## Description

## Columns

  column             data type
  ------------------ -------------
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('lf_241129', 'sessions_full', 'view', '# `lf_241129.sessions_full`

## Description

## Columns

  column                      data type
  --------------------------- ------------
  `user_id`                   `int4`
  `user_name`                 `varchar`
  `session_id`                `int4`
  `truncated_database_name`   `varchar`
  `session_duration`          `interval`
  `remotehost`                `varchar`
  `remoteport`                `int4`
  `number_locks`              `int4`

## Column Descriptions

### user_id

### user_name

### session_id

### truncated_database_name

### session_duration

### remotehost

### remoteport

### number_locks
'),
  ('sf_241129', 'wqueues_memory', 'view', '# `sf_241129.wqueues_memory`

## Description

## Columns

  column         data type
  -------------- -----------
  `qname`        `varchar`
  `slots`        `int2`
  `ra3_xlplus`   `int4`
  `ra3_4x`       `int4`
  `ra3_16x`      `int4`
  `dc2_min`      `int4`
  `dc2_max`      `int4`

## Column Descriptions

### qname

### slots

### ra3_xlplus

### ra3_4x

### ra3_16x

### dc2_min

### dc2_max
'),
  ('lf_241129', 'steps_totals_data_on_both_for_users_by_day', 'view', '# `lf_241129.steps_totals_data_on_both_for_users_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `usesysid`                     `int4`
  `user_name`                    `varchar`
  `node_type`                    `varchar`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### usesysid

### user_name

### node_type

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'transactions_stats_commits_by_minute', 'view', '# `lf_241129.transactions_stats_commits_by_minute`

## Description

## Columns

  column                                data type
  ------------------------------------- -------------
  `start_ts`                            `timestamp`
  `min_transaction_batch_queue_start`   `interval`
  `max_transaction_batch_queue_start`   `interval`
  `med_transaction_batch_queue_start`   `float4`
  `avg_transaction_batch_queue_start`   `interval`
  `std_transaction_batch_queue_start`   `float4`
  `sum_transaction_batch_queue_start`   `interval`
  `min_batch_queue_duration`            `interval`
  `max_batch_queue_duration`            `interval`
  `med_batch_queue_duration`            `float4`
  `avg_batch_queue_duration`            `interval`
  `std_batch_queue_duration`            `float4`
  `sum_batch_queue_duration`            `interval`
  `min_batch_leader_duration`           `interval`
  `max_batch_leader_duration`           `interval`
  `med_batch_leader_duration`           `float4`
  `avg_batch_leader_duration`           `interval`
  `std_batch_leader_duration`           `float4`
  `sum_batch_leader_duration`           `interval`
  `min_batch_worker_duration`           `interval`
  `max_batch_worker_duration`           `interval`
  `med_batch_worker_duration`           `float4`
  `avg_batch_worker_duration`           `interval`
  `std_batch_worker_duration`           `float4`
  `sum_batch_worker_duration`           `interval`
  `min_number_batch_transactions`       `int8`
  `max_number_batch_transactions`       `int8`
  `med_number_batch_transactions`       `int8`
  `avg_number_batch_transactions`       `float4`
  `std_number_batch_transactions`       `float4`
  `sum_number_batch_transactions`       `int8`
  `min_number_batch_blocks`             `int8`
  `max_number_batch_blocks`             `int8`
  `med_number_batch_blocks`             `int8`
  `avg_number_batch_blocks`             `float4`
  `std_number_batch_blocks`             `float4`
  `sum_number_batch_blocks`             `int8`
  `number_batches`                      `int8`

## Column Descriptions

### start_ts

### min_transaction_batch_queue_start

### max_transaction_batch_queue_start

### med_transaction_batch_queue_start

### avg_transaction_batch_queue_start

### std_transaction_batch_queue_start

### sum_transaction_batch_queue_start

### min_batch_queue_duration

### max_batch_queue_duration

### med_batch_queue_duration

### avg_batch_queue_duration

### std_batch_queue_duration

### sum_batch_queue_duration

### min_batch_leader_duration

### max_batch_leader_duration

### med_batch_leader_duration

### avg_batch_leader_duration

### std_batch_leader_duration

### sum_batch_leader_duration

### min_batch_worker_duration

### max_batch_worker_duration

### med_batch_worker_duration

### avg_batch_worker_duration

### std_batch_worker_duration

### sum_batch_worker_duration

### min_number_batch_transactions

### max_number_batch_transactions

### med_number_batch_transactions

### avg_number_batch_transactions

### std_number_batch_transactions

### sum_number_batch_transactions

### min_number_batch_blocks

### max_number_batch_blocks

### med_number_batch_blocks

### avg_number_batch_blocks

### std_number_batch_blocks

### sum_number_batch_blocks

### number_batches
'),
  ('lf_241129', 'pprocedures', 'view', '# `lf_241129.pprocedures`

## Description

## Columns

  column                            data type
  --------------------------------- -----------
  `schema_id`                       `int4`
  `schema_name`                     `varchar`
  `procid`                          `int4`
  `procname`                        `varchar`
  `argument_base_data_type_ids`     `varchar`
  `argument_base_data_type_names`   `varchar`
  `user_id`                         `int4`
  `user_name`                       `varchar`
  `run_as_owner`                    `bool`

## Column Descriptions

### schema_id

### schema_name

### procid

### procname

### argument_base_data_type_ids

### argument_base_data_type_names

### user_id

### user_name

### run_as_owner
'),
  ('base_241129', 'fixed_pg_language', 'view', '# `base_241129.fixed_pg_language`

## Description

## Columns

  column            data type
  ----------------- -----------
  `language_id`     `int4`
  `language_name`   `varchar`
  `user_id`         `int4`

## Column Descriptions

### language_id

### language_name

### user_id
'),
  ('lf_241129', 'steps_totals_data_on_both_for_slices_by_second', 'view', '# `lf_241129.steps_totals_data_on_both_for_slices_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `slice_ordinal`                `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### slice_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_slices_by_all', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_slices_by_all`

## Description

## Columns

  column                data type
  --------------------- -----------
  `node_ordinal`        `int2`
  `slice_ordinal`       `int2`
  `slice_type`          `varchar`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'queries_totals_result_cache_for_users_by_minute', 'view', '# `lf_241129.queries_totals_result_cache_for_users_by_minute`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('lf_241129', 'ttables_stats_dists_sorts_for_all_by_count', 'view', '# `lf_241129.ttables_stats_dists_sorts_for_all_by_count`

## Description

## Columns

  column               data type
  -------------------- -----------
  `all_compound`       `int4`
  `all_interleaved`    `int4`
  `all_unsorted`       `int4`
  `even_compound`      `int4`
  `even_interleaved`   `int4`
  `even_unsorted`      `int4`
  `key_compound`       `int4`
  `key_interleaved`    `int4`
  `key_unsorted`       `int4`

## Column Descriptions

### all_compound

### all_interleaved

### all_unsorted

### even_compound

### even_interleaved

### even_unsorted

### key_compound

### key_interleaved

### key_unsorted
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_all_by_second', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### start_ts

### ddelete

### iinsert

### scan
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_node_by_all', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_node_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `n`         `int2`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### n

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'vacuum_totals_for_all_by_day', 'view', '# `lf_241129.vacuum_totals_for_all_by_day`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_statements_by_day', 'view', '# `lf_241129.steps_stats_durations_on_both_for_statements_by_day`

## Description

## Columns

  column             data type
  ------------------ -------------
  `statement_name`   `varchar`
  `node_type`        `varchar`
  `step_type`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `number_steps`     `int8`

## Column Descriptions

### statement_name

### node_type

### step_type

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### number_steps
'),
  ('lf_241129', 'queries_totals_result_cache_for_all_by_all', 'view', '# `lf_241129.queries_totals_result_cache_for_all_by_all`

## Description

## Columns

  column           data type
  ---------------- -----------
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### number_hit

### number_miss

### number_total

### percentage
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_users_by_day', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_users_by_day`

## Description

## Columns

  column                data type
  --------------------- -------------
  `user_id`             `int4`
  `node_type`           `varchar`
  `user_name`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### user_id

### node_type

### user_name

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'vacuum_totals_for_all_by_minute', 'view', '# `lf_241129.vacuum_totals_for_all_by_minute`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### start_ts

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_all_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_all_concise`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `node`      `varchar`
  `aggr`      `numeric`
  `bcast`     `numeric`
  `del`       `numeric`
  `dist`      `numeric`
  `hash`      `numeric`
  `hjoin`     `numeric`
  `iinsert`   `numeric`
  `llimit`    `numeric`
  `mmerge`    `numeric`
  `mjoin`     `numeric`
  `nloop`     `numeric`
  `rreturn`   `numeric`
  `save`      `numeric`
  `ssort`     `numeric`
  `uunique`   `numeric`
  `wwindow`   `numeric`

## Column Descriptions

### uuser

### node

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_tables_by_minute_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_tables_by_minute_complete`

## Description

## Columns

  column          data type
  --------------- -------------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `node`          `varchar`
  `start_ts`      `timestamp`
  `aggr`          `varchar`
  `ddelete`       `varchar`
  `hashjoin`      `varchar`
  `iinsert`       `varchar`
  `mergejoin`     `varchar`
  `nestloop_op`   `varchar`
  `save`          `varchar`
  `ssort`         `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### aggr

### ddelete

### hashjoin

### iinsert

### mergejoin

### nestloop_op

### save

### ssort
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_users_by_day', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `start_ts`   `date`
  `aborts`     `int4`

## Column Descriptions

### uuser

### start_ts

### aborts
'),
  ('lf_241129', 'ffunctions_text', 'view', '# `lf_241129.ffunctions_text`

## Description

## Columns

  column            data type
  ----------------- -----------
  `schema_id`       `int4`
  `schema_name`     `varchar`
  `function_id`     `int4`
  `function_name`   `varchar`
  `function_text`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### function_id

### function_name

### function_text
'),
  ('lf_241129', 'transactions_totals_commits_by_day', 'view', '# `lf_241129.transactions_totals_commits_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `sum_batch_leader_duration`   `interval`
  `sum_batch_worker_duration`   `interval`
  `sum_batch_idle_duration`     `interval`
  `sum_batch_exec_duration`     `interval`

## Column Descriptions

### start_ts

### sum_batch_leader_duration

### sum_batch_worker_duration

### sum_batch_idle_duration

### sum_batch_exec_duration
'),
  ('lf_241129', 'analyze_totals_for_users_by_second', 'view', '# `lf_241129.analyze_totals_for_users_by_second`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `user_id`                                       `int4`
  `user_name`                                     `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('base_241129', 'fixed_pg_attribute_for_views', 'view', '# `base_241129.fixed_pg_attribute_for_views`

## Description

## Columns

  column                  data type
  ----------------------- -----------
  `view_id`               `int4`
  `column_name`           `varchar`
  `column_ordinal`        `int2`
  `atttypmod`             `int4`
  `column_notnull_flag`   `bool`

## Column Descriptions

### view_id

### column_name

### column_ordinal

### atttypmod

### column_notnull_flag
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_schemas_by_day', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_schemas_by_day`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('base_241129', 'transactions_compilation', 'view', '# `base_241129.transactions_compilation`

## Description

## Columns

  column                           data type
  -------------------------------- ------------
  `user_id`                        `int4`
  `transaction_id`                 `int8`
  `transaction_compile_duration`   `interval`

## Column Descriptions

### user_id

### transaction_id

### transaction_compile_duration
'),
  ('lf_241129', 'transactions_stats_commits_by_all', 'view', '# `lf_241129.transactions_stats_commits_by_all`

## Description

## Columns

  column                                data type
  ------------------------------------- ------------
  `min_transaction_batch_queue_start`   `interval`
  `max_transaction_batch_queue_start`   `interval`
  `med_transaction_batch_queue_start`   `float4`
  `avg_transaction_batch_queue_start`   `interval`
  `std_transaction_batch_queue_start`   `float4`
  `sum_transaction_batch_queue_start`   `interval`
  `min_batch_queue_duration`            `interval`
  `max_batch_queue_duration`            `interval`
  `med_batch_queue_duration`            `float4`
  `avg_batch_queue_duration`            `interval`
  `std_batch_queue_duration`            `float4`
  `sum_batch_queue_duration`            `interval`
  `min_batch_leader_duration`           `interval`
  `max_batch_leader_duration`           `interval`
  `med_batch_leader_duration`           `float4`
  `avg_batch_leader_duration`           `interval`
  `std_batch_leader_duration`           `float4`
  `sum_batch_leader_duration`           `interval`
  `min_batch_worker_duration`           `interval`
  `max_batch_worker_duration`           `interval`
  `med_batch_worker_duration`           `float4`
  `avg_batch_worker_duration`           `interval`
  `std_batch_worker_duration`           `float4`
  `sum_batch_worker_duration`           `interval`
  `min_number_batch_transactions`       `int8`
  `max_number_batch_transactions`       `int8`
  `med_number_batch_transactions`       `int8`
  `avg_number_batch_transactions`       `float4`
  `std_number_batch_transactions`       `float4`
  `sum_number_batch_transactions`       `int8`
  `min_number_batch_blocks`             `int8`
  `max_number_batch_blocks`             `int8`
  `med_number_batch_blocks`             `int8`
  `avg_number_batch_blocks`             `float4`
  `std_number_batch_blocks`             `float4`
  `sum_number_batch_blocks`             `int8`
  `number_batches`                      `int8`

## Column Descriptions

### min_transaction_batch_queue_start

### max_transaction_batch_queue_start

### med_transaction_batch_queue_start

### avg_transaction_batch_queue_start

### std_transaction_batch_queue_start

### sum_transaction_batch_queue_start

### min_batch_queue_duration

### max_batch_queue_duration

### med_batch_queue_duration

### avg_batch_queue_duration

### std_batch_queue_duration

### sum_batch_queue_duration

### min_batch_leader_duration

### max_batch_leader_duration

### med_batch_leader_duration

### avg_batch_leader_duration

### std_batch_leader_duration

### sum_batch_leader_duration

### min_batch_worker_duration

### max_batch_worker_duration

### med_batch_worker_duration

### avg_batch_worker_duration

### std_batch_worker_duration

### sum_batch_worker_duration

### min_number_batch_transactions

### max_number_batch_transactions

### med_number_batch_transactions

### avg_number_batch_transactions

### std_number_batch_transactions

### sum_number_batch_transactions

### min_number_batch_blocks

### max_number_batch_blocks

### med_number_batch_blocks

### avg_number_batch_blocks

### std_number_batch_blocks

### sum_number_batch_blocks

### number_batches
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_tables_by_hour', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_tables_by_hour`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `table_id`          `int4`
  `table_name`        `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('lf_241129', 'queries_totals_result_cache_for_all_by_second', 'view', '# `lf_241129.queries_totals_result_cache_for_all_by_second`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_queues_by_minute', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_queues_by_minute`

## Description

## Columns

  column           data type
  ---------------- -------------
  `queue_id`       `int2`
  `queue_name`     `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### number_abort
'),
  ('sf_241129', 'steps_totals_data_on_both_for_statements_by_all', 'view', '# `sf_241129.steps_totals_data_on_both_for_statements_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `stmt`      `varchar`
  `node`      `varchar`
  `aggr`      `varchar`
  `bcast`     `varchar`
  `ddelete`   `varchar`
  `dist`      `varchar`
  `hash`      `varchar`
  `hjoin`     `varchar`
  `iinsert`   `varchar`
  `llimit`    `varchar`
  `mmerge`    `varchar`
  `mjoin`     `varchar`
  `nloop`     `varchar`
  `rreturn`   `varchar`
  `save`      `varchar`
  `ssort`     `varchar`
  `uunique`   `varchar`
  `wwindow`   `varchar`

## Column Descriptions

### stmt

### node

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'ttables_stats_dists_sorts_for_all_by_blocks', 'view', '# `lf_241129.ttables_stats_dists_sorts_for_all_by_blocks`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `all_compound_sorted`         `int8`
  `all_interleaved_sorted`      `int8`
  `all_unsorted_sorted`         `int8`
  `all_compound_unsorted`       `int8`
  `all_interleaved_unsorted`    `int8`
  `all_unsorted_unsorted`       `int8`
  `all_compound_total`          `int8`
  `all_interleaved_total`       `int8`
  `all_unsorted_total`          `int8`
  `even_compound_sorted`        `int8`
  `even_interleaved_sorted`     `int8`
  `even_unsorted_sorted`        `int8`
  `even_compound_unsorted`      `int8`
  `even_interleaved_unsorted`   `int8`
  `even_unsorted_unsorted`      `int8`
  `even_compound_total`         `int8`
  `even_interleaved_total`      `int8`
  `even_unsorted_total`         `int8`
  `key_compound_sorted`         `int8`
  `key_interleaved_sorted`      `int8`
  `key_unsorted_sorted`         `int8`
  `key_compound_unsorted`       `int8`
  `key_interleaved_unsorted`    `int8`
  `key_unsorted_unsorted`       `int8`
  `key_compound_total`          `int8`
  `key_interleaved_total`       `int8`
  `key_unsorted_total`          `int8`

## Column Descriptions

### all_compound_sorted

### all_interleaved_sorted

### all_unsorted_sorted

### all_compound_unsorted

### all_interleaved_unsorted

### all_unsorted_unsorted

### all_compound_total

### all_interleaved_total

### all_unsorted_total

### even_compound_sorted

### even_interleaved_sorted

### even_unsorted_sorted

### even_compound_unsorted

### even_interleaved_unsorted

### even_unsorted_unsorted

### even_compound_total

### even_interleaved_total

### even_unsorted_total

### key_compound_sorted

### key_interleaved_sorted

### key_unsorted_sorted

### key_compound_unsorted

### key_interleaved_unsorted

### key_unsorted_unsorted

### key_compound_total

### key_interleaved_total

### key_unsorted_total
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_all_by_hour', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_all_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('sf_241129', 'objects', 'view', '# `sf_241129.objects`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `sschema`   `varchar`
  `oobject`   `varchar`
  `ttype`     `varchar`

## Column Descriptions

### uuser

### sschema

### oobject

### ttype
'),
  ('sf_241129', 'blocks_rows_totals_for_table', 'view', '# `sf_241129.blocks_rows_totals_for_table`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `sblks`     `varchar`
  `dsblks`    `varchar`
  `ublks`     `varchar`
  `tblks`     `varchar`
  `srows`     `varchar`
  `dsrows`    `varchar`
  `urows`     `varchar`
  `trows`     `varchar`

## Column Descriptions

### sschema

### ttable

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_second_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_second_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'steps_totals_data_on_both_for_slices_by_minute', 'view', '# `sf_241129.steps_totals_data_on_both_for_slices_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `slice`      `int2`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### slice

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_users_by_hour', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### uuser

### start_ts

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'analyze_totals_for_all_by_hour', 'view', '# `lf_241129.analyze_totals_for_all_by_hour`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('lf_241129', 'transactions_stats_commits_by_hour', 'view', '# `lf_241129.transactions_stats_commits_by_hour`

## Description

## Columns

  column                                data type
  ------------------------------------- -------------
  `start_ts`                            `timestamp`
  `min_transaction_batch_queue_start`   `interval`
  `max_transaction_batch_queue_start`   `interval`
  `med_transaction_batch_queue_start`   `float4`
  `avg_transaction_batch_queue_start`   `interval`
  `std_transaction_batch_queue_start`   `float4`
  `sum_transaction_batch_queue_start`   `interval`
  `min_batch_queue_duration`            `interval`
  `max_batch_queue_duration`            `interval`
  `med_batch_queue_duration`            `float4`
  `avg_batch_queue_duration`            `interval`
  `std_batch_queue_duration`            `float4`
  `sum_batch_queue_duration`            `interval`
  `min_batch_leader_duration`           `interval`
  `max_batch_leader_duration`           `interval`
  `med_batch_leader_duration`           `float4`
  `avg_batch_leader_duration`           `interval`
  `std_batch_leader_duration`           `float4`
  `sum_batch_leader_duration`           `interval`
  `min_batch_worker_duration`           `interval`
  `max_batch_worker_duration`           `interval`
  `med_batch_worker_duration`           `float4`
  `avg_batch_worker_duration`           `interval`
  `std_batch_worker_duration`           `float4`
  `sum_batch_worker_duration`           `interval`
  `min_number_batch_transactions`       `int8`
  `max_number_batch_transactions`       `int8`
  `med_number_batch_transactions`       `int8`
  `avg_number_batch_transactions`       `float4`
  `std_number_batch_transactions`       `float4`
  `sum_number_batch_transactions`       `int8`
  `min_number_batch_blocks`             `int8`
  `max_number_batch_blocks`             `int8`
  `med_number_batch_blocks`             `int8`
  `avg_number_batch_blocks`             `float4`
  `std_number_batch_blocks`             `float4`
  `sum_number_batch_blocks`             `int8`
  `number_batches`                      `int8`

## Column Descriptions

### start_ts

### min_transaction_batch_queue_start

### max_transaction_batch_queue_start

### med_transaction_batch_queue_start

### avg_transaction_batch_queue_start

### std_transaction_batch_queue_start

### sum_transaction_batch_queue_start

### min_batch_queue_duration

### max_batch_queue_duration

### med_batch_queue_duration

### avg_batch_queue_duration

### std_batch_queue_duration

### sum_batch_queue_duration

### min_batch_leader_duration

### max_batch_leader_duration

### med_batch_leader_duration

### avg_batch_leader_duration

### std_batch_leader_duration

### sum_batch_leader_duration

### min_batch_worker_duration

### max_batch_worker_duration

### med_batch_worker_duration

### avg_batch_worker_duration

### std_batch_worker_duration

### sum_batch_worker_duration

### min_number_batch_transactions

### max_number_batch_transactions

### med_number_batch_transactions

### avg_number_batch_transactions

### std_number_batch_transactions

### sum_number_batch_transactions

### min_number_batch_blocks

### max_number_batch_blocks

### med_number_batch_blocks

### avg_number_batch_blocks

### std_number_batch_blocks

### sum_number_batch_blocks

### number_batches
'),
  ('sf_241129', 'analyze_totals_for_database_by_minute', 'view', '# `sf_241129.analyze_totals_for_database_by_minute`

## Description

## Columns

  column          data type
  --------------- -------------
  `dbname`        `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### dbname

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('base_241129', 'transactions', 'view', '# `base_241129.transactions`

## Description

## Columns

  column                                      data type
  ------------------------------------------- -------------
  `user_id`                                   `int4`
  `session_id`                                `int4`
  `transaction_id`                            `int8`
  `transaction_start_ts`                      `timestamp`
  `transaction_end_ts`                        `timestamp`
  `transaction_query_queue_duration`          `interval`
  `transaction_query_compile_exec_duration`   `interval`
  `transaction_query_total_duration`          `interval`
  `transaction_idle_duration`                 `interval`
  `transaction_duration`                      `interval`
  `transaction_number_csc_queries`            `int4`
  `transaction_number_rc_queries`             `int4`
  `transaction_number_leader_queries`         `int4`
  `transaction_number_worker_queries`         `int4`
  `transaction_number_total_queries`          `int4`
  `transaction_abort_flag`                    `bool`

## Column Descriptions

### user_id

### session_id

### transaction_id

### transaction_start_ts

### transaction_end_ts

### transaction_query_queue_duration

### transaction_query_compile_exec_duration

### transaction_query_total_duration

### transaction_idle_duration

### transaction_duration

### transaction_number_csc_queries

### transaction_number_rc_queries

### transaction_number_leader_queries

### transaction_number_worker_queries

### transaction_number_total_queries

### transaction_abort_flag
'),
  ('lf_241129', 'transactions_totals_commits_by_second', 'view', '# `lf_241129.transactions_totals_commits_by_second`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `sum_batch_leader_duration`   `interval`
  `sum_batch_worker_duration`   `interval`
  `sum_batch_idle_duration`     `interval`
  `sum_batch_exec_duration`     `interval`

## Column Descriptions

### start_ts

### sum_batch_leader_duration

### sum_batch_worker_duration

### sum_batch_idle_duration

### sum_batch_exec_duration
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_all_by_hour', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_all_by_hour`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_users_by_minute', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_users_by_minute`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `user_id`                                     `int4`
  `user_name`                                   `varchar`
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('sf_241129', 'analyze_totals_for_users_by_hour', 'view', '# `sf_241129.analyze_totals_for_users_by_hour`

## Description

## Columns

  column          data type
  --------------- -------------
  `uuser`         `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### uuser

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_tables_by_day', 'view', '# `lf_241129.steps_stats_durations_on_both_for_tables_by_day`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `table_id`                    `int4`
  `table_name`                  `varchar`
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps
'),
  ('sf_241129', 'blocks_rows_totals_for_slice', 'view', '# `sf_241129.blocks_rows_totals_for_slice`

## Description

## Columns

  column     data type
  ---------- -----------
  `node`     `int2`
  `slice`    `int2`
  `ttype`    `varchar`
  `sblks`    `varchar`
  `dsblks`   `varchar`
  `ublks`    `varchar`
  `tblks`    `varchar`
  `srows`    `varchar`
  `dsrows`   `varchar`
  `urows`    `varchar`
  `trows`    `varchar`

## Column Descriptions

### node

### slice

### ttype

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_all_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_all_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `node_type`      `varchar`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### node_type

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('base_241129', 'queries_compilation', 'view', '# `base_241129.queries_compilation`

## Description

## Columns

  column                     data type
  -------------------------- ------------
  `user_id`                  `int4`
  `query_id`                 `int4`
  `query_compile_duration`   `interval`

## Column Descriptions

### user_id

### query_id

### query_compile_duration
'),
  ('sf_241129', 'transactions', 'view', '# `sf_241129.transactions`

## Description

## Columns

  column        data type
  ------------- -------------
  `uuser`       `varchar`
  `pid`         `int4`
  `xid`         `int8`
  `start_ts`    `timestamp`
  `end_ts`      `timestamp`
  `qry_queue`   `numeric`
  `qry_cexec`   `numeric`
  `qry_total`   `numeric`
  `xid_idle`    `numeric`
  `xid_total`   `numeric`
  `csc`         `int4`
  `rc`          `int4`
  `ldr`         `int4`
  `wrk`         `int4`
  `abt`         `varchar`

## Column Descriptions

### uuser

### pid

### xid

### start_ts

### end_ts

### qry_queue

### qry_cexec

### qry_total

### xid_idle

### xid_total

### csc

### rc

### ldr

### wrk

### abt
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_tables_by_all', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_tables_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `varchar`
  `sschema`   `varchar`
  `ttable`    `varchar`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### node

### sschema

### ttable

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_users_by_hour', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_users_by_hour`

## Description

## Columns

  column                data type
  --------------------- -------------
  `user_id`             `int4`
  `node_type`           `varchar`
  `user_name`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### user_id

### node_type

### user_name

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'batches', 'view', '# `lf_241129.batches`

## Description

## Columns

  column                                     data type
  ------------------------------------------ -------------
  `batch_id`                                 `int8`
  `batch_startqueue`                         `timestamp`
  `batch_leader_startwork`                   `timestamp`
  `batch_worker_startwork`                   `timestamp`
  `batch_endlocal`                           `timestamp`
  `batch_worker_startglobal`                 `timestamp`
  `batch_leader_endtime`                     `timestamp`
  `batch_worker_endtime`                     `timestamp`
  `batch_endtime`                            `timestamp`
  `batch_queue_duration`                     `interval`
  `batch_worker_local_duration`              `interval`
  `batch_worker_global_duration`             `interval`
  `batch_worker_total_duration`              `interval`
  `batch_worker_duration`                    `interval`
  `batch_leader_and_worker_local_duration`   `interval`
  `batch_leader_duration`                    `interval`
  `batch_exec_duration`                      `interval`
  `batch_total_duration`                     `interval`
  `number_batch_blocks`                      `int8`
  `number_batch_headers`                     `int8`
  `number_batch_transactions`                `int8`

## Column Descriptions

### batch_id

### batch_startqueue

### batch_leader_startwork

### batch_worker_startwork

### batch_endlocal

### batch_worker_startglobal

### batch_leader_endtime

### batch_worker_endtime

### batch_endtime

### batch_queue_duration

### batch_worker_local_duration

### batch_worker_global_duration

### batch_worker_total_duration

### batch_worker_duration

### batch_leader_and_worker_local_duration

### batch_leader_duration

### batch_exec_duration

### batch_total_duration

### number_batch_blocks

### number_batch_headers

### number_batch_transactions
'),
  ('lf_241129', 'queries_text_on_both_fast', 'view', '# `lf_241129.queries_text_on_both_fast`

## Description

## Columns

  column               data type
  -------------------- -------------
  `session_id`         `int4`
  `transaction_id`     `int8`
  `query_id`           `int4`
  `query_start_ts`     `timestamp`
  `query_text_order`   `int4`
  `query_text`         `varchar`

## Column Descriptions

### session_id

### transaction_id

### query_id

### query_start_ts

### query_text_order

### query_text
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_statements_by_day', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_statements_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### statement_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('lf_241129', 'pprocedures_text', 'view', '# `lf_241129.pprocedures_text`

## Description

## Columns

  column             data type
  ------------------ -----------
  `schema_id`        `int4`
  `schema_name`      `varchar`
  `procedure_id`     `int4`
  `procedure_name`   `varchar`
  `procedure_text`   `varchar`

## Column Descriptions

### schema_id

### schema_name

### procedure_id

### procedure_name

### procedure_text
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_all_by_second', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### duration

### queries
'),
  ('ro_241129', 'sp_uninstall_version', 'procedure', '# `ro_241129.sp_uninstall_version`

## Description

## Arguments

  argument    data type
  ----------- ----------------
  `version`   `varchar( 6 )`

## Argument Descriptions

### version
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_tables_by_day', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_tables_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `date`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### ddelete

### iinsert

### scan
'),
  ('base_241129', 'queries_for_worker', 'view', '# `base_241129.queries_for_worker`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `user_id`                       `int4`
  `session_id`                    `int4`
  `transaction_id`                `int8`
  `query_id`                      `int4`
  `queue_id`                      `int2`
  `query_start_ts`                `timestamp`
  `query_end_ts`                  `timestamp`
  `query_queue_duration`          `interval`
  `query_compile_exec_duration`   `interval`
  `query_wlm_duration`            `interval`
  `query_total_duration`          `interval`
  `query_abort_flag`              `bool`
  `query_csc_flag`                `bool`
  `query_leader_node_flag`        `bool`
  `query_rc_flag`                 `bool`
  `query_text`                    `varchar`

## Column Descriptions

### user_id

### session_id

### transaction_id

### query_id

### queue_id

### query_start_ts

### query_end_ts

### query_queue_duration

### query_compile_exec_duration

### query_wlm_duration

### query_total_duration

### query_abort_flag

### query_csc_flag

### query_leader_node_flag

### query_rc_flag

### query_text
'),
  ('sf_241129', 'steps_totals_data_on_both_for_all_by_hour', 'view', '# `sf_241129.steps_totals_data_on_both_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('base_241129', 'ttables_activity_fast', 'view', '# `base_241129.ttables_activity_fast`

## Description

## Columns

  column                   data type
  ------------------------ -----------
  `table_id`               `int4`
  `number_rows_deleted`    `int8`
  `number_rows_inserted`   `int8`

## Column Descriptions

### table_id

### number_rows_deleted

### number_rows_inserted
'),
  ('lf_241129', 'transactions_batches', 'view', '# `lf_241129.transactions_batches`

## Description

## Columns

  column                                      data type
  ------------------------------------------- -------------
  `user_id`                                   `int4`
  `user_name`                                 `varchar`
  `session_id`                                `int4`
  `transaction_id`                            `int8`
  `transaction_start_ts`                      `timestamp`
  `transaction_end_ts`                        `timestamp`
  `transaction_query_queue_duration`          `interval`
  `transaction_query_compile_exec_duration`   `interval`
  `transaction_query_total_duration`          `interval`
  `transaction_idle_duration`                 `interval`
  `transaction_duration`                      `interval`
  `transaction_number_csc_queries`            `int4`
  `transaction_number_rc_queries`             `int4`
  `transaction_number_leader_queries`         `int4`
  `transaction_number_worker_queries`         `int4`
  `transaction_number_total_queries`          `int4`
  `transaction_abort_flag`                    `bool`
  `batch_id`                                  `int8`
  `transaction_batch_queue_start`             `timestamp`
  `transaction_batch_queue_duration`          `interval`
  `batch_start_ts`                            `timestamp`
  `batch_end_ts`                              `timestamp`
  `batch_leader_duration`                     `interval`
  `batch_worker_duration`                     `interval`
  `batch_queue_duration`                      `interval`
  `batch_exec_duration`                       `interval`
  `batch_total_duration`                      `interval`
  `number_batch_blocks`                       `int8`
  `number_batch_headers`                      `int8`
  `number_batch_transactions`                 `int8`

## Column Descriptions

### user_id

### user_name

### session_id

### transaction_id

### transaction_start_ts

### transaction_end_ts

### transaction_query_queue_duration

### transaction_query_compile_exec_duration

### transaction_query_total_duration

### transaction_idle_duration

### transaction_duration

### transaction_number_csc_queries

### transaction_number_rc_queries

### transaction_number_leader_queries

### transaction_number_worker_queries

### transaction_number_total_queries

### transaction_abort_flag

### batch_id

### transaction_batch_queue_start

### transaction_batch_queue_duration

### batch_start_ts

### batch_end_ts

### batch_leader_duration

### batch_worker_duration

### batch_queue_duration

### batch_exec_duration

### batch_total_duration

### number_batch_blocks

### number_batch_headers

### number_batch_transactions
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_queues_by_second', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_queues_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `queue_id`                     `int2`
  `queue_name`                   `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('sf_241129', 'objects_totals_for_schema', 'view', '# `sf_241129.objects_totals_for_schema`

## Description

## Columns

  column           data type
  ---------------- -----------
  `sschema`        `varchar`
  `ttables`        `int4`
  `ttables_cols`   `int4`
  `nviews`         `int4`
  `nviews_cols`    `int4`
  `lviews`         `int4`
  `mviews`         `int4`
  `mviews_cols`    `int4`
  `funcs`          `int4`
  `procs`          `int4`

## Column Descriptions

### sschema

### ttables

### ttables_cols

### nviews

### nviews_cols

### lviews

### mviews

### mviews_cols

### funcs

### procs
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_all_by_minute', 'view', '# `lf_241129.steps_stats_durations_on_both_for_all_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_users_by_day', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `start_ts`   `date`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### uuser

### start_ts

### duration

### queries
'),
  ('sf_241129', 'vacuum_totals_for_users_by_day', 'view', '# `sf_241129.vacuum_totals_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `start_ts`   `date`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### uuser

### start_ts

### manual

### auto

### total

### dur
'),
  ('lf_241129', 'ttables_stats_datatypes_encodings_by_count_and_blocks', 'view', '# `lf_241129.ttables_stats_datatypes_encodings_by_count_and_blocks`

## Description

## Columns

  column                       data type
  ---------------------------- -----------
  `base_data_type_id`          `int4`
  `base_data_type_name`        `varchar`
  `number_az64_columns`        `int4`
  `number_bytedict_columns`    `int4`
  `number_delta_columns`       `int4`
  `number_delta32k_columns`    `int4`
  `number_lzo_columns`         `int4`
  `number_mostly8_columns`     `int4`
  `number_mostly16_columns`    `int4`
  `number_mostly32_columns`    `int4`
  `number_raw_columns`         `int4`
  `number_runlength_columns`   `int4`
  `number_text255_columns`     `int4`
  `number_text32k_columns`     `int4`
  `number_zstd_columns`        `int4`
  `number_total_columns`       `int4`
  `number_az64_blocks`         `int8`
  `number_bytedict_blocks`     `int8`
  `number_delta_blocks`        `int8`
  `number_delta32k_blocks`     `int8`
  `number_lzo_blocks`          `int8`
  `number_mostly8_blocks`      `int8`
  `number_mostly16_blocks`     `int8`
  `number_mostly32_blocks`     `int8`
  `number_raw_blocks`          `int8`
  `number_runlength_blocks`    `int8`
  `number_text255_blocks`      `int8`
  `number_text32k_blocks`      `int8`
  `number_zstd_blocks`         `int8`
  `number_total_blocks`        `int8`

## Column Descriptions

### base_data_type_id

### base_data_type_name

### number_az64_columns

### number_bytedict_columns

### number_delta_columns

### number_delta32k_columns

### number_lzo_columns

### number_mostly8_columns

### number_mostly16_columns

### number_mostly32_columns

### number_raw_columns

### number_runlength_columns

### number_text255_columns

### number_text32k_columns

### number_zstd_columns

### number_total_columns

### number_az64_blocks

### number_bytedict_blocks

### number_delta_blocks

### number_delta32k_blocks

### number_lzo_blocks

### number_mostly8_blocks

### number_mostly16_blocks

### number_mostly32_blocks

### number_raw_blocks

### number_runlength_blocks

### number_text255_blocks

### number_text32k_blocks

### number_zstd_blocks

### number_total_blocks
'),
  ('base_241129', 'fixed_pg_class_for_tables', 'view', '# `base_241129.fixed_pg_class_for_tables`

## Description

## Columns

  column         data type
  -------------- -----------
  `table_id`     `int4`
  `table_name`   `varchar`
  `schema_id`    `int4`
  `user_id`      `int4`
  `relnatts`     `int2`

## Column Descriptions

### table_id

### table_name

### schema_id

### user_id

### relnatts
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_all_by_all', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_all_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `ddelete`   `varchar`
  `iinsert`   `varchar`
  `scan`      `varchar`

## Column Descriptions

### ddelete

### iinsert

### scan
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_all_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_all_concise`

## Description

## Columns

  column      data type
  ----------- -----------
  `n`         `int2`
  `s`         `int2`
  `t`         `varchar`
  `aggr`      `numeric`
  `bcast`     `numeric`
  `del`       `numeric`
  `dist`      `numeric`
  `hash`      `numeric`
  `hjoin`     `numeric`
  `iinsert`   `numeric`
  `llimit`    `numeric`
  `mmerge`    `numeric`
  `mjoin`     `numeric`
  `nloop`     `numeric`
  `rreturn`   `numeric`
  `save`      `numeric`
  `ssort`     `numeric`
  `uunique`   `numeric`
  `wwindow`   `numeric`

## Column Descriptions

### n

### s

### t

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'wqueues_totals_slot_utilization_by_hour', 'view', '# `sf_241129.wqueues_totals_slot_utilization_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `slot`       `int2`
  `start_ts`   `timestamp`
  `dur`        `numeric`

## Column Descriptions

### qname

### slot

### start_ts

### dur
'),
  ('sf_241129', 'queries_totals_result_cache_for_users_by_hour', 'view', '# `sf_241129.queries_totals_result_cache_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### uuser

### start_ts

### hit

### miss

### total

### %
'),
  ('sf_241129', 'objects_dependencies', 'view', '# `sf_241129.objects_dependencies`

## Description

## Columns

  column                 data type
  ---------------------- -----------
  `ddepth`               `int2`
  `parent_owner_name`    `varchar`
  `parent_schema_name`   `varchar`
  `parent_object_type`   `varchar`
  `parent_object_name`   `varchar`
  `child_owner_name`     `varchar`
  `child_schema_name`    `varchar`
  `child_object_type`    `varchar`
  `child_object_name`    `varchar`

## Column Descriptions

### ddepth

### parent_owner_name

### parent_schema_name

### parent_object_type

### parent_object_name

### child_owner_name

### child_schema_name

### child_object_type

### child_object_name
'),
  ('sf_241129', 'views_materialized', 'view', '# `sf_241129.views_materialized`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `vview`     `varchar`
  `uuser`     `varchar`
  `status`    `varchar`

## Column Descriptions

### sschema

### vview

### uuser

### status
'),
  ('sf_241129', 'ttables_blocks_rows_activity', 'view', '# `sf_241129.ttables_blocks_rows_activity`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `uuser`     `varchar`
  `age`       `int4`
  `cols`      `int2`
  `ssort`     `varchar`
  `skeys`     `int2`
  `dist`      `varchar`
  `slices`    `int2`
  `sblks`     `int8`
  `dsblks`    `int8`
  `ublks`     `int8`
  `tblks`     `int8`
  `srows`     `int8`
  `dsrows`    `int8`
  `urows`     `int8`
  `trows`     `int8`
  `drops`     `int8`
  `irows`     `int8`

## Column Descriptions

### sschema

### ttable

### uuser

### age

### cols

### ssort

### skeys

### dist

### slices

### sblks

### dsblks

### ublks

### tblks

### srows

### dsrows

### urows

### trows

### drops

### irows
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_query', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_query`

## Description

## Columns

  column                data type
  --------------------- -----------
  `query_id`            `int4`
  `node_type`           `varchar`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### query_id

### node_type

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'transactions_stats_commits_by_second', 'view', '# `lf_241129.transactions_stats_commits_by_second`

## Description

## Columns

  column                                data type
  ------------------------------------- -------------
  `start_ts`                            `timestamp`
  `min_transaction_batch_queue_start`   `interval`
  `max_transaction_batch_queue_start`   `interval`
  `med_transaction_batch_queue_start`   `float4`
  `avg_transaction_batch_queue_start`   `interval`
  `std_transaction_batch_queue_start`   `float4`
  `sum_transaction_batch_queue_start`   `interval`
  `min_batch_queue_duration`            `interval`
  `max_batch_queue_duration`            `interval`
  `med_batch_queue_duration`            `float4`
  `avg_batch_queue_duration`            `interval`
  `std_batch_queue_duration`            `float4`
  `sum_batch_queue_duration`            `interval`
  `min_batch_leader_duration`           `interval`
  `max_batch_leader_duration`           `interval`
  `med_batch_leader_duration`           `float4`
  `avg_batch_leader_duration`           `interval`
  `std_batch_leader_duration`           `float4`
  `sum_batch_leader_duration`           `interval`
  `min_batch_worker_duration`           `interval`
  `max_batch_worker_duration`           `interval`
  `med_batch_worker_duration`           `float4`
  `avg_batch_worker_duration`           `interval`
  `std_batch_worker_duration`           `float4`
  `sum_batch_worker_duration`           `interval`
  `min_number_batch_transactions`       `int8`
  `max_number_batch_transactions`       `int8`
  `med_number_batch_transactions`       `int8`
  `avg_number_batch_transactions`       `float4`
  `std_number_batch_transactions`       `float4`
  `sum_number_batch_transactions`       `int8`
  `min_number_batch_blocks`             `int8`
  `max_number_batch_blocks`             `int8`
  `med_number_batch_blocks`             `int8`
  `avg_number_batch_blocks`             `float4`
  `std_number_batch_blocks`             `float4`
  `sum_number_batch_blocks`             `int8`
  `number_batches`                      `int8`

## Column Descriptions

### start_ts

### min_transaction_batch_queue_start

### max_transaction_batch_queue_start

### med_transaction_batch_queue_start

### avg_transaction_batch_queue_start

### std_transaction_batch_queue_start

### sum_transaction_batch_queue_start

### min_batch_queue_duration

### max_batch_queue_duration

### med_batch_queue_duration

### avg_batch_queue_duration

### std_batch_queue_duration

### sum_batch_queue_duration

### min_batch_leader_duration

### max_batch_leader_duration

### med_batch_leader_duration

### avg_batch_leader_duration

### std_batch_leader_duration

### sum_batch_leader_duration

### min_batch_worker_duration

### max_batch_worker_duration

### med_batch_worker_duration

### avg_batch_worker_duration

### std_batch_worker_duration

### sum_batch_worker_duration

### min_number_batch_transactions

### max_number_batch_transactions

### med_number_batch_transactions

### avg_number_batch_transactions

### std_number_batch_transactions

### sum_number_batch_transactions

### min_number_batch_blocks

### max_number_batch_blocks

### med_number_batch_blocks

### avg_number_batch_blocks

### std_number_batch_blocks

### sum_number_batch_blocks

### number_batches
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_users_by_hour', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### uuser

### start_ts

### rread

### to_disk
'),
  ('lf_241129', 'ttables_creation_for_all_by_minute', 'view', '# `lf_241129.ttables_creation_for_all_by_minute`

## Description

## Columns

  column            data type
  ----------------- -------------
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### start_ts

### number_tables
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_second_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_second_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_statements_by_day', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_statements_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `stmt`       `varchar`
  `start_ts`   `date`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### stmt

### start_ts

### queue

### compile

### exec

### queries
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_all_by_all', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_all_by_all`

## Description

## Columns

  column                data type
  --------------------- -----------
  `node_type`           `varchar`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_type

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_tables_by_second', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_tables_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### ddelete

### iinsert

### scan
'),
  ('base_241129', 'fixed_stl_ddltext', 'view', '# `base_241129.fixed_stl_ddltext`

## Description

## Columns

  column                          data type
  ------------------------------- -------------
  `user_id`                       `int4`
  `transaction_id`                `int8`
  `session_id`                    `int4`
  `queue_id`                      `int2`
  `query_start_ts`                `timestamp`
  `query_end_ts`                  `timestamp`
  `sequence`                      `int4`
  `query_text`                    `varchar`
  `query_queue_duration`          `interval`
  `query_wlm_duration`            `interval`
  `query_compile_exec_duration`   `interval`
  `query_total_duration`          `interval`

## Column Descriptions

### user_id

### transaction_id

### session_id

### queue_id

### query_start_ts

### query_end_ts

### sequence

### query_text

### query_queue_duration

### query_wlm_duration

### query_compile_exec_duration

### query_total_duration
'),
  ('sf_241129', 'steps_totals_data_on_both_for_statements_by_day', 'view', '# `sf_241129.steps_totals_data_on_both_for_statements_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `stmt`       `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### stmt

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('base_241129', 'hr_bytes', 'function', '# `base_241129.hr_bytes`

## Description

## Returns

`varchar`

## Arguments

  argument   data type
  ---------- -----------
  `bytes`    `int8`

## Argument Descriptions

### bytes
'),
  ('sf_241129', 'steps_totals_data_on_both_for_users_by_hour', 'view', '# `sf_241129.steps_totals_data_on_both_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'views_late_binding', 'view', '# `lf_241129.views_late_binding`

## Description

## Columns

  column          data type
  --------------- -----------
  `schema_id`     `int4`
  `schema_name`   `varchar`
  `view_id`       `int4`
  `view_name`     `varchar`
  `user_id`       `int4`
  `user_name`     `varchar`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### user_id

### user_name
'),
  ('sf_241129', 'vacuum_totals_for_tables_by_hour', 'view', '# `sf_241129.vacuum_totals_for_tables_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_users_by_second', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### uuser

### start_ts

### ddelete

### iinsert

### scan
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_queues_by_day', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_queues_by_day`

## Description

## Columns

  column                data type
  --------------------- -------------
  `queue_id`            `int2`
  `queue_name`          `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### queue_id

### queue_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'analyze_totals_for_tables_by_day', 'view', '# `sf_241129.analyze_totals_for_tables_by_day`

## Description

## Columns

  column          data type
  --------------- -----------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `start_ts`      `date`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'wqueues_totals_slot_utilization_by_minute', 'view', '# `lf_241129.wqueues_totals_slot_utilization_by_minute`

## Description

## Columns

  column          data type
  --------------- -------------
  `queue_id`      `int2`
  `queue_name`    `varchar`
  `slot_number`   `int2`
  `start_ts`      `timestamp`
  `duration`      `interval`

## Column Descriptions

### queue_id

### queue_name

### slot_number

### start_ts

### duration
'),
  ('lf_241129', 'transactions_totals_commits_by_all', 'view', '# `lf_241129.transactions_totals_commits_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- ------------
  `sum_batch_leader_duration`   `interval`
  `sum_batch_worker_duration`   `interval`
  `sum_batch_idle_duration`     `interval`
  `sum_batch_exec_duration`     `interval`

## Column Descriptions

### sum_batch_leader_duration

### sum_batch_worker_duration

### sum_batch_idle_duration

### sum_batch_exec_duration
'),
  ('ro_241129', 'sp_revoke_version_from_public', 'procedure', '# `ro_241129.sp_revoke_version_from_public`

## Description

## Arguments

  argument    data type
  ----------- ----------------
  `version`   `varchar( 6 )`

## Argument Descriptions

### version
'),
  ('lf_241129', 'utils_most_recent_copy_error_for_current_user', 'view', '# `lf_241129.utils_most_recent_copy_error_for_current_user`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `event_ts`                     `timestamp`
  `schema_id`                    `int4`
  `schema_name`                  `varchar`
  `table_id`                     `int4`
  `table_name`                   `varchar`
  `column_name`                  `varchar`
  `column_full_data_type_name`   `varchar`
  `line_number`                  `varchar`
  `field_value`                  `varchar`
  `reason`                       `varchar`
  `file_name`                    `varchar`

## Column Descriptions

### event_ts

### schema_id

### schema_name

### table_id

### table_name

### column_name

### column_full_data_type_name

### line_number

### field_value

### reason

### file_name
'),
  ('lf_241129', 'ddatabases', 'view', '# `lf_241129.ddatabases`

## Description

## Columns

  column               data type
  -------------------- -----------
  `database_id`        `int4`
  `database_name`      `varchar`
  `user_id`            `int4`
  `user_name`          `varchar`
  `isolation_method`   `varchar`

## Column Descriptions

### database_id

### database_name

### user_id

### user_name

### isolation_method
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_slices_by_minute', 'view', '# `lf_241129.steps_stats_durations_on_both_for_slices_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `slice_ordinal`               `int2`
  `slice_type`                  `varchar`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### slice_ordinal

### slice_type

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_users_by_hour', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'vacuum_totals_for_all_by_all', 'view', '# `sf_241129.vacuum_totals_for_all_by_all`

## Description

## Columns

  column     data type
  ---------- -----------
  `manual`   `varchar`
  `auto`     `varchar`
  `total`    `int4`
  `dur`      `numeric`

## Column Descriptions

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_second', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_schemas_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('base_241129', 'fixed_stv_wlm_query_state', 'view', '# `base_241129.fixed_stv_wlm_query_state`

## Description

## Columns

  column               data type
  -------------------- -------------
  `exec_time`          `int8`
  `query_id`           `int4`
  `query_priority`     `varchar`
  `queue_time`         `int8`
  `queue_id`           `int2`
  `query_slot_count`   `int2`
  `query_state`        `varchar`
  `task_id`            `int4`
  `wlm_start_time`     `timestamp`
  `transaction_id`     `int8`

## Column Descriptions

### exec_time

### query_id

### query_priority

### queue_time

### queue_id

### query_slot_count

### query_state

### task_id

### wlm_start_time

### transaction_id
'),
  ('sf_241129', 'steps_totals_data_on_both_for_users_by_second', 'view', '# `sf_241129.steps_totals_data_on_both_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_all_by_day', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_all_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_users_by_day', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_users_by_day`

## Description

## Columns

  column             data type
  ------------------ -------------
  `user_id`          `int4`
  `user_name`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('sf_241129', 'queries_totals_result_cache_for_all_by_hour', 'view', '# `sf_241129.queries_totals_result_cache_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### start_ts

### hit

### miss

### total

### %
'),
  ('sf_241129', 'blocks_rows_totals_for_tables_by_node', 'view', '# `sf_241129.blocks_rows_totals_for_tables_by_node`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `dist`      `varchar`
  `a`         `varchar`
  `n`         `int2`
  `sblks`     `varchar`
  `ublks`     `varchar`
  `tblks`     `varchar`
  `srows`     `varchar`
  `urows`     `varchar`
  `trows`     `varchar`

## Column Descriptions

### sschema

### ttable

### dist

### a

### n

### sblks

### ublks

### tblks

### srows

### urows

### trows
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_node_by_hour', 'view', '# `lf_241129.steps_stats_durations_on_both_for_node_by_hour`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `start_ts`                    `timestamp`
  `node_ordinal`                `int2`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_bcast_duration`          `interval`
  `max_bcast_duration`          `interval`
  `med_bcast_duration`          `float4`
  `avg_bcast_duration`          `interval`
  `std_bcast_duration`          `float4`
  `number_bcast_steps`          `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_dist_duration`           `interval`
  `max_dist_duration`           `interval`
  `med_dist_duration`           `float4`
  `avg_dist_duration`           `interval`
  `std_dist_duration`           `float4`
  `number_dist_steps`           `int8`
  `min_hash_duration`           `interval`
  `max_hash_duration`           `interval`
  `med_hash_duration`           `float4`
  `avg_hash_duration`           `interval`
  `std_hash_duration`           `float4`
  `number_hash_steps`           `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_limit_duration`          `interval`
  `max_limit_duration`          `interval`
  `med_limit_duration`          `float4`
  `avg_limit_duration`          `interval`
  `std_limit_duration`          `float4`
  `number_limit_steps`          `int8`
  `min_merge_duration`          `interval`
  `max_merge_duration`          `interval`
  `med_merge_duration`          `float4`
  `avg_merge_duration`          `interval`
  `std_merge_duration`          `float4`
  `number_merge_steps`          `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_parse_duration`          `interval`
  `max_parse_duration`          `interval`
  `med_parse_duration`          `float4`
  `avg_parse_duration`          `interval`
  `std_parse_duration`          `float4`
  `number_parse_steps`          `int8`
  `min_project_duration`        `interval`
  `max_project_duration`        `interval`
  `med_project_duration`        `float4`
  `avg_project_duration`        `interval`
  `std_project_duration`        `float4`
  `number_project_steps`        `int8`
  `min_return_duration`         `interval`
  `max_return_duration`         `interval`
  `med_return_duration`         `float4`
  `avg_return_duration`         `interval`
  `std_return_duration`         `float4`
  `number_return_steps`         `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`
  `min_unique_duration`         `interval`
  `max_unique_duration`         `interval`
  `med_unique_duration`         `float4`
  `avg_unique_duration`         `interval`
  `std_unique_duration`         `float4`
  `number_unique_steps`         `int8`
  `min_window_duration`         `interval`
  `max_window_duration`         `interval`
  `med_window_duration`         `float4`
  `avg_window_duration`         `interval`
  `std_window_duration`         `float4`
  `number_window_steps`         `int8`

## Column Descriptions

### start_ts

### node_ordinal

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_bcast_duration

### max_bcast_duration

### med_bcast_duration

### avg_bcast_duration

### std_bcast_duration

### number_bcast_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_dist_duration

### max_dist_duration

### med_dist_duration

### avg_dist_duration

### std_dist_duration

### number_dist_steps

### min_hash_duration

### max_hash_duration

### med_hash_duration

### avg_hash_duration

### std_hash_duration

### number_hash_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_limit_duration

### max_limit_duration

### med_limit_duration

### avg_limit_duration

### std_limit_duration

### number_limit_steps

### min_merge_duration

### max_merge_duration

### med_merge_duration

### avg_merge_duration

### std_merge_duration

### number_merge_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_parse_duration

### max_parse_duration

### med_parse_duration

### avg_parse_duration

### std_parse_duration

### number_parse_steps

### min_project_duration

### max_project_duration

### med_project_duration

### avg_project_duration

### std_project_duration

### number_project_steps

### min_return_duration

### max_return_duration

### med_return_duration

### avg_return_duration

### std_return_duration

### number_return_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps

### min_unique_duration

### max_unique_duration

### med_unique_duration

### avg_unique_duration

### std_unique_duration

### number_unique_steps

### min_window_duration

### max_window_duration

### med_window_duration

### avg_window_duration

### std_window_duration

### number_window_steps
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_minute_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_minute_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_users_by_second', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_users_by_second`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `user_id`                                     `int4`
  `user_name`                                   `varchar`
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('sf_241129', 'steps_totals_data_on_both_for_statements_by_hour', 'view', '# `sf_241129.steps_totals_data_on_both_for_statements_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `stmt`       `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### stmt

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_all_by_minute', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### queue

### compile

### exec

### queries
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_hour_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_hour_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `n`              `int2`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('base_241129', 'fixed_svl_compile', 'view', '# `base_241129.fixed_svl_compile`

## Description

## Columns

  column                       data type
  ---------------------------- -------------
  `compile_flag`               `bool`
  `segment_compile_end_ts`     `timestamp`
  `session_id`                 `int4`
  `query_id`                   `int4`
  `segment_ordinal`            `int2`
  `segment_compile_start_ts`   `timestamp`
  `user_id`                    `int4`
  `transaction_id`             `int8`
  `segment_compile_duration`   `interval`

## Column Descriptions

### compile_flag

### segment_compile_end_ts

### session_id

### query_id

### segment_ordinal

### segment_compile_start_ts

### user_id

### transaction_id

### segment_compile_duration
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_node_by_minute', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_node_by_minute`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'privileges_granted_to_groups', 'view', '# `sf_241129.privileges_granted_to_groups`

## Description

## Columns

  column      data type
  ----------- -----------
  `ggroup`    `varchar`
  `ttype`     `varchar`
  `sschema`   `varchar`
  `oobject`   `varchar`
  `rt`        `varchar`
  `args`      `varchar`
  `oowner`    `varchar`
  `privs`     `varchar`

## Column Descriptions

### ggroup

### ttype

### sschema

### oobject

### rt

### args

### oowner

### privs
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_slices_by_minute_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_slices_by_minute_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `start_ts`       `timestamp`
  `n`              `int2`
  `s`              `int2`
  `t`              `varchar`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### start_ts

### n

### s

### t

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_queues_by_all', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_queues_by_all`

## Description

## Columns

  column           data type
  ---------------- -----------
  `queue_id`       `int2`
  `queue_name`     `varchar`
  `number_abort`   `int4`

## Column Descriptions

### queue_id

### queue_name

### number_abort
'),
  ('lf_241129', 'ttables_creation_for_schemas_by_minute', 'view', '# `lf_241129.ttables_creation_for_schemas_by_minute`

## Description

## Columns

  column            data type
  ----------------- -------------
  `schema_name`     `varchar`
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### schema_name

### start_ts

### number_tables
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_users_by_minute', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### uuser

### start_ts

### aborts
'),
  ('sf_241129', 'steps_totals_data_on_both_for_node_by_minute', 'view', '# `sf_241129.steps_totals_data_on_both_for_node_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `int2`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'queries_text_on_workers_full', 'view', '# `lf_241129.queries_text_on_workers_full`

## Description

## Columns

  column               data type
  -------------------- -----------
  `session_id`         `int4`
  `transaction_id`     `int8`
  `query_id`           `int4`
  `query_text_order`   `int4`
  `query_text`         `varchar`

## Column Descriptions

### session_id

### transaction_id

### query_id

### query_text_order

### query_text
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_queues_by_day', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_queues_by_day`

## Description

## Columns

  column           data type
  ---------------- -------------
  `queue_id`       `int2`
  `queue_name`     `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### number_abort
'),
  ('lf_241129', 'views_materialized_table_blocks_rows', 'view', '# `lf_241129.views_materialized_table_blocks_rows`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `view_id`                        `int4`
  `view_name`                      `varchar`
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `failures`                       `varchar`
  `autorefresh`                    `varchar`
  `refresh_type`                   `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `table_creation`                 `timestamp`
  `table_age`                      `interval`
  `table_number_columns`           `int2`
  `table_sort_type`                `varchar`
  `table_number_sortkeys`          `int2`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`
  `number_slices`                  `int2`
  `number_sorted_blocks`           `int8`
  `number_unsorted_blocks`         `int8`
  `number_total_blocks`            `int8`
  `number_sorted_rows`             `int8`
  `number_unsorted_rows`           `int8`
  `number_total_rows`              `int8`

## Column Descriptions

### schema_id

### schema_name

### view_id

### view_name

### user_id

### user_name

### failures

### autorefresh

### refresh_type

### table_id

### table_name

### table_creation

### table_age

### table_number_columns

### table_sort_type

### table_number_sortkeys

### table_distribution_type

### table_distribution_auto_flag

### number_slices

### number_sorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('sf_241129', 'analyze_totals_for_database_by_hour', 'view', '# `sf_241129.analyze_totals_for_database_by_hour`

## Description

## Columns

  column          data type
  --------------- -------------
  `dbname`        `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### dbname

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_users_by_day', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_users_by_day`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -------------
  `user_id`                                     `int4`
  `user_name`                                   `varchar`
  `start_ts`                                    `timestamp`
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_all_by_second', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_all_by_second`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('lf_241129', 'blocks_rows_totals_for_tables_by_slice', 'view', '# `lf_241129.blocks_rows_totals_for_tables_by_slice`

## Description

## Columns

  column                           data type
  -------------------------------- -----------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`
  `node_ordinal`                   `int2`
  `slice_ordinal`                  `int2`
  `slice_type`                     `varchar`
  `number_sorted_blocks`           `int8`
  `number_unsorted_blocks`         `int8`
  `number_total_blocks`            `int8`
  `number_sorted_rows`             `int8`
  `number_unsorted_rows`           `int8`
  `number_total_rows`              `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### table_distribution_type

### table_distribution_auto_flag

### node_ordinal

### slice_ordinal

### slice_type

### number_sorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('lf_241129', 'transactions_totals_query_counts_on_both_for_users_by_second', 'view', '# `lf_241129.transactions_totals_query_counts_on_both_for_users_by_second`

## Description

## Columns

  column                           data type
  -------------------------------- -------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `start_ts`                       `timestamp`
  `number_transactions`            `int8`
  `number_writing_transactions`    `int8`
  `number_readonly_transactions`   `int8`
  `number_leader_queries`          `int8`
  `number_worker_queries`          `int8`
  `number_total_queries`           `int8`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_transactions

### number_writing_transactions

### number_readonly_transactions

### number_leader_queries

### number_worker_queries

### number_total_queries
'),
  ('lf_241129', 'steps_totals_data_on_both_for_slices_by_hour', 'view', '# `lf_241129.steps_totals_data_on_both_for_slices_by_hour`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `slice_ordinal`                `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### slice_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'blocks_rows_totals_for_cluster', 'view', '# `lf_241129.blocks_rows_totals_for_cluster`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `number_nodes`             `int2`
  `number_partial_slices`    `int2`
  `number_full_slices`       `int2`
  `number_sorted_blocks`     `int8`
  `number_desorted_blocks`   `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`
  `number_sorted_rows`       `int8`
  `number_desorted_rows`     `int8`
  `number_unsorted_rows`     `int8`
  `number_total_rows`        `int8`

## Column Descriptions

### number_nodes

### number_partial_slices

### number_full_slices

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('ro_241129', 'sp_revoke_versionless_sf_schema_from_public', 'procedure', '# `ro_241129.sp_revoke_versionless_sf_schema_from_public`

## Description

## Arguments

No arguments.
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_all_by_minute', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'queries_totals_result_cache_for_users_by_all', 'view', '# `lf_241129.queries_totals_result_cache_for_users_by_all`

## Description

## Columns

  column           data type
  ---------------- -----------
  `user_id`        `int4`
  `user_name`      `varchar`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### user_id

### user_name

### number_hit

### number_miss

### number_total

### percentage
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_users_by_all', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_users_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `rread`     `varchar`
  `to_disk`   `varchar`

## Column Descriptions

### uuser

### rread

### to_disk
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_schemas_by_day', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_schemas_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `sschema`    `varchar`
  `start_ts`   `date`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### sschema

### start_ts

### ddelete

### iinsert

### scan
'),
  ('sf_241129', 'ttables_creation_for_all_by_hour', 'view', '# `sf_241129.ttables_creation_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `ttables`    `int4`

## Column Descriptions

### start_ts

### ttables
'),
  ('sf_241129', 'vacuum_stats_for_all_by_minute', 'view', '# `sf_241129.vacuum_stats_for_all_by_minute`

## Description

## Columns

  column              data type
  ------------------- -------------
  `start_ts`          `timestamp`
  `new blocks`        `varchar`
  `replaced blocks`   `varchar`
  `new rows`          `varchar`
  `deleted rows`      `varchar`
  `xid duration`      `varchar`
  `qids`              `int8`
  `xids`              `int8`

## Column Descriptions

### start_ts

### new blocks

### replaced blocks

### new rows

### deleted rows

### xid duration

### qids

### xids
'),
  ('sf_241129', 'queries_for_leader', 'view', '# `sf_241129.queries_for_leader`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `pid`        `int4`
  `xid`        `int8`
  `start_ts`   `timestamp`
  `end_ts`     `timestamp`
  `dur`        `numeric`
  `ttext`      `varchar`

## Column Descriptions

### uuser

### pid

### xid

### start_ts

### end_ts

### dur

### ttext
'),
  ('base_241129', 'fixed_pg_proc_for_procedures', 'view', '# `base_241129.fixed_pg_proc_for_procedures`

## Description

## Columns

  column             data type
  ------------------ -------------
  `procedure_id`     `int4`
  `procedure_name`   `varchar`
  `schema_id`        `int4`
  `user_id`          `int4`
  `language_id`      `int4`
  `language_name`    `varchar`
  `procedure_text`   `text`
  `proargtypes`      `oidvector`

## Column Descriptions

### procedure_id

### procedure_name

### schema_id

### user_id

### language_id

### language_name

### procedure_text

### proargtypes
'),
  ('base_241129', 'transactions_in_batches', 'view', '# `base_241129.transactions_in_batches`

## Description

## Columns

  column                            data type
  --------------------------------- -------------
  `batch_id`                        `int8`
  `transaction_id`                  `int8`
  `transaction_batch_queue_start`   `timestamp`

## Column Descriptions

### batch_id

### transaction_id

### transaction_batch_queue_start
'),
  ('sf_241129', 'analyze_totals_for_all_by_hour', 'view', '# `sf_241129.analyze_totals_for_all_by_hour`

## Description

## Columns

  column          data type
  --------------- -------------
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('lf_241129', 'vacuum_totals_for_users_by_all', 'view', '# `lf_241129.vacuum_totals_for_users_by_all`

## Description

## Columns

  column                           data type
  -------------------------------- ------------
  `user_id`                        `int4`
  `user_name`                      `varchar`
  `number_manual_delete`           `int4`
  `number_manual_delete_skipped`   `int4`
  `number_manual_full`             `int4`
  `number_manual_full_skipped`     `int4`
  `number_manual_recluster`        `int4`
  `number_manual_reindex`          `int4`
  `number_manual_sort`             `int4`
  `number_manual_sort_skipped`     `int4`
  `number_auto_delete`             `int4`
  `number_auto_delete_skipped`     `int4`
  `number_auto_full`               `int4`
  `number_auto_full_skipped`       `int4`
  `number_auto_recluster`          `int4`
  `number_auto_reindex`            `int4`
  `number_auto_sort`               `int4`
  `number_auto_sort_skipped`       `int4`
  `number_total_vacuum`            `int4`
  `total_duration`                 `interval`

## Column Descriptions

### user_id

### user_name

### number_manual_delete

### number_manual_delete_skipped

### number_manual_full

### number_manual_full_skipped

### number_manual_recluster

### number_manual_reindex

### number_manual_sort

### number_manual_sort_skipped

### number_auto_delete

### number_auto_delete_skipped

### number_auto_full

### number_auto_full_skipped

### number_auto_recluster

### number_auto_reindex

### number_auto_sort

### number_auto_sort_skipped

### number_total_vacuum

### total_duration
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_statements_by_second', 'view', '# `lf_241129.steps_stats_durations_on_both_for_statements_by_second`

## Description

## Columns

  column             data type
  ------------------ -------------
  `statement_name`   `varchar`
  `node_type`        `varchar`
  `step_type`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `number_steps`     `int8`

## Column Descriptions

### statement_name

### node_type

### step_type

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### number_steps
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_all_by_hour', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_query', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_query`

## Description

## Columns

  column      data type
  ----------- -----------
  `qid`       `int4`
  `queue`     `varchar`
  `compile`   `varchar`
  `exec`      `varchar`

## Column Descriptions

### qid

### queue

### compile

### exec
'),
  ('lf_241129', 'analyze_totals_status_for_schema', 'view', '# `lf_241129.analyze_totals_status_for_schema`

## Description

## Columns

  column                   data type
  ------------------------ -----------
  `schema_id`              `int4`
  `schema_name`            `varchar`
  `number_good_stats`      `int4`
  `number_stale_stats`     `int4`
  `number_missing_stats`   `int4`
  `number_total_stats`     `int4`

## Column Descriptions

### schema_id

### schema_name

### number_good_stats

### number_stale_stats

### number_missing_stats

### number_total_stats
'),
  ('sf_241129', 'analyze_totals_for_tables_by_second', 'view', '# `sf_241129.analyze_totals_for_tables_by_second`

## Description

## Columns

  column          data type
  --------------- -------------
  `sschema`       `varchar`
  `ttable`        `varchar`
  `start_ts`      `timestamp`
  `manual`        `varchar`
  `automatic`     `varchar`
  `autoanalyze`   `varchar`
  `total`         `varchar`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### automatic

### autoanalyze

### total
'),
  ('sf_241129', 'locks_by_table', 'view', '# `sf_241129.locks_by_table`

## Description

## Columns

  column      data type
  ----------- -----------
  `uuser`     `varchar`
  `pid`       `int4`
  `xid`       `int8`
  `sschema`   `varchar`
  `ttable`    `varchar`
  `dur`       `numeric`
  `ttype`     `varchar`
  `s`         `varchar`
  `ttext`     `varchar`

## Column Descriptions

### uuser

### pid

### xid

### sschema

### ttable

### dur

### ttype

### s

### ttext
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_all_by_minute', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_all_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('sf_241129', 'pprocedures_text', 'view', '# `sf_241129.pprocedures_text`

## Description

## Columns

  column             data type
  ------------------ -----------
  `procedure_id`     `int4`
  `procedure_text`   `varchar`

## Column Descriptions

### procedure_id

### procedure_text
'),
  ('lf_241129', 'blocks_rows_totals_for_table', 'view', '# `lf_241129.blocks_rows_totals_for_table`

## Description

## Columns

  column                     data type
  -------------------------- -----------
  `schema_id`                `int4`
  `schema_name`              `varchar`
  `table_id`                 `int4`
  `table_name`               `varchar`
  `table_sort_type`          `varchar`
  `number_sorted_blocks`     `int8`
  `number_desorted_blocks`   `int8`
  `number_unsorted_blocks`   `int8`
  `number_total_blocks`      `int8`
  `number_sorted_rows`       `int8`
  `number_desorted_rows`     `int8`
  `number_unsorted_rows`     `int8`
  `number_total_rows`        `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### table_sort_type

### number_sorted_blocks

### number_desorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_desorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('lf_241129', 'analyze_totals_for_users_by_hour', 'view', '# `lf_241129.analyze_totals_for_users_by_hour`

## Description

## Columns

  column                                          data type
  ----------------------------------------------- -------------
  `user_id`                                       `int4`
  `user_name`                                     `varchar`
  `start_ts`                                      `timestamp`
  `number_manual_full_analyze`                    `int4`
  `number_manual_predictatecolumn_analyze`        `int4`
  `number_manual_skipped_analyze`                 `int4`
  `number_automatic_full_analyze`                 `int4`
  `number_automatic_predictatecolumn_analyze`     `int4`
  `number_automatic_skipped_analyze`              `int4`
  `number_autoanalyze_full_analyze`               `int4`
  `number_autoanalyze_predictatecolumn_analyze`   `int4`
  `number_autoanalyze_skipped_analyze`            `int4`
  `number_total_full_analyze`                     `int4`
  `number_total_predictatecolumn_analyze`         `int4`
  `number_total_skipped_analyze`                  `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_manual_full_analyze

### number_manual_predictatecolumn_analyze

### number_manual_skipped_analyze

### number_automatic_full_analyze

### number_automatic_predictatecolumn_analyze

### number_automatic_skipped_analyze

### number_autoanalyze_full_analyze

### number_autoanalyze_predictatecolumn_analyze

### number_autoanalyze_skipped_analyze

### number_total_full_analyze

### number_total_predictatecolumn_analyze

### number_total_skipped_analyze
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_users_by_all', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_users_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `node`      `varchar`
  `uuser`     `varchar`
  `t1`        `varchar`
  `t2bmer`    `varchar`
  `t3`        `varchar`
  `t9`        `varchar`
  `t16`       `varchar`
  `t17`       `varchar`
  `t18`       `varchar`
  `t21`       `varchar`
  `t22`       `varchar`
  `t23`       `varchar`
  `t24`       `varchar`
  `t25`       `varchar`
  `t28`       `varchar`
  `t29bmer`   `varchar`
  `t30`       `varchar`
  `t32`       `varchar`
  `t33`       `varchar`
  `t35`       `varchar`
  `t36`       `varchar`
  `t37`       `varchar`

## Column Descriptions

### node

### uuser

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_all_by_all', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_all_by_all`

## Description

## Columns

  column      data type
  ----------- -----------
  `queue`     `varchar`
  `compile`   `varchar`
  `exec`      `varchar`
  `queries`   `int4`

## Column Descriptions

### queue

### compile

### exec

### queries
'),
  ('lf_241129', 'blocks_rows_totals_for_tables_by_node', 'view', '# `lf_241129.blocks_rows_totals_for_tables_by_node`

## Description

## Columns

  column                           data type
  -------------------------------- -----------
  `schema_id`                      `int4`
  `schema_name`                    `varchar`
  `table_id`                       `int4`
  `table_name`                     `varchar`
  `table_distribution_type`        `varchar`
  `table_distribution_auto_flag`   `bool`
  `node_ordinal`                   `int2`
  `number_sorted_blocks`           `int8`
  `number_unsorted_blocks`         `int8`
  `number_total_blocks`            `int8`
  `number_sorted_rows`             `int8`
  `number_unsorted_rows`           `int8`
  `number_total_rows`              `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### table_distribution_type

### table_distribution_auto_flag

### node_ordinal

### number_sorted_blocks

### number_unsorted_blocks

### number_total_blocks

### number_sorted_rows

### number_unsorted_rows

### number_total_rows
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_day', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_schemas_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `sschema`    `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_users_by_second', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_users_by_second`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_abort
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_second', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_schemas_by_second`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_queues_by_day', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_queues_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `qname`      `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### qname

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_slices_by_minute', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_slices_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `n`          `int2`
  `s`          `int2`
  `t`          `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### s

### t

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_schemas_by_second', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_schemas_by_second`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('sf_241129', 'queries_compilation_stream_segment', 'view', '# `sf_241129.queries_compilation_stream_segment`

## Description

## Columns

  column       data type
  ------------ -------------
  `qid`        `int4`
  `stream`     `int2`
  `segment`    `int2`
  `start_ts`   `timestamp`
  `end_ts`     `timestamp`
  `dur`        `numeric`
  `status`     `varchar`

## Column Descriptions

### qid

### stream

### segment

### start_ts

### end_ts

### dur

### status
'),
  ('lf_241129', 'queries_text_for_leader', 'view', '# `lf_241129.queries_text_for_leader`

## Description

## Columns

  column               data type
  -------------------- -------------
  `session_id`         `int4`
  `transaction_id`     `int8`
  `query_start_ts`     `timestamp`
  `query_text_order`   `int4`
  `query_text`         `varchar`

## Column Descriptions

### session_id

### transaction_id

### query_start_ts

### query_text_order

### query_text
'),
  ('lf_241129', 'queries_compilation_stream_segment', 'view', '# `lf_241129.queries_compilation_stream_segment`

## Description

## Columns

  column                   data type
  ------------------------ -------------
  `query_id`               `int4`
  `stream_ordinal`         `int2`
  `segment_ordinal`        `int2`
  `compilation_start`      `timestamp`
  `compilation_end`        `timestamp`
  `compilation_duration`   `interval`
  `compilation_status`     `varchar`

## Column Descriptions

### query_id

### stream_ordinal

### segment_ordinal

### compilation_start

### compilation_end

### compilation_duration

### compilation_status
'),
  ('sf_241129', 'transactions_totals_commits_by_all', 'view', '# `sf_241129.transactions_totals_commits_by_all`

## Description

## Columns

  column     data type
  ---------- -----------
  `leader`   `numeric`
  `worker`   `numeric`
  `idle`     `numeric`
  `exec`     `numeric`

## Column Descriptions

### leader

### worker

### idle

### exec
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_day', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_schemas_by_day`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_queues_by_second', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_queues_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### qname

### start_ts

### aborts
'),
  ('sf_241129', 'blocks_rows_totals_for_tables_by_slice', 'view', '# `sf_241129.blocks_rows_totals_for_tables_by_slice`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `ttable`    `varchar`
  `dist`      `varchar`
  `a`         `varchar`
  `n`         `int2`
  `s`         `int2`
  `t`         `varchar`
  `sblks`     `varchar`
  `ublks`     `varchar`
  `tblks`     `varchar`
  `srows`     `varchar`
  `urows`     `varchar`
  `trows`     `varchar`

## Column Descriptions

### sschema

### ttable

### dist

### a

### n

### s

### t

### sblks

### ublks

### tblks

### srows

### urows

### trows
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_minute_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_minute_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_day_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_day_concise`

## Description

## Columns

  column       data type
  ------------ -----------
  `start_ts`   `date`
  `n`          `int2`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_users_by_day', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_users_by_day`

## Description

## Columns

  column          data type
  --------------- -----------
  `uuser`         `varchar`
  `start_ts`      `date`
  `leadin`        `varchar`
  `wlm_leadin`    `varchar`
  `queue`         `varchar`
  `compile`       `varchar`
  `exec`          `varchar`
  `wlm_leadout`   `varchar`
  `leadout`       `varchar`
  `total`         `varchar`
  `unact`         `varchar`
  `queries`       `int4`

## Column Descriptions

### uuser

### start_ts

### leadin

### wlm_leadin

### queue

### compile

### exec

### wlm_leadout

### leadout

### total

### unact

### queries
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_users_by_minute', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_users_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'queries_stats_durations_on_leader_for_users_by_minute', 'view', '# `lf_241129.queries_stats_durations_on_leader_for_users_by_minute`

## Description

## Columns

  column             data type
  ------------------ -------------
  `user_id`          `int4`
  `user_name`        `varchar`
  `start_ts`         `timestamp`
  `min_duration`     `interval`
  `max_duration`     `interval`
  `med_duration`     `float4`
  `avg_duration`     `interval`
  `std_duration`     `float4`
  `sum_duration`     `interval`
  `number_queries`   `int4`

## Column Descriptions

### user_id

### user_name

### start_ts

### min_duration

### max_duration

### med_duration

### avg_duration

### std_duration

### sum_duration

### number_queries
'),
  ('lf_241129', 'steps_stats_durations_on_both_for_tables_by_minute', 'view', '# `lf_241129.steps_stats_durations_on_both_for_tables_by_minute`

## Description

## Columns

  column                        data type
  ----------------------------- -------------
  `schema_id`                   `int4`
  `schema_name`                 `varchar`
  `table_id`                    `int4`
  `table_name`                  `varchar`
  `node_type`                   `varchar`
  `start_ts`                    `timestamp`
  `min_aggr_duration`           `interval`
  `max_aggr_duration`           `interval`
  `med_aggr_duration`           `float4`
  `avg_aggr_duration`           `interval`
  `std_aggr_duration`           `float4`
  `number_aggr_steps`           `int8`
  `min_delete_duration`         `interval`
  `max_delete_duration`         `interval`
  `med_delete_duration`         `float4`
  `avg_delete_duration`         `interval`
  `std_delete_duration`         `float4`
  `number_delete_steps`         `int8`
  `min_hashjoin_duration`       `interval`
  `max_hashjoin_duration`       `interval`
  `med_hashjoin_duration`       `float4`
  `avg_hashjoin_duration`       `interval`
  `std_hashjoin_duration`       `float4`
  `number_hashjoin_steps`       `int8`
  `min_insert_duration`         `interval`
  `max_insert_duration`         `interval`
  `med_insert_duration`         `float4`
  `avg_insert_duration`         `interval`
  `std_insert_duration`         `float4`
  `number_insert_steps`         `int8`
  `min_mergejoin_duration`      `interval`
  `max_mergejoin_duration`      `interval`
  `med_mergejoin_duration`      `float4`
  `avg_mergejoin_duration`      `interval`
  `std_mergejoin_duration`      `float4`
  `number_mergejoin_steps`      `int8`
  `min_nestloop_duration`       `interval`
  `max_nestloop_duration`       `interval`
  `med_nestloop_duration`       `float4`
  `avg_nestloop_duration`       `interval`
  `std_nestloop_duration`       `float4`
  `number_nestloop_steps`       `int8`
  `min_save_duration`           `interval`
  `max_save_duration`           `interval`
  `med_save_duration`           `float4`
  `avg_save_duration`           `interval`
  `std_save_duration`           `float4`
  `number_save_steps`           `int8`
  `min_sort_duration`           `interval`
  `max_sort_duration`           `interval`
  `med_sort_duration`           `float4`
  `avg_sort_duration`           `interval`
  `std_sort_duration`           `float4`
  `number_sort_steps`           `int8`
  `min_spatial_join_duration`   `interval`
  `max_spatial_join_duration`   `interval`
  `med_spatial_join_duration`   `float4`
  `avg_spatial_join_duration`   `interval`
  `std_spatial_join_duration`   `float4`
  `number_spatial_join_steps`   `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### node_type

### start_ts

### min_aggr_duration

### max_aggr_duration

### med_aggr_duration

### avg_aggr_duration

### std_aggr_duration

### number_aggr_steps

### min_delete_duration

### max_delete_duration

### med_delete_duration

### avg_delete_duration

### std_delete_duration

### number_delete_steps

### min_hashjoin_duration

### max_hashjoin_duration

### med_hashjoin_duration

### avg_hashjoin_duration

### std_hashjoin_duration

### number_hashjoin_steps

### min_insert_duration

### max_insert_duration

### med_insert_duration

### avg_insert_duration

### std_insert_duration

### number_insert_steps

### min_mergejoin_duration

### max_mergejoin_duration

### med_mergejoin_duration

### avg_mergejoin_duration

### std_mergejoin_duration

### number_mergejoin_steps

### min_nestloop_duration

### max_nestloop_duration

### med_nestloop_duration

### avg_nestloop_duration

### std_nestloop_duration

### number_nestloop_steps

### min_save_duration

### max_save_duration

### med_save_duration

### avg_save_duration

### std_save_duration

### number_save_steps

### min_sort_duration

### max_sort_duration

### med_sort_duration

### avg_sort_duration

### std_sort_duration

### number_sort_steps

### min_spatial_join_duration

### max_spatial_join_duration

### med_spatial_join_duration

### avg_spatial_join_duration

### std_spatial_join_duration

### number_spatial_join_steps
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_slices_by_minute', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_slices_by_minute`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_ordinal`        `int2`
  `slice_ordinal`       `int2`
  `slice_type`          `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### slice_ordinal

### slice_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_all_by_day_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_all_by_day_complete`

## Description

## Columns

  column           data type
  ---------------- -----------
  `node_type`      `varchar`
  `start_ts`       `date`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### node_type

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('lf_241129', 'objects_totals_for_schema', 'view', '# `lf_241129.objects_totals_for_schema`

## Description

## Columns

  column                               data type
  ------------------------------------ -----------
  `schema_id`                          `int4`
  `schema_name`                        `varchar`
  `number_tables`                      `int4`
  `number_table_columns`               `int4`
  `number_normal_views`                `int4`
  `number_normal_view_columns`         `int4`
  `number_late_binding_views`          `int4`
  `number_materialized_views`          `int4`
  `number_materialized_view_columns`   `int4`
  `number_functions`                   `int4`
  `number_procedures`                  `int4`

## Column Descriptions

### schema_id

### schema_name

### number_tables

### number_table_columns

### number_normal_views

### number_normal_view_columns

### number_late_binding_views

### number_materialized_views

### number_materialized_view_columns

### number_functions

### number_procedures
'),
  ('sf_241129', 'views_late_binding', 'view', '# `sf_241129.views_late_binding`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `vview`     `varchar`
  `uuser`     `varchar`

## Column Descriptions

### sschema

### vview

### uuser
'),
  ('lf_241129', 'queries_totals_blocks_on_workers_for_all_by_all', 'view', '# `lf_241129.queries_totals_blocks_on_workers_for_all_by_all`

## Description

## Columns

  column                                        data type
  --------------------------------------------- -----------
  `notnulled_sum_query_number_blocks_read`      `int8`
  `notnulled_sum_query_number_blocks_to_disk`   `int8`

## Column Descriptions

### notnulled_sum_query_number_blocks_read

### notnulled_sum_query_number_blocks_to_disk
'),
  ('sf_241129', 'vacuum_totals_for_users_by_hour', 'view', '# `sf_241129.vacuum_totals_for_users_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### uuser

### start_ts

### manual

### auto

### total

### dur
'),
  ('sf_241129', 'queries_totals_result_cache_for_all_by_minute', 'view', '# `sf_241129.queries_totals_result_cache_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### start_ts

### hit

### miss

### total

### %
'),
  ('base_241129', 'fixed_pg_namespace', 'view', '# `base_241129.fixed_pg_namespace`

## Description

## Columns

  column          data type
  --------------- -----------
  `schema_id`     `int4`
  `schema_name`   `varchar`
  `user_id`       `int4`

## Column Descriptions

### schema_id

### schema_name

### user_id
'),
  ('sf_241129', 'queries_totals_aborts_on_workers_for_all_by_hour', 'view', '# `sf_241129.queries_totals_aborts_on_workers_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `aborts`     `int4`

## Column Descriptions

### start_ts

### aborts
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_all_by_all', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_all_by_all`

## Description

## Columns

  column                        data type
  ----------------------------- -----------
  `notnulled_sum_delete_rows`   `int8`
  `notnulled_sum_insert_rows`   `int8`
  `notnulled_sum_scan_rows`     `int8`

## Column Descriptions

### notnulled_sum_delete_rows

### notnulled_sum_insert_rows

### notnulled_sum_scan_rows
'),
  ('lf_241129', 'queries_totals_aborts_on_workers_for_queues_by_second', 'view', '# `lf_241129.queries_totals_aborts_on_workers_for_queues_by_second`

## Description

## Columns

  column           data type
  ---------------- -------------
  `queue_id`       `int2`
  `queue_name`     `varchar`
  `start_ts`       `timestamp`
  `number_abort`   `int4`

## Column Descriptions

### queue_id

### queue_name

### start_ts

### number_abort
'),
  ('sf_241129', 'docs', 'view', '# `sf_241129.docs`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `oobject`   `varchar`
  `ttype`     `varchar`
  `doc`       `varchar`

## Column Descriptions

### sschema

### oobject

### ttype

### doc
'),
  ('sf_241129', 'wqueues_qmr_rules', 'view', '# `sf_241129.wqueues_qmr_rules`

## Description

## Columns

  column         data type
  -------------- -----------
  `oorder`       `int2`
  `rrule`        `varchar`
  `qname`        `varchar`
  `aaction`      `varchar`
  `conditions`   `varchar`

## Column Descriptions

### oorder

### rrule

### qname

### aaction

### conditions
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_queues_by_second', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_queues_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `qname`      `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### qname

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'queries_stats_durations_on_both_by_transaction', 'view', '# `sf_241129.queries_stats_durations_on_both_by_transaction`

## Description

## Columns

  column      data type
  ----------- -----------
  `xid`       `int8`
  `queue`     `varchar`
  `compile`   `varchar`
  `exec`      `varchar`
  `idle`      `varchar`
  `queries`   `int4`

## Column Descriptions

### xid

### queue

### compile

### exec

### idle

### queries
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_all_by_hour', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_all_by_hour`

## Description

## Columns

  column                data type
  --------------------- -------------
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'queries_stats_durations_on_leader_for_all_by_hour', 'view', '# `sf_241129.queries_stats_durations_on_leader_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `duration`   `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### duration

### queries
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_statements_by_minute', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_statements_by_minute`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `statement_name`               `varchar`
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### statement_name

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_queues_by_all', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_queues_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ ------------
  `queue_id`                     `int2`
  `queue_name`                   `varchar`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### queue_id

### queue_name

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('lf_241129', 'steps_totals_data_on_both_for_query', 'view', '# `lf_241129.steps_totals_data_on_both_for_query`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `query_id`                     `int4`
  `node_type`                    `varchar`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### query_id

### node_type

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'ttables_creation_for_all_by_day', 'view', '# `lf_241129.ttables_creation_for_all_by_day`

## Description

## Columns

  column            data type
  ----------------- -------------
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### start_ts

### number_tables
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_schemas_by_minute', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_schemas_by_minute`

## Description

## Columns

  column                data type
  --------------------- -------------
  `schema_id`           `int4`
  `schema_name`         `varchar`
  `node_type`           `varchar`
  `start_ts`            `timestamp`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### schema_id

### schema_name

### node_type

### start_ts

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_totals_rows_on_workers_for_users_by_day', 'view', '# `sf_241129.steps_totals_rows_on_workers_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `start_ts`   `date`
  `ddelete`    `varchar`
  `iinsert`    `varchar`
  `scan`       `varchar`

## Column Descriptions

### uuser

### start_ts

### ddelete

### iinsert

### scan
'),
  ('sf_241129', 'ttables_stats_dists_sorts_for_schemas_by_count', 'view', '# `sf_241129.ttables_stats_dists_sorts_for_schemas_by_count`

## Description

## Columns

  column      data type
  ----------- -----------
  `sschema`   `varchar`
  `aall`      `varchar`
  `even`      `varchar`
  `kkey`      `varchar`

## Column Descriptions

### sschema

### aall

### even

### kkey
'),
  ('lf_241129', 'steps_totals_scan_data_on_both_for_node_by_all', 'view', '# `lf_241129.steps_totals_scan_data_on_both_for_node_by_all`

## Description

## Columns

  column                data type
  --------------------- -----------
  `node_ordinal`        `int2`
  `sum_type_1_rows`     `int8`
  `sum_type_1_bytes`    `int8`
  `sum_type_2m_rows`    `int8`
  `sum_type_2e_rows`    `int8`
  `sum_type_2_rows`     `int8`
  `sum_type_2_bytes`    `int8`
  `sum_type_3_rows`     `int8`
  `sum_type_3_bytes`    `int8`
  `sum_type_9_rows`     `int8`
  `sum_type_9_bytes`    `int8`
  `sum_type_16_rows`    `int8`
  `sum_type_16_bytes`   `int8`
  `sum_type_17_rows`    `int8`
  `sum_type_17_bytes`   `int8`
  `sum_type_18_rows`    `int8`
  `sum_type_18_bytes`   `int8`
  `sum_type_21_rows`    `int8`
  `sum_type_21_bytes`   `int8`
  `sum_type_22_rows`    `int8`
  `sum_type_22_bytes`   `int8`
  `sum_type_23_rows`    `int8`
  `sum_type_23_bytes`   `int8`
  `sum_type_24_rows`    `int8`
  `sum_type_24_bytes`   `int8`
  `sum_type_25_rows`    `int8`
  `sum_type_25_bytes`   `int8`
  `sum_type_28_rows`    `int8`
  `sum_type_28_bytes`   `int8`
  `sum_type_29m_rows`   `int8`
  `sum_type_29e_rows`   `int8`
  `sum_type_29_rows`    `int8`
  `sum_type_29_bytes`   `int8`
  `sum_type_30_rows`    `int8`
  `sum_type_30_bytes`   `int8`
  `sum_type_32_rows`    `int8`
  `sum_type_32_bytes`   `int8`
  `sum_type_33_rows`    `int8`
  `sum_type_33_bytes`   `int8`
  `sum_type_35_rows`    `int8`
  `sum_type_35_bytes`   `int8`
  `sum_type_36_rows`    `int8`
  `sum_type_36_bytes`   `int8`
  `sum_type_37_rows`    `int8`
  `sum_type_37_bytes`   `int8`

## Column Descriptions

### node_ordinal

### sum_type_1_rows

### sum_type_1_bytes

### sum_type_2m_rows

### sum_type_2e_rows

### sum_type_2_rows

### sum_type_2_bytes

### sum_type_3_rows

### sum_type_3_bytes

### sum_type_9_rows

### sum_type_9_bytes

### sum_type_16_rows

### sum_type_16_bytes

### sum_type_17_rows

### sum_type_17_bytes

### sum_type_18_rows

### sum_type_18_bytes

### sum_type_21_rows

### sum_type_21_bytes

### sum_type_22_rows

### sum_type_22_bytes

### sum_type_23_rows

### sum_type_23_bytes

### sum_type_24_rows

### sum_type_24_bytes

### sum_type_25_rows

### sum_type_25_bytes

### sum_type_28_rows

### sum_type_28_bytes

### sum_type_29m_rows

### sum_type_29e_rows

### sum_type_29_rows

### sum_type_29_bytes

### sum_type_30_rows

### sum_type_30_bytes

### sum_type_32_rows

### sum_type_32_bytes

### sum_type_33_rows

### sum_type_33_bytes

### sum_type_35_rows

### sum_type_35_bytes

### sum_type_36_rows

### sum_type_36_bytes

### sum_type_37_rows

### sum_type_37_bytes
'),
  ('sf_241129', 'steps_totals_data_on_both_for_all_by_second', 'view', '# `sf_241129.steps_totals_data_on_both_for_all_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `node`       `varchar`
  `start_ts`   `timestamp`
  `aggr`       `varchar`
  `bcast`      `varchar`
  `ddelete`    `varchar`
  `dist`       `varchar`
  `hash`       `varchar`
  `hjoin`      `varchar`
  `iinsert`    `varchar`
  `llimit`     `varchar`
  `mmerge`     `varchar`
  `mjoin`      `varchar`
  `nloop`      `varchar`
  `rreturn`    `varchar`
  `save`       `varchar`
  `ssort`      `varchar`
  `uunique`    `varchar`
  `wwindow`    `varchar`

## Column Descriptions

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('lf_241129', 'analyze_totals_status_for_all', 'view', '# `lf_241129.analyze_totals_status_for_all`

## Description

## Columns

  column                   data type
  ------------------------ -----------
  `number_good_stats`      `int4`
  `number_stale_stats`     `int4`
  `number_missing_stats`   `int4`
  `number_total_stats`     `int4`

## Column Descriptions

### number_good_stats

### number_stale_stats

### number_missing_stats

### number_total_stats
'),
  ('sf_241129', 'ttables_zonemap', 'view', '# `sf_241129.ttables_zonemap`

## Description

## Columns

  column        data type
  ------------- -----------
  `sschema`     `varchar`
  `ttable`      `varchar`
  `ccolumn`     `varchar`
  `data_type`   `varchar`
  `eencoding`   `varchar`
  `0-20`        `int8`
  `20-40`       `int8`
  `40-60`       `int8`
  `60-80`       `int8`
  `80-100`      `int8`
  `oover`       `int8`
  `under`       `int8`
  `sat`         `int8`

## Column Descriptions

### sschema

### ttable

### ccolumn

### data_type

### eencoding

### 0-20

### 20-40

### 40-60

### 60-80

### 80-100

### oover

### under

### sat
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_all_by_minute', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_all_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### start_ts

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('base_241129', 'fixed_pg_group', 'view', '# `base_241129.fixed_pg_group`

## Description

## Columns

  column         data type
  -------------- -----------
  `group_id`     `int4`
  `group_name`   `varchar`

## Column Descriptions

### group_id

### group_name
'),
  ('lf_241129', 'queries_stats_durations_on_workers_for_all_by_second', 'view', '# `lf_241129.queries_stats_durations_on_workers_for_all_by_second`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `start_ts`                     `timestamp`
  `min_query_queue_duration`     `interval`
  `max_query_queue_duration`     `interval`
  `med_query_queue_duration`     `float4`
  `avg_query_queue_duration`     `interval`
  `std_query_queue_duration`     `float4`
  `sum_query_queue_duration`     `interval`
  `min_query_compile_duration`   `interval`
  `max_query_compile_duration`   `interval`
  `med_query_compile_duration`   `float4`
  `avg_query_compile_duration`   `interval`
  `std_query_compile_duration`   `float4`
  `sum_query_compile_duration`   `interval`
  `min_query_exec_duration`      `interval`
  `max_query_exec_duration`      `interval`
  `med_query_exec_duration`      `float4`
  `avg_query_exec_duration`      `interval`
  `std_query_exec_duration`      `float4`
  `sum_query_exec_duration`      `interval`
  `number_queries`               `int4`

## Column Descriptions

### start_ts

### min_query_queue_duration

### max_query_queue_duration

### med_query_queue_duration

### avg_query_queue_duration

### std_query_queue_duration

### sum_query_queue_duration

### min_query_compile_duration

### max_query_compile_duration

### med_query_compile_duration

### avg_query_compile_duration

### std_query_compile_duration

### sum_query_compile_duration

### min_query_exec_duration

### max_query_exec_duration

### med_query_exec_duration

### avg_query_exec_duration

### std_query_exec_duration

### sum_query_exec_duration

### number_queries
'),
  ('sf_241129', 'vacuum_totals_for_tables_by_day', 'view', '# `sf_241129.vacuum_totals_for_tables_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `start_ts`   `date`
  `manual`     `varchar`
  `auto`       `varchar`
  `total`      `int4`
  `dur`        `numeric`

## Column Descriptions

### sschema

### ttable

### start_ts

### manual

### auto

### total

### dur
'),
  ('lf_241129', 'steps_totals_rows_on_workers_for_tables_by_minute', 'view', '# `lf_241129.steps_totals_rows_on_workers_for_tables_by_minute`

## Description

## Columns

  column              data type
  ------------------- -------------
  `schema_id`         `int4`
  `schema_name`       `varchar`
  `table_id`          `int4`
  `table_name`        `varchar`
  `start_ts`          `timestamp`
  `sum_delete_rows`   `int8`
  `sum_insert_rows`   `int8`
  `sum_scan_rows`     `int8`

## Column Descriptions

### schema_id

### schema_name

### table_id

### table_name

### start_ts

### sum_delete_rows

### sum_insert_rows

### sum_scan_rows
'),
  ('lf_241129', 'steps_totals_data_on_both_for_node_by_day', 'view', '# `lf_241129.steps_totals_data_on_both_for_node_by_day`

## Description

## Columns

  column                         data type
  ------------------------------ -------------
  `node_ordinal`                 `int2`
  `start_ts`                     `timestamp`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_ordinal

### start_ts

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('sf_241129', 'queries_totals_result_cache_for_users_by_second', 'view', '# `sf_241129.queries_totals_result_cache_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `hit`        `int8`
  `miss`       `int8`
  `total`      `int8`
  `%`          `numeric`

## Column Descriptions

### uuser

### start_ts

### hit

### miss

### total

### %
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_users_by_day', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_users_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `node`       `varchar`
  `start_ts`   `date`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_node_by_day', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_node_by_day`

## Description

## Columns

  column       data type
  ------------ -----------
  `n`          `int2`
  `start_ts`   `date`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### n

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('lf_241129', 'ttables_creation_for_all_by_second', 'view', '# `lf_241129.ttables_creation_for_all_by_second`

## Description

## Columns

  column            data type
  ----------------- -------------
  `start_ts`        `timestamp`
  `number_tables`   `int4`

## Column Descriptions

### start_ts

### number_tables
'),
  ('sf_241129', 'transactions_totals_commits_by_minute', 'view', '# `sf_241129.transactions_totals_commits_by_minute`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `leader`     `numeric`
  `worker`     `numeric`
  `idle`       `numeric`
  `exec`       `numeric`

## Column Descriptions

### start_ts

### leader

### worker

### idle

### exec
'),
  ('sf_241129', 'queries_totals_blocks_on_workers_for_users_by_second', 'view', '# `sf_241129.queries_totals_blocks_on_workers_for_users_by_second`

## Description

## Columns

  column       data type
  ------------ -------------
  `uuser`      `varchar`
  `start_ts`   `timestamp`
  `rread`      `varchar`
  `to_disk`    `varchar`

## Column Descriptions

### uuser

### start_ts

### rread

### to_disk
'),
  ('sf_241129', 'steps_totals_scan_data_on_both_for_tables_by_hour', 'view', '# `sf_241129.steps_totals_scan_data_on_both_for_tables_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `sschema`    `varchar`
  `ttable`     `varchar`
  `node`       `varchar`
  `start_ts`   `timestamp`
  `t1`         `varchar`
  `t2bmer`     `varchar`
  `t3`         `varchar`
  `t9`         `varchar`
  `t16`        `varchar`
  `t17`        `varchar`
  `t18`        `varchar`
  `t21`        `varchar`
  `t22`        `varchar`
  `t23`        `varchar`
  `t24`        `varchar`
  `t25`        `varchar`
  `t28`        `varchar`
  `t29bmer`    `varchar`
  `t30`        `varchar`
  `t32`        `varchar`
  `t33`        `varchar`
  `t35`        `varchar`
  `t36`        `varchar`
  `t37`        `varchar`

## Column Descriptions

### sschema

### ttable

### node

### start_ts

### t1

### t2bmer

### t3

### t9

### t16

### t17

### t18

### t21

### t22

### t23

### t24

### t25

### t28

### t29bmer

### t30

### t32

### t33

### t35

### t36

### t37
'),
  ('sf_241129', 'transactions_stats_commits_by_all', 'view', '# `sf_241129.transactions_stats_commits_by_all`

## Description

## Columns

  column           data type
  ---------------- -----------
  `xid queuing`    `varchar`
  `bid queuing`    `varchar`
  `ldr duration`   `varchar`
  `wrk duration`   `varchar`
  `xids/bid`       `varchar`
  `blks/bid`       `varchar`
  `bids`           `int8`

## Column Descriptions

### xid queuing

### bid queuing

### ldr duration

### wrk duration

### xids/bid

### blks/bid

### bids
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_node_by_minute_concise', 'view', '# `sf_241129.steps_stats_durations_on_both_for_node_by_minute_concise`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `n`          `int2`
  `aggr`       `numeric`
  `bcast`      `numeric`
  `del`        `numeric`
  `dist`       `numeric`
  `hash`       `numeric`
  `hjoin`      `numeric`
  `iinsert`    `numeric`
  `llimit`     `numeric`
  `mmerge`     `numeric`
  `mjoin`      `numeric`
  `nloop`      `numeric`
  `rreturn`    `numeric`
  `save`       `numeric`
  `ssort`      `numeric`
  `uunique`    `numeric`
  `wwindow`    `numeric`

## Column Descriptions

### start_ts

### n

### aggr

### bcast

### del

### dist

### hash

### hjoin

### iinsert

### llimit

### mmerge

### mjoin

### nloop

### rreturn

### save

### ssort

### uunique

### wwindow
'),
  ('sf_241129', 'steps_stats_durations_on_both_for_users_by_second_complete', 'view', '# `sf_241129.steps_stats_durations_on_both_for_users_by_second_complete`

## Description

## Columns

  column           data type
  ---------------- -------------
  `uuser`          `varchar`
  `node`           `varchar`
  `start_ts`       `timestamp`
  `aggr`           `varchar`
  `bcast`          `varchar`
  `ddelete`        `varchar`
  `dist`           `varchar`
  `hash`           `varchar`
  `hashjoin`       `varchar`
  `iinsert`        `varchar`
  `llimit`         `varchar`
  `mmerge`         `varchar`
  `mergejoin`      `varchar`
  `nestloop_op`    `varchar`
  `parse`          `varchar`
  `project`        `varchar`
  `rreturn`        `varchar`
  `save`           `varchar`
  `ssort`          `varchar`
  `spatial_join`   `varchar`
  `uunique`        `varchar`
  `wwindow`        `varchar`

## Column Descriptions

### uuser

### node

### start_ts

### aggr

### bcast

### ddelete

### dist

### hash

### hashjoin

### iinsert

### llimit

### mmerge

### mergejoin

### nestloop_op

### parse

### project

### rreturn

### save

### ssort

### spatial_join

### uunique

### wwindow
'),
  ('sf_241129', 'transactions_totals_query_counts_on_both_for_users_by_all', 'view', '# `sf_241129.transactions_totals_query_counts_on_both_for_users_by_all`

## Description

## Columns

  column       data type
  ------------ -----------
  `uuser`      `varchar`
  `rowxids`    `int8`
  `wr xids`    `int8`
  `xids`       `int8`
  `ldr qids`   `int8`
  `wrk qids`   `int8`
  `qids`       `int8`

## Column Descriptions

### uuser

### rowxids

### wr xids

### xids

### ldr qids

### wrk qids

### qids
'),
  ('sf_241129', 'wqueues_qmr_log', 'view', '# `sf_241129.wqueues_qmr_log`

## Description

## Columns

  column       data type
  ------------ -------------
  `event_ts`   `timestamp`
  `rrule`      `varchar`
  `aaction`    `varchar`
  `qname`      `varchar`
  `uuser`      `varchar`
  `qid`        `int4`

## Column Descriptions

### event_ts

### rrule

### aaction

### qname

### uuser

### qid
'),
  ('lf_241129', 'steps_totals_data_on_both_for_statements_by_all', 'view', '# `lf_241129.steps_totals_data_on_both_for_statements_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `statement_name`               `varchar`
  `node_type`                    `varchar`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### statement_name

### node_type

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
'),
  ('lf_241129', 'queries_totals_result_cache_for_users_by_hour', 'view', '# `lf_241129.queries_totals_result_cache_for_users_by_hour`

## Description

## Columns

  column           data type
  ---------------- -------------
  `user_id`        `int4`
  `user_name`      `varchar`
  `start_ts`       `timestamp`
  `number_hit`     `int8`
  `number_miss`    `int8`
  `number_total`   `int8`
  `percentage`     `float4`

## Column Descriptions

### user_id

### user_name

### start_ts

### number_hit

### number_miss

### number_total

### percentage
'),
  ('sf_241129', 'queries_stats_durations_on_workers_for_all_by_hour', 'view', '# `sf_241129.queries_stats_durations_on_workers_for_all_by_hour`

## Description

## Columns

  column       data type
  ------------ -------------
  `start_ts`   `timestamp`
  `queue`      `varchar`
  `compile`    `varchar`
  `exec`       `varchar`
  `queries`    `int4`

## Column Descriptions

### start_ts

### queue

### compile

### exec

### queries
'),
  ('lf_241129', 'steps_totals_data_on_both_for_node_by_all', 'view', '# `lf_241129.steps_totals_data_on_both_for_node_by_all`

## Description

## Columns

  column                         data type
  ------------------------------ -----------
  `node_ordinal`                 `int2`
  `sum_aggr_on_disk_bytes`       `int8`
  `sum_aggr_in_memory_bytes`     `int8`
  `sum_aggr_on_disk_rows`        `int8`
  `sum_aggr_in_memory_rows`      `int8`
  `sum_bcast_bytes`              `int8`
  `sum_bcast_packets`            `int8`
  `sum_bcast_rows`               `int8`
  `sum_delete_rows`              `int8`
  `sum_dist_bytes`               `int8`
  `sum_dist_packets`             `int8`
  `sum_dist_rows`                `int8`
  `sum_hash_on_disk_bytes`       `int8`
  `sum_hash_in_memory_bytes`     `int8`
  `sum_hash_on_disk_rows`        `int8`
  `sum_hash_in_memory_rows`      `int8`
  `sum_hashjoin_rows`            `int8`
  `sum_insert_rows`              `int8`
  `sum_limit_rows`               `int8`
  `sum_merge_rows`               `int8`
  `sum_mergejoin_rows`           `int8`
  `sum_nestloop_rows`            `int8`
  `sum_parse_rows`               `int8`
  `sum_project_rows`             `int8`
  `sum_return_bytes`             `int8`
  `sum_return_packets`           `int8`
  `sum_return_rows`              `int8`
  `sum_save_on_disk_bytes`       `int8`
  `sum_save_in_memory_bytes`     `int8`
  `sum_save_on_disk_rows`        `int8`
  `sum_save_in_memory_rows`      `int8`
  `sum_sort_on_disk_bytes`       `int8`
  `sum_sort_in_memory_bytes`     `int8`
  `sum_sort_on_disk_rows`        `int8`
  `sum_sort_in_memory_rows`      `int8`
  `sum_spatial_join_rows`        `int8`
  `sum_unique_on_disk_bytes`     `int8`
  `sum_unique_in_memory_bytes`   `int8`
  `sum_unique_on_disk_rows`      `int8`
  `sum_unique_in_memory_rows`    `int8`
  `sum_window_rows`              `int8`

## Column Descriptions

### node_ordinal

### sum_aggr_on_disk_bytes

### sum_aggr_in_memory_bytes

### sum_aggr_on_disk_rows

### sum_aggr_in_memory_rows

### sum_bcast_bytes

### sum_bcast_packets

### sum_bcast_rows

### sum_delete_rows

### sum_dist_bytes

### sum_dist_packets

### sum_dist_rows

### sum_hash_on_disk_bytes

### sum_hash_in_memory_bytes

### sum_hash_on_disk_rows

### sum_hash_in_memory_rows

### sum_hashjoin_rows

### sum_insert_rows

### sum_limit_rows

### sum_merge_rows

### sum_mergejoin_rows

### sum_nestloop_rows

### sum_parse_rows

### sum_project_rows

### sum_return_bytes

### sum_return_packets

### sum_return_rows

### sum_save_on_disk_bytes

### sum_save_in_memory_bytes

### sum_save_on_disk_rows

### sum_save_in_memory_rows

### sum_sort_on_disk_bytes

### sum_sort_in_memory_bytes

### sum_sort_on_disk_rows

### sum_sort_in_memory_rows

### sum_spatial_join_rows

### sum_unique_on_disk_bytes

### sum_unique_in_memory_bytes

### sum_unique_on_disk_rows

### sum_unique_in_memory_rows

### sum_window_rows
');
commit;
vacuum full base_241129.docs to 100 percent;
vacuum full base_241129.numbers to 100 percent;
vacuum full base_241129.strings to 100 percent;
begin;
analyze base_241129.docs;
analyze base_241129.numbers;
analyze base_241129.strings;
commit;
select 'Release 241129 installed.' as info;
select vversion as installed_versions from ro.installed_versions;